// source: ubo/v1/ubo.proto
/**
 * @fileoverview
 * @enhanceable
 * @suppress {missingRequire} reports error on implicit type usages.
 * @suppress {messageConventions} JS Compiler reports an error if a variable or
 *     field starts with 'MSG_' and isn't a translatable message.
 * @public
 */
// GENERATED CODE -- DO NOT EDIT!
/* eslint-disable */
// @ts-nocheck

var jspb = require('google-protobuf');
var goog = jspb;
var global = (function() {
  if (this) { return this; }
  if (typeof window !== 'undefined') { return window; }
  if (typeof global !== 'undefined') { return global; }
  if (typeof self !== 'undefined') { return self; }
  return Function('return this')();
}.call(null));

var package_info_v1_package_info_pb = require('../../package_info/v1/package_info_pb.js');
goog.object.extend(proto, package_info_v1_package_info_pb);
goog.exportSymbol('proto.ubo.v1.AcceptableAssistanceFrame', null, global);
goog.exportSymbol('proto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase', null, global);
goog.exportSymbol('proto.ubo.v1.Action', null, global);
goog.exportSymbol('proto.ubo.v1.Action.ActionCase', null, global);
goog.exportSymbol('proto.ubo.v1.ActionItem', null, global);
goog.exportSymbol('proto.ubo.v1.ApplicationItem', null, global);
goog.exportSymbol('proto.ubo.v1.ApplicationItem.Application', null, global);
goog.exportSymbol('proto.ubo.v1.ApplicationItem.Application.ApplicationCase', null, global);
goog.exportSymbol('proto.ubo.v1.AssistanceAudioFrame', null, global);
goog.exportSymbol('proto.ubo.v1.AssistanceFrame', null, global);
goog.exportSymbol('proto.ubo.v1.AssistanceImageFrame', null, global);
goog.exportSymbol('proto.ubo.v1.AssistanceTextFrame', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantAddMcpServerAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantAddMcpServerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantDeleteMcpServerAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantDeleteMcpServerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantDownloadOllamaModelAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantDownloadOllamaModelEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantHandleReportEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantImageGeneratorName', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantLLMName', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantReportAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSTTName', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSetIsActiveAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSetSelectedImageGeneratorAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSetSelectedLLMAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSetSelectedModelAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSetSelectedSTTAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSetSelectedTTSAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantStartListeningAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantState', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantState.EnabledMcpServers', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantState.McpServersDict', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantState.ProviderSetupStatusDict', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantState.SelectedModelsDict', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantStopListeningAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantSyncMcpServersAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantTTSName', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantToggleListeningAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantToggleMcpServerAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantUpdateProvidersAction', null, global);
goog.exportSymbol('proto.ubo.v1.AssistantUpdateProvidersEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioChangeVolumeAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioDevice', null, global);
goog.exportSymbol('proto.ubo.v1.AudioEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioInstallDriverAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioInstallDriverEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlayAudioSampleAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlayAudioSampleEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlayAudioSequenceAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlayAudioSequenceEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlayChimeAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlayChimeEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlayRecordingAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlaybackDoneAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioPlaybackDoneEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioReportSampleAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioReportSampleEvent', null, global);
goog.exportSymbol('proto.ubo.v1.AudioSample', null, global);
goog.exportSymbol('proto.ubo.v1.AudioSetMuteStatusAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioSetVolumeAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioStartRecordingAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioState', null, global);
goog.exportSymbol('proto.ubo.v1.AudioStopRecordingAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioToggleMuteStatusAction', null, global);
goog.exportSymbol('proto.ubo.v1.AudioToggleRecordingAction', null, global);
goog.exportSymbol('proto.ubo.v1.AvailableInputDescription', null, global);
goog.exportSymbol('proto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase', null, global);
goog.exportSymbol('proto.ubo.v1.BaseMenu', null, global);
goog.exportSymbol('proto.ubo.v1.BasicType', null, global);
goog.exportSymbol('proto.ubo.v1.BasicTypeOptional', null, global);
goog.exportSymbol('proto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase', null, global);
goog.exportSymbol('proto.ubo.v1.CameraAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraDetectAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraDetectEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraDetectedEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraInstallDriverAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraInstallDriverEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraReinitializeEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraReportBarcodeAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraReportImageEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraRestoreDefaultAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraRestoreDefaultEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraSetAvailableCamerasAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraSetIndexAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraStartViewfinderAction', null, global);
goog.exportSymbol('proto.ubo.v1.CameraStartViewfinderEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraState', null, global);
goog.exportSymbol('proto.ubo.v1.CameraState.AvailableCameras', null, global);
goog.exportSymbol('proto.ubo.v1.CameraStopViewfinderEvent', null, global);
goog.exportSymbol('proto.ubo.v1.CameraType', null, global);
goog.exportSymbol('proto.ubo.v1.Chime', null, global);
goog.exportSymbol('proto.ubo.v1.CloseApplicationAction', null, global);
goog.exportSymbol('proto.ubo.v1.CloseApplicationEvent', null, global);
goog.exportSymbol('proto.ubo.v1.ConnectionState', null, global);
goog.exportSymbol('proto.ubo.v1.DeregisterRegularAppAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayBlankAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayBlankEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayBlankTimeout', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayCompressedRenderEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayPauseAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayRedrawAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayRedrawEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayRenderEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayResumeAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplaySetBlankTimeoutAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayState', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayUnblankAction', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayUnblankEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DisplayUpdateActivityAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageFetchAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageFetchCompositionEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageFetchEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRegisterAppEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageReleaseAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageReleaseCompositionEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRemoveAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRemoveCompositionEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRemoveContainerAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRemoveContainerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRemoveEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRunAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRunCompositionEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageRunContainerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageSetDockerIdAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageSetStatusAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageSetStatusAction.Ports', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageStopAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageStopCompositionEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageStopContainerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerImageUpdateMetadataAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerInstallAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerInstallEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerItemStatus', null, global);
goog.exportSymbol('proto.ubo.v1.DockerRemoveUsernameAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerServiceState', null, global);
goog.exportSymbol('proto.ubo.v1.DockerServiceState.UsernamesDict', null, global);
goog.exportSymbol('proto.ubo.v1.DockerSetStatusAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerStartAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerStartEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerState', null, global);
goog.exportSymbol('proto.ubo.v1.DockerStatus', null, global);
goog.exportSymbol('proto.ubo.v1.DockerStopAction', null, global);
goog.exportSymbol('proto.ubo.v1.DockerStopEvent', null, global);
goog.exportSymbol('proto.ubo.v1.DockerStoreUsernameAction', null, global);
goog.exportSymbol('proto.ubo.v1.ErrorReport', null, global);
goog.exportSymbol('proto.ubo.v1.Event', null, global);
goog.exportSymbol('proto.ubo.v1.Event.EventCase', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemAction', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemCopyAction', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemCopyEvent', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemEvent', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemMoveAction', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemMoveEvent', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemRemoveAction', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemRemoveEvent', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemReportSelectionAction', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemSelectEvent', null, global);
goog.exportSymbol('proto.ubo.v1.FileSystemState', null, global);
goog.exportSymbol('proto.ubo.v1.HeadedMenu', null, global);
goog.exportSymbol('proto.ubo.v1.HeadlessMenu', null, global);
goog.exportSymbol('proto.ubo.v1.IconState', null, global);
goog.exportSymbol('proto.ubo.v1.ImageState', null, global);
goog.exportSymbol('proto.ubo.v1.ImageState.Ports', null, global);
goog.exportSymbol('proto.ubo.v1.Importance', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredAction', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredEvent', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredHandleReceivedCodeAction', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredSendCodeAction', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredSendCodeEvent', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredSetShouldPropagateAction', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredSetShouldReceiveAction', null, global);
goog.exportSymbol('proto.ubo.v1.InfraredState', null, global);
goog.exportSymbol('proto.ubo.v1.InitEvent', null, global);
goog.exportSymbol('proto.ubo.v1.InputAction', null, global);
goog.exportSymbol('proto.ubo.v1.InputCancelAction', null, global);
goog.exportSymbol('proto.ubo.v1.InputCancelEvent', null, global);
goog.exportSymbol('proto.ubo.v1.InputDemandAction', null, global);
goog.exportSymbol('proto.ubo.v1.InputDescription', null, global);
goog.exportSymbol('proto.ubo.v1.InputFieldDescription', null, global);
goog.exportSymbol('proto.ubo.v1.InputFieldDescription.Options', null, global);
goog.exportSymbol('proto.ubo.v1.InputFieldType', null, global);
goog.exportSymbol('proto.ubo.v1.InputMethod', null, global);
goog.exportSymbol('proto.ubo.v1.InputProvideAction', null, global);
goog.exportSymbol('proto.ubo.v1.InputProvideEvent', null, global);
goog.exportSymbol('proto.ubo.v1.InputResolveAction', null, global);
goog.exportSymbol('proto.ubo.v1.InputResolveEvent', null, global);
goog.exportSymbol('proto.ubo.v1.InputResult', null, global);
goog.exportSymbol('proto.ubo.v1.IpAction', null, global);
goog.exportSymbol('proto.ubo.v1.IpEvent', null, global);
goog.exportSymbol('proto.ubo.v1.IpNetworkInterface', null, global);
goog.exportSymbol('proto.ubo.v1.IpSetIsConnectedAction', null, global);
goog.exportSymbol('proto.ubo.v1.IpState', null, global);
goog.exportSymbol('proto.ubo.v1.IpUpdateInterfacesAction', null, global);
goog.exportSymbol('proto.ubo.v1.Item', null, global);
goog.exportSymbol('proto.ubo.v1.Key', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadAction', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadAction.HeldKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadAction.PressedKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyHoldAction', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyPressAction', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyReleaseAction', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyUnholdAction', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType', null, global);
goog.exportSymbol('proto.ubo.v1.KeypadState', null, global);
goog.exportSymbol('proto.ubo.v1.LightDMAction', null, global);
goog.exportSymbol('proto.ubo.v1.LightDMClearEnabledStateAction', null, global);
goog.exportSymbol('proto.ubo.v1.LightDMState', null, global);
goog.exportSymbol('proto.ubo.v1.LightDMUpdateStateAction', null, global);
goog.exportSymbol('proto.ubo.v1.MainAction', null, global);
goog.exportSymbol('proto.ubo.v1.MainEvent', null, global);
goog.exportSymbol('proto.ubo.v1.MainState', null, global);
goog.exportSymbol('proto.ubo.v1.MainState.AppsItemsPrioritiesDict', null, global);
goog.exportSymbol('proto.ubo.v1.MainState.Path', null, global);
goog.exportSymbol('proto.ubo.v1.MainState.RecordedSequence', null, global);
goog.exportSymbol('proto.ubo.v1.MainState.SettingsItemsPrioritiesDict', null, global);
goog.exportSymbol('proto.ubo.v1.McpServerMetadata', null, global);
goog.exportSymbol('proto.ubo.v1.McpServerType', null, global);
goog.exportSymbol('proto.ubo.v1.Menu', null, global);
goog.exportSymbol('proto.ubo.v1.Menu.MenuCase', null, global);
goog.exportSymbol('proto.ubo.v1.MenuAction', null, global);
goog.exportSymbol('proto.ubo.v1.MenuChooseByIconAction', null, global);
goog.exportSymbol('proto.ubo.v1.MenuChooseByIconEvent', null, global);
goog.exportSymbol('proto.ubo.v1.MenuChooseByIndexAction', null, global);
goog.exportSymbol('proto.ubo.v1.MenuChooseByIndexEvent', null, global);
goog.exportSymbol('proto.ubo.v1.MenuChooseByLabelAction', null, global);
goog.exportSymbol('proto.ubo.v1.MenuChooseByLabelEvent', null, global);
goog.exportSymbol('proto.ubo.v1.MenuEvent', null, global);
goog.exportSymbol('proto.ubo.v1.MenuGoBackAction', null, global);
goog.exportSymbol('proto.ubo.v1.MenuGoBackEvent', null, global);
goog.exportSymbol('proto.ubo.v1.MenuGoHomeAction', null, global);
goog.exportSymbol('proto.ubo.v1.MenuGoHomeEvent', null, global);
goog.exportSymbol('proto.ubo.v1.MenuScrollAction', null, global);
goog.exportSymbol('proto.ubo.v1.MenuScrollDirection', null, global);
goog.exportSymbol('proto.ubo.v1.MenuScrollEvent', null, global);
goog.exportSymbol('proto.ubo.v1.NetState', null, global);
goog.exportSymbol('proto.ubo.v1.Notification', null, global);
goog.exportSymbol('proto.ubo.v1.Notification.Actions', null, global);
goog.exportSymbol('proto.ubo.v1.Notification.ActionsItem', null, global);
goog.exportSymbol('proto.ubo.v1.Notification.ActionsItem.ActionsItemCase', null, global);
goog.exportSymbol('proto.ubo.v1.Notification.OnClose', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationActionItem', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationApplicationItem', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.Application', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.InitializationArgs', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationDispatchItem', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationDispatchItem.StoreAction', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationDispatchItem.StoreAction2', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationDisplayType', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsAction', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsAddAction', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsClearAction', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsClearAllAction', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsClearByIdAction', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsClearEvent', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsDisplayAction', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsDisplayEvent', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsEvent', null, global);
goog.exportSymbol('proto.ubo.v1.NotificationsState', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationAction', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationArgs', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationEvent', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationArgs', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase', null, global);
goog.exportSymbol('proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2', null, global);
goog.exportSymbol('proto.ubo.v1.PathInputDescription', null, global);
goog.exportSymbol('proto.ubo.v1.PathSelectorConfig', null, global);
goog.exportSymbol('proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes', null, global);
goog.exportSymbol('proto.ubo.v1.PowerAction', null, global);
goog.exportSymbol('proto.ubo.v1.PowerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.PowerOffAction', null, global);
goog.exportSymbol('proto.ubo.v1.PowerOffEvent', null, global);
goog.exportSymbol('proto.ubo.v1.QRCodeInputDescription', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectAction', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectDoneDownloadingAction', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectEvent', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectLoginEvent', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectSetPendingAction', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectSetStatusAction', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectStartDownloadingAction', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectState', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectStatus', null, global);
goog.exportSymbol('proto.ubo.v1.RPiConnectUpdateServiceStateAction', null, global);
goog.exportSymbol('proto.ubo.v1.ReadableInformation', null, global);
goog.exportSymbol('proto.ubo.v1.RebootAction', null, global);
goog.exportSymbol('proto.ubo.v1.RebootEvent', null, global);
goog.exportSymbol('proto.ubo.v1.RegisterAppAction', null, global);
goog.exportSymbol('proto.ubo.v1.RegisterRegularAppAction', null, global);
goog.exportSymbol('proto.ubo.v1.RegisterSettingAppAction', null, global);
goog.exportSymbol('proto.ubo.v1.ReplayRecordedSequenceAction', null, global);
goog.exportSymbol('proto.ubo.v1.ReplayRecordedSequenceEvent', null, global);
goog.exportSymbol('proto.ubo.v1.ReportReplayingDoneAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbColor', null, global);
goog.exportSymbol('proto.ubo.v1.RgbColorElement', null, global);
goog.exportSymbol('proto.ubo.v1.RgbColorElement.RgbColorElementCase', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingBlankAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingBlinkAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingColorfulCommandAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingCommandAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingCommandEvent', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingEvent', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingFillDownfromAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingFillUptoAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingProgressWheelAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingProgressWheelStepAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingPulseAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingRainbowAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingSequenceAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingSetAllAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingSetBrightnessAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingSetEnabledAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingSetIsBusyAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingSpinningWheelAction', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingState', null, global);
goog.exportSymbol('proto.ubo.v1.RgbRingWaitableCommandAction', null, global);
goog.exportSymbol('proto.ubo.v1.SSHAction', null, global);
goog.exportSymbol('proto.ubo.v1.SSHClearEnabledStateAction', null, global);
goog.exportSymbol('proto.ubo.v1.SSHState', null, global);
goog.exportSymbol('proto.ubo.v1.SSHUpdateStateAction', null, global);
goog.exportSymbol('proto.ubo.v1.ScreenshotEvent', null, global);
goog.exportSymbol('proto.ubo.v1.Sensor', null, global);
goog.exportSymbol('proto.ubo.v1.SensorState', null, global);
goog.exportSymbol('proto.ubo.v1.SensorsAction', null, global);
goog.exportSymbol('proto.ubo.v1.SensorsReportReadingAction', null, global);
goog.exportSymbol('proto.ubo.v1.SensorsState', null, global);
goog.exportSymbol('proto.ubo.v1.ServiceState', null, global);
goog.exportSymbol('proto.ubo.v1.ServiceState.Errors', null, global);
goog.exportSymbol('proto.ubo.v1.ServicesStatus', null, global);
goog.exportSymbol('proto.ubo.v1.SetAreEnclosuresVisibleAction', null, global);
goog.exportSymbol('proto.ubo.v1.SetMenuPathAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsCategory', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsClearServiceErrorsAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsReportServiceErrorAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsServiceAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsServiceEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsServiceSetIsEnabledAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsServiceSetLogLevelAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsServiceSetShouldRestartAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsServiceSetStatusAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsSetServicesAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsStartServiceAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsStartServiceEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsState', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsState.ServicesDict', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsStopServiceAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsStopServiceEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsToggleBetaVersionsAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsTogglePdbSignalAction', null, global);
goog.exportSymbol('proto.ubo.v1.SettingsToggleVisualDebugAction', null, global);
goog.exportSymbol('proto.ubo.v1.SnapshotEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionEngineName', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Action', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Action2', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Phrase', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionIntent.Phrase1', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportSpeechAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportTextEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionState', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechRecognitionStatus', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisEngineName', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisReadTextAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisState', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus', null, global);
goog.exportSymbol('proto.ubo.v1.StatusIconsAction', null, global);
goog.exportSymbol('proto.ubo.v1.StatusIconsRegisterAction', null, global);
goog.exportSymbol('proto.ubo.v1.StatusIconsState', null, global);
goog.exportSymbol('proto.ubo.v1.StoreRecordedSequenceEvent', null, global);
goog.exportSymbol('proto.ubo.v1.SubMenuItem', null, global);
goog.exportSymbol('proto.ubo.v1.ToggleRecordingAction', null, global);
goog.exportSymbol('proto.ubo.v1.UboApplicationItem', null, global);
goog.exportSymbol('proto.ubo.v1.UboApplicationItem.Application', null, global);
goog.exportSymbol('proto.ubo.v1.UboApplicationItem.Application.ApplicationCase', null, global);
goog.exportSymbol('proto.ubo.v1.UboApplicationItem.InitializationArgs', null, global);
goog.exportSymbol('proto.ubo.v1.UboApplicationItem.InitializationKwargsDict', null, global);
goog.exportSymbol('proto.ubo.v1.UboDispatchItem', null, global);
goog.exportSymbol('proto.ubo.v1.UboDispatchItem.StoreAction', null, global);
goog.exportSymbol('proto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase', null, global);
goog.exportSymbol('proto.ubo.v1.UboDispatchItem.StoreAction2', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateLightDMState', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerAction', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerCheckEvent', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerEvent', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerReportFailedCheckAction', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerRequestCheckAction', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerRequestUpdateAction', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerSetVersionsAction', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerState', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerState.RecentVersions', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateManagerUpdateEvent', null, global);
goog.exportSymbol('proto.ubo.v1.UpdateStatus', null, global);
goog.exportSymbol('proto.ubo.v1.UserState', null, global);
goog.exportSymbol('proto.ubo.v1.UsersAction', null, global);
goog.exportSymbol('proto.ubo.v1.UsersCreateUserAction', null, global);
goog.exportSymbol('proto.ubo.v1.UsersCreateUserEvent', null, global);
goog.exportSymbol('proto.ubo.v1.UsersDeleteUserAction', null, global);
goog.exportSymbol('proto.ubo.v1.UsersDeleteUserEvent', null, global);
goog.exportSymbol('proto.ubo.v1.UsersEvent', null, global);
goog.exportSymbol('proto.ubo.v1.UsersResetPasswordAction', null, global);
goog.exportSymbol('proto.ubo.v1.UsersResetPasswordEvent', null, global);
goog.exportSymbol('proto.ubo.v1.UsersSetUsersAction', null, global);
goog.exportSymbol('proto.ubo.v1.UsersState', null, global);
goog.exportSymbol('proto.ubo.v1.UsersState.Users', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeAction', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeDoneDownloadingAction', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeEvent', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeLoginEvent', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeRestartEvent', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeSetPendingAction', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeSetStatusAction', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeStartDownloadingAction', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeState', null, global);
goog.exportSymbol('proto.ubo.v1.VSCodeStatus', null, global);
goog.exportSymbol('proto.ubo.v1.WebUIEvent', null, global);
goog.exportSymbol('proto.ubo.v1.WebUIInitializeEvent', null, global);
goog.exportSymbol('proto.ubo.v1.WebUIInputDescription', null, global);
goog.exportSymbol('proto.ubo.v1.WebUIInputDescription.Fields', null, global);
goog.exportSymbol('proto.ubo.v1.WebUIState', null, global);
goog.exportSymbol('proto.ubo.v1.WebUIStopEvent', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiAction', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiConnection', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiEvent', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiInputConnectionAction', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiInputConnectionEvent', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiSetHasVisitedOnboardingAction', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiState', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiState.Connections', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiType', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiUpdateAction', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiUpdateRequestAction', null, global);
goog.exportSymbol('proto.ubo.v1.WiFiUpdateRequestEvent', null, global);
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.BaseMenu = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.BaseMenu.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.BaseMenu, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.BaseMenu.displayName = 'proto.ubo.v1.BaseMenu';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.HeadedMenu = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.HeadedMenu.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.HeadedMenu, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.HeadedMenu.displayName = 'proto.ubo.v1.HeadedMenu';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.HeadlessMenu = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.HeadlessMenu.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.HeadlessMenu, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.HeadlessMenu.displayName = 'proto.ubo.v1.HeadlessMenu';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Item = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.Item, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Item.displayName = 'proto.ubo.v1.Item';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ActionItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ActionItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ActionItem.displayName = 'proto.ubo.v1.ActionItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ApplicationItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ApplicationItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ApplicationItem.displayName = 'proto.ubo.v1.ApplicationItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ApplicationItem.Application = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.ApplicationItem.Application.oneofGroups_);
};
goog.inherits(proto.ubo.v1.ApplicationItem.Application, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ApplicationItem.Application.displayName = 'proto.ubo.v1.ApplicationItem.Application';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SubMenuItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SubMenuItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SubMenuItem.displayName = 'proto.ubo.v1.SubMenuItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Menu = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Menu.oneofGroups_);
};
goog.inherits(proto.ubo.v1.Menu, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Menu.displayName = 'proto.ubo.v1.Menu';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MainAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MainAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MainAction.displayName = 'proto.ubo.v1.MainAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateLightDMState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateLightDMState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateLightDMState.displayName = 'proto.ubo.v1.UpdateLightDMState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RegisterAppAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RegisterAppAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RegisterAppAction.displayName = 'proto.ubo.v1.RegisterAppAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RegisterRegularAppAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RegisterRegularAppAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RegisterRegularAppAction.displayName = 'proto.ubo.v1.RegisterRegularAppAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DeregisterRegularAppAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DeregisterRegularAppAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DeregisterRegularAppAction.displayName = 'proto.ubo.v1.DeregisterRegularAppAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RegisterSettingAppAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RegisterSettingAppAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RegisterSettingAppAction.displayName = 'proto.ubo.v1.RegisterSettingAppAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.PowerAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.PowerAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.PowerAction.displayName = 'proto.ubo.v1.PowerAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.PowerOffAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.PowerOffAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.PowerOffAction.displayName = 'proto.ubo.v1.PowerOffAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RebootAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RebootAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RebootAction.displayName = 'proto.ubo.v1.RebootAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SetMenuPathAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.SetMenuPathAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.SetMenuPathAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SetMenuPathAction.displayName = 'proto.ubo.v1.SetMenuPathAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SetAreEnclosuresVisibleAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SetAreEnclosuresVisibleAction.displayName = 'proto.ubo.v1.SetAreEnclosuresVisibleAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuAction.displayName = 'proto.ubo.v1.MenuAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuGoBackAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuGoBackAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuGoBackAction.displayName = 'proto.ubo.v1.MenuGoBackAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuGoHomeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuGoHomeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuGoHomeAction.displayName = 'proto.ubo.v1.MenuGoHomeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuChooseByIconAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuChooseByIconAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuChooseByIconAction.displayName = 'proto.ubo.v1.MenuChooseByIconAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuChooseByLabelAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuChooseByLabelAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuChooseByLabelAction.displayName = 'proto.ubo.v1.MenuChooseByLabelAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuChooseByIndexAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuChooseByIndexAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuChooseByIndexAction.displayName = 'proto.ubo.v1.MenuChooseByIndexAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuScrollAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuScrollAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuScrollAction.displayName = 'proto.ubo.v1.MenuScrollAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationAction.displayName = 'proto.ubo.v1.OpenApplicationAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationAction.InitializationArgs.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationArgs, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationAction.InitializationArgs.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationArgs';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2 = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_);
};
goog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.displayName = 'proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CloseApplicationAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CloseApplicationAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CloseApplicationAction.displayName = 'proto.ubo.v1.CloseApplicationAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MainEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MainEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MainEvent.displayName = 'proto.ubo.v1.MainEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InitEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InitEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InitEvent.displayName = 'proto.ubo.v1.InitEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuEvent.displayName = 'proto.ubo.v1.MenuEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuGoBackEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuGoBackEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuGoBackEvent.displayName = 'proto.ubo.v1.MenuGoBackEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuGoHomeEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuGoHomeEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuGoHomeEvent.displayName = 'proto.ubo.v1.MenuGoHomeEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuChooseByIconEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuChooseByIconEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuChooseByIconEvent.displayName = 'proto.ubo.v1.MenuChooseByIconEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuChooseByLabelEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuChooseByLabelEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuChooseByLabelEvent.displayName = 'proto.ubo.v1.MenuChooseByLabelEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuChooseByIndexEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuChooseByIndexEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuChooseByIndexEvent.displayName = 'proto.ubo.v1.MenuChooseByIndexEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MenuScrollEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MenuScrollEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MenuScrollEvent.displayName = 'proto.ubo.v1.MenuScrollEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationEvent.displayName = 'proto.ubo.v1.OpenApplicationEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationEvent.InitializationArgs.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationArgs, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationEvent.InitializationArgs.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationArgs';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2 = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_);
};
goog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.displayName = 'proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CloseApplicationEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CloseApplicationEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CloseApplicationEvent.displayName = 'proto.ubo.v1.CloseApplicationEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.PowerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.PowerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.PowerEvent.displayName = 'proto.ubo.v1.PowerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.PowerOffEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.PowerOffEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.PowerOffEvent.displayName = 'proto.ubo.v1.PowerOffEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RebootEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RebootEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RebootEvent.displayName = 'proto.ubo.v1.RebootEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ScreenshotEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ScreenshotEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ScreenshotEvent.displayName = 'proto.ubo.v1.ScreenshotEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SnapshotEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SnapshotEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SnapshotEvent.displayName = 'proto.ubo.v1.SnapshotEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ToggleRecordingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ToggleRecordingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ToggleRecordingAction.displayName = 'proto.ubo.v1.ToggleRecordingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.StoreRecordedSequenceEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.StoreRecordedSequenceEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.StoreRecordedSequenceEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.StoreRecordedSequenceEvent.displayName = 'proto.ubo.v1.StoreRecordedSequenceEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ReplayRecordedSequenceAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ReplayRecordedSequenceAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ReplayRecordedSequenceAction.displayName = 'proto.ubo.v1.ReplayRecordedSequenceAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ReplayRecordedSequenceEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ReplayRecordedSequenceEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ReplayRecordedSequenceEvent.displayName = 'proto.ubo.v1.ReplayRecordedSequenceEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ReportReplayingDoneAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ReportReplayingDoneAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ReportReplayingDoneAction.displayName = 'proto.ubo.v1.ReportReplayingDoneAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MainState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.MainState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MainState.displayName = 'proto.ubo.v1.MainState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MainState.Path = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.MainState.Path.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.MainState.Path, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MainState.Path.displayName = 'proto.ubo.v1.MainState.Path';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.MainState.AppsItemsPrioritiesDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MainState.AppsItemsPrioritiesDict.displayName = 'proto.ubo.v1.MainState.AppsItemsPrioritiesDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.MainState.SettingsItemsPrioritiesDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.displayName = 'proto.ubo.v1.MainState.SettingsItemsPrioritiesDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.MainState.RecordedSequence = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.MainState.RecordedSequence.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.MainState.RecordedSequence, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.MainState.RecordedSequence.displayName = 'proto.ubo.v1.MainState.RecordedSequence';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UboDispatchItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UboDispatchItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UboDispatchItem.displayName = 'proto.ubo.v1.UboDispatchItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UboDispatchItem.StoreAction2 = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UboDispatchItem.StoreAction2.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.UboDispatchItem.StoreAction2, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UboDispatchItem.StoreAction2.displayName = 'proto.ubo.v1.UboDispatchItem.StoreAction2';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UboDispatchItem.StoreAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_);
};
goog.inherits(proto.ubo.v1.UboDispatchItem.StoreAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UboDispatchItem.StoreAction.displayName = 'proto.ubo.v1.UboDispatchItem.StoreAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UboApplicationItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UboApplicationItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UboApplicationItem.displayName = 'proto.ubo.v1.UboApplicationItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UboApplicationItem.Application = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_);
};
goog.inherits(proto.ubo.v1.UboApplicationItem.Application, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UboApplicationItem.Application.displayName = 'proto.ubo.v1.UboApplicationItem.Application';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UboApplicationItem.InitializationArgs.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.UboApplicationItem.InitializationArgs, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UboApplicationItem.InitializationArgs.displayName = 'proto.ubo.v1.UboApplicationItem.InitializationArgs';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.UboApplicationItem.InitializationKwargsDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.displayName = 'proto.ubo.v1.UboApplicationItem.InitializationKwargsDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.BasicTypeOptional = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.BasicTypeOptional.oneofGroups_);
};
goog.inherits(proto.ubo.v1.BasicTypeOptional, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.BasicTypeOptional.displayName = 'proto.ubo.v1.BasicTypeOptional';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.BasicType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.BasicType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.BasicType.displayName = 'proto.ubo.v1.BasicType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputResult = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputResult, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputResult.displayName = 'proto.ubo.v1.InputResult';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputFieldDescription = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputFieldDescription, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputFieldDescription.displayName = 'proto.ubo.v1.InputFieldDescription';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputFieldDescription.Options = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.InputFieldDescription.Options.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.InputFieldDescription.Options, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputFieldDescription.Options.displayName = 'proto.ubo.v1.InputFieldDescription.Options';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputDescription = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputDescription, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputDescription.displayName = 'proto.ubo.v1.InputDescription';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WebUIInputDescription = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WebUIInputDescription, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WebUIInputDescription.displayName = 'proto.ubo.v1.WebUIInputDescription';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WebUIInputDescription.Fields = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.WebUIInputDescription.Fields.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.WebUIInputDescription.Fields, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WebUIInputDescription.Fields.displayName = 'proto.ubo.v1.WebUIInputDescription.Fields';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.QRCodeInputDescription = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.QRCodeInputDescription, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.QRCodeInputDescription.displayName = 'proto.ubo.v1.QRCodeInputDescription';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.PathInputDescription = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.PathInputDescription, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.PathInputDescription.displayName = 'proto.ubo.v1.PathInputDescription';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputAction.displayName = 'proto.ubo.v1.InputAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputDemandAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputDemandAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputDemandAction.displayName = 'proto.ubo.v1.InputDemandAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputResolveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputResolveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputResolveAction.displayName = 'proto.ubo.v1.InputResolveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputCancelAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputCancelAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputCancelAction.displayName = 'proto.ubo.v1.InputCancelAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputProvideAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputProvideAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputProvideAction.displayName = 'proto.ubo.v1.InputProvideAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputResolveEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputResolveEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputResolveEvent.displayName = 'proto.ubo.v1.InputResolveEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputCancelEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputCancelEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputCancelEvent.displayName = 'proto.ubo.v1.InputCancelEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InputProvideEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InputProvideEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InputProvideEvent.displayName = 'proto.ubo.v1.InputProvideEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AvailableInputDescription = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.AvailableInputDescription.oneofGroups_);
};
goog.inherits(proto.ubo.v1.AvailableInputDescription, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AvailableInputDescription.displayName = 'proto.ubo.v1.AvailableInputDescription';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsAction.displayName = 'proto.ubo.v1.SettingsAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsTogglePdbSignalAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsTogglePdbSignalAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsTogglePdbSignalAction.displayName = 'proto.ubo.v1.SettingsTogglePdbSignalAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsToggleVisualDebugAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsToggleVisualDebugAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsToggleVisualDebugAction.displayName = 'proto.ubo.v1.SettingsToggleVisualDebugAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsToggleBetaVersionsAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsToggleBetaVersionsAction.displayName = 'proto.ubo.v1.SettingsToggleBetaVersionsAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsSetServicesAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsSetServicesAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsSetServicesAction.displayName = 'proto.ubo.v1.SettingsSetServicesAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsServiceAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsServiceAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsServiceAction.displayName = 'proto.ubo.v1.SettingsServiceAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsStartServiceAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsStartServiceAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsStartServiceAction.displayName = 'proto.ubo.v1.SettingsStartServiceAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsStopServiceAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsStopServiceAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsStopServiceAction.displayName = 'proto.ubo.v1.SettingsStopServiceAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsServiceSetStatusAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsServiceSetStatusAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsServiceSetStatusAction.displayName = 'proto.ubo.v1.SettingsServiceSetStatusAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsServiceSetIsEnabledAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsServiceSetIsEnabledAction.displayName = 'proto.ubo.v1.SettingsServiceSetIsEnabledAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsServiceSetLogLevelAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsServiceSetLogLevelAction.displayName = 'proto.ubo.v1.SettingsServiceSetLogLevelAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsServiceSetShouldRestartAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsServiceSetShouldRestartAction.displayName = 'proto.ubo.v1.SettingsServiceSetShouldRestartAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsReportServiceErrorAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsReportServiceErrorAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsReportServiceErrorAction.displayName = 'proto.ubo.v1.SettingsReportServiceErrorAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsClearServiceErrorsAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsClearServiceErrorsAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsClearServiceErrorsAction.displayName = 'proto.ubo.v1.SettingsClearServiceErrorsAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsEvent.displayName = 'proto.ubo.v1.SettingsEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsServiceEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsServiceEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsServiceEvent.displayName = 'proto.ubo.v1.SettingsServiceEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsStartServiceEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsStartServiceEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsStartServiceEvent.displayName = 'proto.ubo.v1.SettingsStartServiceEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsStopServiceEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsStopServiceEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsStopServiceEvent.displayName = 'proto.ubo.v1.SettingsStopServiceEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ErrorReport = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ErrorReport, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ErrorReport.displayName = 'proto.ubo.v1.ErrorReport';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ServiceState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ServiceState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ServiceState.displayName = 'proto.ubo.v1.ServiceState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ServiceState.Errors = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.ServiceState.Errors.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.ServiceState.Errors, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ServiceState.Errors.displayName = 'proto.ubo.v1.ServiceState.Errors';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SettingsState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsState.displayName = 'proto.ubo.v1.SettingsState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SettingsState.ServicesDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.SettingsState.ServicesDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SettingsState.ServicesDict.displayName = 'proto.ubo.v1.SettingsState.ServicesDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.IconState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.IconState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.IconState.displayName = 'proto.ubo.v1.IconState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.StatusIconsState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.StatusIconsState.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.StatusIconsState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.StatusIconsState.displayName = 'proto.ubo.v1.StatusIconsState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.StatusIconsAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.StatusIconsAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.StatusIconsAction.displayName = 'proto.ubo.v1.StatusIconsAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.StatusIconsRegisterAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.StatusIconsRegisterAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.StatusIconsRegisterAction.displayName = 'proto.ubo.v1.StatusIconsRegisterAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerAction.displayName = 'proto.ubo.v1.UpdateManagerAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerSetVersionsAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerSetVersionsAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerSetVersionsAction.displayName = 'proto.ubo.v1.UpdateManagerSetVersionsAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.displayName = 'proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerRequestCheckAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerRequestCheckAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerRequestCheckAction.displayName = 'proto.ubo.v1.UpdateManagerRequestCheckAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerReportFailedCheckAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerReportFailedCheckAction.displayName = 'proto.ubo.v1.UpdateManagerReportFailedCheckAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerRequestUpdateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerRequestUpdateAction.displayName = 'proto.ubo.v1.UpdateManagerRequestUpdateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerEvent.displayName = 'proto.ubo.v1.UpdateManagerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerCheckEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerCheckEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerCheckEvent.displayName = 'proto.ubo.v1.UpdateManagerCheckEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerUpdateEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerUpdateEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerUpdateEvent.displayName = 'proto.ubo.v1.UpdateManagerUpdateEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerState.displayName = 'proto.ubo.v1.UpdateManagerState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UpdateManagerState.RecentVersions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UpdateManagerState.RecentVersions.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.UpdateManagerState.RecentVersions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UpdateManagerState.RecentVersions.displayName = 'proto.ubo.v1.UpdateManagerState.RecentVersions';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.McpServerMetadata = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.McpServerMetadata, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.McpServerMetadata.displayName = 'proto.ubo.v1.McpServerMetadata';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantAction.displayName = 'proto.ubo.v1.AssistantAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantSetIsActiveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantSetIsActiveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantSetIsActiveAction.displayName = 'proto.ubo.v1.AssistantSetIsActiveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantSetSelectedSTTAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantSetSelectedSTTAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantSetSelectedSTTAction.displayName = 'proto.ubo.v1.AssistantSetSelectedSTTAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantSetSelectedLLMAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantSetSelectedLLMAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantSetSelectedLLMAction.displayName = 'proto.ubo.v1.AssistantSetSelectedLLMAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantSetSelectedTTSAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantSetSelectedTTSAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantSetSelectedTTSAction.displayName = 'proto.ubo.v1.AssistantSetSelectedTTSAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantSetSelectedImageGeneratorAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.displayName = 'proto.ubo.v1.AssistantSetSelectedImageGeneratorAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantSetSelectedModelAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantSetSelectedModelAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantSetSelectedModelAction.displayName = 'proto.ubo.v1.AssistantSetSelectedModelAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantDownloadOllamaModelAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantDownloadOllamaModelAction.displayName = 'proto.ubo.v1.AssistantDownloadOllamaModelAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistanceFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistanceFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistanceFrame.displayName = 'proto.ubo.v1.AssistanceFrame';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistanceTextFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistanceTextFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistanceTextFrame.displayName = 'proto.ubo.v1.AssistanceTextFrame';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistanceAudioFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistanceAudioFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistanceAudioFrame.displayName = 'proto.ubo.v1.AssistanceAudioFrame';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistanceImageFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistanceImageFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistanceImageFrame.displayName = 'proto.ubo.v1.AssistanceImageFrame';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantReportAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantReportAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantReportAction.displayName = 'proto.ubo.v1.AssistantReportAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantStartListeningAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantStartListeningAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantStartListeningAction.displayName = 'proto.ubo.v1.AssistantStartListeningAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantStopListeningAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantStopListeningAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantStopListeningAction.displayName = 'proto.ubo.v1.AssistantStopListeningAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantToggleListeningAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantToggleListeningAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantToggleListeningAction.displayName = 'proto.ubo.v1.AssistantToggleListeningAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantUpdateProvidersAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantUpdateProvidersAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantUpdateProvidersAction.displayName = 'proto.ubo.v1.AssistantUpdateProvidersAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantAddMcpServerAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantAddMcpServerAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantAddMcpServerAction.displayName = 'proto.ubo.v1.AssistantAddMcpServerAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantToggleMcpServerAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantToggleMcpServerAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantToggleMcpServerAction.displayName = 'proto.ubo.v1.AssistantToggleMcpServerAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantDeleteMcpServerAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantDeleteMcpServerAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantDeleteMcpServerAction.displayName = 'proto.ubo.v1.AssistantDeleteMcpServerAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantSyncMcpServersAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantSyncMcpServersAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantSyncMcpServersAction.displayName = 'proto.ubo.v1.AssistantSyncMcpServersAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantEvent.displayName = 'proto.ubo.v1.AssistantEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantDownloadOllamaModelEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantDownloadOllamaModelEvent.displayName = 'proto.ubo.v1.AssistantDownloadOllamaModelEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantHandleReportEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantHandleReportEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantHandleReportEvent.displayName = 'proto.ubo.v1.AssistantHandleReportEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantUpdateProvidersEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantUpdateProvidersEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantUpdateProvidersEvent.displayName = 'proto.ubo.v1.AssistantUpdateProvidersEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantAddMcpServerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantAddMcpServerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantAddMcpServerEvent.displayName = 'proto.ubo.v1.AssistantAddMcpServerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantDeleteMcpServerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantDeleteMcpServerEvent.displayName = 'proto.ubo.v1.AssistantDeleteMcpServerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AssistantState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantState.displayName = 'proto.ubo.v1.AssistantState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantState.SelectedModelsDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.AssistantState.SelectedModelsDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantState.SelectedModelsDict.displayName = 'proto.ubo.v1.AssistantState.SelectedModelsDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantState.McpServersDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.AssistantState.McpServersDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantState.McpServersDict.displayName = 'proto.ubo.v1.AssistantState.McpServersDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantState.EnabledMcpServers = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.AssistantState.EnabledMcpServers.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.AssistantState.EnabledMcpServers, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantState.EnabledMcpServers.displayName = 'proto.ubo.v1.AssistantState.EnabledMcpServers';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.AssistantState.ProviderSetupStatusDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AssistantState.ProviderSetupStatusDict.displayName = 'proto.ubo.v1.AssistantState.ProviderSetupStatusDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AcceptableAssistanceFrame = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_);
};
goog.inherits(proto.ubo.v1.AcceptableAssistanceFrame, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AcceptableAssistanceFrame.displayName = 'proto.ubo.v1.AcceptableAssistanceFrame';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioAction.displayName = 'proto.ubo.v1.AudioAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioInstallDriverAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioInstallDriverAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioInstallDriverAction.displayName = 'proto.ubo.v1.AudioInstallDriverAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioSetVolumeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioSetVolumeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioSetVolumeAction.displayName = 'proto.ubo.v1.AudioSetVolumeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioChangeVolumeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioChangeVolumeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioChangeVolumeAction.displayName = 'proto.ubo.v1.AudioChangeVolumeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioSetMuteStatusAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioSetMuteStatusAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioSetMuteStatusAction.displayName = 'proto.ubo.v1.AudioSetMuteStatusAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioToggleMuteStatusAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioToggleMuteStatusAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioToggleMuteStatusAction.displayName = 'proto.ubo.v1.AudioToggleMuteStatusAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlayChimeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlayChimeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlayChimeAction.displayName = 'proto.ubo.v1.AudioPlayChimeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlayAudioSampleAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlayAudioSampleAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlayAudioSampleAction.displayName = 'proto.ubo.v1.AudioPlayAudioSampleAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlayAudioSequenceAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlayAudioSequenceAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlayAudioSequenceAction.displayName = 'proto.ubo.v1.AudioPlayAudioSequenceAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioSample = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioSample, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioSample.displayName = 'proto.ubo.v1.AudioSample';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioReportSampleAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioReportSampleAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioReportSampleAction.displayName = 'proto.ubo.v1.AudioReportSampleAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlaybackDoneAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlaybackDoneAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlaybackDoneAction.displayName = 'proto.ubo.v1.AudioPlaybackDoneAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioStartRecordingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioStartRecordingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioStartRecordingAction.displayName = 'proto.ubo.v1.AudioStartRecordingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioStopRecordingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioStopRecordingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioStopRecordingAction.displayName = 'proto.ubo.v1.AudioStopRecordingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioToggleRecordingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioToggleRecordingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioToggleRecordingAction.displayName = 'proto.ubo.v1.AudioToggleRecordingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlayRecordingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlayRecordingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlayRecordingAction.displayName = 'proto.ubo.v1.AudioPlayRecordingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioEvent.displayName = 'proto.ubo.v1.AudioEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioReportSampleEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioReportSampleEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioReportSampleEvent.displayName = 'proto.ubo.v1.AudioReportSampleEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioInstallDriverEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioInstallDriverEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioInstallDriverEvent.displayName = 'proto.ubo.v1.AudioInstallDriverEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlayChimeEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlayChimeEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlayChimeEvent.displayName = 'proto.ubo.v1.AudioPlayChimeEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlayAudioSampleEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlayAudioSampleEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlayAudioSampleEvent.displayName = 'proto.ubo.v1.AudioPlayAudioSampleEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlayAudioSequenceEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlayAudioSequenceEvent.displayName = 'proto.ubo.v1.AudioPlayAudioSequenceEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioPlaybackDoneEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioPlaybackDoneEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioPlaybackDoneEvent.displayName = 'proto.ubo.v1.AudioPlaybackDoneEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.AudioState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.AudioState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.AudioState.displayName = 'proto.ubo.v1.AudioState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraAction.displayName = 'proto.ubo.v1.CameraAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraStartViewfinderAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraStartViewfinderAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraStartViewfinderAction.displayName = 'proto.ubo.v1.CameraStartViewfinderAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraReportBarcodeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraReportBarcodeAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.CameraReportBarcodeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraReportBarcodeAction.displayName = 'proto.ubo.v1.CameraReportBarcodeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraInstallDriverAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraInstallDriverAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraInstallDriverAction.displayName = 'proto.ubo.v1.CameraInstallDriverAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraEvent.displayName = 'proto.ubo.v1.CameraEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraInstallDriverEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraInstallDriverEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraInstallDriverEvent.displayName = 'proto.ubo.v1.CameraInstallDriverEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraRestoreDefaultAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraRestoreDefaultAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraRestoreDefaultAction.displayName = 'proto.ubo.v1.CameraRestoreDefaultAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraRestoreDefaultEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraRestoreDefaultEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraRestoreDefaultEvent.displayName = 'proto.ubo.v1.CameraRestoreDefaultEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraStartViewfinderEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraStartViewfinderEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraStartViewfinderEvent.displayName = 'proto.ubo.v1.CameraStartViewfinderEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraReportImageEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraReportImageEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraReportImageEvent.displayName = 'proto.ubo.v1.CameraReportImageEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraStopViewfinderEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraStopViewfinderEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraStopViewfinderEvent.displayName = 'proto.ubo.v1.CameraStopViewfinderEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraSetIndexAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraSetIndexAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraSetIndexAction.displayName = 'proto.ubo.v1.CameraSetIndexAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraDetectAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraDetectAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraDetectAction.displayName = 'proto.ubo.v1.CameraDetectAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraSetAvailableCamerasAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraSetAvailableCamerasAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.CameraSetAvailableCamerasAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraSetAvailableCamerasAction.displayName = 'proto.ubo.v1.CameraSetAvailableCamerasAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraDetectEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraDetectEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraDetectEvent.displayName = 'proto.ubo.v1.CameraDetectEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraDetectedEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraDetectedEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.CameraDetectedEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraDetectedEvent.displayName = 'proto.ubo.v1.CameraDetectedEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraReinitializeEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.CameraReinitializeEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraReinitializeEvent.displayName = 'proto.ubo.v1.CameraReinitializeEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.CameraState.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.CameraState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraState.displayName = 'proto.ubo.v1.CameraState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.CameraState.AvailableCameras = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.CameraState.AvailableCameras.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.CameraState.AvailableCameras, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.CameraState.AvailableCameras.displayName = 'proto.ubo.v1.CameraState.AvailableCameras';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayAction.displayName = 'proto.ubo.v1.DisplayAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayEvent.displayName = 'proto.ubo.v1.DisplayEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayPauseAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayPauseAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayPauseAction.displayName = 'proto.ubo.v1.DisplayPauseAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayResumeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayResumeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayResumeAction.displayName = 'proto.ubo.v1.DisplayResumeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayRedrawAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayRedrawAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayRedrawAction.displayName = 'proto.ubo.v1.DisplayRedrawAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayBlankAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayBlankAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayBlankAction.displayName = 'proto.ubo.v1.DisplayBlankAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayUnblankAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayUnblankAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayUnblankAction.displayName = 'proto.ubo.v1.DisplayUnblankAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayUpdateActivityAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayUpdateActivityAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayUpdateActivityAction.displayName = 'proto.ubo.v1.DisplayUpdateActivityAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplaySetBlankTimeoutAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplaySetBlankTimeoutAction.displayName = 'proto.ubo.v1.DisplaySetBlankTimeoutAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayRedrawEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayRedrawEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayRedrawEvent.displayName = 'proto.ubo.v1.DisplayRedrawEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayBlankEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayBlankEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayBlankEvent.displayName = 'proto.ubo.v1.DisplayBlankEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayUnblankEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayUnblankEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayUnblankEvent.displayName = 'proto.ubo.v1.DisplayUnblankEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayRenderEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.DisplayRenderEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.DisplayRenderEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayRenderEvent.displayName = 'proto.ubo.v1.DisplayRenderEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayCompressedRenderEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.DisplayCompressedRenderEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.DisplayCompressedRenderEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayCompressedRenderEvent.displayName = 'proto.ubo.v1.DisplayCompressedRenderEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DisplayState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DisplayState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DisplayState.displayName = 'proto.ubo.v1.DisplayState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerAction.displayName = 'proto.ubo.v1.DockerAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerInstallAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerInstallAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerInstallAction.displayName = 'proto.ubo.v1.DockerInstallAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerStartAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerStartAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerStartAction.displayName = 'proto.ubo.v1.DockerStartAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerStopAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerStopAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerStopAction.displayName = 'proto.ubo.v1.DockerStopAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerSetStatusAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerSetStatusAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerSetStatusAction.displayName = 'proto.ubo.v1.DockerSetStatusAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerStoreUsernameAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerStoreUsernameAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerStoreUsernameAction.displayName = 'proto.ubo.v1.DockerStoreUsernameAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerRemoveUsernameAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerRemoveUsernameAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerRemoveUsernameAction.displayName = 'proto.ubo.v1.DockerRemoveUsernameAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageAction.displayName = 'proto.ubo.v1.DockerImageAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageSetStatusAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageSetStatusAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageSetStatusAction.displayName = 'proto.ubo.v1.DockerImageSetStatusAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.DockerImageSetStatusAction.Ports.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.DockerImageSetStatusAction.Ports, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageSetStatusAction.Ports.displayName = 'proto.ubo.v1.DockerImageSetStatusAction.Ports';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageSetDockerIdAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageSetDockerIdAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageSetDockerIdAction.displayName = 'proto.ubo.v1.DockerImageSetDockerIdAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageUpdateMetadataAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageUpdateMetadataAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageUpdateMetadataAction.displayName = 'proto.ubo.v1.DockerImageUpdateMetadataAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageFetchAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageFetchAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageFetchAction.displayName = 'proto.ubo.v1.DockerImageFetchAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRemoveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRemoveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRemoveAction.displayName = 'proto.ubo.v1.DockerImageRemoveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRunAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRunAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRunAction.displayName = 'proto.ubo.v1.DockerImageRunAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageStopAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageStopAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageStopAction.displayName = 'proto.ubo.v1.DockerImageStopAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageReleaseAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageReleaseAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageReleaseAction.displayName = 'proto.ubo.v1.DockerImageReleaseAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRemoveContainerAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRemoveContainerAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRemoveContainerAction.displayName = 'proto.ubo.v1.DockerImageRemoveContainerAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerEvent.displayName = 'proto.ubo.v1.DockerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerInstallEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerInstallEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerInstallEvent.displayName = 'proto.ubo.v1.DockerInstallEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerStartEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerStartEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerStartEvent.displayName = 'proto.ubo.v1.DockerStartEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerStopEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerStopEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerStopEvent.displayName = 'proto.ubo.v1.DockerStopEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerServiceState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerServiceState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerServiceState.displayName = 'proto.ubo.v1.DockerServiceState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerServiceState.UsernamesDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.DockerServiceState.UsernamesDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerServiceState.UsernamesDict.displayName = 'proto.ubo.v1.DockerServiceState.UsernamesDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageEvent.displayName = 'proto.ubo.v1.DockerImageEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRegisterAppEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRegisterAppEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRegisterAppEvent.displayName = 'proto.ubo.v1.DockerImageRegisterAppEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageFetchCompositionEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageFetchCompositionEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageFetchCompositionEvent.displayName = 'proto.ubo.v1.DockerImageFetchCompositionEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageFetchEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageFetchEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageFetchEvent.displayName = 'proto.ubo.v1.DockerImageFetchEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRemoveCompositionEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRemoveCompositionEvent.displayName = 'proto.ubo.v1.DockerImageRemoveCompositionEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRemoveEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRemoveEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRemoveEvent.displayName = 'proto.ubo.v1.DockerImageRemoveEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRunCompositionEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRunCompositionEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRunCompositionEvent.displayName = 'proto.ubo.v1.DockerImageRunCompositionEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRunContainerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRunContainerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRunContainerEvent.displayName = 'proto.ubo.v1.DockerImageRunContainerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageStopCompositionEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageStopCompositionEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageStopCompositionEvent.displayName = 'proto.ubo.v1.DockerImageStopCompositionEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageStopContainerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageStopContainerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageStopContainerEvent.displayName = 'proto.ubo.v1.DockerImageStopContainerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageReleaseCompositionEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageReleaseCompositionEvent.displayName = 'proto.ubo.v1.DockerImageReleaseCompositionEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerImageRemoveContainerEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerImageRemoveContainerEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerImageRemoveContainerEvent.displayName = 'proto.ubo.v1.DockerImageRemoveContainerEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ImageState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ImageState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ImageState.displayName = 'proto.ubo.v1.ImageState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ImageState.Ports = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.ImageState.Ports.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.ImageState.Ports, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ImageState.Ports.displayName = 'proto.ubo.v1.ImageState.Ports';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.DockerState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.DockerState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.DockerState.displayName = 'proto.ubo.v1.DockerState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.PathSelectorConfig = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.PathSelectorConfig, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.PathSelectorConfig.displayName = 'proto.ubo.v1.PathSelectorConfig';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.displayName = 'proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.FileSystemAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemAction.displayName = 'proto.ubo.v1.FileSystemAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemReportSelectionAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.FileSystemReportSelectionAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemReportSelectionAction.displayName = 'proto.ubo.v1.FileSystemReportSelectionAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemCopyAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemCopyAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.FileSystemCopyAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemCopyAction.displayName = 'proto.ubo.v1.FileSystemCopyAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemMoveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemMoveAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.FileSystemMoveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemMoveAction.displayName = 'proto.ubo.v1.FileSystemMoveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemRemoveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemRemoveAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.FileSystemRemoveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemRemoveAction.displayName = 'proto.ubo.v1.FileSystemRemoveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.FileSystemEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemEvent.displayName = 'proto.ubo.v1.FileSystemEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemSelectEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.FileSystemSelectEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemSelectEvent.displayName = 'proto.ubo.v1.FileSystemSelectEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemCopyEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemCopyEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.FileSystemCopyEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemCopyEvent.displayName = 'proto.ubo.v1.FileSystemCopyEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemMoveEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemMoveEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.FileSystemMoveEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemMoveEvent.displayName = 'proto.ubo.v1.FileSystemMoveEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemRemoveEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemRemoveEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.FileSystemRemoveEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemRemoveEvent.displayName = 'proto.ubo.v1.FileSystemRemoveEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.FileSystemState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.FileSystemState.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.FileSystemState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.FileSystemState.displayName = 'proto.ubo.v1.FileSystemState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredAction.displayName = 'proto.ubo.v1.InfraredAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredEvent.displayName = 'proto.ubo.v1.InfraredEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredHandleReceivedCodeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredHandleReceivedCodeAction.displayName = 'proto.ubo.v1.InfraredHandleReceivedCodeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredSendCodeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredSendCodeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredSendCodeAction.displayName = 'proto.ubo.v1.InfraredSendCodeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredSetShouldPropagateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredSetShouldPropagateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredSetShouldPropagateAction.displayName = 'proto.ubo.v1.InfraredSetShouldPropagateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredSetShouldReceiveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredSetShouldReceiveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredSetShouldReceiveAction.displayName = 'proto.ubo.v1.InfraredSetShouldReceiveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredSendCodeEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredSendCodeEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredSendCodeEvent.displayName = 'proto.ubo.v1.InfraredSendCodeEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.InfraredState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.InfraredState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.InfraredState.displayName = 'proto.ubo.v1.InfraredState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.IpAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.IpAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.IpAction.displayName = 'proto.ubo.v1.IpAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.IpEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.IpEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.IpEvent.displayName = 'proto.ubo.v1.IpEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.IpUpdateInterfacesAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.IpUpdateInterfacesAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.IpUpdateInterfacesAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.IpUpdateInterfacesAction.displayName = 'proto.ubo.v1.IpUpdateInterfacesAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.IpSetIsConnectedAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.IpSetIsConnectedAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.IpSetIsConnectedAction.displayName = 'proto.ubo.v1.IpSetIsConnectedAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.IpNetworkInterface = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.IpNetworkInterface.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.IpNetworkInterface, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.IpNetworkInterface.displayName = 'proto.ubo.v1.IpNetworkInterface';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.IpState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.IpState.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.IpState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.IpState.displayName = 'proto.ubo.v1.IpState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.KeypadAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadAction.displayName = 'proto.ubo.v1.KeypadAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadAction.PressedKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadAction.PressedKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadAction.PressedKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadAction.HeldKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadAction.HeldKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadAction.HeldKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyPressAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyPressAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyPressAction.displayName = 'proto.ubo.v1.KeypadKeyPressAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyHoldAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyHoldAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyHoldAction.displayName = 'proto.ubo.v1.KeypadKeyHoldAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyUnholdAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyUnholdAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyUnholdAction.displayName = 'proto.ubo.v1.KeypadKeyUnholdAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyReleaseAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyReleaseAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyReleaseAction.displayName = 'proto.ubo.v1.KeypadKeyReleaseAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.displayName = 'proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.KeypadState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.KeypadState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.KeypadState.displayName = 'proto.ubo.v1.KeypadState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.LightDMAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.LightDMAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.LightDMAction.displayName = 'proto.ubo.v1.LightDMAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.LightDMUpdateStateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.LightDMUpdateStateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.LightDMUpdateStateAction.displayName = 'proto.ubo.v1.LightDMUpdateStateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.LightDMClearEnabledStateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.LightDMClearEnabledStateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.LightDMClearEnabledStateAction.displayName = 'proto.ubo.v1.LightDMClearEnabledStateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.LightDMState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.LightDMState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.LightDMState.displayName = 'proto.ubo.v1.LightDMState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationActionItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationActionItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationActionItem.displayName = 'proto.ubo.v1.NotificationActionItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationApplicationItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationApplicationItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationApplicationItem.displayName = 'proto.ubo.v1.NotificationApplicationItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationApplicationItem.Application = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_);
};
goog.inherits(proto.ubo.v1.NotificationApplicationItem.Application, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationApplicationItem.Application.displayName = 'proto.ubo.v1.NotificationApplicationItem.Application';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.NotificationApplicationItem.InitializationArgs.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.NotificationApplicationItem.InitializationArgs, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationApplicationItem.InitializationArgs.displayName = 'proto.ubo.v1.NotificationApplicationItem.InitializationArgs';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.displayName = 'proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationDispatchItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationDispatchItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationDispatchItem.displayName = 'proto.ubo.v1.NotificationDispatchItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2 = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.NotificationDispatchItem.StoreAction2.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.NotificationDispatchItem.StoreAction2, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationDispatchItem.StoreAction2.displayName = 'proto.ubo.v1.NotificationDispatchItem.StoreAction2';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_);
};
goog.inherits(proto.ubo.v1.NotificationDispatchItem.StoreAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationDispatchItem.StoreAction.displayName = 'proto.ubo.v1.NotificationDispatchItem.StoreAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Notification = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.Notification, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Notification.displayName = 'proto.ubo.v1.Notification';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Notification.ActionsItem = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Notification.ActionsItem.oneofGroups_);
};
goog.inherits(proto.ubo.v1.Notification.ActionsItem, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Notification.ActionsItem.displayName = 'proto.ubo.v1.Notification.ActionsItem';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Notification.Actions = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.Notification.Actions.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.Notification.Actions, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Notification.Actions.displayName = 'proto.ubo.v1.Notification.Actions';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Notification.OnClose = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, null);
};
goog.inherits(proto.ubo.v1.Notification.OnClose, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Notification.OnClose.displayName = 'proto.ubo.v1.Notification.OnClose';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsAction.displayName = 'proto.ubo.v1.NotificationsAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsAddAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsAddAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsAddAction.displayName = 'proto.ubo.v1.NotificationsAddAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsDisplayAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsDisplayAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsDisplayAction.displayName = 'proto.ubo.v1.NotificationsDisplayAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsClearAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsClearAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsClearAction.displayName = 'proto.ubo.v1.NotificationsClearAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsClearByIdAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsClearByIdAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsClearByIdAction.displayName = 'proto.ubo.v1.NotificationsClearByIdAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsClearAllAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsClearAllAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsClearAllAction.displayName = 'proto.ubo.v1.NotificationsClearAllAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsEvent.displayName = 'proto.ubo.v1.NotificationsEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsClearEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsClearEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsClearEvent.displayName = 'proto.ubo.v1.NotificationsClearEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsDisplayEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.NotificationsDisplayEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsDisplayEvent.displayName = 'proto.ubo.v1.NotificationsDisplayEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.NotificationsState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.NotificationsState.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.NotificationsState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.NotificationsState.displayName = 'proto.ubo.v1.NotificationsState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingAction.displayName = 'proto.ubo.v1.RgbRingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingEvent.displayName = 'proto.ubo.v1.RgbRingEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingSetIsBusyAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingSetIsBusyAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingSetIsBusyAction.displayName = 'proto.ubo.v1.RgbRingSetIsBusyAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingCommandAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingCommandAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingCommandAction.displayName = 'proto.ubo.v1.RgbRingCommandAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingWaitableCommandAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingWaitableCommandAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingWaitableCommandAction.displayName = 'proto.ubo.v1.RgbRingWaitableCommandAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingColorfulCommandAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingColorfulCommandAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingColorfulCommandAction.displayName = 'proto.ubo.v1.RgbRingColorfulCommandAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingSetEnabledAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingSetEnabledAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingSetEnabledAction.displayName = 'proto.ubo.v1.RgbRingSetEnabledAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingSetAllAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingSetAllAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingSetAllAction.displayName = 'proto.ubo.v1.RgbRingSetAllAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingSetBrightnessAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingSetBrightnessAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingSetBrightnessAction.displayName = 'proto.ubo.v1.RgbRingSetBrightnessAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingBlankAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingBlankAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingBlankAction.displayName = 'proto.ubo.v1.RgbRingBlankAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingRainbowAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingRainbowAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingRainbowAction.displayName = 'proto.ubo.v1.RgbRingRainbowAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingProgressWheelStepAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingProgressWheelStepAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingProgressWheelStepAction.displayName = 'proto.ubo.v1.RgbRingProgressWheelStepAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingPulseAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingPulseAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingPulseAction.displayName = 'proto.ubo.v1.RgbRingPulseAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingBlinkAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingBlinkAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingBlinkAction.displayName = 'proto.ubo.v1.RgbRingBlinkAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingSpinningWheelAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingSpinningWheelAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingSpinningWheelAction.displayName = 'proto.ubo.v1.RgbRingSpinningWheelAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingProgressWheelAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingProgressWheelAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingProgressWheelAction.displayName = 'proto.ubo.v1.RgbRingProgressWheelAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingFillUptoAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingFillUptoAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingFillUptoAction.displayName = 'proto.ubo.v1.RgbRingFillUptoAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingFillDownfromAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingFillDownfromAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingFillDownfromAction.displayName = 'proto.ubo.v1.RgbRingFillDownfromAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingSequenceAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.RgbRingSequenceAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.RgbRingSequenceAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingSequenceAction.displayName = 'proto.ubo.v1.RgbRingSequenceAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingCommandEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.RgbRingCommandEvent.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.RgbRingCommandEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingCommandEvent.displayName = 'proto.ubo.v1.RgbRingCommandEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbRingState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RgbRingState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbRingState.displayName = 'proto.ubo.v1.RgbRingState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbColorElement = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.RgbColorElement.oneofGroups_);
};
goog.inherits(proto.ubo.v1.RgbColorElement, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbColorElement.displayName = 'proto.ubo.v1.RgbColorElement';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RgbColor = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.RgbColor.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.RgbColor, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RgbColor.displayName = 'proto.ubo.v1.RgbColor';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectAction.displayName = 'proto.ubo.v1.RPiConnectAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectEvent.displayName = 'proto.ubo.v1.RPiConnectEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectStartDownloadingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectStartDownloadingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectStartDownloadingAction.displayName = 'proto.ubo.v1.RPiConnectStartDownloadingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectDoneDownloadingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectDoneDownloadingAction.displayName = 'proto.ubo.v1.RPiConnectDoneDownloadingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectSetPendingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectSetPendingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectSetPendingAction.displayName = 'proto.ubo.v1.RPiConnectSetPendingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectStatus = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectStatus, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectStatus.displayName = 'proto.ubo.v1.RPiConnectStatus';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectSetStatusAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectSetStatusAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectSetStatusAction.displayName = 'proto.ubo.v1.RPiConnectSetStatusAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectLoginEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectLoginEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectLoginEvent.displayName = 'proto.ubo.v1.RPiConnectLoginEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectUpdateServiceStateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectUpdateServiceStateAction.displayName = 'proto.ubo.v1.RPiConnectUpdateServiceStateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.RPiConnectState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.RPiConnectState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.RPiConnectState.displayName = 'proto.ubo.v1.RPiConnectState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SensorsAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SensorsAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SensorsAction.displayName = 'proto.ubo.v1.SensorsAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SensorsReportReadingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SensorsReportReadingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SensorsReportReadingAction.displayName = 'proto.ubo.v1.SensorsReportReadingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SensorState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SensorState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SensorState.displayName = 'proto.ubo.v1.SensorState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SensorsState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SensorsState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SensorsState.displayName = 'proto.ubo.v1.SensorsState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionAction.displayName = 'proto.ubo.v1.SpeechRecognitionAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.displayName = 'proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.displayName = 'proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.displayName = 'proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionIntent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionIntent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionIntent.displayName = 'proto.ubo.v1.SpeechRecognitionIntent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1 = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.SpeechRecognitionIntent.Phrase1.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Phrase1, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionIntent.Phrase1.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Phrase1';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Phrase, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionIntent.Phrase.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Phrase';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2 = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.SpeechRecognitionIntent.Action2.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Action2, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionIntent.Action2.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Action2';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionIntent.Action = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionIntent.Action, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionIntent.Action.displayName = 'proto.ubo.v1.SpeechRecognitionIntent.Action';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.displayName = 'proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.displayName = 'proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionReportSpeechAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionReportSpeechAction.displayName = 'proto.ubo.v1.SpeechRecognitionReportSpeechAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionEvent.displayName = 'proto.ubo.v1.SpeechRecognitionEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionReportTextEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionReportTextEvent.displayName = 'proto.ubo.v1.SpeechRecognitionReportTextEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechRecognitionState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.SpeechRecognitionState.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.SpeechRecognitionState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechRecognitionState.displayName = 'proto.ubo.v1.SpeechRecognitionState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechSynthesisAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechSynthesisAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechSynthesisAction.displayName = 'proto.ubo.v1.SpeechSynthesisAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechSynthesisEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechSynthesisEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechSynthesisEvent.displayName = 'proto.ubo.v1.SpeechSynthesisEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.displayName = 'proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.displayName = 'proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.ReadableInformation = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.ReadableInformation, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.ReadableInformation.displayName = 'proto.ubo.v1.ReadableInformation';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechSynthesisReadTextAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechSynthesisReadTextAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechSynthesisReadTextAction.displayName = 'proto.ubo.v1.SpeechSynthesisReadTextAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.displayName = 'proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SpeechSynthesisState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SpeechSynthesisState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SpeechSynthesisState.displayName = 'proto.ubo.v1.SpeechSynthesisState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SSHAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SSHAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SSHAction.displayName = 'proto.ubo.v1.SSHAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SSHUpdateStateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SSHUpdateStateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SSHUpdateStateAction.displayName = 'proto.ubo.v1.SSHUpdateStateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SSHClearEnabledStateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SSHClearEnabledStateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SSHClearEnabledStateAction.displayName = 'proto.ubo.v1.SSHClearEnabledStateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.SSHState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.SSHState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.SSHState.displayName = 'proto.ubo.v1.SSHState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersAction.displayName = 'proto.ubo.v1.UsersAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersSetUsersAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.UsersSetUsersAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.UsersSetUsersAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersSetUsersAction.displayName = 'proto.ubo.v1.UsersSetUsersAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersCreateUserAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersCreateUserAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersCreateUserAction.displayName = 'proto.ubo.v1.UsersCreateUserAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersDeleteUserAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersDeleteUserAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersDeleteUserAction.displayName = 'proto.ubo.v1.UsersDeleteUserAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersResetPasswordAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersResetPasswordAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersResetPasswordAction.displayName = 'proto.ubo.v1.UsersResetPasswordAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersEvent.displayName = 'proto.ubo.v1.UsersEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersCreateUserEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersCreateUserEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersCreateUserEvent.displayName = 'proto.ubo.v1.UsersCreateUserEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersDeleteUserEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersDeleteUserEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersDeleteUserEvent.displayName = 'proto.ubo.v1.UsersDeleteUserEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersResetPasswordEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersResetPasswordEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersResetPasswordEvent.displayName = 'proto.ubo.v1.UsersResetPasswordEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UserState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UserState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UserState.displayName = 'proto.ubo.v1.UserState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.UsersState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersState.displayName = 'proto.ubo.v1.UsersState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.UsersState.Users = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.UsersState.Users.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.UsersState.Users, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.UsersState.Users.displayName = 'proto.ubo.v1.UsersState.Users';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeAction.displayName = 'proto.ubo.v1.VSCodeAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeEvent.displayName = 'proto.ubo.v1.VSCodeEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeStartDownloadingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeStartDownloadingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeStartDownloadingAction.displayName = 'proto.ubo.v1.VSCodeStartDownloadingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeDoneDownloadingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeDoneDownloadingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeDoneDownloadingAction.displayName = 'proto.ubo.v1.VSCodeDoneDownloadingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeSetPendingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeSetPendingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeSetPendingAction.displayName = 'proto.ubo.v1.VSCodeSetPendingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeStatus = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeStatus, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeStatus.displayName = 'proto.ubo.v1.VSCodeStatus';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeSetStatusAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeSetStatusAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeSetStatusAction.displayName = 'proto.ubo.v1.VSCodeSetStatusAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeLoginEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeLoginEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeLoginEvent.displayName = 'proto.ubo.v1.VSCodeLoginEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeRestartEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeRestartEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeRestartEvent.displayName = 'proto.ubo.v1.VSCodeRestartEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.VSCodeState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.VSCodeState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.VSCodeState.displayName = 'proto.ubo.v1.VSCodeState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WebUIEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WebUIEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WebUIEvent.displayName = 'proto.ubo.v1.WebUIEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WebUIInitializeEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WebUIInitializeEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WebUIInitializeEvent.displayName = 'proto.ubo.v1.WebUIInitializeEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WebUIStopEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WebUIStopEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WebUIStopEvent.displayName = 'proto.ubo.v1.WebUIStopEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WebUIState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.WebUIState.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.WebUIState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WebUIState.displayName = 'proto.ubo.v1.WebUIState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiConnection = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiConnection, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiConnection.displayName = 'proto.ubo.v1.WiFiConnection';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiAction.displayName = 'proto.ubo.v1.WiFiAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiInputConnectionAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiInputConnectionAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiInputConnectionAction.displayName = 'proto.ubo.v1.WiFiInputConnectionAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiSetHasVisitedOnboardingAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.displayName = 'proto.ubo.v1.WiFiSetHasVisitedOnboardingAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiUpdateAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, proto.ubo.v1.WiFiUpdateAction.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.WiFiUpdateAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiUpdateAction.displayName = 'proto.ubo.v1.WiFiUpdateAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiUpdateRequestAction = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiUpdateRequestAction, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiUpdateRequestAction.displayName = 'proto.ubo.v1.WiFiUpdateRequestAction';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiEvent.displayName = 'proto.ubo.v1.WiFiEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiInputConnectionEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiInputConnectionEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiInputConnectionEvent.displayName = 'proto.ubo.v1.WiFiInputConnectionEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiUpdateRequestEvent = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiUpdateRequestEvent, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiUpdateRequestEvent.displayName = 'proto.ubo.v1.WiFiUpdateRequestEvent';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiState = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, 500, null, null);
};
goog.inherits(proto.ubo.v1.WiFiState, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiState.displayName = 'proto.ubo.v1.WiFiState';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.WiFiState.Connections = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, proto.ubo.v1.WiFiState.Connections.repeatedFields_, null);
};
goog.inherits(proto.ubo.v1.WiFiState.Connections, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.WiFiState.Connections.displayName = 'proto.ubo.v1.WiFiState.Connections';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Action = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Action.oneofGroups_);
};
goog.inherits(proto.ubo.v1.Action, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Action.displayName = 'proto.ubo.v1.Action';
}
/**
 * Generated by JsPbCodeGenerator.
 * @param {Array=} opt_data Optional initial data array, typically from a
 * server response, or constructed directly in Javascript. The array is used
 * in place and becomes part of the constructed object. It is not cloned.
 * If no data is provided, the constructed object will be empty, but still
 * valid.
 * @extends {jspb.Message}
 * @constructor
 */
proto.ubo.v1.Event = function(opt_data) {
  jspb.Message.initialize(this, opt_data, 0, -1, null, proto.ubo.v1.Event.oneofGroups_);
};
goog.inherits(proto.ubo.v1.Event, jspb.Message);
if (goog.DEBUG && !COMPILED) {
  /**
   * @public
   * @override
   */
  proto.ubo.v1.Event.displayName = 'proto.ubo.v1.Event';
}

/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.BaseMenu.repeatedFields_ = [3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.BaseMenu.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.BaseMenu.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.BaseMenu} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.BaseMenu.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    title: jspb.Message.getFieldWithDefault(msg, 2, ""),
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.Item.toObject, includeInstance),
    placeholder: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.BaseMenu}
 */
proto.ubo.v1.BaseMenu.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.BaseMenu;
  return proto.ubo.v1.BaseMenu.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.BaseMenu} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.BaseMenu}
 */
proto.ubo.v1.BaseMenu.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 3:
      var value = new proto.ubo.v1.Item;
      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setPlaceholder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.BaseMenu.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.BaseMenu.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.BaseMenu} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.BaseMenu.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTitle();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.ubo.v1.Item.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.BaseMenu.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.BaseMenu} returns this
 */
proto.ubo.v1.BaseMenu.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.BaseMenu} returns this
 */
proto.ubo.v1.BaseMenu.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BaseMenu.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string title = 2;
 * @return {string}
 */
proto.ubo.v1.BaseMenu.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.BaseMenu} returns this
 */
proto.ubo.v1.BaseMenu.prototype.setTitle = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * repeated Item items = 3;
 * @return {!Array<!proto.ubo.v1.Item>}
 */
proto.ubo.v1.BaseMenu.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Item>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Item, 3));
};


/**
 * @param {!Array<!proto.ubo.v1.Item>} value
 * @return {!proto.ubo.v1.BaseMenu} returns this
*/
proto.ubo.v1.BaseMenu.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.ubo.v1.Item=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Item}
 */
proto.ubo.v1.BaseMenu.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.Item, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.BaseMenu} returns this
 */
proto.ubo.v1.BaseMenu.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string placeholder = 4;
 * @return {string}
 */
proto.ubo.v1.BaseMenu.prototype.getPlaceholder = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.BaseMenu} returns this
 */
proto.ubo.v1.BaseMenu.prototype.setPlaceholder = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.BaseMenu} returns this
 */
proto.ubo.v1.BaseMenu.prototype.clearPlaceholder = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BaseMenu.prototype.hasPlaceholder = function() {
  return jspb.Message.getField(this, 4) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.HeadedMenu.repeatedFields_ = [5];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.HeadedMenu.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.HeadedMenu.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.HeadedMenu} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.HeadedMenu.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    heading: jspb.Message.getFieldWithDefault(msg, 2, ""),
    subHeading: jspb.Message.getFieldWithDefault(msg, 3, ""),
    title: jspb.Message.getFieldWithDefault(msg, 4, ""),
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.Item.toObject, includeInstance),
    placeholder: jspb.Message.getFieldWithDefault(msg, 6, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.HeadedMenu}
 */
proto.ubo.v1.HeadedMenu.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.HeadedMenu;
  return proto.ubo.v1.HeadedMenu.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.HeadedMenu} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.HeadedMenu}
 */
proto.ubo.v1.HeadedMenu.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setHeading(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setSubHeading(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 5:
      var value = new proto.ubo.v1.Item;
      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setPlaceholder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.HeadedMenu.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.HeadedMenu.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.HeadedMenu} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.HeadedMenu.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getHeading();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getSubHeading();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getTitle();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      5,
      f,
      proto.ubo.v1.Item.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.HeadedMenu.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.HeadedMenu.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string heading = 2;
 * @return {string}
 */
proto.ubo.v1.HeadedMenu.prototype.getHeading = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.setHeading = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string sub_heading = 3;
 * @return {string}
 */
proto.ubo.v1.HeadedMenu.prototype.getSubHeading = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.setSubHeading = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional string title = 4;
 * @return {string}
 */
proto.ubo.v1.HeadedMenu.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.setTitle = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};


/**
 * repeated Item items = 5;
 * @return {!Array<!proto.ubo.v1.Item>}
 */
proto.ubo.v1.HeadedMenu.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Item>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Item, 5));
};


/**
 * @param {!Array<!proto.ubo.v1.Item>} value
 * @return {!proto.ubo.v1.HeadedMenu} returns this
*/
proto.ubo.v1.HeadedMenu.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 5, value);
};


/**
 * @param {!proto.ubo.v1.Item=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Item}
 */
proto.ubo.v1.HeadedMenu.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 5, opt_value, proto.ubo.v1.Item, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string placeholder = 6;
 * @return {string}
 */
proto.ubo.v1.HeadedMenu.prototype.getPlaceholder = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.setPlaceholder = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.HeadedMenu} returns this
 */
proto.ubo.v1.HeadedMenu.prototype.clearPlaceholder = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.HeadedMenu.prototype.hasPlaceholder = function() {
  return jspb.Message.getField(this, 6) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.HeadlessMenu.repeatedFields_ = [3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.HeadlessMenu.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.HeadlessMenu.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.HeadlessMenu} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.HeadlessMenu.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    title: jspb.Message.getFieldWithDefault(msg, 2, ""),
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.Item.toObject, includeInstance),
    placeholder: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.HeadlessMenu}
 */
proto.ubo.v1.HeadlessMenu.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.HeadlessMenu;
  return proto.ubo.v1.HeadlessMenu.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.HeadlessMenu} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.HeadlessMenu}
 */
proto.ubo.v1.HeadlessMenu.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 3:
      var value = new proto.ubo.v1.Item;
      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setPlaceholder(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.HeadlessMenu.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.HeadlessMenu.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.HeadlessMenu} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.HeadlessMenu.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTitle();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.ubo.v1.Item.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.HeadlessMenu.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadlessMenu} returns this
 */
proto.ubo.v1.HeadlessMenu.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.HeadlessMenu} returns this
 */
proto.ubo.v1.HeadlessMenu.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.HeadlessMenu.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string title = 2;
 * @return {string}
 */
proto.ubo.v1.HeadlessMenu.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadlessMenu} returns this
 */
proto.ubo.v1.HeadlessMenu.prototype.setTitle = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * repeated Item items = 3;
 * @return {!Array<!proto.ubo.v1.Item>}
 */
proto.ubo.v1.HeadlessMenu.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Item>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Item, 3));
};


/**
 * @param {!Array<!proto.ubo.v1.Item>} value
 * @return {!proto.ubo.v1.HeadlessMenu} returns this
*/
proto.ubo.v1.HeadlessMenu.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.ubo.v1.Item=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Item}
 */
proto.ubo.v1.HeadlessMenu.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.Item, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.HeadlessMenu} returns this
 */
proto.ubo.v1.HeadlessMenu.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string placeholder = 4;
 * @return {string}
 */
proto.ubo.v1.HeadlessMenu.prototype.getPlaceholder = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.HeadlessMenu} returns this
 */
proto.ubo.v1.HeadlessMenu.prototype.setPlaceholder = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.HeadlessMenu} returns this
 */
proto.ubo.v1.HeadlessMenu.prototype.clearPlaceholder = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.HeadlessMenu.prototype.hasPlaceholder = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Item.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Item.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Item} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Item.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    key: jspb.Message.getFieldWithDefault(msg, 2, ""),
    label: jspb.Message.getFieldWithDefault(msg, 3, ""),
    color: jspb.Message.getFieldWithDefault(msg, 4, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 5, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 6, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 8, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Item}
 */
proto.ubo.v1.Item.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Item;
  return proto.ubo.v1.Item.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Item} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Item}
 */
proto.ubo.v1.Item.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Item.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Item.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Item} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Item.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeBool(
      7,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeFloat(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.Item.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string key = 2;
 * @return {string}
 */
proto.ubo.v1.Item.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearKey = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasKey = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string label = 3;
 * @return {string}
 */
proto.ubo.v1.Item.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string color = 4;
 * @return {string}
 */
proto.ubo.v1.Item.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearColor = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasColor = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string background_color = 5;
 * @return {string}
 */
proto.ubo.v1.Item.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string icon = 6;
 * @return {string}
 */
proto.ubo.v1.Item.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional bool is_short = 7;
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional float opacity = 8;
 * @return {number}
 */
proto.ubo.v1.Item.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 8, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float progress = 9;
 * @return {number}
 */
proto.ubo.v1.Item.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Item} returns this
 */
proto.ubo.v1.Item.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Item.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 9) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ActionItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ActionItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ActionItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ActionItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    key: jspb.Message.getFieldWithDefault(msg, 2, ""),
    label: jspb.Message.getFieldWithDefault(msg, 3, ""),
    color: jspb.Message.getFieldWithDefault(msg, 4, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 5, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 6, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 8, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ActionItem}
 */
proto.ubo.v1.ActionItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ActionItem;
  return proto.ubo.v1.ActionItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ActionItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ActionItem}
 */
proto.ubo.v1.ActionItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ActionItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ActionItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ActionItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ActionItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeBool(
      7,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeFloat(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ActionItem.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string key = 2;
 * @return {string}
 */
proto.ubo.v1.ActionItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string label = 3;
 * @return {string}
 */
proto.ubo.v1.ActionItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string color = 4;
 * @return {string}
 */
proto.ubo.v1.ActionItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string background_color = 5;
 * @return {string}
 */
proto.ubo.v1.ActionItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string icon = 6;
 * @return {string}
 */
proto.ubo.v1.ActionItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional bool is_short = 7;
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional float opacity = 8;
 * @return {number}
 */
proto.ubo.v1.ActionItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 8, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float progress = 9;
 * @return {number}
 */
proto.ubo.v1.ActionItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ActionItem} returns this
 */
proto.ubo.v1.ActionItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ActionItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 9) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ApplicationItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ApplicationItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ApplicationItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ApplicationItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    application: (f = msg.getApplication()) && proto.ubo.v1.ApplicationItem.Application.toObject(includeInstance, f),
    key: jspb.Message.getFieldWithDefault(msg, 3, ""),
    label: jspb.Message.getFieldWithDefault(msg, 4, ""),
    color: jspb.Message.getFieldWithDefault(msg, 5, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 7, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ApplicationItem}
 */
proto.ubo.v1.ApplicationItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ApplicationItem;
  return proto.ubo.v1.ApplicationItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ApplicationItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ApplicationItem}
 */
proto.ubo.v1.ApplicationItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.ApplicationItem.Application;
      reader.readMessage(value,proto.ubo.v1.ApplicationItem.Application.deserializeBinaryFromReader);
      msg.setApplication(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ApplicationItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ApplicationItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ApplicationItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ApplicationItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getApplication();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.ApplicationItem.Application.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.ApplicationItem.Application.oneofGroups_ = [[1]];

/**
 * @enum {number}
 */
proto.ubo.v1.ApplicationItem.Application.ApplicationCase = {
  APPLICATION_NOT_SET: 0,
  STRING: 1
};

/**
 * @return {proto.ubo.v1.ApplicationItem.Application.ApplicationCase}
 */
proto.ubo.v1.ApplicationItem.Application.prototype.getApplicationCase = function() {
  return /** @type {proto.ubo.v1.ApplicationItem.Application.ApplicationCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.ApplicationItem.Application.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ApplicationItem.Application.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ApplicationItem.Application.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ApplicationItem.Application} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ApplicationItem.Application.toObject = function(includeInstance, msg) {
  var f, obj = {
    string: jspb.Message.getFieldWithDefault(msg, 1, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ApplicationItem.Application}
 */
proto.ubo.v1.ApplicationItem.Application.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ApplicationItem.Application;
  return proto.ubo.v1.ApplicationItem.Application.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ApplicationItem.Application} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ApplicationItem.Application}
 */
proto.ubo.v1.ApplicationItem.Application.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setString(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ApplicationItem.Application.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ApplicationItem.Application.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ApplicationItem.Application} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ApplicationItem.Application.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
};


/**
 * optional string string = 1;
 * @return {string}
 */
proto.ubo.v1.ApplicationItem.Application.prototype.getString = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ApplicationItem.Application} returns this
 */
proto.ubo.v1.ApplicationItem.Application.prototype.setString = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.ApplicationItem.Application.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem.Application} returns this
 */
proto.ubo.v1.ApplicationItem.Application.prototype.clearString = function() {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.ApplicationItem.Application.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.Application.prototype.hasString = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ApplicationItem.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Application application = 2;
 * @return {?proto.ubo.v1.ApplicationItem.Application}
 */
proto.ubo.v1.ApplicationItem.prototype.getApplication = function() {
  return /** @type{?proto.ubo.v1.ApplicationItem.Application} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ApplicationItem.Application, 2));
};


/**
 * @param {?proto.ubo.v1.ApplicationItem.Application|undefined} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
*/
proto.ubo.v1.ApplicationItem.prototype.setApplication = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearApplication = function() {
  return this.setApplication(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasApplication = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string key = 3;
 * @return {string}
 */
proto.ubo.v1.ApplicationItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string label = 4;
 * @return {string}
 */
proto.ubo.v1.ApplicationItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string color = 5;
 * @return {string}
 */
proto.ubo.v1.ApplicationItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string background_color = 6;
 * @return {string}
 */
proto.ubo.v1.ApplicationItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string icon = 7;
 * @return {string}
 */
proto.ubo.v1.ApplicationItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional bool is_short = 8;
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float opacity = 9;
 * @return {number}
 */
proto.ubo.v1.ApplicationItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float progress = 10;
 * @return {number}
 */
proto.ubo.v1.ApplicationItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ApplicationItem} returns this
 */
proto.ubo.v1.ApplicationItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ApplicationItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 10) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SubMenuItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SubMenuItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SubMenuItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SubMenuItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboGuiDotMenuDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    subMenu: (f = msg.getSubMenu()) && proto.ubo.v1.Menu.toObject(includeInstance, f),
    key: jspb.Message.getFieldWithDefault(msg, 3, ""),
    label: jspb.Message.getFieldWithDefault(msg, 4, ""),
    color: jspb.Message.getFieldWithDefault(msg, 5, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 7, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SubMenuItem}
 */
proto.ubo.v1.SubMenuItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SubMenuItem;
  return proto.ubo.v1.SubMenuItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SubMenuItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SubMenuItem}
 */
proto.ubo.v1.SubMenuItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboGuiDotMenuDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Menu;
      reader.readMessage(value,proto.ubo.v1.Menu.deserializeBinaryFromReader);
      msg.setSubMenu(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SubMenuItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SubMenuItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SubMenuItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SubMenuItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSubMenu();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Menu.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_gui_dot_menu_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SubMenuItem.prototype.getMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setMetaFieldPackageNameUboGuiDotMenuDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasMetaFieldPackageNameUboGuiDotMenuDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Menu sub_menu = 2;
 * @return {?proto.ubo.v1.Menu}
 */
proto.ubo.v1.SubMenuItem.prototype.getSubMenu = function() {
  return /** @type{?proto.ubo.v1.Menu} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Menu, 2));
};


/**
 * @param {?proto.ubo.v1.Menu|undefined} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
*/
proto.ubo.v1.SubMenuItem.prototype.setSubMenu = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearSubMenu = function() {
  return this.setSubMenu(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasSubMenu = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string key = 3;
 * @return {string}
 */
proto.ubo.v1.SubMenuItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string label = 4;
 * @return {string}
 */
proto.ubo.v1.SubMenuItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string color = 5;
 * @return {string}
 */
proto.ubo.v1.SubMenuItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string background_color = 6;
 * @return {string}
 */
proto.ubo.v1.SubMenuItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string icon = 7;
 * @return {string}
 */
proto.ubo.v1.SubMenuItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional bool is_short = 8;
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float opacity = 9;
 * @return {number}
 */
proto.ubo.v1.SubMenuItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float progress = 10;
 * @return {number}
 */
proto.ubo.v1.SubMenuItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SubMenuItem} returns this
 */
proto.ubo.v1.SubMenuItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SubMenuItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 10) != null;
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.Menu.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.Menu.MenuCase = {
  MENU_NOT_SET: 0,
  HEADED_MENU: 1,
  HEADLESS_MENU: 2
};

/**
 * @return {proto.ubo.v1.Menu.MenuCase}
 */
proto.ubo.v1.Menu.prototype.getMenuCase = function() {
  return /** @type {proto.ubo.v1.Menu.MenuCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Menu.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Menu.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Menu.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Menu} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Menu.toObject = function(includeInstance, msg) {
  var f, obj = {
    headedMenu: (f = msg.getHeadedMenu()) && proto.ubo.v1.HeadedMenu.toObject(includeInstance, f),
    headlessMenu: (f = msg.getHeadlessMenu()) && proto.ubo.v1.HeadlessMenu.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Menu}
 */
proto.ubo.v1.Menu.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Menu;
  return proto.ubo.v1.Menu.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Menu} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Menu}
 */
proto.ubo.v1.Menu.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.HeadedMenu;
      reader.readMessage(value,proto.ubo.v1.HeadedMenu.deserializeBinaryFromReader);
      msg.setHeadedMenu(value);
      break;
    case 2:
      var value = new proto.ubo.v1.HeadlessMenu;
      reader.readMessage(value,proto.ubo.v1.HeadlessMenu.deserializeBinaryFromReader);
      msg.setHeadlessMenu(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Menu.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Menu.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Menu} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Menu.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getHeadedMenu();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.HeadedMenu.serializeBinaryToWriter
    );
  }
  f = message.getHeadlessMenu();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.HeadlessMenu.serializeBinaryToWriter
    );
  }
};


/**
 * optional HeadedMenu headed_menu = 1;
 * @return {?proto.ubo.v1.HeadedMenu}
 */
proto.ubo.v1.Menu.prototype.getHeadedMenu = function() {
  return /** @type{?proto.ubo.v1.HeadedMenu} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.HeadedMenu, 1));
};


/**
 * @param {?proto.ubo.v1.HeadedMenu|undefined} value
 * @return {!proto.ubo.v1.Menu} returns this
*/
proto.ubo.v1.Menu.prototype.setHeadedMenu = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Menu.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Menu} returns this
 */
proto.ubo.v1.Menu.prototype.clearHeadedMenu = function() {
  return this.setHeadedMenu(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Menu.prototype.hasHeadedMenu = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional HeadlessMenu headless_menu = 2;
 * @return {?proto.ubo.v1.HeadlessMenu}
 */
proto.ubo.v1.Menu.prototype.getHeadlessMenu = function() {
  return /** @type{?proto.ubo.v1.HeadlessMenu} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.HeadlessMenu, 2));
};


/**
 * @param {?proto.ubo.v1.HeadlessMenu|undefined} value
 * @return {!proto.ubo.v1.Menu} returns this
*/
proto.ubo.v1.Menu.prototype.setHeadlessMenu = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Menu.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Menu} returns this
 */
proto.ubo.v1.Menu.prototype.clearHeadlessMenu = function() {
  return this.setHeadlessMenu(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Menu.prototype.hasHeadlessMenu = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MainAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MainAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MainAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MainAction}
 */
proto.ubo.v1.MainAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MainAction;
  return proto.ubo.v1.MainAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MainAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MainAction}
 */
proto.ubo.v1.MainAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MainAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MainAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MainAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MainAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MainAction} returns this
 */
proto.ubo.v1.MainAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainAction} returns this
 */
proto.ubo.v1.MainAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateLightDMState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateLightDMState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateLightDMState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateLightDMState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isEnable: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateLightDMState}
 */
proto.ubo.v1.UpdateLightDMState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateLightDMState;
  return proto.ubo.v1.UpdateLightDMState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateLightDMState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateLightDMState}
 */
proto.ubo.v1.UpdateLightDMState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsEnable(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateLightDMState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateLightDMState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateLightDMState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateLightDMState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsActive();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getIsEnable();
  if (f) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateLightDMState.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateLightDMState} returns this
 */
proto.ubo.v1.UpdateLightDMState.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateLightDMState} returns this
 */
proto.ubo.v1.UpdateLightDMState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateLightDMState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.UpdateLightDMState.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.UpdateLightDMState} returns this
 */
proto.ubo.v1.UpdateLightDMState.prototype.setIsActive = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional bool is_enable = 3;
 * @return {boolean}
 */
proto.ubo.v1.UpdateLightDMState.prototype.getIsEnable = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.UpdateLightDMState} returns this
 */
proto.ubo.v1.UpdateLightDMState.prototype.setIsEnable = function(value) {
  return jspb.Message.setProto3BooleanField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RegisterAppAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RegisterAppAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RegisterAppAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RegisterAppAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    menuItem: (f = msg.getMenuItem()) && proto.ubo.v1.Item.toObject(includeInstance, f),
    service: jspb.Message.getFieldWithDefault(msg, 3, ""),
    key: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RegisterAppAction}
 */
proto.ubo.v1.RegisterAppAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RegisterAppAction;
  return proto.ubo.v1.RegisterAppAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RegisterAppAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RegisterAppAction}
 */
proto.ubo.v1.RegisterAppAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Item;
      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);
      msg.setMenuItem(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setService(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RegisterAppAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RegisterAppAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RegisterAppAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RegisterAppAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getMenuItem();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Item.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.RegisterAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
 */
proto.ubo.v1.RegisterAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
 */
proto.ubo.v1.RegisterAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Item menu_item = 2;
 * @return {?proto.ubo.v1.Item}
 */
proto.ubo.v1.RegisterAppAction.prototype.getMenuItem = function() {
  return /** @type{?proto.ubo.v1.Item} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Item, 2));
};


/**
 * @param {?proto.ubo.v1.Item|undefined} value
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
*/
proto.ubo.v1.RegisterAppAction.prototype.setMenuItem = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
 */
proto.ubo.v1.RegisterAppAction.prototype.clearMenuItem = function() {
  return this.setMenuItem(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterAppAction.prototype.hasMenuItem = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string service = 3;
 * @return {string}
 */
proto.ubo.v1.RegisterAppAction.prototype.getService = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
 */
proto.ubo.v1.RegisterAppAction.prototype.setService = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
 */
proto.ubo.v1.RegisterAppAction.prototype.clearService = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterAppAction.prototype.hasService = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string key = 4;
 * @return {string}
 */
proto.ubo.v1.RegisterAppAction.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
 */
proto.ubo.v1.RegisterAppAction.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterAppAction} returns this
 */
proto.ubo.v1.RegisterAppAction.prototype.clearKey = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterAppAction.prototype.hasKey = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RegisterRegularAppAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RegisterRegularAppAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RegisterRegularAppAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    priority: jspb.Message.getFieldWithDefault(msg, 2, 0),
    menuItem: (f = msg.getMenuItem()) && proto.ubo.v1.Item.toObject(includeInstance, f),
    service: jspb.Message.getFieldWithDefault(msg, 4, ""),
    key: jspb.Message.getFieldWithDefault(msg, 5, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RegisterRegularAppAction}
 */
proto.ubo.v1.RegisterRegularAppAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RegisterRegularAppAction;
  return proto.ubo.v1.RegisterRegularAppAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RegisterRegularAppAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RegisterRegularAppAction}
 */
proto.ubo.v1.RegisterRegularAppAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setPriority(value);
      break;
    case 3:
      var value = new proto.ubo.v1.Item;
      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);
      msg.setMenuItem(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setService(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RegisterRegularAppAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RegisterRegularAppAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RegisterRegularAppAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = message.getMenuItem();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.Item.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 priority = 2;
 * @return {number}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.getPriority = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.setPriority = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.clearPriority = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.hasPriority = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Item menu_item = 3;
 * @return {?proto.ubo.v1.Item}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.getMenuItem = function() {
  return /** @type{?proto.ubo.v1.Item} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Item, 3));
};


/**
 * @param {?proto.ubo.v1.Item|undefined} value
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
*/
proto.ubo.v1.RegisterRegularAppAction.prototype.setMenuItem = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.clearMenuItem = function() {
  return this.setMenuItem(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.hasMenuItem = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string service = 4;
 * @return {string}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.getService = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.setService = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.clearService = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.hasService = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string key = 5;
 * @return {string}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterRegularAppAction} returns this
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.clearKey = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterRegularAppAction.prototype.hasKey = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DeregisterRegularAppAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DeregisterRegularAppAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DeregisterRegularAppAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    service: jspb.Message.getFieldWithDefault(msg, 2, ""),
    key: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DeregisterRegularAppAction}
 */
proto.ubo.v1.DeregisterRegularAppAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DeregisterRegularAppAction;
  return proto.ubo.v1.DeregisterRegularAppAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DeregisterRegularAppAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DeregisterRegularAppAction}
 */
proto.ubo.v1.DeregisterRegularAppAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setService(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DeregisterRegularAppAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DeregisterRegularAppAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DeregisterRegularAppAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string service = 2;
 * @return {string}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.getService = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.setService = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.clearService = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.hasService = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string key = 3;
 * @return {string}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DeregisterRegularAppAction} returns this
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.clearKey = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DeregisterRegularAppAction.prototype.hasKey = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RegisterSettingAppAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RegisterSettingAppAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RegisterSettingAppAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    category: jspb.Message.getFieldWithDefault(msg, 2, 0),
    priority: jspb.Message.getFieldWithDefault(msg, 3, 0),
    menuItem: (f = msg.getMenuItem()) && proto.ubo.v1.Item.toObject(includeInstance, f),
    service: jspb.Message.getFieldWithDefault(msg, 5, ""),
    key: jspb.Message.getFieldWithDefault(msg, 6, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RegisterSettingAppAction}
 */
proto.ubo.v1.RegisterSettingAppAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RegisterSettingAppAction;
  return proto.ubo.v1.RegisterSettingAppAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RegisterSettingAppAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RegisterSettingAppAction}
 */
proto.ubo.v1.RegisterSettingAppAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.SettingsCategory} */ (reader.readEnum());
      msg.setCategory(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setPriority(value);
      break;
    case 4:
      var value = new proto.ubo.v1.Item;
      reader.readMessage(value,proto.ubo.v1.Item.deserializeBinaryFromReader);
      msg.setMenuItem(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setService(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RegisterSettingAppAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RegisterSettingAppAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RegisterSettingAppAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getCategory();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getMenuItem();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.Item.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional SettingsCategory category = 2;
 * @return {!proto.ubo.v1.SettingsCategory}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.getCategory = function() {
  return /** @type {!proto.ubo.v1.SettingsCategory} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.SettingsCategory} value
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.setCategory = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional int64 priority = 3;
 * @return {number}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.getPriority = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.setPriority = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.clearPriority = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.hasPriority = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional Item menu_item = 4;
 * @return {?proto.ubo.v1.Item}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.getMenuItem = function() {
  return /** @type{?proto.ubo.v1.Item} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Item, 4));
};


/**
 * @param {?proto.ubo.v1.Item|undefined} value
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
*/
proto.ubo.v1.RegisterSettingAppAction.prototype.setMenuItem = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.clearMenuItem = function() {
  return this.setMenuItem(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.hasMenuItem = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string service = 5;
 * @return {string}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.getService = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.setService = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.clearService = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.hasService = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string key = 6;
 * @return {string}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RegisterSettingAppAction} returns this
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.clearKey = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RegisterSettingAppAction.prototype.hasKey = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.PowerAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.PowerAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.PowerAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.PowerAction}
 */
proto.ubo.v1.PowerAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.PowerAction;
  return proto.ubo.v1.PowerAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.PowerAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.PowerAction}
 */
proto.ubo.v1.PowerAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.PowerAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.PowerAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.PowerAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.PowerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PowerAction} returns this
 */
proto.ubo.v1.PowerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PowerAction} returns this
 */
proto.ubo.v1.PowerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PowerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.PowerOffAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.PowerOffAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.PowerOffAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerOffAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.PowerOffAction}
 */
proto.ubo.v1.PowerOffAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.PowerOffAction;
  return proto.ubo.v1.PowerOffAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.PowerOffAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.PowerOffAction}
 */
proto.ubo.v1.PowerOffAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.PowerOffAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.PowerOffAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.PowerOffAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerOffAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.PowerOffAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PowerOffAction} returns this
 */
proto.ubo.v1.PowerOffAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PowerOffAction} returns this
 */
proto.ubo.v1.PowerOffAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PowerOffAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RebootAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RebootAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RebootAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RebootAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RebootAction}
 */
proto.ubo.v1.RebootAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RebootAction;
  return proto.ubo.v1.RebootAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RebootAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RebootAction}
 */
proto.ubo.v1.RebootAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RebootAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RebootAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RebootAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RebootAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.RebootAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RebootAction} returns this
 */
proto.ubo.v1.RebootAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RebootAction} returns this
 */
proto.ubo.v1.RebootAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RebootAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.SetMenuPathAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SetMenuPathAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SetMenuPathAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SetMenuPathAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SetMenuPathAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    pathList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    depth: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SetMenuPathAction}
 */
proto.ubo.v1.SetMenuPathAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SetMenuPathAction;
  return proto.ubo.v1.SetMenuPathAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SetMenuPathAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SetMenuPathAction}
 */
proto.ubo.v1.SetMenuPathAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addPath(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setDepth(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SetMenuPathAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SetMenuPathAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SetMenuPathAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SetMenuPathAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getPathList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
  f = message.getDepth();
  if (f !== 0) {
    writer.writeInt64(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SetMenuPathAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SetMenuPathAction} returns this
 */
proto.ubo.v1.SetMenuPathAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SetMenuPathAction} returns this
 */
proto.ubo.v1.SetMenuPathAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SetMenuPathAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string path = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.SetMenuPathAction.prototype.getPathList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.SetMenuPathAction} returns this
 */
proto.ubo.v1.SetMenuPathAction.prototype.setPathList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.SetMenuPathAction} returns this
 */
proto.ubo.v1.SetMenuPathAction.prototype.addPath = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.SetMenuPathAction} returns this
 */
proto.ubo.v1.SetMenuPathAction.prototype.clearPathList = function() {
  return this.setPathList([]);
};


/**
 * optional int64 depth = 3;
 * @return {number}
 */
proto.ubo.v1.SetMenuPathAction.prototype.getDepth = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SetMenuPathAction} returns this
 */
proto.ubo.v1.SetMenuPathAction.prototype.setDepth = function(value) {
  return jspb.Message.setProto3IntField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SetAreEnclosuresVisibleAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SetAreEnclosuresVisibleAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isHeaderVisible: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isFooterVisible: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SetAreEnclosuresVisibleAction;
  return proto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SetAreEnclosuresVisibleAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsHeaderVisible(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsFooterVisible(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SetAreEnclosuresVisibleAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SetAreEnclosuresVisibleAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_header_visible = 2;
 * @return {boolean}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.getIsHeaderVisible = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.setIsHeaderVisible = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.clearIsHeaderVisible = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.hasIsHeaderVisible = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_footer_visible = 3;
 * @return {boolean}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.getIsFooterVisible = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.setIsFooterVisible = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SetAreEnclosuresVisibleAction} returns this
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.clearIsFooterVisible = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SetAreEnclosuresVisibleAction.prototype.hasIsFooterVisible = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuAction}
 */
proto.ubo.v1.MenuAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuAction;
  return proto.ubo.v1.MenuAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuAction}
 */
proto.ubo.v1.MenuAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuAction} returns this
 */
proto.ubo.v1.MenuAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuAction} returns this
 */
proto.ubo.v1.MenuAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuGoBackAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuGoBackAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuGoBackAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoBackAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuGoBackAction}
 */
proto.ubo.v1.MenuGoBackAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuGoBackAction;
  return proto.ubo.v1.MenuGoBackAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuGoBackAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuGoBackAction}
 */
proto.ubo.v1.MenuGoBackAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuGoBackAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuGoBackAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuGoBackAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoBackAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuGoBackAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuGoBackAction} returns this
 */
proto.ubo.v1.MenuGoBackAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuGoBackAction} returns this
 */
proto.ubo.v1.MenuGoBackAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuGoBackAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuGoHomeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuGoHomeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuGoHomeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoHomeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuGoHomeAction}
 */
proto.ubo.v1.MenuGoHomeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuGoHomeAction;
  return proto.ubo.v1.MenuGoHomeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuGoHomeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuGoHomeAction}
 */
proto.ubo.v1.MenuGoHomeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuGoHomeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuGoHomeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuGoHomeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoHomeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuGoHomeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuGoHomeAction} returns this
 */
proto.ubo.v1.MenuGoHomeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuGoHomeAction} returns this
 */
proto.ubo.v1.MenuGoHomeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuGoHomeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuChooseByIconAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuChooseByIconAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIconAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuChooseByIconAction}
 */
proto.ubo.v1.MenuChooseByIconAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuChooseByIconAction;
  return proto.ubo.v1.MenuChooseByIconAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuChooseByIconAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuChooseByIconAction}
 */
proto.ubo.v1.MenuChooseByIconAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuChooseByIconAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuChooseByIconAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIconAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIcon();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByIconAction} returns this
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuChooseByIconAction} returns this
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string icon = 2;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByIconAction} returns this
 */
proto.ubo.v1.MenuChooseByIconAction.prototype.setIcon = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuChooseByLabelAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuChooseByLabelAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByLabelAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    label: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuChooseByLabelAction}
 */
proto.ubo.v1.MenuChooseByLabelAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuChooseByLabelAction;
  return proto.ubo.v1.MenuChooseByLabelAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuChooseByLabelAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuChooseByLabelAction}
 */
proto.ubo.v1.MenuChooseByLabelAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuChooseByLabelAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuChooseByLabelAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByLabelAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getLabel();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByLabelAction} returns this
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuChooseByLabelAction} returns this
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string label = 2;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByLabelAction} returns this
 */
proto.ubo.v1.MenuChooseByLabelAction.prototype.setLabel = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuChooseByIndexAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuChooseByIndexAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIndexAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    index: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuChooseByIndexAction}
 */
proto.ubo.v1.MenuChooseByIndexAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuChooseByIndexAction;
  return proto.ubo.v1.MenuChooseByIndexAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuChooseByIndexAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuChooseByIndexAction}
 */
proto.ubo.v1.MenuChooseByIndexAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuChooseByIndexAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuChooseByIndexAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIndexAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByIndexAction} returns this
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuChooseByIndexAction} returns this
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 index = 2;
 * @return {number}
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.MenuChooseByIndexAction} returns this
 */
proto.ubo.v1.MenuChooseByIndexAction.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuScrollAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuScrollAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuScrollAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuScrollAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    direction: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuScrollAction}
 */
proto.ubo.v1.MenuScrollAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuScrollAction;
  return proto.ubo.v1.MenuScrollAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuScrollAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuScrollAction}
 */
proto.ubo.v1.MenuScrollAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.MenuScrollDirection} */ (reader.readEnum());
      msg.setDirection(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuScrollAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuScrollAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuScrollAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuScrollAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDirection();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuScrollAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuScrollAction} returns this
 */
proto.ubo.v1.MenuScrollAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuScrollAction} returns this
 */
proto.ubo.v1.MenuScrollAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuScrollAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional MenuScrollDirection direction = 2;
 * @return {!proto.ubo.v1.MenuScrollDirection}
 */
proto.ubo.v1.MenuScrollAction.prototype.getDirection = function() {
  return /** @type {!proto.ubo.v1.MenuScrollDirection} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.MenuScrollDirection} value
 * @return {!proto.ubo.v1.MenuScrollAction} returns this
 */
proto.ubo.v1.MenuScrollAction.prototype.setDirection = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    applicationId: jspb.Message.getFieldWithDefault(msg, 2, ""),
    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.OpenApplicationAction.InitializationArgs.toObject(includeInstance, f),
    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationAction}
 */
proto.ubo.v1.OpenApplicationAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationAction;
  return proto.ubo.v1.OpenApplicationAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationAction}
 */
proto.ubo.v1.OpenApplicationAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setApplicationId(value);
      break;
    case 3:
      var value = new proto.ubo.v1.OpenApplicationAction.InitializationArgs;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinaryFromReader);
      msg.setInitializationArgs(value);
      break;
    case 4:
      var value = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinaryFromReader);
      msg.setInitializationKwargs(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getApplicationId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getInitializationArgs();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.OpenApplicationAction.InitializationArgs.serializeBinaryToWriter
    );
  }
  f = message.getInitializationKwargs();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationAction.InitializationArgs.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.BasicType.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs}
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationArgs;
  return proto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs}
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationAction.InitializationArgs.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
};


/**
 * repeated BasicType items = 1;
 * @return {!Array<!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.BasicType>} value
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} returns this
*/
proto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.BasicType=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationArgs} returns this
 */
proto.ubo.v1.OpenApplicationAction.InitializationArgs.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.BasicType.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2;
  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
};


/**
 * repeated BasicType items = 1;
 * @return {!Array<!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.BasicType>} value
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} returns this
*/
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.BasicType=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} returns this
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase = {
  INITIALIZATION_KWARGS_VALUE_NOT_SET: 0,
  BASIC_TYPE: 1,
  LIST: 2
};

/**
 * @return {proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.getInitializationKwargsValueCase = function() {
  return /** @type {proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.InitializationKwargsValueCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.toObject = function(includeInstance, msg) {
  var f, obj = {
    basicType: (f = msg.getBasicType()) && proto.ubo.v1.BasicType.toObject(includeInstance, f),
    list: (f = msg.getList()) && proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue;
  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.setBasicType(value);
      break;
    case 2:
      var value = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.deserializeBinaryFromReader);
      msg.setList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getBasicType();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
  f = message.getList();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2.serializeBinaryToWriter
    );
  }
};


/**
 * optional BasicType basic_type = 1;
 * @return {?proto.ubo.v1.BasicType}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.getBasicType = function() {
  return /** @type{?proto.ubo.v1.BasicType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {?proto.ubo.v1.BasicType|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this
*/
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.setBasicType = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.clearBasicType = function() {
  return this.setBasicType(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.hasBasicType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional InitializationKwargsValue2 list = 2;
 * @return {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.getList = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2, 2));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue2|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this
*/
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.setList = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue} returns this
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.clearList = function() {
  return this.setList(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.prototype.hasList = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.toObject) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict;
  return proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.deserializeBinaryFromReader, "", new proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue());
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue.serializeBinaryToWriter);
  }
};


/**
 * map<string, InitializationKwargsValue> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue>}
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      proto.ubo.v1.OpenApplicationAction.InitializationKwargsValue));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} returns this
 */
proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.OpenApplicationAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.OpenApplicationAction} returns this
 */
proto.ubo.v1.OpenApplicationAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationAction} returns this
 */
proto.ubo.v1.OpenApplicationAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string application_id = 2;
 * @return {string}
 */
proto.ubo.v1.OpenApplicationAction.prototype.getApplicationId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.OpenApplicationAction} returns this
 */
proto.ubo.v1.OpenApplicationAction.prototype.setApplicationId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional InitializationArgs initialization_args = 3;
 * @return {?proto.ubo.v1.OpenApplicationAction.InitializationArgs}
 */
proto.ubo.v1.OpenApplicationAction.prototype.getInitializationArgs = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationAction.InitializationArgs} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction.InitializationArgs, 3));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationAction.InitializationArgs|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationAction} returns this
*/
proto.ubo.v1.OpenApplicationAction.prototype.setInitializationArgs = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationAction} returns this
 */
proto.ubo.v1.OpenApplicationAction.prototype.clearInitializationArgs = function() {
  return this.setInitializationArgs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationAction.prototype.hasInitializationArgs = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional InitializationKwargsDict initialization_kwargs = 4;
 * @return {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict}
 */
proto.ubo.v1.OpenApplicationAction.prototype.getInitializationKwargs = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict, 4));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationAction.InitializationKwargsDict|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationAction} returns this
*/
proto.ubo.v1.OpenApplicationAction.prototype.setInitializationKwargs = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationAction} returns this
 */
proto.ubo.v1.OpenApplicationAction.prototype.clearInitializationKwargs = function() {
  return this.setInitializationKwargs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationAction.prototype.hasInitializationKwargs = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CloseApplicationAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CloseApplicationAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CloseApplicationAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CloseApplicationAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    applicationInstanceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CloseApplicationAction}
 */
proto.ubo.v1.CloseApplicationAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CloseApplicationAction;
  return proto.ubo.v1.CloseApplicationAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CloseApplicationAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CloseApplicationAction}
 */
proto.ubo.v1.CloseApplicationAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setApplicationInstanceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CloseApplicationAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CloseApplicationAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CloseApplicationAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CloseApplicationAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getApplicationInstanceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.CloseApplicationAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CloseApplicationAction} returns this
 */
proto.ubo.v1.CloseApplicationAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CloseApplicationAction} returns this
 */
proto.ubo.v1.CloseApplicationAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CloseApplicationAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string application_instance_id = 2;
 * @return {string}
 */
proto.ubo.v1.CloseApplicationAction.prototype.getApplicationInstanceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CloseApplicationAction} returns this
 */
proto.ubo.v1.CloseApplicationAction.prototype.setApplicationInstanceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MainEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MainEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MainEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MainEvent}
 */
proto.ubo.v1.MainEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MainEvent;
  return proto.ubo.v1.MainEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MainEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MainEvent}
 */
proto.ubo.v1.MainEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MainEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MainEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MainEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MainEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MainEvent} returns this
 */
proto.ubo.v1.MainEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainEvent} returns this
 */
proto.ubo.v1.MainEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InitEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InitEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InitEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InitEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InitEvent}
 */
proto.ubo.v1.InitEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InitEvent;
  return proto.ubo.v1.InitEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InitEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InitEvent}
 */
proto.ubo.v1.InitEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InitEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InitEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InitEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InitEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InitEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InitEvent} returns this
 */
proto.ubo.v1.InitEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InitEvent} returns this
 */
proto.ubo.v1.InitEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InitEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuEvent}
 */
proto.ubo.v1.MenuEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuEvent;
  return proto.ubo.v1.MenuEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuEvent}
 */
proto.ubo.v1.MenuEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuEvent} returns this
 */
proto.ubo.v1.MenuEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuEvent} returns this
 */
proto.ubo.v1.MenuEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuGoBackEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuGoBackEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuGoBackEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoBackEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuGoBackEvent}
 */
proto.ubo.v1.MenuGoBackEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuGoBackEvent;
  return proto.ubo.v1.MenuGoBackEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuGoBackEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuGoBackEvent}
 */
proto.ubo.v1.MenuGoBackEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuGoBackEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuGoBackEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuGoBackEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoBackEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuGoBackEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuGoBackEvent} returns this
 */
proto.ubo.v1.MenuGoBackEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuGoBackEvent} returns this
 */
proto.ubo.v1.MenuGoBackEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuGoBackEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuGoHomeEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuGoHomeEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuGoHomeEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoHomeEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuGoHomeEvent}
 */
proto.ubo.v1.MenuGoHomeEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuGoHomeEvent;
  return proto.ubo.v1.MenuGoHomeEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuGoHomeEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuGoHomeEvent}
 */
proto.ubo.v1.MenuGoHomeEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuGoHomeEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuGoHomeEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuGoHomeEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuGoHomeEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuGoHomeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuGoHomeEvent} returns this
 */
proto.ubo.v1.MenuGoHomeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuGoHomeEvent} returns this
 */
proto.ubo.v1.MenuGoHomeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuGoHomeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuChooseByIconEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuChooseByIconEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIconEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuChooseByIconEvent}
 */
proto.ubo.v1.MenuChooseByIconEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuChooseByIconEvent;
  return proto.ubo.v1.MenuChooseByIconEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuChooseByIconEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuChooseByIconEvent}
 */
proto.ubo.v1.MenuChooseByIconEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuChooseByIconEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuChooseByIconEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIconEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIcon();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByIconEvent} returns this
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuChooseByIconEvent} returns this
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string icon = 2;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByIconEvent} returns this
 */
proto.ubo.v1.MenuChooseByIconEvent.prototype.setIcon = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuChooseByLabelEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuChooseByLabelEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByLabelEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    label: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuChooseByLabelEvent}
 */
proto.ubo.v1.MenuChooseByLabelEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuChooseByLabelEvent;
  return proto.ubo.v1.MenuChooseByLabelEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuChooseByLabelEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuChooseByLabelEvent}
 */
proto.ubo.v1.MenuChooseByLabelEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuChooseByLabelEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuChooseByLabelEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByLabelEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getLabel();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByLabelEvent} returns this
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuChooseByLabelEvent} returns this
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string label = 2;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByLabelEvent} returns this
 */
proto.ubo.v1.MenuChooseByLabelEvent.prototype.setLabel = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuChooseByIndexEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuChooseByIndexEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIndexEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    index: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuChooseByIndexEvent}
 */
proto.ubo.v1.MenuChooseByIndexEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuChooseByIndexEvent;
  return proto.ubo.v1.MenuChooseByIndexEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuChooseByIndexEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuChooseByIndexEvent}
 */
proto.ubo.v1.MenuChooseByIndexEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuChooseByIndexEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuChooseByIndexEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuChooseByIndexEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuChooseByIndexEvent} returns this
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuChooseByIndexEvent} returns this
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 index = 2;
 * @return {number}
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.MenuChooseByIndexEvent} returns this
 */
proto.ubo.v1.MenuChooseByIndexEvent.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MenuScrollEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MenuScrollEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MenuScrollEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuScrollEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    direction: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MenuScrollEvent}
 */
proto.ubo.v1.MenuScrollEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MenuScrollEvent;
  return proto.ubo.v1.MenuScrollEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MenuScrollEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MenuScrollEvent}
 */
proto.ubo.v1.MenuScrollEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.MenuScrollDirection} */ (reader.readEnum());
      msg.setDirection(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MenuScrollEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MenuScrollEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MenuScrollEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MenuScrollEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDirection();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MenuScrollEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MenuScrollEvent} returns this
 */
proto.ubo.v1.MenuScrollEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MenuScrollEvent} returns this
 */
proto.ubo.v1.MenuScrollEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MenuScrollEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional MenuScrollDirection direction = 2;
 * @return {!proto.ubo.v1.MenuScrollDirection}
 */
proto.ubo.v1.MenuScrollEvent.prototype.getDirection = function() {
  return /** @type {!proto.ubo.v1.MenuScrollDirection} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.MenuScrollDirection} value
 * @return {!proto.ubo.v1.MenuScrollEvent} returns this
 */
proto.ubo.v1.MenuScrollEvent.prototype.setDirection = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    applicationId: jspb.Message.getFieldWithDefault(msg, 2, ""),
    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.OpenApplicationEvent.InitializationArgs.toObject(includeInstance, f),
    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationEvent}
 */
proto.ubo.v1.OpenApplicationEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationEvent;
  return proto.ubo.v1.OpenApplicationEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationEvent}
 */
proto.ubo.v1.OpenApplicationEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setApplicationId(value);
      break;
    case 3:
      var value = new proto.ubo.v1.OpenApplicationEvent.InitializationArgs;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinaryFromReader);
      msg.setInitializationArgs(value);
      break;
    case 4:
      var value = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinaryFromReader);
      msg.setInitializationKwargs(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getApplicationId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getInitializationArgs();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.OpenApplicationEvent.InitializationArgs.serializeBinaryToWriter
    );
  }
  f = message.getInitializationKwargs();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationEvent.InitializationArgs.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.BasicType.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationArgs;
  return proto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationEvent.InitializationArgs.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
};


/**
 * repeated BasicType items = 1;
 * @return {!Array<!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.BasicType>} value
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} returns this
*/
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.BasicType=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationArgs} returns this
 */
proto.ubo.v1.OpenApplicationEvent.InitializationArgs.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.BasicType.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2;
  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
};


/**
 * repeated BasicType items = 1;
 * @return {!Array<!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.BasicType>} value
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} returns this
*/
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.BasicType=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} returns this
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase = {
  INITIALIZATION_KWARGS_VALUE_NOT_SET: 0,
  BASIC_TYPE: 1,
  LIST: 2
};

/**
 * @return {proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.getInitializationKwargsValueCase = function() {
  return /** @type {proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.InitializationKwargsValueCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.toObject = function(includeInstance, msg) {
  var f, obj = {
    basicType: (f = msg.getBasicType()) && proto.ubo.v1.BasicType.toObject(includeInstance, f),
    list: (f = msg.getList()) && proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue;
  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.setBasicType(value);
      break;
    case 2:
      var value = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.deserializeBinaryFromReader);
      msg.setList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getBasicType();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
  f = message.getList();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2.serializeBinaryToWriter
    );
  }
};


/**
 * optional BasicType basic_type = 1;
 * @return {?proto.ubo.v1.BasicType}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.getBasicType = function() {
  return /** @type{?proto.ubo.v1.BasicType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {?proto.ubo.v1.BasicType|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this
*/
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.setBasicType = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.clearBasicType = function() {
  return this.setBasicType(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.hasBasicType = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional InitializationKwargsValue2 list = 2;
 * @return {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.getList = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2, 2));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue2|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this
*/
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.setList = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue} returns this
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.clearList = function() {
  return this.setList(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.prototype.hasList = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.toObject) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict;
  return proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.deserializeBinaryFromReader, "", new proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue());
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue.serializeBinaryToWriter);
  }
};


/**
 * map<string, InitializationKwargsValue> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue>}
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      proto.ubo.v1.OpenApplicationEvent.InitializationKwargsValue));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} returns this
 */
proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.OpenApplicationEvent} returns this
 */
proto.ubo.v1.OpenApplicationEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationEvent} returns this
 */
proto.ubo.v1.OpenApplicationEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string application_id = 2;
 * @return {string}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.getApplicationId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.OpenApplicationEvent} returns this
 */
proto.ubo.v1.OpenApplicationEvent.prototype.setApplicationId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional InitializationArgs initialization_args = 3;
 * @return {?proto.ubo.v1.OpenApplicationEvent.InitializationArgs}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.getInitializationArgs = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationEvent.InitializationArgs} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent.InitializationArgs, 3));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationEvent.InitializationArgs|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationEvent} returns this
*/
proto.ubo.v1.OpenApplicationEvent.prototype.setInitializationArgs = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationEvent} returns this
 */
proto.ubo.v1.OpenApplicationEvent.prototype.clearInitializationArgs = function() {
  return this.setInitializationArgs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.hasInitializationArgs = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional InitializationKwargsDict initialization_kwargs = 4;
 * @return {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.getInitializationKwargs = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict, 4));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationEvent.InitializationKwargsDict|undefined} value
 * @return {!proto.ubo.v1.OpenApplicationEvent} returns this
*/
proto.ubo.v1.OpenApplicationEvent.prototype.setInitializationKwargs = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.OpenApplicationEvent} returns this
 */
proto.ubo.v1.OpenApplicationEvent.prototype.clearInitializationKwargs = function() {
  return this.setInitializationKwargs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.OpenApplicationEvent.prototype.hasInitializationKwargs = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CloseApplicationEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CloseApplicationEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CloseApplicationEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CloseApplicationEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    applicationInstanceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CloseApplicationEvent}
 */
proto.ubo.v1.CloseApplicationEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CloseApplicationEvent;
  return proto.ubo.v1.CloseApplicationEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CloseApplicationEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CloseApplicationEvent}
 */
proto.ubo.v1.CloseApplicationEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setApplicationInstanceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CloseApplicationEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CloseApplicationEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CloseApplicationEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CloseApplicationEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getApplicationInstanceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.CloseApplicationEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CloseApplicationEvent} returns this
 */
proto.ubo.v1.CloseApplicationEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CloseApplicationEvent} returns this
 */
proto.ubo.v1.CloseApplicationEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CloseApplicationEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string application_instance_id = 2;
 * @return {string}
 */
proto.ubo.v1.CloseApplicationEvent.prototype.getApplicationInstanceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CloseApplicationEvent} returns this
 */
proto.ubo.v1.CloseApplicationEvent.prototype.setApplicationInstanceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.PowerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.PowerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.PowerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.PowerEvent}
 */
proto.ubo.v1.PowerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.PowerEvent;
  return proto.ubo.v1.PowerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.PowerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.PowerEvent}
 */
proto.ubo.v1.PowerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.PowerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.PowerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.PowerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.PowerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PowerEvent} returns this
 */
proto.ubo.v1.PowerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PowerEvent} returns this
 */
proto.ubo.v1.PowerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PowerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.PowerOffEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.PowerOffEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.PowerOffEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerOffEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.PowerOffEvent}
 */
proto.ubo.v1.PowerOffEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.PowerOffEvent;
  return proto.ubo.v1.PowerOffEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.PowerOffEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.PowerOffEvent}
 */
proto.ubo.v1.PowerOffEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.PowerOffEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.PowerOffEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.PowerOffEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PowerOffEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.PowerOffEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PowerOffEvent} returns this
 */
proto.ubo.v1.PowerOffEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PowerOffEvent} returns this
 */
proto.ubo.v1.PowerOffEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PowerOffEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RebootEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RebootEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RebootEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RebootEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RebootEvent}
 */
proto.ubo.v1.RebootEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RebootEvent;
  return proto.ubo.v1.RebootEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RebootEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RebootEvent}
 */
proto.ubo.v1.RebootEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RebootEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RebootEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RebootEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RebootEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.RebootEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RebootEvent} returns this
 */
proto.ubo.v1.RebootEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RebootEvent} returns this
 */
proto.ubo.v1.RebootEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RebootEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ScreenshotEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ScreenshotEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ScreenshotEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ScreenshotEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ScreenshotEvent}
 */
proto.ubo.v1.ScreenshotEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ScreenshotEvent;
  return proto.ubo.v1.ScreenshotEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ScreenshotEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ScreenshotEvent}
 */
proto.ubo.v1.ScreenshotEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ScreenshotEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ScreenshotEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ScreenshotEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ScreenshotEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ScreenshotEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ScreenshotEvent} returns this
 */
proto.ubo.v1.ScreenshotEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ScreenshotEvent} returns this
 */
proto.ubo.v1.ScreenshotEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ScreenshotEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SnapshotEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SnapshotEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SnapshotEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SnapshotEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SnapshotEvent}
 */
proto.ubo.v1.SnapshotEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SnapshotEvent;
  return proto.ubo.v1.SnapshotEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SnapshotEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SnapshotEvent}
 */
proto.ubo.v1.SnapshotEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SnapshotEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SnapshotEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SnapshotEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SnapshotEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SnapshotEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SnapshotEvent} returns this
 */
proto.ubo.v1.SnapshotEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SnapshotEvent} returns this
 */
proto.ubo.v1.SnapshotEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SnapshotEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ToggleRecordingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ToggleRecordingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ToggleRecordingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ToggleRecordingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ToggleRecordingAction}
 */
proto.ubo.v1.ToggleRecordingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ToggleRecordingAction;
  return proto.ubo.v1.ToggleRecordingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ToggleRecordingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ToggleRecordingAction}
 */
proto.ubo.v1.ToggleRecordingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ToggleRecordingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ToggleRecordingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ToggleRecordingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ToggleRecordingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ToggleRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ToggleRecordingAction} returns this
 */
proto.ubo.v1.ToggleRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ToggleRecordingAction} returns this
 */
proto.ubo.v1.ToggleRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ToggleRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.StoreRecordedSequenceEvent.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.StoreRecordedSequenceEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.StoreRecordedSequenceEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StoreRecordedSequenceEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    recordedSequenceList: jspb.Message.toObjectList(msg.getRecordedSequenceList(),
    proto.ubo.v1.Action.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.StoreRecordedSequenceEvent}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.StoreRecordedSequenceEvent;
  return proto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.StoreRecordedSequenceEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.StoreRecordedSequenceEvent}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Action;
      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);
      msg.addRecordedSequence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.StoreRecordedSequenceEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.StoreRecordedSequenceEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StoreRecordedSequenceEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getRecordedSequenceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.Action.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated Action recorded_sequence = 2;
 * @return {!Array<!proto.ubo.v1.Action>}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.getRecordedSequenceList = function() {
  return /** @type{!Array<!proto.ubo.v1.Action>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.Action>} value
 * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this
*/
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.setRecordedSequenceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.Action=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Action}
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.addRecordedSequence = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.Action, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.StoreRecordedSequenceEvent} returns this
 */
proto.ubo.v1.StoreRecordedSequenceEvent.prototype.clearRecordedSequenceList = function() {
  return this.setRecordedSequenceList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ReplayRecordedSequenceAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ReplayRecordedSequenceAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ReplayRecordedSequenceAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReplayRecordedSequenceAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ReplayRecordedSequenceAction}
 */
proto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ReplayRecordedSequenceAction;
  return proto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ReplayRecordedSequenceAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ReplayRecordedSequenceAction}
 */
proto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ReplayRecordedSequenceAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ReplayRecordedSequenceAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ReplayRecordedSequenceAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReplayRecordedSequenceAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ReplayRecordedSequenceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ReplayRecordedSequenceAction} returns this
 */
proto.ubo.v1.ReplayRecordedSequenceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ReplayRecordedSequenceAction} returns this
 */
proto.ubo.v1.ReplayRecordedSequenceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ReplayRecordedSequenceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ReplayRecordedSequenceEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ReplayRecordedSequenceEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent}
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ReplayRecordedSequenceEvent;
  return proto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ReplayRecordedSequenceEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent}
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ReplayRecordedSequenceEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ReplayRecordedSequenceEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent} returns this
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ReplayRecordedSequenceEvent} returns this
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ReplayRecordedSequenceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ReportReplayingDoneAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ReportReplayingDoneAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ReportReplayingDoneAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReportReplayingDoneAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ReportReplayingDoneAction}
 */
proto.ubo.v1.ReportReplayingDoneAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ReportReplayingDoneAction;
  return proto.ubo.v1.ReportReplayingDoneAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ReportReplayingDoneAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ReportReplayingDoneAction}
 */
proto.ubo.v1.ReportReplayingDoneAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ReportReplayingDoneAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ReportReplayingDoneAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ReportReplayingDoneAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReportReplayingDoneAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ReportReplayingDoneAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ReportReplayingDoneAction} returns this
 */
proto.ubo.v1.ReportReplayingDoneAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ReportReplayingDoneAction} returns this
 */
proto.ubo.v1.ReportReplayingDoneAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ReportReplayingDoneAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MainState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MainState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MainState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotCoreDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    menu: (f = msg.getMenu()) && proto.ubo.v1.Menu.toObject(includeInstance, f),
    path: (f = msg.getPath()) && proto.ubo.v1.MainState.Path.toObject(includeInstance, f),
    depth: jspb.Message.getFieldWithDefault(msg, 4, 0),
    isHeaderVisible: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    isFooterVisible: jspb.Message.getBooleanFieldWithDefault(msg, 6, false),
    appsItemsPriorities: (f = msg.getAppsItemsPriorities()) && proto.ubo.v1.MainState.AppsItemsPrioritiesDict.toObject(includeInstance, f),
    settingsItemsPriorities: (f = msg.getSettingsItemsPriorities()) && proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.toObject(includeInstance, f),
    isRecording: jspb.Message.getBooleanFieldWithDefault(msg, 9, false),
    isReplaying: jspb.Message.getBooleanFieldWithDefault(msg, 10, false),
    recordedSequence: (f = msg.getRecordedSequence()) && proto.ubo.v1.MainState.RecordedSequence.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MainState}
 */
proto.ubo.v1.MainState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MainState;
  return proto.ubo.v1.MainState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MainState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MainState}
 */
proto.ubo.v1.MainState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Menu;
      reader.readMessage(value,proto.ubo.v1.Menu.deserializeBinaryFromReader);
      msg.setMenu(value);
      break;
    case 3:
      var value = new proto.ubo.v1.MainState.Path;
      reader.readMessage(value,proto.ubo.v1.MainState.Path.deserializeBinaryFromReader);
      msg.setPath(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setDepth(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsHeaderVisible(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsFooterVisible(value);
      break;
    case 7:
      var value = new proto.ubo.v1.MainState.AppsItemsPrioritiesDict;
      reader.readMessage(value,proto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinaryFromReader);
      msg.setAppsItemsPriorities(value);
      break;
    case 8:
      var value = new proto.ubo.v1.MainState.SettingsItemsPrioritiesDict;
      reader.readMessage(value,proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinaryFromReader);
      msg.setSettingsItemsPriorities(value);
      break;
    case 9:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsRecording(value);
      break;
    case 10:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsReplaying(value);
      break;
    case 11:
      var value = new proto.ubo.v1.MainState.RecordedSequence;
      reader.readMessage(value,proto.ubo.v1.MainState.RecordedSequence.deserializeBinaryFromReader);
      msg.setRecordedSequence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MainState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MainState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MainState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getMenu();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Menu.serializeBinaryToWriter
    );
  }
  f = message.getPath();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.MainState.Path.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
  f = message.getAppsItemsPriorities();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.ubo.v1.MainState.AppsItemsPrioritiesDict.serializeBinaryToWriter
    );
  }
  f = message.getSettingsItemsPriorities();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeBool(
      9,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeBool(
      10,
      f
    );
  }
  f = message.getRecordedSequence();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.ubo.v1.MainState.RecordedSequence.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.MainState.Path.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MainState.Path.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MainState.Path.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MainState.Path} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.Path.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MainState.Path}
 */
proto.ubo.v1.MainState.Path.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MainState.Path;
  return proto.ubo.v1.MainState.Path.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MainState.Path} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MainState.Path}
 */
proto.ubo.v1.MainState.Path.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MainState.Path.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MainState.Path.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MainState.Path} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.Path.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.MainState.Path.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.MainState.Path} returns this
 */
proto.ubo.v1.MainState.Path.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.MainState.Path} returns this
 */
proto.ubo.v1.MainState.Path.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.MainState.Path} returns this
 */
proto.ubo.v1.MainState.Path.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MainState.AppsItemsPrioritiesDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict}
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MainState.AppsItemsPrioritiesDict;
  return proto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict}
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readInt64, null, "", 0);
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MainState.AppsItemsPrioritiesDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeInt64);
  }
};


/**
 * map<string, int64> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,number>}
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,number>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.MainState.AppsItemsPrioritiesDict} returns this
 */
proto.ubo.v1.MainState.AppsItemsPrioritiesDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict}
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MainState.SettingsItemsPrioritiesDict;
  return proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict}
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readInt64, null, "", 0);
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeInt64);
  }
};


/**
 * map<string, int64> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,number>}
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,number>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} returns this
 */
proto.ubo.v1.MainState.SettingsItemsPrioritiesDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.MainState.RecordedSequence.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.MainState.RecordedSequence.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.MainState.RecordedSequence.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.MainState.RecordedSequence} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.RecordedSequence.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.Action.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.MainState.RecordedSequence}
 */
proto.ubo.v1.MainState.RecordedSequence.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.MainState.RecordedSequence;
  return proto.ubo.v1.MainState.RecordedSequence.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.MainState.RecordedSequence} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.MainState.RecordedSequence}
 */
proto.ubo.v1.MainState.RecordedSequence.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.Action;
      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.MainState.RecordedSequence.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.MainState.RecordedSequence.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.MainState.RecordedSequence} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.MainState.RecordedSequence.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.Action.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Action items = 1;
 * @return {!Array<!proto.ubo.v1.Action>}
 */
proto.ubo.v1.MainState.RecordedSequence.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Action>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Action>} value
 * @return {!proto.ubo.v1.MainState.RecordedSequence} returns this
*/
proto.ubo.v1.MainState.RecordedSequence.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.Action=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Action}
 */
proto.ubo.v1.MainState.RecordedSequence.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Action, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.MainState.RecordedSequence} returns this
 */
proto.ubo.v1.MainState.RecordedSequence.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_core_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.MainState.prototype.getMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.setMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotCoreDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Menu menu = 2;
 * @return {?proto.ubo.v1.Menu}
 */
proto.ubo.v1.MainState.prototype.getMenu = function() {
  return /** @type{?proto.ubo.v1.Menu} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Menu, 2));
};


/**
 * @param {?proto.ubo.v1.Menu|undefined} value
 * @return {!proto.ubo.v1.MainState} returns this
*/
proto.ubo.v1.MainState.prototype.setMenu = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearMenu = function() {
  return this.setMenu(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasMenu = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Path path = 3;
 * @return {?proto.ubo.v1.MainState.Path}
 */
proto.ubo.v1.MainState.prototype.getPath = function() {
  return /** @type{?proto.ubo.v1.MainState.Path} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.Path, 3));
};


/**
 * @param {?proto.ubo.v1.MainState.Path|undefined} value
 * @return {!proto.ubo.v1.MainState} returns this
*/
proto.ubo.v1.MainState.prototype.setPath = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearPath = function() {
  return this.setPath(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasPath = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 depth = 4;
 * @return {number}
 */
proto.ubo.v1.MainState.prototype.getDepth = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.setDepth = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearDepth = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasDepth = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_header_visible = 5;
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.getIsHeaderVisible = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.setIsHeaderVisible = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearIsHeaderVisible = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasIsHeaderVisible = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional bool is_footer_visible = 6;
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.getIsFooterVisible = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 6, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.setIsFooterVisible = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearIsFooterVisible = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasIsFooterVisible = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional AppsItemsPrioritiesDict apps_items_priorities = 7;
 * @return {?proto.ubo.v1.MainState.AppsItemsPrioritiesDict}
 */
proto.ubo.v1.MainState.prototype.getAppsItemsPriorities = function() {
  return /** @type{?proto.ubo.v1.MainState.AppsItemsPrioritiesDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.AppsItemsPrioritiesDict, 7));
};


/**
 * @param {?proto.ubo.v1.MainState.AppsItemsPrioritiesDict|undefined} value
 * @return {!proto.ubo.v1.MainState} returns this
*/
proto.ubo.v1.MainState.prototype.setAppsItemsPriorities = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearAppsItemsPriorities = function() {
  return this.setAppsItemsPriorities(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasAppsItemsPriorities = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional SettingsItemsPrioritiesDict settings_items_priorities = 8;
 * @return {?proto.ubo.v1.MainState.SettingsItemsPrioritiesDict}
 */
proto.ubo.v1.MainState.prototype.getSettingsItemsPriorities = function() {
  return /** @type{?proto.ubo.v1.MainState.SettingsItemsPrioritiesDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.SettingsItemsPrioritiesDict, 8));
};


/**
 * @param {?proto.ubo.v1.MainState.SettingsItemsPrioritiesDict|undefined} value
 * @return {!proto.ubo.v1.MainState} returns this
*/
proto.ubo.v1.MainState.prototype.setSettingsItemsPriorities = function(value) {
  return jspb.Message.setWrapperField(this, 8, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearSettingsItemsPriorities = function() {
  return this.setSettingsItemsPriorities(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasSettingsItemsPriorities = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional bool is_recording = 9;
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.getIsRecording = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 9, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.setIsRecording = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearIsRecording = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasIsRecording = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional bool is_replaying = 10;
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.getIsReplaying = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 10, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.setIsReplaying = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearIsReplaying = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasIsReplaying = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional RecordedSequence recorded_sequence = 11;
 * @return {?proto.ubo.v1.MainState.RecordedSequence}
 */
proto.ubo.v1.MainState.prototype.getRecordedSequence = function() {
  return /** @type{?proto.ubo.v1.MainState.RecordedSequence} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MainState.RecordedSequence, 11));
};


/**
 * @param {?proto.ubo.v1.MainState.RecordedSequence|undefined} value
 * @return {!proto.ubo.v1.MainState} returns this
*/
proto.ubo.v1.MainState.prototype.setRecordedSequence = function(value) {
  return jspb.Message.setWrapperField(this, 11, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.MainState} returns this
 */
proto.ubo.v1.MainState.prototype.clearRecordedSequence = function() {
  return this.setRecordedSequence(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.MainState.prototype.hasRecordedSequence = function() {
  return jspb.Message.getField(this, 11) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UboDispatchItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UboDispatchItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UboDispatchItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboDispatchItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUboActions: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    storeAction: (f = msg.getStoreAction()) && proto.ubo.v1.UboDispatchItem.StoreAction.toObject(includeInstance, f),
    key: jspb.Message.getFieldWithDefault(msg, 3, ""),
    label: jspb.Message.getFieldWithDefault(msg, 4, ""),
    color: jspb.Message.getFieldWithDefault(msg, 5, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 7, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UboDispatchItem}
 */
proto.ubo.v1.UboDispatchItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UboDispatchItem;
  return proto.ubo.v1.UboDispatchItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UboDispatchItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UboDispatchItem}
 */
proto.ubo.v1.UboDispatchItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUboActions(value);
      break;
    case 2:
      var value = new proto.ubo.v1.UboDispatchItem.StoreAction;
      reader.readMessage(value,proto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinaryFromReader);
      msg.setStoreAction(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UboDispatchItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UboDispatchItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UboDispatchItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboDispatchItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getStoreAction();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.UboDispatchItem.StoreAction.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UboDispatchItem.StoreAction2.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UboDispatchItem.StoreAction2} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.Action.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2}
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UboDispatchItem.StoreAction2;
  return proto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UboDispatchItem.StoreAction2} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2}
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.Action;
      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UboDispatchItem.StoreAction2.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UboDispatchItem.StoreAction2} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.Action.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Action items = 1;
 * @return {!Array<!proto.ubo.v1.Action>}
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Action>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Action>} value
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2} returns this
*/
proto.ubo.v1.UboDispatchItem.StoreAction2.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.Action=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Action}
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Action, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction2} returns this
 */
proto.ubo.v1.UboDispatchItem.StoreAction2.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase = {
  STORE_ACTION_NOT_SET: 0,
  UBO_ACTION: 1,
  LIST: 2
};

/**
 * @return {proto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.getStoreActionCase = function() {
  return /** @type {proto.ubo.v1.UboDispatchItem.StoreAction.StoreActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UboDispatchItem.StoreAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UboDispatchItem.StoreAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboDispatchItem.StoreAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    uboAction: (f = msg.getUboAction()) && proto.ubo.v1.Action.toObject(includeInstance, f),
    list: (f = msg.getList()) && proto.ubo.v1.UboDispatchItem.StoreAction2.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UboDispatchItem.StoreAction;
  return proto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UboDispatchItem.StoreAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.Action;
      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);
      msg.setUboAction(value);
      break;
    case 2:
      var value = new proto.ubo.v1.UboDispatchItem.StoreAction2;
      reader.readMessage(value,proto.ubo.v1.UboDispatchItem.StoreAction2.deserializeBinaryFromReader);
      msg.setList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UboDispatchItem.StoreAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UboDispatchItem.StoreAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboDispatchItem.StoreAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getUboAction();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.Action.serializeBinaryToWriter
    );
  }
  f = message.getList();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.UboDispatchItem.StoreAction2.serializeBinaryToWriter
    );
  }
};


/**
 * optional Action ubo_action = 1;
 * @return {?proto.ubo.v1.Action}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.getUboAction = function() {
  return /** @type{?proto.ubo.v1.Action} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Action, 1));
};


/**
 * @param {?proto.ubo.v1.Action|undefined} value
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this
*/
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.setUboAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.clearUboAction = function() {
  return this.setUboAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.hasUboAction = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional StoreAction2 list = 2;
 * @return {?proto.ubo.v1.UboDispatchItem.StoreAction2}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.getList = function() {
  return /** @type{?proto.ubo.v1.UboDispatchItem.StoreAction2} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UboDispatchItem.StoreAction2, 2));
};


/**
 * @param {?proto.ubo.v1.UboDispatchItem.StoreAction2|undefined} value
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this
*/
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.setList = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.UboDispatchItem.StoreAction.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem.StoreAction} returns this
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.clearList = function() {
  return this.setList(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.StoreAction.prototype.hasList = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_ubo_actions = 1000;
 * @return {string}
 */
proto.ubo.v1.UboDispatchItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotUboActions = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional StoreAction store_action = 2;
 * @return {?proto.ubo.v1.UboDispatchItem.StoreAction}
 */
proto.ubo.v1.UboDispatchItem.prototype.getStoreAction = function() {
  return /** @type{?proto.ubo.v1.UboDispatchItem.StoreAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UboDispatchItem.StoreAction, 2));
};


/**
 * @param {?proto.ubo.v1.UboDispatchItem.StoreAction|undefined} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
*/
proto.ubo.v1.UboDispatchItem.prototype.setStoreAction = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearStoreAction = function() {
  return this.setStoreAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasStoreAction = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string key = 3;
 * @return {string}
 */
proto.ubo.v1.UboDispatchItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string label = 4;
 * @return {string}
 */
proto.ubo.v1.UboDispatchItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string color = 5;
 * @return {string}
 */
proto.ubo.v1.UboDispatchItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string background_color = 6;
 * @return {string}
 */
proto.ubo.v1.UboDispatchItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string icon = 7;
 * @return {string}
 */
proto.ubo.v1.UboDispatchItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional bool is_short = 8;
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float opacity = 9;
 * @return {number}
 */
proto.ubo.v1.UboDispatchItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float progress = 10;
 * @return {number}
 */
proto.ubo.v1.UboDispatchItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboDispatchItem} returns this
 */
proto.ubo.v1.UboDispatchItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboDispatchItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 10) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UboApplicationItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UboApplicationItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UboApplicationItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUboActions: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    application: (f = msg.getApplication()) && proto.ubo.v1.UboApplicationItem.Application.toObject(includeInstance, f),
    applicationId: jspb.Message.getFieldWithDefault(msg, 3, ""),
    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.UboApplicationItem.InitializationArgs.toObject(includeInstance, f),
    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.toObject(includeInstance, f),
    key: jspb.Message.getFieldWithDefault(msg, 6, ""),
    label: jspb.Message.getFieldWithDefault(msg, 7, ""),
    color: jspb.Message.getFieldWithDefault(msg, 8, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 9, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 10, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 11, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 12, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 13, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UboApplicationItem}
 */
proto.ubo.v1.UboApplicationItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UboApplicationItem;
  return proto.ubo.v1.UboApplicationItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UboApplicationItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UboApplicationItem}
 */
proto.ubo.v1.UboApplicationItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUboActions(value);
      break;
    case 2:
      var value = new proto.ubo.v1.UboApplicationItem.Application;
      reader.readMessage(value,proto.ubo.v1.UboApplicationItem.Application.deserializeBinaryFromReader);
      msg.setApplication(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setApplicationId(value);
      break;
    case 4:
      var value = new proto.ubo.v1.UboApplicationItem.InitializationArgs;
      reader.readMessage(value,proto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinaryFromReader);
      msg.setInitializationArgs(value);
      break;
    case 5:
      var value = new proto.ubo.v1.UboApplicationItem.InitializationKwargsDict;
      reader.readMessage(value,proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader);
      msg.setInitializationKwargs(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 10:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 11:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 12:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 13:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UboApplicationItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UboApplicationItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UboApplicationItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getApplication();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.UboApplicationItem.Application.serializeBinaryToWriter
    );
  }
  f = message.getApplicationId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getInitializationArgs();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.UboApplicationItem.InitializationArgs.serializeBinaryToWriter
    );
  }
  f = message.getInitializationKwargs();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeString(
      10,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeBool(
      11,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeFloat(
      12,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeFloat(
      13,
      f
    );
  }
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.UboApplicationItem.Application.oneofGroups_ = [[1]];

/**
 * @enum {number}
 */
proto.ubo.v1.UboApplicationItem.Application.ApplicationCase = {
  APPLICATION_NOT_SET: 0,
  STRING: 1
};

/**
 * @return {proto.ubo.v1.UboApplicationItem.Application.ApplicationCase}
 */
proto.ubo.v1.UboApplicationItem.Application.prototype.getApplicationCase = function() {
  return /** @type {proto.ubo.v1.UboApplicationItem.Application.ApplicationCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UboApplicationItem.Application.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UboApplicationItem.Application.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UboApplicationItem.Application} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.Application.toObject = function(includeInstance, msg) {
  var f, obj = {
    string: jspb.Message.getFieldWithDefault(msg, 1, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UboApplicationItem.Application}
 */
proto.ubo.v1.UboApplicationItem.Application.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UboApplicationItem.Application;
  return proto.ubo.v1.UboApplicationItem.Application.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UboApplicationItem.Application} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UboApplicationItem.Application}
 */
proto.ubo.v1.UboApplicationItem.Application.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setString(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UboApplicationItem.Application.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UboApplicationItem.Application.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UboApplicationItem.Application} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.Application.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
};


/**
 * optional string string = 1;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.Application.prototype.getString = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem.Application} returns this
 */
proto.ubo.v1.UboApplicationItem.Application.prototype.setString = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem.Application} returns this
 */
proto.ubo.v1.UboApplicationItem.Application.prototype.clearString = function() {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.UboApplicationItem.Application.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.Application.prototype.hasString = function() {
  return jspb.Message.getField(this, 1) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UboApplicationItem.InitializationArgs.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UboApplicationItem.InitializationArgs} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.BasicType.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs}
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UboApplicationItem.InitializationArgs;
  return proto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UboApplicationItem.InitializationArgs} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs}
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UboApplicationItem.InitializationArgs.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UboApplicationItem.InitializationArgs} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
};


/**
 * repeated BasicType items = 1;
 * @return {!Array<!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.BasicType>} value
 * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs} returns this
*/
proto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.BasicType=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.UboApplicationItem.InitializationArgs} returns this
 */
proto.ubo.v1.UboApplicationItem.InitializationArgs.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.BasicType.toObject) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict}
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UboApplicationItem.InitializationKwargsDict;
  return proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict}
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.BasicType.deserializeBinaryFromReader, "", new proto.ubo.v1.BasicType());
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.BasicType.serializeBinaryToWriter);
  }
};


/**
 * map<string, BasicType> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!proto.ubo.v1.BasicType>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      proto.ubo.v1.BasicType));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} returns this
 */
proto.ubo.v1.UboApplicationItem.InitializationKwargsDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_ubo_actions = 1000;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotUboActions = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUboActions = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Application application = 2;
 * @return {?proto.ubo.v1.UboApplicationItem.Application}
 */
proto.ubo.v1.UboApplicationItem.prototype.getApplication = function() {
  return /** @type{?proto.ubo.v1.UboApplicationItem.Application} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UboApplicationItem.Application, 2));
};


/**
 * @param {?proto.ubo.v1.UboApplicationItem.Application|undefined} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
*/
proto.ubo.v1.UboApplicationItem.prototype.setApplication = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearApplication = function() {
  return this.setApplication(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasApplication = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string application_id = 3;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.prototype.getApplicationId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setApplicationId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional InitializationArgs initialization_args = 4;
 * @return {?proto.ubo.v1.UboApplicationItem.InitializationArgs}
 */
proto.ubo.v1.UboApplicationItem.prototype.getInitializationArgs = function() {
  return /** @type{?proto.ubo.v1.UboApplicationItem.InitializationArgs} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UboApplicationItem.InitializationArgs, 4));
};


/**
 * @param {?proto.ubo.v1.UboApplicationItem.InitializationArgs|undefined} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
*/
proto.ubo.v1.UboApplicationItem.prototype.setInitializationArgs = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearInitializationArgs = function() {
  return this.setInitializationArgs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasInitializationArgs = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional InitializationKwargsDict initialization_kwargs = 5;
 * @return {?proto.ubo.v1.UboApplicationItem.InitializationKwargsDict}
 */
proto.ubo.v1.UboApplicationItem.prototype.getInitializationKwargs = function() {
  return /** @type{?proto.ubo.v1.UboApplicationItem.InitializationKwargsDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UboApplicationItem.InitializationKwargsDict, 5));
};


/**
 * @param {?proto.ubo.v1.UboApplicationItem.InitializationKwargsDict|undefined} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
*/
proto.ubo.v1.UboApplicationItem.prototype.setInitializationKwargs = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearInitializationKwargs = function() {
  return this.setInitializationKwargs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasInitializationKwargs = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string key = 6;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string label = 7;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string color = 8;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string background_color = 9;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional string icon = 10;
 * @return {string}
 */
proto.ubo.v1.UboApplicationItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 10, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional bool is_short = 11;
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional float opacity = 12;
 * @return {number}
 */
proto.ubo.v1.UboApplicationItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 12, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional float progress = 13;
 * @return {number}
 */
proto.ubo.v1.UboApplicationItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 13, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UboApplicationItem} returns this
 */
proto.ubo.v1.UboApplicationItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UboApplicationItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 13) != null;
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.BasicTypeOptional.oneofGroups_ = [[1,2,3,4,5]];

/**
 * @enum {number}
 */
proto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase = {
  BASIC_TYPE_OPTIONAL_NOT_SET: 0,
  BOOL: 1,
  BYTES: 2,
  FLOAT: 3,
  INT64: 4,
  STRING: 5
};

/**
 * @return {proto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getBasicTypeOptionalCase = function() {
  return /** @type {proto.ubo.v1.BasicTypeOptional.BasicTypeOptionalCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.BasicTypeOptional.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.BasicTypeOptional.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.BasicTypeOptional} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.BasicTypeOptional.toObject = function(includeInstance, msg) {
  var f, obj = {
    bool: jspb.Message.getBooleanFieldWithDefault(msg, 1, false),
    bytes: msg.getBytes_asB64(),
    pb_float: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),
    int64: jspb.Message.getFieldWithDefault(msg, 4, 0),
    string: jspb.Message.getFieldWithDefault(msg, 5, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.BasicTypeOptional}
 */
proto.ubo.v1.BasicTypeOptional.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.BasicTypeOptional;
  return proto.ubo.v1.BasicTypeOptional.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.BasicTypeOptional} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.BasicTypeOptional}
 */
proto.ubo.v1.BasicTypeOptional.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBool(value);
      break;
    case 2:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setBytes(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setFloat(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setInt64(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setString(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.BasicTypeOptional.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.BasicTypeOptional.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.BasicTypeOptional} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.BasicTypeOptional.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {boolean} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeBool(
      1,
      f
    );
  }
  f = /** @type {!(string|Uint8Array)} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBytes(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
};


/**
 * optional bool bool = 1;
 * @return {boolean}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getBool = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 1, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.setBool = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.clearBool = function() {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BasicTypeOptional.prototype.hasBool = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional bytes bytes = 2;
 * @return {string}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getBytes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * optional bytes bytes = 2;
 * This is a type-conversion wrapper around `getBytes()`
 * @return {string}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getBytes_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getBytes()));
};


/**
 * optional bytes bytes = 2;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getBytes()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getBytes_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getBytes()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.setBytes = function(value) {
  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.clearBytes = function() {
  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BasicTypeOptional.prototype.hasBytes = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float float = 3;
 * @return {number}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getFloat = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.setFloat = function(value) {
  return jspb.Message.setOneofField(this, 3, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.clearFloat = function() {
  return jspb.Message.setOneofField(this, 3, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BasicTypeOptional.prototype.hasFloat = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 int64 = 4;
 * @return {number}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getInt64 = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.setInt64 = function(value) {
  return jspb.Message.setOneofField(this, 4, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.clearInt64 = function() {
  return jspb.Message.setOneofField(this, 4, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BasicTypeOptional.prototype.hasInt64 = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string string = 5;
 * @return {string}
 */
proto.ubo.v1.BasicTypeOptional.prototype.getString = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.setString = function(value) {
  return jspb.Message.setOneofField(this, 5, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.BasicTypeOptional} returns this
 */
proto.ubo.v1.BasicTypeOptional.prototype.clearString = function() {
  return jspb.Message.setOneofField(this, 5, proto.ubo.v1.BasicTypeOptional.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BasicTypeOptional.prototype.hasString = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.BasicType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.BasicType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.BasicType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.BasicType.toObject = function(includeInstance, msg) {
  var f, obj = {
    items: (f = msg.getItems()) && proto.ubo.v1.BasicTypeOptional.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.BasicType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.BasicType;
  return proto.ubo.v1.BasicType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.BasicType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.BasicType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicTypeOptional;
      reader.readMessage(value,proto.ubo.v1.BasicTypeOptional.deserializeBinaryFromReader);
      msg.setItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.BasicType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.BasicType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.BasicType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.BasicType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItems();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.BasicTypeOptional.serializeBinaryToWriter
    );
  }
};


/**
 * optional BasicTypeOptional items = 1;
 * @return {?proto.ubo.v1.BasicTypeOptional}
 */
proto.ubo.v1.BasicType.prototype.getItems = function() {
  return /** @type{?proto.ubo.v1.BasicTypeOptional} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.BasicTypeOptional, 1));
};


/**
 * @param {?proto.ubo.v1.BasicTypeOptional|undefined} value
 * @return {!proto.ubo.v1.BasicType} returns this
*/
proto.ubo.v1.BasicType.prototype.setItems = function(value) {
  return jspb.Message.setWrapperField(this, 1, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.BasicType} returns this
 */
proto.ubo.v1.BasicType.prototype.clearItems = function() {
  return this.setItems(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.BasicType.prototype.hasItems = function() {
  return jspb.Message.getField(this, 1) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputResult.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputResult.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputResult} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputResult.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    dataMap: (f = msg.getDataMap()) ? f.toObject(includeInstance, undefined) : [],
    filesMap: (f = msg.getFilesMap()) ? f.toObject(includeInstance, undefined) : [],
    method: jspb.Message.getFieldWithDefault(msg, 4, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputResult}
 */
proto.ubo.v1.InputResult.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputResult;
  return proto.ubo.v1.InputResult.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputResult} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputResult}
 */
proto.ubo.v1.InputResult.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = msg.getDataMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, "", "");
         });
      break;
    case 3:
      var value = msg.getFilesMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readBytes, null, "", "");
         });
      break;
    case 4:
      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());
      msg.setMethod(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputResult.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputResult.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputResult} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputResult.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDataMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(2, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);
  }
  f = message.getFilesMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(3, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeBytes);
  }
  f = message.getMethod();
  if (f !== 0.0) {
    writer.writeEnum(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputResult.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputResult} returns this
 */
proto.ubo.v1.InputResult.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputResult} returns this
 */
proto.ubo.v1.InputResult.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputResult.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * map<string, string> data = 2;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,string>}
 */
proto.ubo.v1.InputResult.prototype.getDataMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,string>} */ (
      jspb.Message.getMapField(this, 2, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.InputResult} returns this
 */
proto.ubo.v1.InputResult.prototype.clearDataMap = function() {
  this.getDataMap().clear();
  return this;};


/**
 * map<string, bytes> files = 3;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!(string|Uint8Array)>}
 */
proto.ubo.v1.InputResult.prototype.getFilesMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!(string|Uint8Array)>} */ (
      jspb.Message.getMapField(this, 3, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.InputResult} returns this
 */
proto.ubo.v1.InputResult.prototype.clearFilesMap = function() {
  this.getFilesMap().clear();
  return this;};


/**
 * optional InputMethod method = 4;
 * @return {!proto.ubo.v1.InputMethod}
 */
proto.ubo.v1.InputResult.prototype.getMethod = function() {
  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {!proto.ubo.v1.InputMethod} value
 * @return {!proto.ubo.v1.InputResult} returns this
 */
proto.ubo.v1.InputResult.prototype.setMethod = function(value) {
  return jspb.Message.setProto3EnumField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputFieldDescription.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputFieldDescription.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputFieldDescription} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputFieldDescription.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    name: jspb.Message.getFieldWithDefault(msg, 2, ""),
    label: jspb.Message.getFieldWithDefault(msg, 3, ""),
    type: jspb.Message.getFieldWithDefault(msg, 4, 0),
    description: jspb.Message.getFieldWithDefault(msg, 5, ""),
    title: jspb.Message.getFieldWithDefault(msg, 6, ""),
    fileMimetype: jspb.Message.getFieldWithDefault(msg, 7, ""),
    pattern: jspb.Message.getFieldWithDefault(msg, 8, ""),
    defaultValue: jspb.Message.getFieldWithDefault(msg, 9, ""),
    options: (f = msg.getOptions()) && proto.ubo.v1.InputFieldDescription.Options.toObject(includeInstance, f),
    required: jspb.Message.getBooleanFieldWithDefault(msg, 11, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputFieldDescription}
 */
proto.ubo.v1.InputFieldDescription.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputFieldDescription;
  return proto.ubo.v1.InputFieldDescription.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputFieldDescription} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputFieldDescription}
 */
proto.ubo.v1.InputFieldDescription.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 4:
      var value = /** @type {!proto.ubo.v1.InputFieldType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setDescription(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setFileMimetype(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setPattern(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setDefaultValue(value);
      break;
    case 10:
      var value = new proto.ubo.v1.InputFieldDescription.Options;
      reader.readMessage(value,proto.ubo.v1.InputFieldDescription.Options.deserializeBinaryFromReader);
      msg.setOptions(value);
      break;
    case 11:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setRequired(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputFieldDescription.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputFieldDescription.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputFieldDescription} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputFieldDescription.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getName();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getLabel();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getType();
  if (f !== 0.0) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getOptions();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.ubo.v1.InputFieldDescription.Options.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeBool(
      11,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.InputFieldDescription.Options.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputFieldDescription.Options.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputFieldDescription.Options.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputFieldDescription.Options} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputFieldDescription.Options.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputFieldDescription.Options}
 */
proto.ubo.v1.InputFieldDescription.Options.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputFieldDescription.Options;
  return proto.ubo.v1.InputFieldDescription.Options.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputFieldDescription.Options} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputFieldDescription.Options}
 */
proto.ubo.v1.InputFieldDescription.Options.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputFieldDescription.Options.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputFieldDescription.Options.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputFieldDescription.Options} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputFieldDescription.Options.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.InputFieldDescription.Options.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.InputFieldDescription.Options} returns this
 */
proto.ubo.v1.InputFieldDescription.Options.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.InputFieldDescription.Options} returns this
 */
proto.ubo.v1.InputFieldDescription.Options.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.InputFieldDescription.Options} returns this
 */
proto.ubo.v1.InputFieldDescription.Options.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string name = 2;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setName = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string label = 3;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setLabel = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional InputFieldType type = 4;
 * @return {!proto.ubo.v1.InputFieldType}
 */
proto.ubo.v1.InputFieldDescription.prototype.getType = function() {
  return /** @type {!proto.ubo.v1.InputFieldType} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {!proto.ubo.v1.InputFieldType} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setType = function(value) {
  return jspb.Message.setProto3EnumField(this, 4, value);
};


/**
 * optional string description = 5;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getDescription = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setDescription = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearDescription = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasDescription = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string title = 6;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setTitle = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearTitle = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasTitle = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string file_mimetype = 7;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getFileMimetype = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setFileMimetype = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearFileMimetype = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasFileMimetype = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string pattern = 8;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getPattern = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setPattern = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearPattern = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasPattern = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string default_value = 9;
 * @return {string}
 */
proto.ubo.v1.InputFieldDescription.prototype.getDefaultValue = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setDefaultValue = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearDefaultValue = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasDefaultValue = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional Options options = 10;
 * @return {?proto.ubo.v1.InputFieldDescription.Options}
 */
proto.ubo.v1.InputFieldDescription.prototype.getOptions = function() {
  return /** @type{?proto.ubo.v1.InputFieldDescription.Options} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputFieldDescription.Options, 10));
};


/**
 * @param {?proto.ubo.v1.InputFieldDescription.Options|undefined} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
*/
proto.ubo.v1.InputFieldDescription.prototype.setOptions = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearOptions = function() {
  return this.setOptions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasOptions = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional bool required = 11;
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.getRequired = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.setRequired = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputFieldDescription} returns this
 */
proto.ubo.v1.InputFieldDescription.prototype.clearRequired = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputFieldDescription.prototype.hasRequired = function() {
  return jspb.Message.getField(this, 11) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputDescription.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputDescription.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputDescription} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputDescription.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),
    id: jspb.Message.getFieldWithDefault(msg, 3, ""),
    title: jspb.Message.getFieldWithDefault(msg, 4, ""),
    prompt: jspb.Message.getFieldWithDefault(msg, 5, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputDescription}
 */
proto.ubo.v1.InputDescription.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputDescription;
  return proto.ubo.v1.InputDescription.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputDescription} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputDescription}
 */
proto.ubo.v1.InputDescription.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());
      msg.setInputMethod(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setPrompt(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputDescription.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputDescription.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputDescription} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputDescription.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getInputMethod();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional InputMethod input_method = 2;
 * @return {!proto.ubo.v1.InputMethod}
 */
proto.ubo.v1.InputDescription.prototype.getInputMethod = function() {
  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.InputMethod} value
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.setInputMethod = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional string id = 3;
 * @return {string}
 */
proto.ubo.v1.InputDescription.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.setId = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.clearId = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputDescription.prototype.hasId = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string title = 4;
 * @return {string}
 */
proto.ubo.v1.InputDescription.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.setTitle = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.clearTitle = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputDescription.prototype.hasTitle = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string prompt = 5;
 * @return {string}
 */
proto.ubo.v1.InputDescription.prototype.getPrompt = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.setPrompt = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputDescription} returns this
 */
proto.ubo.v1.InputDescription.prototype.clearPrompt = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputDescription.prototype.hasPrompt = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WebUIInputDescription.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WebUIInputDescription.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WebUIInputDescription} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIInputDescription.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),
    fields: (f = msg.getFields()) && proto.ubo.v1.WebUIInputDescription.Fields.toObject(includeInstance, f),
    id: jspb.Message.getFieldWithDefault(msg, 4, ""),
    title: jspb.Message.getFieldWithDefault(msg, 5, ""),
    prompt: jspb.Message.getFieldWithDefault(msg, 6, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WebUIInputDescription}
 */
proto.ubo.v1.WebUIInputDescription.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WebUIInputDescription;
  return proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WebUIInputDescription} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WebUIInputDescription}
 */
proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());
      msg.setInputMethod(value);
      break;
    case 3:
      var value = new proto.ubo.v1.WebUIInputDescription.Fields;
      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.Fields.deserializeBinaryFromReader);
      msg.setFields(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setPrompt(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WebUIInputDescription.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WebUIInputDescription} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getFields();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.WebUIInputDescription.Fields.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.WebUIInputDescription.Fields.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WebUIInputDescription.Fields.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WebUIInputDescription.Fields.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WebUIInputDescription.Fields} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIInputDescription.Fields.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.InputFieldDescription.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WebUIInputDescription.Fields}
 */
proto.ubo.v1.WebUIInputDescription.Fields.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WebUIInputDescription.Fields;
  return proto.ubo.v1.WebUIInputDescription.Fields.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WebUIInputDescription.Fields} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WebUIInputDescription.Fields}
 */
proto.ubo.v1.WebUIInputDescription.Fields.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.InputFieldDescription;
      reader.readMessage(value,proto.ubo.v1.InputFieldDescription.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WebUIInputDescription.Fields.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WebUIInputDescription.Fields.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WebUIInputDescription.Fields} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIInputDescription.Fields.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.InputFieldDescription.serializeBinaryToWriter
    );
  }
};


/**
 * repeated InputFieldDescription items = 1;
 * @return {!Array<!proto.ubo.v1.InputFieldDescription>}
 */
proto.ubo.v1.WebUIInputDescription.Fields.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.InputFieldDescription>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.InputFieldDescription, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.InputFieldDescription>} value
 * @return {!proto.ubo.v1.WebUIInputDescription.Fields} returns this
*/
proto.ubo.v1.WebUIInputDescription.Fields.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.InputFieldDescription=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.InputFieldDescription}
 */
proto.ubo.v1.WebUIInputDescription.Fields.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.InputFieldDescription, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.WebUIInputDescription.Fields} returns this
 */
proto.ubo.v1.WebUIInputDescription.Fields.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.WebUIInputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional InputMethod input_method = 2;
 * @return {!proto.ubo.v1.InputMethod}
 */
proto.ubo.v1.WebUIInputDescription.prototype.getInputMethod = function() {
  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.InputMethod} value
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.setInputMethod = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.clearInputMethod = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInputDescription.prototype.hasInputMethod = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Fields fields = 3;
 * @return {?proto.ubo.v1.WebUIInputDescription.Fields}
 */
proto.ubo.v1.WebUIInputDescription.prototype.getFields = function() {
  return /** @type{?proto.ubo.v1.WebUIInputDescription.Fields} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInputDescription.Fields, 3));
};


/**
 * @param {?proto.ubo.v1.WebUIInputDescription.Fields|undefined} value
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
*/
proto.ubo.v1.WebUIInputDescription.prototype.setFields = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.clearFields = function() {
  return this.setFields(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInputDescription.prototype.hasFields = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string id = 4;
 * @return {string}
 */
proto.ubo.v1.WebUIInputDescription.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.setId = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.clearId = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInputDescription.prototype.hasId = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string title = 5;
 * @return {string}
 */
proto.ubo.v1.WebUIInputDescription.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.setTitle = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.clearTitle = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInputDescription.prototype.hasTitle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string prompt = 6;
 * @return {string}
 */
proto.ubo.v1.WebUIInputDescription.prototype.getPrompt = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.setPrompt = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIInputDescription} returns this
 */
proto.ubo.v1.WebUIInputDescription.prototype.clearPrompt = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInputDescription.prototype.hasPrompt = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.QRCodeInputDescription.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.QRCodeInputDescription} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.QRCodeInputDescription.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),
    instructions: (f = msg.getInstructions()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),
    pattern: jspb.Message.getFieldWithDefault(msg, 4, ""),
    id: jspb.Message.getFieldWithDefault(msg, 5, ""),
    title: jspb.Message.getFieldWithDefault(msg, 6, ""),
    prompt: jspb.Message.getFieldWithDefault(msg, 7, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.QRCodeInputDescription}
 */
proto.ubo.v1.QRCodeInputDescription.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.QRCodeInputDescription;
  return proto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.QRCodeInputDescription} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.QRCodeInputDescription}
 */
proto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());
      msg.setInputMethod(value);
      break;
    case 3:
      var value = new proto.ubo.v1.ReadableInformation;
      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);
      msg.setInstructions(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setPattern(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setPrompt(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.QRCodeInputDescription} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getInstructions();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional InputMethod input_method = 2;
 * @return {!proto.ubo.v1.InputMethod}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.getInputMethod = function() {
  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.InputMethod} value
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.setInputMethod = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.clearInputMethod = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.hasInputMethod = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional ReadableInformation instructions = 3;
 * @return {?proto.ubo.v1.ReadableInformation}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.getInstructions = function() {
  return /** @type{?proto.ubo.v1.ReadableInformation} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 3));
};


/**
 * @param {?proto.ubo.v1.ReadableInformation|undefined} value
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
*/
proto.ubo.v1.QRCodeInputDescription.prototype.setInstructions = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.clearInstructions = function() {
  return this.setInstructions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.hasInstructions = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string pattern = 4;
 * @return {string}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.getPattern = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.setPattern = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.clearPattern = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.hasPattern = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string id = 5;
 * @return {string}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.setId = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.clearId = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.hasId = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string title = 6;
 * @return {string}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.setTitle = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.clearTitle = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.hasTitle = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string prompt = 7;
 * @return {string}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.getPrompt = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.setPrompt = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.QRCodeInputDescription} returns this
 */
proto.ubo.v1.QRCodeInputDescription.prototype.clearPrompt = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.QRCodeInputDescription.prototype.hasPrompt = function() {
  return jspb.Message.getField(this, 7) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.PathInputDescription.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.PathInputDescription.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.PathInputDescription} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PathInputDescription.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    inputMethod: jspb.Message.getFieldWithDefault(msg, 2, 0),
    selectorConfig: (f = msg.getSelectorConfig()) && proto.ubo.v1.PathSelectorConfig.toObject(includeInstance, f),
    id: jspb.Message.getFieldWithDefault(msg, 4, ""),
    title: jspb.Message.getFieldWithDefault(msg, 5, ""),
    prompt: jspb.Message.getFieldWithDefault(msg, 6, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.PathInputDescription}
 */
proto.ubo.v1.PathInputDescription.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.PathInputDescription;
  return proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.PathInputDescription} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.PathInputDescription}
 */
proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.InputMethod} */ (reader.readEnum());
      msg.setInputMethod(value);
      break;
    case 3:
      var value = new proto.ubo.v1.PathSelectorConfig;
      reader.readMessage(value,proto.ubo.v1.PathSelectorConfig.deserializeBinaryFromReader);
      msg.setSelectorConfig(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setPrompt(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.PathInputDescription.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.PathInputDescription.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.PathInputDescription} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PathInputDescription.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getSelectorConfig();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.PathSelectorConfig.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.PathInputDescription.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathInputDescription.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional InputMethod input_method = 2;
 * @return {!proto.ubo.v1.InputMethod}
 */
proto.ubo.v1.PathInputDescription.prototype.getInputMethod = function() {
  return /** @type {!proto.ubo.v1.InputMethod} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.InputMethod} value
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.setInputMethod = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.clearInputMethod = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathInputDescription.prototype.hasInputMethod = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional PathSelectorConfig selector_config = 3;
 * @return {?proto.ubo.v1.PathSelectorConfig}
 */
proto.ubo.v1.PathInputDescription.prototype.getSelectorConfig = function() {
  return /** @type{?proto.ubo.v1.PathSelectorConfig} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PathSelectorConfig, 3));
};


/**
 * @param {?proto.ubo.v1.PathSelectorConfig|undefined} value
 * @return {!proto.ubo.v1.PathInputDescription} returns this
*/
proto.ubo.v1.PathInputDescription.prototype.setSelectorConfig = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.clearSelectorConfig = function() {
  return this.setSelectorConfig(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathInputDescription.prototype.hasSelectorConfig = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string id = 4;
 * @return {string}
 */
proto.ubo.v1.PathInputDescription.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.setId = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.clearId = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathInputDescription.prototype.hasId = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string title = 5;
 * @return {string}
 */
proto.ubo.v1.PathInputDescription.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.setTitle = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.clearTitle = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathInputDescription.prototype.hasTitle = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string prompt = 6;
 * @return {string}
 */
proto.ubo.v1.PathInputDescription.prototype.getPrompt = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.setPrompt = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathInputDescription} returns this
 */
proto.ubo.v1.PathInputDescription.prototype.clearPrompt = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathInputDescription.prototype.hasPrompt = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputAction}
 */
proto.ubo.v1.InputAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputAction;
  return proto.ubo.v1.InputAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputAction}
 */
proto.ubo.v1.InputAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputAction} returns this
 */
proto.ubo.v1.InputAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputAction} returns this
 */
proto.ubo.v1.InputAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputDemandAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputDemandAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputDemandAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputDemandAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    description: (f = msg.getDescription()) && proto.ubo.v1.AvailableInputDescription.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputDemandAction}
 */
proto.ubo.v1.InputDemandAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputDemandAction;
  return proto.ubo.v1.InputDemandAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputDemandAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputDemandAction}
 */
proto.ubo.v1.InputDemandAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.AvailableInputDescription;
      reader.readMessage(value,proto.ubo.v1.AvailableInputDescription.deserializeBinaryFromReader);
      msg.setDescription(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputDemandAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputDemandAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputDemandAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputDemandAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDescription();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.AvailableInputDescription.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputDemandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputDemandAction} returns this
 */
proto.ubo.v1.InputDemandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputDemandAction} returns this
 */
proto.ubo.v1.InputDemandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputDemandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AvailableInputDescription description = 2;
 * @return {?proto.ubo.v1.AvailableInputDescription}
 */
proto.ubo.v1.InputDemandAction.prototype.getDescription = function() {
  return /** @type{?proto.ubo.v1.AvailableInputDescription} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AvailableInputDescription, 2));
};


/**
 * @param {?proto.ubo.v1.AvailableInputDescription|undefined} value
 * @return {!proto.ubo.v1.InputDemandAction} returns this
*/
proto.ubo.v1.InputDemandAction.prototype.setDescription = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.InputDemandAction} returns this
 */
proto.ubo.v1.InputDemandAction.prototype.clearDescription = function() {
  return this.setDescription(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputDemandAction.prototype.hasDescription = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputResolveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputResolveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputResolveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputResolveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputResolveAction}
 */
proto.ubo.v1.InputResolveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputResolveAction;
  return proto.ubo.v1.InputResolveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputResolveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputResolveAction}
 */
proto.ubo.v1.InputResolveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputResolveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputResolveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputResolveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputResolveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputResolveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputResolveAction} returns this
 */
proto.ubo.v1.InputResolveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputResolveAction} returns this
 */
proto.ubo.v1.InputResolveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputResolveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.InputResolveAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputResolveAction} returns this
 */
proto.ubo.v1.InputResolveAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputCancelAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputCancelAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputCancelAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputCancelAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputCancelAction}
 */
proto.ubo.v1.InputCancelAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputCancelAction;
  return proto.ubo.v1.InputCancelAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputCancelAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputCancelAction}
 */
proto.ubo.v1.InputCancelAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputCancelAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputCancelAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputCancelAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputCancelAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputCancelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputCancelAction} returns this
 */
proto.ubo.v1.InputCancelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputCancelAction} returns this
 */
proto.ubo.v1.InputCancelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputCancelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.InputCancelAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputCancelAction} returns this
 */
proto.ubo.v1.InputCancelAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputProvideAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputProvideAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputProvideAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputProvideAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    value: jspb.Message.getFieldWithDefault(msg, 2, ""),
    result: (f = msg.getResult()) && proto.ubo.v1.InputResult.toObject(includeInstance, f),
    id: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputProvideAction}
 */
proto.ubo.v1.InputProvideAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputProvideAction;
  return proto.ubo.v1.InputProvideAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputProvideAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputProvideAction}
 */
proto.ubo.v1.InputProvideAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setValue(value);
      break;
    case 3:
      var value = new proto.ubo.v1.InputResult;
      reader.readMessage(value,proto.ubo.v1.InputResult.deserializeBinaryFromReader);
      msg.setResult(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputProvideAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputProvideAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputProvideAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputProvideAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getValue();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getResult();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.InputResult.serializeBinaryToWriter
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputProvideAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputProvideAction} returns this
 */
proto.ubo.v1.InputProvideAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputProvideAction} returns this
 */
proto.ubo.v1.InputProvideAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputProvideAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string value = 2;
 * @return {string}
 */
proto.ubo.v1.InputProvideAction.prototype.getValue = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputProvideAction} returns this
 */
proto.ubo.v1.InputProvideAction.prototype.setValue = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional InputResult result = 3;
 * @return {?proto.ubo.v1.InputResult}
 */
proto.ubo.v1.InputProvideAction.prototype.getResult = function() {
  return /** @type{?proto.ubo.v1.InputResult} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResult, 3));
};


/**
 * @param {?proto.ubo.v1.InputResult|undefined} value
 * @return {!proto.ubo.v1.InputProvideAction} returns this
*/
proto.ubo.v1.InputProvideAction.prototype.setResult = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.InputProvideAction} returns this
 */
proto.ubo.v1.InputProvideAction.prototype.clearResult = function() {
  return this.setResult(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputProvideAction.prototype.hasResult = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string id = 4;
 * @return {string}
 */
proto.ubo.v1.InputProvideAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputProvideAction} returns this
 */
proto.ubo.v1.InputProvideAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputResolveEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputResolveEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputResolveEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputResolveEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputResolveEvent}
 */
proto.ubo.v1.InputResolveEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputResolveEvent;
  return proto.ubo.v1.InputResolveEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputResolveEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputResolveEvent}
 */
proto.ubo.v1.InputResolveEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputResolveEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputResolveEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputResolveEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputResolveEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputResolveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputResolveEvent} returns this
 */
proto.ubo.v1.InputResolveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputResolveEvent} returns this
 */
proto.ubo.v1.InputResolveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputResolveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.InputResolveEvent.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputResolveEvent} returns this
 */
proto.ubo.v1.InputResolveEvent.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputCancelEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputCancelEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputCancelEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputCancelEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputCancelEvent}
 */
proto.ubo.v1.InputCancelEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputCancelEvent;
  return proto.ubo.v1.InputCancelEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputCancelEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputCancelEvent}
 */
proto.ubo.v1.InputCancelEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputCancelEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputCancelEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputCancelEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputCancelEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputCancelEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputCancelEvent} returns this
 */
proto.ubo.v1.InputCancelEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputCancelEvent} returns this
 */
proto.ubo.v1.InputCancelEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputCancelEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.InputCancelEvent.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputCancelEvent} returns this
 */
proto.ubo.v1.InputCancelEvent.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InputProvideEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InputProvideEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InputProvideEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputProvideEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotInputDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    value: jspb.Message.getFieldWithDefault(msg, 2, ""),
    result: (f = msg.getResult()) && proto.ubo.v1.InputResult.toObject(includeInstance, f),
    id: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InputProvideEvent}
 */
proto.ubo.v1.InputProvideEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InputProvideEvent;
  return proto.ubo.v1.InputProvideEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InputProvideEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InputProvideEvent}
 */
proto.ubo.v1.InputProvideEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setValue(value);
      break;
    case 3:
      var value = new proto.ubo.v1.InputResult;
      reader.readMessage(value,proto.ubo.v1.InputResult.deserializeBinaryFromReader);
      msg.setResult(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InputProvideEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InputProvideEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InputProvideEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InputProvideEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getValue();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getResult();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.InputResult.serializeBinaryToWriter
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_input_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.InputProvideEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputProvideEvent} returns this
 */
proto.ubo.v1.InputProvideEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InputProvideEvent} returns this
 */
proto.ubo.v1.InputProvideEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputProvideEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotInputDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string value = 2;
 * @return {string}
 */
proto.ubo.v1.InputProvideEvent.prototype.getValue = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputProvideEvent} returns this
 */
proto.ubo.v1.InputProvideEvent.prototype.setValue = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional InputResult result = 3;
 * @return {?proto.ubo.v1.InputResult}
 */
proto.ubo.v1.InputProvideEvent.prototype.getResult = function() {
  return /** @type{?proto.ubo.v1.InputResult} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResult, 3));
};


/**
 * @param {?proto.ubo.v1.InputResult|undefined} value
 * @return {!proto.ubo.v1.InputProvideEvent} returns this
*/
proto.ubo.v1.InputProvideEvent.prototype.setResult = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.InputProvideEvent} returns this
 */
proto.ubo.v1.InputProvideEvent.prototype.clearResult = function() {
  return this.setResult(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InputProvideEvent.prototype.hasResult = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string id = 4;
 * @return {string}
 */
proto.ubo.v1.InputProvideEvent.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InputProvideEvent} returns this
 */
proto.ubo.v1.InputProvideEvent.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.AvailableInputDescription.oneofGroups_ = [[1,2,3]];

/**
 * @enum {number}
 */
proto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase = {
  AVAILABLE_INPUT_DESCRIPTION_NOT_SET: 0,
  PATH_INPUT_DESCRIPTION: 1,
  QR_CODE_INPUT_DESCRIPTION: 2,
  WEB_UI_INPUT_DESCRIPTION: 3
};

/**
 * @return {proto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase}
 */
proto.ubo.v1.AvailableInputDescription.prototype.getAvailableInputDescriptionCase = function() {
  return /** @type {proto.ubo.v1.AvailableInputDescription.AvailableInputDescriptionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AvailableInputDescription.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AvailableInputDescription.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AvailableInputDescription} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AvailableInputDescription.toObject = function(includeInstance, msg) {
  var f, obj = {
    pathInputDescription: (f = msg.getPathInputDescription()) && proto.ubo.v1.PathInputDescription.toObject(includeInstance, f),
    qrCodeInputDescription: (f = msg.getQrCodeInputDescription()) && proto.ubo.v1.QRCodeInputDescription.toObject(includeInstance, f),
    webUiInputDescription: (f = msg.getWebUiInputDescription()) && proto.ubo.v1.WebUIInputDescription.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AvailableInputDescription}
 */
proto.ubo.v1.AvailableInputDescription.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AvailableInputDescription;
  return proto.ubo.v1.AvailableInputDescription.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AvailableInputDescription} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AvailableInputDescription}
 */
proto.ubo.v1.AvailableInputDescription.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.PathInputDescription;
      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);
      msg.setPathInputDescription(value);
      break;
    case 2:
      var value = new proto.ubo.v1.QRCodeInputDescription;
      reader.readMessage(value,proto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader);
      msg.setQrCodeInputDescription(value);
      break;
    case 3:
      var value = new proto.ubo.v1.WebUIInputDescription;
      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader);
      msg.setWebUiInputDescription(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AvailableInputDescription.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AvailableInputDescription.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AvailableInputDescription} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AvailableInputDescription.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getPathInputDescription();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter
    );
  }
  f = message.getQrCodeInputDescription();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter
    );
  }
  f = message.getWebUiInputDescription();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter
    );
  }
};


/**
 * optional PathInputDescription path_input_description = 1;
 * @return {?proto.ubo.v1.PathInputDescription}
 */
proto.ubo.v1.AvailableInputDescription.prototype.getPathInputDescription = function() {
  return /** @type{?proto.ubo.v1.PathInputDescription} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PathInputDescription, 1));
};


/**
 * @param {?proto.ubo.v1.PathInputDescription|undefined} value
 * @return {!proto.ubo.v1.AvailableInputDescription} returns this
*/
proto.ubo.v1.AvailableInputDescription.prototype.setPathInputDescription = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AvailableInputDescription} returns this
 */
proto.ubo.v1.AvailableInputDescription.prototype.clearPathInputDescription = function() {
  return this.setPathInputDescription(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AvailableInputDescription.prototype.hasPathInputDescription = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional QRCodeInputDescription qr_code_input_description = 2;
 * @return {?proto.ubo.v1.QRCodeInputDescription}
 */
proto.ubo.v1.AvailableInputDescription.prototype.getQrCodeInputDescription = function() {
  return /** @type{?proto.ubo.v1.QRCodeInputDescription} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.QRCodeInputDescription, 2));
};


/**
 * @param {?proto.ubo.v1.QRCodeInputDescription|undefined} value
 * @return {!proto.ubo.v1.AvailableInputDescription} returns this
*/
proto.ubo.v1.AvailableInputDescription.prototype.setQrCodeInputDescription = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AvailableInputDescription} returns this
 */
proto.ubo.v1.AvailableInputDescription.prototype.clearQrCodeInputDescription = function() {
  return this.setQrCodeInputDescription(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AvailableInputDescription.prototype.hasQrCodeInputDescription = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional WebUIInputDescription web_ui_input_description = 3;
 * @return {?proto.ubo.v1.WebUIInputDescription}
 */
proto.ubo.v1.AvailableInputDescription.prototype.getWebUiInputDescription = function() {
  return /** @type{?proto.ubo.v1.WebUIInputDescription} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInputDescription, 3));
};


/**
 * @param {?proto.ubo.v1.WebUIInputDescription|undefined} value
 * @return {!proto.ubo.v1.AvailableInputDescription} returns this
*/
proto.ubo.v1.AvailableInputDescription.prototype.setWebUiInputDescription = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.AvailableInputDescription.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AvailableInputDescription} returns this
 */
proto.ubo.v1.AvailableInputDescription.prototype.clearWebUiInputDescription = function() {
  return this.setWebUiInputDescription(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AvailableInputDescription.prototype.hasWebUiInputDescription = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsAction}
 */
proto.ubo.v1.SettingsAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsAction;
  return proto.ubo.v1.SettingsAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsAction}
 */
proto.ubo.v1.SettingsAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsAction} returns this
 */
proto.ubo.v1.SettingsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsAction} returns this
 */
proto.ubo.v1.SettingsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsTogglePdbSignalAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsTogglePdbSignalAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction}
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsTogglePdbSignalAction;
  return proto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsTogglePdbSignalAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction}
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsTogglePdbSignalAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsTogglePdbSignalAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction} returns this
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsTogglePdbSignalAction} returns this
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsTogglePdbSignalAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsToggleVisualDebugAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsToggleVisualDebugAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction}
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsToggleVisualDebugAction;
  return proto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsToggleVisualDebugAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction}
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsToggleVisualDebugAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsToggleVisualDebugAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction} returns this
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsToggleVisualDebugAction} returns this
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsToggleVisualDebugAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsToggleBetaVersionsAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsToggleBetaVersionsAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction}
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsToggleBetaVersionsAction;
  return proto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsToggleBetaVersionsAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction}
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsToggleBetaVersionsAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsToggleBetaVersionsAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction} returns this
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsToggleBetaVersionsAction} returns this
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsToggleBetaVersionsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsSetServicesAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsSetServicesAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsSetServicesAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    servicesMap: (f = msg.getServicesMap()) ? f.toObject(includeInstance, proto.ubo.v1.ServiceState.toObject) : [],
    gapDuration: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsSetServicesAction}
 */
proto.ubo.v1.SettingsSetServicesAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsSetServicesAction;
  return proto.ubo.v1.SettingsSetServicesAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsSetServicesAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsSetServicesAction}
 */
proto.ubo.v1.SettingsSetServicesAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = msg.getServicesMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.ServiceState.deserializeBinaryFromReader, "", new proto.ubo.v1.ServiceState());
         });
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setGapDuration(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsSetServicesAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsSetServicesAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsSetServicesAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServicesMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(2, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.ServiceState.serializeBinaryToWriter);
  }
  f = message.getGapDuration();
  if (f !== 0.0) {
    writer.writeFloat(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * map<string, ServiceState> services = 2;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!proto.ubo.v1.ServiceState>}
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.getServicesMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!proto.ubo.v1.ServiceState>} */ (
      jspb.Message.getMapField(this, 2, opt_noLazyCreate,
      proto.ubo.v1.ServiceState));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.clearServicesMap = function() {
  this.getServicesMap().clear();
  return this;};


/**
 * optional float gap_duration = 3;
 * @return {number}
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.getGapDuration = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SettingsSetServicesAction} returns this
 */
proto.ubo.v1.SettingsSetServicesAction.prototype.setGapDuration = function(value) {
  return jspb.Message.setProto3FloatField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsServiceAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsServiceAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsServiceAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serviceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsServiceAction}
 */
proto.ubo.v1.SettingsServiceAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsServiceAction;
  return proto.ubo.v1.SettingsServiceAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsServiceAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsServiceAction}
 */
proto.ubo.v1.SettingsServiceAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsServiceAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsServiceAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsServiceAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceAction} returns this
 */
proto.ubo.v1.SettingsServiceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsServiceAction} returns this
 */
proto.ubo.v1.SettingsServiceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string service_id = 2;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceAction} returns this
 */
proto.ubo.v1.SettingsServiceAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsStartServiceAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsStartServiceAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStartServiceAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serviceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsStartServiceAction}
 */
proto.ubo.v1.SettingsStartServiceAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsStartServiceAction;
  return proto.ubo.v1.SettingsStartServiceAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsStartServiceAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsStartServiceAction}
 */
proto.ubo.v1.SettingsStartServiceAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsStartServiceAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsStartServiceAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStartServiceAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStartServiceAction} returns this
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsStartServiceAction} returns this
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string service_id = 2;
 * @return {string}
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStartServiceAction} returns this
 */
proto.ubo.v1.SettingsStartServiceAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsStopServiceAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsStopServiceAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStopServiceAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serviceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsStopServiceAction}
 */
proto.ubo.v1.SettingsStopServiceAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsStopServiceAction;
  return proto.ubo.v1.SettingsStopServiceAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsStopServiceAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsStopServiceAction}
 */
proto.ubo.v1.SettingsStopServiceAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsStopServiceAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsStopServiceAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStopServiceAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStopServiceAction} returns this
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsStopServiceAction} returns this
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string service_id = 2;
 * @return {string}
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStopServiceAction} returns this
 */
proto.ubo.v1.SettingsStopServiceAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsServiceSetStatusAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsServiceSetStatusAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetStatusAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    serviceId: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsServiceSetStatusAction}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsServiceSetStatusAction;
  return proto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsServiceSetStatusAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsServiceSetStatusAction}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsServiceSetStatusAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsServiceSetStatusAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetStatusAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsActive();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.setIsActive = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional string service_id = 3;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetStatusAction} returns this
 */
proto.ubo.v1.SettingsServiceSetStatusAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsServiceSetIsEnabledAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    serviceId: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsServiceSetIsEnabledAction;
  return proto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsEnabled(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsServiceSetIsEnabledAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsEnabled();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_enabled = 2;
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.getIsEnabled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.setIsEnabled = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional string service_id = 3;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetIsEnabledAction} returns this
 */
proto.ubo.v1.SettingsServiceSetIsEnabledAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsServiceSetLogLevelAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsServiceSetLogLevelAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    logLevel: jspb.Message.getFieldWithDefault(msg, 2, 0),
    serviceId: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsServiceSetLogLevelAction;
  return proto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsServiceSetLogLevelAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setLogLevel(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsServiceSetLogLevelAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsServiceSetLogLevelAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getLogLevel();
  if (f !== 0) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 log_level = 2;
 * @return {number}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.getLogLevel = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.setLogLevel = function(value) {
  return jspb.Message.setProto3IntField(this, 2, value);
};


/**
 * optional string service_id = 3;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetLogLevelAction} returns this
 */
proto.ubo.v1.SettingsServiceSetLogLevelAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsServiceSetShouldRestartAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    shouldAutoRestart: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    serviceId: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsServiceSetShouldRestartAction;
  return proto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShouldAutoRestart(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsServiceSetShouldRestartAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getShouldAutoRestart();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool should_auto_restart = 2;
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.getShouldAutoRestart = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.setShouldAutoRestart = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional string service_id = 3;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceSetShouldRestartAction} returns this
 */
proto.ubo.v1.SettingsServiceSetShouldRestartAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsReportServiceErrorAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsReportServiceErrorAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsReportServiceErrorAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    error: (f = msg.getError()) && proto.ubo.v1.ErrorReport.toObject(includeInstance, f),
    serviceId: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsReportServiceErrorAction}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsReportServiceErrorAction;
  return proto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsReportServiceErrorAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsReportServiceErrorAction}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.ErrorReport;
      reader.readMessage(value,proto.ubo.v1.ErrorReport.deserializeBinaryFromReader);
      msg.setError(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsReportServiceErrorAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsReportServiceErrorAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsReportServiceErrorAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getError();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.ErrorReport.serializeBinaryToWriter
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional ErrorReport error = 2;
 * @return {?proto.ubo.v1.ErrorReport}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.getError = function() {
  return /** @type{?proto.ubo.v1.ErrorReport} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ErrorReport, 2));
};


/**
 * @param {?proto.ubo.v1.ErrorReport|undefined} value
 * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this
*/
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.setError = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.clearError = function() {
  return this.setError(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.hasError = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string service_id = 3;
 * @return {string}
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsReportServiceErrorAction} returns this
 */
proto.ubo.v1.SettingsReportServiceErrorAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsClearServiceErrorsAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsClearServiceErrorsAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serviceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction}
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsClearServiceErrorsAction;
  return proto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsClearServiceErrorsAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction}
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsClearServiceErrorsAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsClearServiceErrorsAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction} returns this
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction} returns this
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string service_id = 2;
 * @return {string}
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsClearServiceErrorsAction} returns this
 */
proto.ubo.v1.SettingsClearServiceErrorsAction.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsEvent}
 */
proto.ubo.v1.SettingsEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsEvent;
  return proto.ubo.v1.SettingsEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsEvent}
 */
proto.ubo.v1.SettingsEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsEvent} returns this
 */
proto.ubo.v1.SettingsEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsEvent} returns this
 */
proto.ubo.v1.SettingsEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsServiceEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsServiceEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsServiceEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serviceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsServiceEvent}
 */
proto.ubo.v1.SettingsServiceEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsServiceEvent;
  return proto.ubo.v1.SettingsServiceEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsServiceEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsServiceEvent}
 */
proto.ubo.v1.SettingsServiceEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsServiceEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsServiceEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsServiceEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsServiceEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceEvent} returns this
 */
proto.ubo.v1.SettingsServiceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsServiceEvent} returns this
 */
proto.ubo.v1.SettingsServiceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsServiceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string service_id = 2;
 * @return {string}
 */
proto.ubo.v1.SettingsServiceEvent.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsServiceEvent} returns this
 */
proto.ubo.v1.SettingsServiceEvent.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsStartServiceEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsStartServiceEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStartServiceEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    delay: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    serviceId: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsStartServiceEvent}
 */
proto.ubo.v1.SettingsStartServiceEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsStartServiceEvent;
  return proto.ubo.v1.SettingsStartServiceEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsStartServiceEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsStartServiceEvent}
 */
proto.ubo.v1.SettingsStartServiceEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDelay(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsStartServiceEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsStartServiceEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStartServiceEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float delay = 2;
 * @return {number}
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.getDelay = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.setDelay = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.clearDelay = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.hasDelay = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string service_id = 3;
 * @return {string}
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStartServiceEvent} returns this
 */
proto.ubo.v1.SettingsStartServiceEvent.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsStopServiceEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsStopServiceEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStopServiceEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serviceId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsStopServiceEvent}
 */
proto.ubo.v1.SettingsStopServiceEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsStopServiceEvent;
  return proto.ubo.v1.SettingsStopServiceEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsStopServiceEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsStopServiceEvent}
 */
proto.ubo.v1.SettingsStopServiceEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsStopServiceEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsStopServiceEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsStopServiceEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStopServiceEvent} returns this
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsStopServiceEvent} returns this
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string service_id = 2;
 * @return {string}
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsStopServiceEvent} returns this
 */
proto.ubo.v1.SettingsStopServiceEvent.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ErrorReport.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ErrorReport.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ErrorReport} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ErrorReport.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    message: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ErrorReport}
 */
proto.ubo.v1.ErrorReport.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ErrorReport;
  return proto.ubo.v1.ErrorReport.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ErrorReport} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ErrorReport}
 */
proto.ubo.v1.ErrorReport.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setMessage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ErrorReport.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ErrorReport.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ErrorReport} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ErrorReport.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getMessage();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ErrorReport.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ErrorReport} returns this
 */
proto.ubo.v1.ErrorReport.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ErrorReport} returns this
 */
proto.ubo.v1.ErrorReport.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ErrorReport.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float timestamp = 2;
 * @return {number}
 */
proto.ubo.v1.ErrorReport.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.ErrorReport} returns this
 */
proto.ubo.v1.ErrorReport.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional string message = 3;
 * @return {string}
 */
proto.ubo.v1.ErrorReport.prototype.getMessage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ErrorReport} returns this
 */
proto.ubo.v1.ErrorReport.prototype.setMessage = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ServiceState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ServiceState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ServiceState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ServiceState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, ""),
    label: jspb.Message.getFieldWithDefault(msg, 3, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    logLevel: jspb.Message.getFieldWithDefault(msg, 6, 0),
    shouldAutoRestart: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),
    errors: (f = msg.getErrors()) && proto.ubo.v1.ServiceState.Errors.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ServiceState}
 */
proto.ubo.v1.ServiceState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ServiceState;
  return proto.ubo.v1.ServiceState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ServiceState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ServiceState}
 */
proto.ubo.v1.ServiceState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsEnabled(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setLogLevel(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShouldAutoRestart(value);
      break;
    case 8:
      var value = new proto.ubo.v1.ServiceState.Errors;
      reader.readMessage(value,proto.ubo.v1.ServiceState.Errors.deserializeBinaryFromReader);
      msg.setErrors(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ServiceState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ServiceState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ServiceState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ServiceState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getLabel();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getIsActive();
  if (f) {
    writer.writeBool(
      4,
      f
    );
  }
  f = message.getIsEnabled();
  if (f) {
    writer.writeBool(
      5,
      f
    );
  }
  f = message.getLogLevel();
  if (f !== 0) {
    writer.writeInt64(
      6,
      f
    );
  }
  f = message.getShouldAutoRestart();
  if (f) {
    writer.writeBool(
      7,
      f
    );
  }
  f = message.getErrors();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.ubo.v1.ServiceState.Errors.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.ServiceState.Errors.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ServiceState.Errors.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ServiceState.Errors.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ServiceState.Errors} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ServiceState.Errors.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.ErrorReport.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ServiceState.Errors}
 */
proto.ubo.v1.ServiceState.Errors.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ServiceState.Errors;
  return proto.ubo.v1.ServiceState.Errors.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ServiceState.Errors} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ServiceState.Errors}
 */
proto.ubo.v1.ServiceState.Errors.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.ErrorReport;
      reader.readMessage(value,proto.ubo.v1.ErrorReport.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ServiceState.Errors.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ServiceState.Errors.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ServiceState.Errors} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ServiceState.Errors.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.ErrorReport.serializeBinaryToWriter
    );
  }
};


/**
 * repeated ErrorReport items = 1;
 * @return {!Array<!proto.ubo.v1.ErrorReport>}
 */
proto.ubo.v1.ServiceState.Errors.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.ErrorReport>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.ErrorReport, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.ErrorReport>} value
 * @return {!proto.ubo.v1.ServiceState.Errors} returns this
*/
proto.ubo.v1.ServiceState.Errors.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.ErrorReport=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.ErrorReport}
 */
proto.ubo.v1.ServiceState.Errors.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.ErrorReport, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.ServiceState.Errors} returns this
 */
proto.ubo.v1.ServiceState.Errors.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.ServiceState.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ServiceState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.ServiceState.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string label = 3;
 * @return {string}
 */
proto.ubo.v1.ServiceState.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.setLabel = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional bool is_active = 4;
 * @return {boolean}
 */
proto.ubo.v1.ServiceState.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.setIsActive = function(value) {
  return jspb.Message.setProto3BooleanField(this, 4, value);
};


/**
 * optional bool is_enabled = 5;
 * @return {boolean}
 */
proto.ubo.v1.ServiceState.prototype.getIsEnabled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.setIsEnabled = function(value) {
  return jspb.Message.setProto3BooleanField(this, 5, value);
};


/**
 * optional int64 log_level = 6;
 * @return {number}
 */
proto.ubo.v1.ServiceState.prototype.getLogLevel = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.setLogLevel = function(value) {
  return jspb.Message.setProto3IntField(this, 6, value);
};


/**
 * optional bool should_auto_restart = 7;
 * @return {boolean}
 */
proto.ubo.v1.ServiceState.prototype.getShouldAutoRestart = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.setShouldAutoRestart = function(value) {
  return jspb.Message.setProto3BooleanField(this, 7, value);
};


/**
 * optional Errors errors = 8;
 * @return {?proto.ubo.v1.ServiceState.Errors}
 */
proto.ubo.v1.ServiceState.prototype.getErrors = function() {
  return /** @type{?proto.ubo.v1.ServiceState.Errors} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ServiceState.Errors, 8));
};


/**
 * @param {?proto.ubo.v1.ServiceState.Errors|undefined} value
 * @return {!proto.ubo.v1.ServiceState} returns this
*/
proto.ubo.v1.ServiceState.prototype.setErrors = function(value) {
  return jspb.Message.setWrapperField(this, 8, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.ServiceState} returns this
 */
proto.ubo.v1.ServiceState.prototype.clearErrors = function() {
  return this.setErrors(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ServiceState.prototype.hasErrors = function() {
  return jspb.Message.getField(this, 8) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotSettingsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    pdbSignal: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    visualDebug: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    betaVersions: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    services: (f = msg.getServices()) && proto.ubo.v1.SettingsState.ServicesDict.toObject(includeInstance, f),
    servicesStatus: jspb.Message.getFieldWithDefault(msg, 6, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsState}
 */
proto.ubo.v1.SettingsState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsState;
  return proto.ubo.v1.SettingsState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsState}
 */
proto.ubo.v1.SettingsState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setPdbSignal(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setVisualDebug(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBetaVersions(value);
      break;
    case 5:
      var value = new proto.ubo.v1.SettingsState.ServicesDict;
      reader.readMessage(value,proto.ubo.v1.SettingsState.ServicesDict.deserializeBinaryFromReader);
      msg.setServices(value);
      break;
    case 6:
      var value = /** @type {!proto.ubo.v1.ServicesStatus} */ (reader.readEnum());
      msg.setServicesStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = message.getServices();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.ubo.v1.SettingsState.ServicesDict.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.ubo.v1.ServicesStatus} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SettingsState.ServicesDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SettingsState.ServicesDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SettingsState.ServicesDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsState.ServicesDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.ServiceState.toObject) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SettingsState.ServicesDict}
 */
proto.ubo.v1.SettingsState.ServicesDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SettingsState.ServicesDict;
  return proto.ubo.v1.SettingsState.ServicesDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SettingsState.ServicesDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SettingsState.ServicesDict}
 */
proto.ubo.v1.SettingsState.ServicesDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.ServiceState.deserializeBinaryFromReader, "", new proto.ubo.v1.ServiceState());
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SettingsState.ServicesDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SettingsState.ServicesDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SettingsState.ServicesDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SettingsState.ServicesDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.ServiceState.serializeBinaryToWriter);
  }
};


/**
 * map<string, ServiceState> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!proto.ubo.v1.ServiceState>}
 */
proto.ubo.v1.SettingsState.ServicesDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!proto.ubo.v1.ServiceState>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      proto.ubo.v1.ServiceState));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.SettingsState.ServicesDict} returns this
 */
proto.ubo.v1.SettingsState.ServicesDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_settings_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.SettingsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotSettingsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool pdb_signal = 2;
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.getPdbSignal = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.setPdbSignal = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.clearPdbSignal = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.hasPdbSignal = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool visual_debug = 3;
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.getVisualDebug = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.setVisualDebug = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.clearVisualDebug = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.hasVisualDebug = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool beta_versions = 4;
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.getBetaVersions = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.setBetaVersions = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.clearBetaVersions = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.hasBetaVersions = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional ServicesDict services = 5;
 * @return {?proto.ubo.v1.SettingsState.ServicesDict}
 */
proto.ubo.v1.SettingsState.prototype.getServices = function() {
  return /** @type{?proto.ubo.v1.SettingsState.ServicesDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsState.ServicesDict, 5));
};


/**
 * @param {?proto.ubo.v1.SettingsState.ServicesDict|undefined} value
 * @return {!proto.ubo.v1.SettingsState} returns this
*/
proto.ubo.v1.SettingsState.prototype.setServices = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.clearServices = function() {
  return this.setServices(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.hasServices = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional ServicesStatus services_status = 6;
 * @return {!proto.ubo.v1.ServicesStatus}
 */
proto.ubo.v1.SettingsState.prototype.getServicesStatus = function() {
  return /** @type {!proto.ubo.v1.ServicesStatus} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {!proto.ubo.v1.ServicesStatus} value
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.setServicesStatus = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SettingsState} returns this
 */
proto.ubo.v1.SettingsState.prototype.clearServicesStatus = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SettingsState.prototype.hasServicesStatus = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.IconState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.IconState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.IconState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IconState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    symbol: jspb.Message.getFieldWithDefault(msg, 2, ""),
    color: jspb.Message.getFieldWithDefault(msg, 3, ""),
    priority: jspb.Message.getFieldWithDefault(msg, 4, 0),
    serviceId: jspb.Message.getFieldWithDefault(msg, 5, ""),
    id: jspb.Message.getFieldWithDefault(msg, 6, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.IconState}
 */
proto.ubo.v1.IconState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.IconState;
  return proto.ubo.v1.IconState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.IconState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.IconState}
 */
proto.ubo.v1.IconState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setSymbol(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setPriority(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setServiceId(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.IconState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.IconState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.IconState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IconState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSymbol();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getColor();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getPriority();
  if (f !== 0) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = message.getServiceId();
  if (f.length > 0) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.IconState.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IconState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string symbol = 2;
 * @return {string}
 */
proto.ubo.v1.IconState.prototype.getSymbol = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.setSymbol = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string color = 3;
 * @return {string}
 */
proto.ubo.v1.IconState.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.setColor = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional int64 priority = 4;
 * @return {number}
 */
proto.ubo.v1.IconState.prototype.getPriority = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.setPriority = function(value) {
  return jspb.Message.setProto3IntField(this, 4, value);
};


/**
 * optional string service_id = 5;
 * @return {string}
 */
proto.ubo.v1.IconState.prototype.getServiceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.setServiceId = function(value) {
  return jspb.Message.setProto3StringField(this, 5, value);
};


/**
 * optional string id = 6;
 * @return {string}
 */
proto.ubo.v1.IconState.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.setId = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IconState} returns this
 */
proto.ubo.v1.IconState.prototype.clearId = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IconState.prototype.hasId = function() {
  return jspb.Message.getField(this, 6) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.StatusIconsState.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.StatusIconsState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.StatusIconsState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.StatusIconsState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StatusIconsState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    iconsList: jspb.Message.toObjectList(msg.getIconsList(),
    proto.ubo.v1.IconState.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.StatusIconsState}
 */
proto.ubo.v1.StatusIconsState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.StatusIconsState;
  return proto.ubo.v1.StatusIconsState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.StatusIconsState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.StatusIconsState}
 */
proto.ubo.v1.StatusIconsState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);
      break;
    case 2:
      var value = new proto.ubo.v1.IconState;
      reader.readMessage(value,proto.ubo.v1.IconState.deserializeBinaryFromReader);
      msg.addIcons(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.StatusIconsState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.StatusIconsState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.StatusIconsState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StatusIconsState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIconsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.IconState.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.StatusIconsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StatusIconsState} returns this
 */
proto.ubo.v1.StatusIconsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StatusIconsState} returns this
 */
proto.ubo.v1.StatusIconsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StatusIconsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated IconState icons = 2;
 * @return {!Array<!proto.ubo.v1.IconState>}
 */
proto.ubo.v1.StatusIconsState.prototype.getIconsList = function() {
  return /** @type{!Array<!proto.ubo.v1.IconState>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.IconState, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.IconState>} value
 * @return {!proto.ubo.v1.StatusIconsState} returns this
*/
proto.ubo.v1.StatusIconsState.prototype.setIconsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.IconState=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.IconState}
 */
proto.ubo.v1.StatusIconsState.prototype.addIcons = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.IconState, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.StatusIconsState} returns this
 */
proto.ubo.v1.StatusIconsState.prototype.clearIconsList = function() {
  return this.setIconsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.StatusIconsAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.StatusIconsAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.StatusIconsAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StatusIconsAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.StatusIconsAction}
 */
proto.ubo.v1.StatusIconsAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.StatusIconsAction;
  return proto.ubo.v1.StatusIconsAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.StatusIconsAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.StatusIconsAction}
 */
proto.ubo.v1.StatusIconsAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.StatusIconsAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.StatusIconsAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.StatusIconsAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StatusIconsAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.StatusIconsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StatusIconsAction} returns this
 */
proto.ubo.v1.StatusIconsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StatusIconsAction} returns this
 */
proto.ubo.v1.StatusIconsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StatusIconsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.StatusIconsRegisterAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.StatusIconsRegisterAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StatusIconsRegisterAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 2, ""),
    color: jspb.Message.getFieldWithDefault(msg, 3, ""),
    priority: jspb.Message.getFieldWithDefault(msg, 4, 0),
    id: jspb.Message.getFieldWithDefault(msg, 5, ""),
    service: jspb.Message.getFieldWithDefault(msg, 6, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.StatusIconsRegisterAction}
 */
proto.ubo.v1.StatusIconsRegisterAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.StatusIconsRegisterAction;
  return proto.ubo.v1.StatusIconsRegisterAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.StatusIconsRegisterAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.StatusIconsRegisterAction}
 */
proto.ubo.v1.StatusIconsRegisterAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setPriority(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setService(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.StatusIconsRegisterAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.StatusIconsRegisterAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.StatusIconsRegisterAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIcon();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_status_icons_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotStatusIconsDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string icon = 2;
 * @return {string}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.setIcon = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string color = 3;
 * @return {string}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.clearColor = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 priority = 4;
 * @return {number}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.getPriority = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.setPriority = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.clearPriority = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.hasPriority = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string id = 5;
 * @return {string}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.setId = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.clearId = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.hasId = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string service = 6;
 * @return {string}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.getService = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.setService = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.StatusIconsRegisterAction} returns this
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.clearService = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.StatusIconsRegisterAction.prototype.hasService = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerAction}
 */
proto.ubo.v1.UpdateManagerAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerAction;
  return proto.ubo.v1.UpdateManagerAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerAction}
 */
proto.ubo.v1.UpdateManagerAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerAction} returns this
 */
proto.ubo.v1.UpdateManagerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerAction} returns this
 */
proto.ubo.v1.UpdateManagerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerSetVersionsAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    flashNotification: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    currentVersion: jspb.Message.getFieldWithDefault(msg, 3, ""),
    baseImageVariant: jspb.Message.getFieldWithDefault(msg, 4, ""),
    latestVersion: jspb.Message.getFieldWithDefault(msg, 5, ""),
    recentVersions: (f = msg.getRecentVersions()) && proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerSetVersionsAction;
  return proto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setFlashNotification(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setCurrentVersion(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setBaseImageVariant(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setLatestVersion(value);
      break;
    case 6:
      var value = new proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinaryFromReader);
      msg.setRecentVersions(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerSetVersionsAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getFlashNotification();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getCurrentVersion();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getBaseImageVariant();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getLatestVersion();
  if (f.length > 0) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getRecentVersions();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions;
  return proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool flash_notification = 2;
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getFlashNotification = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setFlashNotification = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional string current_version = 3;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getCurrentVersion = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setCurrentVersion = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional string base_image_variant = 4;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getBaseImageVariant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setBaseImageVariant = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};


/**
 * optional string latest_version = 5;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getLatestVersion = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setLatestVersion = function(value) {
  return jspb.Message.setProto3StringField(this, 5, value);
};


/**
 * optional RecentVersions recent_versions = 6;
 * @return {?proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.getRecentVersions = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions, 6));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerSetVersionsAction.RecentVersions|undefined} value
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
*/
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.setRecentVersions = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerSetVersionsAction} returns this
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.clearRecentVersions = function() {
  return this.setRecentVersions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerSetVersionsAction.prototype.hasRecentVersions = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerRequestCheckAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerRequestCheckAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction}
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerRequestCheckAction;
  return proto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerRequestCheckAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction}
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerRequestCheckAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerRequestCheckAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction} returns this
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerRequestCheckAction} returns this
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerRequestCheckAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerReportFailedCheckAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction}
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerReportFailedCheckAction;
  return proto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction}
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerReportFailedCheckAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} returns this
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerReportFailedCheckAction} returns this
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerReportFailedCheckAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerRequestUpdateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerRequestUpdateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    version: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction}
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerRequestUpdateAction;
  return proto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerRequestUpdateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction}
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setVersion(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerRequestUpdateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerRequestUpdateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getVersion();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction} returns this
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction} returns this
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string version = 2;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.getVersion = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerRequestUpdateAction} returns this
 */
proto.ubo.v1.UpdateManagerRequestUpdateAction.prototype.setVersion = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerEvent}
 */
proto.ubo.v1.UpdateManagerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerEvent;
  return proto.ubo.v1.UpdateManagerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerEvent}
 */
proto.ubo.v1.UpdateManagerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerEvent} returns this
 */
proto.ubo.v1.UpdateManagerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerEvent} returns this
 */
proto.ubo.v1.UpdateManagerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerCheckEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerCheckEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerCheckEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerCheckEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerCheckEvent}
 */
proto.ubo.v1.UpdateManagerCheckEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerCheckEvent;
  return proto.ubo.v1.UpdateManagerCheckEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerCheckEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerCheckEvent}
 */
proto.ubo.v1.UpdateManagerCheckEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerCheckEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerCheckEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerCheckEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerCheckEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerCheckEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerCheckEvent} returns this
 */
proto.ubo.v1.UpdateManagerCheckEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerCheckEvent} returns this
 */
proto.ubo.v1.UpdateManagerCheckEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerCheckEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerUpdateEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerUpdateEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerUpdateEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    version: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerUpdateEvent}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerUpdateEvent;
  return proto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerUpdateEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerUpdateEvent}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setVersion(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerUpdateEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerUpdateEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerUpdateEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string version = 2;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.getVersion = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.setVersion = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerUpdateEvent} returns this
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.clearVersion = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerUpdateEvent.prototype.hasVersion = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    currentVersion: jspb.Message.getFieldWithDefault(msg, 2, ""),
    baseImageVariant: jspb.Message.getFieldWithDefault(msg, 3, ""),
    latestVersion: jspb.Message.getFieldWithDefault(msg, 4, ""),
    updateStatus: jspb.Message.getFieldWithDefault(msg, 5, 0),
    recentVersions: (f = msg.getRecentVersions()) && proto.ubo.v1.UpdateManagerState.RecentVersions.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerState}
 */
proto.ubo.v1.UpdateManagerState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerState;
  return proto.ubo.v1.UpdateManagerState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerState}
 */
proto.ubo.v1.UpdateManagerState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setCurrentVersion(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setBaseImageVariant(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setLatestVersion(value);
      break;
    case 5:
      var value = /** @type {!proto.ubo.v1.UpdateStatus} */ (reader.readEnum());
      msg.setUpdateStatus(value);
      break;
    case 6:
      var value = new proto.ubo.v1.UpdateManagerState.RecentVersions;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinaryFromReader);
      msg.setRecentVersions(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.UpdateStatus} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = message.getRecentVersions();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.ubo.v1.UpdateManagerState.RecentVersions.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UpdateManagerState.RecentVersions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UpdateManagerState.RecentVersions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions}
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UpdateManagerState.RecentVersions;
  return proto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UpdateManagerState.RecentVersions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions}
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UpdateManagerState.RecentVersions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UpdateManagerState.RecentVersions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions} returns this
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions} returns this
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.UpdateManagerState.RecentVersions} returns this
 */
proto.ubo.v1.UpdateManagerState.RecentVersions.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_update_manager_dot_types = 1000;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerState.prototype.getMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.setMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotUpdateManagerDotTypes = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string current_version = 2;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerState.prototype.getCurrentVersion = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.setCurrentVersion = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.clearCurrentVersion = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerState.prototype.hasCurrentVersion = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string base_image_variant = 3;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerState.prototype.getBaseImageVariant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.setBaseImageVariant = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.clearBaseImageVariant = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerState.prototype.hasBaseImageVariant = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string latest_version = 4;
 * @return {string}
 */
proto.ubo.v1.UpdateManagerState.prototype.getLatestVersion = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.setLatestVersion = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.clearLatestVersion = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerState.prototype.hasLatestVersion = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional UpdateStatus update_status = 5;
 * @return {!proto.ubo.v1.UpdateStatus}
 */
proto.ubo.v1.UpdateManagerState.prototype.getUpdateStatus = function() {
  return /** @type {!proto.ubo.v1.UpdateStatus} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {!proto.ubo.v1.UpdateStatus} value
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.setUpdateStatus = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.clearUpdateStatus = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerState.prototype.hasUpdateStatus = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional RecentVersions recent_versions = 6;
 * @return {?proto.ubo.v1.UpdateManagerState.RecentVersions}
 */
proto.ubo.v1.UpdateManagerState.prototype.getRecentVersions = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerState.RecentVersions} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerState.RecentVersions, 6));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerState.RecentVersions|undefined} value
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
*/
proto.ubo.v1.UpdateManagerState.prototype.setRecentVersions = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UpdateManagerState} returns this
 */
proto.ubo.v1.UpdateManagerState.prototype.clearRecentVersions = function() {
  return this.setRecentVersions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UpdateManagerState.prototype.hasRecentVersions = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.McpServerMetadata.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.McpServerMetadata.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.McpServerMetadata} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.McpServerMetadata.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serverId: jspb.Message.getFieldWithDefault(msg, 2, ""),
    name: jspb.Message.getFieldWithDefault(msg, 3, ""),
    type: jspb.Message.getFieldWithDefault(msg, 4, 0),
    config: jspb.Message.getFieldWithDefault(msg, 5, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.McpServerMetadata}
 */
proto.ubo.v1.McpServerMetadata.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.McpServerMetadata;
  return proto.ubo.v1.McpServerMetadata.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.McpServerMetadata} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.McpServerMetadata}
 */
proto.ubo.v1.McpServerMetadata.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServerId(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 4:
      var value = /** @type {!proto.ubo.v1.McpServerType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setConfig(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.McpServerMetadata.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.McpServerMetadata.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.McpServerMetadata} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.McpServerMetadata.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServerId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getName();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getType();
  if (f !== 0.0) {
    writer.writeEnum(
      4,
      f
    );
  }
  f = message.getConfig();
  if (f.length > 0) {
    writer.writeString(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.McpServerMetadata.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.McpServerMetadata} returns this
 */
proto.ubo.v1.McpServerMetadata.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.McpServerMetadata} returns this
 */
proto.ubo.v1.McpServerMetadata.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.McpServerMetadata.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string server_id = 2;
 * @return {string}
 */
proto.ubo.v1.McpServerMetadata.prototype.getServerId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.McpServerMetadata} returns this
 */
proto.ubo.v1.McpServerMetadata.prototype.setServerId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string name = 3;
 * @return {string}
 */
proto.ubo.v1.McpServerMetadata.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.McpServerMetadata} returns this
 */
proto.ubo.v1.McpServerMetadata.prototype.setName = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional McpServerType type = 4;
 * @return {!proto.ubo.v1.McpServerType}
 */
proto.ubo.v1.McpServerMetadata.prototype.getType = function() {
  return /** @type {!proto.ubo.v1.McpServerType} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {!proto.ubo.v1.McpServerType} value
 * @return {!proto.ubo.v1.McpServerMetadata} returns this
 */
proto.ubo.v1.McpServerMetadata.prototype.setType = function(value) {
  return jspb.Message.setProto3EnumField(this, 4, value);
};


/**
 * optional string config = 5;
 * @return {string}
 */
proto.ubo.v1.McpServerMetadata.prototype.getConfig = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.McpServerMetadata} returns this
 */
proto.ubo.v1.McpServerMetadata.prototype.setConfig = function(value) {
  return jspb.Message.setProto3StringField(this, 5, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantAction}
 */
proto.ubo.v1.AssistantAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantAction;
  return proto.ubo.v1.AssistantAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantAction}
 */
proto.ubo.v1.AssistantAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantAction} returns this
 */
proto.ubo.v1.AssistantAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantAction} returns this
 */
proto.ubo.v1.AssistantAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantSetIsActiveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantSetIsActiveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetIsActiveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantSetIsActiveAction}
 */
proto.ubo.v1.AssistantSetIsActiveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantSetIsActiveAction;
  return proto.ubo.v1.AssistantSetIsActiveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantSetIsActiveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantSetIsActiveAction}
 */
proto.ubo.v1.AssistantSetIsActiveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantSetIsActiveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantSetIsActiveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetIsActiveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsActive();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSetIsActiveAction} returns this
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantSetIsActiveAction} returns this
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistantSetIsActiveAction} returns this
 */
proto.ubo.v1.AssistantSetIsActiveAction.prototype.setIsActive = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantSetSelectedSTTAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantSetSelectedSTTAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sttName: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction}
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantSetSelectedSTTAction;
  return proto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantSetSelectedSTTAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction}
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.AssistantSTTName} */ (reader.readEnum());
      msg.setSttName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantSetSelectedSTTAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantSetSelectedSTTAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSttName();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AssistantSTTName stt_name = 2;
 * @return {!proto.ubo.v1.AssistantSTTName}
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.getSttName = function() {
  return /** @type {!proto.ubo.v1.AssistantSTTName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantSTTName} value
 * @return {!proto.ubo.v1.AssistantSetSelectedSTTAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedSTTAction.prototype.setSttName = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantSetSelectedLLMAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantSetSelectedLLMAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    llmName: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction}
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantSetSelectedLLMAction;
  return proto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantSetSelectedLLMAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction}
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.AssistantLLMName} */ (reader.readEnum());
      msg.setLlmName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantSetSelectedLLMAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantSetSelectedLLMAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getLlmName();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AssistantLLMName llm_name = 2;
 * @return {!proto.ubo.v1.AssistantLLMName}
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.getLlmName = function() {
  return /** @type {!proto.ubo.v1.AssistantLLMName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantLLMName} value
 * @return {!proto.ubo.v1.AssistantSetSelectedLLMAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedLLMAction.prototype.setLlmName = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantSetSelectedTTSAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantSetSelectedTTSAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    ttsName: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction}
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantSetSelectedTTSAction;
  return proto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantSetSelectedTTSAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction}
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.AssistantTTSName} */ (reader.readEnum());
      msg.setTtsName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantSetSelectedTTSAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantSetSelectedTTSAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTtsName();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AssistantTTSName tts_name = 2;
 * @return {!proto.ubo.v1.AssistantTTSName}
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.getTtsName = function() {
  return /** @type {!proto.ubo.v1.AssistantTTSName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantTTSName} value
 * @return {!proto.ubo.v1.AssistantSetSelectedTTSAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedTTSAction.prototype.setTtsName = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    imageGeneratorName: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction}
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantSetSelectedImageGeneratorAction;
  return proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction}
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (reader.readEnum());
      msg.setImageGeneratorName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImageGeneratorName();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AssistantImageGeneratorName image_generator_name = 2;
 * @return {!proto.ubo.v1.AssistantImageGeneratorName}
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.getImageGeneratorName = function() {
  return /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantImageGeneratorName} value
 * @return {!proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.prototype.setImageGeneratorName = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantSetSelectedModelAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantSetSelectedModelAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedModelAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    model: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantSetSelectedModelAction}
 */
proto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantSetSelectedModelAction;
  return proto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantSetSelectedModelAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantSetSelectedModelAction}
 */
proto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setModel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantSetSelectedModelAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantSetSelectedModelAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSetSelectedModelAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getModel();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSetSelectedModelAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantSetSelectedModelAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string model = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.getModel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSetSelectedModelAction} returns this
 */
proto.ubo.v1.AssistantSetSelectedModelAction.prototype.setModel = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantDownloadOllamaModelAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantDownloadOllamaModelAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    model: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction}
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantDownloadOllamaModelAction;
  return proto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantDownloadOllamaModelAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction}
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setModel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantDownloadOllamaModelAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantDownloadOllamaModelAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getModel();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction} returns this
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction} returns this
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string model = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.getModel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelAction} returns this
 */
proto.ubo.v1.AssistantDownloadOllamaModelAction.prototype.setModel = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistanceFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistanceFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistanceFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),
    id: jspb.Message.getFieldWithDefault(msg, 4, ""),
    index: jspb.Message.getFieldWithDefault(msg, 5, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistanceFrame}
 */
proto.ubo.v1.AssistanceFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistanceFrame;
  return proto.ubo.v1.AssistanceFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistanceFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistanceFrame}
 */
proto.ubo.v1.AssistanceFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsLastFrame(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistanceFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistanceFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistanceFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsLastFrame();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistanceFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceFrame} returns this
 */
proto.ubo.v1.AssistanceFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistanceFrame} returns this
 */
proto.ubo.v1.AssistanceFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistanceFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_last_frame = 2;
 * @return {boolean}
 */
proto.ubo.v1.AssistanceFrame.prototype.getIsLastFrame = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistanceFrame} returns this
 */
proto.ubo.v1.AssistanceFrame.prototype.setIsLastFrame = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional float timestamp = 3;
 * @return {number}
 */
proto.ubo.v1.AssistanceFrame.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceFrame} returns this
 */
proto.ubo.v1.AssistanceFrame.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 3, value);
};


/**
 * optional string id = 4;
 * @return {string}
 */
proto.ubo.v1.AssistanceFrame.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceFrame} returns this
 */
proto.ubo.v1.AssistanceFrame.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};


/**
 * optional int64 index = 5;
 * @return {number}
 */
proto.ubo.v1.AssistanceFrame.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceFrame} returns this
 */
proto.ubo.v1.AssistanceFrame.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 5, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistanceTextFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistanceTextFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceTextFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    text: jspb.Message.getFieldWithDefault(msg, 2, ""),
    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),
    id: jspb.Message.getFieldWithDefault(msg, 5, ""),
    index: jspb.Message.getFieldWithDefault(msg, 6, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistanceTextFrame}
 */
proto.ubo.v1.AssistanceTextFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistanceTextFrame;
  return proto.ubo.v1.AssistanceTextFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistanceTextFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistanceTextFrame}
 */
proto.ubo.v1.AssistanceTextFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsLastFrame(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistanceTextFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistanceTextFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceTextFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getText();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getIsLastFrame();
  if (f) {
    writer.writeBool(
      3,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceTextFrame} returns this
 */
proto.ubo.v1.AssistanceTextFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistanceTextFrame} returns this
 */
proto.ubo.v1.AssistanceTextFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string text = 2;
 * @return {string}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.getText = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceTextFrame} returns this
 */
proto.ubo.v1.AssistanceTextFrame.prototype.setText = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional bool is_last_frame = 3;
 * @return {boolean}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.getIsLastFrame = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistanceTextFrame} returns this
 */
proto.ubo.v1.AssistanceTextFrame.prototype.setIsLastFrame = function(value) {
  return jspb.Message.setProto3BooleanField(this, 3, value);
};


/**
 * optional float timestamp = 4;
 * @return {number}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceTextFrame} returns this
 */
proto.ubo.v1.AssistanceTextFrame.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 4, value);
};


/**
 * optional string id = 5;
 * @return {string}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceTextFrame} returns this
 */
proto.ubo.v1.AssistanceTextFrame.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 5, value);
};


/**
 * optional int64 index = 6;
 * @return {number}
 */
proto.ubo.v1.AssistanceTextFrame.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceTextFrame} returns this
 */
proto.ubo.v1.AssistanceTextFrame.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 6, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistanceAudioFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistanceAudioFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceAudioFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    audio: (f = msg.getAudio()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f),
    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),
    id: jspb.Message.getFieldWithDefault(msg, 5, ""),
    index: jspb.Message.getFieldWithDefault(msg, 6, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistanceAudioFrame}
 */
proto.ubo.v1.AssistanceAudioFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistanceAudioFrame;
  return proto.ubo.v1.AssistanceAudioFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistanceAudioFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistanceAudioFrame}
 */
proto.ubo.v1.AssistanceAudioFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setAudio(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsLastFrame(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 6:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistanceAudioFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistanceAudioFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceAudioFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getAudio();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
  f = message.getIsLastFrame();
  if (f) {
    writer.writeBool(
      3,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AudioSample audio = 2;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.getAudio = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 2));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
*/
proto.ubo.v1.AssistanceAudioFrame.prototype.setAudio = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.clearAudio = function() {
  return this.setAudio(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.hasAudio = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_last_frame = 3;
 * @return {boolean}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.getIsLastFrame = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.setIsLastFrame = function(value) {
  return jspb.Message.setProto3BooleanField(this, 3, value);
};


/**
 * optional float timestamp = 4;
 * @return {number}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 4, value);
};


/**
 * optional string id = 5;
 * @return {string}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 5, value);
};


/**
 * optional int64 index = 6;
 * @return {number}
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceAudioFrame} returns this
 */
proto.ubo.v1.AssistanceAudioFrame.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 6, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistanceImageFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistanceImageFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceImageFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: msg.getImage_asB64(),
    width: jspb.Message.getFieldWithDefault(msg, 3, 0),
    height: jspb.Message.getFieldWithDefault(msg, 4, 0),
    format: jspb.Message.getFieldWithDefault(msg, 5, ""),
    metadataMap: (f = msg.getMetadataMap()) ? f.toObject(includeInstance, undefined) : [],
    isLastFrame: jspb.Message.getBooleanFieldWithDefault(msg, 7, false),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 8, 0.0),
    id: jspb.Message.getFieldWithDefault(msg, 9, ""),
    index: jspb.Message.getFieldWithDefault(msg, 10, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistanceImageFrame}
 */
proto.ubo.v1.AssistanceImageFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistanceImageFrame;
  return proto.ubo.v1.AssistanceImageFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistanceImageFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistanceImageFrame}
 */
proto.ubo.v1.AssistanceImageFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setImage(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWidth(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setHeight(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setFormat(value);
      break;
    case 6:
      var value = msg.getMetadataMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, "", "");
         });
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsLastFrame(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistanceImageFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistanceImageFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistanceImageFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      2,
      f
    );
  }
  f = message.getWidth();
  if (f !== 0) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getHeight();
  if (f !== 0) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = message.getFormat();
  if (f.length > 0) {
    writer.writeString(
      5,
      f
    );
  }
  f = message.getMetadataMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(6, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);
  }
  f = message.getIsLastFrame();
  if (f) {
    writer.writeBool(
      7,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      8,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      9,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      10,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bytes image = 2;
 * @return {string}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * optional bytes image = 2;
 * This is a type-conversion wrapper around `getImage()`
 * @return {string}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getImage_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getImage()));
};


/**
 * optional bytes image = 2;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getImage()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getImage_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getImage()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setImage = function(value) {
  return jspb.Message.setProto3BytesField(this, 2, value);
};


/**
 * optional int64 width = 3;
 * @return {number}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getWidth = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setWidth = function(value) {
  return jspb.Message.setProto3IntField(this, 3, value);
};


/**
 * optional int64 height = 4;
 * @return {number}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getHeight = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setHeight = function(value) {
  return jspb.Message.setProto3IntField(this, 4, value);
};


/**
 * optional string format = 5;
 * @return {string}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getFormat = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setFormat = function(value) {
  return jspb.Message.setProto3StringField(this, 5, value);
};


/**
 * map<string, string> metadata = 6;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,string>}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getMetadataMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,string>} */ (
      jspb.Message.getMapField(this, 6, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.clearMetadataMap = function() {
  this.getMetadataMap().clear();
  return this;};


/**
 * optional bool is_last_frame = 7;
 * @return {boolean}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getIsLastFrame = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setIsLastFrame = function(value) {
  return jspb.Message.setProto3BooleanField(this, 7, value);
};


/**
 * optional float timestamp = 8;
 * @return {number}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 8, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 8, value);
};


/**
 * optional string id = 9;
 * @return {string}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 9, value);
};


/**
 * optional int64 index = 10;
 * @return {number}
 */
proto.ubo.v1.AssistanceImageFrame.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 10, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AssistanceImageFrame} returns this
 */
proto.ubo.v1.AssistanceImageFrame.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 10, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantReportAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantReportAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantReportAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantReportAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sourceId: jspb.Message.getFieldWithDefault(msg, 2, ""),
    data: (f = msg.getData()) && proto.ubo.v1.AcceptableAssistanceFrame.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantReportAction}
 */
proto.ubo.v1.AssistantReportAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantReportAction;
  return proto.ubo.v1.AssistantReportAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantReportAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantReportAction}
 */
proto.ubo.v1.AssistantReportAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setSourceId(value);
      break;
    case 3:
      var value = new proto.ubo.v1.AcceptableAssistanceFrame;
      reader.readMessage(value,proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader);
      msg.setData(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantReportAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantReportAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantReportAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantReportAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSourceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getData();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantReportAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantReportAction} returns this
 */
proto.ubo.v1.AssistantReportAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantReportAction} returns this
 */
proto.ubo.v1.AssistantReportAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantReportAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string source_id = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantReportAction.prototype.getSourceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantReportAction} returns this
 */
proto.ubo.v1.AssistantReportAction.prototype.setSourceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional AcceptableAssistanceFrame data = 3;
 * @return {?proto.ubo.v1.AcceptableAssistanceFrame}
 */
proto.ubo.v1.AssistantReportAction.prototype.getData = function() {
  return /** @type{?proto.ubo.v1.AcceptableAssistanceFrame} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AcceptableAssistanceFrame, 3));
};


/**
 * @param {?proto.ubo.v1.AcceptableAssistanceFrame|undefined} value
 * @return {!proto.ubo.v1.AssistantReportAction} returns this
*/
proto.ubo.v1.AssistantReportAction.prototype.setData = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AssistantReportAction} returns this
 */
proto.ubo.v1.AssistantReportAction.prototype.clearData = function() {
  return this.setData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantReportAction.prototype.hasData = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantStartListeningAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantStartListeningAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantStartListeningAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantStartListeningAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantStartListeningAction}
 */
proto.ubo.v1.AssistantStartListeningAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantStartListeningAction;
  return proto.ubo.v1.AssistantStartListeningAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantStartListeningAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantStartListeningAction}
 */
proto.ubo.v1.AssistantStartListeningAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantStartListeningAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantStartListeningAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantStartListeningAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantStartListeningAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantStartListeningAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantStartListeningAction} returns this
 */
proto.ubo.v1.AssistantStartListeningAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantStartListeningAction} returns this
 */
proto.ubo.v1.AssistantStartListeningAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantStartListeningAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantStopListeningAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantStopListeningAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantStopListeningAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantStopListeningAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantStopListeningAction}
 */
proto.ubo.v1.AssistantStopListeningAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantStopListeningAction;
  return proto.ubo.v1.AssistantStopListeningAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantStopListeningAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantStopListeningAction}
 */
proto.ubo.v1.AssistantStopListeningAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantStopListeningAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantStopListeningAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantStopListeningAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantStopListeningAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantStopListeningAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantStopListeningAction} returns this
 */
proto.ubo.v1.AssistantStopListeningAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantStopListeningAction} returns this
 */
proto.ubo.v1.AssistantStopListeningAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantStopListeningAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantToggleListeningAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantToggleListeningAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantToggleListeningAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantToggleListeningAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantToggleListeningAction}
 */
proto.ubo.v1.AssistantToggleListeningAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantToggleListeningAction;
  return proto.ubo.v1.AssistantToggleListeningAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantToggleListeningAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantToggleListeningAction}
 */
proto.ubo.v1.AssistantToggleListeningAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantToggleListeningAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantToggleListeningAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantToggleListeningAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantToggleListeningAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantToggleListeningAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantToggleListeningAction} returns this
 */
proto.ubo.v1.AssistantToggleListeningAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantToggleListeningAction} returns this
 */
proto.ubo.v1.AssistantToggleListeningAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantToggleListeningAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantUpdateProvidersAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantUpdateProvidersAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantUpdateProvidersAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantUpdateProvidersAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantUpdateProvidersAction}
 */
proto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantUpdateProvidersAction;
  return proto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantUpdateProvidersAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantUpdateProvidersAction}
 */
proto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantUpdateProvidersAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantUpdateProvidersAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantUpdateProvidersAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantUpdateProvidersAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantUpdateProvidersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantUpdateProvidersAction} returns this
 */
proto.ubo.v1.AssistantUpdateProvidersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantUpdateProvidersAction} returns this
 */
proto.ubo.v1.AssistantUpdateProvidersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantUpdateProvidersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantAddMcpServerAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantAddMcpServerAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantAddMcpServerAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    name: jspb.Message.getFieldWithDefault(msg, 2, ""),
    type: jspb.Message.getFieldWithDefault(msg, 3, 0),
    config: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantAddMcpServerAction}
 */
proto.ubo.v1.AssistantAddMcpServerAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantAddMcpServerAction;
  return proto.ubo.v1.AssistantAddMcpServerAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantAddMcpServerAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantAddMcpServerAction}
 */
proto.ubo.v1.AssistantAddMcpServerAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.McpServerType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setConfig(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantAddMcpServerAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantAddMcpServerAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantAddMcpServerAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getName();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getType();
  if (f !== 0.0) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = message.getConfig();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string name = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.setName = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional McpServerType type = 3;
 * @return {!proto.ubo.v1.McpServerType}
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.getType = function() {
  return /** @type {!proto.ubo.v1.McpServerType} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.McpServerType} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.setType = function(value) {
  return jspb.Message.setProto3EnumField(this, 3, value);
};


/**
 * optional string config = 4;
 * @return {string}
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.getConfig = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerAction} returns this
 */
proto.ubo.v1.AssistantAddMcpServerAction.prototype.setConfig = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantToggleMcpServerAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantToggleMcpServerAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantToggleMcpServerAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serverId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantToggleMcpServerAction}
 */
proto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantToggleMcpServerAction;
  return proto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantToggleMcpServerAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantToggleMcpServerAction}
 */
proto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServerId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantToggleMcpServerAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantToggleMcpServerAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantToggleMcpServerAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServerId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantToggleMcpServerAction} returns this
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantToggleMcpServerAction} returns this
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string server_id = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.getServerId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantToggleMcpServerAction} returns this
 */
proto.ubo.v1.AssistantToggleMcpServerAction.prototype.setServerId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantDeleteMcpServerAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantDeleteMcpServerAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serverId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction}
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantDeleteMcpServerAction;
  return proto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantDeleteMcpServerAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction}
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServerId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantDeleteMcpServerAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantDeleteMcpServerAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServerId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction} returns this
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction} returns this
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string server_id = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.getServerId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerAction} returns this
 */
proto.ubo.v1.AssistantDeleteMcpServerAction.prototype.setServerId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantSyncMcpServersAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantSyncMcpServersAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantSyncMcpServersAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSyncMcpServersAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantSyncMcpServersAction}
 */
proto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantSyncMcpServersAction;
  return proto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantSyncMcpServersAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantSyncMcpServersAction}
 */
proto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantSyncMcpServersAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantSyncMcpServersAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantSyncMcpServersAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantSyncMcpServersAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantSyncMcpServersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantSyncMcpServersAction} returns this
 */
proto.ubo.v1.AssistantSyncMcpServersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantSyncMcpServersAction} returns this
 */
proto.ubo.v1.AssistantSyncMcpServersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantSyncMcpServersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantEvent}
 */
proto.ubo.v1.AssistantEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantEvent;
  return proto.ubo.v1.AssistantEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantEvent}
 */
proto.ubo.v1.AssistantEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantEvent} returns this
 */
proto.ubo.v1.AssistantEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantEvent} returns this
 */
proto.ubo.v1.AssistantEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantDownloadOllamaModelEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    model: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent}
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantDownloadOllamaModelEvent;
  return proto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent}
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setModel(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantDownloadOllamaModelEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getModel();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} returns this
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} returns this
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string model = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.getModel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDownloadOllamaModelEvent} returns this
 */
proto.ubo.v1.AssistantDownloadOllamaModelEvent.prototype.setModel = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantHandleReportEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantHandleReportEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantHandleReportEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sourceId: jspb.Message.getFieldWithDefault(msg, 2, ""),
    data: (f = msg.getData()) && proto.ubo.v1.AcceptableAssistanceFrame.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantHandleReportEvent}
 */
proto.ubo.v1.AssistantHandleReportEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantHandleReportEvent;
  return proto.ubo.v1.AssistantHandleReportEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantHandleReportEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantHandleReportEvent}
 */
proto.ubo.v1.AssistantHandleReportEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setSourceId(value);
      break;
    case 3:
      var value = new proto.ubo.v1.AcceptableAssistanceFrame;
      reader.readMessage(value,proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader);
      msg.setData(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantHandleReportEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantHandleReportEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantHandleReportEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSourceId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getData();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string source_id = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.getSourceId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.setSourceId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional AcceptableAssistanceFrame data = 3;
 * @return {?proto.ubo.v1.AcceptableAssistanceFrame}
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.getData = function() {
  return /** @type{?proto.ubo.v1.AcceptableAssistanceFrame} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AcceptableAssistanceFrame, 3));
};


/**
 * @param {?proto.ubo.v1.AcceptableAssistanceFrame|undefined} value
 * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this
*/
proto.ubo.v1.AssistantHandleReportEvent.prototype.setData = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AssistantHandleReportEvent} returns this
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.clearData = function() {
  return this.setData(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantHandleReportEvent.prototype.hasData = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantUpdateProvidersEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantUpdateProvidersEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent}
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantUpdateProvidersEvent;
  return proto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantUpdateProvidersEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent}
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantUpdateProvidersEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantUpdateProvidersEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent} returns this
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantUpdateProvidersEvent} returns this
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantUpdateProvidersEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantAddMcpServerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantAddMcpServerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantAddMcpServerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    name: jspb.Message.getFieldWithDefault(msg, 2, ""),
    type: jspb.Message.getFieldWithDefault(msg, 3, 0),
    config: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantAddMcpServerEvent}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantAddMcpServerEvent;
  return proto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantAddMcpServerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantAddMcpServerEvent}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.McpServerType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setConfig(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantAddMcpServerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantAddMcpServerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantAddMcpServerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getName();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getType();
  if (f !== 0.0) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = message.getConfig();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string name = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.setName = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional McpServerType type = 3;
 * @return {!proto.ubo.v1.McpServerType}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.getType = function() {
  return /** @type {!proto.ubo.v1.McpServerType} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.McpServerType} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.setType = function(value) {
  return jspb.Message.setProto3EnumField(this, 3, value);
};


/**
 * optional string config = 4;
 * @return {string}
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.getConfig = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantAddMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantAddMcpServerEvent.prototype.setConfig = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantDeleteMcpServerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantDeleteMcpServerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    serverId: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent}
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantDeleteMcpServerEvent;
  return proto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantDeleteMcpServerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent}
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setServerId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantDeleteMcpServerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantDeleteMcpServerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getServerId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string server_id = 2;
 * @return {string}
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.getServerId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantDeleteMcpServerEvent} returns this
 */
proto.ubo.v1.AssistantDeleteMcpServerEvent.prototype.setServerId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAssistant: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isListening: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isMicrophoneMute: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    selectedStt: jspb.Message.getFieldWithDefault(msg, 5, 0),
    selectedLlm: jspb.Message.getFieldWithDefault(msg, 6, 0),
    selectedModels: (f = msg.getSelectedModels()) && proto.ubo.v1.AssistantState.SelectedModelsDict.toObject(includeInstance, f),
    selectedTts: jspb.Message.getFieldWithDefault(msg, 8, 0),
    selectedImageGenerator: jspb.Message.getFieldWithDefault(msg, 9, 0),
    mcpServers: (f = msg.getMcpServers()) && proto.ubo.v1.AssistantState.McpServersDict.toObject(includeInstance, f),
    enabledMcpServers: (f = msg.getEnabledMcpServers()) && proto.ubo.v1.AssistantState.EnabledMcpServers.toObject(includeInstance, f),
    enabledMcpServersWithMetadataJson: jspb.Message.getFieldWithDefault(msg, 12, ""),
    providerSetupStatus: (f = msg.getProviderSetupStatus()) && proto.ubo.v1.AssistantState.ProviderSetupStatusDict.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantState}
 */
proto.ubo.v1.AssistantState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantState;
  return proto.ubo.v1.AssistantState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantState}
 */
proto.ubo.v1.AssistantState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsListening(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsMicrophoneMute(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 5:
      var value = /** @type {!proto.ubo.v1.AssistantSTTName} */ (reader.readEnum());
      msg.setSelectedStt(value);
      break;
    case 6:
      var value = /** @type {!proto.ubo.v1.AssistantLLMName} */ (reader.readEnum());
      msg.setSelectedLlm(value);
      break;
    case 7:
      var value = new proto.ubo.v1.AssistantState.SelectedModelsDict;
      reader.readMessage(value,proto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinaryFromReader);
      msg.setSelectedModels(value);
      break;
    case 8:
      var value = /** @type {!proto.ubo.v1.AssistantTTSName} */ (reader.readEnum());
      msg.setSelectedTts(value);
      break;
    case 9:
      var value = /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (reader.readEnum());
      msg.setSelectedImageGenerator(value);
      break;
    case 10:
      var value = new proto.ubo.v1.AssistantState.McpServersDict;
      reader.readMessage(value,proto.ubo.v1.AssistantState.McpServersDict.deserializeBinaryFromReader);
      msg.setMcpServers(value);
      break;
    case 11:
      var value = new proto.ubo.v1.AssistantState.EnabledMcpServers;
      reader.readMessage(value,proto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinaryFromReader);
      msg.setEnabledMcpServers(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.setEnabledMcpServersWithMetadataJson(value);
      break;
    case 13:
      var value = new proto.ubo.v1.AssistantState.ProviderSetupStatusDict;
      reader.readMessage(value,proto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinaryFromReader);
      msg.setProviderSetupStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.AssistantSTTName} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.AssistantLLMName} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = message.getSelectedModels();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.ubo.v1.AssistantState.SelectedModelsDict.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.ubo.v1.AssistantTTSName} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeEnum(
      8,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeEnum(
      9,
      f
    );
  }
  f = message.getMcpServers();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.ubo.v1.AssistantState.McpServersDict.serializeBinaryToWriter
    );
  }
  f = message.getEnabledMcpServers();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.ubo.v1.AssistantState.EnabledMcpServers.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeString(
      12,
      f
    );
  }
  f = message.getProviderSetupStatus();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.ubo.v1.AssistantState.ProviderSetupStatusDict.serializeBinaryToWriter
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantState.SelectedModelsDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantState.SelectedModelsDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantState.SelectedModelsDict}
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantState.SelectedModelsDict;
  return proto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantState.SelectedModelsDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantState.SelectedModelsDict}
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, "", "");
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantState.SelectedModelsDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantState.SelectedModelsDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);
  }
};


/**
 * map<string, string> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,string>}
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,string>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.AssistantState.SelectedModelsDict} returns this
 */
proto.ubo.v1.AssistantState.SelectedModelsDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantState.McpServersDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantState.McpServersDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantState.McpServersDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.McpServersDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.McpServerMetadata.toObject) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantState.McpServersDict}
 */
proto.ubo.v1.AssistantState.McpServersDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantState.McpServersDict;
  return proto.ubo.v1.AssistantState.McpServersDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantState.McpServersDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantState.McpServersDict}
 */
proto.ubo.v1.AssistantState.McpServersDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.McpServerMetadata.deserializeBinaryFromReader, "", new proto.ubo.v1.McpServerMetadata());
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantState.McpServersDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantState.McpServersDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantState.McpServersDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.McpServersDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.McpServerMetadata.serializeBinaryToWriter);
  }
};


/**
 * map<string, McpServerMetadata> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!proto.ubo.v1.McpServerMetadata>}
 */
proto.ubo.v1.AssistantState.McpServersDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!proto.ubo.v1.McpServerMetadata>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      proto.ubo.v1.McpServerMetadata));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.AssistantState.McpServersDict} returns this
 */
proto.ubo.v1.AssistantState.McpServersDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantState.EnabledMcpServers.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantState.EnabledMcpServers} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers}
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantState.EnabledMcpServers;
  return proto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantState.EnabledMcpServers} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers}
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantState.EnabledMcpServers.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantState.EnabledMcpServers} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers} returns this
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers} returns this
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.AssistantState.EnabledMcpServers} returns this
 */
proto.ubo.v1.AssistantState.EnabledMcpServers.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AssistantState.ProviderSetupStatusDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict}
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AssistantState.ProviderSetupStatusDict;
  return proto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict}
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readBool, null, "", false);
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AssistantState.ProviderSetupStatusDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeBool);
  }
};


/**
 * map<string, bool> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,boolean>}
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,boolean>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.AssistantState.ProviderSetupStatusDict} returns this
 */
proto.ubo.v1.AssistantState.ProviderSetupStatusDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_assistant = 1000;
 * @return {string}
 */
proto.ubo.v1.AssistantState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAssistant = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_listening = 2;
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.getIsListening = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setIsListening = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearIsListening = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasIsListening = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_microphone_mute = 3;
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.getIsMicrophoneMute = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setIsMicrophoneMute = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearIsMicrophoneMute = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasIsMicrophoneMute = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool is_active = 4;
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setIsActive = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearIsActive = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasIsActive = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional AssistantSTTName selected_stt = 5;
 * @return {!proto.ubo.v1.AssistantSTTName}
 */
proto.ubo.v1.AssistantState.prototype.getSelectedStt = function() {
  return /** @type {!proto.ubo.v1.AssistantSTTName} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantSTTName} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setSelectedStt = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearSelectedStt = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasSelectedStt = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional AssistantLLMName selected_llm = 6;
 * @return {!proto.ubo.v1.AssistantLLMName}
 */
proto.ubo.v1.AssistantState.prototype.getSelectedLlm = function() {
  return /** @type {!proto.ubo.v1.AssistantLLMName} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantLLMName} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setSelectedLlm = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearSelectedLlm = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasSelectedLlm = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional SelectedModelsDict selected_models = 7;
 * @return {?proto.ubo.v1.AssistantState.SelectedModelsDict}
 */
proto.ubo.v1.AssistantState.prototype.getSelectedModels = function() {
  return /** @type{?proto.ubo.v1.AssistantState.SelectedModelsDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.SelectedModelsDict, 7));
};


/**
 * @param {?proto.ubo.v1.AssistantState.SelectedModelsDict|undefined} value
 * @return {!proto.ubo.v1.AssistantState} returns this
*/
proto.ubo.v1.AssistantState.prototype.setSelectedModels = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearSelectedModels = function() {
  return this.setSelectedModels(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasSelectedModels = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional AssistantTTSName selected_tts = 8;
 * @return {!proto.ubo.v1.AssistantTTSName}
 */
proto.ubo.v1.AssistantState.prototype.getSelectedTts = function() {
  return /** @type {!proto.ubo.v1.AssistantTTSName} */ (jspb.Message.getFieldWithDefault(this, 8, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantTTSName} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setSelectedTts = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearSelectedTts = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasSelectedTts = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional AssistantImageGeneratorName selected_image_generator = 9;
 * @return {!proto.ubo.v1.AssistantImageGeneratorName}
 */
proto.ubo.v1.AssistantState.prototype.getSelectedImageGenerator = function() {
  return /** @type {!proto.ubo.v1.AssistantImageGeneratorName} */ (jspb.Message.getFieldWithDefault(this, 9, 0));
};


/**
 * @param {!proto.ubo.v1.AssistantImageGeneratorName} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setSelectedImageGenerator = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearSelectedImageGenerator = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasSelectedImageGenerator = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional McpServersDict mcp_servers = 10;
 * @return {?proto.ubo.v1.AssistantState.McpServersDict}
 */
proto.ubo.v1.AssistantState.prototype.getMcpServers = function() {
  return /** @type{?proto.ubo.v1.AssistantState.McpServersDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.McpServersDict, 10));
};


/**
 * @param {?proto.ubo.v1.AssistantState.McpServersDict|undefined} value
 * @return {!proto.ubo.v1.AssistantState} returns this
*/
proto.ubo.v1.AssistantState.prototype.setMcpServers = function(value) {
  return jspb.Message.setWrapperField(this, 10, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearMcpServers = function() {
  return this.setMcpServers(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasMcpServers = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional EnabledMcpServers enabled_mcp_servers = 11;
 * @return {?proto.ubo.v1.AssistantState.EnabledMcpServers}
 */
proto.ubo.v1.AssistantState.prototype.getEnabledMcpServers = function() {
  return /** @type{?proto.ubo.v1.AssistantState.EnabledMcpServers} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.EnabledMcpServers, 11));
};


/**
 * @param {?proto.ubo.v1.AssistantState.EnabledMcpServers|undefined} value
 * @return {!proto.ubo.v1.AssistantState} returns this
*/
proto.ubo.v1.AssistantState.prototype.setEnabledMcpServers = function(value) {
  return jspb.Message.setWrapperField(this, 11, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearEnabledMcpServers = function() {
  return this.setEnabledMcpServers(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasEnabledMcpServers = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional string enabled_mcp_servers_with_metadata_json = 12;
 * @return {string}
 */
proto.ubo.v1.AssistantState.prototype.getEnabledMcpServersWithMetadataJson = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 12, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.setEnabledMcpServersWithMetadataJson = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearEnabledMcpServersWithMetadataJson = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasEnabledMcpServersWithMetadataJson = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional ProviderSetupStatusDict provider_setup_status = 13;
 * @return {?proto.ubo.v1.AssistantState.ProviderSetupStatusDict}
 */
proto.ubo.v1.AssistantState.prototype.getProviderSetupStatus = function() {
  return /** @type{?proto.ubo.v1.AssistantState.ProviderSetupStatusDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantState.ProviderSetupStatusDict, 13));
};


/**
 * @param {?proto.ubo.v1.AssistantState.ProviderSetupStatusDict|undefined} value
 * @return {!proto.ubo.v1.AssistantState} returns this
*/
proto.ubo.v1.AssistantState.prototype.setProviderSetupStatus = function(value) {
  return jspb.Message.setWrapperField(this, 13, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AssistantState} returns this
 */
proto.ubo.v1.AssistantState.prototype.clearProviderSetupStatus = function() {
  return this.setProviderSetupStatus(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AssistantState.prototype.hasProviderSetupStatus = function() {
  return jspb.Message.getField(this, 13) != null;
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_ = [[1,2,3]];

/**
 * @enum {number}
 */
proto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase = {
  ACCEPTABLE_ASSISTANCE_FRAME_NOT_SET: 0,
  ASSISTANCE_AUDIO_FRAME: 1,
  ASSISTANCE_IMAGE_FRAME: 2,
  ASSISTANCE_TEXT_FRAME: 3
};

/**
 * @return {proto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.getAcceptableAssistanceFrameCase = function() {
  return /** @type {proto.ubo.v1.AcceptableAssistanceFrame.AcceptableAssistanceFrameCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AcceptableAssistanceFrame.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AcceptableAssistanceFrame} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AcceptableAssistanceFrame.toObject = function(includeInstance, msg) {
  var f, obj = {
    assistanceAudioFrame: (f = msg.getAssistanceAudioFrame()) && proto.ubo.v1.AssistanceAudioFrame.toObject(includeInstance, f),
    assistanceImageFrame: (f = msg.getAssistanceImageFrame()) && proto.ubo.v1.AssistanceImageFrame.toObject(includeInstance, f),
    assistanceTextFrame: (f = msg.getAssistanceTextFrame()) && proto.ubo.v1.AssistanceTextFrame.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame}
 */
proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AcceptableAssistanceFrame;
  return proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AcceptableAssistanceFrame} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame}
 */
proto.ubo.v1.AcceptableAssistanceFrame.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.AssistanceAudioFrame;
      reader.readMessage(value,proto.ubo.v1.AssistanceAudioFrame.deserializeBinaryFromReader);
      msg.setAssistanceAudioFrame(value);
      break;
    case 2:
      var value = new proto.ubo.v1.AssistanceImageFrame;
      reader.readMessage(value,proto.ubo.v1.AssistanceImageFrame.deserializeBinaryFromReader);
      msg.setAssistanceImageFrame(value);
      break;
    case 3:
      var value = new proto.ubo.v1.AssistanceTextFrame;
      reader.readMessage(value,proto.ubo.v1.AssistanceTextFrame.deserializeBinaryFromReader);
      msg.setAssistanceTextFrame(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AcceptableAssistanceFrame} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AcceptableAssistanceFrame.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAssistanceAudioFrame();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.AssistanceAudioFrame.serializeBinaryToWriter
    );
  }
  f = message.getAssistanceImageFrame();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.AssistanceImageFrame.serializeBinaryToWriter
    );
  }
  f = message.getAssistanceTextFrame();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.AssistanceTextFrame.serializeBinaryToWriter
    );
  }
};


/**
 * optional AssistanceAudioFrame assistance_audio_frame = 1;
 * @return {?proto.ubo.v1.AssistanceAudioFrame}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.getAssistanceAudioFrame = function() {
  return /** @type{?proto.ubo.v1.AssistanceAudioFrame} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistanceAudioFrame, 1));
};


/**
 * @param {?proto.ubo.v1.AssistanceAudioFrame|undefined} value
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this
*/
proto.ubo.v1.AcceptableAssistanceFrame.prototype.setAssistanceAudioFrame = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.clearAssistanceAudioFrame = function() {
  return this.setAssistanceAudioFrame(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.hasAssistanceAudioFrame = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional AssistanceImageFrame assistance_image_frame = 2;
 * @return {?proto.ubo.v1.AssistanceImageFrame}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.getAssistanceImageFrame = function() {
  return /** @type{?proto.ubo.v1.AssistanceImageFrame} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistanceImageFrame, 2));
};


/**
 * @param {?proto.ubo.v1.AssistanceImageFrame|undefined} value
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this
*/
proto.ubo.v1.AcceptableAssistanceFrame.prototype.setAssistanceImageFrame = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.clearAssistanceImageFrame = function() {
  return this.setAssistanceImageFrame(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.hasAssistanceImageFrame = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional AssistanceTextFrame assistance_text_frame = 3;
 * @return {?proto.ubo.v1.AssistanceTextFrame}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.getAssistanceTextFrame = function() {
  return /** @type{?proto.ubo.v1.AssistanceTextFrame} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistanceTextFrame, 3));
};


/**
 * @param {?proto.ubo.v1.AssistanceTextFrame|undefined} value
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this
*/
proto.ubo.v1.AcceptableAssistanceFrame.prototype.setAssistanceTextFrame = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.AcceptableAssistanceFrame.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AcceptableAssistanceFrame} returns this
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.clearAssistanceTextFrame = function() {
  return this.setAssistanceTextFrame(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AcceptableAssistanceFrame.prototype.hasAssistanceTextFrame = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioAction}
 */
proto.ubo.v1.AudioAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioAction;
  return proto.ubo.v1.AudioAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioAction}
 */
proto.ubo.v1.AudioAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioAction} returns this
 */
proto.ubo.v1.AudioAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioAction} returns this
 */
proto.ubo.v1.AudioAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioInstallDriverAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioInstallDriverAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioInstallDriverAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioInstallDriverAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioInstallDriverAction}
 */
proto.ubo.v1.AudioInstallDriverAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioInstallDriverAction;
  return proto.ubo.v1.AudioInstallDriverAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioInstallDriverAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioInstallDriverAction}
 */
proto.ubo.v1.AudioInstallDriverAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioInstallDriverAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioInstallDriverAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioInstallDriverAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioInstallDriverAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioInstallDriverAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioInstallDriverAction} returns this
 */
proto.ubo.v1.AudioInstallDriverAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioInstallDriverAction} returns this
 */
proto.ubo.v1.AudioInstallDriverAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioInstallDriverAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioSetVolumeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioSetVolumeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioSetVolumeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    volume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    device: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioSetVolumeAction}
 */
proto.ubo.v1.AudioSetVolumeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioSetVolumeAction;
  return proto.ubo.v1.AudioSetVolumeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioSetVolumeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioSetVolumeAction}
 */
proto.ubo.v1.AudioSetVolumeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setVolume(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());
      msg.setDevice(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioSetVolumeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioSetVolumeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioSetVolumeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getVolume();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getDevice();
  if (f !== 0.0) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float volume = 2;
 * @return {number}
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.getVolume = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.setVolume = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional AudioDevice device = 3;
 * @return {!proto.ubo.v1.AudioDevice}
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.getDevice = function() {
  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.AudioDevice} value
 * @return {!proto.ubo.v1.AudioSetVolumeAction} returns this
 */
proto.ubo.v1.AudioSetVolumeAction.prototype.setDevice = function(value) {
  return jspb.Message.setProto3EnumField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioChangeVolumeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioChangeVolumeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioChangeVolumeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    amount: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    device: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioChangeVolumeAction}
 */
proto.ubo.v1.AudioChangeVolumeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioChangeVolumeAction;
  return proto.ubo.v1.AudioChangeVolumeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioChangeVolumeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioChangeVolumeAction}
 */
proto.ubo.v1.AudioChangeVolumeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setAmount(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());
      msg.setDevice(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioChangeVolumeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioChangeVolumeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioChangeVolumeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getAmount();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getDevice();
  if (f !== 0.0) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float amount = 2;
 * @return {number}
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.getAmount = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.setAmount = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional AudioDevice device = 3;
 * @return {!proto.ubo.v1.AudioDevice}
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.getDevice = function() {
  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.AudioDevice} value
 * @return {!proto.ubo.v1.AudioChangeVolumeAction} returns this
 */
proto.ubo.v1.AudioChangeVolumeAction.prototype.setDevice = function(value) {
  return jspb.Message.setProto3EnumField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioSetMuteStatusAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioSetMuteStatusAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioSetMuteStatusAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isMute: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    device: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioSetMuteStatusAction}
 */
proto.ubo.v1.AudioSetMuteStatusAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioSetMuteStatusAction;
  return proto.ubo.v1.AudioSetMuteStatusAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioSetMuteStatusAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioSetMuteStatusAction}
 */
proto.ubo.v1.AudioSetMuteStatusAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsMute(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());
      msg.setDevice(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioSetMuteStatusAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioSetMuteStatusAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioSetMuteStatusAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsMute();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getDevice();
  if (f !== 0.0) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_mute = 2;
 * @return {boolean}
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.getIsMute = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.setIsMute = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional AudioDevice device = 3;
 * @return {!proto.ubo.v1.AudioDevice}
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.getDevice = function() {
  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.AudioDevice} value
 * @return {!proto.ubo.v1.AudioSetMuteStatusAction} returns this
 */
proto.ubo.v1.AudioSetMuteStatusAction.prototype.setDevice = function(value) {
  return jspb.Message.setProto3EnumField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioToggleMuteStatusAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioToggleMuteStatusAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioToggleMuteStatusAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    device: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioToggleMuteStatusAction}
 */
proto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioToggleMuteStatusAction;
  return proto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioToggleMuteStatusAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioToggleMuteStatusAction}
 */
proto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.AudioDevice} */ (reader.readEnum());
      msg.setDevice(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioToggleMuteStatusAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioToggleMuteStatusAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioToggleMuteStatusAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDevice();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioToggleMuteStatusAction} returns this
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioToggleMuteStatusAction} returns this
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AudioDevice device = 2;
 * @return {!proto.ubo.v1.AudioDevice}
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.getDevice = function() {
  return /** @type {!proto.ubo.v1.AudioDevice} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.AudioDevice} value
 * @return {!proto.ubo.v1.AudioToggleMuteStatusAction} returns this
 */
proto.ubo.v1.AudioToggleMuteStatusAction.prototype.setDevice = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlayChimeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlayChimeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayChimeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    name: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlayChimeAction}
 */
proto.ubo.v1.AudioPlayChimeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlayChimeAction;
  return proto.ubo.v1.AudioPlayChimeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlayChimeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlayChimeAction}
 */
proto.ubo.v1.AudioPlayChimeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlayChimeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlayChimeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayChimeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getName();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayChimeAction} returns this
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayChimeAction} returns this
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string name = 2;
 * @return {string}
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayChimeAction} returns this
 */
proto.ubo.v1.AudioPlayChimeAction.prototype.setName = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlayAudioSampleAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlayAudioSampleAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSampleAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleAction}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlayAudioSampleAction;
  return proto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlayAudioSampleAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleAction}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setSample(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlayAudioSampleAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlayAudioSampleAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSampleAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSample();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AudioSample sample = 2;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.getSample = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 2));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this
*/
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.setSample = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.clearSample = function() {
  return this.setSample(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSampleAction.prototype.hasSample = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlayAudioSequenceAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlayAudioSequenceAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f),
    id: jspb.Message.getFieldWithDefault(msg, 3, ""),
    index: jspb.Message.getFieldWithDefault(msg, 4, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlayAudioSequenceAction;
  return proto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlayAudioSequenceAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setSample(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlayAudioSequenceAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlayAudioSequenceAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSample();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional AudioSample sample = 2;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getSample = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 2));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this
*/
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setSample = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.clearSample = function() {
  return this.setSample(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.hasSample = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string id = 3;
 * @return {string}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional int64 index = 4;
 * @return {number}
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceAction} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceAction.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioSample.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioSample.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioSample} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioSample.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    data: msg.getData_asB64(),
    channels: jspb.Message.getFieldWithDefault(msg, 3, 0),
    rate: jspb.Message.getFieldWithDefault(msg, 4, 0),
    width: jspb.Message.getFieldWithDefault(msg, 5, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioSample.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioSample;
  return proto.ubo.v1.AudioSample.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioSample} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioSample.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setData(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setChannels(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setRate(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWidth(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioSample.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioSample.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioSample} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioSample.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getData_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      2,
      f
    );
  }
  f = message.getChannels();
  if (f !== 0) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getRate();
  if (f !== 0) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = message.getWidth();
  if (f !== 0) {
    writer.writeInt64(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioSample.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioSample} returns this
 */
proto.ubo.v1.AudioSample.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioSample} returns this
 */
proto.ubo.v1.AudioSample.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioSample.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bytes data = 2;
 * @return {string}
 */
proto.ubo.v1.AudioSample.prototype.getData = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * optional bytes data = 2;
 * This is a type-conversion wrapper around `getData()`
 * @return {string}
 */
proto.ubo.v1.AudioSample.prototype.getData_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getData()));
};


/**
 * optional bytes data = 2;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getData()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioSample.prototype.getData_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getData()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.AudioSample} returns this
 */
proto.ubo.v1.AudioSample.prototype.setData = function(value) {
  return jspb.Message.setProto3BytesField(this, 2, value);
};


/**
 * optional int64 channels = 3;
 * @return {number}
 */
proto.ubo.v1.AudioSample.prototype.getChannels = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioSample} returns this
 */
proto.ubo.v1.AudioSample.prototype.setChannels = function(value) {
  return jspb.Message.setProto3IntField(this, 3, value);
};


/**
 * optional int64 rate = 4;
 * @return {number}
 */
proto.ubo.v1.AudioSample.prototype.getRate = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioSample} returns this
 */
proto.ubo.v1.AudioSample.prototype.setRate = function(value) {
  return jspb.Message.setProto3IntField(this, 4, value);
};


/**
 * optional int64 width = 5;
 * @return {number}
 */
proto.ubo.v1.AudioSample.prototype.getWidth = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioSample} returns this
 */
proto.ubo.v1.AudioSample.prototype.setWidth = function(value) {
  return jspb.Message.setProto3IntField(this, 5, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioReportSampleAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioReportSampleAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioReportSampleAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    sampleSpeechRecognition: msg.getSampleSpeechRecognition_asB64(),
    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioReportSampleAction}
 */
proto.ubo.v1.AudioReportSampleAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioReportSampleAction;
  return proto.ubo.v1.AudioReportSampleAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioReportSampleAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioReportSampleAction}
 */
proto.ubo.v1.AudioReportSampleAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 3:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setSampleSpeechRecognition(value);
      break;
    case 4:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setSample(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioReportSampleAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioReportSampleAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioReportSampleAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getSampleSpeechRecognition_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      3,
      f
    );
  }
  f = message.getSample();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioReportSampleAction} returns this
 */
proto.ubo.v1.AudioReportSampleAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioReportSampleAction} returns this
 */
proto.ubo.v1.AudioReportSampleAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float timestamp = 2;
 * @return {number}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioReportSampleAction} returns this
 */
proto.ubo.v1.AudioReportSampleAction.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional bytes sample_speech_recognition = 3;
 * @return {string}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.getSampleSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * optional bytes sample_speech_recognition = 3;
 * This is a type-conversion wrapper around `getSampleSpeechRecognition()`
 * @return {string}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.getSampleSpeechRecognition_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getSampleSpeechRecognition()));
};


/**
 * optional bytes sample_speech_recognition = 3;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getSampleSpeechRecognition()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.getSampleSpeechRecognition_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getSampleSpeechRecognition()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.AudioReportSampleAction} returns this
 */
proto.ubo.v1.AudioReportSampleAction.prototype.setSampleSpeechRecognition = function(value) {
  return jspb.Message.setProto3BytesField(this, 3, value);
};


/**
 * optional AudioSample sample = 4;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.getSample = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 4));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AudioReportSampleAction} returns this
*/
proto.ubo.v1.AudioReportSampleAction.prototype.setSample = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AudioReportSampleAction} returns this
 */
proto.ubo.v1.AudioReportSampleAction.prototype.clearSample = function() {
  return this.setSample(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioReportSampleAction.prototype.hasSample = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlaybackDoneAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlaybackDoneAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlaybackDoneAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlaybackDoneAction}
 */
proto.ubo.v1.AudioPlaybackDoneAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlaybackDoneAction;
  return proto.ubo.v1.AudioPlaybackDoneAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlaybackDoneAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlaybackDoneAction}
 */
proto.ubo.v1.AudioPlaybackDoneAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlaybackDoneAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlaybackDoneAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlaybackDoneAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlaybackDoneAction} returns this
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlaybackDoneAction} returns this
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlaybackDoneAction} returns this
 */
proto.ubo.v1.AudioPlaybackDoneAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioStartRecordingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioStartRecordingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioStartRecordingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioStartRecordingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioStartRecordingAction}
 */
proto.ubo.v1.AudioStartRecordingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioStartRecordingAction;
  return proto.ubo.v1.AudioStartRecordingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioStartRecordingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioStartRecordingAction}
 */
proto.ubo.v1.AudioStartRecordingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioStartRecordingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioStartRecordingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioStartRecordingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioStartRecordingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioStartRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioStartRecordingAction} returns this
 */
proto.ubo.v1.AudioStartRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioStartRecordingAction} returns this
 */
proto.ubo.v1.AudioStartRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioStartRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioStopRecordingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioStopRecordingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioStopRecordingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioStopRecordingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioStopRecordingAction}
 */
proto.ubo.v1.AudioStopRecordingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioStopRecordingAction;
  return proto.ubo.v1.AudioStopRecordingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioStopRecordingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioStopRecordingAction}
 */
proto.ubo.v1.AudioStopRecordingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioStopRecordingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioStopRecordingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioStopRecordingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioStopRecordingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioStopRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioStopRecordingAction} returns this
 */
proto.ubo.v1.AudioStopRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioStopRecordingAction} returns this
 */
proto.ubo.v1.AudioStopRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioStopRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioToggleRecordingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioToggleRecordingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioToggleRecordingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioToggleRecordingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioToggleRecordingAction}
 */
proto.ubo.v1.AudioToggleRecordingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioToggleRecordingAction;
  return proto.ubo.v1.AudioToggleRecordingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioToggleRecordingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioToggleRecordingAction}
 */
proto.ubo.v1.AudioToggleRecordingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioToggleRecordingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioToggleRecordingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioToggleRecordingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioToggleRecordingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioToggleRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioToggleRecordingAction} returns this
 */
proto.ubo.v1.AudioToggleRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioToggleRecordingAction} returns this
 */
proto.ubo.v1.AudioToggleRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioToggleRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlayRecordingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlayRecordingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlayRecordingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayRecordingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlayRecordingAction}
 */
proto.ubo.v1.AudioPlayRecordingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlayRecordingAction;
  return proto.ubo.v1.AudioPlayRecordingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlayRecordingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlayRecordingAction}
 */
proto.ubo.v1.AudioPlayRecordingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlayRecordingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlayRecordingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlayRecordingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayRecordingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlayRecordingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayRecordingAction} returns this
 */
proto.ubo.v1.AudioPlayRecordingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayRecordingAction} returns this
 */
proto.ubo.v1.AudioPlayRecordingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayRecordingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioEvent}
 */
proto.ubo.v1.AudioEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioEvent;
  return proto.ubo.v1.AudioEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioEvent}
 */
proto.ubo.v1.AudioEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioEvent} returns this
 */
proto.ubo.v1.AudioEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioEvent} returns this
 */
proto.ubo.v1.AudioEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioReportSampleEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioReportSampleEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioReportSampleEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    sampleSpeechRecognition: msg.getSampleSpeechRecognition_asB64(),
    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioReportSampleEvent}
 */
proto.ubo.v1.AudioReportSampleEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioReportSampleEvent;
  return proto.ubo.v1.AudioReportSampleEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioReportSampleEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioReportSampleEvent}
 */
proto.ubo.v1.AudioReportSampleEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 3:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setSampleSpeechRecognition(value);
      break;
    case 4:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setSample(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioReportSampleEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioReportSampleEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioReportSampleEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getSampleSpeechRecognition_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      3,
      f
    );
  }
  f = message.getSample();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float timestamp = 2;
 * @return {number}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional bytes sample_speech_recognition = 3;
 * @return {string}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.getSampleSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * optional bytes sample_speech_recognition = 3;
 * This is a type-conversion wrapper around `getSampleSpeechRecognition()`
 * @return {string}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.getSampleSpeechRecognition_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getSampleSpeechRecognition()));
};


/**
 * optional bytes sample_speech_recognition = 3;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getSampleSpeechRecognition()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.getSampleSpeechRecognition_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getSampleSpeechRecognition()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.setSampleSpeechRecognition = function(value) {
  return jspb.Message.setProto3BytesField(this, 3, value);
};


/**
 * optional AudioSample sample = 4;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.getSample = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 4));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this
*/
proto.ubo.v1.AudioReportSampleEvent.prototype.setSample = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AudioReportSampleEvent} returns this
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.clearSample = function() {
  return this.setSample(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioReportSampleEvent.prototype.hasSample = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioInstallDriverEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioInstallDriverEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioInstallDriverEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioInstallDriverEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioInstallDriverEvent}
 */
proto.ubo.v1.AudioInstallDriverEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioInstallDriverEvent;
  return proto.ubo.v1.AudioInstallDriverEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioInstallDriverEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioInstallDriverEvent}
 */
proto.ubo.v1.AudioInstallDriverEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioInstallDriverEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioInstallDriverEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioInstallDriverEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioInstallDriverEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioInstallDriverEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioInstallDriverEvent} returns this
 */
proto.ubo.v1.AudioInstallDriverEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioInstallDriverEvent} returns this
 */
proto.ubo.v1.AudioInstallDriverEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioInstallDriverEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlayChimeEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlayChimeEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayChimeEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    name: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlayChimeEvent}
 */
proto.ubo.v1.AudioPlayChimeEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlayChimeEvent;
  return proto.ubo.v1.AudioPlayChimeEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlayChimeEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlayChimeEvent}
 */
proto.ubo.v1.AudioPlayChimeEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlayChimeEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlayChimeEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayChimeEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getName();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayChimeEvent} returns this
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayChimeEvent} returns this
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string name = 2;
 * @return {string}
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayChimeEvent} returns this
 */
proto.ubo.v1.AudioPlayChimeEvent.prototype.setName = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlayAudioSampleEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlayAudioSampleEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    volume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlayAudioSampleEvent;
  return proto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlayAudioSampleEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setVolume(value);
      break;
    case 3:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setSample(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlayAudioSampleEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlayAudioSampleEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getVolume();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getSample();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float volume = 2;
 * @return {number}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.getVolume = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.setVolume = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional AudioSample sample = 3;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.getSample = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 3));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this
*/
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.setSample = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSampleEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.clearSample = function() {
  return this.setSample(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSampleEvent.prototype.hasSample = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlayAudioSequenceEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlayAudioSequenceEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    volume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    sample: (f = msg.getSample()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f),
    id: jspb.Message.getFieldWithDefault(msg, 4, ""),
    index: jspb.Message.getFieldWithDefault(msg, 5, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlayAudioSequenceEvent;
  return proto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlayAudioSequenceEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setVolume(value);
      break;
    case 3:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setSample(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlayAudioSequenceEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlayAudioSequenceEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getVolume();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getSample();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float volume = 2;
 * @return {number}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getVolume = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setVolume = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional AudioSample sample = 3;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getSample = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 3));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this
*/
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setSample = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.clearSample = function() {
  return this.setSample(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.hasSample = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string id = 4;
 * @return {string}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};


/**
 * optional int64 index = 5;
 * @return {number}
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioPlayAudioSequenceEvent} returns this
 */
proto.ubo.v1.AudioPlayAudioSequenceEvent.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 5, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioPlaybackDoneEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioPlaybackDoneEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlaybackDoneEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioPlaybackDoneEvent}
 */
proto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioPlaybackDoneEvent;
  return proto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioPlaybackDoneEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioPlaybackDoneEvent}
 */
proto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioPlaybackDoneEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioPlaybackDoneEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioPlaybackDoneEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlaybackDoneEvent} returns this
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioPlaybackDoneEvent} returns this
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioPlaybackDoneEvent} returns this
 */
proto.ubo.v1.AudioPlaybackDoneEvent.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.AudioState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.AudioState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.AudioState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotAudio: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    playbackVolume: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    isPlaybackMute: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    captureVolume: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),
    isCaptureMute: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    isRecording: jspb.Message.getBooleanFieldWithDefault(msg, 6, false),
    recording: (f = msg.getRecording()) && proto.ubo.v1.AudioSample.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.AudioState}
 */
proto.ubo.v1.AudioState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.AudioState;
  return proto.ubo.v1.AudioState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.AudioState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.AudioState}
 */
proto.ubo.v1.AudioState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPlaybackVolume(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsPlaybackMute(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setCaptureVolume(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsCaptureMute(value);
      break;
    case 6:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsRecording(value);
      break;
    case 7:
      var value = new proto.ubo.v1.AudioSample;
      reader.readMessage(value,proto.ubo.v1.AudioSample.deserializeBinaryFromReader);
      msg.setRecording(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.AudioState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.AudioState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.AudioState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.AudioState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeBool(
      6,
      f
    );
  }
  f = message.getRecording();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.ubo.v1.AudioSample.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_audio = 1000;
 * @return {string}
 */
proto.ubo.v1.AudioState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotAudio = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float playback_volume = 2;
 * @return {number}
 */
proto.ubo.v1.AudioState.prototype.getPlaybackVolume = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.setPlaybackVolume = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.clearPlaybackVolume = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.hasPlaybackVolume = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_playback_mute = 3;
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.getIsPlaybackMute = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.setIsPlaybackMute = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.clearIsPlaybackMute = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.hasIsPlaybackMute = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float capture_volume = 4;
 * @return {number}
 */
proto.ubo.v1.AudioState.prototype.getCaptureVolume = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.setCaptureVolume = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.clearCaptureVolume = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.hasCaptureVolume = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_capture_mute = 5;
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.getIsCaptureMute = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.setIsCaptureMute = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.clearIsCaptureMute = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.hasIsCaptureMute = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional bool is_recording = 6;
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.getIsRecording = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 6, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.setIsRecording = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.clearIsRecording = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.hasIsRecording = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional AudioSample recording = 7;
 * @return {?proto.ubo.v1.AudioSample}
 */
proto.ubo.v1.AudioState.prototype.getRecording = function() {
  return /** @type{?proto.ubo.v1.AudioSample} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSample, 7));
};


/**
 * @param {?proto.ubo.v1.AudioSample|undefined} value
 * @return {!proto.ubo.v1.AudioState} returns this
*/
proto.ubo.v1.AudioState.prototype.setRecording = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.AudioState} returns this
 */
proto.ubo.v1.AudioState.prototype.clearRecording = function() {
  return this.setRecording(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.AudioState.prototype.hasRecording = function() {
  return jspb.Message.getField(this, 7) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraAction}
 */
proto.ubo.v1.CameraAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraAction;
  return proto.ubo.v1.CameraAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraAction}
 */
proto.ubo.v1.CameraAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraAction} returns this
 */
proto.ubo.v1.CameraAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraAction} returns this
 */
proto.ubo.v1.CameraAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraStartViewfinderAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraStartViewfinderAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraStartViewfinderAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    pattern: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraStartViewfinderAction}
 */
proto.ubo.v1.CameraStartViewfinderAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraStartViewfinderAction;
  return proto.ubo.v1.CameraStartViewfinderAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraStartViewfinderAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraStartViewfinderAction}
 */
proto.ubo.v1.CameraStartViewfinderAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setPattern(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraStartViewfinderAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraStartViewfinderAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraStartViewfinderAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string pattern = 2;
 * @return {string}
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.getPattern = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.setPattern = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraStartViewfinderAction} returns this
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.clearPattern = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraStartViewfinderAction.prototype.hasPattern = function() {
  return jspb.Message.getField(this, 2) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.CameraReportBarcodeAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraReportBarcodeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraReportBarcodeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraReportBarcodeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    codesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraReportBarcodeAction}
 */
proto.ubo.v1.CameraReportBarcodeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraReportBarcodeAction;
  return proto.ubo.v1.CameraReportBarcodeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraReportBarcodeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraReportBarcodeAction}
 */
proto.ubo.v1.CameraReportBarcodeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addCodes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraReportBarcodeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraReportBarcodeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraReportBarcodeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getCodesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string codes = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.getCodesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.setCodesList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.addCodes = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.CameraReportBarcodeAction} returns this
 */
proto.ubo.v1.CameraReportBarcodeAction.prototype.clearCodesList = function() {
  return this.setCodesList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraInstallDriverAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraInstallDriverAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraInstallDriverAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    make: jspb.Message.getFieldWithDefault(msg, 2, ""),
    model: jspb.Message.getFieldWithDefault(msg, 3, ""),
    variant: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraInstallDriverAction}
 */
proto.ubo.v1.CameraInstallDriverAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraInstallDriverAction;
  return proto.ubo.v1.CameraInstallDriverAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraInstallDriverAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraInstallDriverAction}
 */
proto.ubo.v1.CameraInstallDriverAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setMake(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setModel(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setVariant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraInstallDriverAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraInstallDriverAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraInstallDriverAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getMake();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getModel();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getVariant();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string make = 2;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.getMake = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.setMake = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string model = 3;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.getModel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.setModel = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional string variant = 4;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.getVariant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverAction} returns this
 */
proto.ubo.v1.CameraInstallDriverAction.prototype.setVariant = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraEvent}
 */
proto.ubo.v1.CameraEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraEvent;
  return proto.ubo.v1.CameraEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraEvent}
 */
proto.ubo.v1.CameraEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraEvent} returns this
 */
proto.ubo.v1.CameraEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraEvent} returns this
 */
proto.ubo.v1.CameraEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraInstallDriverEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraInstallDriverEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraInstallDriverEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    make: jspb.Message.getFieldWithDefault(msg, 2, ""),
    model: jspb.Message.getFieldWithDefault(msg, 3, ""),
    variant: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraInstallDriverEvent}
 */
proto.ubo.v1.CameraInstallDriverEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraInstallDriverEvent;
  return proto.ubo.v1.CameraInstallDriverEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraInstallDriverEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraInstallDriverEvent}
 */
proto.ubo.v1.CameraInstallDriverEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setMake(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setModel(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setVariant(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraInstallDriverEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraInstallDriverEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraInstallDriverEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getMake();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getModel();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getVariant();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string make = 2;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.getMake = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.setMake = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string model = 3;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.getModel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.setModel = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional string variant = 4;
 * @return {string}
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.getVariant = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraInstallDriverEvent} returns this
 */
proto.ubo.v1.CameraInstallDriverEvent.prototype.setVariant = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraRestoreDefaultAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraRestoreDefaultAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraRestoreDefaultAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraRestoreDefaultAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraRestoreDefaultAction}
 */
proto.ubo.v1.CameraRestoreDefaultAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraRestoreDefaultAction;
  return proto.ubo.v1.CameraRestoreDefaultAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraRestoreDefaultAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraRestoreDefaultAction}
 */
proto.ubo.v1.CameraRestoreDefaultAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraRestoreDefaultAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraRestoreDefaultAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraRestoreDefaultAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraRestoreDefaultAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraRestoreDefaultAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraRestoreDefaultAction} returns this
 */
proto.ubo.v1.CameraRestoreDefaultAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraRestoreDefaultAction} returns this
 */
proto.ubo.v1.CameraRestoreDefaultAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraRestoreDefaultAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraRestoreDefaultEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraRestoreDefaultEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraRestoreDefaultEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraRestoreDefaultEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraRestoreDefaultEvent}
 */
proto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraRestoreDefaultEvent;
  return proto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraRestoreDefaultEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraRestoreDefaultEvent}
 */
proto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraRestoreDefaultEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraRestoreDefaultEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraRestoreDefaultEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraRestoreDefaultEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraRestoreDefaultEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraRestoreDefaultEvent} returns this
 */
proto.ubo.v1.CameraRestoreDefaultEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraRestoreDefaultEvent} returns this
 */
proto.ubo.v1.CameraRestoreDefaultEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraRestoreDefaultEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraStartViewfinderEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraStartViewfinderEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraStartViewfinderEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    pattern: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraStartViewfinderEvent}
 */
proto.ubo.v1.CameraStartViewfinderEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraStartViewfinderEvent;
  return proto.ubo.v1.CameraStartViewfinderEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraStartViewfinderEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraStartViewfinderEvent}
 */
proto.ubo.v1.CameraStartViewfinderEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setPattern(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraStartViewfinderEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraStartViewfinderEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraStartViewfinderEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string pattern = 2;
 * @return {string}
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.getPattern = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.setPattern = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraStartViewfinderEvent} returns this
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.clearPattern = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraStartViewfinderEvent.prototype.hasPattern = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraReportImageEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraReportImageEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraReportImageEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    data: msg.getData_asB64(),
    width: jspb.Message.getFieldWithDefault(msg, 4, 0),
    height: jspb.Message.getFieldWithDefault(msg, 5, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraReportImageEvent}
 */
proto.ubo.v1.CameraReportImageEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraReportImageEvent;
  return proto.ubo.v1.CameraReportImageEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraReportImageEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraReportImageEvent}
 */
proto.ubo.v1.CameraReportImageEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 3:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setData(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWidth(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setHeight(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraReportImageEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraReportImageEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraReportImageEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getData_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      3,
      f
    );
  }
  f = message.getWidth();
  if (f !== 0) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = message.getHeight();
  if (f !== 0) {
    writer.writeInt64(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraReportImageEvent} returns this
 */
proto.ubo.v1.CameraReportImageEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraReportImageEvent} returns this
 */
proto.ubo.v1.CameraReportImageEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float timestamp = 2;
 * @return {number}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.CameraReportImageEvent} returns this
 */
proto.ubo.v1.CameraReportImageEvent.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional bytes data = 3;
 * @return {string}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.getData = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * optional bytes data = 3;
 * This is a type-conversion wrapper around `getData()`
 * @return {string}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.getData_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getData()));
};


/**
 * optional bytes data = 3;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getData()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.getData_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getData()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.CameraReportImageEvent} returns this
 */
proto.ubo.v1.CameraReportImageEvent.prototype.setData = function(value) {
  return jspb.Message.setProto3BytesField(this, 3, value);
};


/**
 * optional int64 width = 4;
 * @return {number}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.getWidth = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.CameraReportImageEvent} returns this
 */
proto.ubo.v1.CameraReportImageEvent.prototype.setWidth = function(value) {
  return jspb.Message.setProto3IntField(this, 4, value);
};


/**
 * optional int64 height = 5;
 * @return {number}
 */
proto.ubo.v1.CameraReportImageEvent.prototype.getHeight = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.CameraReportImageEvent} returns this
 */
proto.ubo.v1.CameraReportImageEvent.prototype.setHeight = function(value) {
  return jspb.Message.setProto3IntField(this, 5, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraStopViewfinderEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraStopViewfinderEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraStopViewfinderEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraStopViewfinderEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraStopViewfinderEvent}
 */
proto.ubo.v1.CameraStopViewfinderEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraStopViewfinderEvent;
  return proto.ubo.v1.CameraStopViewfinderEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraStopViewfinderEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraStopViewfinderEvent}
 */
proto.ubo.v1.CameraStopViewfinderEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraStopViewfinderEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraStopViewfinderEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraStopViewfinderEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraStopViewfinderEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraStopViewfinderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraStopViewfinderEvent} returns this
 */
proto.ubo.v1.CameraStopViewfinderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraStopViewfinderEvent} returns this
 */
proto.ubo.v1.CameraStopViewfinderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraStopViewfinderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraSetIndexAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraSetIndexAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraSetIndexAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraSetIndexAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    index: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraSetIndexAction}
 */
proto.ubo.v1.CameraSetIndexAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraSetIndexAction;
  return proto.ubo.v1.CameraSetIndexAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraSetIndexAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraSetIndexAction}
 */
proto.ubo.v1.CameraSetIndexAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraSetIndexAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraSetIndexAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraSetIndexAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraSetIndexAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIndex();
  if (f !== 0) {
    writer.writeInt64(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraSetIndexAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraSetIndexAction} returns this
 */
proto.ubo.v1.CameraSetIndexAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraSetIndexAction} returns this
 */
proto.ubo.v1.CameraSetIndexAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraSetIndexAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 index = 2;
 * @return {number}
 */
proto.ubo.v1.CameraSetIndexAction.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.CameraSetIndexAction} returns this
 */
proto.ubo.v1.CameraSetIndexAction.prototype.setIndex = function(value) {
  return jspb.Message.setProto3IntField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraDetectAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraDetectAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraDetectAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraDetectAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraDetectAction}
 */
proto.ubo.v1.CameraDetectAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraDetectAction;
  return proto.ubo.v1.CameraDetectAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraDetectAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraDetectAction}
 */
proto.ubo.v1.CameraDetectAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraDetectAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraDetectAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraDetectAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraDetectAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraDetectAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraDetectAction} returns this
 */
proto.ubo.v1.CameraDetectAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraDetectAction} returns this
 */
proto.ubo.v1.CameraDetectAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraDetectAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraSetAvailableCamerasAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraSetAvailableCamerasAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    availableCamerasList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction}
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraSetAvailableCamerasAction;
  return proto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraSetAvailableCamerasAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction}
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);
      for (var i = 0; i < values.length; i++) {
        msg.addAvailableCameras(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraSetAvailableCamerasAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraSetAvailableCamerasAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getAvailableCamerasList();
  if (f.length > 0) {
    writer.writePackedInt64(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated int64 available_cameras = 2;
 * @return {!Array<number>}
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.getAvailableCamerasList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<number>} value
 * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.setAvailableCamerasList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.addAvailableCameras = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.CameraSetAvailableCamerasAction} returns this
 */
proto.ubo.v1.CameraSetAvailableCamerasAction.prototype.clearAvailableCamerasList = function() {
  return this.setAvailableCamerasList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraDetectEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraDetectEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraDetectEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraDetectEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraDetectEvent}
 */
proto.ubo.v1.CameraDetectEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraDetectEvent;
  return proto.ubo.v1.CameraDetectEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraDetectEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraDetectEvent}
 */
proto.ubo.v1.CameraDetectEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraDetectEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraDetectEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraDetectEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraDetectEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraDetectEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraDetectEvent} returns this
 */
proto.ubo.v1.CameraDetectEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraDetectEvent} returns this
 */
proto.ubo.v1.CameraDetectEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraDetectEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.CameraDetectedEvent.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraDetectedEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraDetectedEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraDetectedEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraDetectedEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    availableCamerasList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraDetectedEvent}
 */
proto.ubo.v1.CameraDetectedEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraDetectedEvent;
  return proto.ubo.v1.CameraDetectedEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraDetectedEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraDetectedEvent}
 */
proto.ubo.v1.CameraDetectedEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);
      for (var i = 0; i < values.length; i++) {
        msg.addAvailableCameras(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraDetectedEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraDetectedEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraDetectedEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraDetectedEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getAvailableCamerasList();
  if (f.length > 0) {
    writer.writePackedInt64(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraDetectedEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraDetectedEvent} returns this
 */
proto.ubo.v1.CameraDetectedEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraDetectedEvent} returns this
 */
proto.ubo.v1.CameraDetectedEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraDetectedEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated int64 available_cameras = 2;
 * @return {!Array<number>}
 */
proto.ubo.v1.CameraDetectedEvent.prototype.getAvailableCamerasList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<number>} value
 * @return {!proto.ubo.v1.CameraDetectedEvent} returns this
 */
proto.ubo.v1.CameraDetectedEvent.prototype.setAvailableCamerasList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.CameraDetectedEvent} returns this
 */
proto.ubo.v1.CameraDetectedEvent.prototype.addAvailableCameras = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.CameraDetectedEvent} returns this
 */
proto.ubo.v1.CameraDetectedEvent.prototype.clearAvailableCamerasList = function() {
  return this.setAvailableCamerasList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraReinitializeEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraReinitializeEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraReinitializeEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraReinitializeEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraReinitializeEvent}
 */
proto.ubo.v1.CameraReinitializeEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraReinitializeEvent;
  return proto.ubo.v1.CameraReinitializeEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraReinitializeEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraReinitializeEvent}
 */
proto.ubo.v1.CameraReinitializeEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraReinitializeEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraReinitializeEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraReinitializeEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraReinitializeEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraReinitializeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraReinitializeEvent} returns this
 */
proto.ubo.v1.CameraReinitializeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraReinitializeEvent} returns this
 */
proto.ubo.v1.CameraReinitializeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraReinitializeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.CameraState.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotCamera: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    queueList: jspb.Message.toObjectList(msg.getQueueList(),
    proto.ubo.v1.QRCodeInputDescription.toObject, includeInstance),
    selectedCameraIndex: jspb.Message.getFieldWithDefault(msg, 3, 0),
    availableCameras: (f = msg.getAvailableCameras()) && proto.ubo.v1.CameraState.AvailableCameras.toObject(includeInstance, f),
    cameraType: jspb.Message.getFieldWithDefault(msg, 5, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraState}
 */
proto.ubo.v1.CameraState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraState;
  return proto.ubo.v1.CameraState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraState}
 */
proto.ubo.v1.CameraState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera(value);
      break;
    case 2:
      var value = new proto.ubo.v1.QRCodeInputDescription;
      reader.readMessage(value,proto.ubo.v1.QRCodeInputDescription.deserializeBinaryFromReader);
      msg.addQueue(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setSelectedCameraIndex(value);
      break;
    case 4:
      var value = new proto.ubo.v1.CameraState.AvailableCameras;
      reader.readMessage(value,proto.ubo.v1.CameraState.AvailableCameras.deserializeBinaryFromReader);
      msg.setAvailableCameras(value);
      break;
    case 5:
      var value = /** @type {!proto.ubo.v1.CameraType} */ (reader.readEnum());
      msg.setCameraType(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getQueueList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.QRCodeInputDescription.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getAvailableCameras();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.CameraState.AvailableCameras.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.ubo.v1.CameraType} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.CameraState.AvailableCameras.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.CameraState.AvailableCameras.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.CameraState.AvailableCameras.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.CameraState.AvailableCameras} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraState.AvailableCameras.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.CameraState.AvailableCameras}
 */
proto.ubo.v1.CameraState.AvailableCameras.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.CameraState.AvailableCameras;
  return proto.ubo.v1.CameraState.AvailableCameras.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.CameraState.AvailableCameras} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.CameraState.AvailableCameras}
 */
proto.ubo.v1.CameraState.AvailableCameras.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.CameraState.AvailableCameras.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.CameraState.AvailableCameras.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.CameraState.AvailableCameras} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.CameraState.AvailableCameras.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedInt64(
      1,
      f
    );
  }
};


/**
 * repeated int64 items = 1;
 * @return {!Array<number>}
 */
proto.ubo.v1.CameraState.AvailableCameras.prototype.getItemsList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<number>} value
 * @return {!proto.ubo.v1.CameraState.AvailableCameras} returns this
 */
proto.ubo.v1.CameraState.AvailableCameras.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.CameraState.AvailableCameras} returns this
 */
proto.ubo.v1.CameraState.AvailableCameras.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.CameraState.AvailableCameras} returns this
 */
proto.ubo.v1.CameraState.AvailableCameras.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_camera = 1000;
 * @return {string}
 */
proto.ubo.v1.CameraState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotCamera = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated QRCodeInputDescription queue = 2;
 * @return {!Array<!proto.ubo.v1.QRCodeInputDescription>}
 */
proto.ubo.v1.CameraState.prototype.getQueueList = function() {
  return /** @type{!Array<!proto.ubo.v1.QRCodeInputDescription>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.QRCodeInputDescription, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.QRCodeInputDescription>} value
 * @return {!proto.ubo.v1.CameraState} returns this
*/
proto.ubo.v1.CameraState.prototype.setQueueList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.QRCodeInputDescription=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.QRCodeInputDescription}
 */
proto.ubo.v1.CameraState.prototype.addQueue = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.QRCodeInputDescription, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.clearQueueList = function() {
  return this.setQueueList([]);
};


/**
 * optional int64 selected_camera_index = 3;
 * @return {number}
 */
proto.ubo.v1.CameraState.prototype.getSelectedCameraIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.setSelectedCameraIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.clearSelectedCameraIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraState.prototype.hasSelectedCameraIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional AvailableCameras available_cameras = 4;
 * @return {?proto.ubo.v1.CameraState.AvailableCameras}
 */
proto.ubo.v1.CameraState.prototype.getAvailableCameras = function() {
  return /** @type{?proto.ubo.v1.CameraState.AvailableCameras} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraState.AvailableCameras, 4));
};


/**
 * @param {?proto.ubo.v1.CameraState.AvailableCameras|undefined} value
 * @return {!proto.ubo.v1.CameraState} returns this
*/
proto.ubo.v1.CameraState.prototype.setAvailableCameras = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.clearAvailableCameras = function() {
  return this.setAvailableCameras(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraState.prototype.hasAvailableCameras = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional CameraType camera_type = 5;
 * @return {!proto.ubo.v1.CameraType}
 */
proto.ubo.v1.CameraState.prototype.getCameraType = function() {
  return /** @type {!proto.ubo.v1.CameraType} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {!proto.ubo.v1.CameraType} value
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.setCameraType = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.CameraState} returns this
 */
proto.ubo.v1.CameraState.prototype.clearCameraType = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.CameraState.prototype.hasCameraType = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayAction}
 */
proto.ubo.v1.DisplayAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayAction;
  return proto.ubo.v1.DisplayAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayAction}
 */
proto.ubo.v1.DisplayAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayAction} returns this
 */
proto.ubo.v1.DisplayAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayAction} returns this
 */
proto.ubo.v1.DisplayAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayEvent}
 */
proto.ubo.v1.DisplayEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayEvent;
  return proto.ubo.v1.DisplayEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayEvent}
 */
proto.ubo.v1.DisplayEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayEvent} returns this
 */
proto.ubo.v1.DisplayEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayEvent} returns this
 */
proto.ubo.v1.DisplayEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayPauseAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayPauseAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayPauseAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayPauseAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayPauseAction}
 */
proto.ubo.v1.DisplayPauseAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayPauseAction;
  return proto.ubo.v1.DisplayPauseAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayPauseAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayPauseAction}
 */
proto.ubo.v1.DisplayPauseAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayPauseAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayPauseAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayPauseAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayPauseAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayPauseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayPauseAction} returns this
 */
proto.ubo.v1.DisplayPauseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayPauseAction} returns this
 */
proto.ubo.v1.DisplayPauseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayPauseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayResumeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayResumeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayResumeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayResumeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayResumeAction}
 */
proto.ubo.v1.DisplayResumeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayResumeAction;
  return proto.ubo.v1.DisplayResumeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayResumeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayResumeAction}
 */
proto.ubo.v1.DisplayResumeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayResumeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayResumeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayResumeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayResumeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayResumeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayResumeAction} returns this
 */
proto.ubo.v1.DisplayResumeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayResumeAction} returns this
 */
proto.ubo.v1.DisplayResumeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayResumeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayRedrawAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayRedrawAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayRedrawAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayRedrawAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayRedrawAction}
 */
proto.ubo.v1.DisplayRedrawAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayRedrawAction;
  return proto.ubo.v1.DisplayRedrawAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayRedrawAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayRedrawAction}
 */
proto.ubo.v1.DisplayRedrawAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayRedrawAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayRedrawAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayRedrawAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayRedrawAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayRedrawAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayRedrawAction} returns this
 */
proto.ubo.v1.DisplayRedrawAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayRedrawAction} returns this
 */
proto.ubo.v1.DisplayRedrawAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayRedrawAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayBlankAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayBlankAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayBlankAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayBlankAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayBlankAction}
 */
proto.ubo.v1.DisplayBlankAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayBlankAction;
  return proto.ubo.v1.DisplayBlankAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayBlankAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayBlankAction}
 */
proto.ubo.v1.DisplayBlankAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayBlankAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayBlankAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayBlankAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayBlankAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayBlankAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayBlankAction} returns this
 */
proto.ubo.v1.DisplayBlankAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayBlankAction} returns this
 */
proto.ubo.v1.DisplayBlankAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayBlankAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayUnblankAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayUnblankAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayUnblankAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayUnblankAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayUnblankAction}
 */
proto.ubo.v1.DisplayUnblankAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayUnblankAction;
  return proto.ubo.v1.DisplayUnblankAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayUnblankAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayUnblankAction}
 */
proto.ubo.v1.DisplayUnblankAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayUnblankAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayUnblankAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayUnblankAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayUnblankAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayUnblankAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayUnblankAction} returns this
 */
proto.ubo.v1.DisplayUnblankAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayUnblankAction} returns this
 */
proto.ubo.v1.DisplayUnblankAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayUnblankAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayUpdateActivityAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayUpdateActivityAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayUpdateActivityAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayUpdateActivityAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayUpdateActivityAction}
 */
proto.ubo.v1.DisplayUpdateActivityAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayUpdateActivityAction;
  return proto.ubo.v1.DisplayUpdateActivityAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayUpdateActivityAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayUpdateActivityAction}
 */
proto.ubo.v1.DisplayUpdateActivityAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayUpdateActivityAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayUpdateActivityAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayUpdateActivityAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayUpdateActivityAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayUpdateActivityAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayUpdateActivityAction} returns this
 */
proto.ubo.v1.DisplayUpdateActivityAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayUpdateActivityAction} returns this
 */
proto.ubo.v1.DisplayUpdateActivityAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayUpdateActivityAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplaySetBlankTimeoutAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplaySetBlankTimeoutAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timeout: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction}
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplaySetBlankTimeoutAction;
  return proto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplaySetBlankTimeoutAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction}
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (reader.readEnum());
      msg.setTimeout(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplaySetBlankTimeoutAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplaySetBlankTimeoutAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimeout();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction} returns this
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction} returns this
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional DisplayBlankTimeout timeout = 2;
 * @return {!proto.ubo.v1.DisplayBlankTimeout}
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.getTimeout = function() {
  return /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.DisplayBlankTimeout} value
 * @return {!proto.ubo.v1.DisplaySetBlankTimeoutAction} returns this
 */
proto.ubo.v1.DisplaySetBlankTimeoutAction.prototype.setTimeout = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayRedrawEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayRedrawEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayRedrawEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayRedrawEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayRedrawEvent}
 */
proto.ubo.v1.DisplayRedrawEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayRedrawEvent;
  return proto.ubo.v1.DisplayRedrawEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayRedrawEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayRedrawEvent}
 */
proto.ubo.v1.DisplayRedrawEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayRedrawEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayRedrawEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayRedrawEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayRedrawEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayRedrawEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayRedrawEvent} returns this
 */
proto.ubo.v1.DisplayRedrawEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayRedrawEvent} returns this
 */
proto.ubo.v1.DisplayRedrawEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayRedrawEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayBlankEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayBlankEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayBlankEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayBlankEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayBlankEvent}
 */
proto.ubo.v1.DisplayBlankEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayBlankEvent;
  return proto.ubo.v1.DisplayBlankEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayBlankEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayBlankEvent}
 */
proto.ubo.v1.DisplayBlankEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayBlankEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayBlankEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayBlankEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayBlankEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayBlankEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayBlankEvent} returns this
 */
proto.ubo.v1.DisplayBlankEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayBlankEvent} returns this
 */
proto.ubo.v1.DisplayBlankEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayBlankEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayUnblankEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayUnblankEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayUnblankEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayUnblankEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayUnblankEvent}
 */
proto.ubo.v1.DisplayUnblankEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayUnblankEvent;
  return proto.ubo.v1.DisplayUnblankEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayUnblankEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayUnblankEvent}
 */
proto.ubo.v1.DisplayUnblankEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayUnblankEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayUnblankEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayUnblankEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayUnblankEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayUnblankEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayUnblankEvent} returns this
 */
proto.ubo.v1.DisplayUnblankEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayUnblankEvent} returns this
 */
proto.ubo.v1.DisplayUnblankEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayUnblankEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.DisplayRenderEvent.repeatedFields_ = [4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayRenderEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayRenderEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayRenderEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    data: msg.getData_asB64(),
    rectangleList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f,
    density: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayRenderEvent}
 */
proto.ubo.v1.DisplayRenderEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayRenderEvent;
  return proto.ubo.v1.DisplayRenderEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayRenderEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayRenderEvent}
 */
proto.ubo.v1.DisplayRenderEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 3:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setData(value);
      break;
    case 4:
      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);
      for (var i = 0; i < values.length; i++) {
        msg.addRectangle(values[i]);
      }
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDensity(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayRenderEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayRenderEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayRenderEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getData_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      3,
      f
    );
  }
  f = message.getRectangleList();
  if (f.length > 0) {
    writer.writePackedInt64(
      4,
      f
    );
  }
  f = message.getDensity();
  if (f !== 0.0) {
    writer.writeFloat(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float timestamp = 2;
 * @return {number}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional bytes data = 3;
 * @return {string}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.getData = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * optional bytes data = 3;
 * This is a type-conversion wrapper around `getData()`
 * @return {string}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.getData_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getData()));
};


/**
 * optional bytes data = 3;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getData()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.getData_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getData()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.setData = function(value) {
  return jspb.Message.setProto3BytesField(this, 3, value);
};


/**
 * repeated int64 rectangle = 4;
 * @return {!Array<number>}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.getRectangleList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 4));
};


/**
 * @param {!Array<number>} value
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.setRectangleList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.addRectangle = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.clearRectangleList = function() {
  return this.setRectangleList([]);
};


/**
 * optional float density = 5;
 * @return {number}
 */
proto.ubo.v1.DisplayRenderEvent.prototype.getDensity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.DisplayRenderEvent} returns this
 */
proto.ubo.v1.DisplayRenderEvent.prototype.setDensity = function(value) {
  return jspb.Message.setProto3FloatField(this, 5, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.DisplayCompressedRenderEvent.repeatedFields_ = [4];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayCompressedRenderEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayCompressedRenderEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayCompressedRenderEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    compressedData: msg.getCompressedData_asB64(),
    rectangleList: (f = jspb.Message.getRepeatedField(msg, 4)) == null ? undefined : f,
    density: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayCompressedRenderEvent;
  return proto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayCompressedRenderEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 3:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setCompressedData(value);
      break;
    case 4:
      var values = /** @type {!Array<number>} */ (reader.isDelimited() ? reader.readPackedInt64() : [reader.readInt64()]);
      for (var i = 0; i < values.length; i++) {
        msg.addRectangle(values[i]);
      }
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setDensity(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayCompressedRenderEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayCompressedRenderEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayCompressedRenderEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getCompressedData_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      3,
      f
    );
  }
  f = message.getRectangleList();
  if (f.length > 0) {
    writer.writePackedInt64(
      4,
      f
    );
  }
  f = message.getDensity();
  if (f !== 0.0) {
    writer.writeFloat(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float timestamp = 2;
 * @return {number}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional bytes compressed_data = 3;
 * @return {string}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.getCompressedData = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * optional bytes compressed_data = 3;
 * This is a type-conversion wrapper around `getCompressedData()`
 * @return {string}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.getCompressedData_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getCompressedData()));
};


/**
 * optional bytes compressed_data = 3;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getCompressedData()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.getCompressedData_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getCompressedData()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.setCompressedData = function(value) {
  return jspb.Message.setProto3BytesField(this, 3, value);
};


/**
 * repeated int64 rectangle = 4;
 * @return {!Array<number>}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.getRectangleList = function() {
  return /** @type {!Array<number>} */ (jspb.Message.getRepeatedField(this, 4));
};


/**
 * @param {!Array<number>} value
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.setRectangleList = function(value) {
  return jspb.Message.setField(this, 4, value || []);
};


/**
 * @param {number} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.addRectangle = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 4, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.clearRectangleList = function() {
  return this.setRectangleList([]);
};


/**
 * optional float density = 5;
 * @return {number}
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.getDensity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.DisplayCompressedRenderEvent} returns this
 */
proto.ubo.v1.DisplayCompressedRenderEvent.prototype.setDensity = function(value) {
  return jspb.Message.setProto3FloatField(this, 5, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DisplayState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DisplayState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DisplayState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDisplay: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isPaused: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isBlanked: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    lastActivityTime: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0),
    selectedBlankTimeout: jspb.Message.getFieldWithDefault(msg, 5, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DisplayState}
 */
proto.ubo.v1.DisplayState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DisplayState;
  return proto.ubo.v1.DisplayState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DisplayState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DisplayState}
 */
proto.ubo.v1.DisplayState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsPaused(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsBlanked(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setLastActivityTime(value);
      break;
    case 5:
      var value = /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (reader.readEnum());
      msg.setSelectedBlankTimeout(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DisplayState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DisplayState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DisplayState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DisplayState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_display = 1000;
 * @return {string}
 */
proto.ubo.v1.DisplayState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDisplay = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_paused = 2;
 * @return {boolean}
 */
proto.ubo.v1.DisplayState.prototype.getIsPaused = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.setIsPaused = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.clearIsPaused = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayState.prototype.hasIsPaused = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_blanked = 3;
 * @return {boolean}
 */
proto.ubo.v1.DisplayState.prototype.getIsBlanked = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.setIsBlanked = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.clearIsBlanked = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayState.prototype.hasIsBlanked = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional float last_activity_time = 4;
 * @return {number}
 */
proto.ubo.v1.DisplayState.prototype.getLastActivityTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.setLastActivityTime = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.clearLastActivityTime = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayState.prototype.hasLastActivityTime = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional DisplayBlankTimeout selected_blank_timeout = 5;
 * @return {!proto.ubo.v1.DisplayBlankTimeout}
 */
proto.ubo.v1.DisplayState.prototype.getSelectedBlankTimeout = function() {
  return /** @type {!proto.ubo.v1.DisplayBlankTimeout} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {!proto.ubo.v1.DisplayBlankTimeout} value
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.setSelectedBlankTimeout = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DisplayState} returns this
 */
proto.ubo.v1.DisplayState.prototype.clearSelectedBlankTimeout = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DisplayState.prototype.hasSelectedBlankTimeout = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerAction}
 */
proto.ubo.v1.DockerAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerAction;
  return proto.ubo.v1.DockerAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerAction}
 */
proto.ubo.v1.DockerAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerAction} returns this
 */
proto.ubo.v1.DockerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerAction} returns this
 */
proto.ubo.v1.DockerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerInstallAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerInstallAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerInstallAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerInstallAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerInstallAction}
 */
proto.ubo.v1.DockerInstallAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerInstallAction;
  return proto.ubo.v1.DockerInstallAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerInstallAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerInstallAction}
 */
proto.ubo.v1.DockerInstallAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerInstallAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerInstallAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerInstallAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerInstallAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerInstallAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerInstallAction} returns this
 */
proto.ubo.v1.DockerInstallAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerInstallAction} returns this
 */
proto.ubo.v1.DockerInstallAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerInstallAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerStartAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerStartAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerStartAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStartAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerStartAction}
 */
proto.ubo.v1.DockerStartAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerStartAction;
  return proto.ubo.v1.DockerStartAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerStartAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerStartAction}
 */
proto.ubo.v1.DockerStartAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerStartAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerStartAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerStartAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStartAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerStartAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerStartAction} returns this
 */
proto.ubo.v1.DockerStartAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerStartAction} returns this
 */
proto.ubo.v1.DockerStartAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerStartAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerStopAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerStopAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerStopAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStopAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerStopAction}
 */
proto.ubo.v1.DockerStopAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerStopAction;
  return proto.ubo.v1.DockerStopAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerStopAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerStopAction}
 */
proto.ubo.v1.DockerStopAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerStopAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerStopAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerStopAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStopAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerStopAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerStopAction} returns this
 */
proto.ubo.v1.DockerStopAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerStopAction} returns this
 */
proto.ubo.v1.DockerStopAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerStopAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerSetStatusAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerSetStatusAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerSetStatusAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerSetStatusAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    status: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerSetStatusAction}
 */
proto.ubo.v1.DockerSetStatusAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerSetStatusAction;
  return proto.ubo.v1.DockerSetStatusAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerSetStatusAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerSetStatusAction}
 */
proto.ubo.v1.DockerSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.DockerStatus} */ (reader.readEnum());
      msg.setStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerSetStatusAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerSetStatusAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerSetStatusAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerSetStatusAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getStatus();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerSetStatusAction} returns this
 */
proto.ubo.v1.DockerSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerSetStatusAction} returns this
 */
proto.ubo.v1.DockerSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional DockerStatus status = 2;
 * @return {!proto.ubo.v1.DockerStatus}
 */
proto.ubo.v1.DockerSetStatusAction.prototype.getStatus = function() {
  return /** @type {!proto.ubo.v1.DockerStatus} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.DockerStatus} value
 * @return {!proto.ubo.v1.DockerSetStatusAction} returns this
 */
proto.ubo.v1.DockerSetStatusAction.prototype.setStatus = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerStoreUsernameAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerStoreUsernameAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStoreUsernameAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    registry: jspb.Message.getFieldWithDefault(msg, 2, ""),
    username: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerStoreUsernameAction}
 */
proto.ubo.v1.DockerStoreUsernameAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerStoreUsernameAction;
  return proto.ubo.v1.DockerStoreUsernameAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerStoreUsernameAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerStoreUsernameAction}
 */
proto.ubo.v1.DockerStoreUsernameAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setRegistry(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setUsername(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerStoreUsernameAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerStoreUsernameAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStoreUsernameAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getRegistry();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getUsername();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string registry = 2;
 * @return {string}
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.getRegistry = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.setRegistry = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string username = 3;
 * @return {string}
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.getUsername = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerStoreUsernameAction} returns this
 */
proto.ubo.v1.DockerStoreUsernameAction.prototype.setUsername = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerRemoveUsernameAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerRemoveUsernameAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerRemoveUsernameAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    registry: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerRemoveUsernameAction}
 */
proto.ubo.v1.DockerRemoveUsernameAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerRemoveUsernameAction;
  return proto.ubo.v1.DockerRemoveUsernameAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerRemoveUsernameAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerRemoveUsernameAction}
 */
proto.ubo.v1.DockerRemoveUsernameAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setRegistry(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerRemoveUsernameAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerRemoveUsernameAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerRemoveUsernameAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getRegistry();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerRemoveUsernameAction} returns this
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerRemoveUsernameAction} returns this
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string registry = 2;
 * @return {string}
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.getRegistry = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerRemoveUsernameAction} returns this
 */
proto.ubo.v1.DockerRemoveUsernameAction.prototype.setRegistry = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageAction}
 */
proto.ubo.v1.DockerImageAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageAction;
  return proto.ubo.v1.DockerImageAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageAction}
 */
proto.ubo.v1.DockerImageAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageAction} returns this
 */
proto.ubo.v1.DockerImageAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageAction} returns this
 */
proto.ubo.v1.DockerImageAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageAction} returns this
 */
proto.ubo.v1.DockerImageAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageSetStatusAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageSetStatusAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageSetStatusAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    status: jspb.Message.getFieldWithDefault(msg, 2, 0),
    ports: (f = msg.getPorts()) && proto.ubo.v1.DockerImageSetStatusAction.Ports.toObject(includeInstance, f),
    ip: jspb.Message.getFieldWithDefault(msg, 4, ""),
    image: jspb.Message.getFieldWithDefault(msg, 5, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction}
 */
proto.ubo.v1.DockerImageSetStatusAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageSetStatusAction;
  return proto.ubo.v1.DockerImageSetStatusAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageSetStatusAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction}
 */
proto.ubo.v1.DockerImageSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.DockerItemStatus} */ (reader.readEnum());
      msg.setStatus(value);
      break;
    case 3:
      var value = new proto.ubo.v1.DockerImageSetStatusAction.Ports;
      reader.readMessage(value,proto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinaryFromReader);
      msg.setPorts(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setIp(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageSetStatusAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageSetStatusAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageSetStatusAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getStatus();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getPorts();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.DockerImageSetStatusAction.Ports.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageSetStatusAction.Ports.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageSetStatusAction.Ports} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports}
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageSetStatusAction.Ports;
  return proto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageSetStatusAction.Ports} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports}
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageSetStatusAction.Ports.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageSetStatusAction.Ports} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction.Ports} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.Ports.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional DockerItemStatus status = 2;
 * @return {!proto.ubo.v1.DockerItemStatus}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.getStatus = function() {
  return /** @type {!proto.ubo.v1.DockerItemStatus} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.DockerItemStatus} value
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.setStatus = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional Ports ports = 3;
 * @return {?proto.ubo.v1.DockerImageSetStatusAction.Ports}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.getPorts = function() {
  return /** @type{?proto.ubo.v1.DockerImageSetStatusAction.Ports} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageSetStatusAction.Ports, 3));
};


/**
 * @param {?proto.ubo.v1.DockerImageSetStatusAction.Ports|undefined} value
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
*/
proto.ubo.v1.DockerImageSetStatusAction.prototype.setPorts = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.clearPorts = function() {
  return this.setPorts(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.hasPorts = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string ip = 4;
 * @return {string}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.getIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.setIp = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.clearIp = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.hasIp = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string image = 5;
 * @return {string}
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageSetStatusAction} returns this
 */
proto.ubo.v1.DockerImageSetStatusAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 5, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageSetDockerIdAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageSetDockerIdAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageSetDockerIdAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    dockerId: jspb.Message.getFieldWithDefault(msg, 2, ""),
    image: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageSetDockerIdAction}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageSetDockerIdAction;
  return proto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageSetDockerIdAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageSetDockerIdAction}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setDockerId(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageSetDockerIdAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageSetDockerIdAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageSetDockerIdAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDockerId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string docker_id = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.getDockerId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.setDockerId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string image = 3;
 * @return {string}
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageSetDockerIdAction} returns this
 */
proto.ubo.v1.DockerImageSetDockerIdAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageUpdateMetadataAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageUpdateMetadataAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    instructions: jspb.Message.getFieldWithDefault(msg, 2, ""),
    image: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageUpdateMetadataAction;
  return proto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageUpdateMetadataAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setInstructions(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageUpdateMetadataAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageUpdateMetadataAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string instructions = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.getInstructions = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.setInstructions = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.clearInstructions = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.hasInstructions = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string image = 3;
 * @return {string}
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageUpdateMetadataAction} returns this
 */
proto.ubo.v1.DockerImageUpdateMetadataAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageFetchAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageFetchAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageFetchAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageFetchAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageFetchAction}
 */
proto.ubo.v1.DockerImageFetchAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageFetchAction;
  return proto.ubo.v1.DockerImageFetchAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageFetchAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageFetchAction}
 */
proto.ubo.v1.DockerImageFetchAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageFetchAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageFetchAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageFetchAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageFetchAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageFetchAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageFetchAction} returns this
 */
proto.ubo.v1.DockerImageFetchAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageFetchAction} returns this
 */
proto.ubo.v1.DockerImageFetchAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageFetchAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageFetchAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageFetchAction} returns this
 */
proto.ubo.v1.DockerImageFetchAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRemoveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRemoveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRemoveAction}
 */
proto.ubo.v1.DockerImageRemoveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRemoveAction;
  return proto.ubo.v1.DockerImageRemoveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRemoveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRemoveAction}
 */
proto.ubo.v1.DockerImageRemoveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRemoveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRemoveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveAction} returns this
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRemoveAction} returns this
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveAction} returns this
 */
proto.ubo.v1.DockerImageRemoveAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRunAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRunAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRunAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRunAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRunAction}
 */
proto.ubo.v1.DockerImageRunAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRunAction;
  return proto.ubo.v1.DockerImageRunAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRunAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRunAction}
 */
proto.ubo.v1.DockerImageRunAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRunAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRunAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRunAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRunAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRunAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRunAction} returns this
 */
proto.ubo.v1.DockerImageRunAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRunAction} returns this
 */
proto.ubo.v1.DockerImageRunAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRunAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRunAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRunAction} returns this
 */
proto.ubo.v1.DockerImageRunAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageStopAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageStopAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageStopAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageStopAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageStopAction}
 */
proto.ubo.v1.DockerImageStopAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageStopAction;
  return proto.ubo.v1.DockerImageStopAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageStopAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageStopAction}
 */
proto.ubo.v1.DockerImageStopAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageStopAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageStopAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageStopAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageStopAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageStopAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageStopAction} returns this
 */
proto.ubo.v1.DockerImageStopAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageStopAction} returns this
 */
proto.ubo.v1.DockerImageStopAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageStopAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageStopAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageStopAction} returns this
 */
proto.ubo.v1.DockerImageStopAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageReleaseAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageReleaseAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageReleaseAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageReleaseAction}
 */
proto.ubo.v1.DockerImageReleaseAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageReleaseAction;
  return proto.ubo.v1.DockerImageReleaseAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageReleaseAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageReleaseAction}
 */
proto.ubo.v1.DockerImageReleaseAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageReleaseAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageReleaseAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageReleaseAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageReleaseAction} returns this
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageReleaseAction} returns this
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageReleaseAction} returns this
 */
proto.ubo.v1.DockerImageReleaseAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRemoveContainerAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRemoveContainerAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveContainerAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRemoveContainerAction}
 */
proto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRemoveContainerAction;
  return proto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRemoveContainerAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRemoveContainerAction}
 */
proto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRemoveContainerAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRemoveContainerAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveContainerAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveContainerAction} returns this
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRemoveContainerAction} returns this
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveContainerAction} returns this
 */
proto.ubo.v1.DockerImageRemoveContainerAction.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerEvent}
 */
proto.ubo.v1.DockerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerEvent;
  return proto.ubo.v1.DockerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerEvent}
 */
proto.ubo.v1.DockerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerEvent} returns this
 */
proto.ubo.v1.DockerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerEvent} returns this
 */
proto.ubo.v1.DockerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerInstallEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerInstallEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerInstallEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerInstallEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerInstallEvent}
 */
proto.ubo.v1.DockerInstallEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerInstallEvent;
  return proto.ubo.v1.DockerInstallEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerInstallEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerInstallEvent}
 */
proto.ubo.v1.DockerInstallEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerInstallEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerInstallEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerInstallEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerInstallEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerInstallEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerInstallEvent} returns this
 */
proto.ubo.v1.DockerInstallEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerInstallEvent} returns this
 */
proto.ubo.v1.DockerInstallEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerInstallEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerStartEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerStartEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerStartEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStartEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerStartEvent}
 */
proto.ubo.v1.DockerStartEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerStartEvent;
  return proto.ubo.v1.DockerStartEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerStartEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerStartEvent}
 */
proto.ubo.v1.DockerStartEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerStartEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerStartEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerStartEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStartEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerStartEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerStartEvent} returns this
 */
proto.ubo.v1.DockerStartEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerStartEvent} returns this
 */
proto.ubo.v1.DockerStartEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerStartEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerStopEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerStopEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerStopEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStopEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerStopEvent}
 */
proto.ubo.v1.DockerStopEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerStopEvent;
  return proto.ubo.v1.DockerStopEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerStopEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerStopEvent}
 */
proto.ubo.v1.DockerStopEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerStopEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerStopEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerStopEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerStopEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerStopEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerStopEvent} returns this
 */
proto.ubo.v1.DockerStopEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerStopEvent} returns this
 */
proto.ubo.v1.DockerStopEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerStopEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerServiceState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerServiceState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerServiceState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerServiceState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    status: jspb.Message.getFieldWithDefault(msg, 2, 0),
    usernames: (f = msg.getUsernames()) && proto.ubo.v1.DockerServiceState.UsernamesDict.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerServiceState}
 */
proto.ubo.v1.DockerServiceState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerServiceState;
  return proto.ubo.v1.DockerServiceState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerServiceState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerServiceState}
 */
proto.ubo.v1.DockerServiceState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.DockerStatus} */ (reader.readEnum());
      msg.setStatus(value);
      break;
    case 3:
      var value = new proto.ubo.v1.DockerServiceState.UsernamesDict;
      reader.readMessage(value,proto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinaryFromReader);
      msg.setUsernames(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerServiceState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerServiceState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerServiceState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerServiceState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.DockerStatus} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getUsernames();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.DockerServiceState.UsernamesDict.serializeBinaryToWriter
    );
  }
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerServiceState.UsernamesDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerServiceState.UsernamesDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, undefined) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerServiceState.UsernamesDict}
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerServiceState.UsernamesDict;
  return proto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerServiceState.UsernamesDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerServiceState.UsernamesDict}
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readString, null, "", "");
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerServiceState.UsernamesDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerServiceState.UsernamesDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeString);
  }
};


/**
 * map<string, string> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,string>}
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,string>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      null));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.DockerServiceState.UsernamesDict} returns this
 */
proto.ubo.v1.DockerServiceState.UsernamesDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerServiceState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerServiceState} returns this
 */
proto.ubo.v1.DockerServiceState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerServiceState} returns this
 */
proto.ubo.v1.DockerServiceState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerServiceState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional DockerStatus status = 2;
 * @return {!proto.ubo.v1.DockerStatus}
 */
proto.ubo.v1.DockerServiceState.prototype.getStatus = function() {
  return /** @type {!proto.ubo.v1.DockerStatus} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.DockerStatus} value
 * @return {!proto.ubo.v1.DockerServiceState} returns this
 */
proto.ubo.v1.DockerServiceState.prototype.setStatus = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerServiceState} returns this
 */
proto.ubo.v1.DockerServiceState.prototype.clearStatus = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerServiceState.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional UsernamesDict usernames = 3;
 * @return {?proto.ubo.v1.DockerServiceState.UsernamesDict}
 */
proto.ubo.v1.DockerServiceState.prototype.getUsernames = function() {
  return /** @type{?proto.ubo.v1.DockerServiceState.UsernamesDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerServiceState.UsernamesDict, 3));
};


/**
 * @param {?proto.ubo.v1.DockerServiceState.UsernamesDict|undefined} value
 * @return {!proto.ubo.v1.DockerServiceState} returns this
*/
proto.ubo.v1.DockerServiceState.prototype.setUsernames = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.DockerServiceState} returns this
 */
proto.ubo.v1.DockerServiceState.prototype.clearUsernames = function() {
  return this.setUsernames(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerServiceState.prototype.hasUsernames = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageEvent}
 */
proto.ubo.v1.DockerImageEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageEvent;
  return proto.ubo.v1.DockerImageEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageEvent}
 */
proto.ubo.v1.DockerImageEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageEvent} returns this
 */
proto.ubo.v1.DockerImageEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageEvent} returns this
 */
proto.ubo.v1.DockerImageEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageEvent} returns this
 */
proto.ubo.v1.DockerImageEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRegisterAppEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRegisterAppEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRegisterAppEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRegisterAppEvent}
 */
proto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRegisterAppEvent;
  return proto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRegisterAppEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRegisterAppEvent}
 */
proto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRegisterAppEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRegisterAppEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRegisterAppEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRegisterAppEvent} returns this
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRegisterAppEvent} returns this
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRegisterAppEvent} returns this
 */
proto.ubo.v1.DockerImageRegisterAppEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageFetchCompositionEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageFetchCompositionEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent}
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageFetchCompositionEvent;
  return proto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageFetchCompositionEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent}
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageFetchCompositionEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageFetchCompositionEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageFetchCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageFetchCompositionEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageFetchEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageFetchEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageFetchEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageFetchEvent}
 */
proto.ubo.v1.DockerImageFetchEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageFetchEvent;
  return proto.ubo.v1.DockerImageFetchEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageFetchEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageFetchEvent}
 */
proto.ubo.v1.DockerImageFetchEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageFetchEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageFetchEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageFetchEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageFetchEvent} returns this
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageFetchEvent} returns this
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageFetchEvent} returns this
 */
proto.ubo.v1.DockerImageFetchEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRemoveCompositionEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRemoveCompositionEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent}
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRemoveCompositionEvent;
  return proto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRemoveCompositionEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent}
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRemoveCompositionEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRemoveCompositionEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveCompositionEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRemoveEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRemoveEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRemoveEvent}
 */
proto.ubo.v1.DockerImageRemoveEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRemoveEvent;
  return proto.ubo.v1.DockerImageRemoveEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRemoveEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRemoveEvent}
 */
proto.ubo.v1.DockerImageRemoveEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRemoveEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRemoveEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRemoveEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRunCompositionEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRunCompositionEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRunCompositionEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRunCompositionEvent}
 */
proto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRunCompositionEvent;
  return proto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRunCompositionEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRunCompositionEvent}
 */
proto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRunCompositionEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRunCompositionEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRunCompositionEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRunCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRunCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRunCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageRunCompositionEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRunContainerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRunContainerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRunContainerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRunContainerEvent}
 */
proto.ubo.v1.DockerImageRunContainerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRunContainerEvent;
  return proto.ubo.v1.DockerImageRunContainerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRunContainerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRunContainerEvent}
 */
proto.ubo.v1.DockerImageRunContainerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRunContainerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRunContainerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRunContainerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRunContainerEvent} returns this
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRunContainerEvent} returns this
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRunContainerEvent} returns this
 */
proto.ubo.v1.DockerImageRunContainerEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageStopCompositionEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageStopCompositionEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageStopCompositionEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageStopCompositionEvent}
 */
proto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageStopCompositionEvent;
  return proto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageStopCompositionEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageStopCompositionEvent}
 */
proto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageStopCompositionEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageStopCompositionEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageStopCompositionEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageStopCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageStopCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageStopCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageStopCompositionEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageStopContainerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageStopContainerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageStopContainerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageStopContainerEvent}
 */
proto.ubo.v1.DockerImageStopContainerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageStopContainerEvent;
  return proto.ubo.v1.DockerImageStopContainerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageStopContainerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageStopContainerEvent}
 */
proto.ubo.v1.DockerImageStopContainerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageStopContainerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageStopContainerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageStopContainerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageStopContainerEvent} returns this
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageStopContainerEvent} returns this
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageStopContainerEvent} returns this
 */
proto.ubo.v1.DockerImageStopContainerEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageReleaseCompositionEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageReleaseCompositionEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent}
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageReleaseCompositionEvent;
  return proto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageReleaseCompositionEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent}
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageReleaseCompositionEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageReleaseCompositionEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageReleaseCompositionEvent} returns this
 */
proto.ubo.v1.DockerImageReleaseCompositionEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerImageRemoveContainerEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerImageRemoveContainerEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    image: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent}
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerImageRemoveContainerEvent;
  return proto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerImageRemoveContainerEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent}
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setImage(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerImageRemoveContainerEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerImageRemoveContainerEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getImage();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string image = 2;
 * @return {string}
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.getImage = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerImageRemoveContainerEvent} returns this
 */
proto.ubo.v1.DockerImageRemoveContainerEvent.prototype.setImage = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ImageState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ImageState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ImageState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ImageState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, ""),
    label: jspb.Message.getFieldWithDefault(msg, 3, ""),
    instructions: jspb.Message.getFieldWithDefault(msg, 4, ""),
    status: jspb.Message.getFieldWithDefault(msg, 5, 0),
    containerIp: jspb.Message.getFieldWithDefault(msg, 6, ""),
    dockerId: jspb.Message.getFieldWithDefault(msg, 7, ""),
    ports: (f = msg.getPorts()) && proto.ubo.v1.ImageState.Ports.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ImageState}
 */
proto.ubo.v1.ImageState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ImageState;
  return proto.ubo.v1.ImageState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ImageState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ImageState}
 */
proto.ubo.v1.ImageState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setInstructions(value);
      break;
    case 5:
      var value = /** @type {!proto.ubo.v1.DockerItemStatus} */ (reader.readEnum());
      msg.setStatus(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setContainerIp(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setDockerId(value);
      break;
    case 8:
      var value = new proto.ubo.v1.ImageState.Ports;
      reader.readMessage(value,proto.ubo.v1.ImageState.Ports.deserializeBinaryFromReader);
      msg.setPorts(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ImageState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ImageState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ImageState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ImageState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getLabel();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.DockerItemStatus} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeEnum(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = message.getPorts();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.ubo.v1.ImageState.Ports.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.ImageState.Ports.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ImageState.Ports.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ImageState.Ports.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ImageState.Ports} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ImageState.Ports.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ImageState.Ports}
 */
proto.ubo.v1.ImageState.Ports.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ImageState.Ports;
  return proto.ubo.v1.ImageState.Ports.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ImageState.Ports} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ImageState.Ports}
 */
proto.ubo.v1.ImageState.Ports.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ImageState.Ports.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ImageState.Ports.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ImageState.Ports} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ImageState.Ports.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.ImageState.Ports.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.ImageState.Ports} returns this
 */
proto.ubo.v1.ImageState.Ports.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.ImageState.Ports} returns this
 */
proto.ubo.v1.ImageState.Ports.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.ImageState.Ports} returns this
 */
proto.ubo.v1.ImageState.Ports.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.ImageState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ImageState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.ImageState.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string label = 3;
 * @return {string}
 */
proto.ubo.v1.ImageState.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.setLabel = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional string instructions = 4;
 * @return {string}
 */
proto.ubo.v1.ImageState.prototype.getInstructions = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.setInstructions = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.clearInstructions = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ImageState.prototype.hasInstructions = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional DockerItemStatus status = 5;
 * @return {!proto.ubo.v1.DockerItemStatus}
 */
proto.ubo.v1.ImageState.prototype.getStatus = function() {
  return /** @type {!proto.ubo.v1.DockerItemStatus} */ (jspb.Message.getFieldWithDefault(this, 5, 0));
};


/**
 * @param {!proto.ubo.v1.DockerItemStatus} value
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.setStatus = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.clearStatus = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ImageState.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string container_ip = 6;
 * @return {string}
 */
proto.ubo.v1.ImageState.prototype.getContainerIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.setContainerIp = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.clearContainerIp = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ImageState.prototype.hasContainerIp = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string docker_id = 7;
 * @return {string}
 */
proto.ubo.v1.ImageState.prototype.getDockerId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.setDockerId = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.clearDockerId = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ImageState.prototype.hasDockerId = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional Ports ports = 8;
 * @return {?proto.ubo.v1.ImageState.Ports}
 */
proto.ubo.v1.ImageState.prototype.getPorts = function() {
  return /** @type{?proto.ubo.v1.ImageState.Ports} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ImageState.Ports, 8));
};


/**
 * @param {?proto.ubo.v1.ImageState.Ports|undefined} value
 * @return {!proto.ubo.v1.ImageState} returns this
*/
proto.ubo.v1.ImageState.prototype.setPorts = function(value) {
  return jspb.Message.setWrapperField(this, 8, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.ImageState} returns this
 */
proto.ubo.v1.ImageState.prototype.clearPorts = function() {
  return this.setPorts(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ImageState.prototype.hasPorts = function() {
  return jspb.Message.getField(this, 8) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.DockerState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.DockerState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.DockerState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotDocker: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    service: (f = msg.getService()) && proto.ubo.v1.DockerServiceState.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.DockerState}
 */
proto.ubo.v1.DockerState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.DockerState;
  return proto.ubo.v1.DockerState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.DockerState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.DockerState}
 */
proto.ubo.v1.DockerState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker(value);
      break;
    case 2:
      var value = new proto.ubo.v1.DockerServiceState;
      reader.readMessage(value,proto.ubo.v1.DockerServiceState.deserializeBinaryFromReader);
      msg.setService(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.DockerState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.DockerState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.DockerState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.DockerState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getService();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.DockerServiceState.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_docker = 1000;
 * @return {string}
 */
proto.ubo.v1.DockerState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.DockerState} returns this
 */
proto.ubo.v1.DockerState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.DockerState} returns this
 */
proto.ubo.v1.DockerState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotDocker = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional DockerServiceState service = 2;
 * @return {?proto.ubo.v1.DockerServiceState}
 */
proto.ubo.v1.DockerState.prototype.getService = function() {
  return /** @type{?proto.ubo.v1.DockerServiceState} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerServiceState, 2));
};


/**
 * @param {?proto.ubo.v1.DockerServiceState|undefined} value
 * @return {!proto.ubo.v1.DockerState} returns this
*/
proto.ubo.v1.DockerState.prototype.setService = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.DockerState} returns this
 */
proto.ubo.v1.DockerState.prototype.clearService = function() {
  return this.setService(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.DockerState.prototype.hasService = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.PathSelectorConfig.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.PathSelectorConfig.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.PathSelectorConfig} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PathSelectorConfig.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    initialPath: jspb.Message.getFieldWithDefault(msg, 2, ""),
    showHidden: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    acceptsFiles: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    acceptsDirectories: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    acceptableSuffixes: (f = msg.getAcceptableSuffixes()) && proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.PathSelectorConfig}
 */
proto.ubo.v1.PathSelectorConfig.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.PathSelectorConfig;
  return proto.ubo.v1.PathSelectorConfig.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.PathSelectorConfig} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.PathSelectorConfig}
 */
proto.ubo.v1.PathSelectorConfig.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setInitialPath(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShowHidden(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setAcceptsFiles(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setAcceptsDirectories(value);
      break;
    case 6:
      var value = new proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes;
      reader.readMessage(value,proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinaryFromReader);
      msg.setAcceptableSuffixes(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.PathSelectorConfig.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.PathSelectorConfig.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.PathSelectorConfig} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PathSelectorConfig.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = message.getAcceptableSuffixes();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes}
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes;
  return proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes}
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} returns this
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} returns this
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} returns this
 */
proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.PathSelectorConfig.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string initial_path = 2;
 * @return {string}
 */
proto.ubo.v1.PathSelectorConfig.prototype.getInitialPath = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.setInitialPath = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.clearInitialPath = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.hasInitialPath = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool show_hidden = 3;
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.getShowHidden = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.setShowHidden = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.clearShowHidden = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.hasShowHidden = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool accepts_files = 4;
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.getAcceptsFiles = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.setAcceptsFiles = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.clearAcceptsFiles = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.hasAcceptsFiles = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool accepts_directories = 5;
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.getAcceptsDirectories = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.setAcceptsDirectories = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.clearAcceptsDirectories = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.hasAcceptsDirectories = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional AcceptableSuffixes acceptable_suffixes = 6;
 * @return {?proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes}
 */
proto.ubo.v1.PathSelectorConfig.prototype.getAcceptableSuffixes = function() {
  return /** @type{?proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes, 6));
};


/**
 * @param {?proto.ubo.v1.PathSelectorConfig.AcceptableSuffixes|undefined} value
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
*/
proto.ubo.v1.PathSelectorConfig.prototype.setAcceptableSuffixes = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.PathSelectorConfig} returns this
 */
proto.ubo.v1.PathSelectorConfig.prototype.clearAcceptableSuffixes = function() {
  return this.setAcceptableSuffixes(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.PathSelectorConfig.prototype.hasAcceptableSuffixes = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemAction}
 */
proto.ubo.v1.FileSystemAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemAction;
  return proto.ubo.v1.FileSystemAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemAction}
 */
proto.ubo.v1.FileSystemAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemAction} returns this
 */
proto.ubo.v1.FileSystemAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemAction} returns this
 */
proto.ubo.v1.FileSystemAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemReportSelectionAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemReportSelectionAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemReportSelectionAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    path: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemReportSelectionAction}
 */
proto.ubo.v1.FileSystemReportSelectionAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemReportSelectionAction;
  return proto.ubo.v1.FileSystemReportSelectionAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemReportSelectionAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemReportSelectionAction}
 */
proto.ubo.v1.FileSystemReportSelectionAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setPath(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemReportSelectionAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemReportSelectionAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemReportSelectionAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getPath();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemReportSelectionAction} returns this
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemReportSelectionAction} returns this
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string path = 2;
 * @return {string}
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.getPath = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemReportSelectionAction} returns this
 */
proto.ubo.v1.FileSystemReportSelectionAction.prototype.setPath = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.FileSystemCopyAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemCopyAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemCopyAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemCopyAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemCopyAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    destination: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemCopyAction}
 */
proto.ubo.v1.FileSystemCopyAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemCopyAction;
  return proto.ubo.v1.FileSystemCopyAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemCopyAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemCopyAction}
 */
proto.ubo.v1.FileSystemCopyAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addSources(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setDestination(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemCopyAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemCopyAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemCopyAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemCopyAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSourcesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
  f = message.getDestination();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemCopyAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemCopyAction} returns this
 */
proto.ubo.v1.FileSystemCopyAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemCopyAction} returns this
 */
proto.ubo.v1.FileSystemCopyAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemCopyAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string sources = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.FileSystemCopyAction.prototype.getSourcesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.FileSystemCopyAction} returns this
 */
proto.ubo.v1.FileSystemCopyAction.prototype.setSourcesList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.FileSystemCopyAction} returns this
 */
proto.ubo.v1.FileSystemCopyAction.prototype.addSources = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.FileSystemCopyAction} returns this
 */
proto.ubo.v1.FileSystemCopyAction.prototype.clearSourcesList = function() {
  return this.setSourcesList([]);
};


/**
 * optional string destination = 3;
 * @return {string}
 */
proto.ubo.v1.FileSystemCopyAction.prototype.getDestination = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemCopyAction} returns this
 */
proto.ubo.v1.FileSystemCopyAction.prototype.setDestination = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.FileSystemMoveAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemMoveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemMoveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemMoveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemMoveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    destination: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemMoveAction}
 */
proto.ubo.v1.FileSystemMoveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemMoveAction;
  return proto.ubo.v1.FileSystemMoveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemMoveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemMoveAction}
 */
proto.ubo.v1.FileSystemMoveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addSources(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setDestination(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemMoveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemMoveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemMoveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemMoveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSourcesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
  f = message.getDestination();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemMoveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemMoveAction} returns this
 */
proto.ubo.v1.FileSystemMoveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemMoveAction} returns this
 */
proto.ubo.v1.FileSystemMoveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemMoveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string sources = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.FileSystemMoveAction.prototype.getSourcesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.FileSystemMoveAction} returns this
 */
proto.ubo.v1.FileSystemMoveAction.prototype.setSourcesList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.FileSystemMoveAction} returns this
 */
proto.ubo.v1.FileSystemMoveAction.prototype.addSources = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.FileSystemMoveAction} returns this
 */
proto.ubo.v1.FileSystemMoveAction.prototype.clearSourcesList = function() {
  return this.setSourcesList([]);
};


/**
 * optional string destination = 3;
 * @return {string}
 */
proto.ubo.v1.FileSystemMoveAction.prototype.getDestination = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemMoveAction} returns this
 */
proto.ubo.v1.FileSystemMoveAction.prototype.setDestination = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.FileSystemRemoveAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemRemoveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemRemoveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemRemoveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    pathsList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemRemoveAction}
 */
proto.ubo.v1.FileSystemRemoveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemRemoveAction;
  return proto.ubo.v1.FileSystemRemoveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemRemoveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemRemoveAction}
 */
proto.ubo.v1.FileSystemRemoveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addPaths(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemRemoveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemRemoveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemRemoveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getPathsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string paths = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.getPathsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.setPathsList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.addPaths = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.FileSystemRemoveAction} returns this
 */
proto.ubo.v1.FileSystemRemoveAction.prototype.clearPathsList = function() {
  return this.setPathsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemEvent}
 */
proto.ubo.v1.FileSystemEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemEvent;
  return proto.ubo.v1.FileSystemEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemEvent}
 */
proto.ubo.v1.FileSystemEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemEvent} returns this
 */
proto.ubo.v1.FileSystemEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemEvent} returns this
 */
proto.ubo.v1.FileSystemEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemSelectEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemSelectEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemSelectEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    description: (f = msg.getDescription()) && proto.ubo.v1.PathInputDescription.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemSelectEvent}
 */
proto.ubo.v1.FileSystemSelectEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemSelectEvent;
  return proto.ubo.v1.FileSystemSelectEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemSelectEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemSelectEvent}
 */
proto.ubo.v1.FileSystemSelectEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = new proto.ubo.v1.PathInputDescription;
      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);
      msg.setDescription(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemSelectEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemSelectEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemSelectEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDescription();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional PathInputDescription description = 2;
 * @return {?proto.ubo.v1.PathInputDescription}
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.getDescription = function() {
  return /** @type{?proto.ubo.v1.PathInputDescription} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PathInputDescription, 2));
};


/**
 * @param {?proto.ubo.v1.PathInputDescription|undefined} value
 * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this
*/
proto.ubo.v1.FileSystemSelectEvent.prototype.setDescription = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.FileSystemSelectEvent} returns this
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.clearDescription = function() {
  return this.setDescription(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemSelectEvent.prototype.hasDescription = function() {
  return jspb.Message.getField(this, 2) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.FileSystemCopyEvent.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemCopyEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemCopyEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemCopyEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    destination: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemCopyEvent}
 */
proto.ubo.v1.FileSystemCopyEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemCopyEvent;
  return proto.ubo.v1.FileSystemCopyEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemCopyEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemCopyEvent}
 */
proto.ubo.v1.FileSystemCopyEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addSources(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setDestination(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemCopyEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemCopyEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemCopyEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSourcesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
  f = message.getDestination();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string sources = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.getSourcesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.setSourcesList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.addSources = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.clearSourcesList = function() {
  return this.setSourcesList([]);
};


/**
 * optional string destination = 3;
 * @return {string}
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.getDestination = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemCopyEvent} returns this
 */
proto.ubo.v1.FileSystemCopyEvent.prototype.setDestination = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.FileSystemMoveEvent.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemMoveEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemMoveEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemMoveEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sourcesList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f,
    destination: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemMoveEvent}
 */
proto.ubo.v1.FileSystemMoveEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemMoveEvent;
  return proto.ubo.v1.FileSystemMoveEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemMoveEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemMoveEvent}
 */
proto.ubo.v1.FileSystemMoveEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addSources(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setDestination(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemMoveEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemMoveEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemMoveEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSourcesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
  f = message.getDestination();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string sources = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.getSourcesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.setSourcesList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.addSources = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.clearSourcesList = function() {
  return this.setSourcesList([]);
};


/**
 * optional string destination = 3;
 * @return {string}
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.getDestination = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemMoveEvent} returns this
 */
proto.ubo.v1.FileSystemMoveEvent.prototype.setDestination = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.FileSystemRemoveEvent.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemRemoveEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemRemoveEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemRemoveEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    pathsList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemRemoveEvent}
 */
proto.ubo.v1.FileSystemRemoveEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemRemoveEvent;
  return proto.ubo.v1.FileSystemRemoveEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemRemoveEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemRemoveEvent}
 */
proto.ubo.v1.FileSystemRemoveEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addPaths(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemRemoveEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemRemoveEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemRemoveEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getPathsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string paths = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.getPathsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.setPathsList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.addPaths = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.FileSystemRemoveEvent} returns this
 */
proto.ubo.v1.FileSystemRemoveEvent.prototype.clearPathsList = function() {
  return this.setPathsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.FileSystemState.repeatedFields_ = [3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.FileSystemState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.FileSystemState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.FileSystemState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    currentInput: (f = msg.getCurrentInput()) && proto.ubo.v1.PathInputDescription.toObject(includeInstance, f),
    queueList: jspb.Message.toObjectList(msg.getQueueList(),
    proto.ubo.v1.PathInputDescription.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.FileSystemState}
 */
proto.ubo.v1.FileSystemState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.FileSystemState;
  return proto.ubo.v1.FileSystemState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.FileSystemState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.FileSystemState}
 */
proto.ubo.v1.FileSystemState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem(value);
      break;
    case 2:
      var value = new proto.ubo.v1.PathInputDescription;
      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);
      msg.setCurrentInput(value);
      break;
    case 3:
      var value = new proto.ubo.v1.PathInputDescription;
      reader.readMessage(value,proto.ubo.v1.PathInputDescription.deserializeBinaryFromReader);
      msg.addQueue(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.FileSystemState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.FileSystemState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.FileSystemState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.FileSystemState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getCurrentInput();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter
    );
  }
  f = message.getQueueList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.ubo.v1.PathInputDescription.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_file_system = 1000;
 * @return {string}
 */
proto.ubo.v1.FileSystemState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.FileSystemState} returns this
 */
proto.ubo.v1.FileSystemState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.FileSystemState} returns this
 */
proto.ubo.v1.FileSystemState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotFileSystem = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional PathInputDescription current_input = 2;
 * @return {?proto.ubo.v1.PathInputDescription}
 */
proto.ubo.v1.FileSystemState.prototype.getCurrentInput = function() {
  return /** @type{?proto.ubo.v1.PathInputDescription} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PathInputDescription, 2));
};


/**
 * @param {?proto.ubo.v1.PathInputDescription|undefined} value
 * @return {!proto.ubo.v1.FileSystemState} returns this
*/
proto.ubo.v1.FileSystemState.prototype.setCurrentInput = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.FileSystemState} returns this
 */
proto.ubo.v1.FileSystemState.prototype.clearCurrentInput = function() {
  return this.setCurrentInput(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.FileSystemState.prototype.hasCurrentInput = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated PathInputDescription queue = 3;
 * @return {!Array<!proto.ubo.v1.PathInputDescription>}
 */
proto.ubo.v1.FileSystemState.prototype.getQueueList = function() {
  return /** @type{!Array<!proto.ubo.v1.PathInputDescription>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.PathInputDescription, 3));
};


/**
 * @param {!Array<!proto.ubo.v1.PathInputDescription>} value
 * @return {!proto.ubo.v1.FileSystemState} returns this
*/
proto.ubo.v1.FileSystemState.prototype.setQueueList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.ubo.v1.PathInputDescription=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.PathInputDescription}
 */
proto.ubo.v1.FileSystemState.prototype.addQueue = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.PathInputDescription, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.FileSystemState} returns this
 */
proto.ubo.v1.FileSystemState.prototype.clearQueueList = function() {
  return this.setQueueList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredAction}
 */
proto.ubo.v1.InfraredAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredAction;
  return proto.ubo.v1.InfraredAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredAction}
 */
proto.ubo.v1.InfraredAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredAction} returns this
 */
proto.ubo.v1.InfraredAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredAction} returns this
 */
proto.ubo.v1.InfraredAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredEvent}
 */
proto.ubo.v1.InfraredEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredEvent;
  return proto.ubo.v1.InfraredEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredEvent}
 */
proto.ubo.v1.InfraredEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredEvent} returns this
 */
proto.ubo.v1.InfraredEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredEvent} returns this
 */
proto.ubo.v1.InfraredEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredHandleReceivedCodeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredHandleReceivedCodeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    protocol: jspb.Message.getFieldWithDefault(msg, 2, ""),
    scancode: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredHandleReceivedCodeAction;
  return proto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredHandleReceivedCodeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setProtocol(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setScancode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredHandleReceivedCodeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredHandleReceivedCodeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getProtocol();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getScancode();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string protocol = 2;
 * @return {string}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.getProtocol = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.setProtocol = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string scancode = 3;
 * @return {string}
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.getScancode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredHandleReceivedCodeAction} returns this
 */
proto.ubo.v1.InfraredHandleReceivedCodeAction.prototype.setScancode = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredSendCodeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredSendCodeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSendCodeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    protocol: jspb.Message.getFieldWithDefault(msg, 2, ""),
    scancode: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredSendCodeAction}
 */
proto.ubo.v1.InfraredSendCodeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredSendCodeAction;
  return proto.ubo.v1.InfraredSendCodeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredSendCodeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredSendCodeAction}
 */
proto.ubo.v1.InfraredSendCodeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setProtocol(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setScancode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredSendCodeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredSendCodeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSendCodeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getProtocol();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getScancode();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string protocol = 2;
 * @return {string}
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.getProtocol = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.setProtocol = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string scancode = 3;
 * @return {string}
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.getScancode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSendCodeAction} returns this
 */
proto.ubo.v1.InfraredSendCodeAction.prototype.setScancode = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredSetShouldPropagateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredSetShouldPropagateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    shouldPropagate: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction}
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredSetShouldPropagateAction;
  return proto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredSetShouldPropagateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction}
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShouldPropagate(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredSetShouldPropagateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredSetShouldPropagateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getShouldPropagate();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction} returns this
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction} returns this
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool should_propagate = 2;
 * @return {boolean}
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.getShouldPropagate = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.InfraredSetShouldPropagateAction} returns this
 */
proto.ubo.v1.InfraredSetShouldPropagateAction.prototype.setShouldPropagate = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredSetShouldReceiveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredSetShouldReceiveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    shouldReceive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction}
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredSetShouldReceiveAction;
  return proto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredSetShouldReceiveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction}
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShouldReceive(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredSetShouldReceiveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredSetShouldReceiveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getShouldReceive();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction} returns this
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction} returns this
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool should_receive = 2;
 * @return {boolean}
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.getShouldReceive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.InfraredSetShouldReceiveAction} returns this
 */
proto.ubo.v1.InfraredSetShouldReceiveAction.prototype.setShouldReceive = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredSendCodeEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredSendCodeEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSendCodeEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    protocol: jspb.Message.getFieldWithDefault(msg, 2, ""),
    scancode: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredSendCodeEvent}
 */
proto.ubo.v1.InfraredSendCodeEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredSendCodeEvent;
  return proto.ubo.v1.InfraredSendCodeEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredSendCodeEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredSendCodeEvent}
 */
proto.ubo.v1.InfraredSendCodeEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setProtocol(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setScancode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredSendCodeEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredSendCodeEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredSendCodeEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getProtocol();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getScancode();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string protocol = 2;
 * @return {string}
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.getProtocol = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.setProtocol = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string scancode = 3;
 * @return {string}
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.getScancode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredSendCodeEvent} returns this
 */
proto.ubo.v1.InfraredSendCodeEvent.prototype.setScancode = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.InfraredState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.InfraredState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.InfraredState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotInfrared: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    shouldPropagateKeypadActions: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    shouldReceiveKeypadActions: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.InfraredState}
 */
proto.ubo.v1.InfraredState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.InfraredState;
  return proto.ubo.v1.InfraredState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.InfraredState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.InfraredState}
 */
proto.ubo.v1.InfraredState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShouldPropagateKeypadActions(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShouldReceiveKeypadActions(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.InfraredState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.InfraredState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.InfraredState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.InfraredState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_infrared = 1000;
 * @return {string}
 */
proto.ubo.v1.InfraredState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.InfraredState} returns this
 */
proto.ubo.v1.InfraredState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredState} returns this
 */
proto.ubo.v1.InfraredState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotInfrared = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool should_propagate_keypad_actions = 2;
 * @return {boolean}
 */
proto.ubo.v1.InfraredState.prototype.getShouldPropagateKeypadActions = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.InfraredState} returns this
 */
proto.ubo.v1.InfraredState.prototype.setShouldPropagateKeypadActions = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredState} returns this
 */
proto.ubo.v1.InfraredState.prototype.clearShouldPropagateKeypadActions = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredState.prototype.hasShouldPropagateKeypadActions = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool should_receive_keypad_actions = 3;
 * @return {boolean}
 */
proto.ubo.v1.InfraredState.prototype.getShouldReceiveKeypadActions = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.InfraredState} returns this
 */
proto.ubo.v1.InfraredState.prototype.setShouldReceiveKeypadActions = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.InfraredState} returns this
 */
proto.ubo.v1.InfraredState.prototype.clearShouldReceiveKeypadActions = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.InfraredState.prototype.hasShouldReceiveKeypadActions = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.IpAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.IpAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.IpAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.IpAction}
 */
proto.ubo.v1.IpAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.IpAction;
  return proto.ubo.v1.IpAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.IpAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.IpAction}
 */
proto.ubo.v1.IpAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.IpAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.IpAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.IpAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;
 * @return {string}
 */
proto.ubo.v1.IpAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IpAction} returns this
 */
proto.ubo.v1.IpAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IpAction} returns this
 */
proto.ubo.v1.IpAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IpAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.IpEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.IpEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.IpEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.IpEvent}
 */
proto.ubo.v1.IpEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.IpEvent;
  return proto.ubo.v1.IpEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.IpEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.IpEvent}
 */
proto.ubo.v1.IpEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.IpEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.IpEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.IpEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;
 * @return {string}
 */
proto.ubo.v1.IpEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IpEvent} returns this
 */
proto.ubo.v1.IpEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IpEvent} returns this
 */
proto.ubo.v1.IpEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IpEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.IpUpdateInterfacesAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.IpUpdateInterfacesAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.IpUpdateInterfacesAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpUpdateInterfacesAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    interfacesList: jspb.Message.toObjectList(msg.getInterfacesList(),
    proto.ubo.v1.IpNetworkInterface.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.IpUpdateInterfacesAction}
 */
proto.ubo.v1.IpUpdateInterfacesAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.IpUpdateInterfacesAction;
  return proto.ubo.v1.IpUpdateInterfacesAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.IpUpdateInterfacesAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.IpUpdateInterfacesAction}
 */
proto.ubo.v1.IpUpdateInterfacesAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);
      break;
    case 2:
      var value = new proto.ubo.v1.IpNetworkInterface;
      reader.readMessage(value,proto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader);
      msg.addInterfaces(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.IpUpdateInterfacesAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.IpUpdateInterfacesAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpUpdateInterfacesAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getInterfacesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;
 * @return {string}
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated IpNetworkInterface interfaces = 2;
 * @return {!Array<!proto.ubo.v1.IpNetworkInterface>}
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.getInterfacesList = function() {
  return /** @type{!Array<!proto.ubo.v1.IpNetworkInterface>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.IpNetworkInterface, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.IpNetworkInterface>} value
 * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this
*/
proto.ubo.v1.IpUpdateInterfacesAction.prototype.setInterfacesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.IpNetworkInterface=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.IpNetworkInterface}
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.addInterfaces = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.IpNetworkInterface, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.IpUpdateInterfacesAction} returns this
 */
proto.ubo.v1.IpUpdateInterfacesAction.prototype.clearInterfacesList = function() {
  return this.setInterfacesList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.IpSetIsConnectedAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.IpSetIsConnectedAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpSetIsConnectedAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isConnected: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.IpSetIsConnectedAction}
 */
proto.ubo.v1.IpSetIsConnectedAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.IpSetIsConnectedAction;
  return proto.ubo.v1.IpSetIsConnectedAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.IpSetIsConnectedAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.IpSetIsConnectedAction}
 */
proto.ubo.v1.IpSetIsConnectedAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsConnected(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.IpSetIsConnectedAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.IpSetIsConnectedAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpSetIsConnectedAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsConnected();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;
 * @return {string}
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IpSetIsConnectedAction} returns this
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IpSetIsConnectedAction} returns this
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_connected = 2;
 * @return {boolean}
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.getIsConnected = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.IpSetIsConnectedAction} returns this
 */
proto.ubo.v1.IpSetIsConnectedAction.prototype.setIsConnected = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.IpNetworkInterface.repeatedFields_ = [3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.IpNetworkInterface.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.IpNetworkInterface.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.IpNetworkInterface} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpNetworkInterface.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    name: jspb.Message.getFieldWithDefault(msg, 2, ""),
    ipAddressesList: (f = jspb.Message.getRepeatedField(msg, 3)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.IpNetworkInterface}
 */
proto.ubo.v1.IpNetworkInterface.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.IpNetworkInterface;
  return proto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.IpNetworkInterface} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.IpNetworkInterface}
 */
proto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.addIpAddresses(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.IpNetworkInterface.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.IpNetworkInterface} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getName();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getIpAddressesList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;
 * @return {string}
 */
proto.ubo.v1.IpNetworkInterface.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IpNetworkInterface} returns this
 */
proto.ubo.v1.IpNetworkInterface.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IpNetworkInterface} returns this
 */
proto.ubo.v1.IpNetworkInterface.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IpNetworkInterface.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string name = 2;
 * @return {string}
 */
proto.ubo.v1.IpNetworkInterface.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IpNetworkInterface} returns this
 */
proto.ubo.v1.IpNetworkInterface.prototype.setName = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * repeated string ip_addresses = 3;
 * @return {!Array<string>}
 */
proto.ubo.v1.IpNetworkInterface.prototype.getIpAddressesList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 3));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.IpNetworkInterface} returns this
 */
proto.ubo.v1.IpNetworkInterface.prototype.setIpAddressesList = function(value) {
  return jspb.Message.setField(this, 3, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.IpNetworkInterface} returns this
 */
proto.ubo.v1.IpNetworkInterface.prototype.addIpAddresses = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 3, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.IpNetworkInterface} returns this
 */
proto.ubo.v1.IpNetworkInterface.prototype.clearIpAddressesList = function() {
  return this.setIpAddressesList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.IpState.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.IpState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.IpState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.IpState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotIp: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    interfacesList: jspb.Message.toObjectList(msg.getInterfacesList(),
    proto.ubo.v1.IpNetworkInterface.toObject, includeInstance),
    isConnected: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.IpState}
 */
proto.ubo.v1.IpState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.IpState;
  return proto.ubo.v1.IpState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.IpState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.IpState}
 */
proto.ubo.v1.IpState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp(value);
      break;
    case 2:
      var value = new proto.ubo.v1.IpNetworkInterface;
      reader.readMessage(value,proto.ubo.v1.IpNetworkInterface.deserializeBinaryFromReader);
      msg.addInterfaces(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsConnected(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.IpState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.IpState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.IpState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.IpState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getInterfacesList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.IpNetworkInterface.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ip = 1000;
 * @return {string}
 */
proto.ubo.v1.IpState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.IpState} returns this
 */
proto.ubo.v1.IpState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IpState} returns this
 */
proto.ubo.v1.IpState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IpState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotIp = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated IpNetworkInterface interfaces = 2;
 * @return {!Array<!proto.ubo.v1.IpNetworkInterface>}
 */
proto.ubo.v1.IpState.prototype.getInterfacesList = function() {
  return /** @type{!Array<!proto.ubo.v1.IpNetworkInterface>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.IpNetworkInterface, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.IpNetworkInterface>} value
 * @return {!proto.ubo.v1.IpState} returns this
*/
proto.ubo.v1.IpState.prototype.setInterfacesList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.IpNetworkInterface=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.IpNetworkInterface}
 */
proto.ubo.v1.IpState.prototype.addInterfaces = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.IpNetworkInterface, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.IpState} returns this
 */
proto.ubo.v1.IpState.prototype.clearInterfacesList = function() {
  return this.setInterfacesList([]);
};


/**
 * optional bool is_connected = 3;
 * @return {boolean}
 */
proto.ubo.v1.IpState.prototype.getIsConnected = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.IpState} returns this
 */
proto.ubo.v1.IpState.prototype.setIsConnected = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.IpState} returns this
 */
proto.ubo.v1.IpState.prototype.clearIsConnected = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.IpState.prototype.hasIsConnected = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    key: jspb.Message.getFieldWithDefault(msg, 2, 0),
    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadAction.PressedKeysSetType.toObject(includeInstance, f),
    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadAction.HeldKeysSetType.toObject(includeInstance, f),
    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadAction}
 */
proto.ubo.v1.KeypadAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadAction;
  return proto.ubo.v1.KeypadAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadAction}
 */
proto.ubo.v1.KeypadAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());
      msg.setKey(value);
      break;
    case 3:
      var value = new proto.ubo.v1.KeypadAction.PressedKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinaryFromReader);
      msg.setPressedKeys(value);
      break;
    case 4:
      var value = new proto.ubo.v1.KeypadAction.HeldKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinaryFromReader);
      msg.setHeldKeys(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getKey();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getPressedKeys();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.KeypadAction.PressedKeysSetType.serializeBinaryToWriter
    );
  }
  f = message.getHeldKeys();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.KeypadAction.HeldKeysSetType.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadAction.PressedKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadAction.PressedKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadAction.PressedKeysSetType;
  return proto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadAction.PressedKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadAction.PressedKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadAction.PressedKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadAction.HeldKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadAction.HeldKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadAction.HeldKeysSetType;
  return proto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadAction.HeldKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadAction.HeldKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadAction.HeldKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;
 * @return {string}
 */
proto.ubo.v1.KeypadAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.KeypadAction} returns this
 */
proto.ubo.v1.KeypadAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadAction} returns this
 */
proto.ubo.v1.KeypadAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Key key = 2;
 * @return {!proto.ubo.v1.Key}
 */
proto.ubo.v1.KeypadAction.prototype.getKey = function() {
  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @return {!proto.ubo.v1.KeypadAction} returns this
 */
proto.ubo.v1.KeypadAction.prototype.setKey = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional PressedKeysSetType pressed_keys = 3;
 * @return {?proto.ubo.v1.KeypadAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadAction.prototype.getPressedKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadAction.PressedKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadAction.PressedKeysSetType, 3));
};


/**
 * @param {?proto.ubo.v1.KeypadAction.PressedKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadAction} returns this
*/
proto.ubo.v1.KeypadAction.prototype.setPressedKeys = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadAction} returns this
 */
proto.ubo.v1.KeypadAction.prototype.clearPressedKeys = function() {
  return this.setPressedKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadAction.prototype.hasPressedKeys = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional HeldKeysSetType held_keys = 4;
 * @return {?proto.ubo.v1.KeypadAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadAction.prototype.getHeldKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadAction.HeldKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadAction.HeldKeysSetType, 4));
};


/**
 * @param {?proto.ubo.v1.KeypadAction.HeldKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadAction} returns this
*/
proto.ubo.v1.KeypadAction.prototype.setHeldKeys = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadAction} returns this
 */
proto.ubo.v1.KeypadAction.prototype.clearHeldKeys = function() {
  return this.setHeldKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadAction.prototype.hasHeldKeys = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float time = 5;
 * @return {number}
 */
proto.ubo.v1.KeypadAction.prototype.getTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.KeypadAction} returns this
 */
proto.ubo.v1.KeypadAction.prototype.setTime = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadAction} returns this
 */
proto.ubo.v1.KeypadAction.prototype.clearTime = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadAction.prototype.hasTime = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyPressAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyPressAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyPressAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    key: jspb.Message.getFieldWithDefault(msg, 2, 0),
    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.toObject(includeInstance, f),
    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.toObject(includeInstance, f),
    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyPressAction}
 */
proto.ubo.v1.KeypadKeyPressAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyPressAction;
  return proto.ubo.v1.KeypadKeyPressAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyPressAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyPressAction}
 */
proto.ubo.v1.KeypadKeyPressAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());
      msg.setKey(value);
      break;
    case 3:
      var value = new proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinaryFromReader);
      msg.setPressedKeys(value);
      break;
    case 4:
      var value = new proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinaryFromReader);
      msg.setHeldKeys(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyPressAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyPressAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyPressAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getKey();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getPressedKeys();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.serializeBinaryToWriter
    );
  }
  f = message.getHeldKeys();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType;
  return proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType;
  return proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;
 * @return {string}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Key key = 2;
 * @return {!proto.ubo.v1.Key}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.getKey = function() {
  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.setKey = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional PressedKeysSetType pressed_keys = 3;
 * @return {?proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.getPressedKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType, 3));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyPressAction.PressedKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
*/
proto.ubo.v1.KeypadKeyPressAction.prototype.setPressedKeys = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.clearPressedKeys = function() {
  return this.setPressedKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.hasPressedKeys = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional HeldKeysSetType held_keys = 4;
 * @return {?proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.getHeldKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType, 4));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyPressAction.HeldKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
*/
proto.ubo.v1.KeypadKeyPressAction.prototype.setHeldKeys = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.clearHeldKeys = function() {
  return this.setHeldKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.hasHeldKeys = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float time = 5;
 * @return {number}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.getTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.setTime = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyPressAction} returns this
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.clearTime = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyPressAction.prototype.hasTime = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyHoldAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyHoldAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyHoldAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    key: jspb.Message.getFieldWithDefault(msg, 2, 0),
    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.toObject(includeInstance, f),
    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.toObject(includeInstance, f),
    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction}
 */
proto.ubo.v1.KeypadKeyHoldAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyHoldAction;
  return proto.ubo.v1.KeypadKeyHoldAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyHoldAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction}
 */
proto.ubo.v1.KeypadKeyHoldAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());
      msg.setKey(value);
      break;
    case 3:
      var value = new proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinaryFromReader);
      msg.setPressedKeys(value);
      break;
    case 4:
      var value = new proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinaryFromReader);
      msg.setHeldKeys(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyHoldAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyHoldAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyHoldAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getKey();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getPressedKeys();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.serializeBinaryToWriter
    );
  }
  f = message.getHeldKeys();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType;
  return proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType;
  return proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;
 * @return {string}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Key key = 2;
 * @return {!proto.ubo.v1.Key}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.getKey = function() {
  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.setKey = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional PressedKeysSetType pressed_keys = 3;
 * @return {?proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.getPressedKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType, 3));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyHoldAction.PressedKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
*/
proto.ubo.v1.KeypadKeyHoldAction.prototype.setPressedKeys = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.clearPressedKeys = function() {
  return this.setPressedKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.hasPressedKeys = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional HeldKeysSetType held_keys = 4;
 * @return {?proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.getHeldKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType, 4));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyHoldAction.HeldKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
*/
proto.ubo.v1.KeypadKeyHoldAction.prototype.setHeldKeys = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.clearHeldKeys = function() {
  return this.setHeldKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.hasHeldKeys = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float time = 5;
 * @return {number}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.getTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.setTime = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyHoldAction} returns this
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.clearTime = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyHoldAction.prototype.hasTime = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyUnholdAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyUnholdAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    key: jspb.Message.getFieldWithDefault(msg, 2, 0),
    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.toObject(includeInstance, f),
    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.toObject(includeInstance, f),
    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction}
 */
proto.ubo.v1.KeypadKeyUnholdAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyUnholdAction;
  return proto.ubo.v1.KeypadKeyUnholdAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction}
 */
proto.ubo.v1.KeypadKeyUnholdAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());
      msg.setKey(value);
      break;
    case 3:
      var value = new proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinaryFromReader);
      msg.setPressedKeys(value);
      break;
    case 4:
      var value = new proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinaryFromReader);
      msg.setHeldKeys(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyUnholdAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyUnholdAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getKey();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getPressedKeys();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.serializeBinaryToWriter
    );
  }
  f = message.getHeldKeys();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType;
  return proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType;
  return proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;
 * @return {string}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Key key = 2;
 * @return {!proto.ubo.v1.Key}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.getKey = function() {
  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.setKey = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional PressedKeysSetType pressed_keys = 3;
 * @return {?proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.getPressedKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType, 3));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyUnholdAction.PressedKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
*/
proto.ubo.v1.KeypadKeyUnholdAction.prototype.setPressedKeys = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.clearPressedKeys = function() {
  return this.setPressedKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.hasPressedKeys = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional HeldKeysSetType held_keys = 4;
 * @return {?proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.getHeldKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType, 4));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyUnholdAction.HeldKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
*/
proto.ubo.v1.KeypadKeyUnholdAction.prototype.setHeldKeys = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.clearHeldKeys = function() {
  return this.setHeldKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.hasHeldKeys = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float time = 5;
 * @return {number}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.getTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.setTime = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyUnholdAction} returns this
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.clearTime = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyUnholdAction.prototype.hasTime = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyReleaseAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyReleaseAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    key: jspb.Message.getFieldWithDefault(msg, 2, 0),
    pressedKeys: (f = msg.getPressedKeys()) && proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.toObject(includeInstance, f),
    heldKeys: (f = msg.getHeldKeys()) && proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.toObject(includeInstance, f),
    time: jspb.Message.getFloatingPointFieldWithDefault(msg, 5, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction}
 */
proto.ubo.v1.KeypadKeyReleaseAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyReleaseAction;
  return proto.ubo.v1.KeypadKeyReleaseAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction}
 */
proto.ubo.v1.KeypadKeyReleaseAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.Key} */ (reader.readEnum());
      msg.setKey(value);
      break;
    case 3:
      var value = new proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinaryFromReader);
      msg.setPressedKeys(value);
      break;
    case 4:
      var value = new proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinaryFromReader);
      msg.setHeldKeys(value);
      break;
    case 5:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTime(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyReleaseAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyReleaseAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getKey();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getPressedKeys();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.serializeBinaryToWriter
    );
  }
  f = message.getHeldKeys();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeFloat(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType;
  return proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType;
  return proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var values = /** @type {!Array<!proto.ubo.v1.Key>} */ (reader.isDelimited() ? reader.readPackedEnum() : [reader.readEnum()]);
      for (var i = 0; i < values.length; i++) {
        msg.addItems(values[i]);
      }
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writePackedEnum(
      1,
      f
    );
  }
};


/**
 * repeated Key items = 1;
 * @return {!Array<!proto.ubo.v1.Key>}
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.getItemsList = function() {
  return /** @type {!Array<!proto.ubo.v1.Key>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Key>} value
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;
 * @return {string}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Key key = 2;
 * @return {!proto.ubo.v1.Key}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.getKey = function() {
  return /** @type {!proto.ubo.v1.Key} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.Key} value
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.setKey = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional PressedKeysSetType pressed_keys = 3;
 * @return {?proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.getPressedKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType, 3));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyReleaseAction.PressedKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
*/
proto.ubo.v1.KeypadKeyReleaseAction.prototype.setPressedKeys = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.clearPressedKeys = function() {
  return this.setPressedKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.hasPressedKeys = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional HeldKeysSetType held_keys = 4;
 * @return {?proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.getHeldKeys = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType, 4));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyReleaseAction.HeldKeysSetType|undefined} value
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
*/
proto.ubo.v1.KeypadKeyReleaseAction.prototype.setHeldKeys = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.clearHeldKeys = function() {
  return this.setHeldKeys(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.hasHeldKeys = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional float time = 5;
 * @return {number}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.getTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 5, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.setTime = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadKeyReleaseAction} returns this
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.clearTime = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadKeyReleaseAction.prototype.hasTime = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.KeypadState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.KeypadState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.KeypadState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotKeypad: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    depth: jspb.Message.getFieldWithDefault(msg, 2, 0),
    isConsumed: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.KeypadState}
 */
proto.ubo.v1.KeypadState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.KeypadState;
  return proto.ubo.v1.KeypadState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.KeypadState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.KeypadState}
 */
proto.ubo.v1.KeypadState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setDepth(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsConsumed(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.KeypadState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.KeypadState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.KeypadState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.KeypadState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_keypad = 1000;
 * @return {string}
 */
proto.ubo.v1.KeypadState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.KeypadState} returns this
 */
proto.ubo.v1.KeypadState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadState} returns this
 */
proto.ubo.v1.KeypadState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotKeypad = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 depth = 2;
 * @return {number}
 */
proto.ubo.v1.KeypadState.prototype.getDepth = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.KeypadState} returns this
 */
proto.ubo.v1.KeypadState.prototype.setDepth = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadState} returns this
 */
proto.ubo.v1.KeypadState.prototype.clearDepth = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadState.prototype.hasDepth = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_consumed = 3;
 * @return {boolean}
 */
proto.ubo.v1.KeypadState.prototype.getIsConsumed = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.KeypadState} returns this
 */
proto.ubo.v1.KeypadState.prototype.setIsConsumed = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.KeypadState} returns this
 */
proto.ubo.v1.KeypadState.prototype.clearIsConsumed = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.KeypadState.prototype.hasIsConsumed = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.LightDMAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.LightDMAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.LightDMAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.LightDMAction}
 */
proto.ubo.v1.LightDMAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.LightDMAction;
  return proto.ubo.v1.LightDMAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.LightDMAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.LightDMAction}
 */
proto.ubo.v1.LightDMAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.LightDMAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.LightDMAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.LightDMAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;
 * @return {string}
 */
proto.ubo.v1.LightDMAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.LightDMAction} returns this
 */
proto.ubo.v1.LightDMAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMAction} returns this
 */
proto.ubo.v1.LightDMAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.LightDMUpdateStateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.LightDMUpdateStateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMUpdateStateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    isInstalling: jspb.Message.getBooleanFieldWithDefault(msg, 5, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.LightDMUpdateStateAction}
 */
proto.ubo.v1.LightDMUpdateStateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.LightDMUpdateStateAction;
  return proto.ubo.v1.LightDMUpdateStateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.LightDMUpdateStateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.LightDMUpdateStateAction}
 */
proto.ubo.v1.LightDMUpdateStateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsEnabled(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsInstalled(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsInstalling(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.LightDMUpdateStateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.LightDMUpdateStateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMUpdateStateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;
 * @return {string}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.setIsActive = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsActive = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsActive = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_enabled = 3;
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.getIsEnabled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.setIsEnabled = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsEnabled = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsEnabled = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool is_installed = 4;
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.getIsInstalled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.setIsInstalled = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsInstalled = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsInstalled = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_installing = 5;
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.getIsInstalling = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.setIsInstalling = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMUpdateStateAction} returns this
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.clearIsInstalling = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMUpdateStateAction.prototype.hasIsInstalling = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.LightDMClearEnabledStateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.LightDMClearEnabledStateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.LightDMClearEnabledStateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMClearEnabledStateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.LightDMClearEnabledStateAction}
 */
proto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.LightDMClearEnabledStateAction;
  return proto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.LightDMClearEnabledStateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.LightDMClearEnabledStateAction}
 */
proto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.LightDMClearEnabledStateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.LightDMClearEnabledStateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.LightDMClearEnabledStateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMClearEnabledStateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;
 * @return {string}
 */
proto.ubo.v1.LightDMClearEnabledStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.LightDMClearEnabledStateAction} returns this
 */
proto.ubo.v1.LightDMClearEnabledStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMClearEnabledStateAction} returns this
 */
proto.ubo.v1.LightDMClearEnabledStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMClearEnabledStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.LightDMState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.LightDMState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.LightDMState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotLightdm: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    isInstalling: jspb.Message.getBooleanFieldWithDefault(msg, 5, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.LightDMState}
 */
proto.ubo.v1.LightDMState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.LightDMState;
  return proto.ubo.v1.LightDMState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.LightDMState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.LightDMState}
 */
proto.ubo.v1.LightDMState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsEnabled(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsInstalled(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsInstalling(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.LightDMState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.LightDMState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.LightDMState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.LightDMState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_lightdm = 1000;
 * @return {string}
 */
proto.ubo.v1.LightDMState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotLightdm = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.setIsActive = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.clearIsActive = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.hasIsActive = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_enabled = 3;
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.getIsEnabled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.setIsEnabled = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.clearIsEnabled = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.hasIsEnabled = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool is_installed = 4;
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.getIsInstalled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.setIsInstalled = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.clearIsInstalled = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.hasIsInstalled = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_installing = 5;
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.getIsInstalling = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.setIsInstalling = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.LightDMState} returns this
 */
proto.ubo.v1.LightDMState.prototype.clearIsInstalling = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.LightDMState.prototype.hasIsInstalling = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationActionItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationActionItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationActionItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationActionItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 2, ""),
    dismissNotification: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    closeNotification: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    key: jspb.Message.getFieldWithDefault(msg, 5, ""),
    label: jspb.Message.getFieldWithDefault(msg, 6, ""),
    color: jspb.Message.getFieldWithDefault(msg, 7, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 8, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 9, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 11, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationActionItem}
 */
proto.ubo.v1.NotificationActionItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationActionItem;
  return proto.ubo.v1.NotificationActionItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationActionItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationActionItem}
 */
proto.ubo.v1.NotificationActionItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDismissNotification(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCloseNotification(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 8:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 9:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 11:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationActionItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationActionItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationActionItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationActionItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeString(
      8,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeBool(
      9,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeFloat(
      11,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationActionItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string background_color = 2;
 * @return {string}
 */
proto.ubo.v1.NotificationActionItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool dismiss_notification = 3;
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.getDismissNotification = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setDismissNotification = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearDismissNotification = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasDismissNotification = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool close_notification = 4;
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.getCloseNotification = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setCloseNotification = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearCloseNotification = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasCloseNotification = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string key = 5;
 * @return {string}
 */
proto.ubo.v1.NotificationActionItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string label = 6;
 * @return {string}
 */
proto.ubo.v1.NotificationActionItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string color = 7;
 * @return {string}
 */
proto.ubo.v1.NotificationActionItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional string icon = 8;
 * @return {string}
 */
proto.ubo.v1.NotificationActionItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 8, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional bool is_short = 9;
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 9, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float opacity = 10;
 * @return {number}
 */
proto.ubo.v1.NotificationActionItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional float progress = 11;
 * @return {number}
 */
proto.ubo.v1.NotificationActionItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 11, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationActionItem} returns this
 */
proto.ubo.v1.NotificationActionItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationActionItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 11) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationApplicationItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationApplicationItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 2, ""),
    dismissNotification: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    closeNotification: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    application: (f = msg.getApplication()) && proto.ubo.v1.NotificationApplicationItem.Application.toObject(includeInstance, f),
    applicationId: jspb.Message.getFieldWithDefault(msg, 6, ""),
    initializationArgs: (f = msg.getInitializationArgs()) && proto.ubo.v1.NotificationApplicationItem.InitializationArgs.toObject(includeInstance, f),
    initializationKwargs: (f = msg.getInitializationKwargs()) && proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.toObject(includeInstance, f),
    key: jspb.Message.getFieldWithDefault(msg, 9, ""),
    label: jspb.Message.getFieldWithDefault(msg, 10, ""),
    color: jspb.Message.getFieldWithDefault(msg, 11, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 12, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 13, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 14, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 15, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationApplicationItem}
 */
proto.ubo.v1.NotificationApplicationItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationApplicationItem;
  return proto.ubo.v1.NotificationApplicationItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationApplicationItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationApplicationItem}
 */
proto.ubo.v1.NotificationApplicationItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDismissNotification(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCloseNotification(value);
      break;
    case 5:
      var value = new proto.ubo.v1.NotificationApplicationItem.Application;
      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.Application.deserializeBinaryFromReader);
      msg.setApplication(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setApplicationId(value);
      break;
    case 7:
      var value = new proto.ubo.v1.NotificationApplicationItem.InitializationArgs;
      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinaryFromReader);
      msg.setInitializationArgs(value);
      break;
    case 8:
      var value = new proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict;
      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader);
      msg.setInitializationKwargs(value);
      break;
    case 9:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 10:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 11:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 13:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 14:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 15:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationApplicationItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationApplicationItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = message.getApplication();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.ubo.v1.NotificationApplicationItem.Application.serializeBinaryToWriter
    );
  }
  f = message.getApplicationId();
  if (f.length > 0) {
    writer.writeString(
      6,
      f
    );
  }
  f = message.getInitializationArgs();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.ubo.v1.NotificationApplicationItem.InitializationArgs.serializeBinaryToWriter
    );
  }
  f = message.getInitializationKwargs();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeString(
      9,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeString(
      10,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeString(
      11,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeString(
      12,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeBool(
      13,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeFloat(
      14,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeFloat(
      15,
      f
    );
  }
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_ = [[1]];

/**
 * @enum {number}
 */
proto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase = {
  APPLICATION_NOT_SET: 0,
  STRING: 1
};

/**
 * @return {proto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase}
 */
proto.ubo.v1.NotificationApplicationItem.Application.prototype.getApplicationCase = function() {
  return /** @type {proto.ubo.v1.NotificationApplicationItem.Application.ApplicationCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationApplicationItem.Application.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationApplicationItem.Application.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationApplicationItem.Application} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.Application.toObject = function(includeInstance, msg) {
  var f, obj = {
    string: jspb.Message.getFieldWithDefault(msg, 1, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationApplicationItem.Application}
 */
proto.ubo.v1.NotificationApplicationItem.Application.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationApplicationItem.Application;
  return proto.ubo.v1.NotificationApplicationItem.Application.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationApplicationItem.Application} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationApplicationItem.Application}
 */
proto.ubo.v1.NotificationApplicationItem.Application.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.setString(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationApplicationItem.Application.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationApplicationItem.Application.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationApplicationItem.Application} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.Application.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeString(
      1,
      f
    );
  }
};


/**
 * optional string string = 1;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.Application.prototype.getString = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem.Application} returns this
 */
proto.ubo.v1.NotificationApplicationItem.Application.prototype.setString = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem.Application} returns this
 */
proto.ubo.v1.NotificationApplicationItem.Application.prototype.clearString = function() {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.NotificationApplicationItem.Application.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.Application.prototype.hasString = function() {
  return jspb.Message.getField(this, 1) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationApplicationItem.InitializationArgs.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.BasicType.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationApplicationItem.InitializationArgs;
  return proto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.BasicType;
      reader.readMessage(value,proto.ubo.v1.BasicType.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationApplicationItem.InitializationArgs.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.BasicType.serializeBinaryToWriter
    );
  }
};


/**
 * repeated BasicType items = 1;
 * @return {!Array<!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.BasicType>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.BasicType, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.BasicType>} value
 * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} returns this
*/
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.BasicType=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.BasicType}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.BasicType, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationArgs} returns this
 */
proto.ubo.v1.NotificationApplicationItem.InitializationArgs.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsMap: (f = msg.getItemsMap()) ? f.toObject(includeInstance, proto.ubo.v1.BasicType.toObject) : []
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict;
  return proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = msg.getItemsMap();
      reader.readMessage(value, function(message, reader) {
        jspb.Map.deserializeBinary(message, reader, jspb.BinaryReader.prototype.readString, jspb.BinaryReader.prototype.readMessage, proto.ubo.v1.BasicType.deserializeBinaryFromReader, "", new proto.ubo.v1.BasicType());
         });
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsMap(true);
  if (f && f.getLength() > 0) {
    f.serializeBinary(1, writer, jspb.BinaryWriter.prototype.writeString, jspb.BinaryWriter.prototype.writeMessage, proto.ubo.v1.BasicType.serializeBinaryToWriter);
  }
};


/**
 * map<string, BasicType> items = 1;
 * @param {boolean=} opt_noLazyCreate Do not create the map if
 * empty, instead returning `undefined`
 * @return {!jspb.Map<string,!proto.ubo.v1.BasicType>}
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.getItemsMap = function(opt_noLazyCreate) {
  return /** @type {!jspb.Map<string,!proto.ubo.v1.BasicType>} */ (
      jspb.Message.getMapField(this, 1, opt_noLazyCreate,
      proto.ubo.v1.BasicType));
};


/**
 * Clears values from the map. The map will be non-null.
 * @return {!proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} returns this
 */
proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict.prototype.clearItemsMap = function() {
  this.getItemsMap().clear();
  return this;};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string background_color = 2;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool dismiss_notification = 3;
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getDismissNotification = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setDismissNotification = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearDismissNotification = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasDismissNotification = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool close_notification = 4;
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getCloseNotification = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setCloseNotification = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearCloseNotification = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasCloseNotification = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional Application application = 5;
 * @return {?proto.ubo.v1.NotificationApplicationItem.Application}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getApplication = function() {
  return /** @type{?proto.ubo.v1.NotificationApplicationItem.Application} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem.Application, 5));
};


/**
 * @param {?proto.ubo.v1.NotificationApplicationItem.Application|undefined} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
*/
proto.ubo.v1.NotificationApplicationItem.prototype.setApplication = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearApplication = function() {
  return this.setApplication(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasApplication = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string application_id = 6;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getApplicationId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setApplicationId = function(value) {
  return jspb.Message.setProto3StringField(this, 6, value);
};


/**
 * optional InitializationArgs initialization_args = 7;
 * @return {?proto.ubo.v1.NotificationApplicationItem.InitializationArgs}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getInitializationArgs = function() {
  return /** @type{?proto.ubo.v1.NotificationApplicationItem.InitializationArgs} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem.InitializationArgs, 7));
};


/**
 * @param {?proto.ubo.v1.NotificationApplicationItem.InitializationArgs|undefined} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
*/
proto.ubo.v1.NotificationApplicationItem.prototype.setInitializationArgs = function(value) {
  return jspb.Message.setWrapperField(this, 7, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearInitializationArgs = function() {
  return this.setInitializationArgs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasInitializationArgs = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional InitializationKwargsDict initialization_kwargs = 8;
 * @return {?proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getInitializationKwargs = function() {
  return /** @type{?proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict, 8));
};


/**
 * @param {?proto.ubo.v1.NotificationApplicationItem.InitializationKwargsDict|undefined} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
*/
proto.ubo.v1.NotificationApplicationItem.prototype.setInitializationKwargs = function(value) {
  return jspb.Message.setWrapperField(this, 8, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearInitializationKwargs = function() {
  return this.setInitializationKwargs(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasInitializationKwargs = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional string key = 9;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 9, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional string label = 10;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 10, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional string color = 11;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 11, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional string icon = 12;
 * @return {string}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 12, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional bool is_short = 13;
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 13, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional float opacity = 14;
 * @return {number}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 14, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional float progress = 15;
 * @return {number}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 15, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 15, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationApplicationItem} returns this
 */
proto.ubo.v1.NotificationApplicationItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 15, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationApplicationItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 15) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationDispatchItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationDispatchItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationDispatchItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    storeAction: (f = msg.getStoreAction()) && proto.ubo.v1.NotificationDispatchItem.StoreAction.toObject(includeInstance, f),
    key: jspb.Message.getFieldWithDefault(msg, 3, ""),
    label: jspb.Message.getFieldWithDefault(msg, 4, ""),
    color: jspb.Message.getFieldWithDefault(msg, 5, ""),
    backgroundColor: jspb.Message.getFieldWithDefault(msg, 6, ""),
    icon: jspb.Message.getFieldWithDefault(msg, 7, ""),
    isShort: jspb.Message.getBooleanFieldWithDefault(msg, 8, false),
    opacity: jspb.Message.getFloatingPointFieldWithDefault(msg, 9, 0.0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 10, 0.0),
    dismissNotification: jspb.Message.getBooleanFieldWithDefault(msg, 11, false),
    closeNotification: jspb.Message.getBooleanFieldWithDefault(msg, 12, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationDispatchItem}
 */
proto.ubo.v1.NotificationDispatchItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationDispatchItem;
  return proto.ubo.v1.NotificationDispatchItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationDispatchItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationDispatchItem}
 */
proto.ubo.v1.NotificationDispatchItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = new proto.ubo.v1.NotificationDispatchItem.StoreAction;
      reader.readMessage(value,proto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinaryFromReader);
      msg.setStoreAction(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setKey(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setLabel(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 6:
      var value = /** @type {string} */ (reader.readString());
      msg.setBackgroundColor(value);
      break;
    case 7:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 8:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsShort(value);
      break;
    case 9:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setOpacity(value);
      break;
    case 10:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    case 11:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDismissNotification(value);
      break;
    case 12:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setCloseNotification(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationDispatchItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationDispatchItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationDispatchItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getStoreAction();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.NotificationDispatchItem.StoreAction.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeString(
      6,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeString(
      7,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeBool(
      8,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeFloat(
      9,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeFloat(
      10,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 11));
  if (f != null) {
    writer.writeBool(
      11,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeBool(
      12,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationDispatchItem.StoreAction2.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.Action.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationDispatchItem.StoreAction2;
  return proto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.Action;
      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationDispatchItem.StoreAction2.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.Action.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Action items = 1;
 * @return {!Array<!proto.ubo.v1.Action>}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Action>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Action, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Action>} value
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} returns this
*/
proto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.Action=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Action}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Action, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction2} returns this
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction2.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase = {
  STORE_ACTION_NOT_SET: 0,
  UBO_ACTION: 1,
  LIST: 2
};

/**
 * @return {proto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.getStoreActionCase = function() {
  return /** @type {proto.ubo.v1.NotificationDispatchItem.StoreAction.StoreActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationDispatchItem.StoreAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    uboAction: (f = msg.getUboAction()) && proto.ubo.v1.Action.toObject(includeInstance, f),
    list: (f = msg.getList()) && proto.ubo.v1.NotificationDispatchItem.StoreAction2.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationDispatchItem.StoreAction;
  return proto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.Action;
      reader.readMessage(value,proto.ubo.v1.Action.deserializeBinaryFromReader);
      msg.setUboAction(value);
      break;
    case 2:
      var value = new proto.ubo.v1.NotificationDispatchItem.StoreAction2;
      reader.readMessage(value,proto.ubo.v1.NotificationDispatchItem.StoreAction2.deserializeBinaryFromReader);
      msg.setList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationDispatchItem.StoreAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationDispatchItem.StoreAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getUboAction();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.Action.serializeBinaryToWriter
    );
  }
  f = message.getList();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.NotificationDispatchItem.StoreAction2.serializeBinaryToWriter
    );
  }
};


/**
 * optional Action ubo_action = 1;
 * @return {?proto.ubo.v1.Action}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.getUboAction = function() {
  return /** @type{?proto.ubo.v1.Action} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Action, 1));
};


/**
 * @param {?proto.ubo.v1.Action|undefined} value
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this
*/
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.setUboAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.clearUboAction = function() {
  return this.setUboAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.hasUboAction = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional StoreAction2 list = 2;
 * @return {?proto.ubo.v1.NotificationDispatchItem.StoreAction2}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.getList = function() {
  return /** @type{?proto.ubo.v1.NotificationDispatchItem.StoreAction2} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationDispatchItem.StoreAction2, 2));
};


/**
 * @param {?proto.ubo.v1.NotificationDispatchItem.StoreAction2|undefined} value
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this
*/
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.setList = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.NotificationDispatchItem.StoreAction.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem.StoreAction} returns this
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.clearList = function() {
  return this.setList(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.StoreAction.prototype.hasList = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional StoreAction store_action = 2;
 * @return {?proto.ubo.v1.NotificationDispatchItem.StoreAction}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getStoreAction = function() {
  return /** @type{?proto.ubo.v1.NotificationDispatchItem.StoreAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationDispatchItem.StoreAction, 2));
};


/**
 * @param {?proto.ubo.v1.NotificationDispatchItem.StoreAction|undefined} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
*/
proto.ubo.v1.NotificationDispatchItem.prototype.setStoreAction = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearStoreAction = function() {
  return this.setStoreAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasStoreAction = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string key = 3;
 * @return {string}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getKey = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setKey = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearKey = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasKey = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string label = 4;
 * @return {string}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getLabel = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setLabel = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearLabel = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasLabel = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string color = 5;
 * @return {string}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearColor = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasColor = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional string background_color = 6;
 * @return {string}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getBackgroundColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 6, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setBackgroundColor = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearBackgroundColor = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasBackgroundColor = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional string icon = 7;
 * @return {string}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 7, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional bool is_short = 8;
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getIsShort = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 8, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setIsShort = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearIsShort = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasIsShort = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional float opacity = 9;
 * @return {number}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getOpacity = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 9, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setOpacity = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearOpacity = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasOpacity = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional float progress = 10;
 * @return {number}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 10, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional bool dismiss_notification = 11;
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getDismissNotification = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 11, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setDismissNotification = function(value) {
  return jspb.Message.setField(this, 11, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearDismissNotification = function() {
  return jspb.Message.setField(this, 11, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasDismissNotification = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional bool close_notification = 12;
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.getCloseNotification = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 12, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.setCloseNotification = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationDispatchItem} returns this
 */
proto.ubo.v1.NotificationDispatchItem.prototype.clearCloseNotification = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationDispatchItem.prototype.hasCloseNotification = function() {
  return jspb.Message.getField(this, 12) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Notification.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Notification.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Notification} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, ""),
    title: jspb.Message.getFieldWithDefault(msg, 3, ""),
    content: jspb.Message.getFieldWithDefault(msg, 4, ""),
    extraInformation: (f = msg.getExtraInformation()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),
    importance: jspb.Message.getFieldWithDefault(msg, 6, 0),
    chime: jspb.Message.getFieldWithDefault(msg, 7, 0),
    timestamp: jspb.Message.getFieldWithDefault(msg, 8, 0),
    isRead: jspb.Message.getBooleanFieldWithDefault(msg, 9, false),
    sender: jspb.Message.getFieldWithDefault(msg, 10, ""),
    actions: (f = msg.getActions()) && proto.ubo.v1.Notification.Actions.toObject(includeInstance, f),
    icon: jspb.Message.getFieldWithDefault(msg, 12, ""),
    color: jspb.Message.getFieldWithDefault(msg, 13, ""),
    expirationTimestamp: jspb.Message.getFieldWithDefault(msg, 14, 0),
    displayType: jspb.Message.getFieldWithDefault(msg, 15, 0),
    flashTime: jspb.Message.getFloatingPointFieldWithDefault(msg, 16, 0.0),
    showDismissAction: jspb.Message.getBooleanFieldWithDefault(msg, 17, false),
    dismissOnClose: jspb.Message.getBooleanFieldWithDefault(msg, 18, false),
    onClose: (f = msg.getOnClose()) && proto.ubo.v1.Notification.OnClose.toObject(includeInstance, f),
    blink: jspb.Message.getBooleanFieldWithDefault(msg, 20, false),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 21, 0.0),
    progressWeight: jspb.Message.getFloatingPointFieldWithDefault(msg, 22, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Notification}
 */
proto.ubo.v1.Notification.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Notification;
  return proto.ubo.v1.Notification.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Notification} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Notification}
 */
proto.ubo.v1.Notification.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setTitle(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setContent(value);
      break;
    case 5:
      var value = new proto.ubo.v1.ReadableInformation;
      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);
      msg.setExtraInformation(value);
      break;
    case 6:
      var value = /** @type {!proto.ubo.v1.Importance} */ (reader.readEnum());
      msg.setImportance(value);
      break;
    case 7:
      var value = /** @type {!proto.ubo.v1.Chime} */ (reader.readEnum());
      msg.setChime(value);
      break;
    case 8:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setTimestamp(value);
      break;
    case 9:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsRead(value);
      break;
    case 10:
      var value = /** @type {string} */ (reader.readString());
      msg.setSender(value);
      break;
    case 11:
      var value = new proto.ubo.v1.Notification.Actions;
      reader.readMessage(value,proto.ubo.v1.Notification.Actions.deserializeBinaryFromReader);
      msg.setActions(value);
      break;
    case 12:
      var value = /** @type {string} */ (reader.readString());
      msg.setIcon(value);
      break;
    case 13:
      var value = /** @type {string} */ (reader.readString());
      msg.setColor(value);
      break;
    case 14:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setExpirationTimestamp(value);
      break;
    case 15:
      var value = /** @type {!proto.ubo.v1.NotificationDisplayType} */ (reader.readEnum());
      msg.setDisplayType(value);
      break;
    case 16:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setFlashTime(value);
      break;
    case 17:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setShowDismissAction(value);
      break;
    case 18:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setDismissOnClose(value);
      break;
    case 19:
      var value = new proto.ubo.v1.Notification.OnClose;
      reader.readMessage(value,proto.ubo.v1.Notification.OnClose.deserializeBinaryFromReader);
      msg.setOnClose(value);
      break;
    case 20:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setBlink(value);
      break;
    case 21:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    case 22:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgressWeight(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Notification.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Notification.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Notification} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getTitle();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getContent();
  if (f.length > 0) {
    writer.writeString(
      4,
      f
    );
  }
  f = message.getExtraInformation();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter
    );
  }
  f = /** @type {!proto.ubo.v1.Importance} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.Chime} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeEnum(
      7,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 8));
  if (f != null) {
    writer.writeInt64(
      8,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 9));
  if (f != null) {
    writer.writeBool(
      9,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 10));
  if (f != null) {
    writer.writeString(
      10,
      f
    );
  }
  f = message.getActions();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.ubo.v1.Notification.Actions.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 12));
  if (f != null) {
    writer.writeString(
      12,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 13));
  if (f != null) {
    writer.writeString(
      13,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 14));
  if (f != null) {
    writer.writeInt64(
      14,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.NotificationDisplayType} */ (jspb.Message.getField(message, 15));
  if (f != null) {
    writer.writeEnum(
      15,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 16));
  if (f != null) {
    writer.writeFloat(
      16,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 17));
  if (f != null) {
    writer.writeBool(
      17,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 18));
  if (f != null) {
    writer.writeBool(
      18,
      f
    );
  }
  f = message.getOnClose();
  if (f != null) {
    writer.writeMessage(
      19,
      f,
      proto.ubo.v1.Notification.OnClose.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 20));
  if (f != null) {
    writer.writeBool(
      20,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 21));
  if (f != null) {
    writer.writeFloat(
      21,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 22));
  if (f != null) {
    writer.writeFloat(
      22,
      f
    );
  }
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.Notification.ActionsItem.oneofGroups_ = [[1,2,3]];

/**
 * @enum {number}
 */
proto.ubo.v1.Notification.ActionsItem.ActionsItemCase = {
  ACTIONS_ITEM_NOT_SET: 0,
  NOTIFICATION_ACTION_ITEM: 1,
  NOTIFICATION_APPLICATION_ITEM: 2,
  NOTIFICATION_DISPATCH_ITEM: 3
};

/**
 * @return {proto.ubo.v1.Notification.ActionsItem.ActionsItemCase}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.getActionsItemCase = function() {
  return /** @type {proto.ubo.v1.Notification.ActionsItem.ActionsItemCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Notification.ActionsItem.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Notification.ActionsItem} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.ActionsItem.toObject = function(includeInstance, msg) {
  var f, obj = {
    notificationActionItem: (f = msg.getNotificationActionItem()) && proto.ubo.v1.NotificationActionItem.toObject(includeInstance, f),
    notificationApplicationItem: (f = msg.getNotificationApplicationItem()) && proto.ubo.v1.NotificationApplicationItem.toObject(includeInstance, f),
    notificationDispatchItem: (f = msg.getNotificationDispatchItem()) && proto.ubo.v1.NotificationDispatchItem.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Notification.ActionsItem}
 */
proto.ubo.v1.Notification.ActionsItem.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Notification.ActionsItem;
  return proto.ubo.v1.Notification.ActionsItem.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Notification.ActionsItem} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Notification.ActionsItem}
 */
proto.ubo.v1.Notification.ActionsItem.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.NotificationActionItem;
      reader.readMessage(value,proto.ubo.v1.NotificationActionItem.deserializeBinaryFromReader);
      msg.setNotificationActionItem(value);
      break;
    case 2:
      var value = new proto.ubo.v1.NotificationApplicationItem;
      reader.readMessage(value,proto.ubo.v1.NotificationApplicationItem.deserializeBinaryFromReader);
      msg.setNotificationApplicationItem(value);
      break;
    case 3:
      var value = new proto.ubo.v1.NotificationDispatchItem;
      reader.readMessage(value,proto.ubo.v1.NotificationDispatchItem.deserializeBinaryFromReader);
      msg.setNotificationDispatchItem(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Notification.ActionsItem.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Notification.ActionsItem} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.ActionsItem.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getNotificationActionItem();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.NotificationActionItem.serializeBinaryToWriter
    );
  }
  f = message.getNotificationApplicationItem();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.NotificationApplicationItem.serializeBinaryToWriter
    );
  }
  f = message.getNotificationDispatchItem();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.NotificationDispatchItem.serializeBinaryToWriter
    );
  }
};


/**
 * optional NotificationActionItem notification_action_item = 1;
 * @return {?proto.ubo.v1.NotificationActionItem}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.getNotificationActionItem = function() {
  return /** @type{?proto.ubo.v1.NotificationActionItem} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationActionItem, 1));
};


/**
 * @param {?proto.ubo.v1.NotificationActionItem|undefined} value
 * @return {!proto.ubo.v1.Notification.ActionsItem} returns this
*/
proto.ubo.v1.Notification.ActionsItem.prototype.setNotificationActionItem = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Notification.ActionsItem} returns this
 */
proto.ubo.v1.Notification.ActionsItem.prototype.clearNotificationActionItem = function() {
  return this.setNotificationActionItem(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.hasNotificationActionItem = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional NotificationApplicationItem notification_application_item = 2;
 * @return {?proto.ubo.v1.NotificationApplicationItem}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.getNotificationApplicationItem = function() {
  return /** @type{?proto.ubo.v1.NotificationApplicationItem} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationApplicationItem, 2));
};


/**
 * @param {?proto.ubo.v1.NotificationApplicationItem|undefined} value
 * @return {!proto.ubo.v1.Notification.ActionsItem} returns this
*/
proto.ubo.v1.Notification.ActionsItem.prototype.setNotificationApplicationItem = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Notification.ActionsItem} returns this
 */
proto.ubo.v1.Notification.ActionsItem.prototype.clearNotificationApplicationItem = function() {
  return this.setNotificationApplicationItem(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.hasNotificationApplicationItem = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional NotificationDispatchItem notification_dispatch_item = 3;
 * @return {?proto.ubo.v1.NotificationDispatchItem}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.getNotificationDispatchItem = function() {
  return /** @type{?proto.ubo.v1.NotificationDispatchItem} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationDispatchItem, 3));
};


/**
 * @param {?proto.ubo.v1.NotificationDispatchItem|undefined} value
 * @return {!proto.ubo.v1.Notification.ActionsItem} returns this
*/
proto.ubo.v1.Notification.ActionsItem.prototype.setNotificationDispatchItem = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.Notification.ActionsItem.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Notification.ActionsItem} returns this
 */
proto.ubo.v1.Notification.ActionsItem.prototype.clearNotificationDispatchItem = function() {
  return this.setNotificationDispatchItem(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.ActionsItem.prototype.hasNotificationDispatchItem = function() {
  return jspb.Message.getField(this, 3) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.Notification.Actions.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Notification.Actions.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Notification.Actions.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Notification.Actions} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.Actions.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.Notification.ActionsItem.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Notification.Actions}
 */
proto.ubo.v1.Notification.Actions.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Notification.Actions;
  return proto.ubo.v1.Notification.Actions.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Notification.Actions} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Notification.Actions}
 */
proto.ubo.v1.Notification.Actions.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.Notification.ActionsItem;
      reader.readMessage(value,proto.ubo.v1.Notification.ActionsItem.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Notification.Actions.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Notification.Actions.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Notification.Actions} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.Actions.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.Notification.ActionsItem.serializeBinaryToWriter
    );
  }
};


/**
 * repeated ActionsItem items = 1;
 * @return {!Array<!proto.ubo.v1.Notification.ActionsItem>}
 */
proto.ubo.v1.Notification.Actions.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Notification.ActionsItem>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Notification.ActionsItem, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.Notification.ActionsItem>} value
 * @return {!proto.ubo.v1.Notification.Actions} returns this
*/
proto.ubo.v1.Notification.Actions.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.Notification.ActionsItem=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Notification.ActionsItem}
 */
proto.ubo.v1.Notification.Actions.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.Notification.ActionsItem, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.Notification.Actions} returns this
 */
proto.ubo.v1.Notification.Actions.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Notification.OnClose.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Notification.OnClose.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Notification.OnClose} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.OnClose.toObject = function(includeInstance, msg) {
  var f, obj = {

  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Notification.OnClose}
 */
proto.ubo.v1.Notification.OnClose.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Notification.OnClose;
  return proto.ubo.v1.Notification.OnClose.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Notification.OnClose} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Notification.OnClose}
 */
proto.ubo.v1.Notification.OnClose.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Notification.OnClose.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Notification.OnClose.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Notification.OnClose} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Notification.OnClose.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.Notification.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.Notification.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setId = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearId = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasId = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string title = 3;
 * @return {string}
 */
proto.ubo.v1.Notification.prototype.getTitle = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setTitle = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional string content = 4;
 * @return {string}
 */
proto.ubo.v1.Notification.prototype.getContent = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setContent = function(value) {
  return jspb.Message.setProto3StringField(this, 4, value);
};


/**
 * optional ReadableInformation extra_information = 5;
 * @return {?proto.ubo.v1.ReadableInformation}
 */
proto.ubo.v1.Notification.prototype.getExtraInformation = function() {
  return /** @type{?proto.ubo.v1.ReadableInformation} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 5));
};


/**
 * @param {?proto.ubo.v1.ReadableInformation|undefined} value
 * @return {!proto.ubo.v1.Notification} returns this
*/
proto.ubo.v1.Notification.prototype.setExtraInformation = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearExtraInformation = function() {
  return this.setExtraInformation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasExtraInformation = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional Importance importance = 6;
 * @return {!proto.ubo.v1.Importance}
 */
proto.ubo.v1.Notification.prototype.getImportance = function() {
  return /** @type {!proto.ubo.v1.Importance} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {!proto.ubo.v1.Importance} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setImportance = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearImportance = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasImportance = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional Chime chime = 7;
 * @return {!proto.ubo.v1.Chime}
 */
proto.ubo.v1.Notification.prototype.getChime = function() {
  return /** @type {!proto.ubo.v1.Chime} */ (jspb.Message.getFieldWithDefault(this, 7, 0));
};


/**
 * @param {!proto.ubo.v1.Chime} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setChime = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearChime = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasChime = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional int64 timestamp = 8;
 * @return {number}
 */
proto.ubo.v1.Notification.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 8, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setTimestamp = function(value) {
  return jspb.Message.setField(this, 8, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearTimestamp = function() {
  return jspb.Message.setField(this, 8, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasTimestamp = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional bool is_read = 9;
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.getIsRead = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 9, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setIsRead = function(value) {
  return jspb.Message.setField(this, 9, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearIsRead = function() {
  return jspb.Message.setField(this, 9, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasIsRead = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional string sender = 10;
 * @return {string}
 */
proto.ubo.v1.Notification.prototype.getSender = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 10, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setSender = function(value) {
  return jspb.Message.setField(this, 10, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearSender = function() {
  return jspb.Message.setField(this, 10, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasSender = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional Actions actions = 11;
 * @return {?proto.ubo.v1.Notification.Actions}
 */
proto.ubo.v1.Notification.prototype.getActions = function() {
  return /** @type{?proto.ubo.v1.Notification.Actions} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification.Actions, 11));
};


/**
 * @param {?proto.ubo.v1.Notification.Actions|undefined} value
 * @return {!proto.ubo.v1.Notification} returns this
*/
proto.ubo.v1.Notification.prototype.setActions = function(value) {
  return jspb.Message.setWrapperField(this, 11, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearActions = function() {
  return this.setActions(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasActions = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional string icon = 12;
 * @return {string}
 */
proto.ubo.v1.Notification.prototype.getIcon = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 12, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setIcon = function(value) {
  return jspb.Message.setField(this, 12, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearIcon = function() {
  return jspb.Message.setField(this, 12, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasIcon = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional string color = 13;
 * @return {string}
 */
proto.ubo.v1.Notification.prototype.getColor = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 13, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setColor = function(value) {
  return jspb.Message.setField(this, 13, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearColor = function() {
  return jspb.Message.setField(this, 13, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasColor = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional int64 expiration_timestamp = 14;
 * @return {number}
 */
proto.ubo.v1.Notification.prototype.getExpirationTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 14, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setExpirationTimestamp = function(value) {
  return jspb.Message.setField(this, 14, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearExpirationTimestamp = function() {
  return jspb.Message.setField(this, 14, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasExpirationTimestamp = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional NotificationDisplayType display_type = 15;
 * @return {!proto.ubo.v1.NotificationDisplayType}
 */
proto.ubo.v1.Notification.prototype.getDisplayType = function() {
  return /** @type {!proto.ubo.v1.NotificationDisplayType} */ (jspb.Message.getFieldWithDefault(this, 15, 0));
};


/**
 * @param {!proto.ubo.v1.NotificationDisplayType} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setDisplayType = function(value) {
  return jspb.Message.setField(this, 15, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearDisplayType = function() {
  return jspb.Message.setField(this, 15, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasDisplayType = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional float flash_time = 16;
 * @return {number}
 */
proto.ubo.v1.Notification.prototype.getFlashTime = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 16, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setFlashTime = function(value) {
  return jspb.Message.setField(this, 16, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearFlashTime = function() {
  return jspb.Message.setField(this, 16, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasFlashTime = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional bool show_dismiss_action = 17;
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.getShowDismissAction = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 17, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setShowDismissAction = function(value) {
  return jspb.Message.setField(this, 17, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearShowDismissAction = function() {
  return jspb.Message.setField(this, 17, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasShowDismissAction = function() {
  return jspb.Message.getField(this, 17) != null;
};


/**
 * optional bool dismiss_on_close = 18;
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.getDismissOnClose = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 18, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setDismissOnClose = function(value) {
  return jspb.Message.setField(this, 18, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearDismissOnClose = function() {
  return jspb.Message.setField(this, 18, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasDismissOnClose = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional OnClose on_close = 19;
 * @return {?proto.ubo.v1.Notification.OnClose}
 */
proto.ubo.v1.Notification.prototype.getOnClose = function() {
  return /** @type{?proto.ubo.v1.Notification.OnClose} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification.OnClose, 19));
};


/**
 * @param {?proto.ubo.v1.Notification.OnClose|undefined} value
 * @return {!proto.ubo.v1.Notification} returns this
*/
proto.ubo.v1.Notification.prototype.setOnClose = function(value) {
  return jspb.Message.setWrapperField(this, 19, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearOnClose = function() {
  return this.setOnClose(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasOnClose = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * optional bool blink = 20;
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.getBlink = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 20, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setBlink = function(value) {
  return jspb.Message.setField(this, 20, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearBlink = function() {
  return jspb.Message.setField(this, 20, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasBlink = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * optional float progress = 21;
 * @return {number}
 */
proto.ubo.v1.Notification.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 21, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 21, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 21, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 21) != null;
};


/**
 * optional float progress_weight = 22;
 * @return {number}
 */
proto.ubo.v1.Notification.prototype.getProgressWeight = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 22, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.setProgressWeight = function(value) {
  return jspb.Message.setField(this, 22, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.Notification} returns this
 */
proto.ubo.v1.Notification.prototype.clearProgressWeight = function() {
  return jspb.Message.setField(this, 22, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Notification.prototype.hasProgressWeight = function() {
  return jspb.Message.getField(this, 22) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsAction}
 */
proto.ubo.v1.NotificationsAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsAction;
  return proto.ubo.v1.NotificationsAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsAction}
 */
proto.ubo.v1.NotificationsAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsAction} returns this
 */
proto.ubo.v1.NotificationsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsAction} returns this
 */
proto.ubo.v1.NotificationsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsAddAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsAddAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsAddAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsAddAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsAddAction}
 */
proto.ubo.v1.NotificationsAddAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsAddAction;
  return proto.ubo.v1.NotificationsAddAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsAddAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsAddAction}
 */
proto.ubo.v1.NotificationsAddAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Notification;
      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);
      msg.setNotification(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsAddAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsAddAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsAddAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsAddAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getNotification();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Notification.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsAddAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsAddAction} returns this
 */
proto.ubo.v1.NotificationsAddAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsAddAction} returns this
 */
proto.ubo.v1.NotificationsAddAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsAddAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Notification notification = 2;
 * @return {?proto.ubo.v1.Notification}
 */
proto.ubo.v1.NotificationsAddAction.prototype.getNotification = function() {
  return /** @type{?proto.ubo.v1.Notification} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));
};


/**
 * @param {?proto.ubo.v1.Notification|undefined} value
 * @return {!proto.ubo.v1.NotificationsAddAction} returns this
*/
proto.ubo.v1.NotificationsAddAction.prototype.setNotification = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationsAddAction} returns this
 */
proto.ubo.v1.NotificationsAddAction.prototype.clearNotification = function() {
  return this.setNotification(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsAddAction.prototype.hasNotification = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsDisplayAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsDisplayAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsDisplayAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f),
    index: jspb.Message.getFieldWithDefault(msg, 3, 0),
    count: jspb.Message.getFieldWithDefault(msg, 4, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsDisplayAction}
 */
proto.ubo.v1.NotificationsDisplayAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsDisplayAction;
  return proto.ubo.v1.NotificationsDisplayAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsDisplayAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsDisplayAction}
 */
proto.ubo.v1.NotificationsDisplayAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Notification;
      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);
      msg.setNotification(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setCount(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsDisplayAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsDisplayAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsDisplayAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getNotification();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Notification.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Notification notification = 2;
 * @return {?proto.ubo.v1.Notification}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.getNotification = function() {
  return /** @type{?proto.ubo.v1.Notification} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));
};


/**
 * @param {?proto.ubo.v1.Notification|undefined} value
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
*/
proto.ubo.v1.NotificationsDisplayAction.prototype.setNotification = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.clearNotification = function() {
  return this.setNotification(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.hasNotification = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 index = 3;
 * @return {number}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.setIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.clearIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.hasIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 count = 4;
 * @return {number}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.getCount = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.setCount = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayAction} returns this
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.clearCount = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayAction.prototype.hasCount = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsClearAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsClearAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsClearAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsClearAction}
 */
proto.ubo.v1.NotificationsClearAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsClearAction;
  return proto.ubo.v1.NotificationsClearAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsClearAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsClearAction}
 */
proto.ubo.v1.NotificationsClearAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Notification;
      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);
      msg.setNotification(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsClearAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsClearAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsClearAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getNotification();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Notification.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsClearAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsClearAction} returns this
 */
proto.ubo.v1.NotificationsClearAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsClearAction} returns this
 */
proto.ubo.v1.NotificationsClearAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsClearAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Notification notification = 2;
 * @return {?proto.ubo.v1.Notification}
 */
proto.ubo.v1.NotificationsClearAction.prototype.getNotification = function() {
  return /** @type{?proto.ubo.v1.Notification} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));
};


/**
 * @param {?proto.ubo.v1.Notification|undefined} value
 * @return {!proto.ubo.v1.NotificationsClearAction} returns this
*/
proto.ubo.v1.NotificationsClearAction.prototype.setNotification = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationsClearAction} returns this
 */
proto.ubo.v1.NotificationsClearAction.prototype.clearNotification = function() {
  return this.setNotification(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsClearAction.prototype.hasNotification = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsClearByIdAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsClearByIdAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearByIdAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsClearByIdAction}
 */
proto.ubo.v1.NotificationsClearByIdAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsClearByIdAction;
  return proto.ubo.v1.NotificationsClearByIdAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsClearByIdAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsClearByIdAction}
 */
proto.ubo.v1.NotificationsClearByIdAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsClearByIdAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsClearByIdAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearByIdAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsClearByIdAction} returns this
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsClearByIdAction} returns this
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsClearByIdAction} returns this
 */
proto.ubo.v1.NotificationsClearByIdAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsClearAllAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsClearAllAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsClearAllAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearAllAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsClearAllAction}
 */
proto.ubo.v1.NotificationsClearAllAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsClearAllAction;
  return proto.ubo.v1.NotificationsClearAllAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsClearAllAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsClearAllAction}
 */
proto.ubo.v1.NotificationsClearAllAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsClearAllAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsClearAllAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsClearAllAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearAllAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsClearAllAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsClearAllAction} returns this
 */
proto.ubo.v1.NotificationsClearAllAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsClearAllAction} returns this
 */
proto.ubo.v1.NotificationsClearAllAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsClearAllAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsEvent}
 */
proto.ubo.v1.NotificationsEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsEvent;
  return proto.ubo.v1.NotificationsEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsEvent}
 */
proto.ubo.v1.NotificationsEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsEvent} returns this
 */
proto.ubo.v1.NotificationsEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsEvent} returns this
 */
proto.ubo.v1.NotificationsEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsClearEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsClearEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsClearEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsClearEvent}
 */
proto.ubo.v1.NotificationsClearEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsClearEvent;
  return proto.ubo.v1.NotificationsClearEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsClearEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsClearEvent}
 */
proto.ubo.v1.NotificationsClearEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Notification;
      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);
      msg.setNotification(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsClearEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsClearEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsClearEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsClearEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getNotification();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Notification.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsClearEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsClearEvent} returns this
 */
proto.ubo.v1.NotificationsClearEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsClearEvent} returns this
 */
proto.ubo.v1.NotificationsClearEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsClearEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Notification notification = 2;
 * @return {?proto.ubo.v1.Notification}
 */
proto.ubo.v1.NotificationsClearEvent.prototype.getNotification = function() {
  return /** @type{?proto.ubo.v1.Notification} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));
};


/**
 * @param {?proto.ubo.v1.Notification|undefined} value
 * @return {!proto.ubo.v1.NotificationsClearEvent} returns this
*/
proto.ubo.v1.NotificationsClearEvent.prototype.setNotification = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationsClearEvent} returns this
 */
proto.ubo.v1.NotificationsClearEvent.prototype.clearNotification = function() {
  return this.setNotification(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsClearEvent.prototype.hasNotification = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsDisplayEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsDisplayEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsDisplayEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    notification: (f = msg.getNotification()) && proto.ubo.v1.Notification.toObject(includeInstance, f),
    index: jspb.Message.getFieldWithDefault(msg, 3, 0),
    count: jspb.Message.getFieldWithDefault(msg, 4, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsDisplayEvent}
 */
proto.ubo.v1.NotificationsDisplayEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsDisplayEvent;
  return proto.ubo.v1.NotificationsDisplayEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsDisplayEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsDisplayEvent}
 */
proto.ubo.v1.NotificationsDisplayEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Notification;
      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);
      msg.setNotification(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setIndex(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setCount(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsDisplayEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsDisplayEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsDisplayEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getNotification();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.Notification.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Notification notification = 2;
 * @return {?proto.ubo.v1.Notification}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.getNotification = function() {
  return /** @type{?proto.ubo.v1.Notification} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.Notification, 2));
};


/**
 * @param {?proto.ubo.v1.Notification|undefined} value
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
*/
proto.ubo.v1.NotificationsDisplayEvent.prototype.setNotification = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.clearNotification = function() {
  return this.setNotification(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.hasNotification = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 index = 3;
 * @return {number}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.getIndex = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.setIndex = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.clearIndex = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.hasIndex = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 count = 4;
 * @return {number}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.getCount = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.setCount = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsDisplayEvent} returns this
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.clearCount = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsDisplayEvent.prototype.hasCount = function() {
  return jspb.Message.getField(this, 4) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.NotificationsState.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.NotificationsState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.NotificationsState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.NotificationsState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotNotifications: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    notificationsList: jspb.Message.toObjectList(msg.getNotificationsList(),
    proto.ubo.v1.Notification.toObject, includeInstance),
    unreadCount: jspb.Message.getFieldWithDefault(msg, 3, 0),
    progress: jspb.Message.getFloatingPointFieldWithDefault(msg, 4, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.NotificationsState}
 */
proto.ubo.v1.NotificationsState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.NotificationsState;
  return proto.ubo.v1.NotificationsState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.NotificationsState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.NotificationsState}
 */
proto.ubo.v1.NotificationsState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications(value);
      break;
    case 2:
      var value = new proto.ubo.v1.Notification;
      reader.readMessage(value,proto.ubo.v1.Notification.deserializeBinaryFromReader);
      msg.addNotifications(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setUnreadCount(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setProgress(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.NotificationsState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.NotificationsState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.NotificationsState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.NotificationsState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getNotificationsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.Notification.serializeBinaryToWriter
    );
  }
  f = message.getUnreadCount();
  if (f !== 0) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeFloat(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_notifications = 1000;
 * @return {string}
 */
proto.ubo.v1.NotificationsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.NotificationsState} returns this
 */
proto.ubo.v1.NotificationsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsState} returns this
 */
proto.ubo.v1.NotificationsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotNotifications = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated Notification notifications = 2;
 * @return {!Array<!proto.ubo.v1.Notification>}
 */
proto.ubo.v1.NotificationsState.prototype.getNotificationsList = function() {
  return /** @type{!Array<!proto.ubo.v1.Notification>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.Notification, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.Notification>} value
 * @return {!proto.ubo.v1.NotificationsState} returns this
*/
proto.ubo.v1.NotificationsState.prototype.setNotificationsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.Notification=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.Notification}
 */
proto.ubo.v1.NotificationsState.prototype.addNotifications = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.Notification, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.NotificationsState} returns this
 */
proto.ubo.v1.NotificationsState.prototype.clearNotificationsList = function() {
  return this.setNotificationsList([]);
};


/**
 * optional int64 unread_count = 3;
 * @return {number}
 */
proto.ubo.v1.NotificationsState.prototype.getUnreadCount = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationsState} returns this
 */
proto.ubo.v1.NotificationsState.prototype.setUnreadCount = function(value) {
  return jspb.Message.setProto3IntField(this, 3, value);
};


/**
 * optional float progress = 4;
 * @return {number}
 */
proto.ubo.v1.NotificationsState.prototype.getProgress = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 4, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.NotificationsState} returns this
 */
proto.ubo.v1.NotificationsState.prototype.setProgress = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.NotificationsState} returns this
 */
proto.ubo.v1.NotificationsState.prototype.clearProgress = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.NotificationsState.prototype.hasProgress = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingAction}
 */
proto.ubo.v1.RgbRingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingAction;
  return proto.ubo.v1.RgbRingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingAction}
 */
proto.ubo.v1.RgbRingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingAction} returns this
 */
proto.ubo.v1.RgbRingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingAction} returns this
 */
proto.ubo.v1.RgbRingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingEvent}
 */
proto.ubo.v1.RgbRingEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingEvent;
  return proto.ubo.v1.RgbRingEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingEvent}
 */
proto.ubo.v1.RgbRingEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingEvent} returns this
 */
proto.ubo.v1.RgbRingEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingEvent} returns this
 */
proto.ubo.v1.RgbRingEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingSetIsBusyAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingSetIsBusyAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetIsBusyAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isBusy: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingSetIsBusyAction}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingSetIsBusyAction;
  return proto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingSetIsBusyAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingSetIsBusyAction}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsBusy(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingSetIsBusyAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingSetIsBusyAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetIsBusyAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_busy = 2;
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.getIsBusy = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.setIsBusy = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetIsBusyAction} returns this
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.clearIsBusy = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetIsBusyAction.prototype.hasIsBusy = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingCommandAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingCommandAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingCommandAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingCommandAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingCommandAction}
 */
proto.ubo.v1.RgbRingCommandAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingCommandAction;
  return proto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingCommandAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingCommandAction}
 */
proto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingCommandAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingCommandAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingCommandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingCommandAction} returns this
 */
proto.ubo.v1.RgbRingCommandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingCommandAction} returns this
 */
proto.ubo.v1.RgbRingCommandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingCommandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingWaitableCommandAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingWaitableCommandAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingWaitableCommandAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    wait: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingWaitableCommandAction}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingWaitableCommandAction;
  return proto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingWaitableCommandAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingWaitableCommandAction}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWait(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingWaitableCommandAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingWaitableCommandAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingWaitableCommandAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 wait = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.getWait = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.setWait = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingWaitableCommandAction} returns this
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.clearWait = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingWaitableCommandAction.prototype.hasWait = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingColorfulCommandAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingColorfulCommandAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingColorfulCommandAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingColorfulCommandAction}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingColorfulCommandAction;
  return proto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingColorfulCommandAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingColorfulCommandAction}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingColorfulCommandAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingColorfulCommandAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingColorfulCommandAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional RgbColor color = 2;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 2));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this
*/
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingColorfulCommandAction} returns this
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingColorfulCommandAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingSetEnabledAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingSetEnabledAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetEnabledAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    enabled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingSetEnabledAction}
 */
proto.ubo.v1.RgbRingSetEnabledAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingSetEnabledAction;
  return proto.ubo.v1.RgbRingSetEnabledAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingSetEnabledAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingSetEnabledAction}
 */
proto.ubo.v1.RgbRingSetEnabledAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setEnabled(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingSetEnabledAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingSetEnabledAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetEnabledAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool enabled = 2;
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.getEnabled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.setEnabled = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetEnabledAction} returns this
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.clearEnabled = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetEnabledAction.prototype.hasEnabled = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingSetAllAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingSetAllAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetAllAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingSetAllAction}
 */
proto.ubo.v1.RgbRingSetAllAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingSetAllAction;
  return proto.ubo.v1.RgbRingSetAllAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingSetAllAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingSetAllAction}
 */
proto.ubo.v1.RgbRingSetAllAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingSetAllAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingSetAllAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetAllAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional RgbColor color = 2;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 2));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this
*/
proto.ubo.v1.RgbRingSetAllAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetAllAction} returns this
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetAllAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingSetBrightnessAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingSetBrightnessAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetBrightnessAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    brightness: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingSetBrightnessAction}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingSetBrightnessAction;
  return proto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingSetBrightnessAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingSetBrightnessAction}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setBrightness(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingSetBrightnessAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingSetBrightnessAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSetBrightnessAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float brightness = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.getBrightness = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.setBrightness = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSetBrightnessAction} returns this
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.clearBrightness = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSetBrightnessAction.prototype.hasBrightness = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingBlankAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingBlankAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingBlankAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingBlankAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingBlankAction}
 */
proto.ubo.v1.RgbRingBlankAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingBlankAction;
  return proto.ubo.v1.RgbRingBlankAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingBlankAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingBlankAction}
 */
proto.ubo.v1.RgbRingBlankAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingBlankAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingBlankAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingBlankAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingBlankAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingBlankAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingBlankAction} returns this
 */
proto.ubo.v1.RgbRingBlankAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingBlankAction} returns this
 */
proto.ubo.v1.RgbRingBlankAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingBlankAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingRainbowAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingRainbowAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingRainbowAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    wait: jspb.Message.getFieldWithDefault(msg, 2, 0),
    rounds: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingRainbowAction}
 */
proto.ubo.v1.RgbRingRainbowAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingRainbowAction;
  return proto.ubo.v1.RgbRingRainbowAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingRainbowAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingRainbowAction}
 */
proto.ubo.v1.RgbRingRainbowAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWait(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setRounds(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingRainbowAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingRainbowAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingRainbowAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = message.getRounds();
  if (f !== 0) {
    writer.writeInt64(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 wait = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.getWait = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.setWait = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.clearWait = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.hasWait = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 rounds = 3;
 * @return {number}
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.getRounds = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingRainbowAction} returns this
 */
proto.ubo.v1.RgbRingRainbowAction.prototype.setRounds = function(value) {
  return jspb.Message.setProto3IntField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingProgressWheelStepAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingProgressWheelStepAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingProgressWheelStepAction;
  return proto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingProgressWheelStepAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingProgressWheelStepAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingProgressWheelStepAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional RgbColor color = 2;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 2));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this
*/
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingProgressWheelStepAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingProgressWheelStepAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingPulseAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingPulseAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingPulseAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    repetitions: jspb.Message.getFieldWithDefault(msg, 2, 0),
    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingPulseAction}
 */
proto.ubo.v1.RgbRingPulseAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingPulseAction;
  return proto.ubo.v1.RgbRingPulseAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingPulseAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingPulseAction}
 */
proto.ubo.v1.RgbRingPulseAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setRepetitions(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWait(value);
      break;
    case 4:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingPulseAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingPulseAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingPulseAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
 */
proto.ubo.v1.RgbRingPulseAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
 */
proto.ubo.v1.RgbRingPulseAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 repetitions = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.getRepetitions = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
 */
proto.ubo.v1.RgbRingPulseAction.prototype.setRepetitions = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
 */
proto.ubo.v1.RgbRingPulseAction.prototype.clearRepetitions = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.hasRepetitions = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 wait = 3;
 * @return {number}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.getWait = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
 */
proto.ubo.v1.RgbRingPulseAction.prototype.setWait = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
 */
proto.ubo.v1.RgbRingPulseAction.prototype.clearWait = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.hasWait = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional RgbColor color = 4;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
*/
proto.ubo.v1.RgbRingPulseAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingPulseAction} returns this
 */
proto.ubo.v1.RgbRingPulseAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingPulseAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingBlinkAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingBlinkAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingBlinkAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    repetitions: jspb.Message.getFieldWithDefault(msg, 2, 0),
    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingBlinkAction}
 */
proto.ubo.v1.RgbRingBlinkAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingBlinkAction;
  return proto.ubo.v1.RgbRingBlinkAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingBlinkAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingBlinkAction}
 */
proto.ubo.v1.RgbRingBlinkAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setRepetitions(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWait(value);
      break;
    case 4:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingBlinkAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingBlinkAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingBlinkAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 repetitions = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.getRepetitions = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.setRepetitions = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.clearRepetitions = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.hasRepetitions = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 wait = 3;
 * @return {number}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.getWait = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.setWait = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.clearWait = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.hasWait = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional RgbColor color = 4;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
*/
proto.ubo.v1.RgbRingBlinkAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingBlinkAction} returns this
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingBlinkAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingSpinningWheelAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingSpinningWheelAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSpinningWheelAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    length: jspb.Message.getFieldWithDefault(msg, 2, 0),
    repetitions: jspb.Message.getFieldWithDefault(msg, 3, 0),
    wait: jspb.Message.getFieldWithDefault(msg, 4, 0),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingSpinningWheelAction;
  return proto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingSpinningWheelAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setLength(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setRepetitions(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWait(value);
      break;
    case 5:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingSpinningWheelAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingSpinningWheelAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSpinningWheelAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 length = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.getLength = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.setLength = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearLength = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasLength = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 repetitions = 3;
 * @return {number}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.getRepetitions = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.setRepetitions = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearRepetitions = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasRepetitions = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 wait = 4;
 * @return {number}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.getWait = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.setWait = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearWait = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasWait = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional RgbColor color = 5;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 5));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
*/
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 5, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSpinningWheelAction} returns this
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSpinningWheelAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 5) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingProgressWheelAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingProgressWheelAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingProgressWheelAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    percentage: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction}
 */
proto.ubo.v1.RgbRingProgressWheelAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingProgressWheelAction;
  return proto.ubo.v1.RgbRingProgressWheelAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingProgressWheelAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction}
 */
proto.ubo.v1.RgbRingProgressWheelAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPercentage(value);
      break;
    case 3:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingProgressWheelAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingProgressWheelAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingProgressWheelAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float percentage = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.getPercentage = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.setPercentage = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.clearPercentage = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.hasPercentage = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional RgbColor color = 3;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 3));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this
*/
proto.ubo.v1.RgbRingProgressWheelAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingProgressWheelAction} returns this
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingProgressWheelAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingFillUptoAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingFillUptoAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingFillUptoAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    percentage: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingFillUptoAction}
 */
proto.ubo.v1.RgbRingFillUptoAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingFillUptoAction;
  return proto.ubo.v1.RgbRingFillUptoAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingFillUptoAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingFillUptoAction}
 */
proto.ubo.v1.RgbRingFillUptoAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPercentage(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWait(value);
      break;
    case 4:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingFillUptoAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingFillUptoAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingFillUptoAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float percentage = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.getPercentage = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.setPercentage = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.clearPercentage = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.hasPercentage = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 wait = 3;
 * @return {number}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.getWait = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.setWait = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.clearWait = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.hasWait = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional RgbColor color = 4;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
*/
proto.ubo.v1.RgbRingFillUptoAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillUptoAction} returns this
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillUptoAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingFillDownfromAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingFillDownfromAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingFillDownfromAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    percentage: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    wait: jspb.Message.getFieldWithDefault(msg, 3, 0),
    color: (f = msg.getColor()) && proto.ubo.v1.RgbColor.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction}
 */
proto.ubo.v1.RgbRingFillDownfromAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingFillDownfromAction;
  return proto.ubo.v1.RgbRingFillDownfromAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingFillDownfromAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction}
 */
proto.ubo.v1.RgbRingFillDownfromAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setPercentage(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setWait(value);
      break;
    case 4:
      var value = new proto.ubo.v1.RgbColor;
      reader.readMessage(value,proto.ubo.v1.RgbColor.deserializeBinaryFromReader);
      msg.setColor(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingFillDownfromAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingFillDownfromAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingFillDownfromAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
  f = message.getColor();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.RgbColor.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float percentage = 2;
 * @return {number}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.getPercentage = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.setPercentage = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.clearPercentage = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.hasPercentage = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 wait = 3;
 * @return {number}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.getWait = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.setWait = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.clearWait = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.hasWait = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional RgbColor color = 4;
 * @return {?proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.getColor = function() {
  return /** @type{?proto.ubo.v1.RgbColor} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbColor, 4));
};


/**
 * @param {?proto.ubo.v1.RgbColor|undefined} value
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
*/
proto.ubo.v1.RgbRingFillDownfromAction.prototype.setColor = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RgbRingFillDownfromAction} returns this
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.clearColor = function() {
  return this.setColor(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingFillDownfromAction.prototype.hasColor = function() {
  return jspb.Message.getField(this, 4) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.RgbRingSequenceAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingSequenceAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingSequenceAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSequenceAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sequenceList: jspb.Message.toObjectList(msg.getSequenceList(),
    proto.ubo.v1.RgbRingCommandAction.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingSequenceAction}
 */
proto.ubo.v1.RgbRingSequenceAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingSequenceAction;
  return proto.ubo.v1.RgbRingSequenceAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingSequenceAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingSequenceAction}
 */
proto.ubo.v1.RgbRingSequenceAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = new proto.ubo.v1.RgbRingCommandAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader);
      msg.addSequence(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingSequenceAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingSequenceAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingSequenceAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSequenceList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated RgbRingCommandAction sequence = 2;
 * @return {!Array<!proto.ubo.v1.RgbRingCommandAction>}
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.getSequenceList = function() {
  return /** @type{!Array<!proto.ubo.v1.RgbRingCommandAction>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.RgbRingCommandAction, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.RgbRingCommandAction>} value
 * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this
*/
proto.ubo.v1.RgbRingSequenceAction.prototype.setSequenceList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.RgbRingCommandAction=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.RgbRingCommandAction}
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.addSequence = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.RgbRingCommandAction, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.RgbRingSequenceAction} returns this
 */
proto.ubo.v1.RgbRingSequenceAction.prototype.clearSequenceList = function() {
  return this.setSequenceList([]);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.RgbRingCommandEvent.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingCommandEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingCommandEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingCommandEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    commandList: (f = jspb.Message.getRepeatedField(msg, 2)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingCommandEvent}
 */
proto.ubo.v1.RgbRingCommandEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingCommandEvent;
  return proto.ubo.v1.RgbRingCommandEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingCommandEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingCommandEvent}
 */
proto.ubo.v1.RgbRingCommandEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.addCommand(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingCommandEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingCommandEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingCommandEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getCommandList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated string command = 2;
 * @return {!Array<string>}
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.getCommandList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 2));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.setCommandList = function(value) {
  return jspb.Message.setField(this, 2, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.addCommand = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 2, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.RgbRingCommandEvent} returns this
 */
proto.ubo.v1.RgbRingCommandEvent.prototype.clearCommandList = function() {
  return this.setCommandList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbRingState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbRingState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbRingState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isBusy: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbRingState}
 */
proto.ubo.v1.RgbRingState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbRingState;
  return proto.ubo.v1.RgbRingState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbRingState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbRingState}
 */
proto.ubo.v1.RgbRingState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsBusy(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbRingState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbRingState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbRingState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbRingState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsBusy();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rgb_ring = 1000;
 * @return {string}
 */
proto.ubo.v1.RgbRingState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RgbRingState} returns this
 */
proto.ubo.v1.RgbRingState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbRingState} returns this
 */
proto.ubo.v1.RgbRingState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbRingState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRgbRing = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_busy = 2;
 * @return {boolean}
 */
proto.ubo.v1.RgbRingState.prototype.getIsBusy = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RgbRingState} returns this
 */
proto.ubo.v1.RgbRingState.prototype.setIsBusy = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.RgbColorElement.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.RgbColorElement.RgbColorElementCase = {
  RGB_COLOR_ELEMENT_NOT_SET: 0,
  FLOAT: 1,
  INT64: 2
};

/**
 * @return {proto.ubo.v1.RgbColorElement.RgbColorElementCase}
 */
proto.ubo.v1.RgbColorElement.prototype.getRgbColorElementCase = function() {
  return /** @type {proto.ubo.v1.RgbColorElement.RgbColorElementCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.RgbColorElement.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbColorElement.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbColorElement.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbColorElement} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbColorElement.toObject = function(includeInstance, msg) {
  var f, obj = {
    pb_float: jspb.Message.getFloatingPointFieldWithDefault(msg, 1, 0.0),
    int64: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbColorElement}
 */
proto.ubo.v1.RgbColorElement.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbColorElement;
  return proto.ubo.v1.RgbColorElement.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbColorElement} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbColorElement}
 */
proto.ubo.v1.RgbColorElement.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setFloat(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setInt64(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbColorElement.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbColorElement.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbColorElement} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbColorElement.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {number} */ (jspb.Message.getField(message, 1));
  if (f != null) {
    writer.writeFloat(
      1,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
};


/**
 * optional float float = 1;
 * @return {number}
 */
proto.ubo.v1.RgbColorElement.prototype.getFloat = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 1, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbColorElement} returns this
 */
proto.ubo.v1.RgbColorElement.prototype.setFloat = function(value) {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.RgbColorElement.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbColorElement} returns this
 */
proto.ubo.v1.RgbColorElement.prototype.clearFloat = function() {
  return jspb.Message.setOneofField(this, 1, proto.ubo.v1.RgbColorElement.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbColorElement.prototype.hasFloat = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional int64 int64 = 2;
 * @return {number}
 */
proto.ubo.v1.RgbColorElement.prototype.getInt64 = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RgbColorElement} returns this
 */
proto.ubo.v1.RgbColorElement.prototype.setInt64 = function(value) {
  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.RgbColorElement.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RgbColorElement} returns this
 */
proto.ubo.v1.RgbColorElement.prototype.clearInt64 = function() {
  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.RgbColorElement.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RgbColorElement.prototype.hasInt64 = function() {
  return jspb.Message.getField(this, 2) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.RgbColor.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RgbColor.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RgbColor.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RgbColor} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbColor.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.RgbColorElement.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbColor.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RgbColor;
  return proto.ubo.v1.RgbColor.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RgbColor} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RgbColor}
 */
proto.ubo.v1.RgbColor.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.RgbColorElement;
      reader.readMessage(value,proto.ubo.v1.RgbColorElement.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RgbColor.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RgbColor.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RgbColor} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RgbColor.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.RgbColorElement.serializeBinaryToWriter
    );
  }
};


/**
 * repeated RgbColorElement items = 1;
 * @return {!Array<!proto.ubo.v1.RgbColorElement>}
 */
proto.ubo.v1.RgbColor.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.RgbColorElement>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.RgbColorElement, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.RgbColorElement>} value
 * @return {!proto.ubo.v1.RgbColor} returns this
*/
proto.ubo.v1.RgbColor.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.RgbColorElement=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.RgbColorElement}
 */
proto.ubo.v1.RgbColor.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.RgbColorElement, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.RgbColor} returns this
 */
proto.ubo.v1.RgbColor.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectAction}
 */
proto.ubo.v1.RPiConnectAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectAction;
  return proto.ubo.v1.RPiConnectAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectAction}
 */
proto.ubo.v1.RPiConnectAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectAction} returns this
 */
proto.ubo.v1.RPiConnectAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectAction} returns this
 */
proto.ubo.v1.RPiConnectAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectEvent}
 */
proto.ubo.v1.RPiConnectEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectEvent;
  return proto.ubo.v1.RPiConnectEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectEvent}
 */
proto.ubo.v1.RPiConnectEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectEvent} returns this
 */
proto.ubo.v1.RPiConnectEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectEvent} returns this
 */
proto.ubo.v1.RPiConnectEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectStartDownloadingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectStartDownloadingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction}
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectStartDownloadingAction;
  return proto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectStartDownloadingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction}
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectStartDownloadingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectStartDownloadingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction} returns this
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectStartDownloadingAction} returns this
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectStartDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectDoneDownloadingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectDoneDownloadingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction}
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectDoneDownloadingAction;
  return proto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectDoneDownloadingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction}
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectDoneDownloadingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectDoneDownloadingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction} returns this
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectDoneDownloadingAction} returns this
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectDoneDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectSetPendingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectSetPendingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectSetPendingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectSetPendingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectSetPendingAction}
 */
proto.ubo.v1.RPiConnectSetPendingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectSetPendingAction;
  return proto.ubo.v1.RPiConnectSetPendingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectSetPendingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectSetPendingAction}
 */
proto.ubo.v1.RPiConnectSetPendingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectSetPendingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectSetPendingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectSetPendingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectSetPendingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectSetPendingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectSetPendingAction} returns this
 */
proto.ubo.v1.RPiConnectSetPendingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectSetPendingAction} returns this
 */
proto.ubo.v1.RPiConnectSetPendingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectSetPendingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectStatus.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectStatus.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectStatus} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectStatus.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    screenSharingSessions: jspb.Message.getFieldWithDefault(msg, 2, 0),
    remoteShellSessions: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectStatus}
 */
proto.ubo.v1.RPiConnectStatus.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectStatus;
  return proto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectStatus} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectStatus}
 */
proto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setScreenSharingSessions(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setRemoteShellSessions(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectStatus.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectStatus} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeInt64(
      2,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeInt64(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectStatus.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectStatus} returns this
 */
proto.ubo.v1.RPiConnectStatus.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectStatus} returns this
 */
proto.ubo.v1.RPiConnectStatus.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectStatus.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional int64 screen_sharing_sessions = 2;
 * @return {number}
 */
proto.ubo.v1.RPiConnectStatus.prototype.getScreenSharingSessions = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RPiConnectStatus} returns this
 */
proto.ubo.v1.RPiConnectStatus.prototype.setScreenSharingSessions = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectStatus} returns this
 */
proto.ubo.v1.RPiConnectStatus.prototype.clearScreenSharingSessions = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectStatus.prototype.hasScreenSharingSessions = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional int64 remote_shell_sessions = 3;
 * @return {number}
 */
proto.ubo.v1.RPiConnectStatus.prototype.getRemoteShellSessions = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.RPiConnectStatus} returns this
 */
proto.ubo.v1.RPiConnectStatus.prototype.setRemoteShellSessions = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectStatus} returns this
 */
proto.ubo.v1.RPiConnectStatus.prototype.clearRemoteShellSessions = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectStatus.prototype.hasRemoteShellSessions = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectSetStatusAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectSetStatusAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectSetStatusAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isSignedIn: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    status: (f = msg.getStatus()) && proto.ubo.v1.RPiConnectStatus.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction}
 */
proto.ubo.v1.RPiConnectSetStatusAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectSetStatusAction;
  return proto.ubo.v1.RPiConnectSetStatusAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectSetStatusAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction}
 */
proto.ubo.v1.RPiConnectSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsInstalled(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsSignedIn(value);
      break;
    case 4:
      var value = new proto.ubo.v1.RPiConnectStatus;
      reader.readMessage(value,proto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader);
      msg.setStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectSetStatusAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectSetStatusAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectSetStatusAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsInstalled();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = message.getStatus();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_installed = 2;
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.getIsInstalled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.setIsInstalled = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional bool is_signed_in = 3;
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.getIsSignedIn = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.setIsSignedIn = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.clearIsSignedIn = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.hasIsSignedIn = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional RPiConnectStatus status = 4;
 * @return {?proto.ubo.v1.RPiConnectStatus}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.getStatus = function() {
  return /** @type{?proto.ubo.v1.RPiConnectStatus} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectStatus, 4));
};


/**
 * @param {?proto.ubo.v1.RPiConnectStatus|undefined} value
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this
*/
proto.ubo.v1.RPiConnectSetStatusAction.prototype.setStatus = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectSetStatusAction} returns this
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.clearStatus = function() {
  return this.setStatus(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectSetStatusAction.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectLoginEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectLoginEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectLoginEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectLoginEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectLoginEvent}
 */
proto.ubo.v1.RPiConnectLoginEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectLoginEvent;
  return proto.ubo.v1.RPiConnectLoginEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectLoginEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectLoginEvent}
 */
proto.ubo.v1.RPiConnectLoginEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectLoginEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectLoginEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectLoginEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectLoginEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectLoginEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectLoginEvent} returns this
 */
proto.ubo.v1.RPiConnectLoginEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectLoginEvent} returns this
 */
proto.ubo.v1.RPiConnectLoginEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectLoginEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectUpdateServiceStateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectUpdateServiceStateAction;
  return proto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectUpdateServiceStateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.setIsActive = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectUpdateServiceStateAction} returns this
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.clearIsActive = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectUpdateServiceStateAction.prototype.hasIsActive = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.RPiConnectState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.RPiConnectState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.RPiConnectState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isDownloading: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    isInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    isSignedIn: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    status: (f = msg.getStatus()) && proto.ubo.v1.RPiConnectStatus.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.RPiConnectState}
 */
proto.ubo.v1.RPiConnectState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.RPiConnectState;
  return proto.ubo.v1.RPiConnectState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.RPiConnectState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.RPiConnectState}
 */
proto.ubo.v1.RPiConnectState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsDownloading(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsInstalled(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsSignedIn(value);
      break;
    case 6:
      var value = new proto.ubo.v1.RPiConnectStatus;
      reader.readMessage(value,proto.ubo.v1.RPiConnectStatus.deserializeBinaryFromReader);
      msg.setStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.RPiConnectState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.RPiConnectState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.RPiConnectState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.RPiConnectState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = message.getStatus();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.ubo.v1.RPiConnectStatus.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_rpi_connect = 1000;
 * @return {string}
 */
proto.ubo.v1.RPiConnectState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotRpiConnect = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_downloading = 2;
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.getIsDownloading = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.setIsDownloading = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.clearIsDownloading = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.hasIsDownloading = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_active = 3;
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.setIsActive = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.clearIsActive = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.hasIsActive = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool is_installed = 4;
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.getIsInstalled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.setIsInstalled = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.clearIsInstalled = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.hasIsInstalled = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_signed_in = 5;
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.getIsSignedIn = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.setIsSignedIn = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.clearIsSignedIn = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.hasIsSignedIn = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional RPiConnectStatus status = 6;
 * @return {?proto.ubo.v1.RPiConnectStatus}
 */
proto.ubo.v1.RPiConnectState.prototype.getStatus = function() {
  return /** @type{?proto.ubo.v1.RPiConnectStatus} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectStatus, 6));
};


/**
 * @param {?proto.ubo.v1.RPiConnectStatus|undefined} value
 * @return {!proto.ubo.v1.RPiConnectState} returns this
*/
proto.ubo.v1.RPiConnectState.prototype.setStatus = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.RPiConnectState} returns this
 */
proto.ubo.v1.RPiConnectState.prototype.clearStatus = function() {
  return this.setStatus(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.RPiConnectState.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SensorsAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SensorsAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SensorsAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorsAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SensorsAction}
 */
proto.ubo.v1.SensorsAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SensorsAction;
  return proto.ubo.v1.SensorsAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SensorsAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SensorsAction}
 */
proto.ubo.v1.SensorsAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SensorsAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SensorsAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SensorsAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorsAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;
 * @return {string}
 */
proto.ubo.v1.SensorsAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SensorsAction} returns this
 */
proto.ubo.v1.SensorsAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SensorsAction} returns this
 */
proto.ubo.v1.SensorsAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SensorsAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SensorsReportReadingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SensorsReportReadingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorsReportReadingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    sensor: jspb.Message.getFieldWithDefault(msg, 2, 0),
    reading: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),
    timestamp: jspb.Message.getFieldWithDefault(msg, 4, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SensorsReportReadingAction}
 */
proto.ubo.v1.SensorsReportReadingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SensorsReportReadingAction;
  return proto.ubo.v1.SensorsReportReadingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SensorsReportReadingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SensorsReportReadingAction}
 */
proto.ubo.v1.SensorsReportReadingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.Sensor} */ (reader.readEnum());
      msg.setSensor(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setReading(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setTimestamp(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SensorsReportReadingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SensorsReportReadingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorsReportReadingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSensor();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getReading();
  if (f !== 0.0) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0) {
    writer.writeInt64(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;
 * @return {string}
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Sensor sensor = 2;
 * @return {!proto.ubo.v1.Sensor}
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.getSensor = function() {
  return /** @type {!proto.ubo.v1.Sensor} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.Sensor} value
 * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.setSensor = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional float reading = 3;
 * @return {number}
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.getReading = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.setReading = function(value) {
  return jspb.Message.setProto3FloatField(this, 3, value);
};


/**
 * optional int64 timestamp = 4;
 * @return {number}
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SensorsReportReadingAction} returns this
 */
proto.ubo.v1.SensorsReportReadingAction.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3IntField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SensorState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SensorState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SensorState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    value: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SensorState}
 */
proto.ubo.v1.SensorState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SensorState;
  return proto.ubo.v1.SensorState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SensorState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SensorState}
 */
proto.ubo.v1.SensorState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setValue(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SensorState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SensorState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SensorState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeFloat(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;
 * @return {string}
 */
proto.ubo.v1.SensorState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SensorState} returns this
 */
proto.ubo.v1.SensorState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SensorState} returns this
 */
proto.ubo.v1.SensorState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SensorState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float value = 2;
 * @return {number}
 */
proto.ubo.v1.SensorState.prototype.getValue = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SensorState} returns this
 */
proto.ubo.v1.SensorState.prototype.setValue = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SensorState} returns this
 */
proto.ubo.v1.SensorState.prototype.clearValue = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SensorState.prototype.hasValue = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SensorsState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SensorsState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SensorsState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorsState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSensors: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    temperature: (f = msg.getTemperature()) && proto.ubo.v1.SensorState.toObject(includeInstance, f),
    light: (f = msg.getLight()) && proto.ubo.v1.SensorState.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SensorsState}
 */
proto.ubo.v1.SensorsState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SensorsState;
  return proto.ubo.v1.SensorsState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SensorsState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SensorsState}
 */
proto.ubo.v1.SensorsState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors(value);
      break;
    case 2:
      var value = new proto.ubo.v1.SensorState;
      reader.readMessage(value,proto.ubo.v1.SensorState.deserializeBinaryFromReader);
      msg.setTemperature(value);
      break;
    case 3:
      var value = new proto.ubo.v1.SensorState;
      reader.readMessage(value,proto.ubo.v1.SensorState.deserializeBinaryFromReader);
      msg.setLight(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SensorsState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SensorsState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SensorsState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SensorsState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTemperature();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.SensorState.serializeBinaryToWriter
    );
  }
  f = message.getLight();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.SensorState.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_sensors = 1000;
 * @return {string}
 */
proto.ubo.v1.SensorsState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SensorsState} returns this
 */
proto.ubo.v1.SensorsState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SensorsState} returns this
 */
proto.ubo.v1.SensorsState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SensorsState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSensors = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional SensorState temperature = 2;
 * @return {?proto.ubo.v1.SensorState}
 */
proto.ubo.v1.SensorsState.prototype.getTemperature = function() {
  return /** @type{?proto.ubo.v1.SensorState} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorState, 2));
};


/**
 * @param {?proto.ubo.v1.SensorState|undefined} value
 * @return {!proto.ubo.v1.SensorsState} returns this
*/
proto.ubo.v1.SensorsState.prototype.setTemperature = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SensorsState} returns this
 */
proto.ubo.v1.SensorsState.prototype.clearTemperature = function() {
  return this.setTemperature(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SensorsState.prototype.hasTemperature = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional SensorState light = 3;
 * @return {?proto.ubo.v1.SensorState}
 */
proto.ubo.v1.SensorsState.prototype.getLight = function() {
  return /** @type{?proto.ubo.v1.SensorState} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorState, 3));
};


/**
 * @param {?proto.ubo.v1.SensorState|undefined} value
 * @return {!proto.ubo.v1.SensorsState} returns this
*/
proto.ubo.v1.SensorsState.prototype.setLight = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SensorsState} returns this
 */
proto.ubo.v1.SensorsState.prototype.clearLight = function() {
  return this.setLight(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SensorsState.prototype.hasLight = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionAction}
 */
proto.ubo.v1.SpeechRecognitionAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionAction;
  return proto.ubo.v1.SpeechRecognitionAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionAction}
 */
proto.ubo.v1.SpeechRecognitionAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    engineName: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction;
  return proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (reader.readEnum());
      msg.setEngineName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional SpeechRecognitionEngineName engine_name = 2;
 * @return {!proto.ubo.v1.SpeechRecognitionEngineName}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.getEngineName = function() {
  return /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.SpeechRecognitionEngineName} value
 * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.setEngineName = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.clearEngineName = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.prototype.hasEngineName = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction}
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction;
  return proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction}
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsActive();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.prototype.setIsActive = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction}
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction;
  return proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction}
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsActive();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} returns this
 */
proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.prototype.setIsActive = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionIntent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionIntent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    phrase: (f = msg.getPhrase()) && proto.ubo.v1.SpeechRecognitionIntent.Phrase.toObject(includeInstance, f),
    action: (f = msg.getAction()) && proto.ubo.v1.SpeechRecognitionIntent.Action.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent}
 */
proto.ubo.v1.SpeechRecognitionIntent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionIntent;
  return proto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent}
 */
proto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = new proto.ubo.v1.SpeechRecognitionIntent.Phrase;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinaryFromReader);
      msg.setPhrase(value);
      break;
    case 3:
      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader);
      msg.setAction(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getPhrase();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.Phrase.serializeBinaryToWriter
    );
  }
  f = message.getAction();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionIntent.Phrase1.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: (f = jspb.Message.getRepeatedField(msg, 1)) == null ? undefined : f
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Phrase1;
  return proto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = /** @type {string} */ (reader.readString());
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionIntent.Phrase1.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedString(
      1,
      f
    );
  }
};


/**
 * repeated string items = 1;
 * @return {!Array<string>}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.getItemsList = function() {
  return /** @type {!Array<string>} */ (jspb.Message.getRepeatedField(this, 1));
};


/**
 * @param {!Array<string>} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.setItemsList = function(value) {
  return jspb.Message.setField(this, 1, value || []);
};


/**
 * @param {string} value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.addItems = function(value, opt_index) {
  return jspb.Message.addToRepeatedField(this, 1, value, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase1} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase1.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase = {
  PHRASE_NOT_SET: 0,
  LIST: 1,
  STRING: 2
};

/**
 * @return {proto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.getPhraseCase = function() {
  return /** @type {proto.ubo.v1.SpeechRecognitionIntent.Phrase.PhraseCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionIntent.Phrase.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.toObject = function(includeInstance, msg) {
  var f, obj = {
    list: (f = msg.getList()) && proto.ubo.v1.SpeechRecognitionIntent.Phrase1.toObject(includeInstance, f),
    string: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Phrase;
  return proto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.SpeechRecognitionIntent.Phrase1;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Phrase1.deserializeBinaryFromReader);
      msg.setList(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setString(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionIntent.Phrase.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getList();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.Phrase1.serializeBinaryToWriter
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional Phrase1 list = 1;
 * @return {?proto.ubo.v1.SpeechRecognitionIntent.Phrase1}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.getList = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Phrase1} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Phrase1, 1));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionIntent.Phrase1|undefined} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this
*/
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.setList = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.clearList = function() {
  return this.setList(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.hasList = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional string string = 2;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.getString = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.setString = function(value) {
  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0], value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Phrase} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.clearString = function() {
  return jspb.Message.setOneofField(this, 2, proto.ubo.v1.SpeechRecognitionIntent.Phrase.oneofGroups_[0], undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionIntent.Phrase.prototype.hasString = function() {
  return jspb.Message.getField(this, 2) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionIntent.Action2.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action2} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.SpeechRecognitionIntent.Action.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Action2;
  return proto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action2} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionIntent.Action2.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action2} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter
    );
  }
};


/**
 * repeated Action items = 1;
 * @return {!Array<!proto.ubo.v1.SpeechRecognitionIntent.Action>}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.SpeechRecognitionIntent.Action>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.SpeechRecognitionIntent.Action>} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2} returns this
*/
proto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.SpeechRecognitionIntent.Action, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action2} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Action2.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_ = [[1,2]];

/**
 * @enum {number}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase = {
  ACTION_NOT_SET: 0,
  UBO_ACTION: 1,
  LIST: 2
};

/**
 * @return {proto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.getActionCase = function() {
  return /** @type {proto.ubo.v1.SpeechRecognitionIntent.Action.ActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionIntent.Action.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.toObject = function(includeInstance, msg) {
  var f, obj = {
    uboAction: (f = msg.getUboAction()) && proto.ubo.v1.SpeechRecognitionIntent.Action.toObject(includeInstance, f),
    list: (f = msg.getList()) && proto.ubo.v1.SpeechRecognitionIntent.Action2.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionIntent.Action;
  return proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action.deserializeBinaryFromReader);
      msg.setUboAction(value);
      break;
    case 2:
      var value = new proto.ubo.v1.SpeechRecognitionIntent.Action2;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.Action2.deserializeBinaryFromReader);
      msg.setList(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionIntent.Action} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getUboAction();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.Action.serializeBinaryToWriter
    );
  }
  f = message.getList();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.Action2.serializeBinaryToWriter
    );
  }
};


/**
 * optional Action ubo_action = 1;
 * @return {?proto.ubo.v1.SpeechRecognitionIntent.Action}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.getUboAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Action} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action, 1));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionIntent.Action|undefined} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this
*/
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.setUboAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.clearUboAction = function() {
  return this.setUboAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.hasUboAction = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional Action2 list = 2;
 * @return {?proto.ubo.v1.SpeechRecognitionIntent.Action2}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.getList = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Action2} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action2, 2));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionIntent.Action2|undefined} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this
*/
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.setList = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.SpeechRecognitionIntent.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent.Action} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.clearList = function() {
  return this.setList(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionIntent.Action.prototype.hasList = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Phrase phrase = 2;
 * @return {?proto.ubo.v1.SpeechRecognitionIntent.Phrase}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.getPhrase = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Phrase} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Phrase, 2));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionIntent.Phrase|undefined} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this
*/
proto.ubo.v1.SpeechRecognitionIntent.prototype.setPhrase = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.clearPhrase = function() {
  return this.setPhrase(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.hasPhrase = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional Action action = 3;
 * @return {?proto.ubo.v1.SpeechRecognitionIntent.Action}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.getAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent.Action} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent.Action, 3));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionIntent.Action|undefined} value
 * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this
*/
proto.ubo.v1.SpeechRecognitionIntent.prototype.setAction = function(value) {
  return jspb.Message.setWrapperField(this, 3, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionIntent} returns this
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.clearAction = function() {
  return this.setAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionIntent.prototype.hasAction = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    wakeWord: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction}
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction;
  return proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction}
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setWakeWord(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getWakeWord();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string wake_word = 2;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.getWakeWord = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.prototype.setWakeWord = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    intent: (f = msg.getIntent()) && proto.ubo.v1.SpeechRecognitionIntent.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction;
  return proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = new proto.ubo.v1.SpeechRecognitionIntent;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader);
      msg.setIntent(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIntent();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional SpeechRecognitionIntent intent = 2;
 * @return {?proto.ubo.v1.SpeechRecognitionIntent}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.getIntent = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionIntent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent, 2));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionIntent|undefined} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this
*/
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.setIntent = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.clearIntent = function() {
  return this.setIntent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.prototype.hasIntent = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionReportSpeechAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    engineName: jspb.Message.getFieldWithDefault(msg, 2, 0),
    text: jspb.Message.getFieldWithDefault(msg, 3, ""),
    audio: msg.getAudio_asB64()
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionReportSpeechAction;
  return proto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (reader.readEnum());
      msg.setEngineName(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 4:
      var value = /** @type {!Uint8Array} */ (reader.readBytes());
      msg.setAudio(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionReportSpeechAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getEngineName();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getText();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
  f = message.getAudio_asU8();
  if (f.length > 0) {
    writer.writeBytes(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional SpeechRecognitionEngineName engine_name = 2;
 * @return {!proto.ubo.v1.SpeechRecognitionEngineName}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getEngineName = function() {
  return /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.SpeechRecognitionEngineName} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setEngineName = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};


/**
 * optional string text = 3;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getText = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setText = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};


/**
 * optional bytes audio = 4;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getAudio = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * optional bytes audio = 4;
 * This is a type-conversion wrapper around `getAudio()`
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getAudio_asB64 = function() {
  return /** @type {string} */ (jspb.Message.bytesAsB64(
      this.getAudio()));
};


/**
 * optional bytes audio = 4;
 * Note that Uint8Array is not supported on all browsers.
 * @see http://caniuse.com/Uint8Array
 * This is a type-conversion wrapper around `getAudio()`
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.getAudio_asU8 = function() {
  return /** @type {!Uint8Array} */ (jspb.Message.bytesAsU8(
      this.getAudio()));
};


/**
 * @param {!(string|Uint8Array)} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportSpeechAction} returns this
 */
proto.ubo.v1.SpeechRecognitionReportSpeechAction.prototype.setAudio = function(value) {
  return jspb.Message.setProto3BytesField(this, 4, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionEvent}
 */
proto.ubo.v1.SpeechRecognitionEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionEvent;
  return proto.ubo.v1.SpeechRecognitionEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionEvent}
 */
proto.ubo.v1.SpeechRecognitionEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionEvent} returns this
 */
proto.ubo.v1.SpeechRecognitionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionEvent} returns this
 */
proto.ubo.v1.SpeechRecognitionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionReportTextEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionReportTextEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    timestamp: jspb.Message.getFloatingPointFieldWithDefault(msg, 2, 0.0),
    text: jspb.Message.getFieldWithDefault(msg, 3, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionReportTextEvent;
  return proto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionReportTextEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setTimestamp(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionReportTextEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionReportTextEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getTimestamp();
  if (f !== 0.0) {
    writer.writeFloat(
      2,
      f
    );
  }
  f = message.getText();
  if (f.length > 0) {
    writer.writeString(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional float timestamp = 2;
 * @return {number}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.getTimestamp = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 2, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.setTimestamp = function(value) {
  return jspb.Message.setProto3FloatField(this, 2, value);
};


/**
 * optional string text = 3;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.getText = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionReportTextEvent} returns this
 */
proto.ubo.v1.SpeechRecognitionReportTextEvent.prototype.setText = function(value) {
  return jspb.Message.setProto3StringField(this, 3, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.SpeechRecognitionState.repeatedFields_ = [3];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechRecognitionState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechRecognitionState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    selectedEngine: jspb.Message.getFieldWithDefault(msg, 2, 0),
    intentsList: jspb.Message.toObjectList(msg.getIntentsList(),
    proto.ubo.v1.SpeechRecognitionIntent.toObject, includeInstance),
    isIntentsActive: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    isAssistantActive: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    status: jspb.Message.getFieldWithDefault(msg, 6, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechRecognitionState}
 */
proto.ubo.v1.SpeechRecognitionState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechRecognitionState;
  return proto.ubo.v1.SpeechRecognitionState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechRecognitionState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechRecognitionState}
 */
proto.ubo.v1.SpeechRecognitionState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (reader.readEnum());
      msg.setSelectedEngine(value);
      break;
    case 3:
      var value = new proto.ubo.v1.SpeechRecognitionIntent;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionIntent.deserializeBinaryFromReader);
      msg.addIntents(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsIntentsActive(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsAssistantActive(value);
      break;
    case 6:
      var value = /** @type {!proto.ubo.v1.SpeechRecognitionStatus} */ (reader.readEnum());
      msg.setStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechRecognitionState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechRecognitionState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechRecognitionState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeEnum(
      2,
      f
    );
  }
  f = message.getIntentsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      3,
      f,
      proto.ubo.v1.SpeechRecognitionIntent.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.SpeechRecognitionStatus} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_recognition = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechRecognition = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional SpeechRecognitionEngineName selected_engine = 2;
 * @return {!proto.ubo.v1.SpeechRecognitionEngineName}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.getSelectedEngine = function() {
  return /** @type {!proto.ubo.v1.SpeechRecognitionEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.SpeechRecognitionEngineName} value
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.setSelectedEngine = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.clearSelectedEngine = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.hasSelectedEngine = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * repeated SpeechRecognitionIntent intents = 3;
 * @return {!Array<!proto.ubo.v1.SpeechRecognitionIntent>}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.getIntentsList = function() {
  return /** @type{!Array<!proto.ubo.v1.SpeechRecognitionIntent>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.SpeechRecognitionIntent, 3));
};


/**
 * @param {!Array<!proto.ubo.v1.SpeechRecognitionIntent>} value
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
*/
proto.ubo.v1.SpeechRecognitionState.prototype.setIntentsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 3, value);
};


/**
 * @param {!proto.ubo.v1.SpeechRecognitionIntent=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.SpeechRecognitionIntent}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.addIntents = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 3, opt_value, proto.ubo.v1.SpeechRecognitionIntent, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.clearIntentsList = function() {
  return this.setIntentsList([]);
};


/**
 * optional bool is_intents_active = 4;
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.getIsIntentsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.setIsIntentsActive = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.clearIsIntentsActive = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.hasIsIntentsActive = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_assistant_active = 5;
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.getIsAssistantActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.setIsAssistantActive = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.clearIsAssistantActive = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.hasIsAssistantActive = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional SpeechRecognitionStatus status = 6;
 * @return {!proto.ubo.v1.SpeechRecognitionStatus}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.getStatus = function() {
  return /** @type {!proto.ubo.v1.SpeechRecognitionStatus} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {!proto.ubo.v1.SpeechRecognitionStatus} value
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.setStatus = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechRecognitionState} returns this
 */
proto.ubo.v1.SpeechRecognitionState.prototype.clearStatus = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechRecognitionState.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechSynthesisAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechSynthesisAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechSynthesisAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechSynthesisAction}
 */
proto.ubo.v1.SpeechSynthesisAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechSynthesisAction;
  return proto.ubo.v1.SpeechSynthesisAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechSynthesisAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechSynthesisAction}
 */
proto.ubo.v1.SpeechSynthesisAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechSynthesisAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechSynthesisAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechSynthesisAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechSynthesisAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechSynthesisAction} returns this
 */
proto.ubo.v1.SpeechSynthesisAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisAction} returns this
 */
proto.ubo.v1.SpeechSynthesisAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechSynthesisEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechSynthesisEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechSynthesisEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechSynthesisEvent}
 */
proto.ubo.v1.SpeechSynthesisEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechSynthesisEvent;
  return proto.ubo.v1.SpeechSynthesisEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechSynthesisEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechSynthesisEvent}
 */
proto.ubo.v1.SpeechSynthesisEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechSynthesisEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechSynthesisEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechSynthesisEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechSynthesisEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechSynthesisEvent} returns this
 */
proto.ubo.v1.SpeechSynthesisEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisEvent} returns this
 */
proto.ubo.v1.SpeechSynthesisEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isAccessKeySet: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus}
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus;
  return proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus}
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsAccessKeySet(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsAccessKeySet();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} returns this
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} returns this
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_access_key_set = 2;
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.getIsAccessKeySet = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus} returns this
 */
proto.ubo.v1.SpeechSynthesisUpdateAccessKeyStatus.prototype.setIsAccessKeySet = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    engineName: jspb.Message.getFieldWithDefault(msg, 2, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction}
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction;
  return proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction}
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    case 2:
      var value = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (reader.readEnum());
      msg.setEngineName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getEngineName();
  if (f !== 0.0) {
    writer.writeEnum(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} returns this
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} returns this
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional SpeechSynthesisEngineName engine_name = 2;
 * @return {!proto.ubo.v1.SpeechSynthesisEngineName}
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.getEngineName = function() {
  return /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getFieldWithDefault(this, 2, 0));
};


/**
 * @param {!proto.ubo.v1.SpeechSynthesisEngineName} value
 * @return {!proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} returns this
 */
proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.prototype.setEngineName = function(value) {
  return jspb.Message.setProto3EnumField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.ReadableInformation.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.ReadableInformation.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.ReadableInformation} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReadableInformation.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    text: jspb.Message.getFieldWithDefault(msg, 2, ""),
    piperText: jspb.Message.getFieldWithDefault(msg, 3, ""),
    picovoiceText: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.ReadableInformation}
 */
proto.ubo.v1.ReadableInformation.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.ReadableInformation;
  return proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.ReadableInformation} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.ReadableInformation}
 */
proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setText(value);
      break;
    case 3:
      var value = /** @type {string} */ (reader.readString());
      msg.setPiperText(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setPicovoiceText(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.ReadableInformation.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.ReadableInformation.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.ReadableInformation} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.ReadableInformation.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getText();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeString(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.ReadableInformation.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ReadableInformation} returns this
 */
proto.ubo.v1.ReadableInformation.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ReadableInformation} returns this
 */
proto.ubo.v1.ReadableInformation.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ReadableInformation.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string text = 2;
 * @return {string}
 */
proto.ubo.v1.ReadableInformation.prototype.getText = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ReadableInformation} returns this
 */
proto.ubo.v1.ReadableInformation.prototype.setText = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional string piper_text = 3;
 * @return {string}
 */
proto.ubo.v1.ReadableInformation.prototype.getPiperText = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 3, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ReadableInformation} returns this
 */
proto.ubo.v1.ReadableInformation.prototype.setPiperText = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ReadableInformation} returns this
 */
proto.ubo.v1.ReadableInformation.prototype.clearPiperText = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ReadableInformation.prototype.hasPiperText = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional string picovoice_text = 4;
 * @return {string}
 */
proto.ubo.v1.ReadableInformation.prototype.getPicovoiceText = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.ReadableInformation} returns this
 */
proto.ubo.v1.ReadableInformation.prototype.setPicovoiceText = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.ReadableInformation} returns this
 */
proto.ubo.v1.ReadableInformation.prototype.clearPicovoiceText = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.ReadableInformation.prototype.hasPicovoiceText = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechSynthesisReadTextAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechSynthesisReadTextAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    information: (f = msg.getInformation()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),
    speechRate: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0),
    engine: jspb.Message.getFieldWithDefault(msg, 4, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechSynthesisReadTextAction;
  return proto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechSynthesisReadTextAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    case 2:
      var value = new proto.ubo.v1.ReadableInformation;
      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);
      msg.setInformation(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setSpeechRate(value);
      break;
    case 4:
      var value = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (reader.readEnum());
      msg.setEngine(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechSynthesisReadTextAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechSynthesisReadTextAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getInformation();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeEnum(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional ReadableInformation information = 2;
 * @return {?proto.ubo.v1.ReadableInformation}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getInformation = function() {
  return /** @type{?proto.ubo.v1.ReadableInformation} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 2));
};


/**
 * @param {?proto.ubo.v1.ReadableInformation|undefined} value
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
*/
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setInformation = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearInformation = function() {
  return this.setInformation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasInformation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float speech_rate = 3;
 * @return {number}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getSpeechRate = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setSpeechRate = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearSpeechRate = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasSpeechRate = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional SpeechSynthesisEngineName engine = 4;
 * @return {!proto.ubo.v1.SpeechSynthesisEngineName}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.getEngine = function() {
  return /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {!proto.ubo.v1.SpeechSynthesisEngineName} value
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.setEngine = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisReadTextAction} returns this
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.clearEngine = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisReadTextAction.prototype.hasEngine = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    information: (f = msg.getInformation()) && proto.ubo.v1.ReadableInformation.toObject(includeInstance, f),
    speechRate: jspb.Message.getFloatingPointFieldWithDefault(msg, 3, 0.0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent;
  return proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    case 2:
      var value = new proto.ubo.v1.ReadableInformation;
      reader.readMessage(value,proto.ubo.v1.ReadableInformation.deserializeBinaryFromReader);
      msg.setInformation(value);
      break;
    case 3:
      var value = /** @type {number} */ (reader.readFloat());
      msg.setSpeechRate(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getInformation();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.ReadableInformation.serializeBinaryToWriter
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeFloat(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional ReadableInformation information = 2;
 * @return {?proto.ubo.v1.ReadableInformation}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.getInformation = function() {
  return /** @type{?proto.ubo.v1.ReadableInformation} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ReadableInformation, 2));
};


/**
 * @param {?proto.ubo.v1.ReadableInformation|undefined} value
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this
*/
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.setInformation = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.clearInformation = function() {
  return this.setInformation(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.hasInformation = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional float speech_rate = 3;
 * @return {number}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.getSpeechRate = function() {
  return /** @type {number} */ (jspb.Message.getFloatingPointFieldWithDefault(this, 3, 0.0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.setSpeechRate = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} returns this
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.clearSpeechRate = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.prototype.hasSpeechRate = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SpeechSynthesisState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SpeechSynthesisState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isAccessKeySet: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    selectedEngine: jspb.Message.getFieldWithDefault(msg, 3, 0)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SpeechSynthesisState}
 */
proto.ubo.v1.SpeechSynthesisState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SpeechSynthesisState;
  return proto.ubo.v1.SpeechSynthesisState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SpeechSynthesisState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SpeechSynthesisState}
 */
proto.ubo.v1.SpeechSynthesisState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsAccessKeySet(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (reader.readEnum());
      msg.setSelectedEngine(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SpeechSynthesisState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SpeechSynthesisState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SpeechSynthesisState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_speech_synthesis = 1000;
 * @return {string}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SpeechSynthesisState} returns this
 */
proto.ubo.v1.SpeechSynthesisState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisState} returns this
 */
proto.ubo.v1.SpeechSynthesisState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSpeechSynthesis = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_access_key_set = 2;
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.getIsAccessKeySet = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SpeechSynthesisState} returns this
 */
proto.ubo.v1.SpeechSynthesisState.prototype.setIsAccessKeySet = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisState} returns this
 */
proto.ubo.v1.SpeechSynthesisState.prototype.clearIsAccessKeySet = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.hasIsAccessKeySet = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional SpeechSynthesisEngineName selected_engine = 3;
 * @return {!proto.ubo.v1.SpeechSynthesisEngineName}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.getSelectedEngine = function() {
  return /** @type {!proto.ubo.v1.SpeechSynthesisEngineName} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.SpeechSynthesisEngineName} value
 * @return {!proto.ubo.v1.SpeechSynthesisState} returns this
 */
proto.ubo.v1.SpeechSynthesisState.prototype.setSelectedEngine = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SpeechSynthesisState} returns this
 */
proto.ubo.v1.SpeechSynthesisState.prototype.clearSelectedEngine = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SpeechSynthesisState.prototype.hasSelectedEngine = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SSHAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SSHAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SSHAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SSHAction}
 */
proto.ubo.v1.SSHAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SSHAction;
  return proto.ubo.v1.SSHAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SSHAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SSHAction}
 */
proto.ubo.v1.SSHAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SSHAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SSHAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SSHAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;
 * @return {string}
 */
proto.ubo.v1.SSHAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SSHAction} returns this
 */
proto.ubo.v1.SSHAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHAction} returns this
 */
proto.ubo.v1.SSHAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SSHUpdateStateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SSHUpdateStateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHUpdateStateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SSHUpdateStateAction}
 */
proto.ubo.v1.SSHUpdateStateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SSHUpdateStateAction;
  return proto.ubo.v1.SSHUpdateStateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SSHUpdateStateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SSHUpdateStateAction}
 */
proto.ubo.v1.SSHUpdateStateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsEnabled(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SSHUpdateStateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SSHUpdateStateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHUpdateStateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;
 * @return {string}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.setIsActive = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.clearIsActive = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.hasIsActive = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_enabled = 3;
 * @return {boolean}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.getIsEnabled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.setIsEnabled = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHUpdateStateAction} returns this
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.clearIsEnabled = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHUpdateStateAction.prototype.hasIsEnabled = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SSHClearEnabledStateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SSHClearEnabledStateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SSHClearEnabledStateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHClearEnabledStateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SSHClearEnabledStateAction}
 */
proto.ubo.v1.SSHClearEnabledStateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SSHClearEnabledStateAction;
  return proto.ubo.v1.SSHClearEnabledStateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SSHClearEnabledStateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SSHClearEnabledStateAction}
 */
proto.ubo.v1.SSHClearEnabledStateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SSHClearEnabledStateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SSHClearEnabledStateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SSHClearEnabledStateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHClearEnabledStateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;
 * @return {string}
 */
proto.ubo.v1.SSHClearEnabledStateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SSHClearEnabledStateAction} returns this
 */
proto.ubo.v1.SSHClearEnabledStateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHClearEnabledStateAction} returns this
 */
proto.ubo.v1.SSHClearEnabledStateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHClearEnabledStateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.SSHState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.SSHState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.SSHState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotSsh: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isActive: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isEnabled: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.SSHState}
 */
proto.ubo.v1.SSHState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.SSHState;
  return proto.ubo.v1.SSHState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.SSHState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.SSHState}
 */
proto.ubo.v1.SSHState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsActive(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsEnabled(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.SSHState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.SSHState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.SSHState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.SSHState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_ssh = 1000;
 * @return {string}
 */
proto.ubo.v1.SSHState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.SSHState} returns this
 */
proto.ubo.v1.SSHState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHState} returns this
 */
proto.ubo.v1.SSHState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotSsh = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_active = 2;
 * @return {boolean}
 */
proto.ubo.v1.SSHState.prototype.getIsActive = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SSHState} returns this
 */
proto.ubo.v1.SSHState.prototype.setIsActive = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHState} returns this
 */
proto.ubo.v1.SSHState.prototype.clearIsActive = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHState.prototype.hasIsActive = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_enabled = 3;
 * @return {boolean}
 */
proto.ubo.v1.SSHState.prototype.getIsEnabled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.SSHState} returns this
 */
proto.ubo.v1.SSHState.prototype.setIsEnabled = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.SSHState} returns this
 */
proto.ubo.v1.SSHState.prototype.clearIsEnabled = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.SSHState.prototype.hasIsEnabled = function() {
  return jspb.Message.getField(this, 3) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersAction}
 */
proto.ubo.v1.UsersAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersAction;
  return proto.ubo.v1.UsersAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersAction}
 */
proto.ubo.v1.UsersAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersAction} returns this
 */
proto.ubo.v1.UsersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersAction} returns this
 */
proto.ubo.v1.UsersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.UsersSetUsersAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersSetUsersAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersSetUsersAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersSetUsersAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersSetUsersAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    usersList: jspb.Message.toObjectList(msg.getUsersList(),
    proto.ubo.v1.UserState.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersSetUsersAction}
 */
proto.ubo.v1.UsersSetUsersAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersSetUsersAction;
  return proto.ubo.v1.UsersSetUsersAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersSetUsersAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersSetUsersAction}
 */
proto.ubo.v1.UsersSetUsersAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    case 2:
      var value = new proto.ubo.v1.UserState;
      reader.readMessage(value,proto.ubo.v1.UserState.deserializeBinaryFromReader);
      msg.addUsers(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersSetUsersAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersSetUsersAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersSetUsersAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersSetUsersAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getUsersList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.UserState.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersSetUsersAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersSetUsersAction} returns this
 */
proto.ubo.v1.UsersSetUsersAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersSetUsersAction} returns this
 */
proto.ubo.v1.UsersSetUsersAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersSetUsersAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated UserState users = 2;
 * @return {!Array<!proto.ubo.v1.UserState>}
 */
proto.ubo.v1.UsersSetUsersAction.prototype.getUsersList = function() {
  return /** @type{!Array<!proto.ubo.v1.UserState>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.UserState, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.UserState>} value
 * @return {!proto.ubo.v1.UsersSetUsersAction} returns this
*/
proto.ubo.v1.UsersSetUsersAction.prototype.setUsersList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.UserState=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.UserState}
 */
proto.ubo.v1.UsersSetUsersAction.prototype.addUsers = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.UserState, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.UsersSetUsersAction} returns this
 */
proto.ubo.v1.UsersSetUsersAction.prototype.clearUsersList = function() {
  return this.setUsersList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersCreateUserAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersCreateUserAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersCreateUserAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersCreateUserAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersCreateUserAction}
 */
proto.ubo.v1.UsersCreateUserAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersCreateUserAction;
  return proto.ubo.v1.UsersCreateUserAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersCreateUserAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersCreateUserAction}
 */
proto.ubo.v1.UsersCreateUserAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersCreateUserAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersCreateUserAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersCreateUserAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersCreateUserAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersCreateUserAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersCreateUserAction} returns this
 */
proto.ubo.v1.UsersCreateUserAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersCreateUserAction} returns this
 */
proto.ubo.v1.UsersCreateUserAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersCreateUserAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersDeleteUserAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersDeleteUserAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersDeleteUserAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersDeleteUserAction}
 */
proto.ubo.v1.UsersDeleteUserAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersDeleteUserAction;
  return proto.ubo.v1.UsersDeleteUserAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersDeleteUserAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersDeleteUserAction}
 */
proto.ubo.v1.UsersDeleteUserAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersDeleteUserAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersDeleteUserAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersDeleteUserAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersDeleteUserAction} returns this
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersDeleteUserAction} returns this
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersDeleteUserAction} returns this
 */
proto.ubo.v1.UsersDeleteUserAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersResetPasswordAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersResetPasswordAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersResetPasswordAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersResetPasswordAction}
 */
proto.ubo.v1.UsersResetPasswordAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersResetPasswordAction;
  return proto.ubo.v1.UsersResetPasswordAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersResetPasswordAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersResetPasswordAction}
 */
proto.ubo.v1.UsersResetPasswordAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersResetPasswordAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersResetPasswordAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersResetPasswordAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersResetPasswordAction} returns this
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersResetPasswordAction} returns this
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersResetPasswordAction} returns this
 */
proto.ubo.v1.UsersResetPasswordAction.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersEvent}
 */
proto.ubo.v1.UsersEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersEvent;
  return proto.ubo.v1.UsersEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersEvent}
 */
proto.ubo.v1.UsersEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersEvent} returns this
 */
proto.ubo.v1.UsersEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersEvent} returns this
 */
proto.ubo.v1.UsersEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersCreateUserEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersCreateUserEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersCreateUserEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersCreateUserEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersCreateUserEvent}
 */
proto.ubo.v1.UsersCreateUserEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersCreateUserEvent;
  return proto.ubo.v1.UsersCreateUserEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersCreateUserEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersCreateUserEvent}
 */
proto.ubo.v1.UsersCreateUserEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersCreateUserEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersCreateUserEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersCreateUserEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersCreateUserEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersCreateUserEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersCreateUserEvent} returns this
 */
proto.ubo.v1.UsersCreateUserEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersCreateUserEvent} returns this
 */
proto.ubo.v1.UsersCreateUserEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersCreateUserEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersDeleteUserEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersDeleteUserEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersDeleteUserEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersDeleteUserEvent}
 */
proto.ubo.v1.UsersDeleteUserEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersDeleteUserEvent;
  return proto.ubo.v1.UsersDeleteUserEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersDeleteUserEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersDeleteUserEvent}
 */
proto.ubo.v1.UsersDeleteUserEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersDeleteUserEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersDeleteUserEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersDeleteUserEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersDeleteUserEvent} returns this
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersDeleteUserEvent} returns this
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersDeleteUserEvent} returns this
 */
proto.ubo.v1.UsersDeleteUserEvent.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersResetPasswordEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersResetPasswordEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersResetPasswordEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersResetPasswordEvent}
 */
proto.ubo.v1.UsersResetPasswordEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersResetPasswordEvent;
  return proto.ubo.v1.UsersResetPasswordEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersResetPasswordEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersResetPasswordEvent}
 */
proto.ubo.v1.UsersResetPasswordEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersResetPasswordEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersResetPasswordEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersResetPasswordEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersResetPasswordEvent} returns this
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersResetPasswordEvent} returns this
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersResetPasswordEvent} returns this
 */
proto.ubo.v1.UsersResetPasswordEvent.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UserState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UserState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UserState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UserState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    id: jspb.Message.getFieldWithDefault(msg, 2, ""),
    isRemovable: jspb.Message.getBooleanFieldWithDefault(msg, 3, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UserState}
 */
proto.ubo.v1.UserState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UserState;
  return proto.ubo.v1.UserState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UserState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UserState}
 */
proto.ubo.v1.UserState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setId(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsRemovable(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UserState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UserState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UserState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UserState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getId();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = message.getIsRemovable();
  if (f) {
    writer.writeBool(
      3,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UserState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UserState} returns this
 */
proto.ubo.v1.UserState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UserState} returns this
 */
proto.ubo.v1.UserState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UserState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string id = 2;
 * @return {string}
 */
proto.ubo.v1.UserState.prototype.getId = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UserState} returns this
 */
proto.ubo.v1.UserState.prototype.setId = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional bool is_removable = 3;
 * @return {boolean}
 */
proto.ubo.v1.UserState.prototype.getIsRemovable = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.UserState} returns this
 */
proto.ubo.v1.UserState.prototype.setIsRemovable = function(value) {
  return jspb.Message.setProto3BooleanField(this, 3, value);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotUsers: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    users: (f = msg.getUsers()) && proto.ubo.v1.UsersState.Users.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersState}
 */
proto.ubo.v1.UsersState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersState;
  return proto.ubo.v1.UsersState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersState}
 */
proto.ubo.v1.UsersState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers(value);
      break;
    case 2:
      var value = new proto.ubo.v1.UsersState.Users;
      reader.readMessage(value,proto.ubo.v1.UsersState.Users.deserializeBinaryFromReader);
      msg.setUsers(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getUsers();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.UsersState.Users.serializeBinaryToWriter
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.UsersState.Users.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.UsersState.Users.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.UsersState.Users.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.UsersState.Users} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersState.Users.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.UserState.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.UsersState.Users}
 */
proto.ubo.v1.UsersState.Users.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.UsersState.Users;
  return proto.ubo.v1.UsersState.Users.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.UsersState.Users} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.UsersState.Users}
 */
proto.ubo.v1.UsersState.Users.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.UserState;
      reader.readMessage(value,proto.ubo.v1.UserState.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.UsersState.Users.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.UsersState.Users.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.UsersState.Users} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.UsersState.Users.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.UserState.serializeBinaryToWriter
    );
  }
};


/**
 * repeated UserState items = 1;
 * @return {!Array<!proto.ubo.v1.UserState>}
 */
proto.ubo.v1.UsersState.Users.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.UserState>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.UserState, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.UserState>} value
 * @return {!proto.ubo.v1.UsersState.Users} returns this
*/
proto.ubo.v1.UsersState.Users.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.UserState=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.UserState}
 */
proto.ubo.v1.UsersState.Users.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.UserState, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.UsersState.Users} returns this
 */
proto.ubo.v1.UsersState.Users.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_users = 1000;
 * @return {string}
 */
proto.ubo.v1.UsersState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.UsersState} returns this
 */
proto.ubo.v1.UsersState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.UsersState} returns this
 */
proto.ubo.v1.UsersState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotUsers = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Users users = 2;
 * @return {?proto.ubo.v1.UsersState.Users}
 */
proto.ubo.v1.UsersState.prototype.getUsers = function() {
  return /** @type{?proto.ubo.v1.UsersState.Users} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersState.Users, 2));
};


/**
 * @param {?proto.ubo.v1.UsersState.Users|undefined} value
 * @return {!proto.ubo.v1.UsersState} returns this
*/
proto.ubo.v1.UsersState.prototype.setUsers = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.UsersState} returns this
 */
proto.ubo.v1.UsersState.prototype.clearUsers = function() {
  return this.setUsers(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.UsersState.prototype.hasUsers = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeAction}
 */
proto.ubo.v1.VSCodeAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeAction;
  return proto.ubo.v1.VSCodeAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeAction}
 */
proto.ubo.v1.VSCodeAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeAction} returns this
 */
proto.ubo.v1.VSCodeAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeAction} returns this
 */
proto.ubo.v1.VSCodeAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeEvent}
 */
proto.ubo.v1.VSCodeEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeEvent;
  return proto.ubo.v1.VSCodeEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeEvent}
 */
proto.ubo.v1.VSCodeEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeEvent} returns this
 */
proto.ubo.v1.VSCodeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeEvent} returns this
 */
proto.ubo.v1.VSCodeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeStartDownloadingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeStartDownloadingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeStartDownloadingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeStartDownloadingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeStartDownloadingAction}
 */
proto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeStartDownloadingAction;
  return proto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeStartDownloadingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeStartDownloadingAction}
 */
proto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeStartDownloadingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeStartDownloadingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeStartDownloadingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeStartDownloadingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeStartDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeStartDownloadingAction} returns this
 */
proto.ubo.v1.VSCodeStartDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeStartDownloadingAction} returns this
 */
proto.ubo.v1.VSCodeStartDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeStartDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeDoneDownloadingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeDoneDownloadingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction}
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeDoneDownloadingAction;
  return proto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeDoneDownloadingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction}
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeDoneDownloadingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeDoneDownloadingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction} returns this
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeDoneDownloadingAction} returns this
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeDoneDownloadingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeSetPendingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeSetPendingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeSetPendingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeSetPendingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeSetPendingAction}
 */
proto.ubo.v1.VSCodeSetPendingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeSetPendingAction;
  return proto.ubo.v1.VSCodeSetPendingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeSetPendingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeSetPendingAction}
 */
proto.ubo.v1.VSCodeSetPendingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeSetPendingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeSetPendingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeSetPendingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeSetPendingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeSetPendingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeSetPendingAction} returns this
 */
proto.ubo.v1.VSCodeSetPendingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeSetPendingAction} returns this
 */
proto.ubo.v1.VSCodeSetPendingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeSetPendingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeStatus.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeStatus.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeStatus} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeStatus.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isServiceInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isRunning: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    name: jspb.Message.getFieldWithDefault(msg, 4, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeStatus}
 */
proto.ubo.v1.VSCodeStatus.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeStatus;
  return proto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeStatus} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeStatus}
 */
proto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsServiceInstalled(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsRunning(value);
      break;
    case 4:
      var value = /** @type {string} */ (reader.readString());
      msg.setName(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeStatus.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeStatus.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeStatus} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeStatus.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsServiceInstalled();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getIsRunning();
  if (f) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeString(
      4,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeStatus.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeStatus} returns this
 */
proto.ubo.v1.VSCodeStatus.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeStatus} returns this
 */
proto.ubo.v1.VSCodeStatus.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeStatus.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_service_installed = 2;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeStatus.prototype.getIsServiceInstalled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeStatus} returns this
 */
proto.ubo.v1.VSCodeStatus.prototype.setIsServiceInstalled = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional bool is_running = 3;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeStatus.prototype.getIsRunning = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeStatus} returns this
 */
proto.ubo.v1.VSCodeStatus.prototype.setIsRunning = function(value) {
  return jspb.Message.setProto3BooleanField(this, 3, value);
};


/**
 * optional string name = 4;
 * @return {string}
 */
proto.ubo.v1.VSCodeStatus.prototype.getName = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 4, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeStatus} returns this
 */
proto.ubo.v1.VSCodeStatus.prototype.setName = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeStatus} returns this
 */
proto.ubo.v1.VSCodeStatus.prototype.clearName = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeStatus.prototype.hasName = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeSetStatusAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeSetStatusAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeSetStatusAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isBinaryInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isLoggedIn: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    status: (f = msg.getStatus()) && proto.ubo.v1.VSCodeStatus.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeSetStatusAction}
 */
proto.ubo.v1.VSCodeSetStatusAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeSetStatusAction;
  return proto.ubo.v1.VSCodeSetStatusAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeSetStatusAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeSetStatusAction}
 */
proto.ubo.v1.VSCodeSetStatusAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsBinaryInstalled(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsLoggedIn(value);
      break;
    case 4:
      var value = new proto.ubo.v1.VSCodeStatus;
      reader.readMessage(value,proto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader);
      msg.setStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeSetStatusAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeSetStatusAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeSetStatusAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getIsBinaryInstalled();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
  f = message.getIsLoggedIn();
  if (f) {
    writer.writeBool(
      3,
      f
    );
  }
  f = message.getStatus();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.VSCodeStatus.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_binary_installed = 2;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.getIsBinaryInstalled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.setIsBinaryInstalled = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};


/**
 * optional bool is_logged_in = 3;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.getIsLoggedIn = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.setIsLoggedIn = function(value) {
  return jspb.Message.setProto3BooleanField(this, 3, value);
};


/**
 * optional VSCodeStatus status = 4;
 * @return {?proto.ubo.v1.VSCodeStatus}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.getStatus = function() {
  return /** @type{?proto.ubo.v1.VSCodeStatus} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeStatus, 4));
};


/**
 * @param {?proto.ubo.v1.VSCodeStatus|undefined} value
 * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this
*/
proto.ubo.v1.VSCodeSetStatusAction.prototype.setStatus = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.VSCodeSetStatusAction} returns this
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.clearStatus = function() {
  return this.setStatus(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeSetStatusAction.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeLoginEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeLoginEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeLoginEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeLoginEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeLoginEvent}
 */
proto.ubo.v1.VSCodeLoginEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeLoginEvent;
  return proto.ubo.v1.VSCodeLoginEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeLoginEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeLoginEvent}
 */
proto.ubo.v1.VSCodeLoginEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeLoginEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeLoginEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeLoginEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeLoginEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeLoginEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeLoginEvent} returns this
 */
proto.ubo.v1.VSCodeLoginEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeLoginEvent} returns this
 */
proto.ubo.v1.VSCodeLoginEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeLoginEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeRestartEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeRestartEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeRestartEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeRestartEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeRestartEvent}
 */
proto.ubo.v1.VSCodeRestartEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeRestartEvent;
  return proto.ubo.v1.VSCodeRestartEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeRestartEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeRestartEvent}
 */
proto.ubo.v1.VSCodeRestartEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeRestartEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeRestartEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeRestartEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeRestartEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeRestartEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeRestartEvent} returns this
 */
proto.ubo.v1.VSCodeRestartEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeRestartEvent} returns this
 */
proto.ubo.v1.VSCodeRestartEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeRestartEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.VSCodeState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.VSCodeState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.VSCodeState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotVscode: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    isPending: jspb.Message.getBooleanFieldWithDefault(msg, 2, false),
    isDownloading: jspb.Message.getBooleanFieldWithDefault(msg, 3, false),
    isBinaryInstalled: jspb.Message.getBooleanFieldWithDefault(msg, 4, false),
    isLoggedIn: jspb.Message.getBooleanFieldWithDefault(msg, 5, false),
    status: (f = msg.getStatus()) && proto.ubo.v1.VSCodeStatus.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.VSCodeState}
 */
proto.ubo.v1.VSCodeState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.VSCodeState;
  return proto.ubo.v1.VSCodeState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.VSCodeState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.VSCodeState}
 */
proto.ubo.v1.VSCodeState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsPending(value);
      break;
    case 3:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsDownloading(value);
      break;
    case 4:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsBinaryInstalled(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setIsLoggedIn(value);
      break;
    case 6:
      var value = new proto.ubo.v1.VSCodeStatus;
      reader.readMessage(value,proto.ubo.v1.VSCodeStatus.deserializeBinaryFromReader);
      msg.setStatus(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.VSCodeState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.VSCodeState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.VSCodeState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.VSCodeState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeBool(
      3,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeBool(
      4,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
  f = message.getStatus();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.ubo.v1.VSCodeStatus.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_vscode = 1000;
 * @return {string}
 */
proto.ubo.v1.VSCodeState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotVscode = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool is_pending = 2;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.getIsPending = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.setIsPending = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.clearIsPending = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.hasIsPending = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional bool is_downloading = 3;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.getIsDownloading = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 3, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.setIsDownloading = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.clearIsDownloading = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.hasIsDownloading = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional bool is_binary_installed = 4;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.getIsBinaryInstalled = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 4, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.setIsBinaryInstalled = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.clearIsBinaryInstalled = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.hasIsBinaryInstalled = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool is_logged_in = 5;
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.getIsLoggedIn = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.setIsLoggedIn = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.clearIsLoggedIn = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.hasIsLoggedIn = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional VSCodeStatus status = 6;
 * @return {?proto.ubo.v1.VSCodeStatus}
 */
proto.ubo.v1.VSCodeState.prototype.getStatus = function() {
  return /** @type{?proto.ubo.v1.VSCodeStatus} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeStatus, 6));
};


/**
 * @param {?proto.ubo.v1.VSCodeStatus|undefined} value
 * @return {!proto.ubo.v1.VSCodeState} returns this
*/
proto.ubo.v1.VSCodeState.prototype.setStatus = function(value) {
  return jspb.Message.setWrapperField(this, 6, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.VSCodeState} returns this
 */
proto.ubo.v1.VSCodeState.prototype.clearStatus = function() {
  return this.setStatus(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.VSCodeState.prototype.hasStatus = function() {
  return jspb.Message.getField(this, 6) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WebUIEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WebUIEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WebUIEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WebUIEvent}
 */
proto.ubo.v1.WebUIEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WebUIEvent;
  return proto.ubo.v1.WebUIEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WebUIEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WebUIEvent}
 */
proto.ubo.v1.WebUIEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WebUIEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WebUIEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WebUIEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;
 * @return {string}
 */
proto.ubo.v1.WebUIEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIEvent} returns this
 */
proto.ubo.v1.WebUIEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIEvent} returns this
 */
proto.ubo.v1.WebUIEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WebUIInitializeEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WebUIInitializeEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIInitializeEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    description: (f = msg.getDescription()) && proto.ubo.v1.WebUIInputDescription.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WebUIInitializeEvent}
 */
proto.ubo.v1.WebUIInitializeEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WebUIInitializeEvent;
  return proto.ubo.v1.WebUIInitializeEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WebUIInitializeEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WebUIInitializeEvent}
 */
proto.ubo.v1.WebUIInitializeEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);
      break;
    case 2:
      var value = new proto.ubo.v1.WebUIInputDescription;
      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader);
      msg.setDescription(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WebUIInitializeEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WebUIInitializeEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIInitializeEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getDescription();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;
 * @return {string}
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional WebUIInputDescription description = 2;
 * @return {?proto.ubo.v1.WebUIInputDescription}
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.getDescription = function() {
  return /** @type{?proto.ubo.v1.WebUIInputDescription} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInputDescription, 2));
};


/**
 * @param {?proto.ubo.v1.WebUIInputDescription|undefined} value
 * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this
*/
proto.ubo.v1.WebUIInitializeEvent.prototype.setDescription = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.WebUIInitializeEvent} returns this
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.clearDescription = function() {
  return this.setDescription(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIInitializeEvent.prototype.hasDescription = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WebUIStopEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WebUIStopEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WebUIStopEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIStopEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WebUIStopEvent}
 */
proto.ubo.v1.WebUIStopEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WebUIStopEvent;
  return proto.ubo.v1.WebUIStopEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WebUIStopEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WebUIStopEvent}
 */
proto.ubo.v1.WebUIStopEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WebUIStopEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WebUIStopEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WebUIStopEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIStopEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;
 * @return {string}
 */
proto.ubo.v1.WebUIStopEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIStopEvent} returns this
 */
proto.ubo.v1.WebUIStopEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIStopEvent} returns this
 */
proto.ubo.v1.WebUIStopEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIStopEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.getField(this, 1000) != null;
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.WebUIState.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WebUIState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WebUIState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WebUIState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWebUi: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    activeInputsList: jspb.Message.toObjectList(msg.getActiveInputsList(),
    proto.ubo.v1.WebUIInputDescription.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WebUIState}
 */
proto.ubo.v1.WebUIState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WebUIState;
  return proto.ubo.v1.WebUIState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WebUIState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WebUIState}
 */
proto.ubo.v1.WebUIState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi(value);
      break;
    case 2:
      var value = new proto.ubo.v1.WebUIInputDescription;
      reader.readMessage(value,proto.ubo.v1.WebUIInputDescription.deserializeBinaryFromReader);
      msg.addActiveInputs(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WebUIState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WebUIState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WebUIState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WebUIState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getActiveInputsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.WebUIInputDescription.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_web_ui = 1000;
 * @return {string}
 */
proto.ubo.v1.WebUIState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WebUIState} returns this
 */
proto.ubo.v1.WebUIState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WebUIState} returns this
 */
proto.ubo.v1.WebUIState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WebUIState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWebUi = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated WebUIInputDescription active_inputs = 2;
 * @return {!Array<!proto.ubo.v1.WebUIInputDescription>}
 */
proto.ubo.v1.WebUIState.prototype.getActiveInputsList = function() {
  return /** @type{!Array<!proto.ubo.v1.WebUIInputDescription>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.WebUIInputDescription, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.WebUIInputDescription>} value
 * @return {!proto.ubo.v1.WebUIState} returns this
*/
proto.ubo.v1.WebUIState.prototype.setActiveInputsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.WebUIInputDescription=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.WebUIInputDescription}
 */
proto.ubo.v1.WebUIState.prototype.addActiveInputs = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.WebUIInputDescription, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.WebUIState} returns this
 */
proto.ubo.v1.WebUIState.prototype.clearActiveInputsList = function() {
  return this.setActiveInputsList([]);
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiConnection.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiConnection.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiConnection} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiConnection.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    ssid: jspb.Message.getFieldWithDefault(msg, 2, ""),
    state: jspb.Message.getFieldWithDefault(msg, 3, 0),
    signalStrength: jspb.Message.getFieldWithDefault(msg, 4, 0),
    password: jspb.Message.getFieldWithDefault(msg, 5, ""),
    type: jspb.Message.getFieldWithDefault(msg, 6, 0),
    hidden: jspb.Message.getBooleanFieldWithDefault(msg, 7, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiConnection}
 */
proto.ubo.v1.WiFiConnection.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiConnection;
  return proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiConnection} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiConnection}
 */
proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    case 2:
      var value = /** @type {string} */ (reader.readString());
      msg.setSsid(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.ConnectionState} */ (reader.readEnum());
      msg.setState(value);
      break;
    case 4:
      var value = /** @type {number} */ (reader.readInt64());
      msg.setSignalStrength(value);
      break;
    case 5:
      var value = /** @type {string} */ (reader.readString());
      msg.setPassword(value);
      break;
    case 6:
      var value = /** @type {!proto.ubo.v1.WiFiType} */ (reader.readEnum());
      msg.setType(value);
      break;
    case 7:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setHidden(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiConnection.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiConnection.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiConnection} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiConnection.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getSsid();
  if (f.length > 0) {
    writer.writeString(
      2,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.ConnectionState} */ (jspb.Message.getField(message, 3));
  if (f != null) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = /** @type {number} */ (jspb.Message.getField(message, 4));
  if (f != null) {
    writer.writeInt64(
      4,
      f
    );
  }
  f = /** @type {string} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeString(
      5,
      f
    );
  }
  f = /** @type {!proto.ubo.v1.WiFiType} */ (jspb.Message.getField(message, 6));
  if (f != null) {
    writer.writeEnum(
      6,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 7));
  if (f != null) {
    writer.writeBool(
      7,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiConnection.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiConnection.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional string ssid = 2;
 * @return {string}
 */
proto.ubo.v1.WiFiConnection.prototype.getSsid = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 2, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.setSsid = function(value) {
  return jspb.Message.setProto3StringField(this, 2, value);
};


/**
 * optional ConnectionState state = 3;
 * @return {!proto.ubo.v1.ConnectionState}
 */
proto.ubo.v1.WiFiConnection.prototype.getState = function() {
  return /** @type {!proto.ubo.v1.ConnectionState} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.ConnectionState} value
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.setState = function(value) {
  return jspb.Message.setField(this, 3, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.clearState = function() {
  return jspb.Message.setField(this, 3, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiConnection.prototype.hasState = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional int64 signal_strength = 4;
 * @return {number}
 */
proto.ubo.v1.WiFiConnection.prototype.getSignalStrength = function() {
  return /** @type {number} */ (jspb.Message.getFieldWithDefault(this, 4, 0));
};


/**
 * @param {number} value
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.setSignalStrength = function(value) {
  return jspb.Message.setField(this, 4, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.clearSignalStrength = function() {
  return jspb.Message.setField(this, 4, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiConnection.prototype.hasSignalStrength = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional string password = 5;
 * @return {string}
 */
proto.ubo.v1.WiFiConnection.prototype.getPassword = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 5, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.setPassword = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.clearPassword = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiConnection.prototype.hasPassword = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional WiFiType type = 6;
 * @return {!proto.ubo.v1.WiFiType}
 */
proto.ubo.v1.WiFiConnection.prototype.getType = function() {
  return /** @type {!proto.ubo.v1.WiFiType} */ (jspb.Message.getFieldWithDefault(this, 6, 0));
};


/**
 * @param {!proto.ubo.v1.WiFiType} value
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.setType = function(value) {
  return jspb.Message.setField(this, 6, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.clearType = function() {
  return jspb.Message.setField(this, 6, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiConnection.prototype.hasType = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional bool hidden = 7;
 * @return {boolean}
 */
proto.ubo.v1.WiFiConnection.prototype.getHidden = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 7, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.setHidden = function(value) {
  return jspb.Message.setField(this, 7, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiConnection} returns this
 */
proto.ubo.v1.WiFiConnection.prototype.clearHidden = function() {
  return jspb.Message.setField(this, 7, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiConnection.prototype.hasHidden = function() {
  return jspb.Message.getField(this, 7) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiAction}
 */
proto.ubo.v1.WiFiAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiAction;
  return proto.ubo.v1.WiFiAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiAction}
 */
proto.ubo.v1.WiFiAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiAction} returns this
 */
proto.ubo.v1.WiFiAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiAction} returns this
 */
proto.ubo.v1.WiFiAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiInputConnectionAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiInputConnectionAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiInputConnectionAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiInputConnectionAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiInputConnectionAction}
 */
proto.ubo.v1.WiFiInputConnectionAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiInputConnectionAction;
  return proto.ubo.v1.WiFiInputConnectionAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiInputConnectionAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiInputConnectionAction}
 */
proto.ubo.v1.WiFiInputConnectionAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiInputConnectionAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiInputConnectionAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiInputConnectionAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiInputConnectionAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiInputConnectionAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiInputConnectionAction} returns this
 */
proto.ubo.v1.WiFiInputConnectionAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiInputConnectionAction} returns this
 */
proto.ubo.v1.WiFiInputConnectionAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiInputConnectionAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    hasVisitedOnboarding: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction}
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiSetHasVisitedOnboardingAction;
  return proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction}
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setHasVisitedOnboarding(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getHasVisitedOnboarding();
  if (f) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} returns this
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} returns this
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool has_visited_onboarding = 2;
 * @return {boolean}
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.getHasVisitedOnboarding = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} returns this
 */
proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.prototype.setHasVisitedOnboarding = function(value) {
  return jspb.Message.setProto3BooleanField(this, 2, value);
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.WiFiUpdateAction.repeatedFields_ = [2];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiUpdateAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiUpdateAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiUpdateAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    connectionsList: jspb.Message.toObjectList(msg.getConnectionsList(),
    proto.ubo.v1.WiFiConnection.toObject, includeInstance),
    state: jspb.Message.getFieldWithDefault(msg, 3, 0),
    currentConnection: (f = msg.getCurrentConnection()) && proto.ubo.v1.WiFiConnection.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiUpdateAction}
 */
proto.ubo.v1.WiFiUpdateAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiUpdateAction;
  return proto.ubo.v1.WiFiUpdateAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiUpdateAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiUpdateAction}
 */
proto.ubo.v1.WiFiUpdateAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    case 2:
      var value = new proto.ubo.v1.WiFiConnection;
      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);
      msg.addConnections(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.NetState} */ (reader.readEnum());
      msg.setState(value);
      break;
    case 4:
      var value = new proto.ubo.v1.WiFiConnection;
      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);
      msg.setCurrentConnection(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiUpdateAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiUpdateAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiUpdateAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getConnectionsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      2,
      f,
      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter
    );
  }
  f = message.getState();
  if (f !== 0.0) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = message.getCurrentConnection();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiUpdateAction} returns this
 */
proto.ubo.v1.WiFiUpdateAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiUpdateAction} returns this
 */
proto.ubo.v1.WiFiUpdateAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * repeated WiFiConnection connections = 2;
 * @return {!Array<!proto.ubo.v1.WiFiConnection>}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.getConnectionsList = function() {
  return /** @type{!Array<!proto.ubo.v1.WiFiConnection>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.WiFiConnection, 2));
};


/**
 * @param {!Array<!proto.ubo.v1.WiFiConnection>} value
 * @return {!proto.ubo.v1.WiFiUpdateAction} returns this
*/
proto.ubo.v1.WiFiUpdateAction.prototype.setConnectionsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 2, value);
};


/**
 * @param {!proto.ubo.v1.WiFiConnection=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.WiFiConnection}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.addConnections = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 2, opt_value, proto.ubo.v1.WiFiConnection, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.WiFiUpdateAction} returns this
 */
proto.ubo.v1.WiFiUpdateAction.prototype.clearConnectionsList = function() {
  return this.setConnectionsList([]);
};


/**
 * optional NetState state = 3;
 * @return {!proto.ubo.v1.NetState}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.getState = function() {
  return /** @type {!proto.ubo.v1.NetState} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.NetState} value
 * @return {!proto.ubo.v1.WiFiUpdateAction} returns this
 */
proto.ubo.v1.WiFiUpdateAction.prototype.setState = function(value) {
  return jspb.Message.setProto3EnumField(this, 3, value);
};


/**
 * optional WiFiConnection current_connection = 4;
 * @return {?proto.ubo.v1.WiFiConnection}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.getCurrentConnection = function() {
  return /** @type{?proto.ubo.v1.WiFiConnection} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiConnection, 4));
};


/**
 * @param {?proto.ubo.v1.WiFiConnection|undefined} value
 * @return {!proto.ubo.v1.WiFiUpdateAction} returns this
*/
proto.ubo.v1.WiFiUpdateAction.prototype.setCurrentConnection = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.WiFiUpdateAction} returns this
 */
proto.ubo.v1.WiFiUpdateAction.prototype.clearCurrentConnection = function() {
  return this.setCurrentConnection(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiUpdateAction.prototype.hasCurrentConnection = function() {
  return jspb.Message.getField(this, 4) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiUpdateRequestAction.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiUpdateRequestAction} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiUpdateRequestAction.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    reset: jspb.Message.getBooleanFieldWithDefault(msg, 2, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiUpdateRequestAction}
 */
proto.ubo.v1.WiFiUpdateRequestAction.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiUpdateRequestAction;
  return proto.ubo.v1.WiFiUpdateRequestAction.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiUpdateRequestAction} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiUpdateRequestAction}
 */
proto.ubo.v1.WiFiUpdateRequestAction.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    case 2:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setReset(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiUpdateRequestAction.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiUpdateRequestAction} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiUpdateRequestAction.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 2));
  if (f != null) {
    writer.writeBool(
      2,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional bool reset = 2;
 * @return {boolean}
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.getReset = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 2, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.setReset = function(value) {
  return jspb.Message.setField(this, 2, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiUpdateRequestAction} returns this
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.clearReset = function() {
  return jspb.Message.setField(this, 2, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiUpdateRequestAction.prototype.hasReset = function() {
  return jspb.Message.getField(this, 2) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiEvent}
 */
proto.ubo.v1.WiFiEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiEvent;
  return proto.ubo.v1.WiFiEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiEvent}
 */
proto.ubo.v1.WiFiEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiEvent} returns this
 */
proto.ubo.v1.WiFiEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiEvent} returns this
 */
proto.ubo.v1.WiFiEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiInputConnectionEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiInputConnectionEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiInputConnectionEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiInputConnectionEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiInputConnectionEvent}
 */
proto.ubo.v1.WiFiInputConnectionEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiInputConnectionEvent;
  return proto.ubo.v1.WiFiInputConnectionEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiInputConnectionEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiInputConnectionEvent}
 */
proto.ubo.v1.WiFiInputConnectionEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiInputConnectionEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiInputConnectionEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiInputConnectionEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiInputConnectionEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiInputConnectionEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiInputConnectionEvent} returns this
 */
proto.ubo.v1.WiFiInputConnectionEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiInputConnectionEvent} returns this
 */
proto.ubo.v1.WiFiInputConnectionEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiInputConnectionEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiUpdateRequestEvent.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiUpdateRequestEvent.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiUpdateRequestEvent} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiUpdateRequestEvent.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, "")
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiUpdateRequestEvent}
 */
proto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiUpdateRequestEvent;
  return proto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiUpdateRequestEvent} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiUpdateRequestEvent}
 */
proto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiUpdateRequestEvent.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiUpdateRequestEvent.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiUpdateRequestEvent} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiUpdateRequestEvent.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiUpdateRequestEvent.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiUpdateRequestEvent} returns this
 */
proto.ubo.v1.WiFiUpdateRequestEvent.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiUpdateRequestEvent} returns this
 */
proto.ubo.v1.WiFiUpdateRequestEvent.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiUpdateRequestEvent.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};





if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiState.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiState.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiState} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiState.toObject = function(includeInstance, msg) {
  var f, obj = {
    metaFieldPackageNameUboAppDotStoreDotServicesDotWifi: jspb.Message.getFieldWithDefault(msg, 1000, ""),
    connections: (f = msg.getConnections()) && proto.ubo.v1.WiFiState.Connections.toObject(includeInstance, f),
    state: jspb.Message.getFieldWithDefault(msg, 3, 0),
    currentConnection: (f = msg.getCurrentConnection()) && proto.ubo.v1.WiFiConnection.toObject(includeInstance, f),
    hasVisitedOnboarding: jspb.Message.getBooleanFieldWithDefault(msg, 5, false)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiState}
 */
proto.ubo.v1.WiFiState.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiState;
  return proto.ubo.v1.WiFiState.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiState} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiState}
 */
proto.ubo.v1.WiFiState.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1000:
      var value = /** @type {string} */ (reader.readString());
      msg.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi(value);
      break;
    case 2:
      var value = new proto.ubo.v1.WiFiState.Connections;
      reader.readMessage(value,proto.ubo.v1.WiFiState.Connections.deserializeBinaryFromReader);
      msg.setConnections(value);
      break;
    case 3:
      var value = /** @type {!proto.ubo.v1.NetState} */ (reader.readEnum());
      msg.setState(value);
      break;
    case 4:
      var value = new proto.ubo.v1.WiFiConnection;
      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);
      msg.setCurrentConnection(value);
      break;
    case 5:
      var value = /** @type {boolean} */ (reader.readBool());
      msg.setHasVisitedOnboarding(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiState.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiState.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiState} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiState.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = /** @type {string} */ (jspb.Message.getField(message, 1000));
  if (f != null) {
    writer.writeString(
      1000,
      f
    );
  }
  f = message.getConnections();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.WiFiState.Connections.serializeBinaryToWriter
    );
  }
  f = message.getState();
  if (f !== 0.0) {
    writer.writeEnum(
      3,
      f
    );
  }
  f = message.getCurrentConnection();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter
    );
  }
  f = /** @type {boolean} */ (jspb.Message.getField(message, 5));
  if (f != null) {
    writer.writeBool(
      5,
      f
    );
  }
};



/**
 * List of repeated fields within this message type.
 * @private {!Array<number>}
 * @const
 */
proto.ubo.v1.WiFiState.Connections.repeatedFields_ = [1];



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.WiFiState.Connections.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.WiFiState.Connections.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.WiFiState.Connections} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiState.Connections.toObject = function(includeInstance, msg) {
  var f, obj = {
    itemsList: jspb.Message.toObjectList(msg.getItemsList(),
    proto.ubo.v1.WiFiConnection.toObject, includeInstance)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.WiFiState.Connections}
 */
proto.ubo.v1.WiFiState.Connections.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.WiFiState.Connections;
  return proto.ubo.v1.WiFiState.Connections.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.WiFiState.Connections} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.WiFiState.Connections}
 */
proto.ubo.v1.WiFiState.Connections.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.WiFiConnection;
      reader.readMessage(value,proto.ubo.v1.WiFiConnection.deserializeBinaryFromReader);
      msg.addItems(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.WiFiState.Connections.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.WiFiState.Connections.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.WiFiState.Connections} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.WiFiState.Connections.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getItemsList();
  if (f.length > 0) {
    writer.writeRepeatedMessage(
      1,
      f,
      proto.ubo.v1.WiFiConnection.serializeBinaryToWriter
    );
  }
};


/**
 * repeated WiFiConnection items = 1;
 * @return {!Array<!proto.ubo.v1.WiFiConnection>}
 */
proto.ubo.v1.WiFiState.Connections.prototype.getItemsList = function() {
  return /** @type{!Array<!proto.ubo.v1.WiFiConnection>} */ (
    jspb.Message.getRepeatedWrapperField(this, proto.ubo.v1.WiFiConnection, 1));
};


/**
 * @param {!Array<!proto.ubo.v1.WiFiConnection>} value
 * @return {!proto.ubo.v1.WiFiState.Connections} returns this
*/
proto.ubo.v1.WiFiState.Connections.prototype.setItemsList = function(value) {
  return jspb.Message.setRepeatedWrapperField(this, 1, value);
};


/**
 * @param {!proto.ubo.v1.WiFiConnection=} opt_value
 * @param {number=} opt_index
 * @return {!proto.ubo.v1.WiFiConnection}
 */
proto.ubo.v1.WiFiState.Connections.prototype.addItems = function(opt_value, opt_index) {
  return jspb.Message.addToRepeatedWrapperField(this, 1, opt_value, proto.ubo.v1.WiFiConnection, opt_index);
};


/**
 * Clears the list making it empty but non-null.
 * @return {!proto.ubo.v1.WiFiState.Connections} returns this
 */
proto.ubo.v1.WiFiState.Connections.prototype.clearItemsList = function() {
  return this.setItemsList([]);
};


/**
 * optional string meta_field_package_name_ubo_app_dot_store_dot_services_dot_wifi = 1000;
 * @return {string}
 */
proto.ubo.v1.WiFiState.prototype.getMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return /** @type {string} */ (jspb.Message.getFieldWithDefault(this, 1000, ""));
};


/**
 * @param {string} value
 * @return {!proto.ubo.v1.WiFiState} returns this
 */
proto.ubo.v1.WiFiState.prototype.setMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function(value) {
  return jspb.Message.setField(this, 1000, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiState} returns this
 */
proto.ubo.v1.WiFiState.prototype.clearMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.setField(this, 1000, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiState.prototype.hasMetaFieldPackageNameUboAppDotStoreDotServicesDotWifi = function() {
  return jspb.Message.getField(this, 1000) != null;
};


/**
 * optional Connections connections = 2;
 * @return {?proto.ubo.v1.WiFiState.Connections}
 */
proto.ubo.v1.WiFiState.prototype.getConnections = function() {
  return /** @type{?proto.ubo.v1.WiFiState.Connections} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiState.Connections, 2));
};


/**
 * @param {?proto.ubo.v1.WiFiState.Connections|undefined} value
 * @return {!proto.ubo.v1.WiFiState} returns this
*/
proto.ubo.v1.WiFiState.prototype.setConnections = function(value) {
  return jspb.Message.setWrapperField(this, 2, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.WiFiState} returns this
 */
proto.ubo.v1.WiFiState.prototype.clearConnections = function() {
  return this.setConnections(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiState.prototype.hasConnections = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional NetState state = 3;
 * @return {!proto.ubo.v1.NetState}
 */
proto.ubo.v1.WiFiState.prototype.getState = function() {
  return /** @type {!proto.ubo.v1.NetState} */ (jspb.Message.getFieldWithDefault(this, 3, 0));
};


/**
 * @param {!proto.ubo.v1.NetState} value
 * @return {!proto.ubo.v1.WiFiState} returns this
 */
proto.ubo.v1.WiFiState.prototype.setState = function(value) {
  return jspb.Message.setProto3EnumField(this, 3, value);
};


/**
 * optional WiFiConnection current_connection = 4;
 * @return {?proto.ubo.v1.WiFiConnection}
 */
proto.ubo.v1.WiFiState.prototype.getCurrentConnection = function() {
  return /** @type{?proto.ubo.v1.WiFiConnection} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiConnection, 4));
};


/**
 * @param {?proto.ubo.v1.WiFiConnection|undefined} value
 * @return {!proto.ubo.v1.WiFiState} returns this
*/
proto.ubo.v1.WiFiState.prototype.setCurrentConnection = function(value) {
  return jspb.Message.setWrapperField(this, 4, value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.WiFiState} returns this
 */
proto.ubo.v1.WiFiState.prototype.clearCurrentConnection = function() {
  return this.setCurrentConnection(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiState.prototype.hasCurrentConnection = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional bool has_visited_onboarding = 5;
 * @return {boolean}
 */
proto.ubo.v1.WiFiState.prototype.getHasVisitedOnboarding = function() {
  return /** @type {boolean} */ (jspb.Message.getBooleanFieldWithDefault(this, 5, false));
};


/**
 * @param {boolean} value
 * @return {!proto.ubo.v1.WiFiState} returns this
 */
proto.ubo.v1.WiFiState.prototype.setHasVisitedOnboarding = function(value) {
  return jspb.Message.setField(this, 5, value);
};


/**
 * Clears the field making it undefined.
 * @return {!proto.ubo.v1.WiFiState} returns this
 */
proto.ubo.v1.WiFiState.prototype.clearHasVisitedOnboarding = function() {
  return jspb.Message.setField(this, 5, undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.WiFiState.prototype.hasHasVisitedOnboarding = function() {
  return jspb.Message.getField(this, 5) != null;
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.Action.oneofGroups_ = [[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194]];

/**
 * @enum {number}
 */
proto.ubo.v1.Action.ActionCase = {
  ACTION_NOT_SET: 0,
  ASSISTANT_ACTION: 1,
  ASSISTANT_ADD_MCP_SERVER_ACTION: 2,
  ASSISTANT_DELETE_MCP_SERVER_ACTION: 3,
  ASSISTANT_DOWNLOAD_OLLAMA_MODEL_ACTION: 4,
  ASSISTANT_REPORT_ACTION: 5,
  ASSISTANT_SET_IS_ACTIVE_ACTION: 6,
  ASSISTANT_SET_SELECTED_IMAGE_GENERATOR_ACTION: 7,
  ASSISTANT_SET_SELECTED_LLM_ACTION: 8,
  ASSISTANT_SET_SELECTED_MODEL_ACTION: 9,
  ASSISTANT_SET_SELECTED_STT_ACTION: 10,
  ASSISTANT_SET_SELECTED_TTS_ACTION: 11,
  ASSISTANT_START_LISTENING_ACTION: 12,
  ASSISTANT_STOP_LISTENING_ACTION: 13,
  ASSISTANT_SYNC_MCP_SERVERS_ACTION: 14,
  ASSISTANT_TOGGLE_LISTENING_ACTION: 15,
  ASSISTANT_TOGGLE_MCP_SERVER_ACTION: 16,
  ASSISTANT_UPDATE_PROVIDERS_ACTION: 17,
  AUDIO_ACTION: 18,
  AUDIO_CHANGE_VOLUME_ACTION: 19,
  AUDIO_INSTALL_DRIVER_ACTION: 20,
  AUDIO_PLAY_AUDIO_SAMPLE_ACTION: 21,
  AUDIO_PLAY_AUDIO_SEQUENCE_ACTION: 22,
  AUDIO_PLAY_CHIME_ACTION: 23,
  AUDIO_PLAY_RECORDING_ACTION: 24,
  AUDIO_PLAYBACK_DONE_ACTION: 25,
  AUDIO_REPORT_SAMPLE_ACTION: 26,
  AUDIO_SET_MUTE_STATUS_ACTION: 27,
  AUDIO_SET_VOLUME_ACTION: 28,
  AUDIO_START_RECORDING_ACTION: 29,
  AUDIO_STOP_RECORDING_ACTION: 30,
  AUDIO_TOGGLE_MUTE_STATUS_ACTION: 31,
  AUDIO_TOGGLE_RECORDING_ACTION: 32,
  CAMERA_ACTION: 33,
  CAMERA_DETECT_ACTION: 34,
  CAMERA_INSTALL_DRIVER_ACTION: 35,
  CAMERA_REPORT_BARCODE_ACTION: 36,
  CAMERA_RESTORE_DEFAULT_ACTION: 37,
  CAMERA_SET_AVAILABLE_CAMERAS_ACTION: 38,
  CAMERA_SET_INDEX_ACTION: 39,
  CAMERA_START_VIEWFINDER_ACTION: 40,
  CLOSE_APPLICATION_ACTION: 41,
  DEREGISTER_REGULAR_APP_ACTION: 42,
  DISPLAY_ACTION: 43,
  DISPLAY_BLANK_ACTION: 44,
  DISPLAY_PAUSE_ACTION: 45,
  DISPLAY_REDRAW_ACTION: 46,
  DISPLAY_RESUME_ACTION: 47,
  DISPLAY_SET_BLANK_TIMEOUT_ACTION: 48,
  DISPLAY_UNBLANK_ACTION: 49,
  DISPLAY_UPDATE_ACTIVITY_ACTION: 50,
  DOCKER_ACTION: 51,
  DOCKER_IMAGE_ACTION: 52,
  DOCKER_IMAGE_FETCH_ACTION: 53,
  DOCKER_IMAGE_RELEASE_ACTION: 54,
  DOCKER_IMAGE_REMOVE_ACTION: 55,
  DOCKER_IMAGE_REMOVE_CONTAINER_ACTION: 56,
  DOCKER_IMAGE_RUN_ACTION: 57,
  DOCKER_IMAGE_SET_DOCKER_ID_ACTION: 58,
  DOCKER_IMAGE_SET_STATUS_ACTION: 59,
  DOCKER_IMAGE_STOP_ACTION: 60,
  DOCKER_IMAGE_UPDATE_METADATA_ACTION: 61,
  DOCKER_INSTALL_ACTION: 62,
  DOCKER_REMOVE_USERNAME_ACTION: 63,
  DOCKER_SET_STATUS_ACTION: 64,
  DOCKER_START_ACTION: 65,
  DOCKER_STOP_ACTION: 66,
  DOCKER_STORE_USERNAME_ACTION: 67,
  FILE_SYSTEM_ACTION: 68,
  FILE_SYSTEM_COPY_ACTION: 69,
  FILE_SYSTEM_MOVE_ACTION: 70,
  FILE_SYSTEM_REMOVE_ACTION: 71,
  FILE_SYSTEM_REPORT_SELECTION_ACTION: 72,
  INFRARED_ACTION: 73,
  INFRARED_HANDLE_RECEIVED_CODE_ACTION: 74,
  INFRARED_SEND_CODE_ACTION: 75,
  INFRARED_SET_SHOULD_PROPAGATE_ACTION: 76,
  INFRARED_SET_SHOULD_RECEIVE_ACTION: 77,
  INPUT_ACTION: 78,
  INPUT_CANCEL_ACTION: 79,
  INPUT_DEMAND_ACTION: 80,
  INPUT_PROVIDE_ACTION: 81,
  INPUT_RESOLVE_ACTION: 82,
  IP_ACTION: 83,
  IP_SET_IS_CONNECTED_ACTION: 84,
  IP_UPDATE_INTERFACES_ACTION: 85,
  KEYPAD_ACTION: 86,
  KEYPAD_KEY_HOLD_ACTION: 87,
  KEYPAD_KEY_PRESS_ACTION: 88,
  KEYPAD_KEY_RELEASE_ACTION: 89,
  KEYPAD_KEY_UNHOLD_ACTION: 90,
  LIGHT_DM_ACTION: 91,
  LIGHT_DM_CLEAR_ENABLED_STATE_ACTION: 92,
  LIGHT_DM_UPDATE_STATE_ACTION: 93,
  MAIN_ACTION: 94,
  MENU_ACTION: 95,
  MENU_CHOOSE_BY_ICON_ACTION: 96,
  MENU_CHOOSE_BY_INDEX_ACTION: 97,
  MENU_CHOOSE_BY_LABEL_ACTION: 98,
  MENU_GO_BACK_ACTION: 99,
  MENU_GO_HOME_ACTION: 100,
  MENU_SCROLL_ACTION: 101,
  NOTIFICATIONS_ACTION: 102,
  NOTIFICATIONS_ADD_ACTION: 103,
  NOTIFICATIONS_CLEAR_ACTION: 104,
  NOTIFICATIONS_CLEAR_ALL_ACTION: 105,
  NOTIFICATIONS_CLEAR_BY_ID_ACTION: 106,
  NOTIFICATIONS_DISPLAY_ACTION: 107,
  OPEN_APPLICATION_ACTION: 108,
  POWER_ACTION: 109,
  POWER_OFF_ACTION: 110,
  R_PI_CONNECT_ACTION: 111,
  R_PI_CONNECT_DONE_DOWNLOADING_ACTION: 112,
  R_PI_CONNECT_SET_PENDING_ACTION: 113,
  R_PI_CONNECT_SET_STATUS_ACTION: 114,
  R_PI_CONNECT_START_DOWNLOADING_ACTION: 115,
  R_PI_CONNECT_UPDATE_SERVICE_STATE_ACTION: 116,
  REBOOT_ACTION: 117,
  REGISTER_APP_ACTION: 118,
  REGISTER_REGULAR_APP_ACTION: 119,
  REGISTER_SETTING_APP_ACTION: 120,
  REPLAY_RECORDED_SEQUENCE_ACTION: 121,
  REPORT_REPLAYING_DONE_ACTION: 122,
  RGB_RING_ACTION: 123,
  RGB_RING_BLANK_ACTION: 124,
  RGB_RING_BLINK_ACTION: 125,
  RGB_RING_COLORFUL_COMMAND_ACTION: 126,
  RGB_RING_COMMAND_ACTION: 127,
  RGB_RING_FILL_DOWNFROM_ACTION: 128,
  RGB_RING_FILL_UPTO_ACTION: 129,
  RGB_RING_PROGRESS_WHEEL_ACTION: 130,
  RGB_RING_PROGRESS_WHEEL_STEP_ACTION: 131,
  RGB_RING_PULSE_ACTION: 132,
  RGB_RING_RAINBOW_ACTION: 133,
  RGB_RING_SEQUENCE_ACTION: 134,
  RGB_RING_SET_ALL_ACTION: 135,
  RGB_RING_SET_BRIGHTNESS_ACTION: 136,
  RGB_RING_SET_ENABLED_ACTION: 137,
  RGB_RING_SET_IS_BUSY_ACTION: 138,
  RGB_RING_SPINNING_WHEEL_ACTION: 139,
  RGB_RING_WAITABLE_COMMAND_ACTION: 140,
  SSH_ACTION: 141,
  SSH_CLEAR_ENABLED_STATE_ACTION: 142,
  SSH_UPDATE_STATE_ACTION: 143,
  SENSORS_ACTION: 144,
  SENSORS_REPORT_READING_ACTION: 145,
  SET_ARE_ENCLOSURES_VISIBLE_ACTION: 146,
  SET_MENU_PATH_ACTION: 147,
  SETTINGS_ACTION: 148,
  SETTINGS_CLEAR_SERVICE_ERRORS_ACTION: 149,
  SETTINGS_REPORT_SERVICE_ERROR_ACTION: 150,
  SETTINGS_SERVICE_ACTION: 151,
  SETTINGS_SERVICE_SET_IS_ENABLED_ACTION: 152,
  SETTINGS_SERVICE_SET_LOG_LEVEL_ACTION: 153,
  SETTINGS_SERVICE_SET_SHOULD_RESTART_ACTION: 154,
  SETTINGS_SERVICE_SET_STATUS_ACTION: 155,
  SETTINGS_SET_SERVICES_ACTION: 156,
  SETTINGS_START_SERVICE_ACTION: 157,
  SETTINGS_STOP_SERVICE_ACTION: 158,
  SETTINGS_TOGGLE_BETA_VERSIONS_ACTION: 159,
  SETTINGS_TOGGLE_PDB_SIGNAL_ACTION: 160,
  SETTINGS_TOGGLE_VISUAL_DEBUG_ACTION: 161,
  SPEECH_RECOGNITION_ACTION: 162,
  SPEECH_RECOGNITION_REPORT_INTENT_DETECTION_ACTION: 163,
  SPEECH_RECOGNITION_REPORT_SPEECH_ACTION: 164,
  SPEECH_RECOGNITION_REPORT_WAKE_WORD_DETECTION_ACTION: 165,
  SPEECH_RECOGNITION_SET_IS_ASSISTANT_ACTIVE_ACTION: 166,
  SPEECH_RECOGNITION_SET_IS_INTENTS_ACTIVE_ACTION: 167,
  SPEECH_RECOGNITION_SET_SELECTED_ENGINE_ACTION: 168,
  SPEECH_SYNTHESIS_ACTION: 169,
  SPEECH_SYNTHESIS_READ_TEXT_ACTION: 170,
  SPEECH_SYNTHESIS_SET_SELECTED_ENGINE_ACTION: 171,
  STATUS_ICONS_ACTION: 172,
  STATUS_ICONS_REGISTER_ACTION: 173,
  TOGGLE_RECORDING_ACTION: 174,
  UPDATE_MANAGER_ACTION: 175,
  UPDATE_MANAGER_REPORT_FAILED_CHECK_ACTION: 176,
  UPDATE_MANAGER_REQUEST_CHECK_ACTION: 177,
  UPDATE_MANAGER_REQUEST_UPDATE_ACTION: 178,
  UPDATE_MANAGER_SET_VERSIONS_ACTION: 179,
  USERS_ACTION: 180,
  USERS_CREATE_USER_ACTION: 181,
  USERS_DELETE_USER_ACTION: 182,
  USERS_RESET_PASSWORD_ACTION: 183,
  USERS_SET_USERS_ACTION: 184,
  VS_CODE_ACTION: 185,
  VS_CODE_DONE_DOWNLOADING_ACTION: 186,
  VS_CODE_SET_PENDING_ACTION: 187,
  VS_CODE_SET_STATUS_ACTION: 188,
  VS_CODE_START_DOWNLOADING_ACTION: 189,
  WI_FI_ACTION: 190,
  WI_FI_INPUT_CONNECTION_ACTION: 191,
  WI_FI_SET_HAS_VISITED_ONBOARDING_ACTION: 192,
  WI_FI_UPDATE_ACTION: 193,
  WI_FI_UPDATE_REQUEST_ACTION: 194
};

/**
 * @return {proto.ubo.v1.Action.ActionCase}
 */
proto.ubo.v1.Action.prototype.getActionCase = function() {
  return /** @type {proto.ubo.v1.Action.ActionCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Action.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Action.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Action.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Action} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Action.toObject = function(includeInstance, msg) {
  var f, obj = {
    assistantAction: (f = msg.getAssistantAction()) && proto.ubo.v1.AssistantAction.toObject(includeInstance, f),
    assistantAddMcpServerAction: (f = msg.getAssistantAddMcpServerAction()) && proto.ubo.v1.AssistantAddMcpServerAction.toObject(includeInstance, f),
    assistantDeleteMcpServerAction: (f = msg.getAssistantDeleteMcpServerAction()) && proto.ubo.v1.AssistantDeleteMcpServerAction.toObject(includeInstance, f),
    assistantDownloadOllamaModelAction: (f = msg.getAssistantDownloadOllamaModelAction()) && proto.ubo.v1.AssistantDownloadOllamaModelAction.toObject(includeInstance, f),
    assistantReportAction: (f = msg.getAssistantReportAction()) && proto.ubo.v1.AssistantReportAction.toObject(includeInstance, f),
    assistantSetIsActiveAction: (f = msg.getAssistantSetIsActiveAction()) && proto.ubo.v1.AssistantSetIsActiveAction.toObject(includeInstance, f),
    assistantSetSelectedImageGeneratorAction: (f = msg.getAssistantSetSelectedImageGeneratorAction()) && proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.toObject(includeInstance, f),
    assistantSetSelectedLlmAction: (f = msg.getAssistantSetSelectedLlmAction()) && proto.ubo.v1.AssistantSetSelectedLLMAction.toObject(includeInstance, f),
    assistantSetSelectedModelAction: (f = msg.getAssistantSetSelectedModelAction()) && proto.ubo.v1.AssistantSetSelectedModelAction.toObject(includeInstance, f),
    assistantSetSelectedSttAction: (f = msg.getAssistantSetSelectedSttAction()) && proto.ubo.v1.AssistantSetSelectedSTTAction.toObject(includeInstance, f),
    assistantSetSelectedTtsAction: (f = msg.getAssistantSetSelectedTtsAction()) && proto.ubo.v1.AssistantSetSelectedTTSAction.toObject(includeInstance, f),
    assistantStartListeningAction: (f = msg.getAssistantStartListeningAction()) && proto.ubo.v1.AssistantStartListeningAction.toObject(includeInstance, f),
    assistantStopListeningAction: (f = msg.getAssistantStopListeningAction()) && proto.ubo.v1.AssistantStopListeningAction.toObject(includeInstance, f),
    assistantSyncMcpServersAction: (f = msg.getAssistantSyncMcpServersAction()) && proto.ubo.v1.AssistantSyncMcpServersAction.toObject(includeInstance, f),
    assistantToggleListeningAction: (f = msg.getAssistantToggleListeningAction()) && proto.ubo.v1.AssistantToggleListeningAction.toObject(includeInstance, f),
    assistantToggleMcpServerAction: (f = msg.getAssistantToggleMcpServerAction()) && proto.ubo.v1.AssistantToggleMcpServerAction.toObject(includeInstance, f),
    assistantUpdateProvidersAction: (f = msg.getAssistantUpdateProvidersAction()) && proto.ubo.v1.AssistantUpdateProvidersAction.toObject(includeInstance, f),
    audioAction: (f = msg.getAudioAction()) && proto.ubo.v1.AudioAction.toObject(includeInstance, f),
    audioChangeVolumeAction: (f = msg.getAudioChangeVolumeAction()) && proto.ubo.v1.AudioChangeVolumeAction.toObject(includeInstance, f),
    audioInstallDriverAction: (f = msg.getAudioInstallDriverAction()) && proto.ubo.v1.AudioInstallDriverAction.toObject(includeInstance, f),
    audioPlayAudioSampleAction: (f = msg.getAudioPlayAudioSampleAction()) && proto.ubo.v1.AudioPlayAudioSampleAction.toObject(includeInstance, f),
    audioPlayAudioSequenceAction: (f = msg.getAudioPlayAudioSequenceAction()) && proto.ubo.v1.AudioPlayAudioSequenceAction.toObject(includeInstance, f),
    audioPlayChimeAction: (f = msg.getAudioPlayChimeAction()) && proto.ubo.v1.AudioPlayChimeAction.toObject(includeInstance, f),
    audioPlayRecordingAction: (f = msg.getAudioPlayRecordingAction()) && proto.ubo.v1.AudioPlayRecordingAction.toObject(includeInstance, f),
    audioPlaybackDoneAction: (f = msg.getAudioPlaybackDoneAction()) && proto.ubo.v1.AudioPlaybackDoneAction.toObject(includeInstance, f),
    audioReportSampleAction: (f = msg.getAudioReportSampleAction()) && proto.ubo.v1.AudioReportSampleAction.toObject(includeInstance, f),
    audioSetMuteStatusAction: (f = msg.getAudioSetMuteStatusAction()) && proto.ubo.v1.AudioSetMuteStatusAction.toObject(includeInstance, f),
    audioSetVolumeAction: (f = msg.getAudioSetVolumeAction()) && proto.ubo.v1.AudioSetVolumeAction.toObject(includeInstance, f),
    audioStartRecordingAction: (f = msg.getAudioStartRecordingAction()) && proto.ubo.v1.AudioStartRecordingAction.toObject(includeInstance, f),
    audioStopRecordingAction: (f = msg.getAudioStopRecordingAction()) && proto.ubo.v1.AudioStopRecordingAction.toObject(includeInstance, f),
    audioToggleMuteStatusAction: (f = msg.getAudioToggleMuteStatusAction()) && proto.ubo.v1.AudioToggleMuteStatusAction.toObject(includeInstance, f),
    audioToggleRecordingAction: (f = msg.getAudioToggleRecordingAction()) && proto.ubo.v1.AudioToggleRecordingAction.toObject(includeInstance, f),
    cameraAction: (f = msg.getCameraAction()) && proto.ubo.v1.CameraAction.toObject(includeInstance, f),
    cameraDetectAction: (f = msg.getCameraDetectAction()) && proto.ubo.v1.CameraDetectAction.toObject(includeInstance, f),
    cameraInstallDriverAction: (f = msg.getCameraInstallDriverAction()) && proto.ubo.v1.CameraInstallDriverAction.toObject(includeInstance, f),
    cameraReportBarcodeAction: (f = msg.getCameraReportBarcodeAction()) && proto.ubo.v1.CameraReportBarcodeAction.toObject(includeInstance, f),
    cameraRestoreDefaultAction: (f = msg.getCameraRestoreDefaultAction()) && proto.ubo.v1.CameraRestoreDefaultAction.toObject(includeInstance, f),
    cameraSetAvailableCamerasAction: (f = msg.getCameraSetAvailableCamerasAction()) && proto.ubo.v1.CameraSetAvailableCamerasAction.toObject(includeInstance, f),
    cameraSetIndexAction: (f = msg.getCameraSetIndexAction()) && proto.ubo.v1.CameraSetIndexAction.toObject(includeInstance, f),
    cameraStartViewfinderAction: (f = msg.getCameraStartViewfinderAction()) && proto.ubo.v1.CameraStartViewfinderAction.toObject(includeInstance, f),
    closeApplicationAction: (f = msg.getCloseApplicationAction()) && proto.ubo.v1.CloseApplicationAction.toObject(includeInstance, f),
    deregisterRegularAppAction: (f = msg.getDeregisterRegularAppAction()) && proto.ubo.v1.DeregisterRegularAppAction.toObject(includeInstance, f),
    displayAction: (f = msg.getDisplayAction()) && proto.ubo.v1.DisplayAction.toObject(includeInstance, f),
    displayBlankAction: (f = msg.getDisplayBlankAction()) && proto.ubo.v1.DisplayBlankAction.toObject(includeInstance, f),
    displayPauseAction: (f = msg.getDisplayPauseAction()) && proto.ubo.v1.DisplayPauseAction.toObject(includeInstance, f),
    displayRedrawAction: (f = msg.getDisplayRedrawAction()) && proto.ubo.v1.DisplayRedrawAction.toObject(includeInstance, f),
    displayResumeAction: (f = msg.getDisplayResumeAction()) && proto.ubo.v1.DisplayResumeAction.toObject(includeInstance, f),
    displaySetBlankTimeoutAction: (f = msg.getDisplaySetBlankTimeoutAction()) && proto.ubo.v1.DisplaySetBlankTimeoutAction.toObject(includeInstance, f),
    displayUnblankAction: (f = msg.getDisplayUnblankAction()) && proto.ubo.v1.DisplayUnblankAction.toObject(includeInstance, f),
    displayUpdateActivityAction: (f = msg.getDisplayUpdateActivityAction()) && proto.ubo.v1.DisplayUpdateActivityAction.toObject(includeInstance, f),
    dockerAction: (f = msg.getDockerAction()) && proto.ubo.v1.DockerAction.toObject(includeInstance, f),
    dockerImageAction: (f = msg.getDockerImageAction()) && proto.ubo.v1.DockerImageAction.toObject(includeInstance, f),
    dockerImageFetchAction: (f = msg.getDockerImageFetchAction()) && proto.ubo.v1.DockerImageFetchAction.toObject(includeInstance, f),
    dockerImageReleaseAction: (f = msg.getDockerImageReleaseAction()) && proto.ubo.v1.DockerImageReleaseAction.toObject(includeInstance, f),
    dockerImageRemoveAction: (f = msg.getDockerImageRemoveAction()) && proto.ubo.v1.DockerImageRemoveAction.toObject(includeInstance, f),
    dockerImageRemoveContainerAction: (f = msg.getDockerImageRemoveContainerAction()) && proto.ubo.v1.DockerImageRemoveContainerAction.toObject(includeInstance, f),
    dockerImageRunAction: (f = msg.getDockerImageRunAction()) && proto.ubo.v1.DockerImageRunAction.toObject(includeInstance, f),
    dockerImageSetDockerIdAction: (f = msg.getDockerImageSetDockerIdAction()) && proto.ubo.v1.DockerImageSetDockerIdAction.toObject(includeInstance, f),
    dockerImageSetStatusAction: (f = msg.getDockerImageSetStatusAction()) && proto.ubo.v1.DockerImageSetStatusAction.toObject(includeInstance, f),
    dockerImageStopAction: (f = msg.getDockerImageStopAction()) && proto.ubo.v1.DockerImageStopAction.toObject(includeInstance, f),
    dockerImageUpdateMetadataAction: (f = msg.getDockerImageUpdateMetadataAction()) && proto.ubo.v1.DockerImageUpdateMetadataAction.toObject(includeInstance, f),
    dockerInstallAction: (f = msg.getDockerInstallAction()) && proto.ubo.v1.DockerInstallAction.toObject(includeInstance, f),
    dockerRemoveUsernameAction: (f = msg.getDockerRemoveUsernameAction()) && proto.ubo.v1.DockerRemoveUsernameAction.toObject(includeInstance, f),
    dockerSetStatusAction: (f = msg.getDockerSetStatusAction()) && proto.ubo.v1.DockerSetStatusAction.toObject(includeInstance, f),
    dockerStartAction: (f = msg.getDockerStartAction()) && proto.ubo.v1.DockerStartAction.toObject(includeInstance, f),
    dockerStopAction: (f = msg.getDockerStopAction()) && proto.ubo.v1.DockerStopAction.toObject(includeInstance, f),
    dockerStoreUsernameAction: (f = msg.getDockerStoreUsernameAction()) && proto.ubo.v1.DockerStoreUsernameAction.toObject(includeInstance, f),
    fileSystemAction: (f = msg.getFileSystemAction()) && proto.ubo.v1.FileSystemAction.toObject(includeInstance, f),
    fileSystemCopyAction: (f = msg.getFileSystemCopyAction()) && proto.ubo.v1.FileSystemCopyAction.toObject(includeInstance, f),
    fileSystemMoveAction: (f = msg.getFileSystemMoveAction()) && proto.ubo.v1.FileSystemMoveAction.toObject(includeInstance, f),
    fileSystemRemoveAction: (f = msg.getFileSystemRemoveAction()) && proto.ubo.v1.FileSystemRemoveAction.toObject(includeInstance, f),
    fileSystemReportSelectionAction: (f = msg.getFileSystemReportSelectionAction()) && proto.ubo.v1.FileSystemReportSelectionAction.toObject(includeInstance, f),
    infraredAction: (f = msg.getInfraredAction()) && proto.ubo.v1.InfraredAction.toObject(includeInstance, f),
    infraredHandleReceivedCodeAction: (f = msg.getInfraredHandleReceivedCodeAction()) && proto.ubo.v1.InfraredHandleReceivedCodeAction.toObject(includeInstance, f),
    infraredSendCodeAction: (f = msg.getInfraredSendCodeAction()) && proto.ubo.v1.InfraredSendCodeAction.toObject(includeInstance, f),
    infraredSetShouldPropagateAction: (f = msg.getInfraredSetShouldPropagateAction()) && proto.ubo.v1.InfraredSetShouldPropagateAction.toObject(includeInstance, f),
    infraredSetShouldReceiveAction: (f = msg.getInfraredSetShouldReceiveAction()) && proto.ubo.v1.InfraredSetShouldReceiveAction.toObject(includeInstance, f),
    inputAction: (f = msg.getInputAction()) && proto.ubo.v1.InputAction.toObject(includeInstance, f),
    inputCancelAction: (f = msg.getInputCancelAction()) && proto.ubo.v1.InputCancelAction.toObject(includeInstance, f),
    inputDemandAction: (f = msg.getInputDemandAction()) && proto.ubo.v1.InputDemandAction.toObject(includeInstance, f),
    inputProvideAction: (f = msg.getInputProvideAction()) && proto.ubo.v1.InputProvideAction.toObject(includeInstance, f),
    inputResolveAction: (f = msg.getInputResolveAction()) && proto.ubo.v1.InputResolveAction.toObject(includeInstance, f),
    ipAction: (f = msg.getIpAction()) && proto.ubo.v1.IpAction.toObject(includeInstance, f),
    ipSetIsConnectedAction: (f = msg.getIpSetIsConnectedAction()) && proto.ubo.v1.IpSetIsConnectedAction.toObject(includeInstance, f),
    ipUpdateInterfacesAction: (f = msg.getIpUpdateInterfacesAction()) && proto.ubo.v1.IpUpdateInterfacesAction.toObject(includeInstance, f),
    keypadAction: (f = msg.getKeypadAction()) && proto.ubo.v1.KeypadAction.toObject(includeInstance, f),
    keypadKeyHoldAction: (f = msg.getKeypadKeyHoldAction()) && proto.ubo.v1.KeypadKeyHoldAction.toObject(includeInstance, f),
    keypadKeyPressAction: (f = msg.getKeypadKeyPressAction()) && proto.ubo.v1.KeypadKeyPressAction.toObject(includeInstance, f),
    keypadKeyReleaseAction: (f = msg.getKeypadKeyReleaseAction()) && proto.ubo.v1.KeypadKeyReleaseAction.toObject(includeInstance, f),
    keypadKeyUnholdAction: (f = msg.getKeypadKeyUnholdAction()) && proto.ubo.v1.KeypadKeyUnholdAction.toObject(includeInstance, f),
    lightDmAction: (f = msg.getLightDmAction()) && proto.ubo.v1.LightDMAction.toObject(includeInstance, f),
    lightDmClearEnabledStateAction: (f = msg.getLightDmClearEnabledStateAction()) && proto.ubo.v1.LightDMClearEnabledStateAction.toObject(includeInstance, f),
    lightDmUpdateStateAction: (f = msg.getLightDmUpdateStateAction()) && proto.ubo.v1.LightDMUpdateStateAction.toObject(includeInstance, f),
    mainAction: (f = msg.getMainAction()) && proto.ubo.v1.MainAction.toObject(includeInstance, f),
    menuAction: (f = msg.getMenuAction()) && proto.ubo.v1.MenuAction.toObject(includeInstance, f),
    menuChooseByIconAction: (f = msg.getMenuChooseByIconAction()) && proto.ubo.v1.MenuChooseByIconAction.toObject(includeInstance, f),
    menuChooseByIndexAction: (f = msg.getMenuChooseByIndexAction()) && proto.ubo.v1.MenuChooseByIndexAction.toObject(includeInstance, f),
    menuChooseByLabelAction: (f = msg.getMenuChooseByLabelAction()) && proto.ubo.v1.MenuChooseByLabelAction.toObject(includeInstance, f),
    menuGoBackAction: (f = msg.getMenuGoBackAction()) && proto.ubo.v1.MenuGoBackAction.toObject(includeInstance, f),
    menuGoHomeAction: (f = msg.getMenuGoHomeAction()) && proto.ubo.v1.MenuGoHomeAction.toObject(includeInstance, f),
    menuScrollAction: (f = msg.getMenuScrollAction()) && proto.ubo.v1.MenuScrollAction.toObject(includeInstance, f),
    notificationsAction: (f = msg.getNotificationsAction()) && proto.ubo.v1.NotificationsAction.toObject(includeInstance, f),
    notificationsAddAction: (f = msg.getNotificationsAddAction()) && proto.ubo.v1.NotificationsAddAction.toObject(includeInstance, f),
    notificationsClearAction: (f = msg.getNotificationsClearAction()) && proto.ubo.v1.NotificationsClearAction.toObject(includeInstance, f),
    notificationsClearAllAction: (f = msg.getNotificationsClearAllAction()) && proto.ubo.v1.NotificationsClearAllAction.toObject(includeInstance, f),
    notificationsClearByIdAction: (f = msg.getNotificationsClearByIdAction()) && proto.ubo.v1.NotificationsClearByIdAction.toObject(includeInstance, f),
    notificationsDisplayAction: (f = msg.getNotificationsDisplayAction()) && proto.ubo.v1.NotificationsDisplayAction.toObject(includeInstance, f),
    openApplicationAction: (f = msg.getOpenApplicationAction()) && proto.ubo.v1.OpenApplicationAction.toObject(includeInstance, f),
    powerAction: (f = msg.getPowerAction()) && proto.ubo.v1.PowerAction.toObject(includeInstance, f),
    powerOffAction: (f = msg.getPowerOffAction()) && proto.ubo.v1.PowerOffAction.toObject(includeInstance, f),
    rPiConnectAction: (f = msg.getRPiConnectAction()) && proto.ubo.v1.RPiConnectAction.toObject(includeInstance, f),
    rPiConnectDoneDownloadingAction: (f = msg.getRPiConnectDoneDownloadingAction()) && proto.ubo.v1.RPiConnectDoneDownloadingAction.toObject(includeInstance, f),
    rPiConnectSetPendingAction: (f = msg.getRPiConnectSetPendingAction()) && proto.ubo.v1.RPiConnectSetPendingAction.toObject(includeInstance, f),
    rPiConnectSetStatusAction: (f = msg.getRPiConnectSetStatusAction()) && proto.ubo.v1.RPiConnectSetStatusAction.toObject(includeInstance, f),
    rPiConnectStartDownloadingAction: (f = msg.getRPiConnectStartDownloadingAction()) && proto.ubo.v1.RPiConnectStartDownloadingAction.toObject(includeInstance, f),
    rPiConnectUpdateServiceStateAction: (f = msg.getRPiConnectUpdateServiceStateAction()) && proto.ubo.v1.RPiConnectUpdateServiceStateAction.toObject(includeInstance, f),
    rebootAction: (f = msg.getRebootAction()) && proto.ubo.v1.RebootAction.toObject(includeInstance, f),
    registerAppAction: (f = msg.getRegisterAppAction()) && proto.ubo.v1.RegisterAppAction.toObject(includeInstance, f),
    registerRegularAppAction: (f = msg.getRegisterRegularAppAction()) && proto.ubo.v1.RegisterRegularAppAction.toObject(includeInstance, f),
    registerSettingAppAction: (f = msg.getRegisterSettingAppAction()) && proto.ubo.v1.RegisterSettingAppAction.toObject(includeInstance, f),
    replayRecordedSequenceAction: (f = msg.getReplayRecordedSequenceAction()) && proto.ubo.v1.ReplayRecordedSequenceAction.toObject(includeInstance, f),
    reportReplayingDoneAction: (f = msg.getReportReplayingDoneAction()) && proto.ubo.v1.ReportReplayingDoneAction.toObject(includeInstance, f),
    rgbRingAction: (f = msg.getRgbRingAction()) && proto.ubo.v1.RgbRingAction.toObject(includeInstance, f),
    rgbRingBlankAction: (f = msg.getRgbRingBlankAction()) && proto.ubo.v1.RgbRingBlankAction.toObject(includeInstance, f),
    rgbRingBlinkAction: (f = msg.getRgbRingBlinkAction()) && proto.ubo.v1.RgbRingBlinkAction.toObject(includeInstance, f),
    rgbRingColorfulCommandAction: (f = msg.getRgbRingColorfulCommandAction()) && proto.ubo.v1.RgbRingColorfulCommandAction.toObject(includeInstance, f),
    rgbRingCommandAction: (f = msg.getRgbRingCommandAction()) && proto.ubo.v1.RgbRingCommandAction.toObject(includeInstance, f),
    rgbRingFillDownfromAction: (f = msg.getRgbRingFillDownfromAction()) && proto.ubo.v1.RgbRingFillDownfromAction.toObject(includeInstance, f),
    rgbRingFillUptoAction: (f = msg.getRgbRingFillUptoAction()) && proto.ubo.v1.RgbRingFillUptoAction.toObject(includeInstance, f),
    rgbRingProgressWheelAction: (f = msg.getRgbRingProgressWheelAction()) && proto.ubo.v1.RgbRingProgressWheelAction.toObject(includeInstance, f),
    rgbRingProgressWheelStepAction: (f = msg.getRgbRingProgressWheelStepAction()) && proto.ubo.v1.RgbRingProgressWheelStepAction.toObject(includeInstance, f),
    rgbRingPulseAction: (f = msg.getRgbRingPulseAction()) && proto.ubo.v1.RgbRingPulseAction.toObject(includeInstance, f),
    rgbRingRainbowAction: (f = msg.getRgbRingRainbowAction()) && proto.ubo.v1.RgbRingRainbowAction.toObject(includeInstance, f),
    rgbRingSequenceAction: (f = msg.getRgbRingSequenceAction()) && proto.ubo.v1.RgbRingSequenceAction.toObject(includeInstance, f),
    rgbRingSetAllAction: (f = msg.getRgbRingSetAllAction()) && proto.ubo.v1.RgbRingSetAllAction.toObject(includeInstance, f),
    rgbRingSetBrightnessAction: (f = msg.getRgbRingSetBrightnessAction()) && proto.ubo.v1.RgbRingSetBrightnessAction.toObject(includeInstance, f),
    rgbRingSetEnabledAction: (f = msg.getRgbRingSetEnabledAction()) && proto.ubo.v1.RgbRingSetEnabledAction.toObject(includeInstance, f),
    rgbRingSetIsBusyAction: (f = msg.getRgbRingSetIsBusyAction()) && proto.ubo.v1.RgbRingSetIsBusyAction.toObject(includeInstance, f),
    rgbRingSpinningWheelAction: (f = msg.getRgbRingSpinningWheelAction()) && proto.ubo.v1.RgbRingSpinningWheelAction.toObject(includeInstance, f),
    rgbRingWaitableCommandAction: (f = msg.getRgbRingWaitableCommandAction()) && proto.ubo.v1.RgbRingWaitableCommandAction.toObject(includeInstance, f),
    sshAction: (f = msg.getSshAction()) && proto.ubo.v1.SSHAction.toObject(includeInstance, f),
    sshClearEnabledStateAction: (f = msg.getSshClearEnabledStateAction()) && proto.ubo.v1.SSHClearEnabledStateAction.toObject(includeInstance, f),
    sshUpdateStateAction: (f = msg.getSshUpdateStateAction()) && proto.ubo.v1.SSHUpdateStateAction.toObject(includeInstance, f),
    sensorsAction: (f = msg.getSensorsAction()) && proto.ubo.v1.SensorsAction.toObject(includeInstance, f),
    sensorsReportReadingAction: (f = msg.getSensorsReportReadingAction()) && proto.ubo.v1.SensorsReportReadingAction.toObject(includeInstance, f),
    setAreEnclosuresVisibleAction: (f = msg.getSetAreEnclosuresVisibleAction()) && proto.ubo.v1.SetAreEnclosuresVisibleAction.toObject(includeInstance, f),
    setMenuPathAction: (f = msg.getSetMenuPathAction()) && proto.ubo.v1.SetMenuPathAction.toObject(includeInstance, f),
    settingsAction: (f = msg.getSettingsAction()) && proto.ubo.v1.SettingsAction.toObject(includeInstance, f),
    settingsClearServiceErrorsAction: (f = msg.getSettingsClearServiceErrorsAction()) && proto.ubo.v1.SettingsClearServiceErrorsAction.toObject(includeInstance, f),
    settingsReportServiceErrorAction: (f = msg.getSettingsReportServiceErrorAction()) && proto.ubo.v1.SettingsReportServiceErrorAction.toObject(includeInstance, f),
    settingsServiceAction: (f = msg.getSettingsServiceAction()) && proto.ubo.v1.SettingsServiceAction.toObject(includeInstance, f),
    settingsServiceSetIsEnabledAction: (f = msg.getSettingsServiceSetIsEnabledAction()) && proto.ubo.v1.SettingsServiceSetIsEnabledAction.toObject(includeInstance, f),
    settingsServiceSetLogLevelAction: (f = msg.getSettingsServiceSetLogLevelAction()) && proto.ubo.v1.SettingsServiceSetLogLevelAction.toObject(includeInstance, f),
    settingsServiceSetShouldRestartAction: (f = msg.getSettingsServiceSetShouldRestartAction()) && proto.ubo.v1.SettingsServiceSetShouldRestartAction.toObject(includeInstance, f),
    settingsServiceSetStatusAction: (f = msg.getSettingsServiceSetStatusAction()) && proto.ubo.v1.SettingsServiceSetStatusAction.toObject(includeInstance, f),
    settingsSetServicesAction: (f = msg.getSettingsSetServicesAction()) && proto.ubo.v1.SettingsSetServicesAction.toObject(includeInstance, f),
    settingsStartServiceAction: (f = msg.getSettingsStartServiceAction()) && proto.ubo.v1.SettingsStartServiceAction.toObject(includeInstance, f),
    settingsStopServiceAction: (f = msg.getSettingsStopServiceAction()) && proto.ubo.v1.SettingsStopServiceAction.toObject(includeInstance, f),
    settingsToggleBetaVersionsAction: (f = msg.getSettingsToggleBetaVersionsAction()) && proto.ubo.v1.SettingsToggleBetaVersionsAction.toObject(includeInstance, f),
    settingsTogglePdbSignalAction: (f = msg.getSettingsTogglePdbSignalAction()) && proto.ubo.v1.SettingsTogglePdbSignalAction.toObject(includeInstance, f),
    settingsToggleVisualDebugAction: (f = msg.getSettingsToggleVisualDebugAction()) && proto.ubo.v1.SettingsToggleVisualDebugAction.toObject(includeInstance, f),
    speechRecognitionAction: (f = msg.getSpeechRecognitionAction()) && proto.ubo.v1.SpeechRecognitionAction.toObject(includeInstance, f),
    speechRecognitionReportIntentDetectionAction: (f = msg.getSpeechRecognitionReportIntentDetectionAction()) && proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.toObject(includeInstance, f),
    speechRecognitionReportSpeechAction: (f = msg.getSpeechRecognitionReportSpeechAction()) && proto.ubo.v1.SpeechRecognitionReportSpeechAction.toObject(includeInstance, f),
    speechRecognitionReportWakeWordDetectionAction: (f = msg.getSpeechRecognitionReportWakeWordDetectionAction()) && proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.toObject(includeInstance, f),
    speechRecognitionSetIsAssistantActiveAction: (f = msg.getSpeechRecognitionSetIsAssistantActiveAction()) && proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.toObject(includeInstance, f),
    speechRecognitionSetIsIntentsActiveAction: (f = msg.getSpeechRecognitionSetIsIntentsActiveAction()) && proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.toObject(includeInstance, f),
    speechRecognitionSetSelectedEngineAction: (f = msg.getSpeechRecognitionSetSelectedEngineAction()) && proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.toObject(includeInstance, f),
    speechSynthesisAction: (f = msg.getSpeechSynthesisAction()) && proto.ubo.v1.SpeechSynthesisAction.toObject(includeInstance, f),
    speechSynthesisReadTextAction: (f = msg.getSpeechSynthesisReadTextAction()) && proto.ubo.v1.SpeechSynthesisReadTextAction.toObject(includeInstance, f),
    speechSynthesisSetSelectedEngineAction: (f = msg.getSpeechSynthesisSetSelectedEngineAction()) && proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.toObject(includeInstance, f),
    statusIconsAction: (f = msg.getStatusIconsAction()) && proto.ubo.v1.StatusIconsAction.toObject(includeInstance, f),
    statusIconsRegisterAction: (f = msg.getStatusIconsRegisterAction()) && proto.ubo.v1.StatusIconsRegisterAction.toObject(includeInstance, f),
    toggleRecordingAction: (f = msg.getToggleRecordingAction()) && proto.ubo.v1.ToggleRecordingAction.toObject(includeInstance, f),
    updateManagerAction: (f = msg.getUpdateManagerAction()) && proto.ubo.v1.UpdateManagerAction.toObject(includeInstance, f),
    updateManagerReportFailedCheckAction: (f = msg.getUpdateManagerReportFailedCheckAction()) && proto.ubo.v1.UpdateManagerReportFailedCheckAction.toObject(includeInstance, f),
    updateManagerRequestCheckAction: (f = msg.getUpdateManagerRequestCheckAction()) && proto.ubo.v1.UpdateManagerRequestCheckAction.toObject(includeInstance, f),
    updateManagerRequestUpdateAction: (f = msg.getUpdateManagerRequestUpdateAction()) && proto.ubo.v1.UpdateManagerRequestUpdateAction.toObject(includeInstance, f),
    updateManagerSetVersionsAction: (f = msg.getUpdateManagerSetVersionsAction()) && proto.ubo.v1.UpdateManagerSetVersionsAction.toObject(includeInstance, f),
    usersAction: (f = msg.getUsersAction()) && proto.ubo.v1.UsersAction.toObject(includeInstance, f),
    usersCreateUserAction: (f = msg.getUsersCreateUserAction()) && proto.ubo.v1.UsersCreateUserAction.toObject(includeInstance, f),
    usersDeleteUserAction: (f = msg.getUsersDeleteUserAction()) && proto.ubo.v1.UsersDeleteUserAction.toObject(includeInstance, f),
    usersResetPasswordAction: (f = msg.getUsersResetPasswordAction()) && proto.ubo.v1.UsersResetPasswordAction.toObject(includeInstance, f),
    usersSetUsersAction: (f = msg.getUsersSetUsersAction()) && proto.ubo.v1.UsersSetUsersAction.toObject(includeInstance, f),
    vsCodeAction: (f = msg.getVsCodeAction()) && proto.ubo.v1.VSCodeAction.toObject(includeInstance, f),
    vsCodeDoneDownloadingAction: (f = msg.getVsCodeDoneDownloadingAction()) && proto.ubo.v1.VSCodeDoneDownloadingAction.toObject(includeInstance, f),
    vsCodeSetPendingAction: (f = msg.getVsCodeSetPendingAction()) && proto.ubo.v1.VSCodeSetPendingAction.toObject(includeInstance, f),
    vsCodeSetStatusAction: (f = msg.getVsCodeSetStatusAction()) && proto.ubo.v1.VSCodeSetStatusAction.toObject(includeInstance, f),
    vsCodeStartDownloadingAction: (f = msg.getVsCodeStartDownloadingAction()) && proto.ubo.v1.VSCodeStartDownloadingAction.toObject(includeInstance, f),
    wiFiAction: (f = msg.getWiFiAction()) && proto.ubo.v1.WiFiAction.toObject(includeInstance, f),
    wiFiInputConnectionAction: (f = msg.getWiFiInputConnectionAction()) && proto.ubo.v1.WiFiInputConnectionAction.toObject(includeInstance, f),
    wiFiSetHasVisitedOnboardingAction: (f = msg.getWiFiSetHasVisitedOnboardingAction()) && proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.toObject(includeInstance, f),
    wiFiUpdateAction: (f = msg.getWiFiUpdateAction()) && proto.ubo.v1.WiFiUpdateAction.toObject(includeInstance, f),
    wiFiUpdateRequestAction: (f = msg.getWiFiUpdateRequestAction()) && proto.ubo.v1.WiFiUpdateRequestAction.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Action}
 */
proto.ubo.v1.Action.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Action;
  return proto.ubo.v1.Action.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Action} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Action}
 */
proto.ubo.v1.Action.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.AssistantAction;
      reader.readMessage(value,proto.ubo.v1.AssistantAction.deserializeBinaryFromReader);
      msg.setAssistantAction(value);
      break;
    case 2:
      var value = new proto.ubo.v1.AssistantAddMcpServerAction;
      reader.readMessage(value,proto.ubo.v1.AssistantAddMcpServerAction.deserializeBinaryFromReader);
      msg.setAssistantAddMcpServerAction(value);
      break;
    case 3:
      var value = new proto.ubo.v1.AssistantDeleteMcpServerAction;
      reader.readMessage(value,proto.ubo.v1.AssistantDeleteMcpServerAction.deserializeBinaryFromReader);
      msg.setAssistantDeleteMcpServerAction(value);
      break;
    case 4:
      var value = new proto.ubo.v1.AssistantDownloadOllamaModelAction;
      reader.readMessage(value,proto.ubo.v1.AssistantDownloadOllamaModelAction.deserializeBinaryFromReader);
      msg.setAssistantDownloadOllamaModelAction(value);
      break;
    case 5:
      var value = new proto.ubo.v1.AssistantReportAction;
      reader.readMessage(value,proto.ubo.v1.AssistantReportAction.deserializeBinaryFromReader);
      msg.setAssistantReportAction(value);
      break;
    case 6:
      var value = new proto.ubo.v1.AssistantSetIsActiveAction;
      reader.readMessage(value,proto.ubo.v1.AssistantSetIsActiveAction.deserializeBinaryFromReader);
      msg.setAssistantSetIsActiveAction(value);
      break;
    case 7:
      var value = new proto.ubo.v1.AssistantSetSelectedImageGeneratorAction;
      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.deserializeBinaryFromReader);
      msg.setAssistantSetSelectedImageGeneratorAction(value);
      break;
    case 8:
      var value = new proto.ubo.v1.AssistantSetSelectedLLMAction;
      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedLLMAction.deserializeBinaryFromReader);
      msg.setAssistantSetSelectedLlmAction(value);
      break;
    case 9:
      var value = new proto.ubo.v1.AssistantSetSelectedModelAction;
      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedModelAction.deserializeBinaryFromReader);
      msg.setAssistantSetSelectedModelAction(value);
      break;
    case 10:
      var value = new proto.ubo.v1.AssistantSetSelectedSTTAction;
      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedSTTAction.deserializeBinaryFromReader);
      msg.setAssistantSetSelectedSttAction(value);
      break;
    case 11:
      var value = new proto.ubo.v1.AssistantSetSelectedTTSAction;
      reader.readMessage(value,proto.ubo.v1.AssistantSetSelectedTTSAction.deserializeBinaryFromReader);
      msg.setAssistantSetSelectedTtsAction(value);
      break;
    case 12:
      var value = new proto.ubo.v1.AssistantStartListeningAction;
      reader.readMessage(value,proto.ubo.v1.AssistantStartListeningAction.deserializeBinaryFromReader);
      msg.setAssistantStartListeningAction(value);
      break;
    case 13:
      var value = new proto.ubo.v1.AssistantStopListeningAction;
      reader.readMessage(value,proto.ubo.v1.AssistantStopListeningAction.deserializeBinaryFromReader);
      msg.setAssistantStopListeningAction(value);
      break;
    case 14:
      var value = new proto.ubo.v1.AssistantSyncMcpServersAction;
      reader.readMessage(value,proto.ubo.v1.AssistantSyncMcpServersAction.deserializeBinaryFromReader);
      msg.setAssistantSyncMcpServersAction(value);
      break;
    case 15:
      var value = new proto.ubo.v1.AssistantToggleListeningAction;
      reader.readMessage(value,proto.ubo.v1.AssistantToggleListeningAction.deserializeBinaryFromReader);
      msg.setAssistantToggleListeningAction(value);
      break;
    case 16:
      var value = new proto.ubo.v1.AssistantToggleMcpServerAction;
      reader.readMessage(value,proto.ubo.v1.AssistantToggleMcpServerAction.deserializeBinaryFromReader);
      msg.setAssistantToggleMcpServerAction(value);
      break;
    case 17:
      var value = new proto.ubo.v1.AssistantUpdateProvidersAction;
      reader.readMessage(value,proto.ubo.v1.AssistantUpdateProvidersAction.deserializeBinaryFromReader);
      msg.setAssistantUpdateProvidersAction(value);
      break;
    case 18:
      var value = new proto.ubo.v1.AudioAction;
      reader.readMessage(value,proto.ubo.v1.AudioAction.deserializeBinaryFromReader);
      msg.setAudioAction(value);
      break;
    case 19:
      var value = new proto.ubo.v1.AudioChangeVolumeAction;
      reader.readMessage(value,proto.ubo.v1.AudioChangeVolumeAction.deserializeBinaryFromReader);
      msg.setAudioChangeVolumeAction(value);
      break;
    case 20:
      var value = new proto.ubo.v1.AudioInstallDriverAction;
      reader.readMessage(value,proto.ubo.v1.AudioInstallDriverAction.deserializeBinaryFromReader);
      msg.setAudioInstallDriverAction(value);
      break;
    case 21:
      var value = new proto.ubo.v1.AudioPlayAudioSampleAction;
      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSampleAction.deserializeBinaryFromReader);
      msg.setAudioPlayAudioSampleAction(value);
      break;
    case 22:
      var value = new proto.ubo.v1.AudioPlayAudioSequenceAction;
      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSequenceAction.deserializeBinaryFromReader);
      msg.setAudioPlayAudioSequenceAction(value);
      break;
    case 23:
      var value = new proto.ubo.v1.AudioPlayChimeAction;
      reader.readMessage(value,proto.ubo.v1.AudioPlayChimeAction.deserializeBinaryFromReader);
      msg.setAudioPlayChimeAction(value);
      break;
    case 24:
      var value = new proto.ubo.v1.AudioPlayRecordingAction;
      reader.readMessage(value,proto.ubo.v1.AudioPlayRecordingAction.deserializeBinaryFromReader);
      msg.setAudioPlayRecordingAction(value);
      break;
    case 25:
      var value = new proto.ubo.v1.AudioPlaybackDoneAction;
      reader.readMessage(value,proto.ubo.v1.AudioPlaybackDoneAction.deserializeBinaryFromReader);
      msg.setAudioPlaybackDoneAction(value);
      break;
    case 26:
      var value = new proto.ubo.v1.AudioReportSampleAction;
      reader.readMessage(value,proto.ubo.v1.AudioReportSampleAction.deserializeBinaryFromReader);
      msg.setAudioReportSampleAction(value);
      break;
    case 27:
      var value = new proto.ubo.v1.AudioSetMuteStatusAction;
      reader.readMessage(value,proto.ubo.v1.AudioSetMuteStatusAction.deserializeBinaryFromReader);
      msg.setAudioSetMuteStatusAction(value);
      break;
    case 28:
      var value = new proto.ubo.v1.AudioSetVolumeAction;
      reader.readMessage(value,proto.ubo.v1.AudioSetVolumeAction.deserializeBinaryFromReader);
      msg.setAudioSetVolumeAction(value);
      break;
    case 29:
      var value = new proto.ubo.v1.AudioStartRecordingAction;
      reader.readMessage(value,proto.ubo.v1.AudioStartRecordingAction.deserializeBinaryFromReader);
      msg.setAudioStartRecordingAction(value);
      break;
    case 30:
      var value = new proto.ubo.v1.AudioStopRecordingAction;
      reader.readMessage(value,proto.ubo.v1.AudioStopRecordingAction.deserializeBinaryFromReader);
      msg.setAudioStopRecordingAction(value);
      break;
    case 31:
      var value = new proto.ubo.v1.AudioToggleMuteStatusAction;
      reader.readMessage(value,proto.ubo.v1.AudioToggleMuteStatusAction.deserializeBinaryFromReader);
      msg.setAudioToggleMuteStatusAction(value);
      break;
    case 32:
      var value = new proto.ubo.v1.AudioToggleRecordingAction;
      reader.readMessage(value,proto.ubo.v1.AudioToggleRecordingAction.deserializeBinaryFromReader);
      msg.setAudioToggleRecordingAction(value);
      break;
    case 33:
      var value = new proto.ubo.v1.CameraAction;
      reader.readMessage(value,proto.ubo.v1.CameraAction.deserializeBinaryFromReader);
      msg.setCameraAction(value);
      break;
    case 34:
      var value = new proto.ubo.v1.CameraDetectAction;
      reader.readMessage(value,proto.ubo.v1.CameraDetectAction.deserializeBinaryFromReader);
      msg.setCameraDetectAction(value);
      break;
    case 35:
      var value = new proto.ubo.v1.CameraInstallDriverAction;
      reader.readMessage(value,proto.ubo.v1.CameraInstallDriverAction.deserializeBinaryFromReader);
      msg.setCameraInstallDriverAction(value);
      break;
    case 36:
      var value = new proto.ubo.v1.CameraReportBarcodeAction;
      reader.readMessage(value,proto.ubo.v1.CameraReportBarcodeAction.deserializeBinaryFromReader);
      msg.setCameraReportBarcodeAction(value);
      break;
    case 37:
      var value = new proto.ubo.v1.CameraRestoreDefaultAction;
      reader.readMessage(value,proto.ubo.v1.CameraRestoreDefaultAction.deserializeBinaryFromReader);
      msg.setCameraRestoreDefaultAction(value);
      break;
    case 38:
      var value = new proto.ubo.v1.CameraSetAvailableCamerasAction;
      reader.readMessage(value,proto.ubo.v1.CameraSetAvailableCamerasAction.deserializeBinaryFromReader);
      msg.setCameraSetAvailableCamerasAction(value);
      break;
    case 39:
      var value = new proto.ubo.v1.CameraSetIndexAction;
      reader.readMessage(value,proto.ubo.v1.CameraSetIndexAction.deserializeBinaryFromReader);
      msg.setCameraSetIndexAction(value);
      break;
    case 40:
      var value = new proto.ubo.v1.CameraStartViewfinderAction;
      reader.readMessage(value,proto.ubo.v1.CameraStartViewfinderAction.deserializeBinaryFromReader);
      msg.setCameraStartViewfinderAction(value);
      break;
    case 41:
      var value = new proto.ubo.v1.CloseApplicationAction;
      reader.readMessage(value,proto.ubo.v1.CloseApplicationAction.deserializeBinaryFromReader);
      msg.setCloseApplicationAction(value);
      break;
    case 42:
      var value = new proto.ubo.v1.DeregisterRegularAppAction;
      reader.readMessage(value,proto.ubo.v1.DeregisterRegularAppAction.deserializeBinaryFromReader);
      msg.setDeregisterRegularAppAction(value);
      break;
    case 43:
      var value = new proto.ubo.v1.DisplayAction;
      reader.readMessage(value,proto.ubo.v1.DisplayAction.deserializeBinaryFromReader);
      msg.setDisplayAction(value);
      break;
    case 44:
      var value = new proto.ubo.v1.DisplayBlankAction;
      reader.readMessage(value,proto.ubo.v1.DisplayBlankAction.deserializeBinaryFromReader);
      msg.setDisplayBlankAction(value);
      break;
    case 45:
      var value = new proto.ubo.v1.DisplayPauseAction;
      reader.readMessage(value,proto.ubo.v1.DisplayPauseAction.deserializeBinaryFromReader);
      msg.setDisplayPauseAction(value);
      break;
    case 46:
      var value = new proto.ubo.v1.DisplayRedrawAction;
      reader.readMessage(value,proto.ubo.v1.DisplayRedrawAction.deserializeBinaryFromReader);
      msg.setDisplayRedrawAction(value);
      break;
    case 47:
      var value = new proto.ubo.v1.DisplayResumeAction;
      reader.readMessage(value,proto.ubo.v1.DisplayResumeAction.deserializeBinaryFromReader);
      msg.setDisplayResumeAction(value);
      break;
    case 48:
      var value = new proto.ubo.v1.DisplaySetBlankTimeoutAction;
      reader.readMessage(value,proto.ubo.v1.DisplaySetBlankTimeoutAction.deserializeBinaryFromReader);
      msg.setDisplaySetBlankTimeoutAction(value);
      break;
    case 49:
      var value = new proto.ubo.v1.DisplayUnblankAction;
      reader.readMessage(value,proto.ubo.v1.DisplayUnblankAction.deserializeBinaryFromReader);
      msg.setDisplayUnblankAction(value);
      break;
    case 50:
      var value = new proto.ubo.v1.DisplayUpdateActivityAction;
      reader.readMessage(value,proto.ubo.v1.DisplayUpdateActivityAction.deserializeBinaryFromReader);
      msg.setDisplayUpdateActivityAction(value);
      break;
    case 51:
      var value = new proto.ubo.v1.DockerAction;
      reader.readMessage(value,proto.ubo.v1.DockerAction.deserializeBinaryFromReader);
      msg.setDockerAction(value);
      break;
    case 52:
      var value = new proto.ubo.v1.DockerImageAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageAction.deserializeBinaryFromReader);
      msg.setDockerImageAction(value);
      break;
    case 53:
      var value = new proto.ubo.v1.DockerImageFetchAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageFetchAction.deserializeBinaryFromReader);
      msg.setDockerImageFetchAction(value);
      break;
    case 54:
      var value = new proto.ubo.v1.DockerImageReleaseAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageReleaseAction.deserializeBinaryFromReader);
      msg.setDockerImageReleaseAction(value);
      break;
    case 55:
      var value = new proto.ubo.v1.DockerImageRemoveAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveAction.deserializeBinaryFromReader);
      msg.setDockerImageRemoveAction(value);
      break;
    case 56:
      var value = new proto.ubo.v1.DockerImageRemoveContainerAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveContainerAction.deserializeBinaryFromReader);
      msg.setDockerImageRemoveContainerAction(value);
      break;
    case 57:
      var value = new proto.ubo.v1.DockerImageRunAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageRunAction.deserializeBinaryFromReader);
      msg.setDockerImageRunAction(value);
      break;
    case 58:
      var value = new proto.ubo.v1.DockerImageSetDockerIdAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageSetDockerIdAction.deserializeBinaryFromReader);
      msg.setDockerImageSetDockerIdAction(value);
      break;
    case 59:
      var value = new proto.ubo.v1.DockerImageSetStatusAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageSetStatusAction.deserializeBinaryFromReader);
      msg.setDockerImageSetStatusAction(value);
      break;
    case 60:
      var value = new proto.ubo.v1.DockerImageStopAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageStopAction.deserializeBinaryFromReader);
      msg.setDockerImageStopAction(value);
      break;
    case 61:
      var value = new proto.ubo.v1.DockerImageUpdateMetadataAction;
      reader.readMessage(value,proto.ubo.v1.DockerImageUpdateMetadataAction.deserializeBinaryFromReader);
      msg.setDockerImageUpdateMetadataAction(value);
      break;
    case 62:
      var value = new proto.ubo.v1.DockerInstallAction;
      reader.readMessage(value,proto.ubo.v1.DockerInstallAction.deserializeBinaryFromReader);
      msg.setDockerInstallAction(value);
      break;
    case 63:
      var value = new proto.ubo.v1.DockerRemoveUsernameAction;
      reader.readMessage(value,proto.ubo.v1.DockerRemoveUsernameAction.deserializeBinaryFromReader);
      msg.setDockerRemoveUsernameAction(value);
      break;
    case 64:
      var value = new proto.ubo.v1.DockerSetStatusAction;
      reader.readMessage(value,proto.ubo.v1.DockerSetStatusAction.deserializeBinaryFromReader);
      msg.setDockerSetStatusAction(value);
      break;
    case 65:
      var value = new proto.ubo.v1.DockerStartAction;
      reader.readMessage(value,proto.ubo.v1.DockerStartAction.deserializeBinaryFromReader);
      msg.setDockerStartAction(value);
      break;
    case 66:
      var value = new proto.ubo.v1.DockerStopAction;
      reader.readMessage(value,proto.ubo.v1.DockerStopAction.deserializeBinaryFromReader);
      msg.setDockerStopAction(value);
      break;
    case 67:
      var value = new proto.ubo.v1.DockerStoreUsernameAction;
      reader.readMessage(value,proto.ubo.v1.DockerStoreUsernameAction.deserializeBinaryFromReader);
      msg.setDockerStoreUsernameAction(value);
      break;
    case 68:
      var value = new proto.ubo.v1.FileSystemAction;
      reader.readMessage(value,proto.ubo.v1.FileSystemAction.deserializeBinaryFromReader);
      msg.setFileSystemAction(value);
      break;
    case 69:
      var value = new proto.ubo.v1.FileSystemCopyAction;
      reader.readMessage(value,proto.ubo.v1.FileSystemCopyAction.deserializeBinaryFromReader);
      msg.setFileSystemCopyAction(value);
      break;
    case 70:
      var value = new proto.ubo.v1.FileSystemMoveAction;
      reader.readMessage(value,proto.ubo.v1.FileSystemMoveAction.deserializeBinaryFromReader);
      msg.setFileSystemMoveAction(value);
      break;
    case 71:
      var value = new proto.ubo.v1.FileSystemRemoveAction;
      reader.readMessage(value,proto.ubo.v1.FileSystemRemoveAction.deserializeBinaryFromReader);
      msg.setFileSystemRemoveAction(value);
      break;
    case 72:
      var value = new proto.ubo.v1.FileSystemReportSelectionAction;
      reader.readMessage(value,proto.ubo.v1.FileSystemReportSelectionAction.deserializeBinaryFromReader);
      msg.setFileSystemReportSelectionAction(value);
      break;
    case 73:
      var value = new proto.ubo.v1.InfraredAction;
      reader.readMessage(value,proto.ubo.v1.InfraredAction.deserializeBinaryFromReader);
      msg.setInfraredAction(value);
      break;
    case 74:
      var value = new proto.ubo.v1.InfraredHandleReceivedCodeAction;
      reader.readMessage(value,proto.ubo.v1.InfraredHandleReceivedCodeAction.deserializeBinaryFromReader);
      msg.setInfraredHandleReceivedCodeAction(value);
      break;
    case 75:
      var value = new proto.ubo.v1.InfraredSendCodeAction;
      reader.readMessage(value,proto.ubo.v1.InfraredSendCodeAction.deserializeBinaryFromReader);
      msg.setInfraredSendCodeAction(value);
      break;
    case 76:
      var value = new proto.ubo.v1.InfraredSetShouldPropagateAction;
      reader.readMessage(value,proto.ubo.v1.InfraredSetShouldPropagateAction.deserializeBinaryFromReader);
      msg.setInfraredSetShouldPropagateAction(value);
      break;
    case 77:
      var value = new proto.ubo.v1.InfraredSetShouldReceiveAction;
      reader.readMessage(value,proto.ubo.v1.InfraredSetShouldReceiveAction.deserializeBinaryFromReader);
      msg.setInfraredSetShouldReceiveAction(value);
      break;
    case 78:
      var value = new proto.ubo.v1.InputAction;
      reader.readMessage(value,proto.ubo.v1.InputAction.deserializeBinaryFromReader);
      msg.setInputAction(value);
      break;
    case 79:
      var value = new proto.ubo.v1.InputCancelAction;
      reader.readMessage(value,proto.ubo.v1.InputCancelAction.deserializeBinaryFromReader);
      msg.setInputCancelAction(value);
      break;
    case 80:
      var value = new proto.ubo.v1.InputDemandAction;
      reader.readMessage(value,proto.ubo.v1.InputDemandAction.deserializeBinaryFromReader);
      msg.setInputDemandAction(value);
      break;
    case 81:
      var value = new proto.ubo.v1.InputProvideAction;
      reader.readMessage(value,proto.ubo.v1.InputProvideAction.deserializeBinaryFromReader);
      msg.setInputProvideAction(value);
      break;
    case 82:
      var value = new proto.ubo.v1.InputResolveAction;
      reader.readMessage(value,proto.ubo.v1.InputResolveAction.deserializeBinaryFromReader);
      msg.setInputResolveAction(value);
      break;
    case 83:
      var value = new proto.ubo.v1.IpAction;
      reader.readMessage(value,proto.ubo.v1.IpAction.deserializeBinaryFromReader);
      msg.setIpAction(value);
      break;
    case 84:
      var value = new proto.ubo.v1.IpSetIsConnectedAction;
      reader.readMessage(value,proto.ubo.v1.IpSetIsConnectedAction.deserializeBinaryFromReader);
      msg.setIpSetIsConnectedAction(value);
      break;
    case 85:
      var value = new proto.ubo.v1.IpUpdateInterfacesAction;
      reader.readMessage(value,proto.ubo.v1.IpUpdateInterfacesAction.deserializeBinaryFromReader);
      msg.setIpUpdateInterfacesAction(value);
      break;
    case 86:
      var value = new proto.ubo.v1.KeypadAction;
      reader.readMessage(value,proto.ubo.v1.KeypadAction.deserializeBinaryFromReader);
      msg.setKeypadAction(value);
      break;
    case 87:
      var value = new proto.ubo.v1.KeypadKeyHoldAction;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyHoldAction.deserializeBinaryFromReader);
      msg.setKeypadKeyHoldAction(value);
      break;
    case 88:
      var value = new proto.ubo.v1.KeypadKeyPressAction;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyPressAction.deserializeBinaryFromReader);
      msg.setKeypadKeyPressAction(value);
      break;
    case 89:
      var value = new proto.ubo.v1.KeypadKeyReleaseAction;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyReleaseAction.deserializeBinaryFromReader);
      msg.setKeypadKeyReleaseAction(value);
      break;
    case 90:
      var value = new proto.ubo.v1.KeypadKeyUnholdAction;
      reader.readMessage(value,proto.ubo.v1.KeypadKeyUnholdAction.deserializeBinaryFromReader);
      msg.setKeypadKeyUnholdAction(value);
      break;
    case 91:
      var value = new proto.ubo.v1.LightDMAction;
      reader.readMessage(value,proto.ubo.v1.LightDMAction.deserializeBinaryFromReader);
      msg.setLightDmAction(value);
      break;
    case 92:
      var value = new proto.ubo.v1.LightDMClearEnabledStateAction;
      reader.readMessage(value,proto.ubo.v1.LightDMClearEnabledStateAction.deserializeBinaryFromReader);
      msg.setLightDmClearEnabledStateAction(value);
      break;
    case 93:
      var value = new proto.ubo.v1.LightDMUpdateStateAction;
      reader.readMessage(value,proto.ubo.v1.LightDMUpdateStateAction.deserializeBinaryFromReader);
      msg.setLightDmUpdateStateAction(value);
      break;
    case 94:
      var value = new proto.ubo.v1.MainAction;
      reader.readMessage(value,proto.ubo.v1.MainAction.deserializeBinaryFromReader);
      msg.setMainAction(value);
      break;
    case 95:
      var value = new proto.ubo.v1.MenuAction;
      reader.readMessage(value,proto.ubo.v1.MenuAction.deserializeBinaryFromReader);
      msg.setMenuAction(value);
      break;
    case 96:
      var value = new proto.ubo.v1.MenuChooseByIconAction;
      reader.readMessage(value,proto.ubo.v1.MenuChooseByIconAction.deserializeBinaryFromReader);
      msg.setMenuChooseByIconAction(value);
      break;
    case 97:
      var value = new proto.ubo.v1.MenuChooseByIndexAction;
      reader.readMessage(value,proto.ubo.v1.MenuChooseByIndexAction.deserializeBinaryFromReader);
      msg.setMenuChooseByIndexAction(value);
      break;
    case 98:
      var value = new proto.ubo.v1.MenuChooseByLabelAction;
      reader.readMessage(value,proto.ubo.v1.MenuChooseByLabelAction.deserializeBinaryFromReader);
      msg.setMenuChooseByLabelAction(value);
      break;
    case 99:
      var value = new proto.ubo.v1.MenuGoBackAction;
      reader.readMessage(value,proto.ubo.v1.MenuGoBackAction.deserializeBinaryFromReader);
      msg.setMenuGoBackAction(value);
      break;
    case 100:
      var value = new proto.ubo.v1.MenuGoHomeAction;
      reader.readMessage(value,proto.ubo.v1.MenuGoHomeAction.deserializeBinaryFromReader);
      msg.setMenuGoHomeAction(value);
      break;
    case 101:
      var value = new proto.ubo.v1.MenuScrollAction;
      reader.readMessage(value,proto.ubo.v1.MenuScrollAction.deserializeBinaryFromReader);
      msg.setMenuScrollAction(value);
      break;
    case 102:
      var value = new proto.ubo.v1.NotificationsAction;
      reader.readMessage(value,proto.ubo.v1.NotificationsAction.deserializeBinaryFromReader);
      msg.setNotificationsAction(value);
      break;
    case 103:
      var value = new proto.ubo.v1.NotificationsAddAction;
      reader.readMessage(value,proto.ubo.v1.NotificationsAddAction.deserializeBinaryFromReader);
      msg.setNotificationsAddAction(value);
      break;
    case 104:
      var value = new proto.ubo.v1.NotificationsClearAction;
      reader.readMessage(value,proto.ubo.v1.NotificationsClearAction.deserializeBinaryFromReader);
      msg.setNotificationsClearAction(value);
      break;
    case 105:
      var value = new proto.ubo.v1.NotificationsClearAllAction;
      reader.readMessage(value,proto.ubo.v1.NotificationsClearAllAction.deserializeBinaryFromReader);
      msg.setNotificationsClearAllAction(value);
      break;
    case 106:
      var value = new proto.ubo.v1.NotificationsClearByIdAction;
      reader.readMessage(value,proto.ubo.v1.NotificationsClearByIdAction.deserializeBinaryFromReader);
      msg.setNotificationsClearByIdAction(value);
      break;
    case 107:
      var value = new proto.ubo.v1.NotificationsDisplayAction;
      reader.readMessage(value,proto.ubo.v1.NotificationsDisplayAction.deserializeBinaryFromReader);
      msg.setNotificationsDisplayAction(value);
      break;
    case 108:
      var value = new proto.ubo.v1.OpenApplicationAction;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationAction.deserializeBinaryFromReader);
      msg.setOpenApplicationAction(value);
      break;
    case 109:
      var value = new proto.ubo.v1.PowerAction;
      reader.readMessage(value,proto.ubo.v1.PowerAction.deserializeBinaryFromReader);
      msg.setPowerAction(value);
      break;
    case 110:
      var value = new proto.ubo.v1.PowerOffAction;
      reader.readMessage(value,proto.ubo.v1.PowerOffAction.deserializeBinaryFromReader);
      msg.setPowerOffAction(value);
      break;
    case 111:
      var value = new proto.ubo.v1.RPiConnectAction;
      reader.readMessage(value,proto.ubo.v1.RPiConnectAction.deserializeBinaryFromReader);
      msg.setRPiConnectAction(value);
      break;
    case 112:
      var value = new proto.ubo.v1.RPiConnectDoneDownloadingAction;
      reader.readMessage(value,proto.ubo.v1.RPiConnectDoneDownloadingAction.deserializeBinaryFromReader);
      msg.setRPiConnectDoneDownloadingAction(value);
      break;
    case 113:
      var value = new proto.ubo.v1.RPiConnectSetPendingAction;
      reader.readMessage(value,proto.ubo.v1.RPiConnectSetPendingAction.deserializeBinaryFromReader);
      msg.setRPiConnectSetPendingAction(value);
      break;
    case 114:
      var value = new proto.ubo.v1.RPiConnectSetStatusAction;
      reader.readMessage(value,proto.ubo.v1.RPiConnectSetStatusAction.deserializeBinaryFromReader);
      msg.setRPiConnectSetStatusAction(value);
      break;
    case 115:
      var value = new proto.ubo.v1.RPiConnectStartDownloadingAction;
      reader.readMessage(value,proto.ubo.v1.RPiConnectStartDownloadingAction.deserializeBinaryFromReader);
      msg.setRPiConnectStartDownloadingAction(value);
      break;
    case 116:
      var value = new proto.ubo.v1.RPiConnectUpdateServiceStateAction;
      reader.readMessage(value,proto.ubo.v1.RPiConnectUpdateServiceStateAction.deserializeBinaryFromReader);
      msg.setRPiConnectUpdateServiceStateAction(value);
      break;
    case 117:
      var value = new proto.ubo.v1.RebootAction;
      reader.readMessage(value,proto.ubo.v1.RebootAction.deserializeBinaryFromReader);
      msg.setRebootAction(value);
      break;
    case 118:
      var value = new proto.ubo.v1.RegisterAppAction;
      reader.readMessage(value,proto.ubo.v1.RegisterAppAction.deserializeBinaryFromReader);
      msg.setRegisterAppAction(value);
      break;
    case 119:
      var value = new proto.ubo.v1.RegisterRegularAppAction;
      reader.readMessage(value,proto.ubo.v1.RegisterRegularAppAction.deserializeBinaryFromReader);
      msg.setRegisterRegularAppAction(value);
      break;
    case 120:
      var value = new proto.ubo.v1.RegisterSettingAppAction;
      reader.readMessage(value,proto.ubo.v1.RegisterSettingAppAction.deserializeBinaryFromReader);
      msg.setRegisterSettingAppAction(value);
      break;
    case 121:
      var value = new proto.ubo.v1.ReplayRecordedSequenceAction;
      reader.readMessage(value,proto.ubo.v1.ReplayRecordedSequenceAction.deserializeBinaryFromReader);
      msg.setReplayRecordedSequenceAction(value);
      break;
    case 122:
      var value = new proto.ubo.v1.ReportReplayingDoneAction;
      reader.readMessage(value,proto.ubo.v1.ReportReplayingDoneAction.deserializeBinaryFromReader);
      msg.setReportReplayingDoneAction(value);
      break;
    case 123:
      var value = new proto.ubo.v1.RgbRingAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingAction.deserializeBinaryFromReader);
      msg.setRgbRingAction(value);
      break;
    case 124:
      var value = new proto.ubo.v1.RgbRingBlankAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingBlankAction.deserializeBinaryFromReader);
      msg.setRgbRingBlankAction(value);
      break;
    case 125:
      var value = new proto.ubo.v1.RgbRingBlinkAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingBlinkAction.deserializeBinaryFromReader);
      msg.setRgbRingBlinkAction(value);
      break;
    case 126:
      var value = new proto.ubo.v1.RgbRingColorfulCommandAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingColorfulCommandAction.deserializeBinaryFromReader);
      msg.setRgbRingColorfulCommandAction(value);
      break;
    case 127:
      var value = new proto.ubo.v1.RgbRingCommandAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingCommandAction.deserializeBinaryFromReader);
      msg.setRgbRingCommandAction(value);
      break;
    case 128:
      var value = new proto.ubo.v1.RgbRingFillDownfromAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingFillDownfromAction.deserializeBinaryFromReader);
      msg.setRgbRingFillDownfromAction(value);
      break;
    case 129:
      var value = new proto.ubo.v1.RgbRingFillUptoAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingFillUptoAction.deserializeBinaryFromReader);
      msg.setRgbRingFillUptoAction(value);
      break;
    case 130:
      var value = new proto.ubo.v1.RgbRingProgressWheelAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingProgressWheelAction.deserializeBinaryFromReader);
      msg.setRgbRingProgressWheelAction(value);
      break;
    case 131:
      var value = new proto.ubo.v1.RgbRingProgressWheelStepAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingProgressWheelStepAction.deserializeBinaryFromReader);
      msg.setRgbRingProgressWheelStepAction(value);
      break;
    case 132:
      var value = new proto.ubo.v1.RgbRingPulseAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingPulseAction.deserializeBinaryFromReader);
      msg.setRgbRingPulseAction(value);
      break;
    case 133:
      var value = new proto.ubo.v1.RgbRingRainbowAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingRainbowAction.deserializeBinaryFromReader);
      msg.setRgbRingRainbowAction(value);
      break;
    case 134:
      var value = new proto.ubo.v1.RgbRingSequenceAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingSequenceAction.deserializeBinaryFromReader);
      msg.setRgbRingSequenceAction(value);
      break;
    case 135:
      var value = new proto.ubo.v1.RgbRingSetAllAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingSetAllAction.deserializeBinaryFromReader);
      msg.setRgbRingSetAllAction(value);
      break;
    case 136:
      var value = new proto.ubo.v1.RgbRingSetBrightnessAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingSetBrightnessAction.deserializeBinaryFromReader);
      msg.setRgbRingSetBrightnessAction(value);
      break;
    case 137:
      var value = new proto.ubo.v1.RgbRingSetEnabledAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingSetEnabledAction.deserializeBinaryFromReader);
      msg.setRgbRingSetEnabledAction(value);
      break;
    case 138:
      var value = new proto.ubo.v1.RgbRingSetIsBusyAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingSetIsBusyAction.deserializeBinaryFromReader);
      msg.setRgbRingSetIsBusyAction(value);
      break;
    case 139:
      var value = new proto.ubo.v1.RgbRingSpinningWheelAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingSpinningWheelAction.deserializeBinaryFromReader);
      msg.setRgbRingSpinningWheelAction(value);
      break;
    case 140:
      var value = new proto.ubo.v1.RgbRingWaitableCommandAction;
      reader.readMessage(value,proto.ubo.v1.RgbRingWaitableCommandAction.deserializeBinaryFromReader);
      msg.setRgbRingWaitableCommandAction(value);
      break;
    case 141:
      var value = new proto.ubo.v1.SSHAction;
      reader.readMessage(value,proto.ubo.v1.SSHAction.deserializeBinaryFromReader);
      msg.setSshAction(value);
      break;
    case 142:
      var value = new proto.ubo.v1.SSHClearEnabledStateAction;
      reader.readMessage(value,proto.ubo.v1.SSHClearEnabledStateAction.deserializeBinaryFromReader);
      msg.setSshClearEnabledStateAction(value);
      break;
    case 143:
      var value = new proto.ubo.v1.SSHUpdateStateAction;
      reader.readMessage(value,proto.ubo.v1.SSHUpdateStateAction.deserializeBinaryFromReader);
      msg.setSshUpdateStateAction(value);
      break;
    case 144:
      var value = new proto.ubo.v1.SensorsAction;
      reader.readMessage(value,proto.ubo.v1.SensorsAction.deserializeBinaryFromReader);
      msg.setSensorsAction(value);
      break;
    case 145:
      var value = new proto.ubo.v1.SensorsReportReadingAction;
      reader.readMessage(value,proto.ubo.v1.SensorsReportReadingAction.deserializeBinaryFromReader);
      msg.setSensorsReportReadingAction(value);
      break;
    case 146:
      var value = new proto.ubo.v1.SetAreEnclosuresVisibleAction;
      reader.readMessage(value,proto.ubo.v1.SetAreEnclosuresVisibleAction.deserializeBinaryFromReader);
      msg.setSetAreEnclosuresVisibleAction(value);
      break;
    case 147:
      var value = new proto.ubo.v1.SetMenuPathAction;
      reader.readMessage(value,proto.ubo.v1.SetMenuPathAction.deserializeBinaryFromReader);
      msg.setSetMenuPathAction(value);
      break;
    case 148:
      var value = new proto.ubo.v1.SettingsAction;
      reader.readMessage(value,proto.ubo.v1.SettingsAction.deserializeBinaryFromReader);
      msg.setSettingsAction(value);
      break;
    case 149:
      var value = new proto.ubo.v1.SettingsClearServiceErrorsAction;
      reader.readMessage(value,proto.ubo.v1.SettingsClearServiceErrorsAction.deserializeBinaryFromReader);
      msg.setSettingsClearServiceErrorsAction(value);
      break;
    case 150:
      var value = new proto.ubo.v1.SettingsReportServiceErrorAction;
      reader.readMessage(value,proto.ubo.v1.SettingsReportServiceErrorAction.deserializeBinaryFromReader);
      msg.setSettingsReportServiceErrorAction(value);
      break;
    case 151:
      var value = new proto.ubo.v1.SettingsServiceAction;
      reader.readMessage(value,proto.ubo.v1.SettingsServiceAction.deserializeBinaryFromReader);
      msg.setSettingsServiceAction(value);
      break;
    case 152:
      var value = new proto.ubo.v1.SettingsServiceSetIsEnabledAction;
      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetIsEnabledAction.deserializeBinaryFromReader);
      msg.setSettingsServiceSetIsEnabledAction(value);
      break;
    case 153:
      var value = new proto.ubo.v1.SettingsServiceSetLogLevelAction;
      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetLogLevelAction.deserializeBinaryFromReader);
      msg.setSettingsServiceSetLogLevelAction(value);
      break;
    case 154:
      var value = new proto.ubo.v1.SettingsServiceSetShouldRestartAction;
      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetShouldRestartAction.deserializeBinaryFromReader);
      msg.setSettingsServiceSetShouldRestartAction(value);
      break;
    case 155:
      var value = new proto.ubo.v1.SettingsServiceSetStatusAction;
      reader.readMessage(value,proto.ubo.v1.SettingsServiceSetStatusAction.deserializeBinaryFromReader);
      msg.setSettingsServiceSetStatusAction(value);
      break;
    case 156:
      var value = new proto.ubo.v1.SettingsSetServicesAction;
      reader.readMessage(value,proto.ubo.v1.SettingsSetServicesAction.deserializeBinaryFromReader);
      msg.setSettingsSetServicesAction(value);
      break;
    case 157:
      var value = new proto.ubo.v1.SettingsStartServiceAction;
      reader.readMessage(value,proto.ubo.v1.SettingsStartServiceAction.deserializeBinaryFromReader);
      msg.setSettingsStartServiceAction(value);
      break;
    case 158:
      var value = new proto.ubo.v1.SettingsStopServiceAction;
      reader.readMessage(value,proto.ubo.v1.SettingsStopServiceAction.deserializeBinaryFromReader);
      msg.setSettingsStopServiceAction(value);
      break;
    case 159:
      var value = new proto.ubo.v1.SettingsToggleBetaVersionsAction;
      reader.readMessage(value,proto.ubo.v1.SettingsToggleBetaVersionsAction.deserializeBinaryFromReader);
      msg.setSettingsToggleBetaVersionsAction(value);
      break;
    case 160:
      var value = new proto.ubo.v1.SettingsTogglePdbSignalAction;
      reader.readMessage(value,proto.ubo.v1.SettingsTogglePdbSignalAction.deserializeBinaryFromReader);
      msg.setSettingsTogglePdbSignalAction(value);
      break;
    case 161:
      var value = new proto.ubo.v1.SettingsToggleVisualDebugAction;
      reader.readMessage(value,proto.ubo.v1.SettingsToggleVisualDebugAction.deserializeBinaryFromReader);
      msg.setSettingsToggleVisualDebugAction(value);
      break;
    case 162:
      var value = new proto.ubo.v1.SpeechRecognitionAction;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionAction.deserializeBinaryFromReader);
      msg.setSpeechRecognitionAction(value);
      break;
    case 163:
      var value = new proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.deserializeBinaryFromReader);
      msg.setSpeechRecognitionReportIntentDetectionAction(value);
      break;
    case 164:
      var value = new proto.ubo.v1.SpeechRecognitionReportSpeechAction;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportSpeechAction.deserializeBinaryFromReader);
      msg.setSpeechRecognitionReportSpeechAction(value);
      break;
    case 165:
      var value = new proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.deserializeBinaryFromReader);
      msg.setSpeechRecognitionReportWakeWordDetectionAction(value);
      break;
    case 166:
      var value = new proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.deserializeBinaryFromReader);
      msg.setSpeechRecognitionSetIsAssistantActiveAction(value);
      break;
    case 167:
      var value = new proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.deserializeBinaryFromReader);
      msg.setSpeechRecognitionSetIsIntentsActiveAction(value);
      break;
    case 168:
      var value = new proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.deserializeBinaryFromReader);
      msg.setSpeechRecognitionSetSelectedEngineAction(value);
      break;
    case 169:
      var value = new proto.ubo.v1.SpeechSynthesisAction;
      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisAction.deserializeBinaryFromReader);
      msg.setSpeechSynthesisAction(value);
      break;
    case 170:
      var value = new proto.ubo.v1.SpeechSynthesisReadTextAction;
      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisReadTextAction.deserializeBinaryFromReader);
      msg.setSpeechSynthesisReadTextAction(value);
      break;
    case 171:
      var value = new proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction;
      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.deserializeBinaryFromReader);
      msg.setSpeechSynthesisSetSelectedEngineAction(value);
      break;
    case 172:
      var value = new proto.ubo.v1.StatusIconsAction;
      reader.readMessage(value,proto.ubo.v1.StatusIconsAction.deserializeBinaryFromReader);
      msg.setStatusIconsAction(value);
      break;
    case 173:
      var value = new proto.ubo.v1.StatusIconsRegisterAction;
      reader.readMessage(value,proto.ubo.v1.StatusIconsRegisterAction.deserializeBinaryFromReader);
      msg.setStatusIconsRegisterAction(value);
      break;
    case 174:
      var value = new proto.ubo.v1.ToggleRecordingAction;
      reader.readMessage(value,proto.ubo.v1.ToggleRecordingAction.deserializeBinaryFromReader);
      msg.setToggleRecordingAction(value);
      break;
    case 175:
      var value = new proto.ubo.v1.UpdateManagerAction;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerAction.deserializeBinaryFromReader);
      msg.setUpdateManagerAction(value);
      break;
    case 176:
      var value = new proto.ubo.v1.UpdateManagerReportFailedCheckAction;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerReportFailedCheckAction.deserializeBinaryFromReader);
      msg.setUpdateManagerReportFailedCheckAction(value);
      break;
    case 177:
      var value = new proto.ubo.v1.UpdateManagerRequestCheckAction;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerRequestCheckAction.deserializeBinaryFromReader);
      msg.setUpdateManagerRequestCheckAction(value);
      break;
    case 178:
      var value = new proto.ubo.v1.UpdateManagerRequestUpdateAction;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerRequestUpdateAction.deserializeBinaryFromReader);
      msg.setUpdateManagerRequestUpdateAction(value);
      break;
    case 179:
      var value = new proto.ubo.v1.UpdateManagerSetVersionsAction;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerSetVersionsAction.deserializeBinaryFromReader);
      msg.setUpdateManagerSetVersionsAction(value);
      break;
    case 180:
      var value = new proto.ubo.v1.UsersAction;
      reader.readMessage(value,proto.ubo.v1.UsersAction.deserializeBinaryFromReader);
      msg.setUsersAction(value);
      break;
    case 181:
      var value = new proto.ubo.v1.UsersCreateUserAction;
      reader.readMessage(value,proto.ubo.v1.UsersCreateUserAction.deserializeBinaryFromReader);
      msg.setUsersCreateUserAction(value);
      break;
    case 182:
      var value = new proto.ubo.v1.UsersDeleteUserAction;
      reader.readMessage(value,proto.ubo.v1.UsersDeleteUserAction.deserializeBinaryFromReader);
      msg.setUsersDeleteUserAction(value);
      break;
    case 183:
      var value = new proto.ubo.v1.UsersResetPasswordAction;
      reader.readMessage(value,proto.ubo.v1.UsersResetPasswordAction.deserializeBinaryFromReader);
      msg.setUsersResetPasswordAction(value);
      break;
    case 184:
      var value = new proto.ubo.v1.UsersSetUsersAction;
      reader.readMessage(value,proto.ubo.v1.UsersSetUsersAction.deserializeBinaryFromReader);
      msg.setUsersSetUsersAction(value);
      break;
    case 185:
      var value = new proto.ubo.v1.VSCodeAction;
      reader.readMessage(value,proto.ubo.v1.VSCodeAction.deserializeBinaryFromReader);
      msg.setVsCodeAction(value);
      break;
    case 186:
      var value = new proto.ubo.v1.VSCodeDoneDownloadingAction;
      reader.readMessage(value,proto.ubo.v1.VSCodeDoneDownloadingAction.deserializeBinaryFromReader);
      msg.setVsCodeDoneDownloadingAction(value);
      break;
    case 187:
      var value = new proto.ubo.v1.VSCodeSetPendingAction;
      reader.readMessage(value,proto.ubo.v1.VSCodeSetPendingAction.deserializeBinaryFromReader);
      msg.setVsCodeSetPendingAction(value);
      break;
    case 188:
      var value = new proto.ubo.v1.VSCodeSetStatusAction;
      reader.readMessage(value,proto.ubo.v1.VSCodeSetStatusAction.deserializeBinaryFromReader);
      msg.setVsCodeSetStatusAction(value);
      break;
    case 189:
      var value = new proto.ubo.v1.VSCodeStartDownloadingAction;
      reader.readMessage(value,proto.ubo.v1.VSCodeStartDownloadingAction.deserializeBinaryFromReader);
      msg.setVsCodeStartDownloadingAction(value);
      break;
    case 190:
      var value = new proto.ubo.v1.WiFiAction;
      reader.readMessage(value,proto.ubo.v1.WiFiAction.deserializeBinaryFromReader);
      msg.setWiFiAction(value);
      break;
    case 191:
      var value = new proto.ubo.v1.WiFiInputConnectionAction;
      reader.readMessage(value,proto.ubo.v1.WiFiInputConnectionAction.deserializeBinaryFromReader);
      msg.setWiFiInputConnectionAction(value);
      break;
    case 192:
      var value = new proto.ubo.v1.WiFiSetHasVisitedOnboardingAction;
      reader.readMessage(value,proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.deserializeBinaryFromReader);
      msg.setWiFiSetHasVisitedOnboardingAction(value);
      break;
    case 193:
      var value = new proto.ubo.v1.WiFiUpdateAction;
      reader.readMessage(value,proto.ubo.v1.WiFiUpdateAction.deserializeBinaryFromReader);
      msg.setWiFiUpdateAction(value);
      break;
    case 194:
      var value = new proto.ubo.v1.WiFiUpdateRequestAction;
      reader.readMessage(value,proto.ubo.v1.WiFiUpdateRequestAction.deserializeBinaryFromReader);
      msg.setWiFiUpdateRequestAction(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Action.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Action.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Action} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Action.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAssistantAction();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.AssistantAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantAddMcpServerAction();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.AssistantAddMcpServerAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantDeleteMcpServerAction();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.AssistantDeleteMcpServerAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantDownloadOllamaModelAction();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.AssistantDownloadOllamaModelAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantReportAction();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.ubo.v1.AssistantReportAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantSetIsActiveAction();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.ubo.v1.AssistantSetIsActiveAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantSetSelectedImageGeneratorAction();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.ubo.v1.AssistantSetSelectedImageGeneratorAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantSetSelectedLlmAction();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.ubo.v1.AssistantSetSelectedLLMAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantSetSelectedModelAction();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.ubo.v1.AssistantSetSelectedModelAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantSetSelectedSttAction();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.ubo.v1.AssistantSetSelectedSTTAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantSetSelectedTtsAction();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.ubo.v1.AssistantSetSelectedTTSAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantStartListeningAction();
  if (f != null) {
    writer.writeMessage(
      12,
      f,
      proto.ubo.v1.AssistantStartListeningAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantStopListeningAction();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.ubo.v1.AssistantStopListeningAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantSyncMcpServersAction();
  if (f != null) {
    writer.writeMessage(
      14,
      f,
      proto.ubo.v1.AssistantSyncMcpServersAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantToggleListeningAction();
  if (f != null) {
    writer.writeMessage(
      15,
      f,
      proto.ubo.v1.AssistantToggleListeningAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantToggleMcpServerAction();
  if (f != null) {
    writer.writeMessage(
      16,
      f,
      proto.ubo.v1.AssistantToggleMcpServerAction.serializeBinaryToWriter
    );
  }
  f = message.getAssistantUpdateProvidersAction();
  if (f != null) {
    writer.writeMessage(
      17,
      f,
      proto.ubo.v1.AssistantUpdateProvidersAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioAction();
  if (f != null) {
    writer.writeMessage(
      18,
      f,
      proto.ubo.v1.AudioAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioChangeVolumeAction();
  if (f != null) {
    writer.writeMessage(
      19,
      f,
      proto.ubo.v1.AudioChangeVolumeAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioInstallDriverAction();
  if (f != null) {
    writer.writeMessage(
      20,
      f,
      proto.ubo.v1.AudioInstallDriverAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlayAudioSampleAction();
  if (f != null) {
    writer.writeMessage(
      21,
      f,
      proto.ubo.v1.AudioPlayAudioSampleAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlayAudioSequenceAction();
  if (f != null) {
    writer.writeMessage(
      22,
      f,
      proto.ubo.v1.AudioPlayAudioSequenceAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlayChimeAction();
  if (f != null) {
    writer.writeMessage(
      23,
      f,
      proto.ubo.v1.AudioPlayChimeAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlayRecordingAction();
  if (f != null) {
    writer.writeMessage(
      24,
      f,
      proto.ubo.v1.AudioPlayRecordingAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlaybackDoneAction();
  if (f != null) {
    writer.writeMessage(
      25,
      f,
      proto.ubo.v1.AudioPlaybackDoneAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioReportSampleAction();
  if (f != null) {
    writer.writeMessage(
      26,
      f,
      proto.ubo.v1.AudioReportSampleAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioSetMuteStatusAction();
  if (f != null) {
    writer.writeMessage(
      27,
      f,
      proto.ubo.v1.AudioSetMuteStatusAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioSetVolumeAction();
  if (f != null) {
    writer.writeMessage(
      28,
      f,
      proto.ubo.v1.AudioSetVolumeAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioStartRecordingAction();
  if (f != null) {
    writer.writeMessage(
      29,
      f,
      proto.ubo.v1.AudioStartRecordingAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioStopRecordingAction();
  if (f != null) {
    writer.writeMessage(
      30,
      f,
      proto.ubo.v1.AudioStopRecordingAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioToggleMuteStatusAction();
  if (f != null) {
    writer.writeMessage(
      31,
      f,
      proto.ubo.v1.AudioToggleMuteStatusAction.serializeBinaryToWriter
    );
  }
  f = message.getAudioToggleRecordingAction();
  if (f != null) {
    writer.writeMessage(
      32,
      f,
      proto.ubo.v1.AudioToggleRecordingAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraAction();
  if (f != null) {
    writer.writeMessage(
      33,
      f,
      proto.ubo.v1.CameraAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraDetectAction();
  if (f != null) {
    writer.writeMessage(
      34,
      f,
      proto.ubo.v1.CameraDetectAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraInstallDriverAction();
  if (f != null) {
    writer.writeMessage(
      35,
      f,
      proto.ubo.v1.CameraInstallDriverAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraReportBarcodeAction();
  if (f != null) {
    writer.writeMessage(
      36,
      f,
      proto.ubo.v1.CameraReportBarcodeAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraRestoreDefaultAction();
  if (f != null) {
    writer.writeMessage(
      37,
      f,
      proto.ubo.v1.CameraRestoreDefaultAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraSetAvailableCamerasAction();
  if (f != null) {
    writer.writeMessage(
      38,
      f,
      proto.ubo.v1.CameraSetAvailableCamerasAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraSetIndexAction();
  if (f != null) {
    writer.writeMessage(
      39,
      f,
      proto.ubo.v1.CameraSetIndexAction.serializeBinaryToWriter
    );
  }
  f = message.getCameraStartViewfinderAction();
  if (f != null) {
    writer.writeMessage(
      40,
      f,
      proto.ubo.v1.CameraStartViewfinderAction.serializeBinaryToWriter
    );
  }
  f = message.getCloseApplicationAction();
  if (f != null) {
    writer.writeMessage(
      41,
      f,
      proto.ubo.v1.CloseApplicationAction.serializeBinaryToWriter
    );
  }
  f = message.getDeregisterRegularAppAction();
  if (f != null) {
    writer.writeMessage(
      42,
      f,
      proto.ubo.v1.DeregisterRegularAppAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplayAction();
  if (f != null) {
    writer.writeMessage(
      43,
      f,
      proto.ubo.v1.DisplayAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplayBlankAction();
  if (f != null) {
    writer.writeMessage(
      44,
      f,
      proto.ubo.v1.DisplayBlankAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplayPauseAction();
  if (f != null) {
    writer.writeMessage(
      45,
      f,
      proto.ubo.v1.DisplayPauseAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplayRedrawAction();
  if (f != null) {
    writer.writeMessage(
      46,
      f,
      proto.ubo.v1.DisplayRedrawAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplayResumeAction();
  if (f != null) {
    writer.writeMessage(
      47,
      f,
      proto.ubo.v1.DisplayResumeAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplaySetBlankTimeoutAction();
  if (f != null) {
    writer.writeMessage(
      48,
      f,
      proto.ubo.v1.DisplaySetBlankTimeoutAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplayUnblankAction();
  if (f != null) {
    writer.writeMessage(
      49,
      f,
      proto.ubo.v1.DisplayUnblankAction.serializeBinaryToWriter
    );
  }
  f = message.getDisplayUpdateActivityAction();
  if (f != null) {
    writer.writeMessage(
      50,
      f,
      proto.ubo.v1.DisplayUpdateActivityAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerAction();
  if (f != null) {
    writer.writeMessage(
      51,
      f,
      proto.ubo.v1.DockerAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageAction();
  if (f != null) {
    writer.writeMessage(
      52,
      f,
      proto.ubo.v1.DockerImageAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageFetchAction();
  if (f != null) {
    writer.writeMessage(
      53,
      f,
      proto.ubo.v1.DockerImageFetchAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageReleaseAction();
  if (f != null) {
    writer.writeMessage(
      54,
      f,
      proto.ubo.v1.DockerImageReleaseAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRemoveAction();
  if (f != null) {
    writer.writeMessage(
      55,
      f,
      proto.ubo.v1.DockerImageRemoveAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRemoveContainerAction();
  if (f != null) {
    writer.writeMessage(
      56,
      f,
      proto.ubo.v1.DockerImageRemoveContainerAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRunAction();
  if (f != null) {
    writer.writeMessage(
      57,
      f,
      proto.ubo.v1.DockerImageRunAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageSetDockerIdAction();
  if (f != null) {
    writer.writeMessage(
      58,
      f,
      proto.ubo.v1.DockerImageSetDockerIdAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageSetStatusAction();
  if (f != null) {
    writer.writeMessage(
      59,
      f,
      proto.ubo.v1.DockerImageSetStatusAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageStopAction();
  if (f != null) {
    writer.writeMessage(
      60,
      f,
      proto.ubo.v1.DockerImageStopAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageUpdateMetadataAction();
  if (f != null) {
    writer.writeMessage(
      61,
      f,
      proto.ubo.v1.DockerImageUpdateMetadataAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerInstallAction();
  if (f != null) {
    writer.writeMessage(
      62,
      f,
      proto.ubo.v1.DockerInstallAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerRemoveUsernameAction();
  if (f != null) {
    writer.writeMessage(
      63,
      f,
      proto.ubo.v1.DockerRemoveUsernameAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerSetStatusAction();
  if (f != null) {
    writer.writeMessage(
      64,
      f,
      proto.ubo.v1.DockerSetStatusAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerStartAction();
  if (f != null) {
    writer.writeMessage(
      65,
      f,
      proto.ubo.v1.DockerStartAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerStopAction();
  if (f != null) {
    writer.writeMessage(
      66,
      f,
      proto.ubo.v1.DockerStopAction.serializeBinaryToWriter
    );
  }
  f = message.getDockerStoreUsernameAction();
  if (f != null) {
    writer.writeMessage(
      67,
      f,
      proto.ubo.v1.DockerStoreUsernameAction.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemAction();
  if (f != null) {
    writer.writeMessage(
      68,
      f,
      proto.ubo.v1.FileSystemAction.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemCopyAction();
  if (f != null) {
    writer.writeMessage(
      69,
      f,
      proto.ubo.v1.FileSystemCopyAction.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemMoveAction();
  if (f != null) {
    writer.writeMessage(
      70,
      f,
      proto.ubo.v1.FileSystemMoveAction.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemRemoveAction();
  if (f != null) {
    writer.writeMessage(
      71,
      f,
      proto.ubo.v1.FileSystemRemoveAction.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemReportSelectionAction();
  if (f != null) {
    writer.writeMessage(
      72,
      f,
      proto.ubo.v1.FileSystemReportSelectionAction.serializeBinaryToWriter
    );
  }
  f = message.getInfraredAction();
  if (f != null) {
    writer.writeMessage(
      73,
      f,
      proto.ubo.v1.InfraredAction.serializeBinaryToWriter
    );
  }
  f = message.getInfraredHandleReceivedCodeAction();
  if (f != null) {
    writer.writeMessage(
      74,
      f,
      proto.ubo.v1.InfraredHandleReceivedCodeAction.serializeBinaryToWriter
    );
  }
  f = message.getInfraredSendCodeAction();
  if (f != null) {
    writer.writeMessage(
      75,
      f,
      proto.ubo.v1.InfraredSendCodeAction.serializeBinaryToWriter
    );
  }
  f = message.getInfraredSetShouldPropagateAction();
  if (f != null) {
    writer.writeMessage(
      76,
      f,
      proto.ubo.v1.InfraredSetShouldPropagateAction.serializeBinaryToWriter
    );
  }
  f = message.getInfraredSetShouldReceiveAction();
  if (f != null) {
    writer.writeMessage(
      77,
      f,
      proto.ubo.v1.InfraredSetShouldReceiveAction.serializeBinaryToWriter
    );
  }
  f = message.getInputAction();
  if (f != null) {
    writer.writeMessage(
      78,
      f,
      proto.ubo.v1.InputAction.serializeBinaryToWriter
    );
  }
  f = message.getInputCancelAction();
  if (f != null) {
    writer.writeMessage(
      79,
      f,
      proto.ubo.v1.InputCancelAction.serializeBinaryToWriter
    );
  }
  f = message.getInputDemandAction();
  if (f != null) {
    writer.writeMessage(
      80,
      f,
      proto.ubo.v1.InputDemandAction.serializeBinaryToWriter
    );
  }
  f = message.getInputProvideAction();
  if (f != null) {
    writer.writeMessage(
      81,
      f,
      proto.ubo.v1.InputProvideAction.serializeBinaryToWriter
    );
  }
  f = message.getInputResolveAction();
  if (f != null) {
    writer.writeMessage(
      82,
      f,
      proto.ubo.v1.InputResolveAction.serializeBinaryToWriter
    );
  }
  f = message.getIpAction();
  if (f != null) {
    writer.writeMessage(
      83,
      f,
      proto.ubo.v1.IpAction.serializeBinaryToWriter
    );
  }
  f = message.getIpSetIsConnectedAction();
  if (f != null) {
    writer.writeMessage(
      84,
      f,
      proto.ubo.v1.IpSetIsConnectedAction.serializeBinaryToWriter
    );
  }
  f = message.getIpUpdateInterfacesAction();
  if (f != null) {
    writer.writeMessage(
      85,
      f,
      proto.ubo.v1.IpUpdateInterfacesAction.serializeBinaryToWriter
    );
  }
  f = message.getKeypadAction();
  if (f != null) {
    writer.writeMessage(
      86,
      f,
      proto.ubo.v1.KeypadAction.serializeBinaryToWriter
    );
  }
  f = message.getKeypadKeyHoldAction();
  if (f != null) {
    writer.writeMessage(
      87,
      f,
      proto.ubo.v1.KeypadKeyHoldAction.serializeBinaryToWriter
    );
  }
  f = message.getKeypadKeyPressAction();
  if (f != null) {
    writer.writeMessage(
      88,
      f,
      proto.ubo.v1.KeypadKeyPressAction.serializeBinaryToWriter
    );
  }
  f = message.getKeypadKeyReleaseAction();
  if (f != null) {
    writer.writeMessage(
      89,
      f,
      proto.ubo.v1.KeypadKeyReleaseAction.serializeBinaryToWriter
    );
  }
  f = message.getKeypadKeyUnholdAction();
  if (f != null) {
    writer.writeMessage(
      90,
      f,
      proto.ubo.v1.KeypadKeyUnholdAction.serializeBinaryToWriter
    );
  }
  f = message.getLightDmAction();
  if (f != null) {
    writer.writeMessage(
      91,
      f,
      proto.ubo.v1.LightDMAction.serializeBinaryToWriter
    );
  }
  f = message.getLightDmClearEnabledStateAction();
  if (f != null) {
    writer.writeMessage(
      92,
      f,
      proto.ubo.v1.LightDMClearEnabledStateAction.serializeBinaryToWriter
    );
  }
  f = message.getLightDmUpdateStateAction();
  if (f != null) {
    writer.writeMessage(
      93,
      f,
      proto.ubo.v1.LightDMUpdateStateAction.serializeBinaryToWriter
    );
  }
  f = message.getMainAction();
  if (f != null) {
    writer.writeMessage(
      94,
      f,
      proto.ubo.v1.MainAction.serializeBinaryToWriter
    );
  }
  f = message.getMenuAction();
  if (f != null) {
    writer.writeMessage(
      95,
      f,
      proto.ubo.v1.MenuAction.serializeBinaryToWriter
    );
  }
  f = message.getMenuChooseByIconAction();
  if (f != null) {
    writer.writeMessage(
      96,
      f,
      proto.ubo.v1.MenuChooseByIconAction.serializeBinaryToWriter
    );
  }
  f = message.getMenuChooseByIndexAction();
  if (f != null) {
    writer.writeMessage(
      97,
      f,
      proto.ubo.v1.MenuChooseByIndexAction.serializeBinaryToWriter
    );
  }
  f = message.getMenuChooseByLabelAction();
  if (f != null) {
    writer.writeMessage(
      98,
      f,
      proto.ubo.v1.MenuChooseByLabelAction.serializeBinaryToWriter
    );
  }
  f = message.getMenuGoBackAction();
  if (f != null) {
    writer.writeMessage(
      99,
      f,
      proto.ubo.v1.MenuGoBackAction.serializeBinaryToWriter
    );
  }
  f = message.getMenuGoHomeAction();
  if (f != null) {
    writer.writeMessage(
      100,
      f,
      proto.ubo.v1.MenuGoHomeAction.serializeBinaryToWriter
    );
  }
  f = message.getMenuScrollAction();
  if (f != null) {
    writer.writeMessage(
      101,
      f,
      proto.ubo.v1.MenuScrollAction.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsAction();
  if (f != null) {
    writer.writeMessage(
      102,
      f,
      proto.ubo.v1.NotificationsAction.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsAddAction();
  if (f != null) {
    writer.writeMessage(
      103,
      f,
      proto.ubo.v1.NotificationsAddAction.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsClearAction();
  if (f != null) {
    writer.writeMessage(
      104,
      f,
      proto.ubo.v1.NotificationsClearAction.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsClearAllAction();
  if (f != null) {
    writer.writeMessage(
      105,
      f,
      proto.ubo.v1.NotificationsClearAllAction.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsClearByIdAction();
  if (f != null) {
    writer.writeMessage(
      106,
      f,
      proto.ubo.v1.NotificationsClearByIdAction.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsDisplayAction();
  if (f != null) {
    writer.writeMessage(
      107,
      f,
      proto.ubo.v1.NotificationsDisplayAction.serializeBinaryToWriter
    );
  }
  f = message.getOpenApplicationAction();
  if (f != null) {
    writer.writeMessage(
      108,
      f,
      proto.ubo.v1.OpenApplicationAction.serializeBinaryToWriter
    );
  }
  f = message.getPowerAction();
  if (f != null) {
    writer.writeMessage(
      109,
      f,
      proto.ubo.v1.PowerAction.serializeBinaryToWriter
    );
  }
  f = message.getPowerOffAction();
  if (f != null) {
    writer.writeMessage(
      110,
      f,
      proto.ubo.v1.PowerOffAction.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectAction();
  if (f != null) {
    writer.writeMessage(
      111,
      f,
      proto.ubo.v1.RPiConnectAction.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectDoneDownloadingAction();
  if (f != null) {
    writer.writeMessage(
      112,
      f,
      proto.ubo.v1.RPiConnectDoneDownloadingAction.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectSetPendingAction();
  if (f != null) {
    writer.writeMessage(
      113,
      f,
      proto.ubo.v1.RPiConnectSetPendingAction.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectSetStatusAction();
  if (f != null) {
    writer.writeMessage(
      114,
      f,
      proto.ubo.v1.RPiConnectSetStatusAction.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectStartDownloadingAction();
  if (f != null) {
    writer.writeMessage(
      115,
      f,
      proto.ubo.v1.RPiConnectStartDownloadingAction.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectUpdateServiceStateAction();
  if (f != null) {
    writer.writeMessage(
      116,
      f,
      proto.ubo.v1.RPiConnectUpdateServiceStateAction.serializeBinaryToWriter
    );
  }
  f = message.getRebootAction();
  if (f != null) {
    writer.writeMessage(
      117,
      f,
      proto.ubo.v1.RebootAction.serializeBinaryToWriter
    );
  }
  f = message.getRegisterAppAction();
  if (f != null) {
    writer.writeMessage(
      118,
      f,
      proto.ubo.v1.RegisterAppAction.serializeBinaryToWriter
    );
  }
  f = message.getRegisterRegularAppAction();
  if (f != null) {
    writer.writeMessage(
      119,
      f,
      proto.ubo.v1.RegisterRegularAppAction.serializeBinaryToWriter
    );
  }
  f = message.getRegisterSettingAppAction();
  if (f != null) {
    writer.writeMessage(
      120,
      f,
      proto.ubo.v1.RegisterSettingAppAction.serializeBinaryToWriter
    );
  }
  f = message.getReplayRecordedSequenceAction();
  if (f != null) {
    writer.writeMessage(
      121,
      f,
      proto.ubo.v1.ReplayRecordedSequenceAction.serializeBinaryToWriter
    );
  }
  f = message.getReportReplayingDoneAction();
  if (f != null) {
    writer.writeMessage(
      122,
      f,
      proto.ubo.v1.ReportReplayingDoneAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingAction();
  if (f != null) {
    writer.writeMessage(
      123,
      f,
      proto.ubo.v1.RgbRingAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingBlankAction();
  if (f != null) {
    writer.writeMessage(
      124,
      f,
      proto.ubo.v1.RgbRingBlankAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingBlinkAction();
  if (f != null) {
    writer.writeMessage(
      125,
      f,
      proto.ubo.v1.RgbRingBlinkAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingColorfulCommandAction();
  if (f != null) {
    writer.writeMessage(
      126,
      f,
      proto.ubo.v1.RgbRingColorfulCommandAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingCommandAction();
  if (f != null) {
    writer.writeMessage(
      127,
      f,
      proto.ubo.v1.RgbRingCommandAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingFillDownfromAction();
  if (f != null) {
    writer.writeMessage(
      128,
      f,
      proto.ubo.v1.RgbRingFillDownfromAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingFillUptoAction();
  if (f != null) {
    writer.writeMessage(
      129,
      f,
      proto.ubo.v1.RgbRingFillUptoAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingProgressWheelAction();
  if (f != null) {
    writer.writeMessage(
      130,
      f,
      proto.ubo.v1.RgbRingProgressWheelAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingProgressWheelStepAction();
  if (f != null) {
    writer.writeMessage(
      131,
      f,
      proto.ubo.v1.RgbRingProgressWheelStepAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingPulseAction();
  if (f != null) {
    writer.writeMessage(
      132,
      f,
      proto.ubo.v1.RgbRingPulseAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingRainbowAction();
  if (f != null) {
    writer.writeMessage(
      133,
      f,
      proto.ubo.v1.RgbRingRainbowAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingSequenceAction();
  if (f != null) {
    writer.writeMessage(
      134,
      f,
      proto.ubo.v1.RgbRingSequenceAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingSetAllAction();
  if (f != null) {
    writer.writeMessage(
      135,
      f,
      proto.ubo.v1.RgbRingSetAllAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingSetBrightnessAction();
  if (f != null) {
    writer.writeMessage(
      136,
      f,
      proto.ubo.v1.RgbRingSetBrightnessAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingSetEnabledAction();
  if (f != null) {
    writer.writeMessage(
      137,
      f,
      proto.ubo.v1.RgbRingSetEnabledAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingSetIsBusyAction();
  if (f != null) {
    writer.writeMessage(
      138,
      f,
      proto.ubo.v1.RgbRingSetIsBusyAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingSpinningWheelAction();
  if (f != null) {
    writer.writeMessage(
      139,
      f,
      proto.ubo.v1.RgbRingSpinningWheelAction.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingWaitableCommandAction();
  if (f != null) {
    writer.writeMessage(
      140,
      f,
      proto.ubo.v1.RgbRingWaitableCommandAction.serializeBinaryToWriter
    );
  }
  f = message.getSshAction();
  if (f != null) {
    writer.writeMessage(
      141,
      f,
      proto.ubo.v1.SSHAction.serializeBinaryToWriter
    );
  }
  f = message.getSshClearEnabledStateAction();
  if (f != null) {
    writer.writeMessage(
      142,
      f,
      proto.ubo.v1.SSHClearEnabledStateAction.serializeBinaryToWriter
    );
  }
  f = message.getSshUpdateStateAction();
  if (f != null) {
    writer.writeMessage(
      143,
      f,
      proto.ubo.v1.SSHUpdateStateAction.serializeBinaryToWriter
    );
  }
  f = message.getSensorsAction();
  if (f != null) {
    writer.writeMessage(
      144,
      f,
      proto.ubo.v1.SensorsAction.serializeBinaryToWriter
    );
  }
  f = message.getSensorsReportReadingAction();
  if (f != null) {
    writer.writeMessage(
      145,
      f,
      proto.ubo.v1.SensorsReportReadingAction.serializeBinaryToWriter
    );
  }
  f = message.getSetAreEnclosuresVisibleAction();
  if (f != null) {
    writer.writeMessage(
      146,
      f,
      proto.ubo.v1.SetAreEnclosuresVisibleAction.serializeBinaryToWriter
    );
  }
  f = message.getSetMenuPathAction();
  if (f != null) {
    writer.writeMessage(
      147,
      f,
      proto.ubo.v1.SetMenuPathAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsAction();
  if (f != null) {
    writer.writeMessage(
      148,
      f,
      proto.ubo.v1.SettingsAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsClearServiceErrorsAction();
  if (f != null) {
    writer.writeMessage(
      149,
      f,
      proto.ubo.v1.SettingsClearServiceErrorsAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsReportServiceErrorAction();
  if (f != null) {
    writer.writeMessage(
      150,
      f,
      proto.ubo.v1.SettingsReportServiceErrorAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsServiceAction();
  if (f != null) {
    writer.writeMessage(
      151,
      f,
      proto.ubo.v1.SettingsServiceAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsServiceSetIsEnabledAction();
  if (f != null) {
    writer.writeMessage(
      152,
      f,
      proto.ubo.v1.SettingsServiceSetIsEnabledAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsServiceSetLogLevelAction();
  if (f != null) {
    writer.writeMessage(
      153,
      f,
      proto.ubo.v1.SettingsServiceSetLogLevelAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsServiceSetShouldRestartAction();
  if (f != null) {
    writer.writeMessage(
      154,
      f,
      proto.ubo.v1.SettingsServiceSetShouldRestartAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsServiceSetStatusAction();
  if (f != null) {
    writer.writeMessage(
      155,
      f,
      proto.ubo.v1.SettingsServiceSetStatusAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsSetServicesAction();
  if (f != null) {
    writer.writeMessage(
      156,
      f,
      proto.ubo.v1.SettingsSetServicesAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsStartServiceAction();
  if (f != null) {
    writer.writeMessage(
      157,
      f,
      proto.ubo.v1.SettingsStartServiceAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsStopServiceAction();
  if (f != null) {
    writer.writeMessage(
      158,
      f,
      proto.ubo.v1.SettingsStopServiceAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsToggleBetaVersionsAction();
  if (f != null) {
    writer.writeMessage(
      159,
      f,
      proto.ubo.v1.SettingsToggleBetaVersionsAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsTogglePdbSignalAction();
  if (f != null) {
    writer.writeMessage(
      160,
      f,
      proto.ubo.v1.SettingsTogglePdbSignalAction.serializeBinaryToWriter
    );
  }
  f = message.getSettingsToggleVisualDebugAction();
  if (f != null) {
    writer.writeMessage(
      161,
      f,
      proto.ubo.v1.SettingsToggleVisualDebugAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionAction();
  if (f != null) {
    writer.writeMessage(
      162,
      f,
      proto.ubo.v1.SpeechRecognitionAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionReportIntentDetectionAction();
  if (f != null) {
    writer.writeMessage(
      163,
      f,
      proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionReportSpeechAction();
  if (f != null) {
    writer.writeMessage(
      164,
      f,
      proto.ubo.v1.SpeechRecognitionReportSpeechAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionReportWakeWordDetectionAction();
  if (f != null) {
    writer.writeMessage(
      165,
      f,
      proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionSetIsAssistantActiveAction();
  if (f != null) {
    writer.writeMessage(
      166,
      f,
      proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionSetIsIntentsActiveAction();
  if (f != null) {
    writer.writeMessage(
      167,
      f,
      proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionSetSelectedEngineAction();
  if (f != null) {
    writer.writeMessage(
      168,
      f,
      proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechSynthesisAction();
  if (f != null) {
    writer.writeMessage(
      169,
      f,
      proto.ubo.v1.SpeechSynthesisAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechSynthesisReadTextAction();
  if (f != null) {
    writer.writeMessage(
      170,
      f,
      proto.ubo.v1.SpeechSynthesisReadTextAction.serializeBinaryToWriter
    );
  }
  f = message.getSpeechSynthesisSetSelectedEngineAction();
  if (f != null) {
    writer.writeMessage(
      171,
      f,
      proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction.serializeBinaryToWriter
    );
  }
  f = message.getStatusIconsAction();
  if (f != null) {
    writer.writeMessage(
      172,
      f,
      proto.ubo.v1.StatusIconsAction.serializeBinaryToWriter
    );
  }
  f = message.getStatusIconsRegisterAction();
  if (f != null) {
    writer.writeMessage(
      173,
      f,
      proto.ubo.v1.StatusIconsRegisterAction.serializeBinaryToWriter
    );
  }
  f = message.getToggleRecordingAction();
  if (f != null) {
    writer.writeMessage(
      174,
      f,
      proto.ubo.v1.ToggleRecordingAction.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerAction();
  if (f != null) {
    writer.writeMessage(
      175,
      f,
      proto.ubo.v1.UpdateManagerAction.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerReportFailedCheckAction();
  if (f != null) {
    writer.writeMessage(
      176,
      f,
      proto.ubo.v1.UpdateManagerReportFailedCheckAction.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerRequestCheckAction();
  if (f != null) {
    writer.writeMessage(
      177,
      f,
      proto.ubo.v1.UpdateManagerRequestCheckAction.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerRequestUpdateAction();
  if (f != null) {
    writer.writeMessage(
      178,
      f,
      proto.ubo.v1.UpdateManagerRequestUpdateAction.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerSetVersionsAction();
  if (f != null) {
    writer.writeMessage(
      179,
      f,
      proto.ubo.v1.UpdateManagerSetVersionsAction.serializeBinaryToWriter
    );
  }
  f = message.getUsersAction();
  if (f != null) {
    writer.writeMessage(
      180,
      f,
      proto.ubo.v1.UsersAction.serializeBinaryToWriter
    );
  }
  f = message.getUsersCreateUserAction();
  if (f != null) {
    writer.writeMessage(
      181,
      f,
      proto.ubo.v1.UsersCreateUserAction.serializeBinaryToWriter
    );
  }
  f = message.getUsersDeleteUserAction();
  if (f != null) {
    writer.writeMessage(
      182,
      f,
      proto.ubo.v1.UsersDeleteUserAction.serializeBinaryToWriter
    );
  }
  f = message.getUsersResetPasswordAction();
  if (f != null) {
    writer.writeMessage(
      183,
      f,
      proto.ubo.v1.UsersResetPasswordAction.serializeBinaryToWriter
    );
  }
  f = message.getUsersSetUsersAction();
  if (f != null) {
    writer.writeMessage(
      184,
      f,
      proto.ubo.v1.UsersSetUsersAction.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeAction();
  if (f != null) {
    writer.writeMessage(
      185,
      f,
      proto.ubo.v1.VSCodeAction.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeDoneDownloadingAction();
  if (f != null) {
    writer.writeMessage(
      186,
      f,
      proto.ubo.v1.VSCodeDoneDownloadingAction.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeSetPendingAction();
  if (f != null) {
    writer.writeMessage(
      187,
      f,
      proto.ubo.v1.VSCodeSetPendingAction.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeSetStatusAction();
  if (f != null) {
    writer.writeMessage(
      188,
      f,
      proto.ubo.v1.VSCodeSetStatusAction.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeStartDownloadingAction();
  if (f != null) {
    writer.writeMessage(
      189,
      f,
      proto.ubo.v1.VSCodeStartDownloadingAction.serializeBinaryToWriter
    );
  }
  f = message.getWiFiAction();
  if (f != null) {
    writer.writeMessage(
      190,
      f,
      proto.ubo.v1.WiFiAction.serializeBinaryToWriter
    );
  }
  f = message.getWiFiInputConnectionAction();
  if (f != null) {
    writer.writeMessage(
      191,
      f,
      proto.ubo.v1.WiFiInputConnectionAction.serializeBinaryToWriter
    );
  }
  f = message.getWiFiSetHasVisitedOnboardingAction();
  if (f != null) {
    writer.writeMessage(
      192,
      f,
      proto.ubo.v1.WiFiSetHasVisitedOnboardingAction.serializeBinaryToWriter
    );
  }
  f = message.getWiFiUpdateAction();
  if (f != null) {
    writer.writeMessage(
      193,
      f,
      proto.ubo.v1.WiFiUpdateAction.serializeBinaryToWriter
    );
  }
  f = message.getWiFiUpdateRequestAction();
  if (f != null) {
    writer.writeMessage(
      194,
      f,
      proto.ubo.v1.WiFiUpdateRequestAction.serializeBinaryToWriter
    );
  }
};


/**
 * optional AssistantAction assistant_action = 1;
 * @return {?proto.ubo.v1.AssistantAction}
 */
proto.ubo.v1.Action.prototype.getAssistantAction = function() {
  return /** @type{?proto.ubo.v1.AssistantAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantAction, 1));
};


/**
 * @param {?proto.ubo.v1.AssistantAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantAction = function() {
  return this.setAssistantAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantAction = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional AssistantAddMcpServerAction assistant_add_mcp_server_action = 2;
 * @return {?proto.ubo.v1.AssistantAddMcpServerAction}
 */
proto.ubo.v1.Action.prototype.getAssistantAddMcpServerAction = function() {
  return /** @type{?proto.ubo.v1.AssistantAddMcpServerAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantAddMcpServerAction, 2));
};


/**
 * @param {?proto.ubo.v1.AssistantAddMcpServerAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantAddMcpServerAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantAddMcpServerAction = function() {
  return this.setAssistantAddMcpServerAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantAddMcpServerAction = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional AssistantDeleteMcpServerAction assistant_delete_mcp_server_action = 3;
 * @return {?proto.ubo.v1.AssistantDeleteMcpServerAction}
 */
proto.ubo.v1.Action.prototype.getAssistantDeleteMcpServerAction = function() {
  return /** @type{?proto.ubo.v1.AssistantDeleteMcpServerAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDeleteMcpServerAction, 3));
};


/**
 * @param {?proto.ubo.v1.AssistantDeleteMcpServerAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantDeleteMcpServerAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantDeleteMcpServerAction = function() {
  return this.setAssistantDeleteMcpServerAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantDeleteMcpServerAction = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional AssistantDownloadOllamaModelAction assistant_download_ollama_model_action = 4;
 * @return {?proto.ubo.v1.AssistantDownloadOllamaModelAction}
 */
proto.ubo.v1.Action.prototype.getAssistantDownloadOllamaModelAction = function() {
  return /** @type{?proto.ubo.v1.AssistantDownloadOllamaModelAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDownloadOllamaModelAction, 4));
};


/**
 * @param {?proto.ubo.v1.AssistantDownloadOllamaModelAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantDownloadOllamaModelAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 4, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantDownloadOllamaModelAction = function() {
  return this.setAssistantDownloadOllamaModelAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantDownloadOllamaModelAction = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional AssistantReportAction assistant_report_action = 5;
 * @return {?proto.ubo.v1.AssistantReportAction}
 */
proto.ubo.v1.Action.prototype.getAssistantReportAction = function() {
  return /** @type{?proto.ubo.v1.AssistantReportAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantReportAction, 5));
};


/**
 * @param {?proto.ubo.v1.AssistantReportAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantReportAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 5, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantReportAction = function() {
  return this.setAssistantReportAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantReportAction = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional AssistantSetIsActiveAction assistant_set_is_active_action = 6;
 * @return {?proto.ubo.v1.AssistantSetIsActiveAction}
 */
proto.ubo.v1.Action.prototype.getAssistantSetIsActiveAction = function() {
  return /** @type{?proto.ubo.v1.AssistantSetIsActiveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetIsActiveAction, 6));
};


/**
 * @param {?proto.ubo.v1.AssistantSetIsActiveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantSetIsActiveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 6, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantSetIsActiveAction = function() {
  return this.setAssistantSetIsActiveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantSetIsActiveAction = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional AssistantSetSelectedImageGeneratorAction assistant_set_selected_image_generator_action = 7;
 * @return {?proto.ubo.v1.AssistantSetSelectedImageGeneratorAction}
 */
proto.ubo.v1.Action.prototype.getAssistantSetSelectedImageGeneratorAction = function() {
  return /** @type{?proto.ubo.v1.AssistantSetSelectedImageGeneratorAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedImageGeneratorAction, 7));
};


/**
 * @param {?proto.ubo.v1.AssistantSetSelectedImageGeneratorAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantSetSelectedImageGeneratorAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 7, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantSetSelectedImageGeneratorAction = function() {
  return this.setAssistantSetSelectedImageGeneratorAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantSetSelectedImageGeneratorAction = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional AssistantSetSelectedLLMAction assistant_set_selected_llm_action = 8;
 * @return {?proto.ubo.v1.AssistantSetSelectedLLMAction}
 */
proto.ubo.v1.Action.prototype.getAssistantSetSelectedLlmAction = function() {
  return /** @type{?proto.ubo.v1.AssistantSetSelectedLLMAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedLLMAction, 8));
};


/**
 * @param {?proto.ubo.v1.AssistantSetSelectedLLMAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantSetSelectedLlmAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 8, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantSetSelectedLlmAction = function() {
  return this.setAssistantSetSelectedLlmAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantSetSelectedLlmAction = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional AssistantSetSelectedModelAction assistant_set_selected_model_action = 9;
 * @return {?proto.ubo.v1.AssistantSetSelectedModelAction}
 */
proto.ubo.v1.Action.prototype.getAssistantSetSelectedModelAction = function() {
  return /** @type{?proto.ubo.v1.AssistantSetSelectedModelAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedModelAction, 9));
};


/**
 * @param {?proto.ubo.v1.AssistantSetSelectedModelAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantSetSelectedModelAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 9, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantSetSelectedModelAction = function() {
  return this.setAssistantSetSelectedModelAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantSetSelectedModelAction = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional AssistantSetSelectedSTTAction assistant_set_selected_stt_action = 10;
 * @return {?proto.ubo.v1.AssistantSetSelectedSTTAction}
 */
proto.ubo.v1.Action.prototype.getAssistantSetSelectedSttAction = function() {
  return /** @type{?proto.ubo.v1.AssistantSetSelectedSTTAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedSTTAction, 10));
};


/**
 * @param {?proto.ubo.v1.AssistantSetSelectedSTTAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantSetSelectedSttAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 10, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantSetSelectedSttAction = function() {
  return this.setAssistantSetSelectedSttAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantSetSelectedSttAction = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional AssistantSetSelectedTTSAction assistant_set_selected_tts_action = 11;
 * @return {?proto.ubo.v1.AssistantSetSelectedTTSAction}
 */
proto.ubo.v1.Action.prototype.getAssistantSetSelectedTtsAction = function() {
  return /** @type{?proto.ubo.v1.AssistantSetSelectedTTSAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSetSelectedTTSAction, 11));
};


/**
 * @param {?proto.ubo.v1.AssistantSetSelectedTTSAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantSetSelectedTtsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 11, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantSetSelectedTtsAction = function() {
  return this.setAssistantSetSelectedTtsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantSetSelectedTtsAction = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional AssistantStartListeningAction assistant_start_listening_action = 12;
 * @return {?proto.ubo.v1.AssistantStartListeningAction}
 */
proto.ubo.v1.Action.prototype.getAssistantStartListeningAction = function() {
  return /** @type{?proto.ubo.v1.AssistantStartListeningAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantStartListeningAction, 12));
};


/**
 * @param {?proto.ubo.v1.AssistantStartListeningAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantStartListeningAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 12, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantStartListeningAction = function() {
  return this.setAssistantStartListeningAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantStartListeningAction = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional AssistantStopListeningAction assistant_stop_listening_action = 13;
 * @return {?proto.ubo.v1.AssistantStopListeningAction}
 */
proto.ubo.v1.Action.prototype.getAssistantStopListeningAction = function() {
  return /** @type{?proto.ubo.v1.AssistantStopListeningAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantStopListeningAction, 13));
};


/**
 * @param {?proto.ubo.v1.AssistantStopListeningAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantStopListeningAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 13, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantStopListeningAction = function() {
  return this.setAssistantStopListeningAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantStopListeningAction = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional AssistantSyncMcpServersAction assistant_sync_mcp_servers_action = 14;
 * @return {?proto.ubo.v1.AssistantSyncMcpServersAction}
 */
proto.ubo.v1.Action.prototype.getAssistantSyncMcpServersAction = function() {
  return /** @type{?proto.ubo.v1.AssistantSyncMcpServersAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantSyncMcpServersAction, 14));
};


/**
 * @param {?proto.ubo.v1.AssistantSyncMcpServersAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantSyncMcpServersAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 14, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantSyncMcpServersAction = function() {
  return this.setAssistantSyncMcpServersAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantSyncMcpServersAction = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional AssistantToggleListeningAction assistant_toggle_listening_action = 15;
 * @return {?proto.ubo.v1.AssistantToggleListeningAction}
 */
proto.ubo.v1.Action.prototype.getAssistantToggleListeningAction = function() {
  return /** @type{?proto.ubo.v1.AssistantToggleListeningAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantToggleListeningAction, 15));
};


/**
 * @param {?proto.ubo.v1.AssistantToggleListeningAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantToggleListeningAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 15, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantToggleListeningAction = function() {
  return this.setAssistantToggleListeningAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantToggleListeningAction = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional AssistantToggleMcpServerAction assistant_toggle_mcp_server_action = 16;
 * @return {?proto.ubo.v1.AssistantToggleMcpServerAction}
 */
proto.ubo.v1.Action.prototype.getAssistantToggleMcpServerAction = function() {
  return /** @type{?proto.ubo.v1.AssistantToggleMcpServerAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantToggleMcpServerAction, 16));
};


/**
 * @param {?proto.ubo.v1.AssistantToggleMcpServerAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantToggleMcpServerAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 16, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantToggleMcpServerAction = function() {
  return this.setAssistantToggleMcpServerAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantToggleMcpServerAction = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional AssistantUpdateProvidersAction assistant_update_providers_action = 17;
 * @return {?proto.ubo.v1.AssistantUpdateProvidersAction}
 */
proto.ubo.v1.Action.prototype.getAssistantUpdateProvidersAction = function() {
  return /** @type{?proto.ubo.v1.AssistantUpdateProvidersAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantUpdateProvidersAction, 17));
};


/**
 * @param {?proto.ubo.v1.AssistantUpdateProvidersAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAssistantUpdateProvidersAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 17, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAssistantUpdateProvidersAction = function() {
  return this.setAssistantUpdateProvidersAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAssistantUpdateProvidersAction = function() {
  return jspb.Message.getField(this, 17) != null;
};


/**
 * optional AudioAction audio_action = 18;
 * @return {?proto.ubo.v1.AudioAction}
 */
proto.ubo.v1.Action.prototype.getAudioAction = function() {
  return /** @type{?proto.ubo.v1.AudioAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioAction, 18));
};


/**
 * @param {?proto.ubo.v1.AudioAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 18, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioAction = function() {
  return this.setAudioAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioAction = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional AudioChangeVolumeAction audio_change_volume_action = 19;
 * @return {?proto.ubo.v1.AudioChangeVolumeAction}
 */
proto.ubo.v1.Action.prototype.getAudioChangeVolumeAction = function() {
  return /** @type{?proto.ubo.v1.AudioChangeVolumeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioChangeVolumeAction, 19));
};


/**
 * @param {?proto.ubo.v1.AudioChangeVolumeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioChangeVolumeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 19, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioChangeVolumeAction = function() {
  return this.setAudioChangeVolumeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioChangeVolumeAction = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * optional AudioInstallDriverAction audio_install_driver_action = 20;
 * @return {?proto.ubo.v1.AudioInstallDriverAction}
 */
proto.ubo.v1.Action.prototype.getAudioInstallDriverAction = function() {
  return /** @type{?proto.ubo.v1.AudioInstallDriverAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioInstallDriverAction, 20));
};


/**
 * @param {?proto.ubo.v1.AudioInstallDriverAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioInstallDriverAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 20, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioInstallDriverAction = function() {
  return this.setAudioInstallDriverAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioInstallDriverAction = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * optional AudioPlayAudioSampleAction audio_play_audio_sample_action = 21;
 * @return {?proto.ubo.v1.AudioPlayAudioSampleAction}
 */
proto.ubo.v1.Action.prototype.getAudioPlayAudioSampleAction = function() {
  return /** @type{?proto.ubo.v1.AudioPlayAudioSampleAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSampleAction, 21));
};


/**
 * @param {?proto.ubo.v1.AudioPlayAudioSampleAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioPlayAudioSampleAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 21, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioPlayAudioSampleAction = function() {
  return this.setAudioPlayAudioSampleAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioPlayAudioSampleAction = function() {
  return jspb.Message.getField(this, 21) != null;
};


/**
 * optional AudioPlayAudioSequenceAction audio_play_audio_sequence_action = 22;
 * @return {?proto.ubo.v1.AudioPlayAudioSequenceAction}
 */
proto.ubo.v1.Action.prototype.getAudioPlayAudioSequenceAction = function() {
  return /** @type{?proto.ubo.v1.AudioPlayAudioSequenceAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSequenceAction, 22));
};


/**
 * @param {?proto.ubo.v1.AudioPlayAudioSequenceAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioPlayAudioSequenceAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 22, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioPlayAudioSequenceAction = function() {
  return this.setAudioPlayAudioSequenceAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioPlayAudioSequenceAction = function() {
  return jspb.Message.getField(this, 22) != null;
};


/**
 * optional AudioPlayChimeAction audio_play_chime_action = 23;
 * @return {?proto.ubo.v1.AudioPlayChimeAction}
 */
proto.ubo.v1.Action.prototype.getAudioPlayChimeAction = function() {
  return /** @type{?proto.ubo.v1.AudioPlayChimeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayChimeAction, 23));
};


/**
 * @param {?proto.ubo.v1.AudioPlayChimeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioPlayChimeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 23, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioPlayChimeAction = function() {
  return this.setAudioPlayChimeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioPlayChimeAction = function() {
  return jspb.Message.getField(this, 23) != null;
};


/**
 * optional AudioPlayRecordingAction audio_play_recording_action = 24;
 * @return {?proto.ubo.v1.AudioPlayRecordingAction}
 */
proto.ubo.v1.Action.prototype.getAudioPlayRecordingAction = function() {
  return /** @type{?proto.ubo.v1.AudioPlayRecordingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayRecordingAction, 24));
};


/**
 * @param {?proto.ubo.v1.AudioPlayRecordingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioPlayRecordingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 24, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioPlayRecordingAction = function() {
  return this.setAudioPlayRecordingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioPlayRecordingAction = function() {
  return jspb.Message.getField(this, 24) != null;
};


/**
 * optional AudioPlaybackDoneAction audio_playback_done_action = 25;
 * @return {?proto.ubo.v1.AudioPlaybackDoneAction}
 */
proto.ubo.v1.Action.prototype.getAudioPlaybackDoneAction = function() {
  return /** @type{?proto.ubo.v1.AudioPlaybackDoneAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlaybackDoneAction, 25));
};


/**
 * @param {?proto.ubo.v1.AudioPlaybackDoneAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioPlaybackDoneAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 25, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioPlaybackDoneAction = function() {
  return this.setAudioPlaybackDoneAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioPlaybackDoneAction = function() {
  return jspb.Message.getField(this, 25) != null;
};


/**
 * optional AudioReportSampleAction audio_report_sample_action = 26;
 * @return {?proto.ubo.v1.AudioReportSampleAction}
 */
proto.ubo.v1.Action.prototype.getAudioReportSampleAction = function() {
  return /** @type{?proto.ubo.v1.AudioReportSampleAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioReportSampleAction, 26));
};


/**
 * @param {?proto.ubo.v1.AudioReportSampleAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioReportSampleAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 26, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioReportSampleAction = function() {
  return this.setAudioReportSampleAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioReportSampleAction = function() {
  return jspb.Message.getField(this, 26) != null;
};


/**
 * optional AudioSetMuteStatusAction audio_set_mute_status_action = 27;
 * @return {?proto.ubo.v1.AudioSetMuteStatusAction}
 */
proto.ubo.v1.Action.prototype.getAudioSetMuteStatusAction = function() {
  return /** @type{?proto.ubo.v1.AudioSetMuteStatusAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSetMuteStatusAction, 27));
};


/**
 * @param {?proto.ubo.v1.AudioSetMuteStatusAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioSetMuteStatusAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 27, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioSetMuteStatusAction = function() {
  return this.setAudioSetMuteStatusAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioSetMuteStatusAction = function() {
  return jspb.Message.getField(this, 27) != null;
};


/**
 * optional AudioSetVolumeAction audio_set_volume_action = 28;
 * @return {?proto.ubo.v1.AudioSetVolumeAction}
 */
proto.ubo.v1.Action.prototype.getAudioSetVolumeAction = function() {
  return /** @type{?proto.ubo.v1.AudioSetVolumeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioSetVolumeAction, 28));
};


/**
 * @param {?proto.ubo.v1.AudioSetVolumeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioSetVolumeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 28, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioSetVolumeAction = function() {
  return this.setAudioSetVolumeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioSetVolumeAction = function() {
  return jspb.Message.getField(this, 28) != null;
};


/**
 * optional AudioStartRecordingAction audio_start_recording_action = 29;
 * @return {?proto.ubo.v1.AudioStartRecordingAction}
 */
proto.ubo.v1.Action.prototype.getAudioStartRecordingAction = function() {
  return /** @type{?proto.ubo.v1.AudioStartRecordingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioStartRecordingAction, 29));
};


/**
 * @param {?proto.ubo.v1.AudioStartRecordingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioStartRecordingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 29, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioStartRecordingAction = function() {
  return this.setAudioStartRecordingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioStartRecordingAction = function() {
  return jspb.Message.getField(this, 29) != null;
};


/**
 * optional AudioStopRecordingAction audio_stop_recording_action = 30;
 * @return {?proto.ubo.v1.AudioStopRecordingAction}
 */
proto.ubo.v1.Action.prototype.getAudioStopRecordingAction = function() {
  return /** @type{?proto.ubo.v1.AudioStopRecordingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioStopRecordingAction, 30));
};


/**
 * @param {?proto.ubo.v1.AudioStopRecordingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioStopRecordingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 30, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioStopRecordingAction = function() {
  return this.setAudioStopRecordingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioStopRecordingAction = function() {
  return jspb.Message.getField(this, 30) != null;
};


/**
 * optional AudioToggleMuteStatusAction audio_toggle_mute_status_action = 31;
 * @return {?proto.ubo.v1.AudioToggleMuteStatusAction}
 */
proto.ubo.v1.Action.prototype.getAudioToggleMuteStatusAction = function() {
  return /** @type{?proto.ubo.v1.AudioToggleMuteStatusAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioToggleMuteStatusAction, 31));
};


/**
 * @param {?proto.ubo.v1.AudioToggleMuteStatusAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioToggleMuteStatusAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 31, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioToggleMuteStatusAction = function() {
  return this.setAudioToggleMuteStatusAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioToggleMuteStatusAction = function() {
  return jspb.Message.getField(this, 31) != null;
};


/**
 * optional AudioToggleRecordingAction audio_toggle_recording_action = 32;
 * @return {?proto.ubo.v1.AudioToggleRecordingAction}
 */
proto.ubo.v1.Action.prototype.getAudioToggleRecordingAction = function() {
  return /** @type{?proto.ubo.v1.AudioToggleRecordingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioToggleRecordingAction, 32));
};


/**
 * @param {?proto.ubo.v1.AudioToggleRecordingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setAudioToggleRecordingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 32, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearAudioToggleRecordingAction = function() {
  return this.setAudioToggleRecordingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasAudioToggleRecordingAction = function() {
  return jspb.Message.getField(this, 32) != null;
};


/**
 * optional CameraAction camera_action = 33;
 * @return {?proto.ubo.v1.CameraAction}
 */
proto.ubo.v1.Action.prototype.getCameraAction = function() {
  return /** @type{?proto.ubo.v1.CameraAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraAction, 33));
};


/**
 * @param {?proto.ubo.v1.CameraAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 33, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraAction = function() {
  return this.setCameraAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraAction = function() {
  return jspb.Message.getField(this, 33) != null;
};


/**
 * optional CameraDetectAction camera_detect_action = 34;
 * @return {?proto.ubo.v1.CameraDetectAction}
 */
proto.ubo.v1.Action.prototype.getCameraDetectAction = function() {
  return /** @type{?proto.ubo.v1.CameraDetectAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraDetectAction, 34));
};


/**
 * @param {?proto.ubo.v1.CameraDetectAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraDetectAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 34, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraDetectAction = function() {
  return this.setCameraDetectAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraDetectAction = function() {
  return jspb.Message.getField(this, 34) != null;
};


/**
 * optional CameraInstallDriverAction camera_install_driver_action = 35;
 * @return {?proto.ubo.v1.CameraInstallDriverAction}
 */
proto.ubo.v1.Action.prototype.getCameraInstallDriverAction = function() {
  return /** @type{?proto.ubo.v1.CameraInstallDriverAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraInstallDriverAction, 35));
};


/**
 * @param {?proto.ubo.v1.CameraInstallDriverAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraInstallDriverAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 35, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraInstallDriverAction = function() {
  return this.setCameraInstallDriverAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraInstallDriverAction = function() {
  return jspb.Message.getField(this, 35) != null;
};


/**
 * optional CameraReportBarcodeAction camera_report_barcode_action = 36;
 * @return {?proto.ubo.v1.CameraReportBarcodeAction}
 */
proto.ubo.v1.Action.prototype.getCameraReportBarcodeAction = function() {
  return /** @type{?proto.ubo.v1.CameraReportBarcodeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraReportBarcodeAction, 36));
};


/**
 * @param {?proto.ubo.v1.CameraReportBarcodeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraReportBarcodeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 36, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraReportBarcodeAction = function() {
  return this.setCameraReportBarcodeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraReportBarcodeAction = function() {
  return jspb.Message.getField(this, 36) != null;
};


/**
 * optional CameraRestoreDefaultAction camera_restore_default_action = 37;
 * @return {?proto.ubo.v1.CameraRestoreDefaultAction}
 */
proto.ubo.v1.Action.prototype.getCameraRestoreDefaultAction = function() {
  return /** @type{?proto.ubo.v1.CameraRestoreDefaultAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraRestoreDefaultAction, 37));
};


/**
 * @param {?proto.ubo.v1.CameraRestoreDefaultAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraRestoreDefaultAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 37, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraRestoreDefaultAction = function() {
  return this.setCameraRestoreDefaultAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraRestoreDefaultAction = function() {
  return jspb.Message.getField(this, 37) != null;
};


/**
 * optional CameraSetAvailableCamerasAction camera_set_available_cameras_action = 38;
 * @return {?proto.ubo.v1.CameraSetAvailableCamerasAction}
 */
proto.ubo.v1.Action.prototype.getCameraSetAvailableCamerasAction = function() {
  return /** @type{?proto.ubo.v1.CameraSetAvailableCamerasAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraSetAvailableCamerasAction, 38));
};


/**
 * @param {?proto.ubo.v1.CameraSetAvailableCamerasAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraSetAvailableCamerasAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 38, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraSetAvailableCamerasAction = function() {
  return this.setCameraSetAvailableCamerasAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraSetAvailableCamerasAction = function() {
  return jspb.Message.getField(this, 38) != null;
};


/**
 * optional CameraSetIndexAction camera_set_index_action = 39;
 * @return {?proto.ubo.v1.CameraSetIndexAction}
 */
proto.ubo.v1.Action.prototype.getCameraSetIndexAction = function() {
  return /** @type{?proto.ubo.v1.CameraSetIndexAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraSetIndexAction, 39));
};


/**
 * @param {?proto.ubo.v1.CameraSetIndexAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraSetIndexAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 39, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraSetIndexAction = function() {
  return this.setCameraSetIndexAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraSetIndexAction = function() {
  return jspb.Message.getField(this, 39) != null;
};


/**
 * optional CameraStartViewfinderAction camera_start_viewfinder_action = 40;
 * @return {?proto.ubo.v1.CameraStartViewfinderAction}
 */
proto.ubo.v1.Action.prototype.getCameraStartViewfinderAction = function() {
  return /** @type{?proto.ubo.v1.CameraStartViewfinderAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraStartViewfinderAction, 40));
};


/**
 * @param {?proto.ubo.v1.CameraStartViewfinderAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCameraStartViewfinderAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 40, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCameraStartViewfinderAction = function() {
  return this.setCameraStartViewfinderAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCameraStartViewfinderAction = function() {
  return jspb.Message.getField(this, 40) != null;
};


/**
 * optional CloseApplicationAction close_application_action = 41;
 * @return {?proto.ubo.v1.CloseApplicationAction}
 */
proto.ubo.v1.Action.prototype.getCloseApplicationAction = function() {
  return /** @type{?proto.ubo.v1.CloseApplicationAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CloseApplicationAction, 41));
};


/**
 * @param {?proto.ubo.v1.CloseApplicationAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setCloseApplicationAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 41, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearCloseApplicationAction = function() {
  return this.setCloseApplicationAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasCloseApplicationAction = function() {
  return jspb.Message.getField(this, 41) != null;
};


/**
 * optional DeregisterRegularAppAction deregister_regular_app_action = 42;
 * @return {?proto.ubo.v1.DeregisterRegularAppAction}
 */
proto.ubo.v1.Action.prototype.getDeregisterRegularAppAction = function() {
  return /** @type{?proto.ubo.v1.DeregisterRegularAppAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DeregisterRegularAppAction, 42));
};


/**
 * @param {?proto.ubo.v1.DeregisterRegularAppAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDeregisterRegularAppAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 42, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDeregisterRegularAppAction = function() {
  return this.setDeregisterRegularAppAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDeregisterRegularAppAction = function() {
  return jspb.Message.getField(this, 42) != null;
};


/**
 * optional DisplayAction display_action = 43;
 * @return {?proto.ubo.v1.DisplayAction}
 */
proto.ubo.v1.Action.prototype.getDisplayAction = function() {
  return /** @type{?proto.ubo.v1.DisplayAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayAction, 43));
};


/**
 * @param {?proto.ubo.v1.DisplayAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplayAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 43, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplayAction = function() {
  return this.setDisplayAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplayAction = function() {
  return jspb.Message.getField(this, 43) != null;
};


/**
 * optional DisplayBlankAction display_blank_action = 44;
 * @return {?proto.ubo.v1.DisplayBlankAction}
 */
proto.ubo.v1.Action.prototype.getDisplayBlankAction = function() {
  return /** @type{?proto.ubo.v1.DisplayBlankAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayBlankAction, 44));
};


/**
 * @param {?proto.ubo.v1.DisplayBlankAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplayBlankAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 44, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplayBlankAction = function() {
  return this.setDisplayBlankAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplayBlankAction = function() {
  return jspb.Message.getField(this, 44) != null;
};


/**
 * optional DisplayPauseAction display_pause_action = 45;
 * @return {?proto.ubo.v1.DisplayPauseAction}
 */
proto.ubo.v1.Action.prototype.getDisplayPauseAction = function() {
  return /** @type{?proto.ubo.v1.DisplayPauseAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayPauseAction, 45));
};


/**
 * @param {?proto.ubo.v1.DisplayPauseAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplayPauseAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 45, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplayPauseAction = function() {
  return this.setDisplayPauseAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplayPauseAction = function() {
  return jspb.Message.getField(this, 45) != null;
};


/**
 * optional DisplayRedrawAction display_redraw_action = 46;
 * @return {?proto.ubo.v1.DisplayRedrawAction}
 */
proto.ubo.v1.Action.prototype.getDisplayRedrawAction = function() {
  return /** @type{?proto.ubo.v1.DisplayRedrawAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayRedrawAction, 46));
};


/**
 * @param {?proto.ubo.v1.DisplayRedrawAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplayRedrawAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 46, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplayRedrawAction = function() {
  return this.setDisplayRedrawAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplayRedrawAction = function() {
  return jspb.Message.getField(this, 46) != null;
};


/**
 * optional DisplayResumeAction display_resume_action = 47;
 * @return {?proto.ubo.v1.DisplayResumeAction}
 */
proto.ubo.v1.Action.prototype.getDisplayResumeAction = function() {
  return /** @type{?proto.ubo.v1.DisplayResumeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayResumeAction, 47));
};


/**
 * @param {?proto.ubo.v1.DisplayResumeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplayResumeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 47, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplayResumeAction = function() {
  return this.setDisplayResumeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplayResumeAction = function() {
  return jspb.Message.getField(this, 47) != null;
};


/**
 * optional DisplaySetBlankTimeoutAction display_set_blank_timeout_action = 48;
 * @return {?proto.ubo.v1.DisplaySetBlankTimeoutAction}
 */
proto.ubo.v1.Action.prototype.getDisplaySetBlankTimeoutAction = function() {
  return /** @type{?proto.ubo.v1.DisplaySetBlankTimeoutAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplaySetBlankTimeoutAction, 48));
};


/**
 * @param {?proto.ubo.v1.DisplaySetBlankTimeoutAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplaySetBlankTimeoutAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 48, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplaySetBlankTimeoutAction = function() {
  return this.setDisplaySetBlankTimeoutAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplaySetBlankTimeoutAction = function() {
  return jspb.Message.getField(this, 48) != null;
};


/**
 * optional DisplayUnblankAction display_unblank_action = 49;
 * @return {?proto.ubo.v1.DisplayUnblankAction}
 */
proto.ubo.v1.Action.prototype.getDisplayUnblankAction = function() {
  return /** @type{?proto.ubo.v1.DisplayUnblankAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayUnblankAction, 49));
};


/**
 * @param {?proto.ubo.v1.DisplayUnblankAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplayUnblankAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 49, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplayUnblankAction = function() {
  return this.setDisplayUnblankAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplayUnblankAction = function() {
  return jspb.Message.getField(this, 49) != null;
};


/**
 * optional DisplayUpdateActivityAction display_update_activity_action = 50;
 * @return {?proto.ubo.v1.DisplayUpdateActivityAction}
 */
proto.ubo.v1.Action.prototype.getDisplayUpdateActivityAction = function() {
  return /** @type{?proto.ubo.v1.DisplayUpdateActivityAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayUpdateActivityAction, 50));
};


/**
 * @param {?proto.ubo.v1.DisplayUpdateActivityAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDisplayUpdateActivityAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 50, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDisplayUpdateActivityAction = function() {
  return this.setDisplayUpdateActivityAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDisplayUpdateActivityAction = function() {
  return jspb.Message.getField(this, 50) != null;
};


/**
 * optional DockerAction docker_action = 51;
 * @return {?proto.ubo.v1.DockerAction}
 */
proto.ubo.v1.Action.prototype.getDockerAction = function() {
  return /** @type{?proto.ubo.v1.DockerAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerAction, 51));
};


/**
 * @param {?proto.ubo.v1.DockerAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 51, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerAction = function() {
  return this.setDockerAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerAction = function() {
  return jspb.Message.getField(this, 51) != null;
};


/**
 * optional DockerImageAction docker_image_action = 52;
 * @return {?proto.ubo.v1.DockerImageAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageAction, 52));
};


/**
 * @param {?proto.ubo.v1.DockerImageAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 52, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageAction = function() {
  return this.setDockerImageAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageAction = function() {
  return jspb.Message.getField(this, 52) != null;
};


/**
 * optional DockerImageFetchAction docker_image_fetch_action = 53;
 * @return {?proto.ubo.v1.DockerImageFetchAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageFetchAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageFetchAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageFetchAction, 53));
};


/**
 * @param {?proto.ubo.v1.DockerImageFetchAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageFetchAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 53, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageFetchAction = function() {
  return this.setDockerImageFetchAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageFetchAction = function() {
  return jspb.Message.getField(this, 53) != null;
};


/**
 * optional DockerImageReleaseAction docker_image_release_action = 54;
 * @return {?proto.ubo.v1.DockerImageReleaseAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageReleaseAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageReleaseAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageReleaseAction, 54));
};


/**
 * @param {?proto.ubo.v1.DockerImageReleaseAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageReleaseAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 54, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageReleaseAction = function() {
  return this.setDockerImageReleaseAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageReleaseAction = function() {
  return jspb.Message.getField(this, 54) != null;
};


/**
 * optional DockerImageRemoveAction docker_image_remove_action = 55;
 * @return {?proto.ubo.v1.DockerImageRemoveAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageRemoveAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageRemoveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveAction, 55));
};


/**
 * @param {?proto.ubo.v1.DockerImageRemoveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageRemoveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 55, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageRemoveAction = function() {
  return this.setDockerImageRemoveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageRemoveAction = function() {
  return jspb.Message.getField(this, 55) != null;
};


/**
 * optional DockerImageRemoveContainerAction docker_image_remove_container_action = 56;
 * @return {?proto.ubo.v1.DockerImageRemoveContainerAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageRemoveContainerAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageRemoveContainerAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveContainerAction, 56));
};


/**
 * @param {?proto.ubo.v1.DockerImageRemoveContainerAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageRemoveContainerAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 56, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageRemoveContainerAction = function() {
  return this.setDockerImageRemoveContainerAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageRemoveContainerAction = function() {
  return jspb.Message.getField(this, 56) != null;
};


/**
 * optional DockerImageRunAction docker_image_run_action = 57;
 * @return {?proto.ubo.v1.DockerImageRunAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageRunAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageRunAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRunAction, 57));
};


/**
 * @param {?proto.ubo.v1.DockerImageRunAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageRunAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 57, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageRunAction = function() {
  return this.setDockerImageRunAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageRunAction = function() {
  return jspb.Message.getField(this, 57) != null;
};


/**
 * optional DockerImageSetDockerIdAction docker_image_set_docker_id_action = 58;
 * @return {?proto.ubo.v1.DockerImageSetDockerIdAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageSetDockerIdAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageSetDockerIdAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageSetDockerIdAction, 58));
};


/**
 * @param {?proto.ubo.v1.DockerImageSetDockerIdAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageSetDockerIdAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 58, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageSetDockerIdAction = function() {
  return this.setDockerImageSetDockerIdAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageSetDockerIdAction = function() {
  return jspb.Message.getField(this, 58) != null;
};


/**
 * optional DockerImageSetStatusAction docker_image_set_status_action = 59;
 * @return {?proto.ubo.v1.DockerImageSetStatusAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageSetStatusAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageSetStatusAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageSetStatusAction, 59));
};


/**
 * @param {?proto.ubo.v1.DockerImageSetStatusAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageSetStatusAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 59, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageSetStatusAction = function() {
  return this.setDockerImageSetStatusAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageSetStatusAction = function() {
  return jspb.Message.getField(this, 59) != null;
};


/**
 * optional DockerImageStopAction docker_image_stop_action = 60;
 * @return {?proto.ubo.v1.DockerImageStopAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageStopAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageStopAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageStopAction, 60));
};


/**
 * @param {?proto.ubo.v1.DockerImageStopAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageStopAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 60, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageStopAction = function() {
  return this.setDockerImageStopAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageStopAction = function() {
  return jspb.Message.getField(this, 60) != null;
};


/**
 * optional DockerImageUpdateMetadataAction docker_image_update_metadata_action = 61;
 * @return {?proto.ubo.v1.DockerImageUpdateMetadataAction}
 */
proto.ubo.v1.Action.prototype.getDockerImageUpdateMetadataAction = function() {
  return /** @type{?proto.ubo.v1.DockerImageUpdateMetadataAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageUpdateMetadataAction, 61));
};


/**
 * @param {?proto.ubo.v1.DockerImageUpdateMetadataAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerImageUpdateMetadataAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 61, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerImageUpdateMetadataAction = function() {
  return this.setDockerImageUpdateMetadataAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerImageUpdateMetadataAction = function() {
  return jspb.Message.getField(this, 61) != null;
};


/**
 * optional DockerInstallAction docker_install_action = 62;
 * @return {?proto.ubo.v1.DockerInstallAction}
 */
proto.ubo.v1.Action.prototype.getDockerInstallAction = function() {
  return /** @type{?proto.ubo.v1.DockerInstallAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerInstallAction, 62));
};


/**
 * @param {?proto.ubo.v1.DockerInstallAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerInstallAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 62, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerInstallAction = function() {
  return this.setDockerInstallAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerInstallAction = function() {
  return jspb.Message.getField(this, 62) != null;
};


/**
 * optional DockerRemoveUsernameAction docker_remove_username_action = 63;
 * @return {?proto.ubo.v1.DockerRemoveUsernameAction}
 */
proto.ubo.v1.Action.prototype.getDockerRemoveUsernameAction = function() {
  return /** @type{?proto.ubo.v1.DockerRemoveUsernameAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerRemoveUsernameAction, 63));
};


/**
 * @param {?proto.ubo.v1.DockerRemoveUsernameAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerRemoveUsernameAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 63, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerRemoveUsernameAction = function() {
  return this.setDockerRemoveUsernameAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerRemoveUsernameAction = function() {
  return jspb.Message.getField(this, 63) != null;
};


/**
 * optional DockerSetStatusAction docker_set_status_action = 64;
 * @return {?proto.ubo.v1.DockerSetStatusAction}
 */
proto.ubo.v1.Action.prototype.getDockerSetStatusAction = function() {
  return /** @type{?proto.ubo.v1.DockerSetStatusAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerSetStatusAction, 64));
};


/**
 * @param {?proto.ubo.v1.DockerSetStatusAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerSetStatusAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 64, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerSetStatusAction = function() {
  return this.setDockerSetStatusAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerSetStatusAction = function() {
  return jspb.Message.getField(this, 64) != null;
};


/**
 * optional DockerStartAction docker_start_action = 65;
 * @return {?proto.ubo.v1.DockerStartAction}
 */
proto.ubo.v1.Action.prototype.getDockerStartAction = function() {
  return /** @type{?proto.ubo.v1.DockerStartAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStartAction, 65));
};


/**
 * @param {?proto.ubo.v1.DockerStartAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerStartAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 65, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerStartAction = function() {
  return this.setDockerStartAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerStartAction = function() {
  return jspb.Message.getField(this, 65) != null;
};


/**
 * optional DockerStopAction docker_stop_action = 66;
 * @return {?proto.ubo.v1.DockerStopAction}
 */
proto.ubo.v1.Action.prototype.getDockerStopAction = function() {
  return /** @type{?proto.ubo.v1.DockerStopAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStopAction, 66));
};


/**
 * @param {?proto.ubo.v1.DockerStopAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerStopAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 66, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerStopAction = function() {
  return this.setDockerStopAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerStopAction = function() {
  return jspb.Message.getField(this, 66) != null;
};


/**
 * optional DockerStoreUsernameAction docker_store_username_action = 67;
 * @return {?proto.ubo.v1.DockerStoreUsernameAction}
 */
proto.ubo.v1.Action.prototype.getDockerStoreUsernameAction = function() {
  return /** @type{?proto.ubo.v1.DockerStoreUsernameAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStoreUsernameAction, 67));
};


/**
 * @param {?proto.ubo.v1.DockerStoreUsernameAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setDockerStoreUsernameAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 67, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearDockerStoreUsernameAction = function() {
  return this.setDockerStoreUsernameAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasDockerStoreUsernameAction = function() {
  return jspb.Message.getField(this, 67) != null;
};


/**
 * optional FileSystemAction file_system_action = 68;
 * @return {?proto.ubo.v1.FileSystemAction}
 */
proto.ubo.v1.Action.prototype.getFileSystemAction = function() {
  return /** @type{?proto.ubo.v1.FileSystemAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemAction, 68));
};


/**
 * @param {?proto.ubo.v1.FileSystemAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setFileSystemAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 68, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearFileSystemAction = function() {
  return this.setFileSystemAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasFileSystemAction = function() {
  return jspb.Message.getField(this, 68) != null;
};


/**
 * optional FileSystemCopyAction file_system_copy_action = 69;
 * @return {?proto.ubo.v1.FileSystemCopyAction}
 */
proto.ubo.v1.Action.prototype.getFileSystemCopyAction = function() {
  return /** @type{?proto.ubo.v1.FileSystemCopyAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemCopyAction, 69));
};


/**
 * @param {?proto.ubo.v1.FileSystemCopyAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setFileSystemCopyAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 69, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearFileSystemCopyAction = function() {
  return this.setFileSystemCopyAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasFileSystemCopyAction = function() {
  return jspb.Message.getField(this, 69) != null;
};


/**
 * optional FileSystemMoveAction file_system_move_action = 70;
 * @return {?proto.ubo.v1.FileSystemMoveAction}
 */
proto.ubo.v1.Action.prototype.getFileSystemMoveAction = function() {
  return /** @type{?proto.ubo.v1.FileSystemMoveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemMoveAction, 70));
};


/**
 * @param {?proto.ubo.v1.FileSystemMoveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setFileSystemMoveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 70, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearFileSystemMoveAction = function() {
  return this.setFileSystemMoveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasFileSystemMoveAction = function() {
  return jspb.Message.getField(this, 70) != null;
};


/**
 * optional FileSystemRemoveAction file_system_remove_action = 71;
 * @return {?proto.ubo.v1.FileSystemRemoveAction}
 */
proto.ubo.v1.Action.prototype.getFileSystemRemoveAction = function() {
  return /** @type{?proto.ubo.v1.FileSystemRemoveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemRemoveAction, 71));
};


/**
 * @param {?proto.ubo.v1.FileSystemRemoveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setFileSystemRemoveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 71, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearFileSystemRemoveAction = function() {
  return this.setFileSystemRemoveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasFileSystemRemoveAction = function() {
  return jspb.Message.getField(this, 71) != null;
};


/**
 * optional FileSystemReportSelectionAction file_system_report_selection_action = 72;
 * @return {?proto.ubo.v1.FileSystemReportSelectionAction}
 */
proto.ubo.v1.Action.prototype.getFileSystemReportSelectionAction = function() {
  return /** @type{?proto.ubo.v1.FileSystemReportSelectionAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemReportSelectionAction, 72));
};


/**
 * @param {?proto.ubo.v1.FileSystemReportSelectionAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setFileSystemReportSelectionAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 72, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearFileSystemReportSelectionAction = function() {
  return this.setFileSystemReportSelectionAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasFileSystemReportSelectionAction = function() {
  return jspb.Message.getField(this, 72) != null;
};


/**
 * optional InfraredAction infrared_action = 73;
 * @return {?proto.ubo.v1.InfraredAction}
 */
proto.ubo.v1.Action.prototype.getInfraredAction = function() {
  return /** @type{?proto.ubo.v1.InfraredAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredAction, 73));
};


/**
 * @param {?proto.ubo.v1.InfraredAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInfraredAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 73, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInfraredAction = function() {
  return this.setInfraredAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInfraredAction = function() {
  return jspb.Message.getField(this, 73) != null;
};


/**
 * optional InfraredHandleReceivedCodeAction infrared_handle_received_code_action = 74;
 * @return {?proto.ubo.v1.InfraredHandleReceivedCodeAction}
 */
proto.ubo.v1.Action.prototype.getInfraredHandleReceivedCodeAction = function() {
  return /** @type{?proto.ubo.v1.InfraredHandleReceivedCodeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredHandleReceivedCodeAction, 74));
};


/**
 * @param {?proto.ubo.v1.InfraredHandleReceivedCodeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInfraredHandleReceivedCodeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 74, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInfraredHandleReceivedCodeAction = function() {
  return this.setInfraredHandleReceivedCodeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInfraredHandleReceivedCodeAction = function() {
  return jspb.Message.getField(this, 74) != null;
};


/**
 * optional InfraredSendCodeAction infrared_send_code_action = 75;
 * @return {?proto.ubo.v1.InfraredSendCodeAction}
 */
proto.ubo.v1.Action.prototype.getInfraredSendCodeAction = function() {
  return /** @type{?proto.ubo.v1.InfraredSendCodeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSendCodeAction, 75));
};


/**
 * @param {?proto.ubo.v1.InfraredSendCodeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInfraredSendCodeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 75, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInfraredSendCodeAction = function() {
  return this.setInfraredSendCodeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInfraredSendCodeAction = function() {
  return jspb.Message.getField(this, 75) != null;
};


/**
 * optional InfraredSetShouldPropagateAction infrared_set_should_propagate_action = 76;
 * @return {?proto.ubo.v1.InfraredSetShouldPropagateAction}
 */
proto.ubo.v1.Action.prototype.getInfraredSetShouldPropagateAction = function() {
  return /** @type{?proto.ubo.v1.InfraredSetShouldPropagateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSetShouldPropagateAction, 76));
};


/**
 * @param {?proto.ubo.v1.InfraredSetShouldPropagateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInfraredSetShouldPropagateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 76, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInfraredSetShouldPropagateAction = function() {
  return this.setInfraredSetShouldPropagateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInfraredSetShouldPropagateAction = function() {
  return jspb.Message.getField(this, 76) != null;
};


/**
 * optional InfraredSetShouldReceiveAction infrared_set_should_receive_action = 77;
 * @return {?proto.ubo.v1.InfraredSetShouldReceiveAction}
 */
proto.ubo.v1.Action.prototype.getInfraredSetShouldReceiveAction = function() {
  return /** @type{?proto.ubo.v1.InfraredSetShouldReceiveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSetShouldReceiveAction, 77));
};


/**
 * @param {?proto.ubo.v1.InfraredSetShouldReceiveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInfraredSetShouldReceiveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 77, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInfraredSetShouldReceiveAction = function() {
  return this.setInfraredSetShouldReceiveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInfraredSetShouldReceiveAction = function() {
  return jspb.Message.getField(this, 77) != null;
};


/**
 * optional InputAction input_action = 78;
 * @return {?proto.ubo.v1.InputAction}
 */
proto.ubo.v1.Action.prototype.getInputAction = function() {
  return /** @type{?proto.ubo.v1.InputAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputAction, 78));
};


/**
 * @param {?proto.ubo.v1.InputAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInputAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 78, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInputAction = function() {
  return this.setInputAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInputAction = function() {
  return jspb.Message.getField(this, 78) != null;
};


/**
 * optional InputCancelAction input_cancel_action = 79;
 * @return {?proto.ubo.v1.InputCancelAction}
 */
proto.ubo.v1.Action.prototype.getInputCancelAction = function() {
  return /** @type{?proto.ubo.v1.InputCancelAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputCancelAction, 79));
};


/**
 * @param {?proto.ubo.v1.InputCancelAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInputCancelAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 79, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInputCancelAction = function() {
  return this.setInputCancelAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInputCancelAction = function() {
  return jspb.Message.getField(this, 79) != null;
};


/**
 * optional InputDemandAction input_demand_action = 80;
 * @return {?proto.ubo.v1.InputDemandAction}
 */
proto.ubo.v1.Action.prototype.getInputDemandAction = function() {
  return /** @type{?proto.ubo.v1.InputDemandAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputDemandAction, 80));
};


/**
 * @param {?proto.ubo.v1.InputDemandAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInputDemandAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 80, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInputDemandAction = function() {
  return this.setInputDemandAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInputDemandAction = function() {
  return jspb.Message.getField(this, 80) != null;
};


/**
 * optional InputProvideAction input_provide_action = 81;
 * @return {?proto.ubo.v1.InputProvideAction}
 */
proto.ubo.v1.Action.prototype.getInputProvideAction = function() {
  return /** @type{?proto.ubo.v1.InputProvideAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputProvideAction, 81));
};


/**
 * @param {?proto.ubo.v1.InputProvideAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInputProvideAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 81, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInputProvideAction = function() {
  return this.setInputProvideAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInputProvideAction = function() {
  return jspb.Message.getField(this, 81) != null;
};


/**
 * optional InputResolveAction input_resolve_action = 82;
 * @return {?proto.ubo.v1.InputResolveAction}
 */
proto.ubo.v1.Action.prototype.getInputResolveAction = function() {
  return /** @type{?proto.ubo.v1.InputResolveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResolveAction, 82));
};


/**
 * @param {?proto.ubo.v1.InputResolveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setInputResolveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 82, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearInputResolveAction = function() {
  return this.setInputResolveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasInputResolveAction = function() {
  return jspb.Message.getField(this, 82) != null;
};


/**
 * optional IpAction ip_action = 83;
 * @return {?proto.ubo.v1.IpAction}
 */
proto.ubo.v1.Action.prototype.getIpAction = function() {
  return /** @type{?proto.ubo.v1.IpAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.IpAction, 83));
};


/**
 * @param {?proto.ubo.v1.IpAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setIpAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 83, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearIpAction = function() {
  return this.setIpAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasIpAction = function() {
  return jspb.Message.getField(this, 83) != null;
};


/**
 * optional IpSetIsConnectedAction ip_set_is_connected_action = 84;
 * @return {?proto.ubo.v1.IpSetIsConnectedAction}
 */
proto.ubo.v1.Action.prototype.getIpSetIsConnectedAction = function() {
  return /** @type{?proto.ubo.v1.IpSetIsConnectedAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.IpSetIsConnectedAction, 84));
};


/**
 * @param {?proto.ubo.v1.IpSetIsConnectedAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setIpSetIsConnectedAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 84, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearIpSetIsConnectedAction = function() {
  return this.setIpSetIsConnectedAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasIpSetIsConnectedAction = function() {
  return jspb.Message.getField(this, 84) != null;
};


/**
 * optional IpUpdateInterfacesAction ip_update_interfaces_action = 85;
 * @return {?proto.ubo.v1.IpUpdateInterfacesAction}
 */
proto.ubo.v1.Action.prototype.getIpUpdateInterfacesAction = function() {
  return /** @type{?proto.ubo.v1.IpUpdateInterfacesAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.IpUpdateInterfacesAction, 85));
};


/**
 * @param {?proto.ubo.v1.IpUpdateInterfacesAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setIpUpdateInterfacesAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 85, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearIpUpdateInterfacesAction = function() {
  return this.setIpUpdateInterfacesAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasIpUpdateInterfacesAction = function() {
  return jspb.Message.getField(this, 85) != null;
};


/**
 * optional KeypadAction keypad_action = 86;
 * @return {?proto.ubo.v1.KeypadAction}
 */
proto.ubo.v1.Action.prototype.getKeypadAction = function() {
  return /** @type{?proto.ubo.v1.KeypadAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadAction, 86));
};


/**
 * @param {?proto.ubo.v1.KeypadAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setKeypadAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 86, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearKeypadAction = function() {
  return this.setKeypadAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasKeypadAction = function() {
  return jspb.Message.getField(this, 86) != null;
};


/**
 * optional KeypadKeyHoldAction keypad_key_hold_action = 87;
 * @return {?proto.ubo.v1.KeypadKeyHoldAction}
 */
proto.ubo.v1.Action.prototype.getKeypadKeyHoldAction = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyHoldAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyHoldAction, 87));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyHoldAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setKeypadKeyHoldAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 87, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearKeypadKeyHoldAction = function() {
  return this.setKeypadKeyHoldAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasKeypadKeyHoldAction = function() {
  return jspb.Message.getField(this, 87) != null;
};


/**
 * optional KeypadKeyPressAction keypad_key_press_action = 88;
 * @return {?proto.ubo.v1.KeypadKeyPressAction}
 */
proto.ubo.v1.Action.prototype.getKeypadKeyPressAction = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyPressAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyPressAction, 88));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyPressAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setKeypadKeyPressAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 88, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearKeypadKeyPressAction = function() {
  return this.setKeypadKeyPressAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasKeypadKeyPressAction = function() {
  return jspb.Message.getField(this, 88) != null;
};


/**
 * optional KeypadKeyReleaseAction keypad_key_release_action = 89;
 * @return {?proto.ubo.v1.KeypadKeyReleaseAction}
 */
proto.ubo.v1.Action.prototype.getKeypadKeyReleaseAction = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyReleaseAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyReleaseAction, 89));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyReleaseAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setKeypadKeyReleaseAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 89, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearKeypadKeyReleaseAction = function() {
  return this.setKeypadKeyReleaseAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasKeypadKeyReleaseAction = function() {
  return jspb.Message.getField(this, 89) != null;
};


/**
 * optional KeypadKeyUnholdAction keypad_key_unhold_action = 90;
 * @return {?proto.ubo.v1.KeypadKeyUnholdAction}
 */
proto.ubo.v1.Action.prototype.getKeypadKeyUnholdAction = function() {
  return /** @type{?proto.ubo.v1.KeypadKeyUnholdAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.KeypadKeyUnholdAction, 90));
};


/**
 * @param {?proto.ubo.v1.KeypadKeyUnholdAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setKeypadKeyUnholdAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 90, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearKeypadKeyUnholdAction = function() {
  return this.setKeypadKeyUnholdAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasKeypadKeyUnholdAction = function() {
  return jspb.Message.getField(this, 90) != null;
};


/**
 * optional LightDMAction light_dm_action = 91;
 * @return {?proto.ubo.v1.LightDMAction}
 */
proto.ubo.v1.Action.prototype.getLightDmAction = function() {
  return /** @type{?proto.ubo.v1.LightDMAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.LightDMAction, 91));
};


/**
 * @param {?proto.ubo.v1.LightDMAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setLightDmAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 91, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearLightDmAction = function() {
  return this.setLightDmAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasLightDmAction = function() {
  return jspb.Message.getField(this, 91) != null;
};


/**
 * optional LightDMClearEnabledStateAction light_dm_clear_enabled_state_action = 92;
 * @return {?proto.ubo.v1.LightDMClearEnabledStateAction}
 */
proto.ubo.v1.Action.prototype.getLightDmClearEnabledStateAction = function() {
  return /** @type{?proto.ubo.v1.LightDMClearEnabledStateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.LightDMClearEnabledStateAction, 92));
};


/**
 * @param {?proto.ubo.v1.LightDMClearEnabledStateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setLightDmClearEnabledStateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 92, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearLightDmClearEnabledStateAction = function() {
  return this.setLightDmClearEnabledStateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasLightDmClearEnabledStateAction = function() {
  return jspb.Message.getField(this, 92) != null;
};


/**
 * optional LightDMUpdateStateAction light_dm_update_state_action = 93;
 * @return {?proto.ubo.v1.LightDMUpdateStateAction}
 */
proto.ubo.v1.Action.prototype.getLightDmUpdateStateAction = function() {
  return /** @type{?proto.ubo.v1.LightDMUpdateStateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.LightDMUpdateStateAction, 93));
};


/**
 * @param {?proto.ubo.v1.LightDMUpdateStateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setLightDmUpdateStateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 93, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearLightDmUpdateStateAction = function() {
  return this.setLightDmUpdateStateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasLightDmUpdateStateAction = function() {
  return jspb.Message.getField(this, 93) != null;
};


/**
 * optional MainAction main_action = 94;
 * @return {?proto.ubo.v1.MainAction}
 */
proto.ubo.v1.Action.prototype.getMainAction = function() {
  return /** @type{?proto.ubo.v1.MainAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MainAction, 94));
};


/**
 * @param {?proto.ubo.v1.MainAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMainAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 94, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMainAction = function() {
  return this.setMainAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMainAction = function() {
  return jspb.Message.getField(this, 94) != null;
};


/**
 * optional MenuAction menu_action = 95;
 * @return {?proto.ubo.v1.MenuAction}
 */
proto.ubo.v1.Action.prototype.getMenuAction = function() {
  return /** @type{?proto.ubo.v1.MenuAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuAction, 95));
};


/**
 * @param {?proto.ubo.v1.MenuAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMenuAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 95, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMenuAction = function() {
  return this.setMenuAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMenuAction = function() {
  return jspb.Message.getField(this, 95) != null;
};


/**
 * optional MenuChooseByIconAction menu_choose_by_icon_action = 96;
 * @return {?proto.ubo.v1.MenuChooseByIconAction}
 */
proto.ubo.v1.Action.prototype.getMenuChooseByIconAction = function() {
  return /** @type{?proto.ubo.v1.MenuChooseByIconAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIconAction, 96));
};


/**
 * @param {?proto.ubo.v1.MenuChooseByIconAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMenuChooseByIconAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 96, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMenuChooseByIconAction = function() {
  return this.setMenuChooseByIconAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMenuChooseByIconAction = function() {
  return jspb.Message.getField(this, 96) != null;
};


/**
 * optional MenuChooseByIndexAction menu_choose_by_index_action = 97;
 * @return {?proto.ubo.v1.MenuChooseByIndexAction}
 */
proto.ubo.v1.Action.prototype.getMenuChooseByIndexAction = function() {
  return /** @type{?proto.ubo.v1.MenuChooseByIndexAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIndexAction, 97));
};


/**
 * @param {?proto.ubo.v1.MenuChooseByIndexAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMenuChooseByIndexAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 97, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMenuChooseByIndexAction = function() {
  return this.setMenuChooseByIndexAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMenuChooseByIndexAction = function() {
  return jspb.Message.getField(this, 97) != null;
};


/**
 * optional MenuChooseByLabelAction menu_choose_by_label_action = 98;
 * @return {?proto.ubo.v1.MenuChooseByLabelAction}
 */
proto.ubo.v1.Action.prototype.getMenuChooseByLabelAction = function() {
  return /** @type{?proto.ubo.v1.MenuChooseByLabelAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByLabelAction, 98));
};


/**
 * @param {?proto.ubo.v1.MenuChooseByLabelAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMenuChooseByLabelAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 98, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMenuChooseByLabelAction = function() {
  return this.setMenuChooseByLabelAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMenuChooseByLabelAction = function() {
  return jspb.Message.getField(this, 98) != null;
};


/**
 * optional MenuGoBackAction menu_go_back_action = 99;
 * @return {?proto.ubo.v1.MenuGoBackAction}
 */
proto.ubo.v1.Action.prototype.getMenuGoBackAction = function() {
  return /** @type{?proto.ubo.v1.MenuGoBackAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoBackAction, 99));
};


/**
 * @param {?proto.ubo.v1.MenuGoBackAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMenuGoBackAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 99, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMenuGoBackAction = function() {
  return this.setMenuGoBackAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMenuGoBackAction = function() {
  return jspb.Message.getField(this, 99) != null;
};


/**
 * optional MenuGoHomeAction menu_go_home_action = 100;
 * @return {?proto.ubo.v1.MenuGoHomeAction}
 */
proto.ubo.v1.Action.prototype.getMenuGoHomeAction = function() {
  return /** @type{?proto.ubo.v1.MenuGoHomeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoHomeAction, 100));
};


/**
 * @param {?proto.ubo.v1.MenuGoHomeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMenuGoHomeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 100, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMenuGoHomeAction = function() {
  return this.setMenuGoHomeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMenuGoHomeAction = function() {
  return jspb.Message.getField(this, 100) != null;
};


/**
 * optional MenuScrollAction menu_scroll_action = 101;
 * @return {?proto.ubo.v1.MenuScrollAction}
 */
proto.ubo.v1.Action.prototype.getMenuScrollAction = function() {
  return /** @type{?proto.ubo.v1.MenuScrollAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuScrollAction, 101));
};


/**
 * @param {?proto.ubo.v1.MenuScrollAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setMenuScrollAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 101, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearMenuScrollAction = function() {
  return this.setMenuScrollAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasMenuScrollAction = function() {
  return jspb.Message.getField(this, 101) != null;
};


/**
 * optional NotificationsAction notifications_action = 102;
 * @return {?proto.ubo.v1.NotificationsAction}
 */
proto.ubo.v1.Action.prototype.getNotificationsAction = function() {
  return /** @type{?proto.ubo.v1.NotificationsAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsAction, 102));
};


/**
 * @param {?proto.ubo.v1.NotificationsAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setNotificationsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 102, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearNotificationsAction = function() {
  return this.setNotificationsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasNotificationsAction = function() {
  return jspb.Message.getField(this, 102) != null;
};


/**
 * optional NotificationsAddAction notifications_add_action = 103;
 * @return {?proto.ubo.v1.NotificationsAddAction}
 */
proto.ubo.v1.Action.prototype.getNotificationsAddAction = function() {
  return /** @type{?proto.ubo.v1.NotificationsAddAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsAddAction, 103));
};


/**
 * @param {?proto.ubo.v1.NotificationsAddAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setNotificationsAddAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 103, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearNotificationsAddAction = function() {
  return this.setNotificationsAddAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasNotificationsAddAction = function() {
  return jspb.Message.getField(this, 103) != null;
};


/**
 * optional NotificationsClearAction notifications_clear_action = 104;
 * @return {?proto.ubo.v1.NotificationsClearAction}
 */
proto.ubo.v1.Action.prototype.getNotificationsClearAction = function() {
  return /** @type{?proto.ubo.v1.NotificationsClearAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearAction, 104));
};


/**
 * @param {?proto.ubo.v1.NotificationsClearAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setNotificationsClearAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 104, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearNotificationsClearAction = function() {
  return this.setNotificationsClearAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasNotificationsClearAction = function() {
  return jspb.Message.getField(this, 104) != null;
};


/**
 * optional NotificationsClearAllAction notifications_clear_all_action = 105;
 * @return {?proto.ubo.v1.NotificationsClearAllAction}
 */
proto.ubo.v1.Action.prototype.getNotificationsClearAllAction = function() {
  return /** @type{?proto.ubo.v1.NotificationsClearAllAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearAllAction, 105));
};


/**
 * @param {?proto.ubo.v1.NotificationsClearAllAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setNotificationsClearAllAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 105, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearNotificationsClearAllAction = function() {
  return this.setNotificationsClearAllAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasNotificationsClearAllAction = function() {
  return jspb.Message.getField(this, 105) != null;
};


/**
 * optional NotificationsClearByIdAction notifications_clear_by_id_action = 106;
 * @return {?proto.ubo.v1.NotificationsClearByIdAction}
 */
proto.ubo.v1.Action.prototype.getNotificationsClearByIdAction = function() {
  return /** @type{?proto.ubo.v1.NotificationsClearByIdAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearByIdAction, 106));
};


/**
 * @param {?proto.ubo.v1.NotificationsClearByIdAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setNotificationsClearByIdAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 106, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearNotificationsClearByIdAction = function() {
  return this.setNotificationsClearByIdAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasNotificationsClearByIdAction = function() {
  return jspb.Message.getField(this, 106) != null;
};


/**
 * optional NotificationsDisplayAction notifications_display_action = 107;
 * @return {?proto.ubo.v1.NotificationsDisplayAction}
 */
proto.ubo.v1.Action.prototype.getNotificationsDisplayAction = function() {
  return /** @type{?proto.ubo.v1.NotificationsDisplayAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsDisplayAction, 107));
};


/**
 * @param {?proto.ubo.v1.NotificationsDisplayAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setNotificationsDisplayAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 107, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearNotificationsDisplayAction = function() {
  return this.setNotificationsDisplayAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasNotificationsDisplayAction = function() {
  return jspb.Message.getField(this, 107) != null;
};


/**
 * optional OpenApplicationAction open_application_action = 108;
 * @return {?proto.ubo.v1.OpenApplicationAction}
 */
proto.ubo.v1.Action.prototype.getOpenApplicationAction = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationAction, 108));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setOpenApplicationAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 108, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearOpenApplicationAction = function() {
  return this.setOpenApplicationAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasOpenApplicationAction = function() {
  return jspb.Message.getField(this, 108) != null;
};


/**
 * optional PowerAction power_action = 109;
 * @return {?proto.ubo.v1.PowerAction}
 */
proto.ubo.v1.Action.prototype.getPowerAction = function() {
  return /** @type{?proto.ubo.v1.PowerAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerAction, 109));
};


/**
 * @param {?proto.ubo.v1.PowerAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setPowerAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 109, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearPowerAction = function() {
  return this.setPowerAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasPowerAction = function() {
  return jspb.Message.getField(this, 109) != null;
};


/**
 * optional PowerOffAction power_off_action = 110;
 * @return {?proto.ubo.v1.PowerOffAction}
 */
proto.ubo.v1.Action.prototype.getPowerOffAction = function() {
  return /** @type{?proto.ubo.v1.PowerOffAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerOffAction, 110));
};


/**
 * @param {?proto.ubo.v1.PowerOffAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setPowerOffAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 110, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearPowerOffAction = function() {
  return this.setPowerOffAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasPowerOffAction = function() {
  return jspb.Message.getField(this, 110) != null;
};


/**
 * optional RPiConnectAction r_pi_connect_action = 111;
 * @return {?proto.ubo.v1.RPiConnectAction}
 */
proto.ubo.v1.Action.prototype.getRPiConnectAction = function() {
  return /** @type{?proto.ubo.v1.RPiConnectAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectAction, 111));
};


/**
 * @param {?proto.ubo.v1.RPiConnectAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRPiConnectAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 111, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRPiConnectAction = function() {
  return this.setRPiConnectAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRPiConnectAction = function() {
  return jspb.Message.getField(this, 111) != null;
};


/**
 * optional RPiConnectDoneDownloadingAction r_pi_connect_done_downloading_action = 112;
 * @return {?proto.ubo.v1.RPiConnectDoneDownloadingAction}
 */
proto.ubo.v1.Action.prototype.getRPiConnectDoneDownloadingAction = function() {
  return /** @type{?proto.ubo.v1.RPiConnectDoneDownloadingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectDoneDownloadingAction, 112));
};


/**
 * @param {?proto.ubo.v1.RPiConnectDoneDownloadingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRPiConnectDoneDownloadingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 112, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRPiConnectDoneDownloadingAction = function() {
  return this.setRPiConnectDoneDownloadingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRPiConnectDoneDownloadingAction = function() {
  return jspb.Message.getField(this, 112) != null;
};


/**
 * optional RPiConnectSetPendingAction r_pi_connect_set_pending_action = 113;
 * @return {?proto.ubo.v1.RPiConnectSetPendingAction}
 */
proto.ubo.v1.Action.prototype.getRPiConnectSetPendingAction = function() {
  return /** @type{?proto.ubo.v1.RPiConnectSetPendingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectSetPendingAction, 113));
};


/**
 * @param {?proto.ubo.v1.RPiConnectSetPendingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRPiConnectSetPendingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 113, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRPiConnectSetPendingAction = function() {
  return this.setRPiConnectSetPendingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRPiConnectSetPendingAction = function() {
  return jspb.Message.getField(this, 113) != null;
};


/**
 * optional RPiConnectSetStatusAction r_pi_connect_set_status_action = 114;
 * @return {?proto.ubo.v1.RPiConnectSetStatusAction}
 */
proto.ubo.v1.Action.prototype.getRPiConnectSetStatusAction = function() {
  return /** @type{?proto.ubo.v1.RPiConnectSetStatusAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectSetStatusAction, 114));
};


/**
 * @param {?proto.ubo.v1.RPiConnectSetStatusAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRPiConnectSetStatusAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 114, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRPiConnectSetStatusAction = function() {
  return this.setRPiConnectSetStatusAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRPiConnectSetStatusAction = function() {
  return jspb.Message.getField(this, 114) != null;
};


/**
 * optional RPiConnectStartDownloadingAction r_pi_connect_start_downloading_action = 115;
 * @return {?proto.ubo.v1.RPiConnectStartDownloadingAction}
 */
proto.ubo.v1.Action.prototype.getRPiConnectStartDownloadingAction = function() {
  return /** @type{?proto.ubo.v1.RPiConnectStartDownloadingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectStartDownloadingAction, 115));
};


/**
 * @param {?proto.ubo.v1.RPiConnectStartDownloadingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRPiConnectStartDownloadingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 115, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRPiConnectStartDownloadingAction = function() {
  return this.setRPiConnectStartDownloadingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRPiConnectStartDownloadingAction = function() {
  return jspb.Message.getField(this, 115) != null;
};


/**
 * optional RPiConnectUpdateServiceStateAction r_pi_connect_update_service_state_action = 116;
 * @return {?proto.ubo.v1.RPiConnectUpdateServiceStateAction}
 */
proto.ubo.v1.Action.prototype.getRPiConnectUpdateServiceStateAction = function() {
  return /** @type{?proto.ubo.v1.RPiConnectUpdateServiceStateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectUpdateServiceStateAction, 116));
};


/**
 * @param {?proto.ubo.v1.RPiConnectUpdateServiceStateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRPiConnectUpdateServiceStateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 116, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRPiConnectUpdateServiceStateAction = function() {
  return this.setRPiConnectUpdateServiceStateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRPiConnectUpdateServiceStateAction = function() {
  return jspb.Message.getField(this, 116) != null;
};


/**
 * optional RebootAction reboot_action = 117;
 * @return {?proto.ubo.v1.RebootAction}
 */
proto.ubo.v1.Action.prototype.getRebootAction = function() {
  return /** @type{?proto.ubo.v1.RebootAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RebootAction, 117));
};


/**
 * @param {?proto.ubo.v1.RebootAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRebootAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 117, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRebootAction = function() {
  return this.setRebootAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRebootAction = function() {
  return jspb.Message.getField(this, 117) != null;
};


/**
 * optional RegisterAppAction register_app_action = 118;
 * @return {?proto.ubo.v1.RegisterAppAction}
 */
proto.ubo.v1.Action.prototype.getRegisterAppAction = function() {
  return /** @type{?proto.ubo.v1.RegisterAppAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RegisterAppAction, 118));
};


/**
 * @param {?proto.ubo.v1.RegisterAppAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRegisterAppAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 118, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRegisterAppAction = function() {
  return this.setRegisterAppAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRegisterAppAction = function() {
  return jspb.Message.getField(this, 118) != null;
};


/**
 * optional RegisterRegularAppAction register_regular_app_action = 119;
 * @return {?proto.ubo.v1.RegisterRegularAppAction}
 */
proto.ubo.v1.Action.prototype.getRegisterRegularAppAction = function() {
  return /** @type{?proto.ubo.v1.RegisterRegularAppAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RegisterRegularAppAction, 119));
};


/**
 * @param {?proto.ubo.v1.RegisterRegularAppAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRegisterRegularAppAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 119, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRegisterRegularAppAction = function() {
  return this.setRegisterRegularAppAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRegisterRegularAppAction = function() {
  return jspb.Message.getField(this, 119) != null;
};


/**
 * optional RegisterSettingAppAction register_setting_app_action = 120;
 * @return {?proto.ubo.v1.RegisterSettingAppAction}
 */
proto.ubo.v1.Action.prototype.getRegisterSettingAppAction = function() {
  return /** @type{?proto.ubo.v1.RegisterSettingAppAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RegisterSettingAppAction, 120));
};


/**
 * @param {?proto.ubo.v1.RegisterSettingAppAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRegisterSettingAppAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 120, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRegisterSettingAppAction = function() {
  return this.setRegisterSettingAppAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRegisterSettingAppAction = function() {
  return jspb.Message.getField(this, 120) != null;
};


/**
 * optional ReplayRecordedSequenceAction replay_recorded_sequence_action = 121;
 * @return {?proto.ubo.v1.ReplayRecordedSequenceAction}
 */
proto.ubo.v1.Action.prototype.getReplayRecordedSequenceAction = function() {
  return /** @type{?proto.ubo.v1.ReplayRecordedSequenceAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ReplayRecordedSequenceAction, 121));
};


/**
 * @param {?proto.ubo.v1.ReplayRecordedSequenceAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setReplayRecordedSequenceAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 121, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearReplayRecordedSequenceAction = function() {
  return this.setReplayRecordedSequenceAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasReplayRecordedSequenceAction = function() {
  return jspb.Message.getField(this, 121) != null;
};


/**
 * optional ReportReplayingDoneAction report_replaying_done_action = 122;
 * @return {?proto.ubo.v1.ReportReplayingDoneAction}
 */
proto.ubo.v1.Action.prototype.getReportReplayingDoneAction = function() {
  return /** @type{?proto.ubo.v1.ReportReplayingDoneAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ReportReplayingDoneAction, 122));
};


/**
 * @param {?proto.ubo.v1.ReportReplayingDoneAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setReportReplayingDoneAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 122, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearReportReplayingDoneAction = function() {
  return this.setReportReplayingDoneAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasReportReplayingDoneAction = function() {
  return jspb.Message.getField(this, 122) != null;
};


/**
 * optional RgbRingAction rgb_ring_action = 123;
 * @return {?proto.ubo.v1.RgbRingAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingAction, 123));
};


/**
 * @param {?proto.ubo.v1.RgbRingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 123, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingAction = function() {
  return this.setRgbRingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingAction = function() {
  return jspb.Message.getField(this, 123) != null;
};


/**
 * optional RgbRingBlankAction rgb_ring_blank_action = 124;
 * @return {?proto.ubo.v1.RgbRingBlankAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingBlankAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingBlankAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingBlankAction, 124));
};


/**
 * @param {?proto.ubo.v1.RgbRingBlankAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingBlankAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 124, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingBlankAction = function() {
  return this.setRgbRingBlankAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingBlankAction = function() {
  return jspb.Message.getField(this, 124) != null;
};


/**
 * optional RgbRingBlinkAction rgb_ring_blink_action = 125;
 * @return {?proto.ubo.v1.RgbRingBlinkAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingBlinkAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingBlinkAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingBlinkAction, 125));
};


/**
 * @param {?proto.ubo.v1.RgbRingBlinkAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingBlinkAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 125, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingBlinkAction = function() {
  return this.setRgbRingBlinkAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingBlinkAction = function() {
  return jspb.Message.getField(this, 125) != null;
};


/**
 * optional RgbRingColorfulCommandAction rgb_ring_colorful_command_action = 126;
 * @return {?proto.ubo.v1.RgbRingColorfulCommandAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingColorfulCommandAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingColorfulCommandAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingColorfulCommandAction, 126));
};


/**
 * @param {?proto.ubo.v1.RgbRingColorfulCommandAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingColorfulCommandAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 126, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingColorfulCommandAction = function() {
  return this.setRgbRingColorfulCommandAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingColorfulCommandAction = function() {
  return jspb.Message.getField(this, 126) != null;
};


/**
 * optional RgbRingCommandAction rgb_ring_command_action = 127;
 * @return {?proto.ubo.v1.RgbRingCommandAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingCommandAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingCommandAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingCommandAction, 127));
};


/**
 * @param {?proto.ubo.v1.RgbRingCommandAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingCommandAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 127, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingCommandAction = function() {
  return this.setRgbRingCommandAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingCommandAction = function() {
  return jspb.Message.getField(this, 127) != null;
};


/**
 * optional RgbRingFillDownfromAction rgb_ring_fill_downfrom_action = 128;
 * @return {?proto.ubo.v1.RgbRingFillDownfromAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingFillDownfromAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingFillDownfromAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingFillDownfromAction, 128));
};


/**
 * @param {?proto.ubo.v1.RgbRingFillDownfromAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingFillDownfromAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 128, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingFillDownfromAction = function() {
  return this.setRgbRingFillDownfromAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingFillDownfromAction = function() {
  return jspb.Message.getField(this, 128) != null;
};


/**
 * optional RgbRingFillUptoAction rgb_ring_fill_upto_action = 129;
 * @return {?proto.ubo.v1.RgbRingFillUptoAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingFillUptoAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingFillUptoAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingFillUptoAction, 129));
};


/**
 * @param {?proto.ubo.v1.RgbRingFillUptoAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingFillUptoAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 129, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingFillUptoAction = function() {
  return this.setRgbRingFillUptoAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingFillUptoAction = function() {
  return jspb.Message.getField(this, 129) != null;
};


/**
 * optional RgbRingProgressWheelAction rgb_ring_progress_wheel_action = 130;
 * @return {?proto.ubo.v1.RgbRingProgressWheelAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingProgressWheelAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingProgressWheelAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingProgressWheelAction, 130));
};


/**
 * @param {?proto.ubo.v1.RgbRingProgressWheelAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingProgressWheelAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 130, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingProgressWheelAction = function() {
  return this.setRgbRingProgressWheelAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingProgressWheelAction = function() {
  return jspb.Message.getField(this, 130) != null;
};


/**
 * optional RgbRingProgressWheelStepAction rgb_ring_progress_wheel_step_action = 131;
 * @return {?proto.ubo.v1.RgbRingProgressWheelStepAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingProgressWheelStepAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingProgressWheelStepAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingProgressWheelStepAction, 131));
};


/**
 * @param {?proto.ubo.v1.RgbRingProgressWheelStepAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingProgressWheelStepAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 131, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingProgressWheelStepAction = function() {
  return this.setRgbRingProgressWheelStepAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingProgressWheelStepAction = function() {
  return jspb.Message.getField(this, 131) != null;
};


/**
 * optional RgbRingPulseAction rgb_ring_pulse_action = 132;
 * @return {?proto.ubo.v1.RgbRingPulseAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingPulseAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingPulseAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingPulseAction, 132));
};


/**
 * @param {?proto.ubo.v1.RgbRingPulseAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingPulseAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 132, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingPulseAction = function() {
  return this.setRgbRingPulseAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingPulseAction = function() {
  return jspb.Message.getField(this, 132) != null;
};


/**
 * optional RgbRingRainbowAction rgb_ring_rainbow_action = 133;
 * @return {?proto.ubo.v1.RgbRingRainbowAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingRainbowAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingRainbowAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingRainbowAction, 133));
};


/**
 * @param {?proto.ubo.v1.RgbRingRainbowAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingRainbowAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 133, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingRainbowAction = function() {
  return this.setRgbRingRainbowAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingRainbowAction = function() {
  return jspb.Message.getField(this, 133) != null;
};


/**
 * optional RgbRingSequenceAction rgb_ring_sequence_action = 134;
 * @return {?proto.ubo.v1.RgbRingSequenceAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingSequenceAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingSequenceAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSequenceAction, 134));
};


/**
 * @param {?proto.ubo.v1.RgbRingSequenceAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingSequenceAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 134, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingSequenceAction = function() {
  return this.setRgbRingSequenceAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingSequenceAction = function() {
  return jspb.Message.getField(this, 134) != null;
};


/**
 * optional RgbRingSetAllAction rgb_ring_set_all_action = 135;
 * @return {?proto.ubo.v1.RgbRingSetAllAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingSetAllAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingSetAllAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetAllAction, 135));
};


/**
 * @param {?proto.ubo.v1.RgbRingSetAllAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingSetAllAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 135, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingSetAllAction = function() {
  return this.setRgbRingSetAllAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingSetAllAction = function() {
  return jspb.Message.getField(this, 135) != null;
};


/**
 * optional RgbRingSetBrightnessAction rgb_ring_set_brightness_action = 136;
 * @return {?proto.ubo.v1.RgbRingSetBrightnessAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingSetBrightnessAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingSetBrightnessAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetBrightnessAction, 136));
};


/**
 * @param {?proto.ubo.v1.RgbRingSetBrightnessAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingSetBrightnessAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 136, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingSetBrightnessAction = function() {
  return this.setRgbRingSetBrightnessAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingSetBrightnessAction = function() {
  return jspb.Message.getField(this, 136) != null;
};


/**
 * optional RgbRingSetEnabledAction rgb_ring_set_enabled_action = 137;
 * @return {?proto.ubo.v1.RgbRingSetEnabledAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingSetEnabledAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingSetEnabledAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetEnabledAction, 137));
};


/**
 * @param {?proto.ubo.v1.RgbRingSetEnabledAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingSetEnabledAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 137, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingSetEnabledAction = function() {
  return this.setRgbRingSetEnabledAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingSetEnabledAction = function() {
  return jspb.Message.getField(this, 137) != null;
};


/**
 * optional RgbRingSetIsBusyAction rgb_ring_set_is_busy_action = 138;
 * @return {?proto.ubo.v1.RgbRingSetIsBusyAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingSetIsBusyAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingSetIsBusyAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSetIsBusyAction, 138));
};


/**
 * @param {?proto.ubo.v1.RgbRingSetIsBusyAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingSetIsBusyAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 138, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingSetIsBusyAction = function() {
  return this.setRgbRingSetIsBusyAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingSetIsBusyAction = function() {
  return jspb.Message.getField(this, 138) != null;
};


/**
 * optional RgbRingSpinningWheelAction rgb_ring_spinning_wheel_action = 139;
 * @return {?proto.ubo.v1.RgbRingSpinningWheelAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingSpinningWheelAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingSpinningWheelAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingSpinningWheelAction, 139));
};


/**
 * @param {?proto.ubo.v1.RgbRingSpinningWheelAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingSpinningWheelAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 139, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingSpinningWheelAction = function() {
  return this.setRgbRingSpinningWheelAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingSpinningWheelAction = function() {
  return jspb.Message.getField(this, 139) != null;
};


/**
 * optional RgbRingWaitableCommandAction rgb_ring_waitable_command_action = 140;
 * @return {?proto.ubo.v1.RgbRingWaitableCommandAction}
 */
proto.ubo.v1.Action.prototype.getRgbRingWaitableCommandAction = function() {
  return /** @type{?proto.ubo.v1.RgbRingWaitableCommandAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingWaitableCommandAction, 140));
};


/**
 * @param {?proto.ubo.v1.RgbRingWaitableCommandAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setRgbRingWaitableCommandAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 140, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearRgbRingWaitableCommandAction = function() {
  return this.setRgbRingWaitableCommandAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasRgbRingWaitableCommandAction = function() {
  return jspb.Message.getField(this, 140) != null;
};


/**
 * optional SSHAction ssh_action = 141;
 * @return {?proto.ubo.v1.SSHAction}
 */
proto.ubo.v1.Action.prototype.getSshAction = function() {
  return /** @type{?proto.ubo.v1.SSHAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SSHAction, 141));
};


/**
 * @param {?proto.ubo.v1.SSHAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSshAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 141, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSshAction = function() {
  return this.setSshAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSshAction = function() {
  return jspb.Message.getField(this, 141) != null;
};


/**
 * optional SSHClearEnabledStateAction ssh_clear_enabled_state_action = 142;
 * @return {?proto.ubo.v1.SSHClearEnabledStateAction}
 */
proto.ubo.v1.Action.prototype.getSshClearEnabledStateAction = function() {
  return /** @type{?proto.ubo.v1.SSHClearEnabledStateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SSHClearEnabledStateAction, 142));
};


/**
 * @param {?proto.ubo.v1.SSHClearEnabledStateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSshClearEnabledStateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 142, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSshClearEnabledStateAction = function() {
  return this.setSshClearEnabledStateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSshClearEnabledStateAction = function() {
  return jspb.Message.getField(this, 142) != null;
};


/**
 * optional SSHUpdateStateAction ssh_update_state_action = 143;
 * @return {?proto.ubo.v1.SSHUpdateStateAction}
 */
proto.ubo.v1.Action.prototype.getSshUpdateStateAction = function() {
  return /** @type{?proto.ubo.v1.SSHUpdateStateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SSHUpdateStateAction, 143));
};


/**
 * @param {?proto.ubo.v1.SSHUpdateStateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSshUpdateStateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 143, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSshUpdateStateAction = function() {
  return this.setSshUpdateStateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSshUpdateStateAction = function() {
  return jspb.Message.getField(this, 143) != null;
};


/**
 * optional SensorsAction sensors_action = 144;
 * @return {?proto.ubo.v1.SensorsAction}
 */
proto.ubo.v1.Action.prototype.getSensorsAction = function() {
  return /** @type{?proto.ubo.v1.SensorsAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorsAction, 144));
};


/**
 * @param {?proto.ubo.v1.SensorsAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSensorsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 144, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSensorsAction = function() {
  return this.setSensorsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSensorsAction = function() {
  return jspb.Message.getField(this, 144) != null;
};


/**
 * optional SensorsReportReadingAction sensors_report_reading_action = 145;
 * @return {?proto.ubo.v1.SensorsReportReadingAction}
 */
proto.ubo.v1.Action.prototype.getSensorsReportReadingAction = function() {
  return /** @type{?proto.ubo.v1.SensorsReportReadingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SensorsReportReadingAction, 145));
};


/**
 * @param {?proto.ubo.v1.SensorsReportReadingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSensorsReportReadingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 145, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSensorsReportReadingAction = function() {
  return this.setSensorsReportReadingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSensorsReportReadingAction = function() {
  return jspb.Message.getField(this, 145) != null;
};


/**
 * optional SetAreEnclosuresVisibleAction set_are_enclosures_visible_action = 146;
 * @return {?proto.ubo.v1.SetAreEnclosuresVisibleAction}
 */
proto.ubo.v1.Action.prototype.getSetAreEnclosuresVisibleAction = function() {
  return /** @type{?proto.ubo.v1.SetAreEnclosuresVisibleAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SetAreEnclosuresVisibleAction, 146));
};


/**
 * @param {?proto.ubo.v1.SetAreEnclosuresVisibleAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSetAreEnclosuresVisibleAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 146, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSetAreEnclosuresVisibleAction = function() {
  return this.setSetAreEnclosuresVisibleAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSetAreEnclosuresVisibleAction = function() {
  return jspb.Message.getField(this, 146) != null;
};


/**
 * optional SetMenuPathAction set_menu_path_action = 147;
 * @return {?proto.ubo.v1.SetMenuPathAction}
 */
proto.ubo.v1.Action.prototype.getSetMenuPathAction = function() {
  return /** @type{?proto.ubo.v1.SetMenuPathAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SetMenuPathAction, 147));
};


/**
 * @param {?proto.ubo.v1.SetMenuPathAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSetMenuPathAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 147, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSetMenuPathAction = function() {
  return this.setSetMenuPathAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSetMenuPathAction = function() {
  return jspb.Message.getField(this, 147) != null;
};


/**
 * optional SettingsAction settings_action = 148;
 * @return {?proto.ubo.v1.SettingsAction}
 */
proto.ubo.v1.Action.prototype.getSettingsAction = function() {
  return /** @type{?proto.ubo.v1.SettingsAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsAction, 148));
};


/**
 * @param {?proto.ubo.v1.SettingsAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 148, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsAction = function() {
  return this.setSettingsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsAction = function() {
  return jspb.Message.getField(this, 148) != null;
};


/**
 * optional SettingsClearServiceErrorsAction settings_clear_service_errors_action = 149;
 * @return {?proto.ubo.v1.SettingsClearServiceErrorsAction}
 */
proto.ubo.v1.Action.prototype.getSettingsClearServiceErrorsAction = function() {
  return /** @type{?proto.ubo.v1.SettingsClearServiceErrorsAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsClearServiceErrorsAction, 149));
};


/**
 * @param {?proto.ubo.v1.SettingsClearServiceErrorsAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsClearServiceErrorsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 149, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsClearServiceErrorsAction = function() {
  return this.setSettingsClearServiceErrorsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsClearServiceErrorsAction = function() {
  return jspb.Message.getField(this, 149) != null;
};


/**
 * optional SettingsReportServiceErrorAction settings_report_service_error_action = 150;
 * @return {?proto.ubo.v1.SettingsReportServiceErrorAction}
 */
proto.ubo.v1.Action.prototype.getSettingsReportServiceErrorAction = function() {
  return /** @type{?proto.ubo.v1.SettingsReportServiceErrorAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsReportServiceErrorAction, 150));
};


/**
 * @param {?proto.ubo.v1.SettingsReportServiceErrorAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsReportServiceErrorAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 150, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsReportServiceErrorAction = function() {
  return this.setSettingsReportServiceErrorAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsReportServiceErrorAction = function() {
  return jspb.Message.getField(this, 150) != null;
};


/**
 * optional SettingsServiceAction settings_service_action = 151;
 * @return {?proto.ubo.v1.SettingsServiceAction}
 */
proto.ubo.v1.Action.prototype.getSettingsServiceAction = function() {
  return /** @type{?proto.ubo.v1.SettingsServiceAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceAction, 151));
};


/**
 * @param {?proto.ubo.v1.SettingsServiceAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsServiceAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 151, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsServiceAction = function() {
  return this.setSettingsServiceAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsServiceAction = function() {
  return jspb.Message.getField(this, 151) != null;
};


/**
 * optional SettingsServiceSetIsEnabledAction settings_service_set_is_enabled_action = 152;
 * @return {?proto.ubo.v1.SettingsServiceSetIsEnabledAction}
 */
proto.ubo.v1.Action.prototype.getSettingsServiceSetIsEnabledAction = function() {
  return /** @type{?proto.ubo.v1.SettingsServiceSetIsEnabledAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetIsEnabledAction, 152));
};


/**
 * @param {?proto.ubo.v1.SettingsServiceSetIsEnabledAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsServiceSetIsEnabledAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 152, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsServiceSetIsEnabledAction = function() {
  return this.setSettingsServiceSetIsEnabledAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsServiceSetIsEnabledAction = function() {
  return jspb.Message.getField(this, 152) != null;
};


/**
 * optional SettingsServiceSetLogLevelAction settings_service_set_log_level_action = 153;
 * @return {?proto.ubo.v1.SettingsServiceSetLogLevelAction}
 */
proto.ubo.v1.Action.prototype.getSettingsServiceSetLogLevelAction = function() {
  return /** @type{?proto.ubo.v1.SettingsServiceSetLogLevelAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetLogLevelAction, 153));
};


/**
 * @param {?proto.ubo.v1.SettingsServiceSetLogLevelAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsServiceSetLogLevelAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 153, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsServiceSetLogLevelAction = function() {
  return this.setSettingsServiceSetLogLevelAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsServiceSetLogLevelAction = function() {
  return jspb.Message.getField(this, 153) != null;
};


/**
 * optional SettingsServiceSetShouldRestartAction settings_service_set_should_restart_action = 154;
 * @return {?proto.ubo.v1.SettingsServiceSetShouldRestartAction}
 */
proto.ubo.v1.Action.prototype.getSettingsServiceSetShouldRestartAction = function() {
  return /** @type{?proto.ubo.v1.SettingsServiceSetShouldRestartAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetShouldRestartAction, 154));
};


/**
 * @param {?proto.ubo.v1.SettingsServiceSetShouldRestartAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsServiceSetShouldRestartAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 154, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsServiceSetShouldRestartAction = function() {
  return this.setSettingsServiceSetShouldRestartAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsServiceSetShouldRestartAction = function() {
  return jspb.Message.getField(this, 154) != null;
};


/**
 * optional SettingsServiceSetStatusAction settings_service_set_status_action = 155;
 * @return {?proto.ubo.v1.SettingsServiceSetStatusAction}
 */
proto.ubo.v1.Action.prototype.getSettingsServiceSetStatusAction = function() {
  return /** @type{?proto.ubo.v1.SettingsServiceSetStatusAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceSetStatusAction, 155));
};


/**
 * @param {?proto.ubo.v1.SettingsServiceSetStatusAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsServiceSetStatusAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 155, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsServiceSetStatusAction = function() {
  return this.setSettingsServiceSetStatusAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsServiceSetStatusAction = function() {
  return jspb.Message.getField(this, 155) != null;
};


/**
 * optional SettingsSetServicesAction settings_set_services_action = 156;
 * @return {?proto.ubo.v1.SettingsSetServicesAction}
 */
proto.ubo.v1.Action.prototype.getSettingsSetServicesAction = function() {
  return /** @type{?proto.ubo.v1.SettingsSetServicesAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsSetServicesAction, 156));
};


/**
 * @param {?proto.ubo.v1.SettingsSetServicesAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsSetServicesAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 156, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsSetServicesAction = function() {
  return this.setSettingsSetServicesAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsSetServicesAction = function() {
  return jspb.Message.getField(this, 156) != null;
};


/**
 * optional SettingsStartServiceAction settings_start_service_action = 157;
 * @return {?proto.ubo.v1.SettingsStartServiceAction}
 */
proto.ubo.v1.Action.prototype.getSettingsStartServiceAction = function() {
  return /** @type{?proto.ubo.v1.SettingsStartServiceAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStartServiceAction, 157));
};


/**
 * @param {?proto.ubo.v1.SettingsStartServiceAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsStartServiceAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 157, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsStartServiceAction = function() {
  return this.setSettingsStartServiceAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsStartServiceAction = function() {
  return jspb.Message.getField(this, 157) != null;
};


/**
 * optional SettingsStopServiceAction settings_stop_service_action = 158;
 * @return {?proto.ubo.v1.SettingsStopServiceAction}
 */
proto.ubo.v1.Action.prototype.getSettingsStopServiceAction = function() {
  return /** @type{?proto.ubo.v1.SettingsStopServiceAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStopServiceAction, 158));
};


/**
 * @param {?proto.ubo.v1.SettingsStopServiceAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsStopServiceAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 158, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsStopServiceAction = function() {
  return this.setSettingsStopServiceAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsStopServiceAction = function() {
  return jspb.Message.getField(this, 158) != null;
};


/**
 * optional SettingsToggleBetaVersionsAction settings_toggle_beta_versions_action = 159;
 * @return {?proto.ubo.v1.SettingsToggleBetaVersionsAction}
 */
proto.ubo.v1.Action.prototype.getSettingsToggleBetaVersionsAction = function() {
  return /** @type{?proto.ubo.v1.SettingsToggleBetaVersionsAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsToggleBetaVersionsAction, 159));
};


/**
 * @param {?proto.ubo.v1.SettingsToggleBetaVersionsAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsToggleBetaVersionsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 159, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsToggleBetaVersionsAction = function() {
  return this.setSettingsToggleBetaVersionsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsToggleBetaVersionsAction = function() {
  return jspb.Message.getField(this, 159) != null;
};


/**
 * optional SettingsTogglePdbSignalAction settings_toggle_pdb_signal_action = 160;
 * @return {?proto.ubo.v1.SettingsTogglePdbSignalAction}
 */
proto.ubo.v1.Action.prototype.getSettingsTogglePdbSignalAction = function() {
  return /** @type{?proto.ubo.v1.SettingsTogglePdbSignalAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsTogglePdbSignalAction, 160));
};


/**
 * @param {?proto.ubo.v1.SettingsTogglePdbSignalAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsTogglePdbSignalAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 160, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsTogglePdbSignalAction = function() {
  return this.setSettingsTogglePdbSignalAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsTogglePdbSignalAction = function() {
  return jspb.Message.getField(this, 160) != null;
};


/**
 * optional SettingsToggleVisualDebugAction settings_toggle_visual_debug_action = 161;
 * @return {?proto.ubo.v1.SettingsToggleVisualDebugAction}
 */
proto.ubo.v1.Action.prototype.getSettingsToggleVisualDebugAction = function() {
  return /** @type{?proto.ubo.v1.SettingsToggleVisualDebugAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsToggleVisualDebugAction, 161));
};


/**
 * @param {?proto.ubo.v1.SettingsToggleVisualDebugAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSettingsToggleVisualDebugAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 161, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSettingsToggleVisualDebugAction = function() {
  return this.setSettingsToggleVisualDebugAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSettingsToggleVisualDebugAction = function() {
  return jspb.Message.getField(this, 161) != null;
};


/**
 * optional SpeechRecognitionAction speech_recognition_action = 162;
 * @return {?proto.ubo.v1.SpeechRecognitionAction}
 */
proto.ubo.v1.Action.prototype.getSpeechRecognitionAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionAction, 162));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechRecognitionAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 162, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechRecognitionAction = function() {
  return this.setSpeechRecognitionAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechRecognitionAction = function() {
  return jspb.Message.getField(this, 162) != null;
};


/**
 * optional SpeechRecognitionReportIntentDetectionAction speech_recognition_report_intent_detection_action = 163;
 * @return {?proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction}
 */
proto.ubo.v1.Action.prototype.getSpeechRecognitionReportIntentDetectionAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction, 163));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionReportIntentDetectionAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechRecognitionReportIntentDetectionAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 163, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechRecognitionReportIntentDetectionAction = function() {
  return this.setSpeechRecognitionReportIntentDetectionAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechRecognitionReportIntentDetectionAction = function() {
  return jspb.Message.getField(this, 163) != null;
};


/**
 * optional SpeechRecognitionReportSpeechAction speech_recognition_report_speech_action = 164;
 * @return {?proto.ubo.v1.SpeechRecognitionReportSpeechAction}
 */
proto.ubo.v1.Action.prototype.getSpeechRecognitionReportSpeechAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionReportSpeechAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportSpeechAction, 164));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionReportSpeechAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechRecognitionReportSpeechAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 164, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechRecognitionReportSpeechAction = function() {
  return this.setSpeechRecognitionReportSpeechAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechRecognitionReportSpeechAction = function() {
  return jspb.Message.getField(this, 164) != null;
};


/**
 * optional SpeechRecognitionReportWakeWordDetectionAction speech_recognition_report_wake_word_detection_action = 165;
 * @return {?proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction}
 */
proto.ubo.v1.Action.prototype.getSpeechRecognitionReportWakeWordDetectionAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction, 165));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionReportWakeWordDetectionAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechRecognitionReportWakeWordDetectionAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 165, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechRecognitionReportWakeWordDetectionAction = function() {
  return this.setSpeechRecognitionReportWakeWordDetectionAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechRecognitionReportWakeWordDetectionAction = function() {
  return jspb.Message.getField(this, 165) != null;
};


/**
 * optional SpeechRecognitionSetIsAssistantActiveAction speech_recognition_set_is_assistant_active_action = 166;
 * @return {?proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction}
 */
proto.ubo.v1.Action.prototype.getSpeechRecognitionSetIsAssistantActiveAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction, 166));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionSetIsAssistantActiveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechRecognitionSetIsAssistantActiveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 166, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechRecognitionSetIsAssistantActiveAction = function() {
  return this.setSpeechRecognitionSetIsAssistantActiveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechRecognitionSetIsAssistantActiveAction = function() {
  return jspb.Message.getField(this, 166) != null;
};


/**
 * optional SpeechRecognitionSetIsIntentsActiveAction speech_recognition_set_is_intents_active_action = 167;
 * @return {?proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction}
 */
proto.ubo.v1.Action.prototype.getSpeechRecognitionSetIsIntentsActiveAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction, 167));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionSetIsIntentsActiveAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechRecognitionSetIsIntentsActiveAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 167, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechRecognitionSetIsIntentsActiveAction = function() {
  return this.setSpeechRecognitionSetIsIntentsActiveAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechRecognitionSetIsIntentsActiveAction = function() {
  return jspb.Message.getField(this, 167) != null;
};


/**
 * optional SpeechRecognitionSetSelectedEngineAction speech_recognition_set_selected_engine_action = 168;
 * @return {?proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction}
 */
proto.ubo.v1.Action.prototype.getSpeechRecognitionSetSelectedEngineAction = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction, 168));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionSetSelectedEngineAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechRecognitionSetSelectedEngineAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 168, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechRecognitionSetSelectedEngineAction = function() {
  return this.setSpeechRecognitionSetSelectedEngineAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechRecognitionSetSelectedEngineAction = function() {
  return jspb.Message.getField(this, 168) != null;
};


/**
 * optional SpeechSynthesisAction speech_synthesis_action = 169;
 * @return {?proto.ubo.v1.SpeechSynthesisAction}
 */
proto.ubo.v1.Action.prototype.getSpeechSynthesisAction = function() {
  return /** @type{?proto.ubo.v1.SpeechSynthesisAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisAction, 169));
};


/**
 * @param {?proto.ubo.v1.SpeechSynthesisAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechSynthesisAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 169, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechSynthesisAction = function() {
  return this.setSpeechSynthesisAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechSynthesisAction = function() {
  return jspb.Message.getField(this, 169) != null;
};


/**
 * optional SpeechSynthesisReadTextAction speech_synthesis_read_text_action = 170;
 * @return {?proto.ubo.v1.SpeechSynthesisReadTextAction}
 */
proto.ubo.v1.Action.prototype.getSpeechSynthesisReadTextAction = function() {
  return /** @type{?proto.ubo.v1.SpeechSynthesisReadTextAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisReadTextAction, 170));
};


/**
 * @param {?proto.ubo.v1.SpeechSynthesisReadTextAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechSynthesisReadTextAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 170, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechSynthesisReadTextAction = function() {
  return this.setSpeechSynthesisReadTextAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechSynthesisReadTextAction = function() {
  return jspb.Message.getField(this, 170) != null;
};


/**
 * optional SpeechSynthesisSetSelectedEngineAction speech_synthesis_set_selected_engine_action = 171;
 * @return {?proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction}
 */
proto.ubo.v1.Action.prototype.getSpeechSynthesisSetSelectedEngineAction = function() {
  return /** @type{?proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction, 171));
};


/**
 * @param {?proto.ubo.v1.SpeechSynthesisSetSelectedEngineAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setSpeechSynthesisSetSelectedEngineAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 171, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearSpeechSynthesisSetSelectedEngineAction = function() {
  return this.setSpeechSynthesisSetSelectedEngineAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasSpeechSynthesisSetSelectedEngineAction = function() {
  return jspb.Message.getField(this, 171) != null;
};


/**
 * optional StatusIconsAction status_icons_action = 172;
 * @return {?proto.ubo.v1.StatusIconsAction}
 */
proto.ubo.v1.Action.prototype.getStatusIconsAction = function() {
  return /** @type{?proto.ubo.v1.StatusIconsAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.StatusIconsAction, 172));
};


/**
 * @param {?proto.ubo.v1.StatusIconsAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setStatusIconsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 172, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearStatusIconsAction = function() {
  return this.setStatusIconsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasStatusIconsAction = function() {
  return jspb.Message.getField(this, 172) != null;
};


/**
 * optional StatusIconsRegisterAction status_icons_register_action = 173;
 * @return {?proto.ubo.v1.StatusIconsRegisterAction}
 */
proto.ubo.v1.Action.prototype.getStatusIconsRegisterAction = function() {
  return /** @type{?proto.ubo.v1.StatusIconsRegisterAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.StatusIconsRegisterAction, 173));
};


/**
 * @param {?proto.ubo.v1.StatusIconsRegisterAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setStatusIconsRegisterAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 173, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearStatusIconsRegisterAction = function() {
  return this.setStatusIconsRegisterAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasStatusIconsRegisterAction = function() {
  return jspb.Message.getField(this, 173) != null;
};


/**
 * optional ToggleRecordingAction toggle_recording_action = 174;
 * @return {?proto.ubo.v1.ToggleRecordingAction}
 */
proto.ubo.v1.Action.prototype.getToggleRecordingAction = function() {
  return /** @type{?proto.ubo.v1.ToggleRecordingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ToggleRecordingAction, 174));
};


/**
 * @param {?proto.ubo.v1.ToggleRecordingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setToggleRecordingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 174, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearToggleRecordingAction = function() {
  return this.setToggleRecordingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasToggleRecordingAction = function() {
  return jspb.Message.getField(this, 174) != null;
};


/**
 * optional UpdateManagerAction update_manager_action = 175;
 * @return {?proto.ubo.v1.UpdateManagerAction}
 */
proto.ubo.v1.Action.prototype.getUpdateManagerAction = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerAction, 175));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUpdateManagerAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 175, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUpdateManagerAction = function() {
  return this.setUpdateManagerAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUpdateManagerAction = function() {
  return jspb.Message.getField(this, 175) != null;
};


/**
 * optional UpdateManagerReportFailedCheckAction update_manager_report_failed_check_action = 176;
 * @return {?proto.ubo.v1.UpdateManagerReportFailedCheckAction}
 */
proto.ubo.v1.Action.prototype.getUpdateManagerReportFailedCheckAction = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerReportFailedCheckAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerReportFailedCheckAction, 176));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerReportFailedCheckAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUpdateManagerReportFailedCheckAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 176, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUpdateManagerReportFailedCheckAction = function() {
  return this.setUpdateManagerReportFailedCheckAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUpdateManagerReportFailedCheckAction = function() {
  return jspb.Message.getField(this, 176) != null;
};


/**
 * optional UpdateManagerRequestCheckAction update_manager_request_check_action = 177;
 * @return {?proto.ubo.v1.UpdateManagerRequestCheckAction}
 */
proto.ubo.v1.Action.prototype.getUpdateManagerRequestCheckAction = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerRequestCheckAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerRequestCheckAction, 177));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerRequestCheckAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUpdateManagerRequestCheckAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 177, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUpdateManagerRequestCheckAction = function() {
  return this.setUpdateManagerRequestCheckAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUpdateManagerRequestCheckAction = function() {
  return jspb.Message.getField(this, 177) != null;
};


/**
 * optional UpdateManagerRequestUpdateAction update_manager_request_update_action = 178;
 * @return {?proto.ubo.v1.UpdateManagerRequestUpdateAction}
 */
proto.ubo.v1.Action.prototype.getUpdateManagerRequestUpdateAction = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerRequestUpdateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerRequestUpdateAction, 178));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerRequestUpdateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUpdateManagerRequestUpdateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 178, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUpdateManagerRequestUpdateAction = function() {
  return this.setUpdateManagerRequestUpdateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUpdateManagerRequestUpdateAction = function() {
  return jspb.Message.getField(this, 178) != null;
};


/**
 * optional UpdateManagerSetVersionsAction update_manager_set_versions_action = 179;
 * @return {?proto.ubo.v1.UpdateManagerSetVersionsAction}
 */
proto.ubo.v1.Action.prototype.getUpdateManagerSetVersionsAction = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerSetVersionsAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerSetVersionsAction, 179));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerSetVersionsAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUpdateManagerSetVersionsAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 179, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUpdateManagerSetVersionsAction = function() {
  return this.setUpdateManagerSetVersionsAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUpdateManagerSetVersionsAction = function() {
  return jspb.Message.getField(this, 179) != null;
};


/**
 * optional UsersAction users_action = 180;
 * @return {?proto.ubo.v1.UsersAction}
 */
proto.ubo.v1.Action.prototype.getUsersAction = function() {
  return /** @type{?proto.ubo.v1.UsersAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersAction, 180));
};


/**
 * @param {?proto.ubo.v1.UsersAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUsersAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 180, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUsersAction = function() {
  return this.setUsersAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUsersAction = function() {
  return jspb.Message.getField(this, 180) != null;
};


/**
 * optional UsersCreateUserAction users_create_user_action = 181;
 * @return {?proto.ubo.v1.UsersCreateUserAction}
 */
proto.ubo.v1.Action.prototype.getUsersCreateUserAction = function() {
  return /** @type{?proto.ubo.v1.UsersCreateUserAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersCreateUserAction, 181));
};


/**
 * @param {?proto.ubo.v1.UsersCreateUserAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUsersCreateUserAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 181, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUsersCreateUserAction = function() {
  return this.setUsersCreateUserAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUsersCreateUserAction = function() {
  return jspb.Message.getField(this, 181) != null;
};


/**
 * optional UsersDeleteUserAction users_delete_user_action = 182;
 * @return {?proto.ubo.v1.UsersDeleteUserAction}
 */
proto.ubo.v1.Action.prototype.getUsersDeleteUserAction = function() {
  return /** @type{?proto.ubo.v1.UsersDeleteUserAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersDeleteUserAction, 182));
};


/**
 * @param {?proto.ubo.v1.UsersDeleteUserAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUsersDeleteUserAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 182, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUsersDeleteUserAction = function() {
  return this.setUsersDeleteUserAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUsersDeleteUserAction = function() {
  return jspb.Message.getField(this, 182) != null;
};


/**
 * optional UsersResetPasswordAction users_reset_password_action = 183;
 * @return {?proto.ubo.v1.UsersResetPasswordAction}
 */
proto.ubo.v1.Action.prototype.getUsersResetPasswordAction = function() {
  return /** @type{?proto.ubo.v1.UsersResetPasswordAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersResetPasswordAction, 183));
};


/**
 * @param {?proto.ubo.v1.UsersResetPasswordAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUsersResetPasswordAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 183, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUsersResetPasswordAction = function() {
  return this.setUsersResetPasswordAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUsersResetPasswordAction = function() {
  return jspb.Message.getField(this, 183) != null;
};


/**
 * optional UsersSetUsersAction users_set_users_action = 184;
 * @return {?proto.ubo.v1.UsersSetUsersAction}
 */
proto.ubo.v1.Action.prototype.getUsersSetUsersAction = function() {
  return /** @type{?proto.ubo.v1.UsersSetUsersAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersSetUsersAction, 184));
};


/**
 * @param {?proto.ubo.v1.UsersSetUsersAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setUsersSetUsersAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 184, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearUsersSetUsersAction = function() {
  return this.setUsersSetUsersAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasUsersSetUsersAction = function() {
  return jspb.Message.getField(this, 184) != null;
};


/**
 * optional VSCodeAction vs_code_action = 185;
 * @return {?proto.ubo.v1.VSCodeAction}
 */
proto.ubo.v1.Action.prototype.getVsCodeAction = function() {
  return /** @type{?proto.ubo.v1.VSCodeAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeAction, 185));
};


/**
 * @param {?proto.ubo.v1.VSCodeAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setVsCodeAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 185, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearVsCodeAction = function() {
  return this.setVsCodeAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasVsCodeAction = function() {
  return jspb.Message.getField(this, 185) != null;
};


/**
 * optional VSCodeDoneDownloadingAction vs_code_done_downloading_action = 186;
 * @return {?proto.ubo.v1.VSCodeDoneDownloadingAction}
 */
proto.ubo.v1.Action.prototype.getVsCodeDoneDownloadingAction = function() {
  return /** @type{?proto.ubo.v1.VSCodeDoneDownloadingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeDoneDownloadingAction, 186));
};


/**
 * @param {?proto.ubo.v1.VSCodeDoneDownloadingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setVsCodeDoneDownloadingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 186, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearVsCodeDoneDownloadingAction = function() {
  return this.setVsCodeDoneDownloadingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasVsCodeDoneDownloadingAction = function() {
  return jspb.Message.getField(this, 186) != null;
};


/**
 * optional VSCodeSetPendingAction vs_code_set_pending_action = 187;
 * @return {?proto.ubo.v1.VSCodeSetPendingAction}
 */
proto.ubo.v1.Action.prototype.getVsCodeSetPendingAction = function() {
  return /** @type{?proto.ubo.v1.VSCodeSetPendingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeSetPendingAction, 187));
};


/**
 * @param {?proto.ubo.v1.VSCodeSetPendingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setVsCodeSetPendingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 187, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearVsCodeSetPendingAction = function() {
  return this.setVsCodeSetPendingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasVsCodeSetPendingAction = function() {
  return jspb.Message.getField(this, 187) != null;
};


/**
 * optional VSCodeSetStatusAction vs_code_set_status_action = 188;
 * @return {?proto.ubo.v1.VSCodeSetStatusAction}
 */
proto.ubo.v1.Action.prototype.getVsCodeSetStatusAction = function() {
  return /** @type{?proto.ubo.v1.VSCodeSetStatusAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeSetStatusAction, 188));
};


/**
 * @param {?proto.ubo.v1.VSCodeSetStatusAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setVsCodeSetStatusAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 188, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearVsCodeSetStatusAction = function() {
  return this.setVsCodeSetStatusAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasVsCodeSetStatusAction = function() {
  return jspb.Message.getField(this, 188) != null;
};


/**
 * optional VSCodeStartDownloadingAction vs_code_start_downloading_action = 189;
 * @return {?proto.ubo.v1.VSCodeStartDownloadingAction}
 */
proto.ubo.v1.Action.prototype.getVsCodeStartDownloadingAction = function() {
  return /** @type{?proto.ubo.v1.VSCodeStartDownloadingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeStartDownloadingAction, 189));
};


/**
 * @param {?proto.ubo.v1.VSCodeStartDownloadingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setVsCodeStartDownloadingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 189, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearVsCodeStartDownloadingAction = function() {
  return this.setVsCodeStartDownloadingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasVsCodeStartDownloadingAction = function() {
  return jspb.Message.getField(this, 189) != null;
};


/**
 * optional WiFiAction wi_fi_action = 190;
 * @return {?proto.ubo.v1.WiFiAction}
 */
proto.ubo.v1.Action.prototype.getWiFiAction = function() {
  return /** @type{?proto.ubo.v1.WiFiAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiAction, 190));
};


/**
 * @param {?proto.ubo.v1.WiFiAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setWiFiAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 190, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearWiFiAction = function() {
  return this.setWiFiAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasWiFiAction = function() {
  return jspb.Message.getField(this, 190) != null;
};


/**
 * optional WiFiInputConnectionAction wi_fi_input_connection_action = 191;
 * @return {?proto.ubo.v1.WiFiInputConnectionAction}
 */
proto.ubo.v1.Action.prototype.getWiFiInputConnectionAction = function() {
  return /** @type{?proto.ubo.v1.WiFiInputConnectionAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiInputConnectionAction, 191));
};


/**
 * @param {?proto.ubo.v1.WiFiInputConnectionAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setWiFiInputConnectionAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 191, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearWiFiInputConnectionAction = function() {
  return this.setWiFiInputConnectionAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasWiFiInputConnectionAction = function() {
  return jspb.Message.getField(this, 191) != null;
};


/**
 * optional WiFiSetHasVisitedOnboardingAction wi_fi_set_has_visited_onboarding_action = 192;
 * @return {?proto.ubo.v1.WiFiSetHasVisitedOnboardingAction}
 */
proto.ubo.v1.Action.prototype.getWiFiSetHasVisitedOnboardingAction = function() {
  return /** @type{?proto.ubo.v1.WiFiSetHasVisitedOnboardingAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiSetHasVisitedOnboardingAction, 192));
};


/**
 * @param {?proto.ubo.v1.WiFiSetHasVisitedOnboardingAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setWiFiSetHasVisitedOnboardingAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 192, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearWiFiSetHasVisitedOnboardingAction = function() {
  return this.setWiFiSetHasVisitedOnboardingAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasWiFiSetHasVisitedOnboardingAction = function() {
  return jspb.Message.getField(this, 192) != null;
};


/**
 * optional WiFiUpdateAction wi_fi_update_action = 193;
 * @return {?proto.ubo.v1.WiFiUpdateAction}
 */
proto.ubo.v1.Action.prototype.getWiFiUpdateAction = function() {
  return /** @type{?proto.ubo.v1.WiFiUpdateAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiUpdateAction, 193));
};


/**
 * @param {?proto.ubo.v1.WiFiUpdateAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setWiFiUpdateAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 193, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearWiFiUpdateAction = function() {
  return this.setWiFiUpdateAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasWiFiUpdateAction = function() {
  return jspb.Message.getField(this, 193) != null;
};


/**
 * optional WiFiUpdateRequestAction wi_fi_update_request_action = 194;
 * @return {?proto.ubo.v1.WiFiUpdateRequestAction}
 */
proto.ubo.v1.Action.prototype.getWiFiUpdateRequestAction = function() {
  return /** @type{?proto.ubo.v1.WiFiUpdateRequestAction} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiUpdateRequestAction, 194));
};


/**
 * @param {?proto.ubo.v1.WiFiUpdateRequestAction|undefined} value
 * @return {!proto.ubo.v1.Action} returns this
*/
proto.ubo.v1.Action.prototype.setWiFiUpdateRequestAction = function(value) {
  return jspb.Message.setOneofWrapperField(this, 194, proto.ubo.v1.Action.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Action} returns this
 */
proto.ubo.v1.Action.prototype.clearWiFiUpdateRequestAction = function() {
  return this.setWiFiUpdateRequestAction(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Action.prototype.hasWiFiUpdateRequestAction = function() {
  return jspb.Message.getField(this, 194) != null;
};



/**
 * Oneof group definitions for this message. Each group defines the field
 * numbers belonging to that group. When of these fields' value is set, all
 * other fields in the group are cleared. During deserialization, if multiple
 * fields are encountered for a group, only the last value seen will be kept.
 * @private {!Array<!Array<number>>}
 * @const
 */
proto.ubo.v1.Event.oneofGroups_ = [[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104]];

/**
 * @enum {number}
 */
proto.ubo.v1.Event.EventCase = {
  EVENT_NOT_SET: 0,
  ASSISTANT_ADD_MCP_SERVER_EVENT: 1,
  ASSISTANT_DELETE_MCP_SERVER_EVENT: 2,
  ASSISTANT_DOWNLOAD_OLLAMA_MODEL_EVENT: 3,
  ASSISTANT_EVENT: 4,
  ASSISTANT_HANDLE_REPORT_EVENT: 5,
  ASSISTANT_UPDATE_PROVIDERS_EVENT: 6,
  AUDIO_EVENT: 7,
  AUDIO_INSTALL_DRIVER_EVENT: 8,
  AUDIO_PLAY_AUDIO_SAMPLE_EVENT: 9,
  AUDIO_PLAY_AUDIO_SEQUENCE_EVENT: 10,
  AUDIO_PLAY_CHIME_EVENT: 11,
  AUDIO_PLAYBACK_DONE_EVENT: 12,
  AUDIO_REPORT_SAMPLE_EVENT: 13,
  CAMERA_DETECT_EVENT: 14,
  CAMERA_DETECTED_EVENT: 15,
  CAMERA_EVENT: 16,
  CAMERA_INSTALL_DRIVER_EVENT: 17,
  CAMERA_REINITIALIZE_EVENT: 18,
  CAMERA_REPORT_IMAGE_EVENT: 19,
  CAMERA_RESTORE_DEFAULT_EVENT: 20,
  CAMERA_START_VIEWFINDER_EVENT: 21,
  CAMERA_STOP_VIEWFINDER_EVENT: 22,
  CLOSE_APPLICATION_EVENT: 23,
  DISPLAY_BLANK_EVENT: 24,
  DISPLAY_COMPRESSED_RENDER_EVENT: 25,
  DISPLAY_EVENT: 26,
  DISPLAY_REDRAW_EVENT: 27,
  DISPLAY_RENDER_EVENT: 28,
  DISPLAY_UNBLANK_EVENT: 29,
  DOCKER_EVENT: 30,
  DOCKER_IMAGE_EVENT: 31,
  DOCKER_IMAGE_FETCH_COMPOSITION_EVENT: 32,
  DOCKER_IMAGE_FETCH_EVENT: 33,
  DOCKER_IMAGE_REGISTER_APP_EVENT: 34,
  DOCKER_IMAGE_RELEASE_COMPOSITION_EVENT: 35,
  DOCKER_IMAGE_REMOVE_COMPOSITION_EVENT: 36,
  DOCKER_IMAGE_REMOVE_CONTAINER_EVENT: 37,
  DOCKER_IMAGE_REMOVE_EVENT: 38,
  DOCKER_IMAGE_RUN_COMPOSITION_EVENT: 39,
  DOCKER_IMAGE_RUN_CONTAINER_EVENT: 40,
  DOCKER_IMAGE_STOP_COMPOSITION_EVENT: 41,
  DOCKER_IMAGE_STOP_CONTAINER_EVENT: 42,
  DOCKER_INSTALL_EVENT: 43,
  DOCKER_START_EVENT: 44,
  DOCKER_STOP_EVENT: 45,
  FILE_SYSTEM_COPY_EVENT: 46,
  FILE_SYSTEM_EVENT: 47,
  FILE_SYSTEM_MOVE_EVENT: 48,
  FILE_SYSTEM_REMOVE_EVENT: 49,
  FILE_SYSTEM_SELECT_EVENT: 50,
  INFRARED_EVENT: 51,
  INFRARED_SEND_CODE_EVENT: 52,
  INIT_EVENT: 53,
  INPUT_CANCEL_EVENT: 54,
  INPUT_PROVIDE_EVENT: 55,
  INPUT_RESOLVE_EVENT: 56,
  IP_EVENT: 57,
  MAIN_EVENT: 58,
  MENU_CHOOSE_BY_ICON_EVENT: 59,
  MENU_CHOOSE_BY_INDEX_EVENT: 60,
  MENU_CHOOSE_BY_LABEL_EVENT: 61,
  MENU_EVENT: 62,
  MENU_GO_BACK_EVENT: 63,
  MENU_GO_HOME_EVENT: 64,
  MENU_SCROLL_EVENT: 65,
  NOTIFICATIONS_CLEAR_EVENT: 66,
  NOTIFICATIONS_DISPLAY_EVENT: 67,
  NOTIFICATIONS_EVENT: 68,
  OPEN_APPLICATION_EVENT: 69,
  POWER_EVENT: 70,
  POWER_OFF_EVENT: 71,
  R_PI_CONNECT_EVENT: 72,
  R_PI_CONNECT_LOGIN_EVENT: 73,
  REBOOT_EVENT: 74,
  REPLAY_RECORDED_SEQUENCE_EVENT: 75,
  RGB_RING_COMMAND_EVENT: 76,
  RGB_RING_EVENT: 77,
  SCREENSHOT_EVENT: 78,
  SETTINGS_EVENT: 79,
  SETTINGS_SERVICE_EVENT: 80,
  SETTINGS_START_SERVICE_EVENT: 81,
  SETTINGS_STOP_SERVICE_EVENT: 82,
  SNAPSHOT_EVENT: 83,
  SPEECH_RECOGNITION_EVENT: 84,
  SPEECH_RECOGNITION_REPORT_TEXT_EVENT: 85,
  SPEECH_SYNTHESIS_EVENT: 86,
  SPEECH_SYNTHESIS_SYNTHESIZE_TEXT_EVENT: 87,
  STORE_RECORDED_SEQUENCE_EVENT: 88,
  UPDATE_MANAGER_CHECK_EVENT: 89,
  UPDATE_MANAGER_EVENT: 90,
  UPDATE_MANAGER_UPDATE_EVENT: 91,
  USERS_CREATE_USER_EVENT: 92,
  USERS_DELETE_USER_EVENT: 93,
  USERS_EVENT: 94,
  USERS_RESET_PASSWORD_EVENT: 95,
  VS_CODE_EVENT: 96,
  VS_CODE_LOGIN_EVENT: 97,
  VS_CODE_RESTART_EVENT: 98,
  WEB_UI_EVENT: 99,
  WEB_UI_INITIALIZE_EVENT: 100,
  WEB_UI_STOP_EVENT: 101,
  WI_FI_EVENT: 102,
  WI_FI_INPUT_CONNECTION_EVENT: 103,
  WI_FI_UPDATE_REQUEST_EVENT: 104
};

/**
 * @return {proto.ubo.v1.Event.EventCase}
 */
proto.ubo.v1.Event.prototype.getEventCase = function() {
  return /** @type {proto.ubo.v1.Event.EventCase} */(jspb.Message.computeOneofCase(this, proto.ubo.v1.Event.oneofGroups_[0]));
};



if (jspb.Message.GENERATE_TO_OBJECT) {
/**
 * Creates an object representation of this proto.
 * Field names that are reserved in JavaScript and will be renamed to pb_name.
 * Optional fields that are not set will be set to undefined.
 * To access a reserved field use, foo.pb_<name>, eg, foo.pb_default.
 * For the list of reserved names please see:
 *     net/proto2/compiler/js/internal/generator.cc#kKeyword.
 * @param {boolean=} opt_includeInstance Deprecated. whether to include the
 *     JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @return {!Object}
 */
proto.ubo.v1.Event.prototype.toObject = function(opt_includeInstance) {
  return proto.ubo.v1.Event.toObject(opt_includeInstance, this);
};


/**
 * Static version of the {@see toObject} method.
 * @param {boolean|undefined} includeInstance Deprecated. Whether to include
 *     the JSPB instance for transitional soy proto support:
 *     http://goto/soy-param-migration
 * @param {!proto.ubo.v1.Event} msg The msg instance to transform.
 * @return {!Object}
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Event.toObject = function(includeInstance, msg) {
  var f, obj = {
    assistantAddMcpServerEvent: (f = msg.getAssistantAddMcpServerEvent()) && proto.ubo.v1.AssistantAddMcpServerEvent.toObject(includeInstance, f),
    assistantDeleteMcpServerEvent: (f = msg.getAssistantDeleteMcpServerEvent()) && proto.ubo.v1.AssistantDeleteMcpServerEvent.toObject(includeInstance, f),
    assistantDownloadOllamaModelEvent: (f = msg.getAssistantDownloadOllamaModelEvent()) && proto.ubo.v1.AssistantDownloadOllamaModelEvent.toObject(includeInstance, f),
    assistantEvent: (f = msg.getAssistantEvent()) && proto.ubo.v1.AssistantEvent.toObject(includeInstance, f),
    assistantHandleReportEvent: (f = msg.getAssistantHandleReportEvent()) && proto.ubo.v1.AssistantHandleReportEvent.toObject(includeInstance, f),
    assistantUpdateProvidersEvent: (f = msg.getAssistantUpdateProvidersEvent()) && proto.ubo.v1.AssistantUpdateProvidersEvent.toObject(includeInstance, f),
    audioEvent: (f = msg.getAudioEvent()) && proto.ubo.v1.AudioEvent.toObject(includeInstance, f),
    audioInstallDriverEvent: (f = msg.getAudioInstallDriverEvent()) && proto.ubo.v1.AudioInstallDriverEvent.toObject(includeInstance, f),
    audioPlayAudioSampleEvent: (f = msg.getAudioPlayAudioSampleEvent()) && proto.ubo.v1.AudioPlayAudioSampleEvent.toObject(includeInstance, f),
    audioPlayAudioSequenceEvent: (f = msg.getAudioPlayAudioSequenceEvent()) && proto.ubo.v1.AudioPlayAudioSequenceEvent.toObject(includeInstance, f),
    audioPlayChimeEvent: (f = msg.getAudioPlayChimeEvent()) && proto.ubo.v1.AudioPlayChimeEvent.toObject(includeInstance, f),
    audioPlaybackDoneEvent: (f = msg.getAudioPlaybackDoneEvent()) && proto.ubo.v1.AudioPlaybackDoneEvent.toObject(includeInstance, f),
    audioReportSampleEvent: (f = msg.getAudioReportSampleEvent()) && proto.ubo.v1.AudioReportSampleEvent.toObject(includeInstance, f),
    cameraDetectEvent: (f = msg.getCameraDetectEvent()) && proto.ubo.v1.CameraDetectEvent.toObject(includeInstance, f),
    cameraDetectedEvent: (f = msg.getCameraDetectedEvent()) && proto.ubo.v1.CameraDetectedEvent.toObject(includeInstance, f),
    cameraEvent: (f = msg.getCameraEvent()) && proto.ubo.v1.CameraEvent.toObject(includeInstance, f),
    cameraInstallDriverEvent: (f = msg.getCameraInstallDriverEvent()) && proto.ubo.v1.CameraInstallDriverEvent.toObject(includeInstance, f),
    cameraReinitializeEvent: (f = msg.getCameraReinitializeEvent()) && proto.ubo.v1.CameraReinitializeEvent.toObject(includeInstance, f),
    cameraReportImageEvent: (f = msg.getCameraReportImageEvent()) && proto.ubo.v1.CameraReportImageEvent.toObject(includeInstance, f),
    cameraRestoreDefaultEvent: (f = msg.getCameraRestoreDefaultEvent()) && proto.ubo.v1.CameraRestoreDefaultEvent.toObject(includeInstance, f),
    cameraStartViewfinderEvent: (f = msg.getCameraStartViewfinderEvent()) && proto.ubo.v1.CameraStartViewfinderEvent.toObject(includeInstance, f),
    cameraStopViewfinderEvent: (f = msg.getCameraStopViewfinderEvent()) && proto.ubo.v1.CameraStopViewfinderEvent.toObject(includeInstance, f),
    closeApplicationEvent: (f = msg.getCloseApplicationEvent()) && proto.ubo.v1.CloseApplicationEvent.toObject(includeInstance, f),
    displayBlankEvent: (f = msg.getDisplayBlankEvent()) && proto.ubo.v1.DisplayBlankEvent.toObject(includeInstance, f),
    displayCompressedRenderEvent: (f = msg.getDisplayCompressedRenderEvent()) && proto.ubo.v1.DisplayCompressedRenderEvent.toObject(includeInstance, f),
    displayEvent: (f = msg.getDisplayEvent()) && proto.ubo.v1.DisplayEvent.toObject(includeInstance, f),
    displayRedrawEvent: (f = msg.getDisplayRedrawEvent()) && proto.ubo.v1.DisplayRedrawEvent.toObject(includeInstance, f),
    displayRenderEvent: (f = msg.getDisplayRenderEvent()) && proto.ubo.v1.DisplayRenderEvent.toObject(includeInstance, f),
    displayUnblankEvent: (f = msg.getDisplayUnblankEvent()) && proto.ubo.v1.DisplayUnblankEvent.toObject(includeInstance, f),
    dockerEvent: (f = msg.getDockerEvent()) && proto.ubo.v1.DockerEvent.toObject(includeInstance, f),
    dockerImageEvent: (f = msg.getDockerImageEvent()) && proto.ubo.v1.DockerImageEvent.toObject(includeInstance, f),
    dockerImageFetchCompositionEvent: (f = msg.getDockerImageFetchCompositionEvent()) && proto.ubo.v1.DockerImageFetchCompositionEvent.toObject(includeInstance, f),
    dockerImageFetchEvent: (f = msg.getDockerImageFetchEvent()) && proto.ubo.v1.DockerImageFetchEvent.toObject(includeInstance, f),
    dockerImageRegisterAppEvent: (f = msg.getDockerImageRegisterAppEvent()) && proto.ubo.v1.DockerImageRegisterAppEvent.toObject(includeInstance, f),
    dockerImageReleaseCompositionEvent: (f = msg.getDockerImageReleaseCompositionEvent()) && proto.ubo.v1.DockerImageReleaseCompositionEvent.toObject(includeInstance, f),
    dockerImageRemoveCompositionEvent: (f = msg.getDockerImageRemoveCompositionEvent()) && proto.ubo.v1.DockerImageRemoveCompositionEvent.toObject(includeInstance, f),
    dockerImageRemoveContainerEvent: (f = msg.getDockerImageRemoveContainerEvent()) && proto.ubo.v1.DockerImageRemoveContainerEvent.toObject(includeInstance, f),
    dockerImageRemoveEvent: (f = msg.getDockerImageRemoveEvent()) && proto.ubo.v1.DockerImageRemoveEvent.toObject(includeInstance, f),
    dockerImageRunCompositionEvent: (f = msg.getDockerImageRunCompositionEvent()) && proto.ubo.v1.DockerImageRunCompositionEvent.toObject(includeInstance, f),
    dockerImageRunContainerEvent: (f = msg.getDockerImageRunContainerEvent()) && proto.ubo.v1.DockerImageRunContainerEvent.toObject(includeInstance, f),
    dockerImageStopCompositionEvent: (f = msg.getDockerImageStopCompositionEvent()) && proto.ubo.v1.DockerImageStopCompositionEvent.toObject(includeInstance, f),
    dockerImageStopContainerEvent: (f = msg.getDockerImageStopContainerEvent()) && proto.ubo.v1.DockerImageStopContainerEvent.toObject(includeInstance, f),
    dockerInstallEvent: (f = msg.getDockerInstallEvent()) && proto.ubo.v1.DockerInstallEvent.toObject(includeInstance, f),
    dockerStartEvent: (f = msg.getDockerStartEvent()) && proto.ubo.v1.DockerStartEvent.toObject(includeInstance, f),
    dockerStopEvent: (f = msg.getDockerStopEvent()) && proto.ubo.v1.DockerStopEvent.toObject(includeInstance, f),
    fileSystemCopyEvent: (f = msg.getFileSystemCopyEvent()) && proto.ubo.v1.FileSystemCopyEvent.toObject(includeInstance, f),
    fileSystemEvent: (f = msg.getFileSystemEvent()) && proto.ubo.v1.FileSystemEvent.toObject(includeInstance, f),
    fileSystemMoveEvent: (f = msg.getFileSystemMoveEvent()) && proto.ubo.v1.FileSystemMoveEvent.toObject(includeInstance, f),
    fileSystemRemoveEvent: (f = msg.getFileSystemRemoveEvent()) && proto.ubo.v1.FileSystemRemoveEvent.toObject(includeInstance, f),
    fileSystemSelectEvent: (f = msg.getFileSystemSelectEvent()) && proto.ubo.v1.FileSystemSelectEvent.toObject(includeInstance, f),
    infraredEvent: (f = msg.getInfraredEvent()) && proto.ubo.v1.InfraredEvent.toObject(includeInstance, f),
    infraredSendCodeEvent: (f = msg.getInfraredSendCodeEvent()) && proto.ubo.v1.InfraredSendCodeEvent.toObject(includeInstance, f),
    initEvent: (f = msg.getInitEvent()) && proto.ubo.v1.InitEvent.toObject(includeInstance, f),
    inputCancelEvent: (f = msg.getInputCancelEvent()) && proto.ubo.v1.InputCancelEvent.toObject(includeInstance, f),
    inputProvideEvent: (f = msg.getInputProvideEvent()) && proto.ubo.v1.InputProvideEvent.toObject(includeInstance, f),
    inputResolveEvent: (f = msg.getInputResolveEvent()) && proto.ubo.v1.InputResolveEvent.toObject(includeInstance, f),
    ipEvent: (f = msg.getIpEvent()) && proto.ubo.v1.IpEvent.toObject(includeInstance, f),
    mainEvent: (f = msg.getMainEvent()) && proto.ubo.v1.MainEvent.toObject(includeInstance, f),
    menuChooseByIconEvent: (f = msg.getMenuChooseByIconEvent()) && proto.ubo.v1.MenuChooseByIconEvent.toObject(includeInstance, f),
    menuChooseByIndexEvent: (f = msg.getMenuChooseByIndexEvent()) && proto.ubo.v1.MenuChooseByIndexEvent.toObject(includeInstance, f),
    menuChooseByLabelEvent: (f = msg.getMenuChooseByLabelEvent()) && proto.ubo.v1.MenuChooseByLabelEvent.toObject(includeInstance, f),
    menuEvent: (f = msg.getMenuEvent()) && proto.ubo.v1.MenuEvent.toObject(includeInstance, f),
    menuGoBackEvent: (f = msg.getMenuGoBackEvent()) && proto.ubo.v1.MenuGoBackEvent.toObject(includeInstance, f),
    menuGoHomeEvent: (f = msg.getMenuGoHomeEvent()) && proto.ubo.v1.MenuGoHomeEvent.toObject(includeInstance, f),
    menuScrollEvent: (f = msg.getMenuScrollEvent()) && proto.ubo.v1.MenuScrollEvent.toObject(includeInstance, f),
    notificationsClearEvent: (f = msg.getNotificationsClearEvent()) && proto.ubo.v1.NotificationsClearEvent.toObject(includeInstance, f),
    notificationsDisplayEvent: (f = msg.getNotificationsDisplayEvent()) && proto.ubo.v1.NotificationsDisplayEvent.toObject(includeInstance, f),
    notificationsEvent: (f = msg.getNotificationsEvent()) && proto.ubo.v1.NotificationsEvent.toObject(includeInstance, f),
    openApplicationEvent: (f = msg.getOpenApplicationEvent()) && proto.ubo.v1.OpenApplicationEvent.toObject(includeInstance, f),
    powerEvent: (f = msg.getPowerEvent()) && proto.ubo.v1.PowerEvent.toObject(includeInstance, f),
    powerOffEvent: (f = msg.getPowerOffEvent()) && proto.ubo.v1.PowerOffEvent.toObject(includeInstance, f),
    rPiConnectEvent: (f = msg.getRPiConnectEvent()) && proto.ubo.v1.RPiConnectEvent.toObject(includeInstance, f),
    rPiConnectLoginEvent: (f = msg.getRPiConnectLoginEvent()) && proto.ubo.v1.RPiConnectLoginEvent.toObject(includeInstance, f),
    rebootEvent: (f = msg.getRebootEvent()) && proto.ubo.v1.RebootEvent.toObject(includeInstance, f),
    replayRecordedSequenceEvent: (f = msg.getReplayRecordedSequenceEvent()) && proto.ubo.v1.ReplayRecordedSequenceEvent.toObject(includeInstance, f),
    rgbRingCommandEvent: (f = msg.getRgbRingCommandEvent()) && proto.ubo.v1.RgbRingCommandEvent.toObject(includeInstance, f),
    rgbRingEvent: (f = msg.getRgbRingEvent()) && proto.ubo.v1.RgbRingEvent.toObject(includeInstance, f),
    screenshotEvent: (f = msg.getScreenshotEvent()) && proto.ubo.v1.ScreenshotEvent.toObject(includeInstance, f),
    settingsEvent: (f = msg.getSettingsEvent()) && proto.ubo.v1.SettingsEvent.toObject(includeInstance, f),
    settingsServiceEvent: (f = msg.getSettingsServiceEvent()) && proto.ubo.v1.SettingsServiceEvent.toObject(includeInstance, f),
    settingsStartServiceEvent: (f = msg.getSettingsStartServiceEvent()) && proto.ubo.v1.SettingsStartServiceEvent.toObject(includeInstance, f),
    settingsStopServiceEvent: (f = msg.getSettingsStopServiceEvent()) && proto.ubo.v1.SettingsStopServiceEvent.toObject(includeInstance, f),
    snapshotEvent: (f = msg.getSnapshotEvent()) && proto.ubo.v1.SnapshotEvent.toObject(includeInstance, f),
    speechRecognitionEvent: (f = msg.getSpeechRecognitionEvent()) && proto.ubo.v1.SpeechRecognitionEvent.toObject(includeInstance, f),
    speechRecognitionReportTextEvent: (f = msg.getSpeechRecognitionReportTextEvent()) && proto.ubo.v1.SpeechRecognitionReportTextEvent.toObject(includeInstance, f),
    speechSynthesisEvent: (f = msg.getSpeechSynthesisEvent()) && proto.ubo.v1.SpeechSynthesisEvent.toObject(includeInstance, f),
    speechSynthesisSynthesizeTextEvent: (f = msg.getSpeechSynthesisSynthesizeTextEvent()) && proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.toObject(includeInstance, f),
    storeRecordedSequenceEvent: (f = msg.getStoreRecordedSequenceEvent()) && proto.ubo.v1.StoreRecordedSequenceEvent.toObject(includeInstance, f),
    updateManagerCheckEvent: (f = msg.getUpdateManagerCheckEvent()) && proto.ubo.v1.UpdateManagerCheckEvent.toObject(includeInstance, f),
    updateManagerEvent: (f = msg.getUpdateManagerEvent()) && proto.ubo.v1.UpdateManagerEvent.toObject(includeInstance, f),
    updateManagerUpdateEvent: (f = msg.getUpdateManagerUpdateEvent()) && proto.ubo.v1.UpdateManagerUpdateEvent.toObject(includeInstance, f),
    usersCreateUserEvent: (f = msg.getUsersCreateUserEvent()) && proto.ubo.v1.UsersCreateUserEvent.toObject(includeInstance, f),
    usersDeleteUserEvent: (f = msg.getUsersDeleteUserEvent()) && proto.ubo.v1.UsersDeleteUserEvent.toObject(includeInstance, f),
    usersEvent: (f = msg.getUsersEvent()) && proto.ubo.v1.UsersEvent.toObject(includeInstance, f),
    usersResetPasswordEvent: (f = msg.getUsersResetPasswordEvent()) && proto.ubo.v1.UsersResetPasswordEvent.toObject(includeInstance, f),
    vsCodeEvent: (f = msg.getVsCodeEvent()) && proto.ubo.v1.VSCodeEvent.toObject(includeInstance, f),
    vsCodeLoginEvent: (f = msg.getVsCodeLoginEvent()) && proto.ubo.v1.VSCodeLoginEvent.toObject(includeInstance, f),
    vsCodeRestartEvent: (f = msg.getVsCodeRestartEvent()) && proto.ubo.v1.VSCodeRestartEvent.toObject(includeInstance, f),
    webUiEvent: (f = msg.getWebUiEvent()) && proto.ubo.v1.WebUIEvent.toObject(includeInstance, f),
    webUiInitializeEvent: (f = msg.getWebUiInitializeEvent()) && proto.ubo.v1.WebUIInitializeEvent.toObject(includeInstance, f),
    webUiStopEvent: (f = msg.getWebUiStopEvent()) && proto.ubo.v1.WebUIStopEvent.toObject(includeInstance, f),
    wiFiEvent: (f = msg.getWiFiEvent()) && proto.ubo.v1.WiFiEvent.toObject(includeInstance, f),
    wiFiInputConnectionEvent: (f = msg.getWiFiInputConnectionEvent()) && proto.ubo.v1.WiFiInputConnectionEvent.toObject(includeInstance, f),
    wiFiUpdateRequestEvent: (f = msg.getWiFiUpdateRequestEvent()) && proto.ubo.v1.WiFiUpdateRequestEvent.toObject(includeInstance, f)
  };

  if (includeInstance) {
    obj.$jspbMessageInstance = msg;
  }
  return obj;
};
}


/**
 * Deserializes binary data (in protobuf wire format).
 * @param {jspb.ByteSource} bytes The bytes to deserialize.
 * @return {!proto.ubo.v1.Event}
 */
proto.ubo.v1.Event.deserializeBinary = function(bytes) {
  var reader = new jspb.BinaryReader(bytes);
  var msg = new proto.ubo.v1.Event;
  return proto.ubo.v1.Event.deserializeBinaryFromReader(msg, reader);
};


/**
 * Deserializes binary data (in protobuf wire format) from the
 * given reader into the given message object.
 * @param {!proto.ubo.v1.Event} msg The message object to deserialize into.
 * @param {!jspb.BinaryReader} reader The BinaryReader to use.
 * @return {!proto.ubo.v1.Event}
 */
proto.ubo.v1.Event.deserializeBinaryFromReader = function(msg, reader) {
  while (reader.nextField()) {
    if (reader.isEndGroup()) {
      break;
    }
    var field = reader.getFieldNumber();
    switch (field) {
    case 1:
      var value = new proto.ubo.v1.AssistantAddMcpServerEvent;
      reader.readMessage(value,proto.ubo.v1.AssistantAddMcpServerEvent.deserializeBinaryFromReader);
      msg.setAssistantAddMcpServerEvent(value);
      break;
    case 2:
      var value = new proto.ubo.v1.AssistantDeleteMcpServerEvent;
      reader.readMessage(value,proto.ubo.v1.AssistantDeleteMcpServerEvent.deserializeBinaryFromReader);
      msg.setAssistantDeleteMcpServerEvent(value);
      break;
    case 3:
      var value = new proto.ubo.v1.AssistantDownloadOllamaModelEvent;
      reader.readMessage(value,proto.ubo.v1.AssistantDownloadOllamaModelEvent.deserializeBinaryFromReader);
      msg.setAssistantDownloadOllamaModelEvent(value);
      break;
    case 4:
      var value = new proto.ubo.v1.AssistantEvent;
      reader.readMessage(value,proto.ubo.v1.AssistantEvent.deserializeBinaryFromReader);
      msg.setAssistantEvent(value);
      break;
    case 5:
      var value = new proto.ubo.v1.AssistantHandleReportEvent;
      reader.readMessage(value,proto.ubo.v1.AssistantHandleReportEvent.deserializeBinaryFromReader);
      msg.setAssistantHandleReportEvent(value);
      break;
    case 6:
      var value = new proto.ubo.v1.AssistantUpdateProvidersEvent;
      reader.readMessage(value,proto.ubo.v1.AssistantUpdateProvidersEvent.deserializeBinaryFromReader);
      msg.setAssistantUpdateProvidersEvent(value);
      break;
    case 7:
      var value = new proto.ubo.v1.AudioEvent;
      reader.readMessage(value,proto.ubo.v1.AudioEvent.deserializeBinaryFromReader);
      msg.setAudioEvent(value);
      break;
    case 8:
      var value = new proto.ubo.v1.AudioInstallDriverEvent;
      reader.readMessage(value,proto.ubo.v1.AudioInstallDriverEvent.deserializeBinaryFromReader);
      msg.setAudioInstallDriverEvent(value);
      break;
    case 9:
      var value = new proto.ubo.v1.AudioPlayAudioSampleEvent;
      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSampleEvent.deserializeBinaryFromReader);
      msg.setAudioPlayAudioSampleEvent(value);
      break;
    case 10:
      var value = new proto.ubo.v1.AudioPlayAudioSequenceEvent;
      reader.readMessage(value,proto.ubo.v1.AudioPlayAudioSequenceEvent.deserializeBinaryFromReader);
      msg.setAudioPlayAudioSequenceEvent(value);
      break;
    case 11:
      var value = new proto.ubo.v1.AudioPlayChimeEvent;
      reader.readMessage(value,proto.ubo.v1.AudioPlayChimeEvent.deserializeBinaryFromReader);
      msg.setAudioPlayChimeEvent(value);
      break;
    case 12:
      var value = new proto.ubo.v1.AudioPlaybackDoneEvent;
      reader.readMessage(value,proto.ubo.v1.AudioPlaybackDoneEvent.deserializeBinaryFromReader);
      msg.setAudioPlaybackDoneEvent(value);
      break;
    case 13:
      var value = new proto.ubo.v1.AudioReportSampleEvent;
      reader.readMessage(value,proto.ubo.v1.AudioReportSampleEvent.deserializeBinaryFromReader);
      msg.setAudioReportSampleEvent(value);
      break;
    case 14:
      var value = new proto.ubo.v1.CameraDetectEvent;
      reader.readMessage(value,proto.ubo.v1.CameraDetectEvent.deserializeBinaryFromReader);
      msg.setCameraDetectEvent(value);
      break;
    case 15:
      var value = new proto.ubo.v1.CameraDetectedEvent;
      reader.readMessage(value,proto.ubo.v1.CameraDetectedEvent.deserializeBinaryFromReader);
      msg.setCameraDetectedEvent(value);
      break;
    case 16:
      var value = new proto.ubo.v1.CameraEvent;
      reader.readMessage(value,proto.ubo.v1.CameraEvent.deserializeBinaryFromReader);
      msg.setCameraEvent(value);
      break;
    case 17:
      var value = new proto.ubo.v1.CameraInstallDriverEvent;
      reader.readMessage(value,proto.ubo.v1.CameraInstallDriverEvent.deserializeBinaryFromReader);
      msg.setCameraInstallDriverEvent(value);
      break;
    case 18:
      var value = new proto.ubo.v1.CameraReinitializeEvent;
      reader.readMessage(value,proto.ubo.v1.CameraReinitializeEvent.deserializeBinaryFromReader);
      msg.setCameraReinitializeEvent(value);
      break;
    case 19:
      var value = new proto.ubo.v1.CameraReportImageEvent;
      reader.readMessage(value,proto.ubo.v1.CameraReportImageEvent.deserializeBinaryFromReader);
      msg.setCameraReportImageEvent(value);
      break;
    case 20:
      var value = new proto.ubo.v1.CameraRestoreDefaultEvent;
      reader.readMessage(value,proto.ubo.v1.CameraRestoreDefaultEvent.deserializeBinaryFromReader);
      msg.setCameraRestoreDefaultEvent(value);
      break;
    case 21:
      var value = new proto.ubo.v1.CameraStartViewfinderEvent;
      reader.readMessage(value,proto.ubo.v1.CameraStartViewfinderEvent.deserializeBinaryFromReader);
      msg.setCameraStartViewfinderEvent(value);
      break;
    case 22:
      var value = new proto.ubo.v1.CameraStopViewfinderEvent;
      reader.readMessage(value,proto.ubo.v1.CameraStopViewfinderEvent.deserializeBinaryFromReader);
      msg.setCameraStopViewfinderEvent(value);
      break;
    case 23:
      var value = new proto.ubo.v1.CloseApplicationEvent;
      reader.readMessage(value,proto.ubo.v1.CloseApplicationEvent.deserializeBinaryFromReader);
      msg.setCloseApplicationEvent(value);
      break;
    case 24:
      var value = new proto.ubo.v1.DisplayBlankEvent;
      reader.readMessage(value,proto.ubo.v1.DisplayBlankEvent.deserializeBinaryFromReader);
      msg.setDisplayBlankEvent(value);
      break;
    case 25:
      var value = new proto.ubo.v1.DisplayCompressedRenderEvent;
      reader.readMessage(value,proto.ubo.v1.DisplayCompressedRenderEvent.deserializeBinaryFromReader);
      msg.setDisplayCompressedRenderEvent(value);
      break;
    case 26:
      var value = new proto.ubo.v1.DisplayEvent;
      reader.readMessage(value,proto.ubo.v1.DisplayEvent.deserializeBinaryFromReader);
      msg.setDisplayEvent(value);
      break;
    case 27:
      var value = new proto.ubo.v1.DisplayRedrawEvent;
      reader.readMessage(value,proto.ubo.v1.DisplayRedrawEvent.deserializeBinaryFromReader);
      msg.setDisplayRedrawEvent(value);
      break;
    case 28:
      var value = new proto.ubo.v1.DisplayRenderEvent;
      reader.readMessage(value,proto.ubo.v1.DisplayRenderEvent.deserializeBinaryFromReader);
      msg.setDisplayRenderEvent(value);
      break;
    case 29:
      var value = new proto.ubo.v1.DisplayUnblankEvent;
      reader.readMessage(value,proto.ubo.v1.DisplayUnblankEvent.deserializeBinaryFromReader);
      msg.setDisplayUnblankEvent(value);
      break;
    case 30:
      var value = new proto.ubo.v1.DockerEvent;
      reader.readMessage(value,proto.ubo.v1.DockerEvent.deserializeBinaryFromReader);
      msg.setDockerEvent(value);
      break;
    case 31:
      var value = new proto.ubo.v1.DockerImageEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageEvent.deserializeBinaryFromReader);
      msg.setDockerImageEvent(value);
      break;
    case 32:
      var value = new proto.ubo.v1.DockerImageFetchCompositionEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageFetchCompositionEvent.deserializeBinaryFromReader);
      msg.setDockerImageFetchCompositionEvent(value);
      break;
    case 33:
      var value = new proto.ubo.v1.DockerImageFetchEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageFetchEvent.deserializeBinaryFromReader);
      msg.setDockerImageFetchEvent(value);
      break;
    case 34:
      var value = new proto.ubo.v1.DockerImageRegisterAppEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageRegisterAppEvent.deserializeBinaryFromReader);
      msg.setDockerImageRegisterAppEvent(value);
      break;
    case 35:
      var value = new proto.ubo.v1.DockerImageReleaseCompositionEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageReleaseCompositionEvent.deserializeBinaryFromReader);
      msg.setDockerImageReleaseCompositionEvent(value);
      break;
    case 36:
      var value = new proto.ubo.v1.DockerImageRemoveCompositionEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveCompositionEvent.deserializeBinaryFromReader);
      msg.setDockerImageRemoveCompositionEvent(value);
      break;
    case 37:
      var value = new proto.ubo.v1.DockerImageRemoveContainerEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveContainerEvent.deserializeBinaryFromReader);
      msg.setDockerImageRemoveContainerEvent(value);
      break;
    case 38:
      var value = new proto.ubo.v1.DockerImageRemoveEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageRemoveEvent.deserializeBinaryFromReader);
      msg.setDockerImageRemoveEvent(value);
      break;
    case 39:
      var value = new proto.ubo.v1.DockerImageRunCompositionEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageRunCompositionEvent.deserializeBinaryFromReader);
      msg.setDockerImageRunCompositionEvent(value);
      break;
    case 40:
      var value = new proto.ubo.v1.DockerImageRunContainerEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageRunContainerEvent.deserializeBinaryFromReader);
      msg.setDockerImageRunContainerEvent(value);
      break;
    case 41:
      var value = new proto.ubo.v1.DockerImageStopCompositionEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageStopCompositionEvent.deserializeBinaryFromReader);
      msg.setDockerImageStopCompositionEvent(value);
      break;
    case 42:
      var value = new proto.ubo.v1.DockerImageStopContainerEvent;
      reader.readMessage(value,proto.ubo.v1.DockerImageStopContainerEvent.deserializeBinaryFromReader);
      msg.setDockerImageStopContainerEvent(value);
      break;
    case 43:
      var value = new proto.ubo.v1.DockerInstallEvent;
      reader.readMessage(value,proto.ubo.v1.DockerInstallEvent.deserializeBinaryFromReader);
      msg.setDockerInstallEvent(value);
      break;
    case 44:
      var value = new proto.ubo.v1.DockerStartEvent;
      reader.readMessage(value,proto.ubo.v1.DockerStartEvent.deserializeBinaryFromReader);
      msg.setDockerStartEvent(value);
      break;
    case 45:
      var value = new proto.ubo.v1.DockerStopEvent;
      reader.readMessage(value,proto.ubo.v1.DockerStopEvent.deserializeBinaryFromReader);
      msg.setDockerStopEvent(value);
      break;
    case 46:
      var value = new proto.ubo.v1.FileSystemCopyEvent;
      reader.readMessage(value,proto.ubo.v1.FileSystemCopyEvent.deserializeBinaryFromReader);
      msg.setFileSystemCopyEvent(value);
      break;
    case 47:
      var value = new proto.ubo.v1.FileSystemEvent;
      reader.readMessage(value,proto.ubo.v1.FileSystemEvent.deserializeBinaryFromReader);
      msg.setFileSystemEvent(value);
      break;
    case 48:
      var value = new proto.ubo.v1.FileSystemMoveEvent;
      reader.readMessage(value,proto.ubo.v1.FileSystemMoveEvent.deserializeBinaryFromReader);
      msg.setFileSystemMoveEvent(value);
      break;
    case 49:
      var value = new proto.ubo.v1.FileSystemRemoveEvent;
      reader.readMessage(value,proto.ubo.v1.FileSystemRemoveEvent.deserializeBinaryFromReader);
      msg.setFileSystemRemoveEvent(value);
      break;
    case 50:
      var value = new proto.ubo.v1.FileSystemSelectEvent;
      reader.readMessage(value,proto.ubo.v1.FileSystemSelectEvent.deserializeBinaryFromReader);
      msg.setFileSystemSelectEvent(value);
      break;
    case 51:
      var value = new proto.ubo.v1.InfraredEvent;
      reader.readMessage(value,proto.ubo.v1.InfraredEvent.deserializeBinaryFromReader);
      msg.setInfraredEvent(value);
      break;
    case 52:
      var value = new proto.ubo.v1.InfraredSendCodeEvent;
      reader.readMessage(value,proto.ubo.v1.InfraredSendCodeEvent.deserializeBinaryFromReader);
      msg.setInfraredSendCodeEvent(value);
      break;
    case 53:
      var value = new proto.ubo.v1.InitEvent;
      reader.readMessage(value,proto.ubo.v1.InitEvent.deserializeBinaryFromReader);
      msg.setInitEvent(value);
      break;
    case 54:
      var value = new proto.ubo.v1.InputCancelEvent;
      reader.readMessage(value,proto.ubo.v1.InputCancelEvent.deserializeBinaryFromReader);
      msg.setInputCancelEvent(value);
      break;
    case 55:
      var value = new proto.ubo.v1.InputProvideEvent;
      reader.readMessage(value,proto.ubo.v1.InputProvideEvent.deserializeBinaryFromReader);
      msg.setInputProvideEvent(value);
      break;
    case 56:
      var value = new proto.ubo.v1.InputResolveEvent;
      reader.readMessage(value,proto.ubo.v1.InputResolveEvent.deserializeBinaryFromReader);
      msg.setInputResolveEvent(value);
      break;
    case 57:
      var value = new proto.ubo.v1.IpEvent;
      reader.readMessage(value,proto.ubo.v1.IpEvent.deserializeBinaryFromReader);
      msg.setIpEvent(value);
      break;
    case 58:
      var value = new proto.ubo.v1.MainEvent;
      reader.readMessage(value,proto.ubo.v1.MainEvent.deserializeBinaryFromReader);
      msg.setMainEvent(value);
      break;
    case 59:
      var value = new proto.ubo.v1.MenuChooseByIconEvent;
      reader.readMessage(value,proto.ubo.v1.MenuChooseByIconEvent.deserializeBinaryFromReader);
      msg.setMenuChooseByIconEvent(value);
      break;
    case 60:
      var value = new proto.ubo.v1.MenuChooseByIndexEvent;
      reader.readMessage(value,proto.ubo.v1.MenuChooseByIndexEvent.deserializeBinaryFromReader);
      msg.setMenuChooseByIndexEvent(value);
      break;
    case 61:
      var value = new proto.ubo.v1.MenuChooseByLabelEvent;
      reader.readMessage(value,proto.ubo.v1.MenuChooseByLabelEvent.deserializeBinaryFromReader);
      msg.setMenuChooseByLabelEvent(value);
      break;
    case 62:
      var value = new proto.ubo.v1.MenuEvent;
      reader.readMessage(value,proto.ubo.v1.MenuEvent.deserializeBinaryFromReader);
      msg.setMenuEvent(value);
      break;
    case 63:
      var value = new proto.ubo.v1.MenuGoBackEvent;
      reader.readMessage(value,proto.ubo.v1.MenuGoBackEvent.deserializeBinaryFromReader);
      msg.setMenuGoBackEvent(value);
      break;
    case 64:
      var value = new proto.ubo.v1.MenuGoHomeEvent;
      reader.readMessage(value,proto.ubo.v1.MenuGoHomeEvent.deserializeBinaryFromReader);
      msg.setMenuGoHomeEvent(value);
      break;
    case 65:
      var value = new proto.ubo.v1.MenuScrollEvent;
      reader.readMessage(value,proto.ubo.v1.MenuScrollEvent.deserializeBinaryFromReader);
      msg.setMenuScrollEvent(value);
      break;
    case 66:
      var value = new proto.ubo.v1.NotificationsClearEvent;
      reader.readMessage(value,proto.ubo.v1.NotificationsClearEvent.deserializeBinaryFromReader);
      msg.setNotificationsClearEvent(value);
      break;
    case 67:
      var value = new proto.ubo.v1.NotificationsDisplayEvent;
      reader.readMessage(value,proto.ubo.v1.NotificationsDisplayEvent.deserializeBinaryFromReader);
      msg.setNotificationsDisplayEvent(value);
      break;
    case 68:
      var value = new proto.ubo.v1.NotificationsEvent;
      reader.readMessage(value,proto.ubo.v1.NotificationsEvent.deserializeBinaryFromReader);
      msg.setNotificationsEvent(value);
      break;
    case 69:
      var value = new proto.ubo.v1.OpenApplicationEvent;
      reader.readMessage(value,proto.ubo.v1.OpenApplicationEvent.deserializeBinaryFromReader);
      msg.setOpenApplicationEvent(value);
      break;
    case 70:
      var value = new proto.ubo.v1.PowerEvent;
      reader.readMessage(value,proto.ubo.v1.PowerEvent.deserializeBinaryFromReader);
      msg.setPowerEvent(value);
      break;
    case 71:
      var value = new proto.ubo.v1.PowerOffEvent;
      reader.readMessage(value,proto.ubo.v1.PowerOffEvent.deserializeBinaryFromReader);
      msg.setPowerOffEvent(value);
      break;
    case 72:
      var value = new proto.ubo.v1.RPiConnectEvent;
      reader.readMessage(value,proto.ubo.v1.RPiConnectEvent.deserializeBinaryFromReader);
      msg.setRPiConnectEvent(value);
      break;
    case 73:
      var value = new proto.ubo.v1.RPiConnectLoginEvent;
      reader.readMessage(value,proto.ubo.v1.RPiConnectLoginEvent.deserializeBinaryFromReader);
      msg.setRPiConnectLoginEvent(value);
      break;
    case 74:
      var value = new proto.ubo.v1.RebootEvent;
      reader.readMessage(value,proto.ubo.v1.RebootEvent.deserializeBinaryFromReader);
      msg.setRebootEvent(value);
      break;
    case 75:
      var value = new proto.ubo.v1.ReplayRecordedSequenceEvent;
      reader.readMessage(value,proto.ubo.v1.ReplayRecordedSequenceEvent.deserializeBinaryFromReader);
      msg.setReplayRecordedSequenceEvent(value);
      break;
    case 76:
      var value = new proto.ubo.v1.RgbRingCommandEvent;
      reader.readMessage(value,proto.ubo.v1.RgbRingCommandEvent.deserializeBinaryFromReader);
      msg.setRgbRingCommandEvent(value);
      break;
    case 77:
      var value = new proto.ubo.v1.RgbRingEvent;
      reader.readMessage(value,proto.ubo.v1.RgbRingEvent.deserializeBinaryFromReader);
      msg.setRgbRingEvent(value);
      break;
    case 78:
      var value = new proto.ubo.v1.ScreenshotEvent;
      reader.readMessage(value,proto.ubo.v1.ScreenshotEvent.deserializeBinaryFromReader);
      msg.setScreenshotEvent(value);
      break;
    case 79:
      var value = new proto.ubo.v1.SettingsEvent;
      reader.readMessage(value,proto.ubo.v1.SettingsEvent.deserializeBinaryFromReader);
      msg.setSettingsEvent(value);
      break;
    case 80:
      var value = new proto.ubo.v1.SettingsServiceEvent;
      reader.readMessage(value,proto.ubo.v1.SettingsServiceEvent.deserializeBinaryFromReader);
      msg.setSettingsServiceEvent(value);
      break;
    case 81:
      var value = new proto.ubo.v1.SettingsStartServiceEvent;
      reader.readMessage(value,proto.ubo.v1.SettingsStartServiceEvent.deserializeBinaryFromReader);
      msg.setSettingsStartServiceEvent(value);
      break;
    case 82:
      var value = new proto.ubo.v1.SettingsStopServiceEvent;
      reader.readMessage(value,proto.ubo.v1.SettingsStopServiceEvent.deserializeBinaryFromReader);
      msg.setSettingsStopServiceEvent(value);
      break;
    case 83:
      var value = new proto.ubo.v1.SnapshotEvent;
      reader.readMessage(value,proto.ubo.v1.SnapshotEvent.deserializeBinaryFromReader);
      msg.setSnapshotEvent(value);
      break;
    case 84:
      var value = new proto.ubo.v1.SpeechRecognitionEvent;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionEvent.deserializeBinaryFromReader);
      msg.setSpeechRecognitionEvent(value);
      break;
    case 85:
      var value = new proto.ubo.v1.SpeechRecognitionReportTextEvent;
      reader.readMessage(value,proto.ubo.v1.SpeechRecognitionReportTextEvent.deserializeBinaryFromReader);
      msg.setSpeechRecognitionReportTextEvent(value);
      break;
    case 86:
      var value = new proto.ubo.v1.SpeechSynthesisEvent;
      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisEvent.deserializeBinaryFromReader);
      msg.setSpeechSynthesisEvent(value);
      break;
    case 87:
      var value = new proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent;
      reader.readMessage(value,proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.deserializeBinaryFromReader);
      msg.setSpeechSynthesisSynthesizeTextEvent(value);
      break;
    case 88:
      var value = new proto.ubo.v1.StoreRecordedSequenceEvent;
      reader.readMessage(value,proto.ubo.v1.StoreRecordedSequenceEvent.deserializeBinaryFromReader);
      msg.setStoreRecordedSequenceEvent(value);
      break;
    case 89:
      var value = new proto.ubo.v1.UpdateManagerCheckEvent;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerCheckEvent.deserializeBinaryFromReader);
      msg.setUpdateManagerCheckEvent(value);
      break;
    case 90:
      var value = new proto.ubo.v1.UpdateManagerEvent;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerEvent.deserializeBinaryFromReader);
      msg.setUpdateManagerEvent(value);
      break;
    case 91:
      var value = new proto.ubo.v1.UpdateManagerUpdateEvent;
      reader.readMessage(value,proto.ubo.v1.UpdateManagerUpdateEvent.deserializeBinaryFromReader);
      msg.setUpdateManagerUpdateEvent(value);
      break;
    case 92:
      var value = new proto.ubo.v1.UsersCreateUserEvent;
      reader.readMessage(value,proto.ubo.v1.UsersCreateUserEvent.deserializeBinaryFromReader);
      msg.setUsersCreateUserEvent(value);
      break;
    case 93:
      var value = new proto.ubo.v1.UsersDeleteUserEvent;
      reader.readMessage(value,proto.ubo.v1.UsersDeleteUserEvent.deserializeBinaryFromReader);
      msg.setUsersDeleteUserEvent(value);
      break;
    case 94:
      var value = new proto.ubo.v1.UsersEvent;
      reader.readMessage(value,proto.ubo.v1.UsersEvent.deserializeBinaryFromReader);
      msg.setUsersEvent(value);
      break;
    case 95:
      var value = new proto.ubo.v1.UsersResetPasswordEvent;
      reader.readMessage(value,proto.ubo.v1.UsersResetPasswordEvent.deserializeBinaryFromReader);
      msg.setUsersResetPasswordEvent(value);
      break;
    case 96:
      var value = new proto.ubo.v1.VSCodeEvent;
      reader.readMessage(value,proto.ubo.v1.VSCodeEvent.deserializeBinaryFromReader);
      msg.setVsCodeEvent(value);
      break;
    case 97:
      var value = new proto.ubo.v1.VSCodeLoginEvent;
      reader.readMessage(value,proto.ubo.v1.VSCodeLoginEvent.deserializeBinaryFromReader);
      msg.setVsCodeLoginEvent(value);
      break;
    case 98:
      var value = new proto.ubo.v1.VSCodeRestartEvent;
      reader.readMessage(value,proto.ubo.v1.VSCodeRestartEvent.deserializeBinaryFromReader);
      msg.setVsCodeRestartEvent(value);
      break;
    case 99:
      var value = new proto.ubo.v1.WebUIEvent;
      reader.readMessage(value,proto.ubo.v1.WebUIEvent.deserializeBinaryFromReader);
      msg.setWebUiEvent(value);
      break;
    case 100:
      var value = new proto.ubo.v1.WebUIInitializeEvent;
      reader.readMessage(value,proto.ubo.v1.WebUIInitializeEvent.deserializeBinaryFromReader);
      msg.setWebUiInitializeEvent(value);
      break;
    case 101:
      var value = new proto.ubo.v1.WebUIStopEvent;
      reader.readMessage(value,proto.ubo.v1.WebUIStopEvent.deserializeBinaryFromReader);
      msg.setWebUiStopEvent(value);
      break;
    case 102:
      var value = new proto.ubo.v1.WiFiEvent;
      reader.readMessage(value,proto.ubo.v1.WiFiEvent.deserializeBinaryFromReader);
      msg.setWiFiEvent(value);
      break;
    case 103:
      var value = new proto.ubo.v1.WiFiInputConnectionEvent;
      reader.readMessage(value,proto.ubo.v1.WiFiInputConnectionEvent.deserializeBinaryFromReader);
      msg.setWiFiInputConnectionEvent(value);
      break;
    case 104:
      var value = new proto.ubo.v1.WiFiUpdateRequestEvent;
      reader.readMessage(value,proto.ubo.v1.WiFiUpdateRequestEvent.deserializeBinaryFromReader);
      msg.setWiFiUpdateRequestEvent(value);
      break;
    default:
      reader.skipField();
      break;
    }
  }
  return msg;
};


/**
 * Serializes the message to binary data (in protobuf wire format).
 * @return {!Uint8Array}
 */
proto.ubo.v1.Event.prototype.serializeBinary = function() {
  var writer = new jspb.BinaryWriter();
  proto.ubo.v1.Event.serializeBinaryToWriter(this, writer);
  return writer.getResultBuffer();
};


/**
 * Serializes the given message to binary data (in protobuf wire
 * format), writing to the given BinaryWriter.
 * @param {!proto.ubo.v1.Event} message
 * @param {!jspb.BinaryWriter} writer
 * @suppress {unusedLocalVariables} f is only used for nested messages
 */
proto.ubo.v1.Event.serializeBinaryToWriter = function(message, writer) {
  var f = undefined;
  f = message.getAssistantAddMcpServerEvent();
  if (f != null) {
    writer.writeMessage(
      1,
      f,
      proto.ubo.v1.AssistantAddMcpServerEvent.serializeBinaryToWriter
    );
  }
  f = message.getAssistantDeleteMcpServerEvent();
  if (f != null) {
    writer.writeMessage(
      2,
      f,
      proto.ubo.v1.AssistantDeleteMcpServerEvent.serializeBinaryToWriter
    );
  }
  f = message.getAssistantDownloadOllamaModelEvent();
  if (f != null) {
    writer.writeMessage(
      3,
      f,
      proto.ubo.v1.AssistantDownloadOllamaModelEvent.serializeBinaryToWriter
    );
  }
  f = message.getAssistantEvent();
  if (f != null) {
    writer.writeMessage(
      4,
      f,
      proto.ubo.v1.AssistantEvent.serializeBinaryToWriter
    );
  }
  f = message.getAssistantHandleReportEvent();
  if (f != null) {
    writer.writeMessage(
      5,
      f,
      proto.ubo.v1.AssistantHandleReportEvent.serializeBinaryToWriter
    );
  }
  f = message.getAssistantUpdateProvidersEvent();
  if (f != null) {
    writer.writeMessage(
      6,
      f,
      proto.ubo.v1.AssistantUpdateProvidersEvent.serializeBinaryToWriter
    );
  }
  f = message.getAudioEvent();
  if (f != null) {
    writer.writeMessage(
      7,
      f,
      proto.ubo.v1.AudioEvent.serializeBinaryToWriter
    );
  }
  f = message.getAudioInstallDriverEvent();
  if (f != null) {
    writer.writeMessage(
      8,
      f,
      proto.ubo.v1.AudioInstallDriverEvent.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlayAudioSampleEvent();
  if (f != null) {
    writer.writeMessage(
      9,
      f,
      proto.ubo.v1.AudioPlayAudioSampleEvent.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlayAudioSequenceEvent();
  if (f != null) {
    writer.writeMessage(
      10,
      f,
      proto.ubo.v1.AudioPlayAudioSequenceEvent.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlayChimeEvent();
  if (f != null) {
    writer.writeMessage(
      11,
      f,
      proto.ubo.v1.AudioPlayChimeEvent.serializeBinaryToWriter
    );
  }
  f = message.getAudioPlaybackDoneEvent();
  if (f != null) {
    writer.writeMessage(
      12,
      f,
      proto.ubo.v1.AudioPlaybackDoneEvent.serializeBinaryToWriter
    );
  }
  f = message.getAudioReportSampleEvent();
  if (f != null) {
    writer.writeMessage(
      13,
      f,
      proto.ubo.v1.AudioReportSampleEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraDetectEvent();
  if (f != null) {
    writer.writeMessage(
      14,
      f,
      proto.ubo.v1.CameraDetectEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraDetectedEvent();
  if (f != null) {
    writer.writeMessage(
      15,
      f,
      proto.ubo.v1.CameraDetectedEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraEvent();
  if (f != null) {
    writer.writeMessage(
      16,
      f,
      proto.ubo.v1.CameraEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraInstallDriverEvent();
  if (f != null) {
    writer.writeMessage(
      17,
      f,
      proto.ubo.v1.CameraInstallDriverEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraReinitializeEvent();
  if (f != null) {
    writer.writeMessage(
      18,
      f,
      proto.ubo.v1.CameraReinitializeEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraReportImageEvent();
  if (f != null) {
    writer.writeMessage(
      19,
      f,
      proto.ubo.v1.CameraReportImageEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraRestoreDefaultEvent();
  if (f != null) {
    writer.writeMessage(
      20,
      f,
      proto.ubo.v1.CameraRestoreDefaultEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraStartViewfinderEvent();
  if (f != null) {
    writer.writeMessage(
      21,
      f,
      proto.ubo.v1.CameraStartViewfinderEvent.serializeBinaryToWriter
    );
  }
  f = message.getCameraStopViewfinderEvent();
  if (f != null) {
    writer.writeMessage(
      22,
      f,
      proto.ubo.v1.CameraStopViewfinderEvent.serializeBinaryToWriter
    );
  }
  f = message.getCloseApplicationEvent();
  if (f != null) {
    writer.writeMessage(
      23,
      f,
      proto.ubo.v1.CloseApplicationEvent.serializeBinaryToWriter
    );
  }
  f = message.getDisplayBlankEvent();
  if (f != null) {
    writer.writeMessage(
      24,
      f,
      proto.ubo.v1.DisplayBlankEvent.serializeBinaryToWriter
    );
  }
  f = message.getDisplayCompressedRenderEvent();
  if (f != null) {
    writer.writeMessage(
      25,
      f,
      proto.ubo.v1.DisplayCompressedRenderEvent.serializeBinaryToWriter
    );
  }
  f = message.getDisplayEvent();
  if (f != null) {
    writer.writeMessage(
      26,
      f,
      proto.ubo.v1.DisplayEvent.serializeBinaryToWriter
    );
  }
  f = message.getDisplayRedrawEvent();
  if (f != null) {
    writer.writeMessage(
      27,
      f,
      proto.ubo.v1.DisplayRedrawEvent.serializeBinaryToWriter
    );
  }
  f = message.getDisplayRenderEvent();
  if (f != null) {
    writer.writeMessage(
      28,
      f,
      proto.ubo.v1.DisplayRenderEvent.serializeBinaryToWriter
    );
  }
  f = message.getDisplayUnblankEvent();
  if (f != null) {
    writer.writeMessage(
      29,
      f,
      proto.ubo.v1.DisplayUnblankEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerEvent();
  if (f != null) {
    writer.writeMessage(
      30,
      f,
      proto.ubo.v1.DockerEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageEvent();
  if (f != null) {
    writer.writeMessage(
      31,
      f,
      proto.ubo.v1.DockerImageEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageFetchCompositionEvent();
  if (f != null) {
    writer.writeMessage(
      32,
      f,
      proto.ubo.v1.DockerImageFetchCompositionEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageFetchEvent();
  if (f != null) {
    writer.writeMessage(
      33,
      f,
      proto.ubo.v1.DockerImageFetchEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRegisterAppEvent();
  if (f != null) {
    writer.writeMessage(
      34,
      f,
      proto.ubo.v1.DockerImageRegisterAppEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageReleaseCompositionEvent();
  if (f != null) {
    writer.writeMessage(
      35,
      f,
      proto.ubo.v1.DockerImageReleaseCompositionEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRemoveCompositionEvent();
  if (f != null) {
    writer.writeMessage(
      36,
      f,
      proto.ubo.v1.DockerImageRemoveCompositionEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRemoveContainerEvent();
  if (f != null) {
    writer.writeMessage(
      37,
      f,
      proto.ubo.v1.DockerImageRemoveContainerEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRemoveEvent();
  if (f != null) {
    writer.writeMessage(
      38,
      f,
      proto.ubo.v1.DockerImageRemoveEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRunCompositionEvent();
  if (f != null) {
    writer.writeMessage(
      39,
      f,
      proto.ubo.v1.DockerImageRunCompositionEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageRunContainerEvent();
  if (f != null) {
    writer.writeMessage(
      40,
      f,
      proto.ubo.v1.DockerImageRunContainerEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageStopCompositionEvent();
  if (f != null) {
    writer.writeMessage(
      41,
      f,
      proto.ubo.v1.DockerImageStopCompositionEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerImageStopContainerEvent();
  if (f != null) {
    writer.writeMessage(
      42,
      f,
      proto.ubo.v1.DockerImageStopContainerEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerInstallEvent();
  if (f != null) {
    writer.writeMessage(
      43,
      f,
      proto.ubo.v1.DockerInstallEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerStartEvent();
  if (f != null) {
    writer.writeMessage(
      44,
      f,
      proto.ubo.v1.DockerStartEvent.serializeBinaryToWriter
    );
  }
  f = message.getDockerStopEvent();
  if (f != null) {
    writer.writeMessage(
      45,
      f,
      proto.ubo.v1.DockerStopEvent.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemCopyEvent();
  if (f != null) {
    writer.writeMessage(
      46,
      f,
      proto.ubo.v1.FileSystemCopyEvent.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemEvent();
  if (f != null) {
    writer.writeMessage(
      47,
      f,
      proto.ubo.v1.FileSystemEvent.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemMoveEvent();
  if (f != null) {
    writer.writeMessage(
      48,
      f,
      proto.ubo.v1.FileSystemMoveEvent.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemRemoveEvent();
  if (f != null) {
    writer.writeMessage(
      49,
      f,
      proto.ubo.v1.FileSystemRemoveEvent.serializeBinaryToWriter
    );
  }
  f = message.getFileSystemSelectEvent();
  if (f != null) {
    writer.writeMessage(
      50,
      f,
      proto.ubo.v1.FileSystemSelectEvent.serializeBinaryToWriter
    );
  }
  f = message.getInfraredEvent();
  if (f != null) {
    writer.writeMessage(
      51,
      f,
      proto.ubo.v1.InfraredEvent.serializeBinaryToWriter
    );
  }
  f = message.getInfraredSendCodeEvent();
  if (f != null) {
    writer.writeMessage(
      52,
      f,
      proto.ubo.v1.InfraredSendCodeEvent.serializeBinaryToWriter
    );
  }
  f = message.getInitEvent();
  if (f != null) {
    writer.writeMessage(
      53,
      f,
      proto.ubo.v1.InitEvent.serializeBinaryToWriter
    );
  }
  f = message.getInputCancelEvent();
  if (f != null) {
    writer.writeMessage(
      54,
      f,
      proto.ubo.v1.InputCancelEvent.serializeBinaryToWriter
    );
  }
  f = message.getInputProvideEvent();
  if (f != null) {
    writer.writeMessage(
      55,
      f,
      proto.ubo.v1.InputProvideEvent.serializeBinaryToWriter
    );
  }
  f = message.getInputResolveEvent();
  if (f != null) {
    writer.writeMessage(
      56,
      f,
      proto.ubo.v1.InputResolveEvent.serializeBinaryToWriter
    );
  }
  f = message.getIpEvent();
  if (f != null) {
    writer.writeMessage(
      57,
      f,
      proto.ubo.v1.IpEvent.serializeBinaryToWriter
    );
  }
  f = message.getMainEvent();
  if (f != null) {
    writer.writeMessage(
      58,
      f,
      proto.ubo.v1.MainEvent.serializeBinaryToWriter
    );
  }
  f = message.getMenuChooseByIconEvent();
  if (f != null) {
    writer.writeMessage(
      59,
      f,
      proto.ubo.v1.MenuChooseByIconEvent.serializeBinaryToWriter
    );
  }
  f = message.getMenuChooseByIndexEvent();
  if (f != null) {
    writer.writeMessage(
      60,
      f,
      proto.ubo.v1.MenuChooseByIndexEvent.serializeBinaryToWriter
    );
  }
  f = message.getMenuChooseByLabelEvent();
  if (f != null) {
    writer.writeMessage(
      61,
      f,
      proto.ubo.v1.MenuChooseByLabelEvent.serializeBinaryToWriter
    );
  }
  f = message.getMenuEvent();
  if (f != null) {
    writer.writeMessage(
      62,
      f,
      proto.ubo.v1.MenuEvent.serializeBinaryToWriter
    );
  }
  f = message.getMenuGoBackEvent();
  if (f != null) {
    writer.writeMessage(
      63,
      f,
      proto.ubo.v1.MenuGoBackEvent.serializeBinaryToWriter
    );
  }
  f = message.getMenuGoHomeEvent();
  if (f != null) {
    writer.writeMessage(
      64,
      f,
      proto.ubo.v1.MenuGoHomeEvent.serializeBinaryToWriter
    );
  }
  f = message.getMenuScrollEvent();
  if (f != null) {
    writer.writeMessage(
      65,
      f,
      proto.ubo.v1.MenuScrollEvent.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsClearEvent();
  if (f != null) {
    writer.writeMessage(
      66,
      f,
      proto.ubo.v1.NotificationsClearEvent.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsDisplayEvent();
  if (f != null) {
    writer.writeMessage(
      67,
      f,
      proto.ubo.v1.NotificationsDisplayEvent.serializeBinaryToWriter
    );
  }
  f = message.getNotificationsEvent();
  if (f != null) {
    writer.writeMessage(
      68,
      f,
      proto.ubo.v1.NotificationsEvent.serializeBinaryToWriter
    );
  }
  f = message.getOpenApplicationEvent();
  if (f != null) {
    writer.writeMessage(
      69,
      f,
      proto.ubo.v1.OpenApplicationEvent.serializeBinaryToWriter
    );
  }
  f = message.getPowerEvent();
  if (f != null) {
    writer.writeMessage(
      70,
      f,
      proto.ubo.v1.PowerEvent.serializeBinaryToWriter
    );
  }
  f = message.getPowerOffEvent();
  if (f != null) {
    writer.writeMessage(
      71,
      f,
      proto.ubo.v1.PowerOffEvent.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectEvent();
  if (f != null) {
    writer.writeMessage(
      72,
      f,
      proto.ubo.v1.RPiConnectEvent.serializeBinaryToWriter
    );
  }
  f = message.getRPiConnectLoginEvent();
  if (f != null) {
    writer.writeMessage(
      73,
      f,
      proto.ubo.v1.RPiConnectLoginEvent.serializeBinaryToWriter
    );
  }
  f = message.getRebootEvent();
  if (f != null) {
    writer.writeMessage(
      74,
      f,
      proto.ubo.v1.RebootEvent.serializeBinaryToWriter
    );
  }
  f = message.getReplayRecordedSequenceEvent();
  if (f != null) {
    writer.writeMessage(
      75,
      f,
      proto.ubo.v1.ReplayRecordedSequenceEvent.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingCommandEvent();
  if (f != null) {
    writer.writeMessage(
      76,
      f,
      proto.ubo.v1.RgbRingCommandEvent.serializeBinaryToWriter
    );
  }
  f = message.getRgbRingEvent();
  if (f != null) {
    writer.writeMessage(
      77,
      f,
      proto.ubo.v1.RgbRingEvent.serializeBinaryToWriter
    );
  }
  f = message.getScreenshotEvent();
  if (f != null) {
    writer.writeMessage(
      78,
      f,
      proto.ubo.v1.ScreenshotEvent.serializeBinaryToWriter
    );
  }
  f = message.getSettingsEvent();
  if (f != null) {
    writer.writeMessage(
      79,
      f,
      proto.ubo.v1.SettingsEvent.serializeBinaryToWriter
    );
  }
  f = message.getSettingsServiceEvent();
  if (f != null) {
    writer.writeMessage(
      80,
      f,
      proto.ubo.v1.SettingsServiceEvent.serializeBinaryToWriter
    );
  }
  f = message.getSettingsStartServiceEvent();
  if (f != null) {
    writer.writeMessage(
      81,
      f,
      proto.ubo.v1.SettingsStartServiceEvent.serializeBinaryToWriter
    );
  }
  f = message.getSettingsStopServiceEvent();
  if (f != null) {
    writer.writeMessage(
      82,
      f,
      proto.ubo.v1.SettingsStopServiceEvent.serializeBinaryToWriter
    );
  }
  f = message.getSnapshotEvent();
  if (f != null) {
    writer.writeMessage(
      83,
      f,
      proto.ubo.v1.SnapshotEvent.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionEvent();
  if (f != null) {
    writer.writeMessage(
      84,
      f,
      proto.ubo.v1.SpeechRecognitionEvent.serializeBinaryToWriter
    );
  }
  f = message.getSpeechRecognitionReportTextEvent();
  if (f != null) {
    writer.writeMessage(
      85,
      f,
      proto.ubo.v1.SpeechRecognitionReportTextEvent.serializeBinaryToWriter
    );
  }
  f = message.getSpeechSynthesisEvent();
  if (f != null) {
    writer.writeMessage(
      86,
      f,
      proto.ubo.v1.SpeechSynthesisEvent.serializeBinaryToWriter
    );
  }
  f = message.getSpeechSynthesisSynthesizeTextEvent();
  if (f != null) {
    writer.writeMessage(
      87,
      f,
      proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent.serializeBinaryToWriter
    );
  }
  f = message.getStoreRecordedSequenceEvent();
  if (f != null) {
    writer.writeMessage(
      88,
      f,
      proto.ubo.v1.StoreRecordedSequenceEvent.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerCheckEvent();
  if (f != null) {
    writer.writeMessage(
      89,
      f,
      proto.ubo.v1.UpdateManagerCheckEvent.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerEvent();
  if (f != null) {
    writer.writeMessage(
      90,
      f,
      proto.ubo.v1.UpdateManagerEvent.serializeBinaryToWriter
    );
  }
  f = message.getUpdateManagerUpdateEvent();
  if (f != null) {
    writer.writeMessage(
      91,
      f,
      proto.ubo.v1.UpdateManagerUpdateEvent.serializeBinaryToWriter
    );
  }
  f = message.getUsersCreateUserEvent();
  if (f != null) {
    writer.writeMessage(
      92,
      f,
      proto.ubo.v1.UsersCreateUserEvent.serializeBinaryToWriter
    );
  }
  f = message.getUsersDeleteUserEvent();
  if (f != null) {
    writer.writeMessage(
      93,
      f,
      proto.ubo.v1.UsersDeleteUserEvent.serializeBinaryToWriter
    );
  }
  f = message.getUsersEvent();
  if (f != null) {
    writer.writeMessage(
      94,
      f,
      proto.ubo.v1.UsersEvent.serializeBinaryToWriter
    );
  }
  f = message.getUsersResetPasswordEvent();
  if (f != null) {
    writer.writeMessage(
      95,
      f,
      proto.ubo.v1.UsersResetPasswordEvent.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeEvent();
  if (f != null) {
    writer.writeMessage(
      96,
      f,
      proto.ubo.v1.VSCodeEvent.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeLoginEvent();
  if (f != null) {
    writer.writeMessage(
      97,
      f,
      proto.ubo.v1.VSCodeLoginEvent.serializeBinaryToWriter
    );
  }
  f = message.getVsCodeRestartEvent();
  if (f != null) {
    writer.writeMessage(
      98,
      f,
      proto.ubo.v1.VSCodeRestartEvent.serializeBinaryToWriter
    );
  }
  f = message.getWebUiEvent();
  if (f != null) {
    writer.writeMessage(
      99,
      f,
      proto.ubo.v1.WebUIEvent.serializeBinaryToWriter
    );
  }
  f = message.getWebUiInitializeEvent();
  if (f != null) {
    writer.writeMessage(
      100,
      f,
      proto.ubo.v1.WebUIInitializeEvent.serializeBinaryToWriter
    );
  }
  f = message.getWebUiStopEvent();
  if (f != null) {
    writer.writeMessage(
      101,
      f,
      proto.ubo.v1.WebUIStopEvent.serializeBinaryToWriter
    );
  }
  f = message.getWiFiEvent();
  if (f != null) {
    writer.writeMessage(
      102,
      f,
      proto.ubo.v1.WiFiEvent.serializeBinaryToWriter
    );
  }
  f = message.getWiFiInputConnectionEvent();
  if (f != null) {
    writer.writeMessage(
      103,
      f,
      proto.ubo.v1.WiFiInputConnectionEvent.serializeBinaryToWriter
    );
  }
  f = message.getWiFiUpdateRequestEvent();
  if (f != null) {
    writer.writeMessage(
      104,
      f,
      proto.ubo.v1.WiFiUpdateRequestEvent.serializeBinaryToWriter
    );
  }
};


/**
 * optional AssistantAddMcpServerEvent assistant_add_mcp_server_event = 1;
 * @return {?proto.ubo.v1.AssistantAddMcpServerEvent}
 */
proto.ubo.v1.Event.prototype.getAssistantAddMcpServerEvent = function() {
  return /** @type{?proto.ubo.v1.AssistantAddMcpServerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantAddMcpServerEvent, 1));
};


/**
 * @param {?proto.ubo.v1.AssistantAddMcpServerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAssistantAddMcpServerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 1, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAssistantAddMcpServerEvent = function() {
  return this.setAssistantAddMcpServerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAssistantAddMcpServerEvent = function() {
  return jspb.Message.getField(this, 1) != null;
};


/**
 * optional AssistantDeleteMcpServerEvent assistant_delete_mcp_server_event = 2;
 * @return {?proto.ubo.v1.AssistantDeleteMcpServerEvent}
 */
proto.ubo.v1.Event.prototype.getAssistantDeleteMcpServerEvent = function() {
  return /** @type{?proto.ubo.v1.AssistantDeleteMcpServerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDeleteMcpServerEvent, 2));
};


/**
 * @param {?proto.ubo.v1.AssistantDeleteMcpServerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAssistantDeleteMcpServerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 2, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAssistantDeleteMcpServerEvent = function() {
  return this.setAssistantDeleteMcpServerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAssistantDeleteMcpServerEvent = function() {
  return jspb.Message.getField(this, 2) != null;
};


/**
 * optional AssistantDownloadOllamaModelEvent assistant_download_ollama_model_event = 3;
 * @return {?proto.ubo.v1.AssistantDownloadOllamaModelEvent}
 */
proto.ubo.v1.Event.prototype.getAssistantDownloadOllamaModelEvent = function() {
  return /** @type{?proto.ubo.v1.AssistantDownloadOllamaModelEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantDownloadOllamaModelEvent, 3));
};


/**
 * @param {?proto.ubo.v1.AssistantDownloadOllamaModelEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAssistantDownloadOllamaModelEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 3, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAssistantDownloadOllamaModelEvent = function() {
  return this.setAssistantDownloadOllamaModelEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAssistantDownloadOllamaModelEvent = function() {
  return jspb.Message.getField(this, 3) != null;
};


/**
 * optional AssistantEvent assistant_event = 4;
 * @return {?proto.ubo.v1.AssistantEvent}
 */
proto.ubo.v1.Event.prototype.getAssistantEvent = function() {
  return /** @type{?proto.ubo.v1.AssistantEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantEvent, 4));
};


/**
 * @param {?proto.ubo.v1.AssistantEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAssistantEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 4, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAssistantEvent = function() {
  return this.setAssistantEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAssistantEvent = function() {
  return jspb.Message.getField(this, 4) != null;
};


/**
 * optional AssistantHandleReportEvent assistant_handle_report_event = 5;
 * @return {?proto.ubo.v1.AssistantHandleReportEvent}
 */
proto.ubo.v1.Event.prototype.getAssistantHandleReportEvent = function() {
  return /** @type{?proto.ubo.v1.AssistantHandleReportEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantHandleReportEvent, 5));
};


/**
 * @param {?proto.ubo.v1.AssistantHandleReportEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAssistantHandleReportEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 5, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAssistantHandleReportEvent = function() {
  return this.setAssistantHandleReportEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAssistantHandleReportEvent = function() {
  return jspb.Message.getField(this, 5) != null;
};


/**
 * optional AssistantUpdateProvidersEvent assistant_update_providers_event = 6;
 * @return {?proto.ubo.v1.AssistantUpdateProvidersEvent}
 */
proto.ubo.v1.Event.prototype.getAssistantUpdateProvidersEvent = function() {
  return /** @type{?proto.ubo.v1.AssistantUpdateProvidersEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AssistantUpdateProvidersEvent, 6));
};


/**
 * @param {?proto.ubo.v1.AssistantUpdateProvidersEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAssistantUpdateProvidersEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 6, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAssistantUpdateProvidersEvent = function() {
  return this.setAssistantUpdateProvidersEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAssistantUpdateProvidersEvent = function() {
  return jspb.Message.getField(this, 6) != null;
};


/**
 * optional AudioEvent audio_event = 7;
 * @return {?proto.ubo.v1.AudioEvent}
 */
proto.ubo.v1.Event.prototype.getAudioEvent = function() {
  return /** @type{?proto.ubo.v1.AudioEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioEvent, 7));
};


/**
 * @param {?proto.ubo.v1.AudioEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAudioEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 7, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAudioEvent = function() {
  return this.setAudioEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAudioEvent = function() {
  return jspb.Message.getField(this, 7) != null;
};


/**
 * optional AudioInstallDriverEvent audio_install_driver_event = 8;
 * @return {?proto.ubo.v1.AudioInstallDriverEvent}
 */
proto.ubo.v1.Event.prototype.getAudioInstallDriverEvent = function() {
  return /** @type{?proto.ubo.v1.AudioInstallDriverEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioInstallDriverEvent, 8));
};


/**
 * @param {?proto.ubo.v1.AudioInstallDriverEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAudioInstallDriverEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 8, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAudioInstallDriverEvent = function() {
  return this.setAudioInstallDriverEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAudioInstallDriverEvent = function() {
  return jspb.Message.getField(this, 8) != null;
};


/**
 * optional AudioPlayAudioSampleEvent audio_play_audio_sample_event = 9;
 * @return {?proto.ubo.v1.AudioPlayAudioSampleEvent}
 */
proto.ubo.v1.Event.prototype.getAudioPlayAudioSampleEvent = function() {
  return /** @type{?proto.ubo.v1.AudioPlayAudioSampleEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSampleEvent, 9));
};


/**
 * @param {?proto.ubo.v1.AudioPlayAudioSampleEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAudioPlayAudioSampleEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 9, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAudioPlayAudioSampleEvent = function() {
  return this.setAudioPlayAudioSampleEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAudioPlayAudioSampleEvent = function() {
  return jspb.Message.getField(this, 9) != null;
};


/**
 * optional AudioPlayAudioSequenceEvent audio_play_audio_sequence_event = 10;
 * @return {?proto.ubo.v1.AudioPlayAudioSequenceEvent}
 */
proto.ubo.v1.Event.prototype.getAudioPlayAudioSequenceEvent = function() {
  return /** @type{?proto.ubo.v1.AudioPlayAudioSequenceEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayAudioSequenceEvent, 10));
};


/**
 * @param {?proto.ubo.v1.AudioPlayAudioSequenceEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAudioPlayAudioSequenceEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 10, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAudioPlayAudioSequenceEvent = function() {
  return this.setAudioPlayAudioSequenceEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAudioPlayAudioSequenceEvent = function() {
  return jspb.Message.getField(this, 10) != null;
};


/**
 * optional AudioPlayChimeEvent audio_play_chime_event = 11;
 * @return {?proto.ubo.v1.AudioPlayChimeEvent}
 */
proto.ubo.v1.Event.prototype.getAudioPlayChimeEvent = function() {
  return /** @type{?proto.ubo.v1.AudioPlayChimeEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlayChimeEvent, 11));
};


/**
 * @param {?proto.ubo.v1.AudioPlayChimeEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAudioPlayChimeEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 11, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAudioPlayChimeEvent = function() {
  return this.setAudioPlayChimeEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAudioPlayChimeEvent = function() {
  return jspb.Message.getField(this, 11) != null;
};


/**
 * optional AudioPlaybackDoneEvent audio_playback_done_event = 12;
 * @return {?proto.ubo.v1.AudioPlaybackDoneEvent}
 */
proto.ubo.v1.Event.prototype.getAudioPlaybackDoneEvent = function() {
  return /** @type{?proto.ubo.v1.AudioPlaybackDoneEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioPlaybackDoneEvent, 12));
};


/**
 * @param {?proto.ubo.v1.AudioPlaybackDoneEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAudioPlaybackDoneEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 12, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAudioPlaybackDoneEvent = function() {
  return this.setAudioPlaybackDoneEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAudioPlaybackDoneEvent = function() {
  return jspb.Message.getField(this, 12) != null;
};


/**
 * optional AudioReportSampleEvent audio_report_sample_event = 13;
 * @return {?proto.ubo.v1.AudioReportSampleEvent}
 */
proto.ubo.v1.Event.prototype.getAudioReportSampleEvent = function() {
  return /** @type{?proto.ubo.v1.AudioReportSampleEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.AudioReportSampleEvent, 13));
};


/**
 * @param {?proto.ubo.v1.AudioReportSampleEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setAudioReportSampleEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 13, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearAudioReportSampleEvent = function() {
  return this.setAudioReportSampleEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasAudioReportSampleEvent = function() {
  return jspb.Message.getField(this, 13) != null;
};


/**
 * optional CameraDetectEvent camera_detect_event = 14;
 * @return {?proto.ubo.v1.CameraDetectEvent}
 */
proto.ubo.v1.Event.prototype.getCameraDetectEvent = function() {
  return /** @type{?proto.ubo.v1.CameraDetectEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraDetectEvent, 14));
};


/**
 * @param {?proto.ubo.v1.CameraDetectEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraDetectEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 14, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraDetectEvent = function() {
  return this.setCameraDetectEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraDetectEvent = function() {
  return jspb.Message.getField(this, 14) != null;
};


/**
 * optional CameraDetectedEvent camera_detected_event = 15;
 * @return {?proto.ubo.v1.CameraDetectedEvent}
 */
proto.ubo.v1.Event.prototype.getCameraDetectedEvent = function() {
  return /** @type{?proto.ubo.v1.CameraDetectedEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraDetectedEvent, 15));
};


/**
 * @param {?proto.ubo.v1.CameraDetectedEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraDetectedEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 15, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraDetectedEvent = function() {
  return this.setCameraDetectedEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraDetectedEvent = function() {
  return jspb.Message.getField(this, 15) != null;
};


/**
 * optional CameraEvent camera_event = 16;
 * @return {?proto.ubo.v1.CameraEvent}
 */
proto.ubo.v1.Event.prototype.getCameraEvent = function() {
  return /** @type{?proto.ubo.v1.CameraEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraEvent, 16));
};


/**
 * @param {?proto.ubo.v1.CameraEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 16, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraEvent = function() {
  return this.setCameraEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraEvent = function() {
  return jspb.Message.getField(this, 16) != null;
};


/**
 * optional CameraInstallDriverEvent camera_install_driver_event = 17;
 * @return {?proto.ubo.v1.CameraInstallDriverEvent}
 */
proto.ubo.v1.Event.prototype.getCameraInstallDriverEvent = function() {
  return /** @type{?proto.ubo.v1.CameraInstallDriverEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraInstallDriverEvent, 17));
};


/**
 * @param {?proto.ubo.v1.CameraInstallDriverEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraInstallDriverEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 17, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraInstallDriverEvent = function() {
  return this.setCameraInstallDriverEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraInstallDriverEvent = function() {
  return jspb.Message.getField(this, 17) != null;
};


/**
 * optional CameraReinitializeEvent camera_reinitialize_event = 18;
 * @return {?proto.ubo.v1.CameraReinitializeEvent}
 */
proto.ubo.v1.Event.prototype.getCameraReinitializeEvent = function() {
  return /** @type{?proto.ubo.v1.CameraReinitializeEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraReinitializeEvent, 18));
};


/**
 * @param {?proto.ubo.v1.CameraReinitializeEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraReinitializeEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 18, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraReinitializeEvent = function() {
  return this.setCameraReinitializeEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraReinitializeEvent = function() {
  return jspb.Message.getField(this, 18) != null;
};


/**
 * optional CameraReportImageEvent camera_report_image_event = 19;
 * @return {?proto.ubo.v1.CameraReportImageEvent}
 */
proto.ubo.v1.Event.prototype.getCameraReportImageEvent = function() {
  return /** @type{?proto.ubo.v1.CameraReportImageEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraReportImageEvent, 19));
};


/**
 * @param {?proto.ubo.v1.CameraReportImageEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraReportImageEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 19, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraReportImageEvent = function() {
  return this.setCameraReportImageEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraReportImageEvent = function() {
  return jspb.Message.getField(this, 19) != null;
};


/**
 * optional CameraRestoreDefaultEvent camera_restore_default_event = 20;
 * @return {?proto.ubo.v1.CameraRestoreDefaultEvent}
 */
proto.ubo.v1.Event.prototype.getCameraRestoreDefaultEvent = function() {
  return /** @type{?proto.ubo.v1.CameraRestoreDefaultEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraRestoreDefaultEvent, 20));
};


/**
 * @param {?proto.ubo.v1.CameraRestoreDefaultEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraRestoreDefaultEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 20, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraRestoreDefaultEvent = function() {
  return this.setCameraRestoreDefaultEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraRestoreDefaultEvent = function() {
  return jspb.Message.getField(this, 20) != null;
};


/**
 * optional CameraStartViewfinderEvent camera_start_viewfinder_event = 21;
 * @return {?proto.ubo.v1.CameraStartViewfinderEvent}
 */
proto.ubo.v1.Event.prototype.getCameraStartViewfinderEvent = function() {
  return /** @type{?proto.ubo.v1.CameraStartViewfinderEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraStartViewfinderEvent, 21));
};


/**
 * @param {?proto.ubo.v1.CameraStartViewfinderEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraStartViewfinderEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 21, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraStartViewfinderEvent = function() {
  return this.setCameraStartViewfinderEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraStartViewfinderEvent = function() {
  return jspb.Message.getField(this, 21) != null;
};


/**
 * optional CameraStopViewfinderEvent camera_stop_viewfinder_event = 22;
 * @return {?proto.ubo.v1.CameraStopViewfinderEvent}
 */
proto.ubo.v1.Event.prototype.getCameraStopViewfinderEvent = function() {
  return /** @type{?proto.ubo.v1.CameraStopViewfinderEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CameraStopViewfinderEvent, 22));
};


/**
 * @param {?proto.ubo.v1.CameraStopViewfinderEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCameraStopViewfinderEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 22, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCameraStopViewfinderEvent = function() {
  return this.setCameraStopViewfinderEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCameraStopViewfinderEvent = function() {
  return jspb.Message.getField(this, 22) != null;
};


/**
 * optional CloseApplicationEvent close_application_event = 23;
 * @return {?proto.ubo.v1.CloseApplicationEvent}
 */
proto.ubo.v1.Event.prototype.getCloseApplicationEvent = function() {
  return /** @type{?proto.ubo.v1.CloseApplicationEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.CloseApplicationEvent, 23));
};


/**
 * @param {?proto.ubo.v1.CloseApplicationEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setCloseApplicationEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 23, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearCloseApplicationEvent = function() {
  return this.setCloseApplicationEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasCloseApplicationEvent = function() {
  return jspb.Message.getField(this, 23) != null;
};


/**
 * optional DisplayBlankEvent display_blank_event = 24;
 * @return {?proto.ubo.v1.DisplayBlankEvent}
 */
proto.ubo.v1.Event.prototype.getDisplayBlankEvent = function() {
  return /** @type{?proto.ubo.v1.DisplayBlankEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayBlankEvent, 24));
};


/**
 * @param {?proto.ubo.v1.DisplayBlankEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDisplayBlankEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 24, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDisplayBlankEvent = function() {
  return this.setDisplayBlankEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDisplayBlankEvent = function() {
  return jspb.Message.getField(this, 24) != null;
};


/**
 * optional DisplayCompressedRenderEvent display_compressed_render_event = 25;
 * @return {?proto.ubo.v1.DisplayCompressedRenderEvent}
 */
proto.ubo.v1.Event.prototype.getDisplayCompressedRenderEvent = function() {
  return /** @type{?proto.ubo.v1.DisplayCompressedRenderEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayCompressedRenderEvent, 25));
};


/**
 * @param {?proto.ubo.v1.DisplayCompressedRenderEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDisplayCompressedRenderEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 25, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDisplayCompressedRenderEvent = function() {
  return this.setDisplayCompressedRenderEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDisplayCompressedRenderEvent = function() {
  return jspb.Message.getField(this, 25) != null;
};


/**
 * optional DisplayEvent display_event = 26;
 * @return {?proto.ubo.v1.DisplayEvent}
 */
proto.ubo.v1.Event.prototype.getDisplayEvent = function() {
  return /** @type{?proto.ubo.v1.DisplayEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayEvent, 26));
};


/**
 * @param {?proto.ubo.v1.DisplayEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDisplayEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 26, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDisplayEvent = function() {
  return this.setDisplayEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDisplayEvent = function() {
  return jspb.Message.getField(this, 26) != null;
};


/**
 * optional DisplayRedrawEvent display_redraw_event = 27;
 * @return {?proto.ubo.v1.DisplayRedrawEvent}
 */
proto.ubo.v1.Event.prototype.getDisplayRedrawEvent = function() {
  return /** @type{?proto.ubo.v1.DisplayRedrawEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayRedrawEvent, 27));
};


/**
 * @param {?proto.ubo.v1.DisplayRedrawEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDisplayRedrawEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 27, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDisplayRedrawEvent = function() {
  return this.setDisplayRedrawEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDisplayRedrawEvent = function() {
  return jspb.Message.getField(this, 27) != null;
};


/**
 * optional DisplayRenderEvent display_render_event = 28;
 * @return {?proto.ubo.v1.DisplayRenderEvent}
 */
proto.ubo.v1.Event.prototype.getDisplayRenderEvent = function() {
  return /** @type{?proto.ubo.v1.DisplayRenderEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayRenderEvent, 28));
};


/**
 * @param {?proto.ubo.v1.DisplayRenderEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDisplayRenderEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 28, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDisplayRenderEvent = function() {
  return this.setDisplayRenderEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDisplayRenderEvent = function() {
  return jspb.Message.getField(this, 28) != null;
};


/**
 * optional DisplayUnblankEvent display_unblank_event = 29;
 * @return {?proto.ubo.v1.DisplayUnblankEvent}
 */
proto.ubo.v1.Event.prototype.getDisplayUnblankEvent = function() {
  return /** @type{?proto.ubo.v1.DisplayUnblankEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DisplayUnblankEvent, 29));
};


/**
 * @param {?proto.ubo.v1.DisplayUnblankEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDisplayUnblankEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 29, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDisplayUnblankEvent = function() {
  return this.setDisplayUnblankEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDisplayUnblankEvent = function() {
  return jspb.Message.getField(this, 29) != null;
};


/**
 * optional DockerEvent docker_event = 30;
 * @return {?proto.ubo.v1.DockerEvent}
 */
proto.ubo.v1.Event.prototype.getDockerEvent = function() {
  return /** @type{?proto.ubo.v1.DockerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerEvent, 30));
};


/**
 * @param {?proto.ubo.v1.DockerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 30, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerEvent = function() {
  return this.setDockerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerEvent = function() {
  return jspb.Message.getField(this, 30) != null;
};


/**
 * optional DockerImageEvent docker_image_event = 31;
 * @return {?proto.ubo.v1.DockerImageEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageEvent, 31));
};


/**
 * @param {?proto.ubo.v1.DockerImageEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 31, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageEvent = function() {
  return this.setDockerImageEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageEvent = function() {
  return jspb.Message.getField(this, 31) != null;
};


/**
 * optional DockerImageFetchCompositionEvent docker_image_fetch_composition_event = 32;
 * @return {?proto.ubo.v1.DockerImageFetchCompositionEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageFetchCompositionEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageFetchCompositionEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageFetchCompositionEvent, 32));
};


/**
 * @param {?proto.ubo.v1.DockerImageFetchCompositionEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageFetchCompositionEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 32, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageFetchCompositionEvent = function() {
  return this.setDockerImageFetchCompositionEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageFetchCompositionEvent = function() {
  return jspb.Message.getField(this, 32) != null;
};


/**
 * optional DockerImageFetchEvent docker_image_fetch_event = 33;
 * @return {?proto.ubo.v1.DockerImageFetchEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageFetchEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageFetchEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageFetchEvent, 33));
};


/**
 * @param {?proto.ubo.v1.DockerImageFetchEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageFetchEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 33, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageFetchEvent = function() {
  return this.setDockerImageFetchEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageFetchEvent = function() {
  return jspb.Message.getField(this, 33) != null;
};


/**
 * optional DockerImageRegisterAppEvent docker_image_register_app_event = 34;
 * @return {?proto.ubo.v1.DockerImageRegisterAppEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageRegisterAppEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageRegisterAppEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRegisterAppEvent, 34));
};


/**
 * @param {?proto.ubo.v1.DockerImageRegisterAppEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageRegisterAppEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 34, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageRegisterAppEvent = function() {
  return this.setDockerImageRegisterAppEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageRegisterAppEvent = function() {
  return jspb.Message.getField(this, 34) != null;
};


/**
 * optional DockerImageReleaseCompositionEvent docker_image_release_composition_event = 35;
 * @return {?proto.ubo.v1.DockerImageReleaseCompositionEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageReleaseCompositionEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageReleaseCompositionEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageReleaseCompositionEvent, 35));
};


/**
 * @param {?proto.ubo.v1.DockerImageReleaseCompositionEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageReleaseCompositionEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 35, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageReleaseCompositionEvent = function() {
  return this.setDockerImageReleaseCompositionEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageReleaseCompositionEvent = function() {
  return jspb.Message.getField(this, 35) != null;
};


/**
 * optional DockerImageRemoveCompositionEvent docker_image_remove_composition_event = 36;
 * @return {?proto.ubo.v1.DockerImageRemoveCompositionEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageRemoveCompositionEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageRemoveCompositionEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveCompositionEvent, 36));
};


/**
 * @param {?proto.ubo.v1.DockerImageRemoveCompositionEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageRemoveCompositionEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 36, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageRemoveCompositionEvent = function() {
  return this.setDockerImageRemoveCompositionEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageRemoveCompositionEvent = function() {
  return jspb.Message.getField(this, 36) != null;
};


/**
 * optional DockerImageRemoveContainerEvent docker_image_remove_container_event = 37;
 * @return {?proto.ubo.v1.DockerImageRemoveContainerEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageRemoveContainerEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageRemoveContainerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveContainerEvent, 37));
};


/**
 * @param {?proto.ubo.v1.DockerImageRemoveContainerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageRemoveContainerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 37, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageRemoveContainerEvent = function() {
  return this.setDockerImageRemoveContainerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageRemoveContainerEvent = function() {
  return jspb.Message.getField(this, 37) != null;
};


/**
 * optional DockerImageRemoveEvent docker_image_remove_event = 38;
 * @return {?proto.ubo.v1.DockerImageRemoveEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageRemoveEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageRemoveEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRemoveEvent, 38));
};


/**
 * @param {?proto.ubo.v1.DockerImageRemoveEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageRemoveEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 38, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageRemoveEvent = function() {
  return this.setDockerImageRemoveEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageRemoveEvent = function() {
  return jspb.Message.getField(this, 38) != null;
};


/**
 * optional DockerImageRunCompositionEvent docker_image_run_composition_event = 39;
 * @return {?proto.ubo.v1.DockerImageRunCompositionEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageRunCompositionEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageRunCompositionEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRunCompositionEvent, 39));
};


/**
 * @param {?proto.ubo.v1.DockerImageRunCompositionEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageRunCompositionEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 39, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageRunCompositionEvent = function() {
  return this.setDockerImageRunCompositionEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageRunCompositionEvent = function() {
  return jspb.Message.getField(this, 39) != null;
};


/**
 * optional DockerImageRunContainerEvent docker_image_run_container_event = 40;
 * @return {?proto.ubo.v1.DockerImageRunContainerEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageRunContainerEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageRunContainerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageRunContainerEvent, 40));
};


/**
 * @param {?proto.ubo.v1.DockerImageRunContainerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageRunContainerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 40, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageRunContainerEvent = function() {
  return this.setDockerImageRunContainerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageRunContainerEvent = function() {
  return jspb.Message.getField(this, 40) != null;
};


/**
 * optional DockerImageStopCompositionEvent docker_image_stop_composition_event = 41;
 * @return {?proto.ubo.v1.DockerImageStopCompositionEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageStopCompositionEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageStopCompositionEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageStopCompositionEvent, 41));
};


/**
 * @param {?proto.ubo.v1.DockerImageStopCompositionEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageStopCompositionEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 41, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageStopCompositionEvent = function() {
  return this.setDockerImageStopCompositionEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageStopCompositionEvent = function() {
  return jspb.Message.getField(this, 41) != null;
};


/**
 * optional DockerImageStopContainerEvent docker_image_stop_container_event = 42;
 * @return {?proto.ubo.v1.DockerImageStopContainerEvent}
 */
proto.ubo.v1.Event.prototype.getDockerImageStopContainerEvent = function() {
  return /** @type{?proto.ubo.v1.DockerImageStopContainerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerImageStopContainerEvent, 42));
};


/**
 * @param {?proto.ubo.v1.DockerImageStopContainerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerImageStopContainerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 42, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerImageStopContainerEvent = function() {
  return this.setDockerImageStopContainerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerImageStopContainerEvent = function() {
  return jspb.Message.getField(this, 42) != null;
};


/**
 * optional DockerInstallEvent docker_install_event = 43;
 * @return {?proto.ubo.v1.DockerInstallEvent}
 */
proto.ubo.v1.Event.prototype.getDockerInstallEvent = function() {
  return /** @type{?proto.ubo.v1.DockerInstallEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerInstallEvent, 43));
};


/**
 * @param {?proto.ubo.v1.DockerInstallEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerInstallEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 43, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerInstallEvent = function() {
  return this.setDockerInstallEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerInstallEvent = function() {
  return jspb.Message.getField(this, 43) != null;
};


/**
 * optional DockerStartEvent docker_start_event = 44;
 * @return {?proto.ubo.v1.DockerStartEvent}
 */
proto.ubo.v1.Event.prototype.getDockerStartEvent = function() {
  return /** @type{?proto.ubo.v1.DockerStartEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStartEvent, 44));
};


/**
 * @param {?proto.ubo.v1.DockerStartEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerStartEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 44, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerStartEvent = function() {
  return this.setDockerStartEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerStartEvent = function() {
  return jspb.Message.getField(this, 44) != null;
};


/**
 * optional DockerStopEvent docker_stop_event = 45;
 * @return {?proto.ubo.v1.DockerStopEvent}
 */
proto.ubo.v1.Event.prototype.getDockerStopEvent = function() {
  return /** @type{?proto.ubo.v1.DockerStopEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.DockerStopEvent, 45));
};


/**
 * @param {?proto.ubo.v1.DockerStopEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setDockerStopEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 45, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearDockerStopEvent = function() {
  return this.setDockerStopEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasDockerStopEvent = function() {
  return jspb.Message.getField(this, 45) != null;
};


/**
 * optional FileSystemCopyEvent file_system_copy_event = 46;
 * @return {?proto.ubo.v1.FileSystemCopyEvent}
 */
proto.ubo.v1.Event.prototype.getFileSystemCopyEvent = function() {
  return /** @type{?proto.ubo.v1.FileSystemCopyEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemCopyEvent, 46));
};


/**
 * @param {?proto.ubo.v1.FileSystemCopyEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setFileSystemCopyEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 46, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearFileSystemCopyEvent = function() {
  return this.setFileSystemCopyEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasFileSystemCopyEvent = function() {
  return jspb.Message.getField(this, 46) != null;
};


/**
 * optional FileSystemEvent file_system_event = 47;
 * @return {?proto.ubo.v1.FileSystemEvent}
 */
proto.ubo.v1.Event.prototype.getFileSystemEvent = function() {
  return /** @type{?proto.ubo.v1.FileSystemEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemEvent, 47));
};


/**
 * @param {?proto.ubo.v1.FileSystemEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setFileSystemEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 47, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearFileSystemEvent = function() {
  return this.setFileSystemEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasFileSystemEvent = function() {
  return jspb.Message.getField(this, 47) != null;
};


/**
 * optional FileSystemMoveEvent file_system_move_event = 48;
 * @return {?proto.ubo.v1.FileSystemMoveEvent}
 */
proto.ubo.v1.Event.prototype.getFileSystemMoveEvent = function() {
  return /** @type{?proto.ubo.v1.FileSystemMoveEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemMoveEvent, 48));
};


/**
 * @param {?proto.ubo.v1.FileSystemMoveEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setFileSystemMoveEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 48, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearFileSystemMoveEvent = function() {
  return this.setFileSystemMoveEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasFileSystemMoveEvent = function() {
  return jspb.Message.getField(this, 48) != null;
};


/**
 * optional FileSystemRemoveEvent file_system_remove_event = 49;
 * @return {?proto.ubo.v1.FileSystemRemoveEvent}
 */
proto.ubo.v1.Event.prototype.getFileSystemRemoveEvent = function() {
  return /** @type{?proto.ubo.v1.FileSystemRemoveEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemRemoveEvent, 49));
};


/**
 * @param {?proto.ubo.v1.FileSystemRemoveEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setFileSystemRemoveEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 49, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearFileSystemRemoveEvent = function() {
  return this.setFileSystemRemoveEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasFileSystemRemoveEvent = function() {
  return jspb.Message.getField(this, 49) != null;
};


/**
 * optional FileSystemSelectEvent file_system_select_event = 50;
 * @return {?proto.ubo.v1.FileSystemSelectEvent}
 */
proto.ubo.v1.Event.prototype.getFileSystemSelectEvent = function() {
  return /** @type{?proto.ubo.v1.FileSystemSelectEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.FileSystemSelectEvent, 50));
};


/**
 * @param {?proto.ubo.v1.FileSystemSelectEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setFileSystemSelectEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 50, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearFileSystemSelectEvent = function() {
  return this.setFileSystemSelectEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasFileSystemSelectEvent = function() {
  return jspb.Message.getField(this, 50) != null;
};


/**
 * optional InfraredEvent infrared_event = 51;
 * @return {?proto.ubo.v1.InfraredEvent}
 */
proto.ubo.v1.Event.prototype.getInfraredEvent = function() {
  return /** @type{?proto.ubo.v1.InfraredEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredEvent, 51));
};


/**
 * @param {?proto.ubo.v1.InfraredEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setInfraredEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 51, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearInfraredEvent = function() {
  return this.setInfraredEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasInfraredEvent = function() {
  return jspb.Message.getField(this, 51) != null;
};


/**
 * optional InfraredSendCodeEvent infrared_send_code_event = 52;
 * @return {?proto.ubo.v1.InfraredSendCodeEvent}
 */
proto.ubo.v1.Event.prototype.getInfraredSendCodeEvent = function() {
  return /** @type{?proto.ubo.v1.InfraredSendCodeEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InfraredSendCodeEvent, 52));
};


/**
 * @param {?proto.ubo.v1.InfraredSendCodeEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setInfraredSendCodeEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 52, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearInfraredSendCodeEvent = function() {
  return this.setInfraredSendCodeEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasInfraredSendCodeEvent = function() {
  return jspb.Message.getField(this, 52) != null;
};


/**
 * optional InitEvent init_event = 53;
 * @return {?proto.ubo.v1.InitEvent}
 */
proto.ubo.v1.Event.prototype.getInitEvent = function() {
  return /** @type{?proto.ubo.v1.InitEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InitEvent, 53));
};


/**
 * @param {?proto.ubo.v1.InitEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setInitEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 53, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearInitEvent = function() {
  return this.setInitEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasInitEvent = function() {
  return jspb.Message.getField(this, 53) != null;
};


/**
 * optional InputCancelEvent input_cancel_event = 54;
 * @return {?proto.ubo.v1.InputCancelEvent}
 */
proto.ubo.v1.Event.prototype.getInputCancelEvent = function() {
  return /** @type{?proto.ubo.v1.InputCancelEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputCancelEvent, 54));
};


/**
 * @param {?proto.ubo.v1.InputCancelEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setInputCancelEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 54, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearInputCancelEvent = function() {
  return this.setInputCancelEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasInputCancelEvent = function() {
  return jspb.Message.getField(this, 54) != null;
};


/**
 * optional InputProvideEvent input_provide_event = 55;
 * @return {?proto.ubo.v1.InputProvideEvent}
 */
proto.ubo.v1.Event.prototype.getInputProvideEvent = function() {
  return /** @type{?proto.ubo.v1.InputProvideEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputProvideEvent, 55));
};


/**
 * @param {?proto.ubo.v1.InputProvideEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setInputProvideEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 55, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearInputProvideEvent = function() {
  return this.setInputProvideEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasInputProvideEvent = function() {
  return jspb.Message.getField(this, 55) != null;
};


/**
 * optional InputResolveEvent input_resolve_event = 56;
 * @return {?proto.ubo.v1.InputResolveEvent}
 */
proto.ubo.v1.Event.prototype.getInputResolveEvent = function() {
  return /** @type{?proto.ubo.v1.InputResolveEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.InputResolveEvent, 56));
};


/**
 * @param {?proto.ubo.v1.InputResolveEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setInputResolveEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 56, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearInputResolveEvent = function() {
  return this.setInputResolveEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasInputResolveEvent = function() {
  return jspb.Message.getField(this, 56) != null;
};


/**
 * optional IpEvent ip_event = 57;
 * @return {?proto.ubo.v1.IpEvent}
 */
proto.ubo.v1.Event.prototype.getIpEvent = function() {
  return /** @type{?proto.ubo.v1.IpEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.IpEvent, 57));
};


/**
 * @param {?proto.ubo.v1.IpEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setIpEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 57, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearIpEvent = function() {
  return this.setIpEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasIpEvent = function() {
  return jspb.Message.getField(this, 57) != null;
};


/**
 * optional MainEvent main_event = 58;
 * @return {?proto.ubo.v1.MainEvent}
 */
proto.ubo.v1.Event.prototype.getMainEvent = function() {
  return /** @type{?proto.ubo.v1.MainEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MainEvent, 58));
};


/**
 * @param {?proto.ubo.v1.MainEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMainEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 58, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMainEvent = function() {
  return this.setMainEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMainEvent = function() {
  return jspb.Message.getField(this, 58) != null;
};


/**
 * optional MenuChooseByIconEvent menu_choose_by_icon_event = 59;
 * @return {?proto.ubo.v1.MenuChooseByIconEvent}
 */
proto.ubo.v1.Event.prototype.getMenuChooseByIconEvent = function() {
  return /** @type{?proto.ubo.v1.MenuChooseByIconEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIconEvent, 59));
};


/**
 * @param {?proto.ubo.v1.MenuChooseByIconEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMenuChooseByIconEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 59, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMenuChooseByIconEvent = function() {
  return this.setMenuChooseByIconEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMenuChooseByIconEvent = function() {
  return jspb.Message.getField(this, 59) != null;
};


/**
 * optional MenuChooseByIndexEvent menu_choose_by_index_event = 60;
 * @return {?proto.ubo.v1.MenuChooseByIndexEvent}
 */
proto.ubo.v1.Event.prototype.getMenuChooseByIndexEvent = function() {
  return /** @type{?proto.ubo.v1.MenuChooseByIndexEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByIndexEvent, 60));
};


/**
 * @param {?proto.ubo.v1.MenuChooseByIndexEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMenuChooseByIndexEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 60, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMenuChooseByIndexEvent = function() {
  return this.setMenuChooseByIndexEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMenuChooseByIndexEvent = function() {
  return jspb.Message.getField(this, 60) != null;
};


/**
 * optional MenuChooseByLabelEvent menu_choose_by_label_event = 61;
 * @return {?proto.ubo.v1.MenuChooseByLabelEvent}
 */
proto.ubo.v1.Event.prototype.getMenuChooseByLabelEvent = function() {
  return /** @type{?proto.ubo.v1.MenuChooseByLabelEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuChooseByLabelEvent, 61));
};


/**
 * @param {?proto.ubo.v1.MenuChooseByLabelEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMenuChooseByLabelEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 61, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMenuChooseByLabelEvent = function() {
  return this.setMenuChooseByLabelEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMenuChooseByLabelEvent = function() {
  return jspb.Message.getField(this, 61) != null;
};


/**
 * optional MenuEvent menu_event = 62;
 * @return {?proto.ubo.v1.MenuEvent}
 */
proto.ubo.v1.Event.prototype.getMenuEvent = function() {
  return /** @type{?proto.ubo.v1.MenuEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuEvent, 62));
};


/**
 * @param {?proto.ubo.v1.MenuEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMenuEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 62, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMenuEvent = function() {
  return this.setMenuEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMenuEvent = function() {
  return jspb.Message.getField(this, 62) != null;
};


/**
 * optional MenuGoBackEvent menu_go_back_event = 63;
 * @return {?proto.ubo.v1.MenuGoBackEvent}
 */
proto.ubo.v1.Event.prototype.getMenuGoBackEvent = function() {
  return /** @type{?proto.ubo.v1.MenuGoBackEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoBackEvent, 63));
};


/**
 * @param {?proto.ubo.v1.MenuGoBackEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMenuGoBackEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 63, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMenuGoBackEvent = function() {
  return this.setMenuGoBackEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMenuGoBackEvent = function() {
  return jspb.Message.getField(this, 63) != null;
};


/**
 * optional MenuGoHomeEvent menu_go_home_event = 64;
 * @return {?proto.ubo.v1.MenuGoHomeEvent}
 */
proto.ubo.v1.Event.prototype.getMenuGoHomeEvent = function() {
  return /** @type{?proto.ubo.v1.MenuGoHomeEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuGoHomeEvent, 64));
};


/**
 * @param {?proto.ubo.v1.MenuGoHomeEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMenuGoHomeEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 64, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMenuGoHomeEvent = function() {
  return this.setMenuGoHomeEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMenuGoHomeEvent = function() {
  return jspb.Message.getField(this, 64) != null;
};


/**
 * optional MenuScrollEvent menu_scroll_event = 65;
 * @return {?proto.ubo.v1.MenuScrollEvent}
 */
proto.ubo.v1.Event.prototype.getMenuScrollEvent = function() {
  return /** @type{?proto.ubo.v1.MenuScrollEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.MenuScrollEvent, 65));
};


/**
 * @param {?proto.ubo.v1.MenuScrollEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setMenuScrollEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 65, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearMenuScrollEvent = function() {
  return this.setMenuScrollEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasMenuScrollEvent = function() {
  return jspb.Message.getField(this, 65) != null;
};


/**
 * optional NotificationsClearEvent notifications_clear_event = 66;
 * @return {?proto.ubo.v1.NotificationsClearEvent}
 */
proto.ubo.v1.Event.prototype.getNotificationsClearEvent = function() {
  return /** @type{?proto.ubo.v1.NotificationsClearEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsClearEvent, 66));
};


/**
 * @param {?proto.ubo.v1.NotificationsClearEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setNotificationsClearEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 66, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearNotificationsClearEvent = function() {
  return this.setNotificationsClearEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasNotificationsClearEvent = function() {
  return jspb.Message.getField(this, 66) != null;
};


/**
 * optional NotificationsDisplayEvent notifications_display_event = 67;
 * @return {?proto.ubo.v1.NotificationsDisplayEvent}
 */
proto.ubo.v1.Event.prototype.getNotificationsDisplayEvent = function() {
  return /** @type{?proto.ubo.v1.NotificationsDisplayEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsDisplayEvent, 67));
};


/**
 * @param {?proto.ubo.v1.NotificationsDisplayEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setNotificationsDisplayEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 67, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearNotificationsDisplayEvent = function() {
  return this.setNotificationsDisplayEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasNotificationsDisplayEvent = function() {
  return jspb.Message.getField(this, 67) != null;
};


/**
 * optional NotificationsEvent notifications_event = 68;
 * @return {?proto.ubo.v1.NotificationsEvent}
 */
proto.ubo.v1.Event.prototype.getNotificationsEvent = function() {
  return /** @type{?proto.ubo.v1.NotificationsEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.NotificationsEvent, 68));
};


/**
 * @param {?proto.ubo.v1.NotificationsEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setNotificationsEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 68, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearNotificationsEvent = function() {
  return this.setNotificationsEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasNotificationsEvent = function() {
  return jspb.Message.getField(this, 68) != null;
};


/**
 * optional OpenApplicationEvent open_application_event = 69;
 * @return {?proto.ubo.v1.OpenApplicationEvent}
 */
proto.ubo.v1.Event.prototype.getOpenApplicationEvent = function() {
  return /** @type{?proto.ubo.v1.OpenApplicationEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.OpenApplicationEvent, 69));
};


/**
 * @param {?proto.ubo.v1.OpenApplicationEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setOpenApplicationEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 69, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearOpenApplicationEvent = function() {
  return this.setOpenApplicationEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasOpenApplicationEvent = function() {
  return jspb.Message.getField(this, 69) != null;
};


/**
 * optional PowerEvent power_event = 70;
 * @return {?proto.ubo.v1.PowerEvent}
 */
proto.ubo.v1.Event.prototype.getPowerEvent = function() {
  return /** @type{?proto.ubo.v1.PowerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerEvent, 70));
};


/**
 * @param {?proto.ubo.v1.PowerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setPowerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 70, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearPowerEvent = function() {
  return this.setPowerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasPowerEvent = function() {
  return jspb.Message.getField(this, 70) != null;
};


/**
 * optional PowerOffEvent power_off_event = 71;
 * @return {?proto.ubo.v1.PowerOffEvent}
 */
proto.ubo.v1.Event.prototype.getPowerOffEvent = function() {
  return /** @type{?proto.ubo.v1.PowerOffEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.PowerOffEvent, 71));
};


/**
 * @param {?proto.ubo.v1.PowerOffEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setPowerOffEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 71, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearPowerOffEvent = function() {
  return this.setPowerOffEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasPowerOffEvent = function() {
  return jspb.Message.getField(this, 71) != null;
};


/**
 * optional RPiConnectEvent r_pi_connect_event = 72;
 * @return {?proto.ubo.v1.RPiConnectEvent}
 */
proto.ubo.v1.Event.prototype.getRPiConnectEvent = function() {
  return /** @type{?proto.ubo.v1.RPiConnectEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectEvent, 72));
};


/**
 * @param {?proto.ubo.v1.RPiConnectEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setRPiConnectEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 72, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearRPiConnectEvent = function() {
  return this.setRPiConnectEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasRPiConnectEvent = function() {
  return jspb.Message.getField(this, 72) != null;
};


/**
 * optional RPiConnectLoginEvent r_pi_connect_login_event = 73;
 * @return {?proto.ubo.v1.RPiConnectLoginEvent}
 */
proto.ubo.v1.Event.prototype.getRPiConnectLoginEvent = function() {
  return /** @type{?proto.ubo.v1.RPiConnectLoginEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RPiConnectLoginEvent, 73));
};


/**
 * @param {?proto.ubo.v1.RPiConnectLoginEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setRPiConnectLoginEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 73, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearRPiConnectLoginEvent = function() {
  return this.setRPiConnectLoginEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasRPiConnectLoginEvent = function() {
  return jspb.Message.getField(this, 73) != null;
};


/**
 * optional RebootEvent reboot_event = 74;
 * @return {?proto.ubo.v1.RebootEvent}
 */
proto.ubo.v1.Event.prototype.getRebootEvent = function() {
  return /** @type{?proto.ubo.v1.RebootEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RebootEvent, 74));
};


/**
 * @param {?proto.ubo.v1.RebootEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setRebootEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 74, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearRebootEvent = function() {
  return this.setRebootEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasRebootEvent = function() {
  return jspb.Message.getField(this, 74) != null;
};


/**
 * optional ReplayRecordedSequenceEvent replay_recorded_sequence_event = 75;
 * @return {?proto.ubo.v1.ReplayRecordedSequenceEvent}
 */
proto.ubo.v1.Event.prototype.getReplayRecordedSequenceEvent = function() {
  return /** @type{?proto.ubo.v1.ReplayRecordedSequenceEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ReplayRecordedSequenceEvent, 75));
};


/**
 * @param {?proto.ubo.v1.ReplayRecordedSequenceEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setReplayRecordedSequenceEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 75, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearReplayRecordedSequenceEvent = function() {
  return this.setReplayRecordedSequenceEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasReplayRecordedSequenceEvent = function() {
  return jspb.Message.getField(this, 75) != null;
};


/**
 * optional RgbRingCommandEvent rgb_ring_command_event = 76;
 * @return {?proto.ubo.v1.RgbRingCommandEvent}
 */
proto.ubo.v1.Event.prototype.getRgbRingCommandEvent = function() {
  return /** @type{?proto.ubo.v1.RgbRingCommandEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingCommandEvent, 76));
};


/**
 * @param {?proto.ubo.v1.RgbRingCommandEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setRgbRingCommandEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 76, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearRgbRingCommandEvent = function() {
  return this.setRgbRingCommandEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasRgbRingCommandEvent = function() {
  return jspb.Message.getField(this, 76) != null;
};


/**
 * optional RgbRingEvent rgb_ring_event = 77;
 * @return {?proto.ubo.v1.RgbRingEvent}
 */
proto.ubo.v1.Event.prototype.getRgbRingEvent = function() {
  return /** @type{?proto.ubo.v1.RgbRingEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.RgbRingEvent, 77));
};


/**
 * @param {?proto.ubo.v1.RgbRingEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setRgbRingEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 77, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearRgbRingEvent = function() {
  return this.setRgbRingEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasRgbRingEvent = function() {
  return jspb.Message.getField(this, 77) != null;
};


/**
 * optional ScreenshotEvent screenshot_event = 78;
 * @return {?proto.ubo.v1.ScreenshotEvent}
 */
proto.ubo.v1.Event.prototype.getScreenshotEvent = function() {
  return /** @type{?proto.ubo.v1.ScreenshotEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.ScreenshotEvent, 78));
};


/**
 * @param {?proto.ubo.v1.ScreenshotEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setScreenshotEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 78, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearScreenshotEvent = function() {
  return this.setScreenshotEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasScreenshotEvent = function() {
  return jspb.Message.getField(this, 78) != null;
};


/**
 * optional SettingsEvent settings_event = 79;
 * @return {?proto.ubo.v1.SettingsEvent}
 */
proto.ubo.v1.Event.prototype.getSettingsEvent = function() {
  return /** @type{?proto.ubo.v1.SettingsEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsEvent, 79));
};


/**
 * @param {?proto.ubo.v1.SettingsEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSettingsEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 79, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSettingsEvent = function() {
  return this.setSettingsEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSettingsEvent = function() {
  return jspb.Message.getField(this, 79) != null;
};


/**
 * optional SettingsServiceEvent settings_service_event = 80;
 * @return {?proto.ubo.v1.SettingsServiceEvent}
 */
proto.ubo.v1.Event.prototype.getSettingsServiceEvent = function() {
  return /** @type{?proto.ubo.v1.SettingsServiceEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsServiceEvent, 80));
};


/**
 * @param {?proto.ubo.v1.SettingsServiceEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSettingsServiceEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 80, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSettingsServiceEvent = function() {
  return this.setSettingsServiceEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSettingsServiceEvent = function() {
  return jspb.Message.getField(this, 80) != null;
};


/**
 * optional SettingsStartServiceEvent settings_start_service_event = 81;
 * @return {?proto.ubo.v1.SettingsStartServiceEvent}
 */
proto.ubo.v1.Event.prototype.getSettingsStartServiceEvent = function() {
  return /** @type{?proto.ubo.v1.SettingsStartServiceEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStartServiceEvent, 81));
};


/**
 * @param {?proto.ubo.v1.SettingsStartServiceEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSettingsStartServiceEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 81, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSettingsStartServiceEvent = function() {
  return this.setSettingsStartServiceEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSettingsStartServiceEvent = function() {
  return jspb.Message.getField(this, 81) != null;
};


/**
 * optional SettingsStopServiceEvent settings_stop_service_event = 82;
 * @return {?proto.ubo.v1.SettingsStopServiceEvent}
 */
proto.ubo.v1.Event.prototype.getSettingsStopServiceEvent = function() {
  return /** @type{?proto.ubo.v1.SettingsStopServiceEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SettingsStopServiceEvent, 82));
};


/**
 * @param {?proto.ubo.v1.SettingsStopServiceEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSettingsStopServiceEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 82, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSettingsStopServiceEvent = function() {
  return this.setSettingsStopServiceEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSettingsStopServiceEvent = function() {
  return jspb.Message.getField(this, 82) != null;
};


/**
 * optional SnapshotEvent snapshot_event = 83;
 * @return {?proto.ubo.v1.SnapshotEvent}
 */
proto.ubo.v1.Event.prototype.getSnapshotEvent = function() {
  return /** @type{?proto.ubo.v1.SnapshotEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SnapshotEvent, 83));
};


/**
 * @param {?proto.ubo.v1.SnapshotEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSnapshotEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 83, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSnapshotEvent = function() {
  return this.setSnapshotEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSnapshotEvent = function() {
  return jspb.Message.getField(this, 83) != null;
};


/**
 * optional SpeechRecognitionEvent speech_recognition_event = 84;
 * @return {?proto.ubo.v1.SpeechRecognitionEvent}
 */
proto.ubo.v1.Event.prototype.getSpeechRecognitionEvent = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionEvent, 84));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSpeechRecognitionEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 84, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSpeechRecognitionEvent = function() {
  return this.setSpeechRecognitionEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSpeechRecognitionEvent = function() {
  return jspb.Message.getField(this, 84) != null;
};


/**
 * optional SpeechRecognitionReportTextEvent speech_recognition_report_text_event = 85;
 * @return {?proto.ubo.v1.SpeechRecognitionReportTextEvent}
 */
proto.ubo.v1.Event.prototype.getSpeechRecognitionReportTextEvent = function() {
  return /** @type{?proto.ubo.v1.SpeechRecognitionReportTextEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechRecognitionReportTextEvent, 85));
};


/**
 * @param {?proto.ubo.v1.SpeechRecognitionReportTextEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSpeechRecognitionReportTextEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 85, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSpeechRecognitionReportTextEvent = function() {
  return this.setSpeechRecognitionReportTextEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSpeechRecognitionReportTextEvent = function() {
  return jspb.Message.getField(this, 85) != null;
};


/**
 * optional SpeechSynthesisEvent speech_synthesis_event = 86;
 * @return {?proto.ubo.v1.SpeechSynthesisEvent}
 */
proto.ubo.v1.Event.prototype.getSpeechSynthesisEvent = function() {
  return /** @type{?proto.ubo.v1.SpeechSynthesisEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisEvent, 86));
};


/**
 * @param {?proto.ubo.v1.SpeechSynthesisEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSpeechSynthesisEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 86, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSpeechSynthesisEvent = function() {
  return this.setSpeechSynthesisEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSpeechSynthesisEvent = function() {
  return jspb.Message.getField(this, 86) != null;
};


/**
 * optional SpeechSynthesisSynthesizeTextEvent speech_synthesis_synthesize_text_event = 87;
 * @return {?proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent}
 */
proto.ubo.v1.Event.prototype.getSpeechSynthesisSynthesizeTextEvent = function() {
  return /** @type{?proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent, 87));
};


/**
 * @param {?proto.ubo.v1.SpeechSynthesisSynthesizeTextEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setSpeechSynthesisSynthesizeTextEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 87, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearSpeechSynthesisSynthesizeTextEvent = function() {
  return this.setSpeechSynthesisSynthesizeTextEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasSpeechSynthesisSynthesizeTextEvent = function() {
  return jspb.Message.getField(this, 87) != null;
};


/**
 * optional StoreRecordedSequenceEvent store_recorded_sequence_event = 88;
 * @return {?proto.ubo.v1.StoreRecordedSequenceEvent}
 */
proto.ubo.v1.Event.prototype.getStoreRecordedSequenceEvent = function() {
  return /** @type{?proto.ubo.v1.StoreRecordedSequenceEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.StoreRecordedSequenceEvent, 88));
};


/**
 * @param {?proto.ubo.v1.StoreRecordedSequenceEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setStoreRecordedSequenceEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 88, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearStoreRecordedSequenceEvent = function() {
  return this.setStoreRecordedSequenceEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasStoreRecordedSequenceEvent = function() {
  return jspb.Message.getField(this, 88) != null;
};


/**
 * optional UpdateManagerCheckEvent update_manager_check_event = 89;
 * @return {?proto.ubo.v1.UpdateManagerCheckEvent}
 */
proto.ubo.v1.Event.prototype.getUpdateManagerCheckEvent = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerCheckEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerCheckEvent, 89));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerCheckEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setUpdateManagerCheckEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 89, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearUpdateManagerCheckEvent = function() {
  return this.setUpdateManagerCheckEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasUpdateManagerCheckEvent = function() {
  return jspb.Message.getField(this, 89) != null;
};


/**
 * optional UpdateManagerEvent update_manager_event = 90;
 * @return {?proto.ubo.v1.UpdateManagerEvent}
 */
proto.ubo.v1.Event.prototype.getUpdateManagerEvent = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerEvent, 90));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setUpdateManagerEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 90, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearUpdateManagerEvent = function() {
  return this.setUpdateManagerEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasUpdateManagerEvent = function() {
  return jspb.Message.getField(this, 90) != null;
};


/**
 * optional UpdateManagerUpdateEvent update_manager_update_event = 91;
 * @return {?proto.ubo.v1.UpdateManagerUpdateEvent}
 */
proto.ubo.v1.Event.prototype.getUpdateManagerUpdateEvent = function() {
  return /** @type{?proto.ubo.v1.UpdateManagerUpdateEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UpdateManagerUpdateEvent, 91));
};


/**
 * @param {?proto.ubo.v1.UpdateManagerUpdateEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setUpdateManagerUpdateEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 91, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearUpdateManagerUpdateEvent = function() {
  return this.setUpdateManagerUpdateEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasUpdateManagerUpdateEvent = function() {
  return jspb.Message.getField(this, 91) != null;
};


/**
 * optional UsersCreateUserEvent users_create_user_event = 92;
 * @return {?proto.ubo.v1.UsersCreateUserEvent}
 */
proto.ubo.v1.Event.prototype.getUsersCreateUserEvent = function() {
  return /** @type{?proto.ubo.v1.UsersCreateUserEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersCreateUserEvent, 92));
};


/**
 * @param {?proto.ubo.v1.UsersCreateUserEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setUsersCreateUserEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 92, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearUsersCreateUserEvent = function() {
  return this.setUsersCreateUserEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasUsersCreateUserEvent = function() {
  return jspb.Message.getField(this, 92) != null;
};


/**
 * optional UsersDeleteUserEvent users_delete_user_event = 93;
 * @return {?proto.ubo.v1.UsersDeleteUserEvent}
 */
proto.ubo.v1.Event.prototype.getUsersDeleteUserEvent = function() {
  return /** @type{?proto.ubo.v1.UsersDeleteUserEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersDeleteUserEvent, 93));
};


/**
 * @param {?proto.ubo.v1.UsersDeleteUserEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setUsersDeleteUserEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 93, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearUsersDeleteUserEvent = function() {
  return this.setUsersDeleteUserEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasUsersDeleteUserEvent = function() {
  return jspb.Message.getField(this, 93) != null;
};


/**
 * optional UsersEvent users_event = 94;
 * @return {?proto.ubo.v1.UsersEvent}
 */
proto.ubo.v1.Event.prototype.getUsersEvent = function() {
  return /** @type{?proto.ubo.v1.UsersEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersEvent, 94));
};


/**
 * @param {?proto.ubo.v1.UsersEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setUsersEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 94, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearUsersEvent = function() {
  return this.setUsersEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasUsersEvent = function() {
  return jspb.Message.getField(this, 94) != null;
};


/**
 * optional UsersResetPasswordEvent users_reset_password_event = 95;
 * @return {?proto.ubo.v1.UsersResetPasswordEvent}
 */
proto.ubo.v1.Event.prototype.getUsersResetPasswordEvent = function() {
  return /** @type{?proto.ubo.v1.UsersResetPasswordEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.UsersResetPasswordEvent, 95));
};


/**
 * @param {?proto.ubo.v1.UsersResetPasswordEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setUsersResetPasswordEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 95, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearUsersResetPasswordEvent = function() {
  return this.setUsersResetPasswordEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasUsersResetPasswordEvent = function() {
  return jspb.Message.getField(this, 95) != null;
};


/**
 * optional VSCodeEvent vs_code_event = 96;
 * @return {?proto.ubo.v1.VSCodeEvent}
 */
proto.ubo.v1.Event.prototype.getVsCodeEvent = function() {
  return /** @type{?proto.ubo.v1.VSCodeEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeEvent, 96));
};


/**
 * @param {?proto.ubo.v1.VSCodeEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setVsCodeEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 96, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearVsCodeEvent = function() {
  return this.setVsCodeEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasVsCodeEvent = function() {
  return jspb.Message.getField(this, 96) != null;
};


/**
 * optional VSCodeLoginEvent vs_code_login_event = 97;
 * @return {?proto.ubo.v1.VSCodeLoginEvent}
 */
proto.ubo.v1.Event.prototype.getVsCodeLoginEvent = function() {
  return /** @type{?proto.ubo.v1.VSCodeLoginEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeLoginEvent, 97));
};


/**
 * @param {?proto.ubo.v1.VSCodeLoginEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setVsCodeLoginEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 97, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearVsCodeLoginEvent = function() {
  return this.setVsCodeLoginEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasVsCodeLoginEvent = function() {
  return jspb.Message.getField(this, 97) != null;
};


/**
 * optional VSCodeRestartEvent vs_code_restart_event = 98;
 * @return {?proto.ubo.v1.VSCodeRestartEvent}
 */
proto.ubo.v1.Event.prototype.getVsCodeRestartEvent = function() {
  return /** @type{?proto.ubo.v1.VSCodeRestartEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.VSCodeRestartEvent, 98));
};


/**
 * @param {?proto.ubo.v1.VSCodeRestartEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setVsCodeRestartEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 98, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearVsCodeRestartEvent = function() {
  return this.setVsCodeRestartEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasVsCodeRestartEvent = function() {
  return jspb.Message.getField(this, 98) != null;
};


/**
 * optional WebUIEvent web_ui_event = 99;
 * @return {?proto.ubo.v1.WebUIEvent}
 */
proto.ubo.v1.Event.prototype.getWebUiEvent = function() {
  return /** @type{?proto.ubo.v1.WebUIEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIEvent, 99));
};


/**
 * @param {?proto.ubo.v1.WebUIEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setWebUiEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 99, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearWebUiEvent = function() {
  return this.setWebUiEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasWebUiEvent = function() {
  return jspb.Message.getField(this, 99) != null;
};


/**
 * optional WebUIInitializeEvent web_ui_initialize_event = 100;
 * @return {?proto.ubo.v1.WebUIInitializeEvent}
 */
proto.ubo.v1.Event.prototype.getWebUiInitializeEvent = function() {
  return /** @type{?proto.ubo.v1.WebUIInitializeEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIInitializeEvent, 100));
};


/**
 * @param {?proto.ubo.v1.WebUIInitializeEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setWebUiInitializeEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 100, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearWebUiInitializeEvent = function() {
  return this.setWebUiInitializeEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasWebUiInitializeEvent = function() {
  return jspb.Message.getField(this, 100) != null;
};


/**
 * optional WebUIStopEvent web_ui_stop_event = 101;
 * @return {?proto.ubo.v1.WebUIStopEvent}
 */
proto.ubo.v1.Event.prototype.getWebUiStopEvent = function() {
  return /** @type{?proto.ubo.v1.WebUIStopEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WebUIStopEvent, 101));
};


/**
 * @param {?proto.ubo.v1.WebUIStopEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setWebUiStopEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 101, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearWebUiStopEvent = function() {
  return this.setWebUiStopEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasWebUiStopEvent = function() {
  return jspb.Message.getField(this, 101) != null;
};


/**
 * optional WiFiEvent wi_fi_event = 102;
 * @return {?proto.ubo.v1.WiFiEvent}
 */
proto.ubo.v1.Event.prototype.getWiFiEvent = function() {
  return /** @type{?proto.ubo.v1.WiFiEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiEvent, 102));
};


/**
 * @param {?proto.ubo.v1.WiFiEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setWiFiEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 102, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearWiFiEvent = function() {
  return this.setWiFiEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasWiFiEvent = function() {
  return jspb.Message.getField(this, 102) != null;
};


/**
 * optional WiFiInputConnectionEvent wi_fi_input_connection_event = 103;
 * @return {?proto.ubo.v1.WiFiInputConnectionEvent}
 */
proto.ubo.v1.Event.prototype.getWiFiInputConnectionEvent = function() {
  return /** @type{?proto.ubo.v1.WiFiInputConnectionEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiInputConnectionEvent, 103));
};


/**
 * @param {?proto.ubo.v1.WiFiInputConnectionEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setWiFiInputConnectionEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 103, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearWiFiInputConnectionEvent = function() {
  return this.setWiFiInputConnectionEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasWiFiInputConnectionEvent = function() {
  return jspb.Message.getField(this, 103) != null;
};


/**
 * optional WiFiUpdateRequestEvent wi_fi_update_request_event = 104;
 * @return {?proto.ubo.v1.WiFiUpdateRequestEvent}
 */
proto.ubo.v1.Event.prototype.getWiFiUpdateRequestEvent = function() {
  return /** @type{?proto.ubo.v1.WiFiUpdateRequestEvent} */ (
    jspb.Message.getWrapperField(this, proto.ubo.v1.WiFiUpdateRequestEvent, 104));
};


/**
 * @param {?proto.ubo.v1.WiFiUpdateRequestEvent|undefined} value
 * @return {!proto.ubo.v1.Event} returns this
*/
proto.ubo.v1.Event.prototype.setWiFiUpdateRequestEvent = function(value) {
  return jspb.Message.setOneofWrapperField(this, 104, proto.ubo.v1.Event.oneofGroups_[0], value);
};


/**
 * Clears the message field making it undefined.
 * @return {!proto.ubo.v1.Event} returns this
 */
proto.ubo.v1.Event.prototype.clearWiFiUpdateRequestEvent = function() {
  return this.setWiFiUpdateRequestEvent(undefined);
};


/**
 * Returns whether this field is set.
 * @return {boolean}
 */
proto.ubo.v1.Event.prototype.hasWiFiUpdateRequestEvent = function() {
  return jspb.Message.getField(this, 104) != null;
};


/**
 * @enum {number}
 */
proto.ubo.v1.SettingsCategory = {
  SETTINGS_CATEGORY_UBO_APP_DOT_STORE_DOT_CORE_DOT_TYPES_UNSPECIFIED: 0,
  SETTINGS_CATEGORY_NETWORK: 1,
  SETTINGS_CATEGORY_REMOTE: 2,
  SETTINGS_CATEGORY_SYSTEM: 3,
  SETTINGS_CATEGORY_HARDWARE: 4,
  SETTINGS_CATEGORY_ASSISTANT: 5,
  SETTINGS_CATEGORY_DOCKER: 6,
  SETTINGS_CATEGORY_ACCESSIBILITY: 7
};

/**
 * @enum {number}
 */
proto.ubo.v1.MenuScrollDirection = {
  MENU_SCROLL_DIRECTION_UBO_APP_DOT_STORE_DOT_CORE_DOT_TYPES_UNSPECIFIED: 0,
  MENU_SCROLL_DIRECTION_UP: 1,
  MENU_SCROLL_DIRECTION_DOWN: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.InputMethod = {
  INPUT_METHOD_UBO_APP_DOT_STORE_DOT_INPUT_DOT_TYPES_UNSPECIFIED: 0,
  INPUT_METHOD_CAMERA: 1,
  INPUT_METHOD_WEB_DASHBOARD: 2,
  INPUT_METHOD_PATH_SELECTOR: 3
};

/**
 * @enum {number}
 */
proto.ubo.v1.InputFieldType = {
  INPUT_FIELD_TYPE_UBO_APP_DOT_STORE_DOT_INPUT_DOT_TYPES_UNSPECIFIED: 0,
  INPUT_FIELD_TYPE_LONG: 1,
  INPUT_FIELD_TYPE_TEXT: 2,
  INPUT_FIELD_TYPE_PASSWORD: 3,
  INPUT_FIELD_TYPE_NUMBER: 4,
  INPUT_FIELD_TYPE_CHECKBOX: 5,
  INPUT_FIELD_TYPE_COLOR: 6,
  INPUT_FIELD_TYPE_SELECT: 7,
  INPUT_FIELD_TYPE_FILE: 8,
  INPUT_FIELD_TYPE_DATE: 9,
  INPUT_FIELD_TYPE_TIME: 10
};

/**
 * @enum {number}
 */
proto.ubo.v1.ServicesStatus = {
  SERVICES_STATUS_UBO_APP_DOT_STORE_DOT_SETTINGS_DOT_TYPES_UNSPECIFIED: 0,
  SERVICES_STATUS_LOADING: 1,
  SERVICES_STATUS_READY: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.UpdateStatus = {
  UPDATE_STATUS_UBO_APP_DOT_STORE_DOT_UPDATE_MANAGER_DOT_TYPES_UNSPECIFIED: 0,
  UPDATE_STATUS_CHECKING: 1,
  UPDATE_STATUS_FAILED_TO_CHECK: 2,
  UPDATE_STATUS_UP_TO_DATE: 3,
  UPDATE_STATUS_OUTDATED: 4,
  UPDATE_STATUS_UPDATING: 5
};

/**
 * @enum {number}
 */
proto.ubo.v1.AssistantSTTName = {
  ASSISTANT_STT_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,
  ASSISTANT_STT_NAME_VOSK: 1,
  ASSISTANT_STT_NAME_GOOGLE_SEGMENTED: 2,
  ASSISTANT_STT_NAME_GOOGLE: 3,
  ASSISTANT_STT_NAME_OPENAI: 4,
  ASSISTANT_STT_NAME_DEEPGRAM: 5,
  ASSISTANT_STT_NAME_ASSEMBLYAI: 6
};

/**
 * @enum {number}
 */
proto.ubo.v1.AssistantLLMName = {
  ASSISTANT_LLM_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,
  ASSISTANT_LLM_NAME_OLLAMA: 1,
  ASSISTANT_LLM_NAME_OLLAMA_ONPREM: 2,
  ASSISTANT_LLM_NAME_GOOGLE: 3,
  ASSISTANT_LLM_NAME_OPENAI: 4,
  ASSISTANT_LLM_NAME_GROK: 5,
  ASSISTANT_LLM_NAME_CEREBRAS: 6
};

/**
 * @enum {number}
 */
proto.ubo.v1.AssistantTTSName = {
  ASSISTANT_TTS_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,
  ASSISTANT_TTS_NAME_PIPER: 1,
  ASSISTANT_TTS_NAME_GOOGLE: 2,
  ASSISTANT_TTS_NAME_OPENAI: 3,
  ASSISTANT_TTS_NAME_ELEVENLABS: 4,
  ASSISTANT_TTS_NAME_RIME: 5
};

/**
 * @enum {number}
 */
proto.ubo.v1.AssistantImageGeneratorName = {
  ASSISTANT_IMAGE_GENERATOR_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,
  ASSISTANT_IMAGE_GENERATOR_NAME_GOOGLE: 1,
  ASSISTANT_IMAGE_GENERATOR_NAME_OPENAI: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.McpServerType = {
  MCP_SERVER_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ASSISTANT_UNSPECIFIED: 0,
  MCP_SERVER_TYPE_STDIO: 1,
  MCP_SERVER_TYPE_SSE: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.AudioDevice = {
  AUDIO_DEVICE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_AUDIO_UNSPECIFIED: 0,
  AUDIO_DEVICE_INPUT: 1,
  AUDIO_DEVICE_OUTPUT: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.CameraType = {
  CAMERA_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_CAMERA_UNSPECIFIED: 0,
  CAMERA_TYPE_DEFAULT: 1,
  CAMERA_TYPE_AUTOFOCUS: 2,
  CAMERA_TYPE_FIXED_FOCUS: 3
};

/**
 * @enum {number}
 */
proto.ubo.v1.DisplayBlankTimeout = {
  DISPLAY_BLANK_TIMEOUT_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_DISPLAY_UNSPECIFIED: 0,
  DISPLAY_BLANK_TIMEOUT_ONE_MINUTE: 1,
  DISPLAY_BLANK_TIMEOUT_FIVE_MINUTES: 2,
  DISPLAY_BLANK_TIMEOUT_TEN_MINUTES: 3,
  DISPLAY_BLANK_TIMEOUT_THIRTY_MINUTES: 4,
  DISPLAY_BLANK_TIMEOUT_ONE_HOUR: 5,
  DISPLAY_BLANK_TIMEOUT_OFF: 6
};

/**
 * @enum {number}
 */
proto.ubo.v1.DockerStatus = {
  DOCKER_STATUS_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_DOCKER_UNSPECIFIED: 0,
  DOCKER_STATUS_UNKNOWN: 1,
  DOCKER_STATUS_NOT_INSTALLED: 2,
  DOCKER_STATUS_INSTALLING: 3,
  DOCKER_STATUS_NOT_RUNNING: 4,
  DOCKER_STATUS_RUNNING: 5,
  DOCKER_STATUS_ERROR: 6
};

/**
 * @enum {number}
 */
proto.ubo.v1.DockerItemStatus = {
  DOCKER_ITEM_STATUS_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_DOCKER_UNSPECIFIED: 0,
  DOCKER_ITEM_STATUS_NOT_AVAILABLE: 1,
  DOCKER_ITEM_STATUS_FETCHING: 2,
  DOCKER_ITEM_STATUS_AVAILABLE: 3,
  DOCKER_ITEM_STATUS_CREATED: 4,
  DOCKER_ITEM_STATUS_RUNNING: 5,
  DOCKER_ITEM_STATUS_ERROR: 6,
  DOCKER_ITEM_STATUS_PROCESSING: 7
};

/**
 * @enum {number}
 */
proto.ubo.v1.NetState = {
  NET_STATE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_ETHERNET_UNSPECIFIED: 0,
  NET_STATE_CONNECTED: 1,
  NET_STATE_DISCONNECTED: 2,
  NET_STATE_PENDING: 3,
  NET_STATE_NEEDS_ATTENTION: 4,
  NET_STATE_UNKNOWN: 5
};

/**
 * @enum {number}
 */
proto.ubo.v1.Key = {
  KEY_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_KEYPAD_UNSPECIFIED: 0,
  KEY_BACK: 1,
  KEY_HOME: 2,
  KEY_UP: 3,
  KEY_DOWN: 4,
  KEY_L1: 5,
  KEY_L2: 6,
  KEY_L3: 7
};

/**
 * @enum {number}
 */
proto.ubo.v1.Importance = {
  IMPORTANCE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_NOTIFICATIONS_UNSPECIFIED: 0,
  IMPORTANCE_CRITICAL: 1,
  IMPORTANCE_HIGH: 2,
  IMPORTANCE_MEDIUM: 3,
  IMPORTANCE_LOW: 4
};

/**
 * @enum {number}
 */
proto.ubo.v1.NotificationDisplayType = {
  NOTIFICATION_DISPLAY_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_NOTIFICATIONS_UNSPECIFIED: 0,
  NOTIFICATION_DISPLAY_TYPE_NOT_SET: 1,
  NOTIFICATION_DISPLAY_TYPE_BACKGROUND: 2,
  NOTIFICATION_DISPLAY_TYPE_FLASH: 3,
  NOTIFICATION_DISPLAY_TYPE_STICKY: 4
};

/**
 * @enum {number}
 */
proto.ubo.v1.Chime = {
  CHIME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_NOTIFICATIONS_UNSPECIFIED: 0,
  CHIME_ADD: 1,
  CHIME_DONE: 2,
  CHIME_FAILURE: 3,
  CHIME_VOLUME_CHANGE: 4
};

/**
 * @enum {number}
 */
proto.ubo.v1.Sensor = {
  SENSOR_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SENSORS_UNSPECIFIED: 0,
  SENSOR_TEMPERATURE: 1,
  SENSOR_LIGHT: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.SpeechRecognitionStatus = {
  SPEECH_RECOGNITION_STATUS_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SPEECH_RECOGNITION_UNSPECIFIED: 0,
  SPEECH_RECOGNITION_STATUS_IDLE: 1,
  SPEECH_RECOGNITION_STATUS_INTENTS_WAITING: 2,
  SPEECH_RECOGNITION_STATUS_ASSISTANT_WAITING: 3
};

/**
 * @enum {number}
 */
proto.ubo.v1.SpeechRecognitionEngineName = {
  SPEECH_RECOGNITION_ENGINE_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SPEECH_RECOGNITION_UNSPECIFIED: 0,
  SPEECH_RECOGNITION_ENGINE_NAME_VOSK: 1,
  SPEECH_RECOGNITION_ENGINE_NAME_GOOGLE: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.SpeechSynthesisEngineName = {
  SPEECH_SYNTHESIS_ENGINE_NAME_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_SPEECH_SYNTHESIS_UNSPECIFIED: 0,
  SPEECH_SYNTHESIS_ENGINE_NAME_PIPER: 1,
  SPEECH_SYNTHESIS_ENGINE_NAME_PICOVOICE: 2
};

/**
 * @enum {number}
 */
proto.ubo.v1.WiFiType = {
  WI_FI_TYPE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_WIFI_UNSPECIFIED: 0,
  WI_FI_TYPE_WEP: 1,
  WI_FI_TYPE_WPA: 2,
  WI_FI_TYPE_WPA2: 3,
  WI_FI_TYPE_NOPASS: 4
};

/**
 * @enum {number}
 */
proto.ubo.v1.ConnectionState = {
  CONNECTION_STATE_UBO_APP_DOT_STORE_DOT_SERVICES_DOT_WIFI_UNSPECIFIED: 0,
  CONNECTION_STATE_CONNECTED: 1,
  CONNECTION_STATE_CONNECTING: 2,
  CONNECTION_STATE_DISCONNECTED: 3,
  CONNECTION_STATE_UNKNOWN: 4
};

goog.object.extend(exports, proto.ubo.v1);
